#include "coeffs.h"

std::vector<std::string> getModelNames() {
	static std::vector<std::string> modelNames = {
		"spv",                   "z3",                    "cassini11",             
		"cassini5",              "soi",                   "p1184",                 
		"burton2009",            "v1",                    "cassini3",              
		"v2",                    "p11as",                 "igrf1960",              
		"igrf1935",              "igrf1945",              "igrf1925",              
		"igrf1910",              "igrf1905",              "igrf1980",              
		"igrf1990",              "igrf2015",              "igrf1940",              
		"igrf1930",              "igrf2005",              "igrf2020",              
		"igrf2010",              "igrf1915",              "igrf2000",              
		"igrf1950",              "igrf1970",              "igrf1965",              
		"igrf1920",              "igrf2025",              "igrf1985",              
		"igrf1975",              "igrf1955",              "igrf1900",              
		"igrf1995",              "gsfco8full",            "gsfco8",                
		"nmoh",                  "kivelson2002a",         "weber2022quad",         
		"kivelson2002b",         "kivelson2002c",         "weber2022dip",          
		"uno2009",               "thebault2018m2",        "uno2009svd",            
		"anderson2010qts04",     "anderson2010dsha",      "anderson2012",          
		"anderson2010dts04",     "anderson2010qsha",      "ness1975",              
		"thebault2018m1",        "anderson2010r",         "anderson2010d",         
		"anderson2010q",         "thebault2018m3",        "gao2021",               
		"cain2003",              "mh2014",                "langlais2019",          
		"gsfcq3full",            "gsfcq3",                "ah5",                   
		"umoh",                  "jrm33",                 "vit4",                  
		"isaac",                 "vipal",                 "vip4",                  
		"jpl15evs",              "p11a",                  "jrm09",                 
		"jpl15ev",               "o4",                    "o6",                    
		"sha",                   "u17ev",                 "gsfc15ev",              
		"gsfc13ev",              "gsfc15evs",             "v117ev",                
	};
	return modelNames;
}

/*--------------------------- Model Definitions-----------------------*/

/* Body : saturn ---  Model : spv */
coeffStruct& _model_coeff_spv() {
	static const std::string name = "spv";
	static const std::string body = "saturn";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		  21160.000000,      0.000000,   1560.000000,      0.000000,      0.000000,
		   2320.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : saturn ---  Model : z3 */
coeffStruct& _model_coeff_z3() {
	static const std::string name = "z3";
	static const std::string body = "saturn";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		  21184.000000,      0.000000,   1544.000000,      0.000000,      0.000000,
		   2150.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : saturn ---  Model : cassini11 */
coeffStruct& _model_coeff_cassini11() {
	static const std::string name = "cassini11";
	static const std::string body = "saturn";
	static const int len = 90;
	static const int nmax = 12;
	static const int ndef = 11;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
	};
	static const std::vector<double> g = {
		  21140.200000,      0.000000,   1581.100000,      0.000000,      0.000000,
		   2260.100000,      0.000000,      0.000000,      0.000000,     91.100000,
		      0.000000,      0.000000,      0.000000,      0.000000,     12.600000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		     17.200000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,    -59.600000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		    -10.500000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,    -12.900000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,     15.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		     18.200000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.300000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : saturn ---  Model : cassini5 */
coeffStruct& _model_coeff_cassini5() {
	static const std::string name = "cassini5";
	static const std::string body = "saturn";
	static const int len = 20;
	static const int nmax = 5;
	static const int ndef = 5;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,
	};
	static const std::vector<double> g = {
		  21191.000000,      0.000000,   1586.000000,      0.000000,      0.000000,
		   2374.000000,      0.000000,      0.000000,      0.000000,     65.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,    185.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : saturn ---  Model : soi */
coeffStruct& _model_coeff_soi() {
	static const std::string name = "soi";
	static const std::string body = "saturn";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		  21084.000000,      0.000000,   1544.000000,      0.000000,      0.000000,
		   2150.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : saturn ---  Model : p1184 */
coeffStruct& _model_coeff_p1184() {
	static const std::string name = "p1184";
	static const std::string body = "saturn";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		  21140.000000,      0.000000,   1600.000000,      0.000000,      0.000000,
		   2260.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : saturn ---  Model : burton2009 */
coeffStruct& _model_coeff_burton2009() {
	static const std::string name = "burton2009";
	static const std::string body = "saturn";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		  21162.000000,      0.000000,   1514.000000,      0.000000,      0.000000,
		   2283.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : saturn ---  Model : v1 */
coeffStruct& _model_coeff_v1() {
	static const std::string name = "v1";
	static const std::string body = "saturn";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		  21224.000000,      0.000000,   1678.000000,      0.000000,      0.000000,
		   2613.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : saturn ---  Model : cassini3 */
coeffStruct& _model_coeff_cassini3() {
	static const std::string name = "cassini3";
	static const std::string body = "saturn";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		  21191.000000,      0.000000,   1586.000000,      0.000000,      0.000000,
		   2374.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : saturn ---  Model : v2 */
coeffStruct& _model_coeff_v2() {
	static const std::string name = "v2";
	static const std::string body = "saturn";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		  21084.000000,      0.000000,   1607.000000,      0.000000,      0.000000,
		   2513.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : saturn ---  Model : p11as */
coeffStruct& _model_coeff_p11as() {
	static const std::string name = "p11as";
	static const std::string body = "saturn";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		  21180.000000,      0.000000,   1980.000000,      0.000000,      0.000000,
		   2350.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1960 */
coeffStruct& _model_coeff_igrf1960() {
	static const std::string name = "igrf1960";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -30421.000000,  -2169.000000,  -1555.000000,   3002.000000,   1590.000000,
		   1302.000000,  -1992.000000,   1289.000000,    878.000000,    957.000000,
		    800.000000,    504.000000,   -394.000000,    269.000000,   -222.000000,
		    362.000000,    242.000000,    -26.000000,   -156.000000,    -63.000000,
		     46.000000,     58.000000,      1.000000,   -237.000000,     -1.000000,
		     -2.000000,   -113.000000,     67.000000,    -56.000000,      5.000000,
		     15.000000,    -32.000000,     -7.000000,     17.000000,      8.000000,
		     15.000000,      6.000000,     -4.000000,    -11.000000,      2.000000,
		     10.000000,     -5.000000,     10.000000,      8.000000,      4.000000,
		      6.000000,      0.000000,     -9.000000,      1.000000,      4.000000,
		     -1.000000,     -2.000000,      3.000000,     -1.000000,      1.000000,
		     -3.000000,      4.000000,      0.000000,     -1.000000,      4.000000,
		      6.000000,      1.000000,     -1.000000,      2.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5791.000000,      0.000000,  -1967.000000,    206.000000,
		      0.000000,   -414.000000,    224.000000,   -130.000000,      0.000000,
		    135.000000,   -278.000000,      3.000000,   -255.000000,      0.000000,
		     16.000000,    125.000000,   -117.000000,   -114.000000,     81.000000,
		      0.000000,    -10.000000,     99.000000,     60.000000,    -20.000000,
		    -11.000000,    -17.000000,      0.000000,    -55.000000,    -28.000000,
		     -6.000000,      7.000000,     23.000000,    -18.000000,    -17.000000,
		      0.000000,     11.000000,    -14.000000,      7.000000,    -18.000000,
		      4.000000,     23.000000,      1.000000,    -20.000000,      0.000000,
		    -18.000000,     12.000000,      2.000000,      0.000000,     -3.000000,
		      9.000000,      8.000000,      0.000000,      5.000000,      0.000000,
		      4.000000,      1.000000,      0.000000,      2.000000,     -5.000000,
		      1.000000,     -1.000000,      6.000000,      0.000000,     -7.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1935 */
coeffStruct& _model_coeff_igrf1935() {
	static const std::string name = "igrf1935";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -30715.000000,  -2306.000000,  -1018.000000,   2984.000000,   1550.000000,
		   1206.000000,  -1740.000000,   1215.000000,    918.000000,    903.000000,
		    744.000000,    565.000000,   -415.000000,    249.000000,   -241.000000,
		    329.000000,    211.000000,    -33.000000,   -136.000000,    -76.000000,
		     59.000000,     53.000000,     -8.000000,   -246.000000,    -25.000000,
		     25.000000,   -106.000000,     74.000000,    -53.000000,      4.000000,
		     23.000000,    -33.000000,    -11.000000,     18.000000,      6.000000,
		     11.000000,      7.000000,     -3.000000,     -9.000000,      1.000000,
		      6.000000,     -6.000000,      8.000000,      7.000000,      8.000000,
		     10.000000,      1.000000,    -12.000000,     11.000000,      1.000000,
		     -2.000000,      3.000000,      0.000000,     -2.000000,     -3.000000,
		     -4.000000,      2.000000,     -5.000000,     -2.000000,      6.000000,
		      4.000000,      0.000000,      2.000000,      3.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5812.000000,      0.000000,  -1520.000000,    586.000000,
		      0.000000,   -494.000000,    146.000000,    101.000000,      0.000000,
		    188.000000,   -226.000000,    -90.000000,   -114.000000,      0.000000,
		    -51.000000,     64.000000,    -64.000000,   -115.000000,     64.000000,
		      0.000000,      4.000000,    104.000000,     25.000000,    -15.000000,
		      4.000000,    -40.000000,      0.000000,    -52.000000,    -17.000000,
		    -14.000000,      7.000000,     29.000000,    -19.000000,    -19.000000,
		      0.000000,      8.000000,    -15.000000,      5.000000,    -15.000000,
		      5.000000,     18.000000,     -5.000000,    -19.000000,      0.000000,
		    -20.000000,     15.000000,      5.000000,     -3.000000,     -3.000000,
		      9.000000,     11.000000,     -2.000000,      2.000000,      0.000000,
		      2.000000,      1.000000,      2.000000,      6.000000,     -4.000000,
		      0.000000,     -1.000000,      4.000000,      0.000000,     -6.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1945 */
coeffStruct& _model_coeff_igrf1945() {
	static const std::string name = "igrf1945";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -30594.000000,  -2285.000000,  -1244.000000,   2990.000000,   1578.000000,
		   1282.000000,  -1834.000000,   1255.000000,    913.000000,    944.000000,
		    776.000000,    544.000000,   -421.000000,    304.000000,   -253.000000,
		    346.000000,    194.000000,    -20.000000,   -142.000000,    -82.000000,
		     59.000000,     57.000000,      6.000000,   -246.000000,    -25.000000,
		     21.000000,   -104.000000,     70.000000,    -40.000000,      0.000000,
		      0.000000,    -29.000000,    -10.000000,     15.000000,     29.000000,
		     13.000000,      7.000000,     -8.000000,     -5.000000,      9.000000,
		      7.000000,    -10.000000,      7.000000,      2.000000,      5.000000,
		    -21.000000,      1.000000,    -11.000000,      3.000000,     16.000000,
		     -3.000000,     -4.000000,     -3.000000,     -4.000000,     -3.000000,
		     11.000000,      1.000000,      2.000000,     -5.000000,     -1.000000,
		      8.000000,     -1.000000,     -3.000000,      5.000000,     -2.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5810.000000,      0.000000,  -1702.000000,    477.000000,
		      0.000000,   -499.000000,    186.000000,    -11.000000,      0.000000,
		    144.000000,   -276.000000,    -55.000000,   -178.000000,      0.000000,
		    -12.000000,     95.000000,    -67.000000,   -119.000000,     82.000000,
		      0.000000,      6.000000,    100.000000,     16.000000,     -9.000000,
		    -16.000000,    -39.000000,      0.000000,    -45.000000,    -18.000000,
		      2.000000,      6.000000,     28.000000,    -17.000000,    -22.000000,
		      0.000000,     12.000000,    -21.000000,    -12.000000,     -7.000000,
		      2.000000,     18.000000,      3.000000,    -11.000000,      0.000000,
		    -27.000000,     17.000000,     29.000000,     -9.000000,      4.000000,
		      9.000000,      6.000000,      1.000000,      8.000000,      0.000000,
		      5.000000,      1.000000,    -20.000000,     -1.000000,     -6.000000,
		      6.000000,     -4.000000,     -2.000000,      0.000000,     -2.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1925 */
coeffStruct& _model_coeff_igrf1925() {
	static const std::string name = "igrf1925";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -30926.000000,  -2318.000000,   -893.000000,   2969.000000,   1471.000000,
		   1140.000000,  -1645.000000,   1202.000000,    881.000000,    891.000000,
		    711.000000,    601.000000,   -426.000000,    217.000000,   -230.000000,
		    326.000000,    226.000000,    -28.000000,   -125.000000,    -69.000000,
		     61.000000,     54.000000,     -9.000000,   -238.000000,    -40.000000,
		     39.000000,   -103.000000,     73.000000,    -54.000000,      3.000000,
		     27.000000,    -35.000000,    -14.000000,     19.000000,      6.000000,
		     11.000000,      7.000000,     -3.000000,     -9.000000,      2.000000,
		      4.000000,     -7.000000,      7.000000,      8.000000,      8.000000,
		     10.000000,      1.000000,    -11.000000,     12.000000,      1.000000,
		     -2.000000,      2.000000,      0.000000,     -1.000000,     -3.000000,
		     -4.000000,      2.000000,     -5.000000,     -2.000000,      6.000000,
		      4.000000,      0.000000,      1.000000,      3.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5817.000000,      0.000000,  -1334.000000,    728.000000,
		      0.000000,   -462.000000,    119.000000,    229.000000,      0.000000,
		    216.000000,   -163.000000,   -130.000000,    -70.000000,      0.000000,
		    -96.000000,     58.000000,    -44.000000,   -122.000000,     51.000000,
		      0.000000,      3.000000,     99.000000,     14.000000,    -18.000000,
		     13.000000,    -52.000000,      0.000000,    -50.000000,    -14.000000,
		    -14.000000,      5.000000,     29.000000,    -17.000000,    -21.000000,
		      0.000000,      8.000000,    -15.000000,      6.000000,    -14.000000,
		      5.000000,     17.000000,     -5.000000,    -19.000000,      0.000000,
		    -20.000000,     14.000000,      5.000000,     -3.000000,     -2.000000,
		      9.000000,     10.000000,     -2.000000,      2.000000,      0.000000,
		      2.000000,      1.000000,      2.000000,      6.000000,     -4.000000,
		      0.000000,     -2.000000,      4.000000,      0.000000,     -6.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1910 */
coeffStruct& _model_coeff_igrf1910() {
	static const std::string name = "igrf1910";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -31354.000000,  -2297.000000,   -769.000000,   2948.000000,   1176.000000,
		   1058.000000,  -1524.000000,   1223.000000,    705.000000,    884.000000,
		    660.000000,    644.000000,   -400.000000,    160.000000,   -201.000000,
		    327.000000,    253.000000,     -9.000000,   -102.000000,    -38.000000,
		     62.000000,     58.000000,    -11.000000,   -224.000000,    -54.000000,
		     54.000000,    -95.000000,     71.000000,    -54.000000,      1.000000,
		     32.000000,    -40.000000,    -19.000000,     18.000000,      6.000000,
		     11.000000,      8.000000,     -4.000000,     -9.000000,      1.000000,
		      2.000000,     -8.000000,      5.000000,      8.000000,      8.000000,
		     10.000000,      1.000000,    -11.000000,     12.000000,      1.000000,
		     -2.000000,      2.000000,      0.000000,     -1.000000,     -3.000000,
		     -4.000000,      2.000000,     -5.000000,     -2.000000,      6.000000,
		      4.000000,      0.000000,      2.000000,      2.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5898.000000,      0.000000,  -1128.000000,   1000.000000,
		      0.000000,   -389.000000,     62.000000,    425.000000,      0.000000,
		    211.000000,    -90.000000,   -189.000000,    -55.000000,      0.000000,
		   -172.000000,     57.000000,    -33.000000,   -126.000000,     21.000000,
		      0.000000,     -5.000000,     89.000000,      5.000000,    -29.000000,
		     28.000000,    -65.000000,      0.000000,    -47.000000,    -14.000000,
		    -12.000000,      1.000000,     28.000000,    -13.000000,    -22.000000,
		      0.000000,      8.000000,    -15.000000,      6.000000,    -13.000000,
		      5.000000,     16.000000,     -5.000000,    -18.000000,      0.000000,
		    -20.000000,     14.000000,      5.000000,     -3.000000,     -2.000000,
		      8.000000,     10.000000,     -2.000000,      2.000000,      0.000000,
		      2.000000,      1.000000,      2.000000,      6.000000,     -4.000000,
		      0.000000,     -2.000000,      4.000000,      0.000000,     -6.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1905 */
coeffStruct& _model_coeff_igrf1905() {
	static const std::string name = "igrf1905";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -31464.000000,  -2298.000000,   -728.000000,   2928.000000,   1041.000000,
		   1037.000000,  -1494.000000,   1239.000000,    635.000000,    880.000000,
		    643.000000,    653.000000,   -380.000000,    146.000000,   -192.000000,
		    328.000000,    259.000000,     -1.000000,    -93.000000,    -26.000000,
		     62.000000,     60.000000,    -11.000000,   -221.000000,    -57.000000,
		     57.000000,    -92.000000,     70.000000,    -54.000000,      0.000000,
		     33.000000,    -41.000000,    -20.000000,     18.000000,      6.000000,
		     11.000000,      8.000000,     -4.000000,     -9.000000,      1.000000,
		      2.000000,     -8.000000,      5.000000,      8.000000,      8.000000,
		     10.000000,      1.000000,    -11.000000,     12.000000,      1.000000,
		     -2.000000,      2.000000,      0.000000,     -1.000000,     -3.000000,
		     -4.000000,      2.000000,     -5.000000,     -2.000000,      6.000000,
		      4.000000,      0.000000,      2.000000,      2.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5909.000000,      0.000000,  -1086.000000,   1065.000000,
		      0.000000,   -357.000000,     34.000000,    480.000000,      0.000000,
		    203.000000,    -77.000000,   -201.000000,    -65.000000,      0.000000,
		   -193.000000,     56.000000,    -32.000000,   -125.000000,     11.000000,
		      0.000000,     -7.000000,     86.000000,      4.000000,    -32.000000,
		     32.000000,    -67.000000,      0.000000,    -46.000000,    -14.000000,
		    -11.000000,      0.000000,     28.000000,    -12.000000,    -22.000000,
		      0.000000,      8.000000,    -15.000000,      7.000000,    -13.000000,
		      5.000000,     16.000000,     -5.000000,    -18.000000,      0.000000,
		    -20.000000,     14.000000,      5.000000,     -3.000000,     -2.000000,
		      8.000000,     10.000000,     -2.000000,      2.000000,      0.000000,
		      2.000000,      1.000000,      2.000000,      6.000000,     -4.000000,
		      0.000000,     -2.000000,      4.000000,      0.000000,     -6.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1980 */
coeffStruct& _model_coeff_igrf1980() {
	static const std::string name = "igrf1980";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -29992.000000,  -1956.000000,  -1997.000000,   3027.000000,   1663.000000,
		   1281.000000,  -2180.000000,   1251.000000,    833.000000,    938.000000,
		    782.000000,    398.000000,   -419.000000,    199.000000,   -218.000000,
		    357.000000,    261.000000,    -74.000000,   -162.000000,    -48.000000,
		     48.000000,     66.000000,     42.000000,   -192.000000,      4.000000,
		     14.000000,   -108.000000,     72.000000,    -59.000000,      2.000000,
		     21.000000,    -12.000000,      1.000000,     11.000000,     -2.000000,
		     18.000000,      6.000000,      0.000000,    -11.000000,     -7.000000,
		      4.000000,      3.000000,      6.000000,     -1.000000,      5.000000,
		     10.000000,      1.000000,    -12.000000,      9.000000,     -3.000000,
		     -1.000000,      7.000000,      2.000000,     -5.000000,     -4.000000,
		     -4.000000,      2.000000,     -5.000000,     -2.000000,      5.000000,
		      3.000000,      1.000000,      2.000000,      3.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5604.000000,      0.000000,  -2129.000000,   -200.000000,
		      0.000000,   -336.000000,    271.000000,   -252.000000,      0.000000,
		    212.000000,   -257.000000,     53.000000,   -297.000000,      0.000000,
		     46.000000,    150.000000,   -151.000000,    -78.000000,     92.000000,
		      0.000000,    -15.000000,     93.000000,     71.000000,    -43.000000,
		     -2.000000,     17.000000,      0.000000,    -82.000000,    -27.000000,
		     -5.000000,     16.000000,     18.000000,    -23.000000,    -10.000000,
		      0.000000,      7.000000,    -18.000000,      4.000000,    -22.000000,
		      9.000000,     16.000000,    -13.000000,    -15.000000,      0.000000,
		    -21.000000,     16.000000,      9.000000,     -5.000000,     -6.000000,
		      9.000000,     10.000000,     -6.000000,      2.000000,      0.000000,
		      1.000000,      0.000000,      3.000000,      6.000000,     -4.000000,
		      0.000000,     -1.000000,      4.000000,      0.000000,     -6.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1990 */
coeffStruct& _model_coeff_igrf1990() {
	static const std::string name = "igrf1990";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -29775.000000,  -1848.000000,  -2131.000000,   3059.000000,   1686.000000,
		   1314.000000,  -2239.000000,   1248.000000,    802.000000,    939.000000,
		    780.000000,    325.000000,   -423.000000,    141.000000,   -214.000000,
		    353.000000,    245.000000,   -109.000000,   -165.000000,    -36.000000,
		     61.000000,     65.000000,     59.000000,   -178.000000,      3.000000,
		     18.000000,    -96.000000,     77.000000,    -64.000000,      2.000000,
		     26.000000,     -1.000000,      5.000000,      9.000000,      0.000000,
		     23.000000,      5.000000,     -1.000000,    -10.000000,    -12.000000,
		      3.000000,      4.000000,      2.000000,     -6.000000,      4.000000,
		      9.000000,      1.000000,    -12.000000,      9.000000,     -4.000000,
		     -2.000000,      7.000000,      1.000000,     -6.000000,     -3.000000,
		     -4.000000,      2.000000,     -5.000000,     -2.000000,      4.000000,
		      3.000000,      1.000000,      3.000000,      3.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5406.000000,      0.000000,  -2279.000000,   -373.000000,
		      0.000000,   -284.000000,    293.000000,   -352.000000,      0.000000,
		    247.000000,   -240.000000,     84.000000,   -299.000000,      0.000000,
		     46.000000,    154.000000,   -153.000000,    -69.000000,     97.000000,
		      0.000000,    -16.000000,     82.000000,     69.000000,    -52.000000,
		      1.000000,     24.000000,      0.000000,    -80.000000,    -26.000000,
		      0.000000,     21.000000,     17.000000,    -23.000000,     -4.000000,
		      0.000000,     10.000000,    -19.000000,      6.000000,    -22.000000,
		     12.000000,     12.000000,    -16.000000,    -10.000000,      0.000000,
		    -20.000000,     15.000000,     11.000000,     -7.000000,     -7.000000,
		      9.000000,      8.000000,     -7.000000,      2.000000,      0.000000,
		      2.000000,      1.000000,      3.000000,      6.000000,     -4.000000,
		      0.000000,     -2.000000,      3.000000,     -1.000000,     -6.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf2015 */
coeffStruct& _model_coeff_igrf2015() {
	static const std::string name = "igrf2015";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -29441.460000,  -1501.770000,  -2445.880000,   3012.200000,   1676.350000,
		   1350.330000,  -2352.260000,   1225.850000,    581.690000,    907.420000,
		    813.680000,    120.490000,   -334.850000,     70.380000,   -232.910000,
		    360.140000,    192.350000,   -140.940000,   -157.400000,      4.300000,
		     69.550000,     67.570000,     72.790000,   -129.850000,    -28.930000,
		     13.140000,    -70.850000,     81.290000,    -75.990000,     -6.790000,
		     51.820000,     15.070000,      9.320000,     -2.880000,      6.610000,
		     23.980000,      8.890000,    -16.780000,     -3.160000,    -20.560000,
		     13.330000,     11.760000,    -15.980000,     -2.020000,      5.330000,
		      8.830000,      3.020000,     -3.220000,      0.670000,    -13.200000,
		     -0.100000,      8.680000,     -9.060000,    -10.540000,     -2.010000,
		     -6.260000,      0.170000,      0.550000,     -0.550000,      1.700000,
		     -0.670000,      2.130000,      2.330000,     -1.800000,     -3.590000,
		      3.000000,     -1.400000,     -2.300000,      2.080000,     -0.790000,
		      0.580000,     -0.700000,      0.140000,      1.700000,     -0.220000,
		      0.440000,      3.490000,     -2.090000,     -0.160000,      0.460000,
		      1.230000,     -0.890000,      0.850000,      0.100000,      0.540000,
		     -0.370000,     -0.430000,      0.220000,     -0.940000,     -0.030000,
		     -0.020000,     -0.920000,      0.420000,      0.630000,     -0.420000,
		      0.960000,     -0.190000,      0.810000,     -0.130000,      0.380000,
		      0.080000,      0.460000,     -0.350000,     -0.360000,
	};
	static const std::vector<double> h = {
		      0.000000,   4795.990000,      0.000000,  -2845.410000,   -642.170000,
		      0.000000,   -115.290000,    245.040000,   -538.700000,      0.000000,
		    283.540000,   -188.430000,    180.950000,   -329.230000,      0.000000,
		     46.980000,    196.980000,   -119.140000,     15.980000,    100.120000,
		      0.000000,    -20.610000,     33.300000,     58.740000,    -66.640000,
		      7.350000,     62.410000,      0.000000,    -54.270000,    -19.530000,
		      5.590000,     24.450000,      3.270000,    -27.500000,     -2.320000,
		      0.000000,     10.040000,    -18.260000,     13.180000,    -14.600000,
		     16.160000,      5.690000,     -9.100000,      2.260000,      0.000000,
		    -21.770000,     10.760000,     11.740000,     -6.740000,     -6.880000,
		      7.790000,      1.040000,     -3.890000,      8.440000,      0.000000,
		      3.280000,     -0.400000,      4.550000,      4.400000,     -7.920000,
		     -0.610000,     -4.160000,     -2.850000,     -1.120000,     -8.720000,
		      0.000000,      0.000000,      2.110000,     -0.600000,     -1.050000,
		      0.760000,     -0.200000,     -2.120000,     -1.440000,     -2.570000,
		     -2.010000,     -2.340000,      0.000000,     -1.080000,      0.370000,
		      1.750000,     -2.190000,      0.270000,      0.720000,     -0.090000,
		      0.290000,      0.230000,     -0.890000,     -0.160000,      0.720000,
		      0.000000,     -0.880000,      0.490000,      1.560000,     -0.500000,
		     -1.240000,     -0.100000,      0.420000,     -0.040000,      0.480000,
		      0.480000,     -0.300000,     -0.430000,     -0.710000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1940 */
coeffStruct& _model_coeff_igrf1940() {
	static const std::string name = "igrf1940";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -30654.000000,  -2292.000000,  -1106.000000,   2981.000000,   1566.000000,
		   1240.000000,  -1790.000000,   1232.000000,    916.000000,    914.000000,
		    762.000000,    550.000000,   -405.000000,    265.000000,   -241.000000,
		    334.000000,    208.000000,    -33.000000,   -141.000000,    -76.000000,
		     57.000000,     54.000000,     -7.000000,   -249.000000,    -18.000000,
		     18.000000,   -107.000000,     74.000000,    -53.000000,      4.000000,
		     20.000000,    -31.000000,     -9.000000,     17.000000,      5.000000,
		     11.000000,      7.000000,     -3.000000,    -10.000000,      1.000000,
		      6.000000,     -5.000000,      9.000000,      7.000000,      8.000000,
		     10.000000,      1.000000,    -12.000000,     11.000000,      1.000000,
		     -2.000000,      3.000000,      1.000000,     -2.000000,     -3.000000,
		     -4.000000,      2.000000,     -5.000000,     -2.000000,      6.000000,
		      4.000000,      0.000000,      2.000000,      3.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5821.000000,      0.000000,  -1614.000000,    528.000000,
		      0.000000,   -499.000000,    163.000000,     43.000000,      0.000000,
		    169.000000,   -252.000000,    -72.000000,   -141.000000,      0.000000,
		    -33.000000,     71.000000,    -75.000000,   -113.000000,     69.000000,
		      0.000000,      4.000000,    105.000000,     33.000000,    -15.000000,
		      0.000000,    -33.000000,      0.000000,    -52.000000,    -18.000000,
		    -14.000000,      7.000000,     29.000000,    -20.000000,    -19.000000,
		      0.000000,      8.000000,    -14.000000,      5.000000,    -15.000000,
		      5.000000,     19.000000,     -5.000000,    -19.000000,      0.000000,
		    -21.000000,     15.000000,      5.000000,     -3.000000,     -3.000000,
		      9.000000,     11.000000,     -2.000000,      2.000000,      0.000000,
		      2.000000,      1.000000,      2.000000,      6.000000,     -4.000000,
		      0.000000,     -1.000000,      4.000000,      0.000000,     -6.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1930 */
coeffStruct& _model_coeff_igrf1930() {
	static const std::string name = "igrf1930";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -30805.000000,  -2316.000000,   -951.000000,   2980.000000,   1517.000000,
		   1172.000000,  -1692.000000,   1205.000000,    907.000000,    896.000000,
		    727.000000,    584.000000,   -422.000000,    234.000000,   -237.000000,
		    327.000000,    218.000000,    -32.000000,   -131.000000,    -74.000000,
		     60.000000,     53.000000,     -9.000000,   -242.000000,    -32.000000,
		     32.000000,   -104.000000,     74.000000,    -54.000000,      4.000000,
		     25.000000,    -34.000000,    -12.000000,     18.000000,      6.000000,
		     11.000000,      7.000000,     -3.000000,     -9.000000,      2.000000,
		      5.000000,     -6.000000,      8.000000,      8.000000,      8.000000,
		     10.000000,      1.000000,    -12.000000,     12.000000,      1.000000,
		     -2.000000,      3.000000,      0.000000,     -2.000000,     -3.000000,
		     -4.000000,      2.000000,     -5.000000,     -2.000000,      6.000000,
		      4.000000,      0.000000,      1.000000,      3.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5808.000000,      0.000000,  -1424.000000,    644.000000,
		      0.000000,   -480.000000,    133.000000,    166.000000,      0.000000,
		    205.000000,   -195.000000,   -109.000000,    -90.000000,      0.000000,
		    -72.000000,     60.000000,    -53.000000,   -118.000000,     58.000000,
		      0.000000,      4.000000,    102.000000,     19.000000,    -16.000000,
		      8.000000,    -46.000000,      0.000000,    -51.000000,    -15.000000,
		    -14.000000,      6.000000,     29.000000,    -18.000000,    -20.000000,
		      0.000000,      8.000000,    -15.000000,      5.000000,    -14.000000,
		      5.000000,     18.000000,     -5.000000,    -19.000000,      0.000000,
		    -20.000000,     14.000000,      5.000000,     -3.000000,     -2.000000,
		      9.000000,     10.000000,     -2.000000,      2.000000,      0.000000,
		      2.000000,      1.000000,      2.000000,      6.000000,     -4.000000,
		      0.000000,     -2.000000,      4.000000,      0.000000,     -6.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf2005 */
coeffStruct& _model_coeff_igrf2005() {
	static const std::string name = "igrf2005";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -29554.630000,  -1669.050000,  -2337.240000,   3047.690000,   1657.760000,
		   1336.300000,  -2305.830000,   1246.390000,    672.510000,    920.550000,
		    797.960000,    210.650000,   -379.860000,    100.000000,   -227.000000,
		    354.410000,    208.950000,   -136.540000,   -168.050000,    -13.550000,
		     73.600000,     69.560000,     76.740000,   -151.340000,    -14.580000,
		     14.580000,    -86.360000,     79.880000,    -74.460000,     -1.650000,
		     38.730000,     12.300000,      9.370000,      5.420000,      1.940000,
		     24.800000,      7.620000,    -11.730000,     -6.880000,    -18.110000,
		     10.170000,      9.360000,    -11.250000,     -4.870000,      5.580000,
		      9.760000,      3.580000,     -6.940000,      5.010000,    -10.760000,
		     -1.250000,      8.760000,     -6.660000,     -9.220000,     -2.170000,
		     -6.120000,      1.420000,     -2.350000,     -0.150000,      3.060000,
		      0.290000,      2.060000,      3.770000,     -0.210000,     -2.090000,
		      2.950000,     -1.600000,     -1.880000,      1.440000,     -0.310000,
		      0.290000,     -0.790000,      0.530000,      1.800000,      0.160000,
		      0.960000,      3.990000,     -2.150000,     -0.290000,      0.210000,
		      0.890000,     -0.380000,      0.960000,     -0.300000,      0.460000,
		     -0.350000,     -0.360000,      0.080000,     -0.490000,     -0.080000,
		     -0.160000,     -0.880000,      0.300000,      0.280000,     -0.430000,
		      1.180000,     -0.370000,      0.750000,     -0.260000,      0.350000,
		     -0.050000,      0.410000,     -0.100000,     -0.180000,
	};
	static const std::vector<double> h = {
		      0.000000,   5077.990000,      0.000000,  -2594.500000,   -515.430000,
		      0.000000,   -198.860000,    269.720000,   -524.720000,      0.000000,
		    282.070000,   -225.230000,    145.150000,   -305.360000,      0.000000,
		     42.720000,    180.250000,   -123.450000,    -19.570000,    103.850000,
		      0.000000,    -20.330000,     54.750000,     63.630000,    -63.530000,
		      0.240000,     50.940000,      0.000000,    -61.140000,    -22.570000,
		      6.820000,     25.350000,     10.930000,    -26.320000,     -4.640000,
		      0.000000,     11.200000,    -20.880000,      9.830000,    -19.710000,
		     16.220000,      7.610000,    -12.760000,     -0.060000,      0.000000,
		    -20.110000,     12.690000,     12.670000,     -6.720000,     -8.160000,
		      8.100000,      2.920000,     -7.730000,      6.010000,      0.000000,
		      2.190000,      0.100000,      4.460000,      4.760000,     -6.580000,
		     -1.010000,     -3.470000,     -0.860000,     -2.310000,     -7.930000,
		      0.000000,      0.260000,      1.440000,     -0.770000,     -2.270000,
		      0.900000,     -0.580000,     -2.690000,     -1.080000,     -1.580000,
		     -1.900000,     -1.390000,      0.000000,     -0.550000,      0.230000,
		      2.380000,     -2.630000,      0.610000,      0.400000,      0.010000,
		      0.020000,      0.280000,     -0.870000,     -0.340000,      0.880000,
		      0.000000,     -0.760000,      0.330000,      1.720000,     -0.540000,
		     -1.070000,     -0.040000,      0.630000,      0.210000,      0.530000,
		      0.380000,     -0.220000,     -0.570000,     -0.820000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf2020 */
coeffStruct& _model_coeff_igrf2020() {
	static const std::string name = "igrf2020";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -29404.800000,  -1450.900000,  -2499.600000,   2982.000000,   1677.000000,
		   1363.200000,  -2381.200000,   1236.200000,    525.700000,    903.000000,
		    809.500000,     86.300000,   -309.400000,     48.000000,   -234.300000,
		    363.200000,    187.800000,   -140.700000,   -151.200000,     13.500000,
		     66.000000,     65.500000,     72.900000,   -121.500000,    -36.200000,
		     13.500000,    -64.700000,     80.600000,    -76.700000,     -8.200000,
		     56.500000,     15.800000,      6.400000,     -7.200000,      9.800000,
		     23.700000,      9.700000,    -17.600000,     -0.500000,    -21.100000,
		     15.300000,     13.700000,    -16.500000,     -0.300000,      5.000000,
		      8.400000,      2.900000,     -1.500000,     -1.100000,    -13.200000,
		      1.100000,      8.800000,     -9.300000,    -11.900000,     -1.900000,
		     -6.200000,     -0.100000,      1.700000,     -0.900000,      0.700000,
		     -0.900000,      1.900000,      1.400000,     -2.400000,     -3.800000,
		      3.000000,     -1.400000,     -2.500000,      2.300000,     -0.900000,
		      0.300000,     -0.700000,     -0.100000,      1.400000,     -0.600000,
		      0.200000,      3.100000,     -2.000000,     -0.100000,      0.500000,
		      1.300000,     -1.200000,      0.700000,      0.300000,      0.500000,
		     -0.300000,     -0.500000,      0.100000,     -1.100000,     -0.300000,
		      0.100000,     -0.900000,      0.500000,      0.700000,     -0.300000,
		      0.800000,      0.000000,      0.800000,      0.000000,      0.400000,
		      0.100000,      0.500000,     -0.500000,     -0.400000,
	};
	static const std::vector<double> h = {
		      0.000000,   4652.500000,      0.000000,  -2991.600000,   -734.600000,
		      0.000000,    -82.100000,    241.900000,   -543.400000,      0.000000,
		    281.900000,   -158.400000,    199.700000,   -349.700000,      0.000000,
		     47.700000,    208.300000,   -121.200000,     32.300000,     98.900000,
		      0.000000,    -19.100000,     25.100000,     52.800000,    -64.500000,
		      8.900000,     68.100000,      0.000000,    -51.500000,    -16.900000,
		      2.200000,     23.500000,     -2.200000,    -27.200000,     -1.800000,
		      0.000000,      8.400000,    -15.300000,     12.800000,    -11.700000,
		     14.900000,      3.600000,     -6.900000,      2.800000,      0.000000,
		    -23.400000,     11.000000,      9.800000,     -5.100000,     -6.300000,
		      7.800000,      0.400000,     -1.400000,      9.600000,      0.000000,
		      3.400000,     -0.200000,      3.600000,      4.800000,     -8.600000,
		     -0.100000,     -4.300000,     -3.400000,     -0.100000,     -8.800000,
		      0.000000,      0.000000,      2.500000,     -0.600000,     -0.400000,
		      0.600000,     -0.200000,     -1.700000,     -1.600000,     -3.000000,
		     -2.000000,     -2.600000,      0.000000,     -1.200000,      0.500000,
		      1.400000,     -1.800000,      0.100000,      0.800000,     -0.200000,
		      0.600000,      0.200000,     -0.900000,      0.000000,      0.500000,
		      0.000000,     -0.900000,      0.600000,      1.400000,     -0.400000,
		     -1.300000,     -0.100000,      0.300000,     -0.100000,      0.500000,
		      0.500000,     -0.400000,     -0.400000,     -0.600000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf2010 */
coeffStruct& _model_coeff_igrf2010() {
	static const std::string name = "igrf2010";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -29496.570000,  -1586.420000,  -2396.060000,   3026.340000,   1668.170000,
		   1339.850000,  -2326.540000,   1232.100000,    633.730000,    912.660000,
		    808.970000,    166.580000,   -356.830000,     89.400000,   -230.870000,
		    357.290000,    200.260000,   -141.050000,   -163.170000,     -8.030000,
		     72.780000,     68.690000,     75.920000,   -141.400000,    -22.830000,
		     13.100000,    -78.090000,     80.440000,    -75.000000,     -4.550000,
		     45.240000,     14.000000,     10.460000,      1.640000,      4.920000,
		     24.410000,      8.210000,    -14.500000,     -5.590000,    -19.340000,
		     11.610000,     10.850000,    -14.050000,     -3.540000,      5.500000,
		      9.450000,      3.450000,     -5.270000,      3.130000,    -12.380000,
		     -0.760000,      8.430000,     -8.420000,    -10.080000,     -1.940000,
		     -6.240000,      0.890000,     -1.070000,     -0.160000,      2.450000,
		     -0.330000,      2.130000,      3.090000,     -1.030000,     -2.800000,
		      3.050000,     -1.480000,     -2.030000,      1.650000,     -0.510000,
		      0.540000,     -0.790000,      0.370000,      1.790000,      0.120000,
		      0.750000,      3.750000,     -2.120000,     -0.210000,      0.300000,
		      1.040000,     -0.630000,      0.950000,     -0.110000,      0.520000,
		     -0.390000,     -0.370000,      0.210000,     -0.770000,      0.040000,
		     -0.090000,     -0.890000,      0.310000,      0.420000,     -0.450000,
		      1.080000,     -0.310000,      0.780000,     -0.180000,      0.380000,
		      0.020000,      0.420000,     -0.260000,     -0.260000,
	};
	static const std::vector<double> h = {
		      0.000000,   4944.260000,      0.000000,  -2708.540000,   -575.730000,
		      0.000000,   -160.400000,    251.750000,   -537.030000,      0.000000,
		    286.480000,   -211.030000,    164.460000,   -309.720000,      0.000000,
		     44.580000,    189.010000,   -118.060000,     -0.010000,    101.040000,
		      0.000000,    -20.900000,     44.180000,     61.540000,    -66.260000,
		      3.020000,     55.400000,      0.000000,    -57.800000,    -21.200000,
		      6.540000,     24.960000,      7.030000,    -27.610000,     -3.280000,
		      0.000000,     10.840000,    -20.030000,     11.830000,    -17.410000,
		     16.710000,      6.960000,    -10.740000,      1.640000,      0.000000,
		    -20.540000,     11.510000,     12.750000,     -7.140000,     -7.420000,
		      7.970000,      2.140000,     -6.080000,      7.010000,      0.000000,
		      2.730000,     -0.100000,      4.710000,      4.440000,     -7.220000,
		     -0.960000,     -3.950000,     -1.990000,     -1.970000,     -8.310000,
		      0.000000,      0.130000,      1.670000,     -0.660000,     -1.760000,
		      0.850000,     -0.390000,     -2.510000,     -1.270000,     -2.110000,
		     -1.940000,     -1.860000,      0.000000,     -0.870000,      0.270000,
		      2.130000,     -2.490000,      0.490000,      0.590000,      0.000000,
		      0.130000,      0.270000,     -0.860000,     -0.230000,      0.870000,
		      0.000000,     -0.870000,      0.300000,      1.660000,     -0.590000,
		     -1.140000,     -0.070000,      0.540000,      0.100000,      0.490000,
		      0.440000,     -0.250000,     -0.530000,     -0.790000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1915 */
coeffStruct& _model_coeff_igrf1915() {
	static const std::string name = "igrf1915";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -31212.000000,  -2306.000000,   -802.000000,   2956.000000,   1309.000000,
		   1084.000000,  -1559.000000,   1212.000000,    778.000000,    887.000000,
		    678.000000,    631.000000,   -416.000000,    178.000000,   -211.000000,
		    327.000000,    245.000000,    -16.000000,   -111.000000,    -51.000000,
		     61.000000,     57.000000,    -10.000000,   -228.000000,    -51.000000,
		     49.000000,    -98.000000,     72.000000,    -54.000000,      2.000000,
		     31.000000,    -38.000000,    -18.000000,     19.000000,      6.000000,
		     11.000000,      8.000000,     -4.000000,     -9.000000,      2.000000,
		      3.000000,     -8.000000,      6.000000,      8.000000,      8.000000,
		     10.000000,      1.000000,    -11.000000,     12.000000,      1.000000,
		     -2.000000,      2.000000,      0.000000,     -1.000000,     -3.000000,
		     -4.000000,      2.000000,     -5.000000,     -2.000000,      6.000000,
		      4.000000,      0.000000,      1.000000,      2.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5875.000000,      0.000000,  -1191.000000,    917.000000,
		      0.000000,   -421.000000,     84.000000,    360.000000,      0.000000,
		    218.000000,   -109.000000,   -173.000000,    -51.000000,      0.000000,
		   -148.000000,     58.000000,    -34.000000,   -126.000000,     32.000000,
		      0.000000,     -2.000000,     93.000000,      8.000000,    -26.000000,
		     23.000000,    -62.000000,      0.000000,    -48.000000,    -14.000000,
		    -12.000000,      2.000000,     28.000000,    -15.000000,    -22.000000,
		      0.000000,      8.000000,    -15.000000,      6.000000,    -13.000000,
		      5.000000,     16.000000,     -5.000000,    -18.000000,      0.000000,
		    -20.000000,     14.000000,      5.000000,     -3.000000,     -2.000000,
		      8.000000,     10.000000,     -2.000000,      2.000000,      0.000000,
		      2.000000,      1.000000,      2.000000,      6.000000,     -4.000000,
		      0.000000,     -2.000000,      4.000000,      0.000000,     -6.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf2000 */
coeffStruct& _model_coeff_igrf2000() {
	static const std::string name = "igrf2000";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -29619.400000,  -1728.200000,  -2267.700000,   3068.400000,   1670.900000,
		   1339.600000,  -2288.000000,   1252.100000,    714.500000,    932.300000,
		    786.800000,    250.000000,   -403.000000,    111.300000,   -218.800000,
		    351.400000,    222.300000,   -130.400000,   -168.600000,    -12.900000,
		     72.300000,     68.200000,     74.200000,   -160.900000,     -5.900000,
		     16.900000,    -90.400000,     79.000000,    -74.000000,      0.000000,
		     33.300000,      9.100000,      6.900000,      7.300000,     -1.200000,
		     24.400000,      6.600000,     -9.200000,     -7.900000,    -16.600000,
		      9.100000,      7.000000,     -7.900000,     -7.000000,      5.000000,
		      9.400000,      3.000000,     -8.400000,      6.300000,     -8.900000,
		     -1.500000,      9.300000,     -4.300000,     -8.200000,     -2.600000,
		     -6.000000,      1.700000,     -3.100000,     -0.500000,      3.700000,
		      1.000000,      2.000000,      4.200000,      0.300000,     -1.100000,
		      2.700000,     -1.700000,     -1.900000,      1.500000,     -0.100000,
		      0.100000,     -0.700000,      0.700000,      1.700000,      0.100000,
		      1.200000,      4.000000,     -2.200000,     -0.300000,      0.200000,
		      0.900000,     -0.200000,      0.900000,     -0.500000,      0.300000,
		     -0.300000,     -0.400000,     -0.100000,     -0.200000,     -0.400000,
		     -0.200000,     -0.900000,      0.300000,      0.100000,     -0.400000,
		      1.300000,     -0.400000,      0.700000,     -0.400000,      0.300000,
		     -0.100000,      0.400000,      0.000000,      0.100000,
	};
	static const std::vector<double> h = {
		      0.000000,   5186.100000,      0.000000,  -2481.600000,   -458.000000,
		      0.000000,   -227.600000,    293.400000,   -491.100000,      0.000000,
		    272.600000,   -231.900000,    119.800000,   -303.800000,      0.000000,
		     43.800000,    171.900000,   -133.100000,    -39.300000,    106.300000,
		      0.000000,    -17.400000,     63.700000,     65.100000,    -61.200000,
		      0.700000,     43.800000,      0.000000,    -64.600000,    -24.200000,
		      6.200000,     24.000000,     14.800000,    -25.400000,     -5.800000,
		      0.000000,     11.900000,    -21.500000,      8.500000,    -21.500000,
		     15.500000,      8.900000,    -14.900000,     -2.100000,      0.000000,
		    -19.700000,     13.400000,     12.500000,     -6.200000,     -8.400000,
		      8.400000,      3.800000,     -8.200000,      4.800000,      0.000000,
		      1.700000,      0.000000,      4.000000,      4.900000,     -5.900000,
		     -1.200000,     -2.900000,      0.200000,     -2.200000,     -7.400000,
		      0.000000,      0.100000,      1.300000,     -0.900000,     -2.600000,
		      0.900000,     -0.700000,     -2.800000,     -0.900000,     -1.200000,
		     -1.900000,     -0.900000,      0.000000,     -0.400000,      0.300000,
		      2.500000,     -2.600000,      0.700000,      0.300000,      0.000000,
		      0.000000,      0.300000,     -0.900000,     -0.400000,      0.800000,
		      0.000000,     -0.900000,      0.200000,      1.800000,     -0.400000,
		     -1.000000,     -0.100000,      0.700000,      0.300000,      0.600000,
		      0.300000,     -0.200000,     -0.500000,     -0.900000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1950 */
coeffStruct& _model_coeff_igrf1950() {
	static const std::string name = "igrf1950";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -30554.000000,  -2250.000000,  -1341.000000,   2998.000000,   1576.000000,
		   1297.000000,  -1889.000000,   1274.000000,    896.000000,    954.000000,
		    792.000000,    528.000000,   -408.000000,    303.000000,   -240.000000,
		    349.000000,    211.000000,    -20.000000,   -147.000000,    -76.000000,
		     54.000000,     57.000000,      4.000000,   -247.000000,    -16.000000,
		     12.000000,   -105.000000,     65.000000,    -55.000000,      2.000000,
		      1.000000,    -40.000000,     -7.000000,      5.000000,     19.000000,
		     22.000000,     15.000000,     -4.000000,     -1.000000,     11.000000,
		     15.000000,    -13.000000,      5.000000,     -1.000000,      3.000000,
		     -7.000000,     -1.000000,    -25.000000,     10.000000,      5.000000,
		     -5.000000,     -2.000000,      3.000000,      8.000000,     -8.000000,
		      4.000000,     -1.000000,     13.000000,     -4.000000,      4.000000,
		     12.000000,      3.000000,      2.000000,     10.000000,      3.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5815.000000,      0.000000,  -1810.000000,    381.000000,
		      0.000000,   -476.000000,    206.000000,    -46.000000,      0.000000,
		    136.000000,   -278.000000,    -37.000000,   -210.000000,      0.000000,
		      3.000000,    103.000000,    -87.000000,   -122.000000,     80.000000,
		      0.000000,     -1.000000,     99.000000,     33.000000,    -12.000000,
		    -12.000000,    -30.000000,      0.000000,    -35.000000,    -17.000000,
		      0.000000,     10.000000,     36.000000,    -18.000000,    -16.000000,
		      0.000000,      5.000000,    -22.000000,      0.000000,    -21.000000,
		     -8.000000,     17.000000,     -4.000000,    -17.000000,      0.000000,
		    -24.000000,     19.000000,     12.000000,      2.000000,      2.000000,
		      8.000000,      8.000000,    -11.000000,     -7.000000,      0.000000,
		     13.000000,     -2.000000,    -10.000000,      2.000000,     -3.000000,
		      6.000000,     -3.000000,      6.000000,     11.000000,      8.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1970 */
coeffStruct& _model_coeff_igrf1970() {
	static const std::string name = "igrf1970";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -30220.000000,  -2068.000000,  -1781.000000,   3000.000000,   1611.000000,
		   1287.000000,  -2091.000000,   1278.000000,    838.000000,    952.000000,
		    800.000000,    461.000000,   -395.000000,    234.000000,   -216.000000,
		    359.000000,    262.000000,    -42.000000,   -160.000000,    -56.000000,
		     43.000000,     64.000000,     15.000000,   -212.000000,      2.000000,
		      3.000000,   -112.000000,     72.000000,    -57.000000,      1.000000,
		     14.000000,    -22.000000,     -2.000000,     13.000000,     -2.000000,
		     14.000000,      6.000000,     -2.000000,    -13.000000,     -3.000000,
		      5.000000,      0.000000,     11.000000,      3.000000,      8.000000,
		     10.000000,      2.000000,    -12.000000,     10.000000,     -1.000000,
		      0.000000,      3.000000,      1.000000,     -1.000000,     -3.000000,
		     -3.000000,      2.000000,     -5.000000,     -1.000000,      6.000000,
		      4.000000,      1.000000,      0.000000,      3.000000,     -1.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5737.000000,      0.000000,  -2047.000000,     25.000000,
		      0.000000,   -366.000000,    251.000000,   -196.000000,      0.000000,
		    167.000000,   -266.000000,     26.000000,   -279.000000,      0.000000,
		     26.000000,    139.000000,   -139.000000,    -91.000000,     83.000000,
		      0.000000,    -12.000000,    100.000000,     72.000000,    -37.000000,
		     -6.000000,      1.000000,      0.000000,    -70.000000,    -27.000000,
		     -4.000000,      8.000000,     23.000000,    -23.000000,    -11.000000,
		      0.000000,      7.000000,    -15.000000,      6.000000,    -17.000000,
		      6.000000,     21.000000,     -6.000000,    -16.000000,      0.000000,
		    -21.000000,     16.000000,      6.000000,     -4.000000,     -5.000000,
		     10.000000,     11.000000,     -2.000000,      1.000000,      0.000000,
		      1.000000,      1.000000,      3.000000,      4.000000,     -4.000000,
		      0.000000,     -1.000000,      3.000000,      1.000000,     -4.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1965 */
coeffStruct& _model_coeff_igrf1965() {
	static const std::string name = "igrf1965";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -30334.000000,  -2119.000000,  -1662.000000,   2997.000000,   1594.000000,
		   1297.000000,  -2038.000000,   1292.000000,    856.000000,    957.000000,
		    804.000000,    479.000000,   -390.000000,    252.000000,   -219.000000,
		    358.000000,    254.000000,    -31.000000,   -157.000000,    -62.000000,
		     45.000000,     61.000000,      8.000000,   -228.000000,      4.000000,
		      1.000000,   -111.000000,     75.000000,    -57.000000,      4.000000,
		     13.000000,    -26.000000,     -6.000000,     13.000000,      1.000000,
		     13.000000,      5.000000,     -4.000000,    -14.000000,      0.000000,
		      8.000000,     -1.000000,     11.000000,      4.000000,      8.000000,
		     10.000000,      2.000000,    -13.000000,     10.000000,     -1.000000,
		     -1.000000,      5.000000,      1.000000,     -2.000000,     -2.000000,
		     -3.000000,      2.000000,     -5.000000,     -2.000000,      4.000000,
		      4.000000,      0.000000,      2.000000,      2.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5776.000000,      0.000000,  -2016.000000,    114.000000,
		      0.000000,   -404.000000,    240.000000,   -165.000000,      0.000000,
		    148.000000,   -269.000000,     13.000000,   -269.000000,      0.000000,
		     19.000000,    128.000000,   -126.000000,    -97.000000,     81.000000,
		      0.000000,    -11.000000,    100.000000,     68.000000,    -32.000000,
		     -8.000000,     -7.000000,      0.000000,    -61.000000,    -27.000000,
		     -2.000000,      6.000000,     26.000000,    -23.000000,    -12.000000,
		      0.000000,      7.000000,    -12.000000,      9.000000,    -16.000000,
		      4.000000,     24.000000,     -3.000000,    -17.000000,      0.000000,
		    -22.000000,     15.000000,      7.000000,     -4.000000,     -5.000000,
		     10.000000,     10.000000,     -4.000000,      1.000000,      0.000000,
		      2.000000,      1.000000,      2.000000,      6.000000,     -4.000000,
		      0.000000,     -2.000000,      3.000000,      0.000000,     -6.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1920 */
coeffStruct& _model_coeff_igrf1920() {
	static const std::string name = "igrf1920";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -31060.000000,  -2317.000000,   -839.000000,   2959.000000,   1407.000000,
		   1111.000000,  -1600.000000,   1205.000000,    839.000000,    889.000000,
		    695.000000,    616.000000,   -424.000000,    199.000000,   -221.000000,
		    326.000000,    236.000000,    -23.000000,   -119.000000,    -62.000000,
		     61.000000,     55.000000,    -10.000000,   -233.000000,    -46.000000,
		     44.000000,   -101.000000,     73.000000,    -54.000000,      2.000000,
		     29.000000,    -37.000000,    -16.000000,     19.000000,      6.000000,
		     11.000000,      7.000000,     -3.000000,     -9.000000,      2.000000,
		      4.000000,     -7.000000,      6.000000,      8.000000,      8.000000,
		     10.000000,      1.000000,    -11.000000,     12.000000,      1.000000,
		     -2.000000,      2.000000,      0.000000,     -1.000000,     -3.000000,
		     -4.000000,      2.000000,     -5.000000,     -2.000000,      6.000000,
		      4.000000,      0.000000,      1.000000,      3.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5845.000000,      0.000000,  -1259.000000,    823.000000,
		      0.000000,   -445.000000,    103.000000,    293.000000,      0.000000,
		    220.000000,   -134.000000,   -153.000000,    -57.000000,      0.000000,
		   -122.000000,     58.000000,    -38.000000,   -125.000000,     43.000000,
		      0.000000,      0.000000,     96.000000,     11.000000,    -22.000000,
		     18.000000,    -57.000000,      0.000000,    -49.000000,    -14.000000,
		    -13.000000,      4.000000,     28.000000,    -16.000000,    -22.000000,
		      0.000000,      8.000000,    -15.000000,      6.000000,    -14.000000,
		      5.000000,     17.000000,     -5.000000,    -19.000000,      0.000000,
		    -20.000000,     14.000000,      5.000000,     -3.000000,     -2.000000,
		      9.000000,     10.000000,     -2.000000,      2.000000,      0.000000,
		      2.000000,      1.000000,      2.000000,      6.000000,     -4.000000,
		      0.000000,     -2.000000,      4.000000,      0.000000,     -6.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf2025 */
coeffStruct& _model_coeff_igrf2025() {
	static const std::string name = "igrf2025";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -29376.300000,  -1413.900000,  -2554.600000,   2947.000000,   1666.500000,
		   1374.200000,  -2410.700000,   1251.700000,    465.700000,    897.000000,
		    801.500000,     56.800000,   -283.400000,     22.500000,   -235.800000,
		    365.700000,    184.800000,   -139.700000,   -144.700000,     18.000000,
		     63.500000,     64.000000,     74.900000,   -115.000000,    -43.200000,
		     13.500000,    -60.200000,     80.100000,    -77.700000,     -8.200000,
		     60.000000,     16.300000,      3.900000,    -11.200000,     13.800000,
		     23.700000,     10.200000,    -18.100000,      1.500000,    -21.600000,
		     17.300000,     15.200000,    -17.000000,      1.700000,      5.000000,
		      8.400000,      2.900000,     -1.500000,     -1.100000,    -13.200000,
		      1.100000,      8.800000,     -9.300000,    -11.900000,     -1.900000,
		     -6.200000,     -0.100000,      1.700000,     -0.900000,      0.700000,
		     -0.900000,      1.900000,      1.400000,     -2.400000,     -3.800000,
		      3.000000,     -1.400000,     -2.500000,      2.300000,     -0.900000,
		      0.300000,     -0.700000,     -0.100000,      1.400000,     -0.600000,
		      0.200000,      3.100000,     -2.000000,     -0.100000,      0.500000,
		      1.300000,     -1.200000,      0.700000,      0.300000,      0.500000,
		     -0.300000,     -0.500000,      0.100000,     -1.100000,     -0.300000,
		      0.100000,     -0.900000,      0.500000,      0.700000,     -0.300000,
		      0.800000,      0.000000,      0.800000,      0.000000,      0.400000,
		      0.100000,      0.500000,     -0.500000,     -0.400000,
	};
	static const std::vector<double> h = {
		      0.000000,   4523.000000,      0.000000,  -3142.600000,   -846.600000,
		      0.000000,    -52.100000,    236.400000,   -540.900000,      0.000000,
		    281.400000,   -125.900000,    217.700000,   -374.700000,      0.000000,
		     47.700000,    220.800000,   -124.200000,     47.300000,    100.400000,
		      0.000000,    -19.100000,     17.100000,     46.300000,    -60.500000,
		      8.900000,     73.100000,      0.000000,    -48.500000,    -13.900000,
		     -1.800000,     22.500000,     -7.700000,    -26.700000,     -0.300000,
		      0.000000,      7.400000,    -12.300000,     11.800000,     -9.200000,
		     13.400000,      1.600000,     -4.400000,      2.800000,      0.000000,
		    -23.400000,     11.000000,      9.800000,     -5.100000,     -6.300000,
		      7.800000,      0.400000,     -1.400000,      9.600000,      0.000000,
		      3.400000,     -0.200000,      3.600000,      4.800000,     -8.600000,
		     -0.100000,     -4.300000,     -3.400000,     -0.100000,     -8.800000,
		      0.000000,      0.000000,      2.500000,     -0.600000,     -0.400000,
		      0.600000,     -0.200000,     -1.700000,     -1.600000,     -3.000000,
		     -2.000000,     -2.600000,      0.000000,     -1.200000,      0.500000,
		      1.400000,     -1.800000,      0.100000,      0.800000,     -0.200000,
		      0.600000,      0.200000,     -0.900000,      0.000000,      0.500000,
		      0.000000,     -0.900000,      0.600000,      1.400000,     -0.400000,
		     -1.300000,     -0.100000,      0.300000,     -0.100000,      0.500000,
		      0.500000,     -0.400000,     -0.400000,     -0.600000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1985 */
coeffStruct& _model_coeff_igrf1985() {
	static const std::string name = "igrf1985";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -29873.000000,  -1905.000000,  -2072.000000,   3044.000000,   1687.000000,
		   1296.000000,  -2208.000000,   1247.000000,    829.000000,    936.000000,
		    780.000000,    361.000000,   -424.000000,    170.000000,   -214.000000,
		    355.000000,    253.000000,    -93.000000,   -164.000000,    -46.000000,
		     53.000000,     65.000000,     51.000000,   -185.000000,      4.000000,
		     16.000000,   -102.000000,     74.000000,    -62.000000,      3.000000,
		     24.000000,     -6.000000,      4.000000,     10.000000,      0.000000,
		     21.000000,      6.000000,      0.000000,    -11.000000,     -9.000000,
		      4.000000,      4.000000,      4.000000,     -4.000000,      5.000000,
		     10.000000,      1.000000,    -12.000000,      9.000000,     -3.000000,
		     -1.000000,      7.000000,      1.000000,     -5.000000,     -4.000000,
		     -4.000000,      3.000000,     -5.000000,     -2.000000,      5.000000,
		      3.000000,      1.000000,      2.000000,      3.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5500.000000,      0.000000,  -2197.000000,   -306.000000,
		      0.000000,   -310.000000,    284.000000,   -297.000000,      0.000000,
		    232.000000,   -249.000000,     69.000000,   -297.000000,      0.000000,
		     47.000000,    150.000000,   -154.000000,    -75.000000,     95.000000,
		      0.000000,    -16.000000,     88.000000,     69.000000,    -48.000000,
		     -1.000000,     21.000000,      0.000000,    -83.000000,    -27.000000,
		     -2.000000,     20.000000,     17.000000,    -23.000000,     -7.000000,
		      0.000000,      8.000000,    -19.000000,      5.000000,    -23.000000,
		     11.000000,     14.000000,    -15.000000,    -11.000000,      0.000000,
		    -21.000000,     15.000000,      9.000000,     -6.000000,     -6.000000,
		      9.000000,      9.000000,     -7.000000,      2.000000,      0.000000,
		      1.000000,      0.000000,      3.000000,      6.000000,     -4.000000,
		      0.000000,     -1.000000,      4.000000,      0.000000,     -6.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1975 */
coeffStruct& _model_coeff_igrf1975() {
	static const std::string name = "igrf1975";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -30100.000000,  -2013.000000,  -1902.000000,   3010.000000,   1632.000000,
		   1276.000000,  -2144.000000,   1260.000000,    830.000000,    946.000000,
		    791.000000,    438.000000,   -405.000000,    216.000000,   -218.000000,
		    356.000000,    264.000000,    -59.000000,   -159.000000,    -49.000000,
		     45.000000,     66.000000,     28.000000,   -198.000000,      1.000000,
		      6.000000,   -111.000000,     71.000000,    -56.000000,      1.000000,
		     16.000000,    -14.000000,      0.000000,     12.000000,     -5.000000,
		     14.000000,      6.000000,     -1.000000,    -12.000000,     -8.000000,
		      4.000000,      0.000000,     10.000000,      1.000000,      7.000000,
		     10.000000,      2.000000,    -12.000000,     10.000000,     -1.000000,
		     -1.000000,      4.000000,      1.000000,     -2.000000,     -3.000000,
		     -3.000000,      2.000000,     -5.000000,     -2.000000,      5.000000,
		      4.000000,      1.000000,      0.000000,      3.000000,     -1.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5675.000000,      0.000000,  -2067.000000,    -68.000000,
		      0.000000,   -333.000000,    262.000000,   -223.000000,      0.000000,
		    191.000000,   -265.000000,     39.000000,   -288.000000,      0.000000,
		     31.000000,    148.000000,   -152.000000,    -83.000000,     88.000000,
		      0.000000,    -13.000000,     99.000000,     75.000000,    -41.000000,
		     -4.000000,     11.000000,      0.000000,    -77.000000,    -26.000000,
		     -5.000000,     10.000000,     22.000000,    -23.000000,    -12.000000,
		      0.000000,      6.000000,    -16.000000,      4.000000,    -19.000000,
		      6.000000,     18.000000,    -10.000000,    -17.000000,      0.000000,
		    -21.000000,     16.000000,      7.000000,     -4.000000,     -5.000000,
		     10.000000,     11.000000,     -3.000000,      1.000000,      0.000000,
		      1.000000,      1.000000,      3.000000,      4.000000,     -4.000000,
		     -1.000000,     -1.000000,      3.000000,      1.000000,     -5.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1955 */
coeffStruct& _model_coeff_igrf1955() {
	static const std::string name = "igrf1955";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -30500.000000,  -2215.000000,  -1440.000000,   3003.000000,   1581.000000,
		   1302.000000,  -1944.000000,   1288.000000,    882.000000,    958.000000,
		    796.000000,    510.000000,   -397.000000,    290.000000,   -229.000000,
		    360.000000,    230.000000,    -23.000000,   -152.000000,    -69.000000,
		     47.000000,     57.000000,      3.000000,   -247.000000,     -8.000000,
		      7.000000,   -107.000000,     65.000000,    -56.000000,      2.000000,
		     10.000000,    -32.000000,    -11.000000,      9.000000,     18.000000,
		     11.000000,      9.000000,     -6.000000,    -14.000000,      6.000000,
		     10.000000,     -7.000000,      6.000000,      9.000000,      4.000000,
		      9.000000,     -4.000000,     -5.000000,      2.000000,      4.000000,
		      1.000000,      2.000000,      2.000000,      5.000000,     -3.000000,
		     -5.000000,     -1.000000,      2.000000,     -3.000000,      7.000000,
		      4.000000,     -2.000000,      6.000000,     -2.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5820.000000,      0.000000,  -1898.000000,    291.000000,
		      0.000000,   -462.000000,    216.000000,    -83.000000,      0.000000,
		    133.000000,   -274.000000,    -23.000000,   -230.000000,      0.000000,
		     15.000000,    110.000000,    -98.000000,   -121.000000,     78.000000,
		      0.000000,     -9.000000,     96.000000,     48.000000,    -16.000000,
		    -12.000000,    -24.000000,      0.000000,    -50.000000,    -24.000000,
		     -4.000000,      8.000000,     28.000000,    -20.000000,    -18.000000,
		      0.000000,     10.000000,    -15.000000,      5.000000,    -23.000000,
		      3.000000,     23.000000,     -4.000000,    -13.000000,      0.000000,
		    -11.000000,     12.000000,      7.000000,      6.000000,     -2.000000,
		     10.000000,      7.000000,     -6.000000,      5.000000,      0.000000,
		     -4.000000,      0.000000,     -8.000000,     -2.000000,     -4.000000,
		      1.000000,     -3.000000,      7.000000,     -1.000000,     -3.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1900 */
coeffStruct& _model_coeff_igrf1900() {
	static const std::string name = "igrf1900";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -31543.000000,  -2298.000000,   -677.000000,   2905.000000,    924.000000,
		   1022.000000,  -1469.000000,   1256.000000,    572.000000,    876.000000,
		    628.000000,    660.000000,   -361.000000,    134.000000,   -184.000000,
		    328.000000,    264.000000,      5.000000,    -86.000000,    -16.000000,
		     63.000000,     61.000000,    -11.000000,   -217.000000,    -58.000000,
		     59.000000,    -90.000000,     70.000000,    -55.000000,      0.000000,
		     34.000000,    -41.000000,    -21.000000,     18.000000,      6.000000,
		     11.000000,      8.000000,     -4.000000,     -9.000000,      1.000000,
		      2.000000,     -9.000000,      5.000000,      8.000000,      8.000000,
		     10.000000,      1.000000,    -11.000000,     12.000000,      1.000000,
		     -2.000000,      2.000000,     -1.000000,     -1.000000,     -3.000000,
		     -4.000000,      2.000000,     -5.000000,     -2.000000,      6.000000,
		      4.000000,      0.000000,      2.000000,      2.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5922.000000,      0.000000,  -1061.000000,   1121.000000,
		      0.000000,   -330.000000,      3.000000,    523.000000,      0.000000,
		    195.000000,    -69.000000,   -210.000000,    -75.000000,      0.000000,
		   -210.000000,     53.000000,    -33.000000,   -124.000000,      3.000000,
		      0.000000,     -9.000000,     83.000000,      2.000000,    -35.000000,
		     36.000000,    -69.000000,      0.000000,    -45.000000,    -13.000000,
		    -10.000000,     -1.000000,     28.000000,    -12.000000,    -22.000000,
		      0.000000,      8.000000,    -14.000000,      7.000000,    -13.000000,
		      5.000000,     16.000000,     -5.000000,    -18.000000,      0.000000,
		    -20.000000,     14.000000,      5.000000,     -3.000000,     -2.000000,
		      8.000000,     10.000000,     -2.000000,      2.000000,      0.000000,
		      2.000000,      1.000000,      2.000000,      6.000000,     -4.000000,
		      0.000000,     -2.000000,      4.000000,      0.000000,     -6.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : earth ---  Model : igrf1995 */
coeffStruct& _model_coeff_igrf1995() {
	static const std::string name = "igrf1995";
	static const std::string body = "earth";
	static const int len = 104;
	static const int nmax = 13;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,
	};
	static const std::vector<double> g = {
		 -29692.000000,  -1784.000000,  -2200.000000,   3070.000000,   1681.000000,
		   1335.000000,  -2267.000000,   1249.000000,    759.000000,    940.000000,
		    780.000000,    290.000000,   -418.000000,    122.000000,   -214.000000,
		    352.000000,    235.000000,   -118.000000,   -166.000000,    -17.000000,
		     68.000000,     67.000000,     68.000000,   -170.000000,     -1.000000,
		     19.000000,    -93.000000,     77.000000,    -72.000000,      1.000000,
		     28.000000,      5.000000,      4.000000,      8.000000,     -2.000000,
		     25.000000,      6.000000,     -6.000000,     -9.000000,    -14.000000,
		      9.000000,      6.000000,     -5.000000,     -7.000000,      4.000000,
		      9.000000,      3.000000,    -10.000000,      8.000000,     -8.000000,
		     -1.000000,     10.000000,     -2.000000,     -8.000000,     -3.000000,
		     -6.000000,      2.000000,     -4.000000,     -1.000000,      4.000000,
		      2.000000,      2.000000,      5.000000,      1.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,   5306.000000,      0.000000,  -2366.000000,   -413.000000,
		      0.000000,   -262.000000,    302.000000,   -427.000000,      0.000000,
		    262.000000,   -236.000000,     97.000000,   -306.000000,      0.000000,
		     46.000000,    165.000000,   -143.000000,    -55.000000,    107.000000,
		      0.000000,    -17.000000,     72.000000,     67.000000,    -58.000000,
		      1.000000,     36.000000,      0.000000,    -69.000000,    -25.000000,
		      4.000000,     24.000000,     17.000000,    -24.000000,     -6.000000,
		      0.000000,     11.000000,    -21.000000,      8.000000,    -23.000000,
		     15.000000,     11.000000,    -16.000000,     -4.000000,      0.000000,
		    -20.000000,     15.000000,     12.000000,     -6.000000,     -8.000000,
		      8.000000,      5.000000,     -8.000000,      3.000000,      0.000000,
		      1.000000,      0.000000,      4.000000,      5.000000,     -5.000000,
		     -1.000000,     -2.000000,      1.000000,     -2.000000,     -7.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : neptune ---  Model : gsfco8full */
coeffStruct& _model_coeff_gsfco8full() {
	static const std::string name = "gsfco8full";
	static const std::string body = "neptune";
	static const int len = 44;
	static const int nmax = 8;
	static const int ndef = 8;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,
	};
	static const std::vector<double> g = {
		   9732.000000,   3220.000000,   7448.000000,    664.000000,   4499.000000,
		  -6592.000000,   4098.000000,  -3581.000000,    484.000000,      0.022430,
		      0.005570,      0.030990,     -0.012870,     -0.050730,     -0.002020,
		     -0.002290,      0.005260,     -0.028460,     -0.014250,     -0.028350,
		     -0.021750,     -0.004660,     -0.012690,     -0.022330,     -0.008870,
		     -0.004960,      0.007550,      0.016710,      0.016780,      0.016250,
		      0.021570,     -0.004830,      0.018730,      0.005840,      0.006640,
		     -0.006890,      0.002380,     -0.000900,     -0.013040,      0.003110,
		     -0.003670,     -0.002490,      0.013330,     -0.012390,
	};
	static const std::vector<double> h = {
		      0.000000,  -9889.000000,      0.000000,  11230.000000,    -70.000000,
		      0.000000,  -3669.000000,   1791.000000,   -770.000000,      0.000000,
		     -0.018890,      0.026070,      0.012040,     -0.004560,      0.000000,
		     -0.007390,     -0.011340,      0.010670,     -0.015510,     -0.010900,
		      0.000000,      0.044320,     -0.015980,      0.017210,      0.003700,
		     -0.019320,      0.014390,      0.000000,     -0.031590,      0.018620,
		     -0.011200,      0.005150,      0.019230,     -0.027490,      0.033440,
		      0.000000,      0.014460,     -0.000790,      0.010430,     -0.000220,
		     -0.004650,      0.010430,     -0.021380,      0.025190,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : neptune ---  Model : gsfco8 */
coeffStruct& _model_coeff_gsfco8() {
	static const std::string name = "gsfco8";
	static const std::string body = "neptune";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		   9732.000000,   3220.000000,   7448.000000,    664.000000,   4499.000000,
		  -6592.000000,   4098.000000,  -3581.000000,    484.000000,
	};
	static const std::vector<double> h = {
		      0.000000,  -9889.000000,      0.000000,  11230.000000,    -70.000000,
		      0.000000,  -3669.000000,   1791.000000,   -770.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : neptune ---  Model : nmoh */
coeffStruct& _model_coeff_nmoh() {
	static const std::string name = "nmoh";
	static const std::string body = "neptune";
	static const int len = 152;
	static const int nmax = 16;
	static const int ndef = 16;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,  14,  14,  14,  14,  14,  14,  14,  14,
		  14,  14,  14,  14,  14,  14,  14,  15,  15,  15,  15,  15,  15,  15,  15,  15,
		  15,  15,  15,  15,  15,  15,  15,  16,  16,  16,  16,  16,  16,  16,  16,  16,
		  16,  16,  16,  16,  16,  16,  16,  16,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,
	};
	static const std::vector<double> g = {
		  10335.720000,   3558.890000,   8565.940000,   -405.990000,   4643.560000,
		  -5749.290000,  11631.960000,  -1888.700000,  -2920.490000,   1851.870000,
		   1331.630000,   5410.370000,   2219.700000,  -2912.020000,  -2346.740000,
		  -2065.290000,   1925.340000,   -777.380000,   1661.950000,  -1909.030000,
		  -2269.280000,   -740.160000,     62.250000,    790.500000,   -330.890000,
		  -1353.390000,    424.120000,   -168.260000,     21.350000,   1018.200000,
		   1135.410000,   -688.690000,    -50.970000,   -692.100000,    920.700000,
		    212.950000,    -89.990000,    908.860000,    108.490000,   -176.230000,
		   -564.440000,    239.920000,    -37.800000,    317.410000,   -248.950000,
		    -75.710000,     51.870000,    -96.240000,   -369.890000,   -266.320000,
		     93.350000,    120.180000,    -14.490000,     31.810000,   -265.870000,
		     -1.500000,   -123.980000,      0.560000,   -292.270000,     40.230000,
		    -13.760000,    205.040000,   -114.740000,     45.550000,    -28.000000,
		    -43.300000,      5.650000,     67.610000,     -9.960000,    -51.470000,
		     61.200000,     42.620000,     90.440000,    -34.150000,    -35.930000,
		     36.040000,    -25.120000,     35.190000,    -16.820000,    114.250000,
		    -62.070000,     33.690000,      0.810000,     52.250000,     -5.570000,
		     10.360000,    -59.350000,     40.200000,    -12.440000,     -2.860000,
		    -10.210000,    -25.430000,     29.250000,    -71.140000,      1.950000,
		    -16.440000,     14.550000,    -22.860000,      5.700000,    -33.560000,
		     15.880000,      7.300000,    -11.570000,      5.020000,    -46.550000,
		    -17.530000,    -30.630000,    -43.140000,    -25.360000,      3.480000,
		    -13.000000,     -1.270000,     -8.000000,     -1.880000,     -3.590000,
		     16.400000,    -12.130000,      3.000000,      1.180000,    -35.130000,
		     -6.960000,    -24.850000,    -14.450000,    -16.040000,     21.040000,
		    -15.760000,     15.620000,    -12.450000,     12.730000,     -9.720000,
		     14.040000,     -7.390000,     -0.480000,      2.430000,     -0.770000,
		     -4.020000,     -2.630000,      8.270000,     -4.800000,      8.290000,
		     17.650000,     -4.710000,     13.050000,     -7.320000,      9.980000,
		     -6.450000,      6.300000,     -1.810000,     -2.990000,      2.740000,
		     -0.610000,     -0.220000,
	};
	static const std::vector<double> h = {
		      0.000000,  -9772.350000,      0.000000,  11138.620000,   -743.140000,
		      0.000000,  -3904.670000,    902.890000,   -245.310000,      0.000000,
		  -2139.600000,   3233.010000,    -50.790000,  -1811.490000,      0.000000,
		   1323.320000,  -1762.770000,   1477.530000,  -2025.080000,  -2940.090000,
		      0.000000,   1624.960000,    518.820000,   1200.430000,    988.610000,
		   -800.070000,   -694.410000,      0.000000,   -300.270000,    701.470000,
		    637.150000,    254.000000,    427.650000,   -904.050000,    382.700000,
		      0.000000,   -489.090000,    352.130000,    235.000000,    -57.240000,
		    -15.100000,   -321.080000,    -73.720000,    204.160000,      0.000000,
		    189.910000,     86.280000,    330.790000,    -69.580000,   -217.500000,
		    108.990000,   -279.720000,    299.000000,   -105.640000,      0.000000,
		    253.640000,    -13.240000,    206.480000,    -41.340000,   -204.700000,
		    141.950000,   -111.460000,     75.810000,     57.690000,    -82.260000,
		      0.000000,     -4.640000,    -18.370000,    -12.320000,    -15.990000,
		   -161.890000,     67.480000,     19.190000,    -57.060000,     95.120000,
		    -69.430000,      7.020000,      0.000000,    -81.580000,    -17.200000,
		    -48.730000,    -28.680000,    -57.610000,    -14.810000,     89.580000,
		    -96.770000,     72.680000,    -29.670000,    -13.100000,     14.800000,
		      0.000000,     -9.780000,    -18.860000,     17.090000,    -43.120000,
		     33.550000,    -54.100000,    104.530000,    -83.490000,     40.830000,
		     -1.620000,    -18.120000,     11.640000,      0.830000,      0.000000,
		     27.650000,     -6.290000,     35.380000,    -25.360000,     44.310000,
		    -39.870000,     69.010000,    -45.040000,      9.540000,     14.300000,
		    -16.810000,      6.450000,      2.580000,     -1.940000,      0.000000,
		     -8.820000,     17.400000,     -9.900000,     13.070000,      0.180000,
		     -2.320000,     17.850000,     -8.410000,    -12.220000,     19.800000,
		    -13.420000,      2.650000,      2.740000,     -1.460000,     -0.370000,
		      0.000000,    -60.830000,     37.450000,    -61.250000,     45.310000,
		    -43.130000,     27.520000,    -17.200000,     12.490000,    -20.230000,
		     17.960000,     -9.170000,      0.400000,      2.200000,     -0.700000,
		     -0.570000,      0.230000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : ganymede ---  Model : kivelson2002a */
coeffStruct& _model_coeff_kivelson2002a() {
	static const std::string name = "kivelson2002a";
	static const std::string body = "ganymede";
	static const int len = 5;
	static const int nmax = 2;
	static const int ndef = 2;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,
	};
	static const std::vector<double> g = {
		   -711.000000,     46.800000,      0.900000,     27.000000,     -0.400000,
	};
	static const std::vector<double> h = {
		      0.000000,     22.300000,      0.000000,      1.800000,    -11.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : ganymede ---  Model : weber2022quad */
coeffStruct& _model_coeff_weber2022quad() {
	static const std::string name = "weber2022quad";
	static const std::string body = "ganymede";
	static const int len = 5;
	static const int nmax = 2;
	static const int ndef = 2;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,
	};
	static const std::vector<double> g = {
		   -748.300000,     41.100000,     22.500000,     23.300000,    -26.800000,
	};
	static const std::vector<double> h = {
		      0.000000,     20.800000,      0.000000,     16.500000,    -10.600000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : ganymede ---  Model : kivelson2002b */
coeffStruct& _model_coeff_kivelson2002b() {
	static const std::string name = "kivelson2002b";
	static const std::string body = "ganymede";
	static const int len = 2;
	static const int nmax = 1;
	static const int ndef = 1;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,
	};
	static const std::vector<int> m = {
		   0,   1,
	};
	static const std::vector<double> g = {
		   -727.300000,     52.800000,
	};
	static const std::vector<double> h = {
		      0.000000,     18.400000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : ganymede ---  Model : kivelson2002c */
coeffStruct& _model_coeff_kivelson2002c() {
	static const std::string name = "kivelson2002c";
	static const std::string body = "ganymede";
	static const int len = 2;
	static const int nmax = 1;
	static const int ndef = 1;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,
	};
	static const std::vector<int> m = {
		   0,   1,
	};
	static const std::vector<double> g = {
		   -716.800000,     49.300000,
	};
	static const std::vector<double> h = {
		      0.000000,     22.200000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : ganymede ---  Model : weber2022dip */
coeffStruct& _model_coeff_weber2022dip() {
	static const std::string name = "weber2022dip";
	static const std::string body = "ganymede";
	static const int len = 2;
	static const int nmax = 1;
	static const int ndef = 1;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,
	};
	static const std::vector<int> m = {
		   0,   1,
	};
	static const std::vector<double> g = {
		   -716.400000,     56.000000,
	};
	static const std::vector<double> h = {
		      0.000000,     27.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : mercury ---  Model : uno2009 */
coeffStruct& _model_coeff_uno2009() {
	static const std::string name = "uno2009";
	static const std::string body = "mercury";
	static const int len = 44;
	static const int nmax = 8;
	static const int ndef = 8;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,
	};
	static const std::vector<double> g = {
		   -227.100000,     11.000000,    -20.700000,      3.100000,      5.100000,
		    -10.300000,     -1.500000,      5.800000,      1.900000,     -4.700000,
		     -0.600000,      2.700000,      1.200000,      1.800000,     -3.300000,
		     -0.400000,      0.800000,     -0.200000,      0.300000,      0.700000,
		     -1.000000,     -0.300000,      0.200000,     -0.500000,     -0.200000,
		     -0.200000,      0.300000,      0.200000,      0.100000,      0.100000,
		     -0.200000,     -0.100000,     -0.200000,     -0.200000,      0.100000,
		      0.100000,      0.300000,      0.100000,     -0.100000,      0.000000,
		      0.000000,      0.100000,     -0.100000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      7.000000,      0.000000,     -3.300000,      3.600000,
		      0.000000,     -4.300000,      2.300000,     -0.400000,      0.000000,
		     -1.700000,      1.500000,      2.000000,      0.000000,      0.000000,
		     -1.200000,      1.000000,      1.400000,      1.100000,      0.300000,
		      0.000000,     -1.000000,      0.100000,      0.400000,      0.500000,
		      0.500000,      0.100000,      0.000000,     -0.400000,     -0.100000,
		      0.000000,      0.000000,      0.200000,      0.100000,      0.100000,
		      0.000000,     -0.100000,      0.100000,     -0.100000,     -0.100000,
		      0.000000,      0.100000,      0.000000,      0.100000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : mercury ---  Model : thebault2018m2 */
coeffStruct& _model_coeff_thebault2018m2() {
	static const std::string name = "thebault2018m2";
	static const std::string body = "mercury";
	static const int len = 20;
	static const int nmax = 5;
	static const int ndef = 5;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,
	};
	static const std::vector<double> g = {
		   -213.600000,      0.900000,    -57.700000,      0.000000,      0.000000,
		    -35.800000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      2.100000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      1.500000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : mercury ---  Model : uno2009svd */
coeffStruct& _model_coeff_uno2009svd() {
	static const std::string name = "uno2009svd";
	static const std::string body = "mercury";
	static const int len = 5;
	static const int nmax = 2;
	static const int ndef = 2;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,
	};
	static const std::vector<double> g = {
		   -169.600000,      6.700000,    -56.000000,      7.900000,     13.700000,
	};
	static const std::vector<double> h = {
		      0.000000,     17.500000,      0.000000,    -45.400000,    -16.300000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : mercury ---  Model : anderson2010qts04 */
coeffStruct& _model_coeff_anderson2010qts04() {
	static const std::string name = "anderson2010qts04";
	static const std::string body = "mercury";
	static const int len = 5;
	static const int nmax = 2;
	static const int ndef = 2;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,
	};
	static const std::vector<double> g = {
		   -213.000000,     -4.000000,    -66.000000,      9.000000,      4.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      7.000000,      0.000000,      5.000000,     -4.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : mercury ---  Model : anderson2010dsha */
coeffStruct& _model_coeff_anderson2010dsha() {
	static const std::string name = "anderson2010dsha";
	static const std::string body = "mercury";
	static const int len = 2;
	static const int nmax = 1;
	static const int ndef = 1;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,
	};
	static const std::vector<int> m = {
		   0,   1,
	};
	static const std::vector<double> g = {
		   -249.000000,    -12.000000,
	};
	static const std::vector<double> h = {
		      0.000000,     16.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : mercury ---  Model : anderson2012 */
coeffStruct& _model_coeff_anderson2012() {
	static const std::string name = "anderson2012";
	static const std::string body = "mercury";
	static const int len = 14;
	static const int nmax = 4;
	static const int ndef = 4;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,
	};
	static const std::vector<double> g = {
		   -190.000000,      0.000000,    -74.600000,      0.000000,      0.000000,
		    -22.000000,      0.000000,      0.000000,      0.000000,     -5.700000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : mercury ---  Model : anderson2010dts04 */
coeffStruct& _model_coeff_anderson2010dts04() {
	static const std::string name = "anderson2010dts04";
	static const std::string body = "mercury";
	static const int len = 2;
	static const int nmax = 1;
	static const int ndef = 1;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,
	};
	static const std::vector<int> m = {
		   0,   1,
	};
	static const std::vector<double> g = {
		   -240.000000,     -1.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      5.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : mercury ---  Model : anderson2010qsha */
coeffStruct& _model_coeff_anderson2010qsha() {
	static const std::string name = "anderson2010qsha";
	static const std::string body = "mercury";
	static const int len = 5;
	static const int nmax = 2;
	static const int ndef = 2;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,
	};
	static const std::vector<double> g = {
		   -182.000000,    -15.000000,   -108.000000,     10.000000,      2.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      9.000000,      0.000000,      6.000000,    -15.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : mercury ---  Model : ness1975 */
coeffStruct& _model_coeff_ness1975() {
	static const std::string name = "ness1975";
	static const std::string body = "mercury";
	static const int len = 2;
	static const int nmax = 1;
	static const int ndef = 1;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,
	};
	static const std::vector<int> m = {
		   0,   1,
	};
	static const std::vector<double> g = {
		   -344.000000,     16.000000,
	};
	static const std::vector<double> h = {
		      0.000000,    -59.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : mercury ---  Model : thebault2018m1 */
coeffStruct& _model_coeff_thebault2018m1() {
	static const std::string name = "thebault2018m1";
	static const std::string body = "mercury";
	static const int len = 20;
	static const int nmax = 5;
	static const int ndef = 5;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,
	};
	static const std::vector<double> g = {
		   -213.200000,      1.200000,    -56.500000,      0.000000,      0.000000,
		    -34.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      2.700000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      1.600000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : mercury ---  Model : anderson2010r */
coeffStruct& _model_coeff_anderson2010r() {
	static const std::string name = "anderson2010r";
	static const std::string body = "mercury";
	static const int len = 5;
	static const int nmax = 2;
	static const int ndef = 2;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,
	};
	static const std::vector<double> g = {
		   -222.000000,     12.000000,    -24.000000,      9.000000,      9.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      2.000000,      0.000000,     -6.000000,      8.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : mercury ---  Model : anderson2010d */
coeffStruct& _model_coeff_anderson2010d() {
	static const std::string name = "anderson2010d";
	static const std::string body = "mercury";
	static const int len = 2;
	static const int nmax = 1;
	static const int ndef = 1;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,
	};
	static const std::vector<int> m = {
		   0,   1,
	};
	static const std::vector<double> g = {
		   -216.000000,     -6.000000,
	};
	static const std::vector<double> h = {
		      0.000000,     14.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : mercury ---  Model : anderson2010q */
coeffStruct& _model_coeff_anderson2010q() {
	static const std::string name = "anderson2010q";
	static const std::string body = "mercury";
	static const int len = 5;
	static const int nmax = 2;
	static const int ndef = 2;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,
	};
	static const std::vector<double> g = {
		   -173.000000,     -7.000000,   -108.000000,     -9.000000,     -1.000000,
	};
	static const std::vector<double> h = {
		      0.000000,     15.000000,      0.000000,     16.000000,    -17.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : mercury ---  Model : thebault2018m3 */
coeffStruct& _model_coeff_thebault2018m3() {
	static const std::string name = "thebault2018m3";
	static const std::string body = "mercury";
	static const int len = 20;
	static const int nmax = 5;
	static const int ndef = 5;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,
	};
	static const std::vector<double> g = {
		   -198.800000,      0.900000,    -77.600000,      0.000000,      0.000000,
		    -20.600000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.600000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000,      1.800000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : mars ---  Model : gao2021 */
coeffStruct& _model_coeff_gao2021() {
	static const std::string name = "gao2021";
	static const std::string body = "mars";
	static const int len = 6215;
	static const int nmax = 110;
	static const int ndef = 110;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,  14,  14,  14,  14,  14,  14,  14,  14,
		  14,  14,  14,  14,  14,  14,  14,  15,  15,  15,  15,  15,  15,  15,  15,  15,
		  15,  15,  15,  15,  15,  15,  15,  16,  16,  16,  16,  16,  16,  16,  16,  16,
		  16,  16,  16,  16,  16,  16,  16,  16,  17,  17,  17,  17,  17,  17,  17,  17,
		  17,  17,  17,  17,  17,  17,  17,  17,  17,  17,  18,  18,  18,  18,  18,  18,
		  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  19,  19,  19,
		  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,
		  19,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,
		  20,  20,  20,  20,  20,  20,  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,
		  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,  22,  22,  22,  22,
		  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,
		  22,  22,  22,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,
		  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  24,  24,  24,  24,  24,
		  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,
		  24,  24,  24,  24,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,
		  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  26,  26,
		  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,
		  26,  26,  26,  26,  26,  26,  26,  26,  26,  27,  27,  27,  27,  27,  27,  27,
		  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,
		  27,  27,  27,  27,  27,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,
		  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,
		  28,  28,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,
		  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,
		  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,
		  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  31,
		  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,
		  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  32,
		  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,
		  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,
		  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,
		  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,
		  33,  33,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,
		  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,
		  34,  34,  34,  34,  34,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,
		  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,
		  35,  35,  35,  35,  35,  35,  35,  35,  35,  36,  36,  36,  36,  36,  36,  36,
		  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,
		  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  37,  37,
		  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,
		  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,
		  37,  37,  37,  37,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,
		  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,
		  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  39,  39,  39,  39,  39,
		  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,
		  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,
		  39,  39,  39,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,
		  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,
		  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  41,  41,  41,  41,
		  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,
		  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,
		  41,  41,  41,  41,  41,  41,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,
		  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,
		  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,
		  42,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,
		  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,
		  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  44,  44,  44,
		  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,
		  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,
		  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  45,  45,  45,  45,  45,  45,
		  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,
		  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,
		  45,  45,  45,  45,  45,  45,  45,  45,  46,  46,  46,  46,  46,  46,  46,  46,
		  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,
		  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,
		  46,  46,  46,  46,  46,  46,  46,  47,  47,  47,  47,  47,  47,  47,  47,  47,
		  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,
		  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,
		  47,  47,  47,  47,  47,  47,  47,  48,  48,  48,  48,  48,  48,  48,  48,  48,
		  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,
		  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,
		  48,  48,  48,  48,  48,  48,  48,  48,  49,  49,  49,  49,  49,  49,  49,  49,
		  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,
		  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,
		  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  50,  50,  50,  50,  50,  50,
		  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,
		  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,
		  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  51,  51,  51,
		  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,
		  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,
		  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,
		  51,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,
		  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,
		  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,
		  52,  52,  52,  52,  52,  52,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,
		  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,
		  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,
		  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  54,  54,  54,  54,
		  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,
		  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,
		  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,
		  54,  54,  54,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,
		  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,
		  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,
		  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  56,  56,  56,  56,  56,
		  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,
		  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,
		  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,
		  56,  56,  56,  56,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,
		  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,
		  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,
		  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  58,  58,
		  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,
		  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,
		  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,
		  58,  58,  58,  58,  58,  58,  58,  58,  58,  59,  59,  59,  59,  59,  59,  59,
		  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,
		  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,
		  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,
		  59,  59,  59,  59,  59,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,
		  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,
		  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,
		  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,
		  60,  60,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,
		  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,
		  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,
		  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,
		  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,
		  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,
		  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,
		  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  63,
		  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,
		  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,
		  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,
		  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  64,
		  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,
		  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,
		  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,
		  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,
		  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,
		  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,
		  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,
		  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,
		  65,  65,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,
		  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,
		  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,
		  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,
		  66,  66,  66,  66,  66,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,
		  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,
		  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,
		  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,
		  67,  67,  67,  67,  67,  67,  67,  67,  67,  68,  68,  68,  68,  68,  68,  68,
		  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,
		  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,
		  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,
		  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  69,  69,
		  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,
		  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,
		  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,
		  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,
		  69,  69,  69,  69,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,
		  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,
		  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,
		  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,
		  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  71,  71,  71,  71,  71,
		  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,
		  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,
		  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,
		  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,
		  71,  71,  71,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,
		  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,
		  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,
		  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,
		  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,
		  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,
		  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,
		  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,
		  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  91,  91,  91,  91,  91,  91,  91,
		  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,
		  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,
		  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,
		  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,
		  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,
		  91,  91,  91,  91,  91,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,
		  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,
		  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,
		  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,
		  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,
		  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,
		  92,  92,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,
		  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,
		  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,
		  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,
		  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,
		  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,
		  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,
		  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,
		  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,
		  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,
		  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,
		  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  95,
		  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,
		  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,
		  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,
		  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,
		  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,
		  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  96,
		  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,
		  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,
		  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,
		  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,
		  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,
		  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,
		  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,
		  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,
		  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,
		  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,
		  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,
		  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,
		  97,  97,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,
		  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,
		  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,
		  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,
		  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,
		  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,
		  98,  98,  98,  98,  98,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,
		  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,
		  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,
		  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,
		  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,
		  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,
		  99,  99,  99,  99,  99,  99,  99,  99,  99, 100, 100, 100, 100, 100, 100, 100,
		 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
		 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
		 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
		 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
		 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
		 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 101, 101,
		 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
		 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
		 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
		 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
		 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
		 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
		 101, 101, 101, 101, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102,
		 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102,
		 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102,
		 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102,
		 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102,
		 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102,
		 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 103, 103, 103, 103, 103,
		 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103,
		 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103,
		 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103,
		 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103,
		 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103,
		 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103,
		 103, 103, 103, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104,
		 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104,
		 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104,
		 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104,
		 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104,
		 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104,
		 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
		 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
		 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
		 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
		 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
		 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
		 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
		  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
		  41,  42,  43,  44,  45,  46,  47,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
		  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
		  41,  42,  43,  44,  45,  46,  47,  48,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,
		  51,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,
		  47,  48,  49,  50,  51,  52,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,
		  52,  53,  54,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,
		  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,
		  53,  54,  55,  56,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,
		  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,
		  50,  51,  52,  53,  54,  55,  56,  57,  58,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,
		  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
		  55,  56,  57,  58,  59,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,
		  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,
		  59,  60,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,
		  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
		  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,
		  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,
		  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
		  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,
		  64,  65,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,
		  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,
		  62,  63,  64,  65,  66,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,
		  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,
		  59,  60,  61,  62,  63,  64,  65,  66,  67,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,
		  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
		  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,
		  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,
		  66,  67,  68,  69,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,
		  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,
		  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,
		  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,
		  69,  70,  71,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,
		  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,
		  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,
		  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,
		  68,  69,  70,  71,  72,  73,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,
		  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,
		  74,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,
		  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,
		  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,
		  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,
		  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,
		  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,
		  70,  71,  72,  73,  74,  75,  76,  77,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,
		  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,
		  72,  73,  74,  75,  76,  77,  78,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
		  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
		  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,
		  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,
		  73,  74,  75,  76,  77,  78,  79,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
		  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
		  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,
		  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,
		  73,  74,  75,  76,  77,  78,  79,  80,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,
		  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,
		  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,
		  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,
		  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,
		  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,
		  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,
		  83,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,
		  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,
		  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,
		  79,  80,  81,  82,  83,  84,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,
		  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,
		  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,
		  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,
		  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,
		  84,  85,  86,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,
		  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,
		  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,
		  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,
		  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,
		  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,
		  85,  86,  87,  88,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,
		  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,
		  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,
		  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,
		  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,
		  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,
		  82,  83,  84,  85,  86,  87,  88,  89,  90,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,
		  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
		  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,
		  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,
		  87,  88,  89,  90,  91,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,
		  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,
		  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,
		  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,
		  91,  92,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,
		  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,
		  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,
		  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
		  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,
		  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,
		  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,
		  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,
		  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,
		  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,
		  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,
		  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,
		  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
		  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,
		  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,
		  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,
		  96,  97,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,
		  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,
		  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,
		  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,
		  94,  95,  96,  97,  98,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,
		  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,
		  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,
		  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,
		  91,  92,  93,  94,  95,  96,  97,  98,  99,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,
		  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
		  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,
		  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,
		  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,
		  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,
		  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,
		  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,
		  98,  99, 100, 101,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,
		  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,
		  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,
		  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,
		  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,
		  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,
		  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,
		  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100,
		 101, 102, 103,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,
		  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,
		  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,
		  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,
		  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103, 104,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,
		  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,
		  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,
		  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99,
		 100, 101, 102, 103, 104, 105,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,
		  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,
		  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,
		  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103, 104, 105,
		 106,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,
		  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,
		  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,
		  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,
		  95,  96,  97,  98,  99, 100, 101, 102, 103, 104, 105, 106, 107,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,
		  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,
		  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,
		  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,
		  99, 100, 101, 102, 103, 104, 105, 106, 107, 108,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,
		  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,
		  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,
		  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100, 101,
		 102, 103, 104, 105, 106, 107, 108, 109,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,
		  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,
		  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,
		  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103,
		 104, 105, 106, 107, 108, 109, 110,
	};
	static const std::vector<double> g = {
		     -1.103552,     -0.285049,      0.052022,      1.059229,      1.047818,
		      0.528941,     -0.590178,     -0.783253,     -0.952194,     -0.909502,
		     -0.805215,      0.115581,      2.167028,      0.665974,      0.521849,
		      0.211260,      0.442377,     -0.182507,      0.491747,      0.565727,
		     -1.146855,     -0.216511,      0.024195,     -1.751028,      0.479033,
		      0.300211,     -0.961832,      1.020821,      1.268267,     -0.706555,
		      0.509822,      0.204439,      0.607324,      1.116579,      1.383266,
		     -0.816299,     -3.176864,      0.811493,      1.385293,     -0.740147,
		     -0.828644,     -0.957786,     -2.097678,     -0.951608,     -0.810744,
		      3.067185,      1.482236,     -1.599490,     -0.349348,      0.750188,
		      0.772647,      0.464582,      0.569356,      1.471306,      2.554274,
		     -0.340154,     -4.095314,     -0.668401,      1.694483,      0.682120,
		     -0.343301,     -0.083650,     -0.860916,     -1.769472,     -0.752501,
		     -2.228412,     -2.113461,      2.988525,      2.443945,     -1.059818,
		     -2.040940,     -0.418892,      0.200440,      0.466140,      0.470180,
		      0.898973,      1.062722,      0.874060,      3.275093,     -0.100281,
		     -2.123036,     -1.740526,      0.654291,      1.245793,     -0.085582,
		     -0.093818,      0.852412,     -1.418445,     -2.386195,     -3.172800,
		      0.042697,     -2.909867,     -2.196452,      2.199222,      3.680481,
		      1.261187,     -0.327065,     -0.584760,      0.056411,      0.131410,
		      1.029875,      2.315609,      2.168222,     -0.138303,     -2.405196,
		      0.596803,      2.796693,     -0.977885,     -2.212144,     -1.959702,
		     -0.008103,      0.803323,      0.450036,      0.307279,      1.122813,
		     -1.293680,     -1.835255,     -2.421894,     -1.541587,      3.790654,
		      0.413251,     -2.942898,     -1.777969,      0.507951,      2.066802,
		      0.150063,     -0.268452,     -1.049121,     -1.170243,     -1.137838,
		     -0.355585,      0.703739,      2.547435,      1.846657,      0.605022,
		     -2.569413,     -3.283098,      2.059019,      3.879068,     -0.062173,
		     -1.567065,     -0.780043,     -0.566281,      0.558376,      0.481921,
		      1.059747,      1.253810,      0.458545,      0.131479,     -2.188061,
		     -0.955249,      0.248072,     -1.571001,      5.630746,      0.362432,
		     -4.081537,     -1.005372,      1.707546,      2.560283,     -0.275994,
		     -0.347441,     -0.444860,     -0.397061,     -0.864507,     -0.181683,
		     -0.476452,      2.266882,      1.291988,      1.376052,      1.322157,
		      3.225038,     -5.779642,     -4.036623,      1.551243,      2.699920,
		     -0.016322,     -1.728414,      1.067522,     -0.188052,      0.542437,
		      0.899423,      1.687587,      1.893431,      0.706149,     -0.255010,
		     -0.580916,     -0.278313,      0.255950,     -0.872629,     -2.453308,
		      0.406760,      6.035156,      0.871100,     -2.939964,     -2.126930,
		      0.390463,      1.741436,      0.326344,     -0.517124,     -1.455116,
		     -2.969388,     -2.829924,     -0.515129,      0.051164,      0.982481,
		      0.428114,      0.103565,      1.122106,     -1.638582,      3.875441,
		      3.585148,     -5.471646,     -3.484817,      2.092610,      1.876638,
		     -0.669649,     -1.581849,      1.310302,      0.537869,      0.981242,
		      2.907445,      2.344485,      1.487105,      0.472767,     -0.050575,
		      0.272364,      0.576014,      0.689072,      1.005607,     -0.599202,
		     -2.923530,     -3.384097,      2.071884,      4.443602,      0.529389,
		     -0.210282,      0.056927,      0.779170,     -0.859579,     -0.111887,
		      1.159511,     -0.089584,     -1.390191,     -2.808927,     -1.166704,
		     -0.784571,      0.251341,      0.823356,     -0.367220,     -0.844964,
		     -1.265468,     -1.359509,      0.659236,      3.574530,      1.316253,
		     -3.557654,     -2.345613,      0.195072,      0.586186,     -0.425941,
		     -0.702128,      0.960917,     -1.651330,     -1.393587,      1.090265,
		      1.885174,      0.999464,     -0.695389,     -2.096407,     -0.501498,
		      0.603881,      0.559600,      1.092041,      0.313762,      1.331554,
		      2.070963,     -4.896511,     -2.762313,      2.974616,      1.832508,
		     -1.155976,     -0.740325,      0.373899,      1.568159,     -1.272878,
		     -1.535688,      1.320155,      0.081461,     -1.370428,     -1.718309,
		      1.493633,      1.673673,      0.852195,      0.998388,     -0.631802,
		      0.281351,     -1.420846,     -1.372015,      0.733432,     -1.412411,
		      3.017374,      1.801738,     -1.311365,     -1.173069,     -0.304932,
		     -0.531440,     -0.887271,     -0.929837,      0.725894,      2.531313,
		     -1.145178,     -1.728080,      1.076687,      2.264489,     -0.269513,
		     -1.271051,     -2.002001,     -1.612503,     -1.223244,     -0.375119,
		      0.534062,      1.956868,      1.680879,      1.766492,      1.473416,
		      0.266340,     -1.469601,     -0.679661,      2.693065,      1.357530,
		     -0.068530,      0.126442,     -0.522606,      0.190083,     -0.286270,
		     -0.178563,      0.045211,     -0.853178,     -2.278866,     -2.371536,
		      1.112019,      2.522573,      0.834714,      1.189207,     -1.137891,
		     -0.888685,     -0.214265,     -2.255621,     -0.252700,      0.124139,
		     -0.605642,     -0.245421,      2.317547,      0.866021,     -2.791290,
		     -0.446626,      0.182858,     -0.100628,     -0.532857,     -2.429649,
		     -1.597019,      2.430065,      1.006522,     -0.188840,      1.099705,
		      3.552283,      0.057421,     -0.807727,      0.275646,     -1.018337,
		     -0.313770,      0.093418,      0.775048,      1.770274,      1.646113,
		      2.285808,     -0.100678,      0.982235,      0.028810,     -2.016974,
		      0.057929,      0.095144,      0.647152,      0.512572,      0.870709,
		      1.681481,      0.338847,     -0.612220,     -1.752323,      1.188910,
		      1.104104,     -0.339723,     -2.057713,     -2.415545,     -2.026906,
		     -0.766838,     -0.843316,      0.714209,      0.186233,     -0.171194,
		     -0.085424,     -2.795615,     -2.137717,     -0.108738,      0.054787,
		     -1.034051,      0.508024,      1.218802,      1.285354,      0.981533,
		     -1.429793,      0.269414,      0.410753,     -0.837108,      1.578120,
		      1.748362,     -1.583688,     -1.531753,     -0.886341,     -0.856805,
		      1.596533,      3.194726,      1.289925,      0.237198,      2.154817,
		      1.210999,      0.949886,      1.648463,      0.586309,      1.509685,
		      1.761524,      0.507631,     -1.146668,     -0.654031,      0.161664,
		     -1.111012,     -0.494926,     -1.528817,      1.283055,      1.128446,
		     -1.931969,     -1.769473,     -0.778847,     -0.174453,      0.697845,
		      0.484399,     -1.455958,      0.661634,      0.562657,     -1.153625,
		     -2.384906,     -0.067028,     -0.014140,     -1.587551,     -2.363846,
		     -2.164527,      0.241510,     -0.010970,      1.868687,     -1.247813,
		     -1.370613,      0.939836,     -0.768526,     -0.198523,     -0.485560,
		      0.000884,      0.030494,     -0.213419,     -1.222544,     -0.072346,
		      0.021775,      1.081378,      1.801522,     -1.578090,     -1.848609,
		      0.444176,      1.385239,     -0.557894,      0.053584,      0.514471,
		      1.649102,      0.999984,      0.187710,      0.486172,      3.644512,
		      0.880886,      0.792025,     -0.316454,     -1.352437,      0.591687,
		      0.278583,      0.470171,     -0.444923,     -0.866246,      1.557238,
		      2.106953,      0.233207,     -0.978615,      0.402914,     -1.019342,
		      1.742707,      3.188411,     -0.566622,     -1.491793,      0.160387,
		     -0.244708,      0.317541,      1.097928,     -0.390543,     -0.404659,
		      0.453809,     -2.692539,     -2.673845,     -0.784234,     -0.089959,
		     -1.701625,     -0.360615,     -1.780938,     -0.525399,     -1.117302,
		      0.040016,      1.306189,      0.989436,      0.867315,      1.816582,
		     -0.011371,     -1.406557,     -0.859637,      0.237756,      0.591176,
		      0.380316,      1.565786,     -1.259384,     -2.360523,     -0.413455,
		      0.195440,      2.124383,      1.436121,     -1.291665,     -1.333440,
		     -0.729588,     -1.274632,     -1.814798,      1.751368,      4.637307,
		      1.294796,      0.388500,      0.279114,      2.987139,      0.854357,
		      0.073590,      1.587592,     -0.344619,     -1.278984,     -1.119289,
		     -0.275152,      0.199957,     -1.195515,      1.287112,      0.829061,
		     -0.005145,     -0.710098,     -0.716209,     -0.641198,      0.921540,
		     -0.958997,      0.690614,      2.796644,     -0.277915,     -1.500732,
		     -0.461273,     -0.763908,     -0.104125,      1.624466,      2.066709,
		      1.956773,      1.914168,     -2.899418,     -3.615368,     -0.649724,
		      1.403753,     -2.103995,      0.355175,     -2.116304,     -0.711243,
		     -0.152162,     -0.737822,      1.087509,     -0.538904,      1.378219,
		      2.381853,      0.302997,      0.206237,     -0.988135,     -0.798732,
		      0.055170,      2.311561,     -0.753225,     -2.830327,      2.065599,
		     -0.126058,     -1.102437,      0.696090,     -0.719640,     -0.640230,
		      2.335463,      1.407575,     -1.076514,     -0.956348,     -1.801617,
		     -5.107101,     -0.984341,      4.136210,      2.488911,     -0.112607,
		     -0.437275,      0.386054,      1.119484,     -1.541734,      0.288616,
		      1.182531,     -0.108015,      0.247673,     -0.926854,     -1.299261,
		      0.521943,     -0.034027,     -0.803876,     -2.810809,      2.978485,
		      0.592420,     -3.398385,      0.206573,      1.072388,      1.320624,
		      0.801148,      0.436891,      1.174473,     -0.039135,      0.543595,
		      0.873617,     -0.911827,     -1.839614,      0.391814,      1.682938,
		      3.543302,      3.898157,     -1.235678,     -3.031709,     -2.115242,
		      1.600902,      0.340620,      1.848363,      0.828941,     -0.693995,
		      1.487803,     -0.684310,     -0.723361,     -0.098806,     -0.295834,
		      1.018928,      1.691366,      1.764995,      0.792313,     -1.707349,
		     -2.376662,     -3.149409,      1.257474,      3.971814,      1.419789,
		     -3.115069,      0.308361,     -1.265459,     -1.352863,      1.989984,
		     -0.024788,     -4.955146,     -1.488289,      2.256720,      0.535595,
		     -0.118441,      1.117225,     -3.671731,     -2.676887,      2.138039,
		      2.806316,      0.672706,     -0.364418,     -1.991488,     -0.016890,
		     -1.192196,     -0.753207,      0.241872,     -0.084441,      0.992274,
		      1.884978,     -0.781521,     -1.441262,     -1.669506,      1.082031,
		     -1.284505,     -3.344522,     -0.094029,      3.981097,     -0.321293,
		     -5.239678,     -1.607462,      0.507613,     -0.810947,      2.458481,
		      1.577985,     -0.696431,     -1.298686,      2.731627,      4.251875,
		      0.631565,     -2.731510,     -1.088452,     -0.789058,      2.609335,
		      4.287339,      1.092965,     -1.759430,     -3.198394,     -1.359722,
		     -0.100888,      0.622830,      2.473457,     -0.199786,      0.895540,
		     -1.366602,     -1.744943,     -0.834898,      0.033876,      0.728520,
		     -0.948671,     -1.059921,      2.196287,      2.319946,     -0.322682,
		      2.129747,     -3.007728,     -0.744251,      1.018687,      1.252300,
		      2.302161,      0.385007,     -0.502637,     -1.763360,     -0.529663,
		      4.042786,      2.421630,     -5.408877,     -3.901461,      0.999824,
		     -0.057600,     -1.058871,     -1.015764,     -3.667833,     -3.051873,
		      0.722461,      3.362671,      2.415808,      0.885804,     -0.440197,
		     -0.673466,     -0.088288,     -0.533917,     -0.621568,     -0.011137,
		      0.318754,      2.024485,     -0.512594,      0.866286,      1.070218,
		     -2.012050,     -0.666109,     -0.177512,      1.316879,     -1.935666,
		      2.657896,      2.676517,      0.651219,     -0.764559,     -1.015514,
		     -0.043011,     -1.472673,      2.085656,      1.089244,     -2.740209,
		     -3.557936,      1.859124,      5.086329,      2.997720,     -0.176910,
		      0.999460,     -0.233058,      2.187465,      2.878509,      0.607069,
		     -0.579794,     -2.269252,     -2.092613,     -0.782523,     -1.911152,
		      1.345831,      0.322821,      1.617744,     -1.046745,     -2.819392,
		     -2.938834,      0.300399,     -0.519931,     -0.516907,     -1.014488,
		     -0.929871,     -0.461493,     -0.949827,      0.567426,      0.997203,
		     -1.649416,     -2.781056,      0.736769,      0.062692,     -1.970002,
		      0.633993,      1.793714,     -1.274267,     -1.999363,     -0.711804,
		      3.807850,      3.024429,     -4.314337,     -4.045126,     -0.260399,
		     -0.144096,      0.308837,     -0.067596,     -2.227340,     -2.153498,
		     -0.605709,      1.143564,      1.327421,      0.632611,      1.058192,
		     -0.059399,     -0.409494,     -1.343682,     -1.025118,      2.162665,
		      1.938153,      0.894847,     -2.061444,      1.237621,      1.849343,
		      2.104839,      1.711266,      0.307631,      1.706061,      3.210667,
		     -1.507247,      0.089625,      2.017823,     -1.253584,     -0.711372,
		      1.977284,     -0.830475,     -0.174701,      0.663878,      3.160052,
		      2.452821,     -3.298555,     -4.137987,      1.901342,      3.440038,
		      2.246186,     -0.150244,      0.129179,     -0.605501,      2.570236,
		      2.254550,     -0.037353,     -0.018650,     -1.622332,     -0.987047,
		     -0.442244,     -1.315732,      0.023573,      1.001164,      1.144724,
		      0.557299,     -0.427921,     -1.737796,     -0.292768,     -1.596391,
		     -0.238736,     -0.506669,     -1.722846,     -0.425350,     -1.367305,
		     -0.645634,      0.412799,      3.820647,      0.967197,     -2.688636,
		      1.175552,      3.482807,     -0.360636,     -1.628776,     -0.606230,
		      0.982551,     -1.956527,     -1.390841,      0.037767,      2.521593,
		      1.370951,     -1.980146,     -1.766504,     -1.516103,     -1.298201,
		      0.039702,     -1.406564,     -2.509507,     -0.397684,      1.421845,
		      2.247354,      0.453832,     -0.623332,      0.081087,      0.212009,
		      1.043156,     -1.171268,     -2.271896,      1.422367,      1.912943,
		      1.277773,      0.153878,     -1.746494,      0.632737,      1.441035,
		      3.287807,      3.847726,      1.115089,      1.168171,      0.223305,
		     -2.064280,     -2.027066,      2.146458,      1.311100,     -3.914474,
		     -0.977351,      2.629917,     -1.674655,     -2.214980,     -0.229312,
		      1.899364,      2.843935,     -0.778852,     -3.660358,      1.114650,
		      1.314308,      2.357059,      1.769957,      0.164190,     -1.795473,
		      2.308241,      1.968764,     -2.229877,     -1.694766,     -1.011847,
		     -0.001275,      1.325251,     -0.204409,     -1.180344,     -0.870536,
		      0.303459,      1.355248,      1.443027,     -1.089509,     -0.447844,
		      0.734927,      1.689484,      0.598469,     -2.684989,     -2.710689,
		     -1.862109,      0.429553,      0.387676,     -0.291831,     -0.336312,
		      3.752959,     -1.333755,     -5.206444,      2.939534,      2.904577,
		     -1.298399,      1.132727,      0.976587,      0.853491,     -1.956751,
		     -1.763962,      0.448770,      2.783214,     -0.792719,     -0.299734,
		     -0.029856,     -0.678563,     -1.678131,      0.705990,     -0.877984,
		     -3.091482,     -1.181549,      1.123705,      3.453797,      1.011656,
		     -1.582269,     -0.984906,      0.523157,      1.052318,     -0.703416,
		     -2.811061,     -0.288790,      0.689688,      0.248089,      0.507574,
		     -2.506657,     -1.750543,     -1.088323,      1.318076,      1.742809,
		      1.357146,     -0.238541,     -0.692097,     -1.949273,      2.293493,
		     -2.078466,     -0.598594,      4.053650,      0.253273,     -3.890893,
		      0.394886,      3.031006,      0.135529,     -1.406860,      0.149910,
		     -0.130166,      1.222315,      0.784463,     -0.714810,      0.922626,
		     -1.459007,      0.259294,      3.021244,      1.521918,     -1.536518,
		      1.252911,      2.244052,     -1.475426,     -2.675993,     -1.466549,
		      0.589266,      1.567929,      2.270887,      0.649834,     -0.003666,
		     -0.113247,     -1.065122,      0.113396,     -0.578108,     -1.449328,
		      1.055048,      1.772880,      3.436426,     -1.458936,     -1.329691,
		      0.036918,     -1.206814,      0.596671,      0.122393,     -2.469778,
		     -0.264120,     -1.029097,      3.636395,     -0.662097,     -4.751778,
		      3.411536,      1.921298,     -4.257046,     -0.501106,      0.995979,
		      1.728264,     -0.864684,     -1.795389,     -1.006453,      1.979104,
		     -2.251770,      0.433024,      1.430522,     -1.466526,     -1.746099,
		      1.448728,      1.199083,     -0.771604,     -0.920187,     -0.203994,
		      2.188325,      2.196796,     -1.144409,     -2.018082,     -1.499815,
		      0.054217,      0.599949,      0.025931,      0.343280,      0.449428,
		      0.705858,      0.213904,     -1.508210,     -3.487097,     -1.593279,
		     -1.189478,      0.667173,      0.809790,      0.040959,     -0.849379,
		      0.473633,      0.583181,      1.285121,      1.937860,     -4.813030,
		     -0.665129,      4.910621,     -0.577031,     -3.027450,      0.673588,
		      1.614600,      0.205417,     -2.243516,      0.201844,      0.388931,
		      0.130051,      0.803383,      2.038311,      2.288725,     -0.748151,
		     -1.720010,      1.303775,      0.716248,     -1.299794,     -0.470563,
		      1.176908,     -0.876025,     -2.724236,     -1.725262,     -0.254239,
		      0.843715,      2.174938,      0.550614,      0.544320,      0.585746,
		      0.567823,      0.467856,     -0.457234,     -2.120475,     -1.559274,
		      0.630556,      2.969139,      0.476956,     -0.530072,     -0.285137,
		      0.369933,      0.169559,     -1.524929,     -0.529165,     -1.251087,
		      0.517338,     -0.434672,      0.487987,      2.059030,     -1.618893,
		     -3.453380,      3.613741,      3.522541,     -2.665269,     -0.743415,
		      1.374714,      1.826909,      0.532246,     -0.375306,     -1.081810,
		      0.459827,     -3.374991,     -0.265747,      1.966339,     -0.315990,
		     -2.716223,     -0.535590,      0.429726,      0.615531,      2.012111,
		      1.289209,      1.528717,      2.145806,      0.179308,     -0.668967,
		     -0.172827,     -1.455304,     -1.968372,     -1.307548,     -2.303798,
		      0.166042,      2.620799,      1.905685,     -0.500974,     -2.334374,
		     -0.206613,      0.132133,     -0.108290,     -0.476210,      0.270533,
		      0.911951,      0.524235,      0.817312,      0.438205,      0.339343,
		      2.826691,      2.587620,     -5.300941,     -0.223139,      3.932902,
		     -1.949794,     -3.526816,      1.627741,      1.162385,      0.758094,
		     -2.396074,     -0.591348,      0.207086,     -0.215993,     -0.752327,
		      2.735314,      1.570830,     -0.803759,     -0.642877,      2.266494,
		      1.849490,      0.452293,     -1.170212,     -0.756790,     -0.940234,
		     -1.926413,     -1.966977,      0.278017,     -1.455875,      1.480107,
		      0.912383,      0.437710,      2.358725,      0.262283,      0.685173,
		      1.046634,      0.204620,     -0.549111,     -0.577799,     -1.108840,
		     -0.173815,      0.059509,      0.566826,     -1.343003,     -0.151812,
		      1.197876,     -0.308979,     -0.485866,     -0.050594,      2.152436,
		     -1.429137,     -2.540053,      3.537448,      2.197176,     -2.270400,
		     -1.809257,      2.145954,      1.757505,     -2.668722,     -2.219333,
		      0.671139,      0.823414,     -0.743611,      0.400095,      1.334994,
		     -0.364376,     -1.868909,      0.224040,      2.127902,      0.547863,
		     -1.111831,     -0.914824,     -0.860065,     -1.924097,      0.084210,
		      0.694975,      0.047017,      1.018268,      0.390973,     -0.186578,
		      1.027076,      0.023131,     -0.304112,      0.451445,     -1.135340,
		     -2.975528,     -2.255712,     -0.406490,      1.237643,      0.335793,
		     -0.641889,     -0.089752,      1.494835,      0.548367,     -0.855768,
		     -1.568871,     -1.353582,      2.542206,      1.154587,     -1.185675,
		     -1.782298,     -0.368996,      2.401263,      0.493081,     -4.969019,
		      1.409366,      4.757727,     -1.109665,     -3.264093,      1.674205,
		      2.057091,     -0.314106,     -1.673014,      0.505608,      0.465578,
		     -1.134017,     -1.652201,      3.626840,      1.547451,     -1.840917,
		     -2.224359,     -0.607211,      0.344378,      2.399694,      2.246659,
		      1.052888,      0.182284,      0.123346,     -0.710355,      0.009973,
		      0.192854,      0.382428,      1.584088,     -1.181039,      0.640878,
		      0.284167,      0.699170,      1.975676,      0.737657,     -0.358790,
		     -0.042554,     -0.496830,      0.073112,     -1.602657,     -0.321183,
		      0.285992,      1.568014,      0.989784,     -1.696122,      0.470830,
		      1.586707,      1.069911,      3.168502,      3.405784,     -2.654502,
		     -2.997113,      5.306370,      0.928055,     -3.617494,      0.389462,
		      2.876841,      1.179044,     -0.971874,     -1.674591,      1.658487,
		      0.636652,     -1.399007,     -0.421618,      1.356757,     -2.629148,
		     -2.755023,      0.421915,      2.256598,      2.511955,      1.442540,
		      0.218057,      0.758147,     -0.849456,      0.214693,     -0.032851,
		     -0.600397,     -0.950904,     -0.763715,     -2.717285,     -1.178056,
		      0.775260,     -0.454830,      0.289148,      0.464031,      0.012158,
		     -0.708870,     -1.756029,     -0.162834,      0.529685,     -0.583389,
		     -0.106357,      0.591974,     -0.247805,     -1.364583,     -0.262283,
		     -0.306368,     -2.040744,      0.704910,     -1.961442,     -1.176318,
		      1.160497,     -3.179468,      1.821705,      3.665669,     -2.755052,
		     -4.835115,      1.357822,      2.865817,     -0.644557,     -2.933896,
		      0.617335,      1.464387,     -1.381125,     -2.366566,      0.461871,
		      1.781956,     -0.084900,     -0.725820,      4.457658,      1.890468,
		     -0.636309,     -0.983160,     -2.445437,     -3.330899,     -1.182048,
		      1.167686,     -0.139374,     -0.308686,      1.328445,      2.151178,
		      0.488236,      1.260178,      1.378796,      2.617983,     -0.681899,
		     -0.111093,     -0.732168,      0.227542,     -0.363894,      1.957782,
		      0.620378,     -0.101646,     -1.334475,     -0.064949,     -0.226295,
		      1.235971,     -0.344435,      0.039710,      2.506968,      0.461384,
		     -2.182239,      0.265916,      0.446564,      0.753680,     -2.116563,
		      4.058918,      0.450569,     -4.428053,     -2.085839,      6.224321,
		      0.877507,     -4.342344,      0.110937,      2.259479,      0.750262,
		      0.294780,      0.132753,      1.470313,      0.490173,     -1.137048,
		     -0.367047,      1.430518,     -3.435654,     -3.645043,     -0.842284,
		      0.039715,      0.783554,      2.462869,      0.966531,     -0.209182,
		      0.473392,      1.565974,      0.592890,     -1.512451,     -0.638189,
		     -0.759489,     -1.448353,     -4.127701,     -0.082677,     -0.503179,
		     -0.737833,     -0.356824,     -0.409114,      1.005000,     -0.504821,
		      0.164976,     -0.098179,     -1.971117,     -0.202903,     -1.314776,
		      0.150304,     -1.568929,     -0.945063,     -0.199723,     -1.141775,
		     -0.004517,     -2.439912,     -1.646753,     -1.380876,      2.811902,
		     -1.381078,     -2.117686,      2.685411,      4.823041,     -3.074594,
		     -3.382325,      1.383543,      1.398591,     -0.852813,     -0.856322,
		     -0.716315,      1.294085,     -0.582313,     -2.282643,     -0.387044,
		      0.036602,     -0.536380,      0.992544,      5.253657,      1.316047,
		      0.125131,      1.846921,      0.589956,     -1.477876,     -1.950740,
		      1.038255,     -0.715965,     -3.102482,     -0.487917,      2.386004,
		     -0.911823,     -0.485016,      0.221229,      2.484115,      1.164082,
		     -0.113096,     -0.397800,      0.815621,     -1.159299,      0.054007,
		     -0.845587,      0.236354,      0.649510,      0.782766,      0.629531,
		      0.419448,      0.754133,     -0.535022,      0.401869,      1.610055,
		     -1.428615,      1.460436,     -0.814369,     -0.363389,      0.323755,
		     -0.705315,     -1.320271,      4.002548,     -0.164060,     -5.047024,
		     -0.219640,      5.701858,      0.769336,     -2.011861,     -0.072620,
		      0.032148,     -1.080968,      0.149015,      0.079997,      2.044542,
		      1.537324,      0.213185,     -0.173294,      1.269151,     -2.326929,
		     -1.752698,     -0.531563,     -1.166402,     -1.677891,      1.048712,
		      1.115878,     -1.497004,     -0.841895,      1.240032,      2.259244,
		     -0.824126,      0.782672,      0.975244,      1.713557,     -2.647748,
		     -0.750627,     -0.149882,      0.384857,     -0.686600,      0.113312,
		      0.365122,      1.011583,     -0.316169,     -0.664908,     -1.412781,
		      0.884028,     -0.549946,     -1.267865,     -1.531061,     -0.125612,
		     -0.133263,     -0.712916,     -1.050178,      1.946283,     -0.672188,
		     -0.033345,     -0.449928,      1.848807,      2.744912,     -3.163283,
		     -1.619720,      2.437472,      3.392612,     -3.594725,     -1.950760,
		      1.676130,      0.599871,     -0.649216,      0.083090,     -0.746549,
		      0.032943,     -0.382411,     -1.243919,     -0.318566,     -1.109820,
		     -1.572310,      1.079202,      4.199291,      0.381770,     -1.311668,
		      1.961471,      1.389908,      0.670726,     -0.191130,      1.723716,
		      1.760571,     -2.850089,     -2.181102,      0.350644,     -0.434682,
		     -2.335708,     -0.469266,     -0.174505,      0.142714,      0.070578,
		      0.015384,      1.289517,      0.019529,      1.054259,     -0.543292,
		     -0.561561,      0.536874,      1.919723,      2.889518,      1.321228,
		      1.885137,     -0.725380,      0.395828,      1.595351,     -0.167996,
		      0.224100,      1.129140,      2.135056,      0.544380,      0.688005,
		      0.733099,     -0.964177,     -0.254285,      2.577167,     -0.427653,
		     -4.428449,      0.305105,      2.252320,     -0.778271,     -0.267123,
		      0.872789,      0.463057,      0.552466,      0.267665,     -1.332172,
		      0.791821,      1.476343,      0.929629,     -0.151683,     -1.044924,
		     -2.897209,     -0.804406,      1.530313,      0.333234,     -2.402398,
		     -1.132938,      0.414757,     -1.672388,     -0.985673,      1.254635,
		      2.213750,     -1.192880,      0.109889,      0.159162,      2.574141,
		     -0.110763,     -0.524433,      0.826472,      0.266951,     -0.376253,
		     -0.562726,     -0.524295,      0.673783,     -0.830196,      0.250569,
		     -1.107244,     -0.934038,     -0.228908,      0.381333,      0.511085,
		     -0.913815,     -0.524013,     -0.214105,     -1.084952,      0.075537,
		     -0.624030,      0.791258,      1.703542,      0.929057,     -2.397188,
		      0.409025,      2.884856,     -2.783016,     -1.452870,      3.378236,
		      2.432384,     -3.073895,     -0.372387,      0.279179,      0.853760,
		     -0.124964,     -0.313056,     -1.601431,      0.384822,     -0.084327,
		     -2.028219,      0.326826,      0.608728,     -0.149562,      0.843673,
		      2.268872,      0.347452,     -2.012957,      1.842854,      2.565253,
		      0.822172,      0.524903,      0.050966,      2.338203,      0.073134,
		      0.115831,      0.596204,     -0.109164,     -3.075694,     -0.649129,
		     -0.416660,      0.186447,     -0.857207,      0.064834,     -0.183198,
		      0.820629,      1.099447,      1.366113,     -0.861908,      0.983495,
		      0.970113,      1.776498,      0.539348,      1.259943,      2.012536,
		      1.035419,     -0.095439,      0.590850,      0.425163,      2.704153,
		     -0.672413,      0.167615,      0.954838,      1.195284,      1.123597,
		      0.812149,     -2.119687,     -0.138432,      2.106497,      0.120559,
		     -2.843911,      1.882920,      1.731385,     -1.025649,     -0.783996,
		     -1.099831,      0.667631,      1.970214,      1.301679,     -0.808162,
		      0.593147,      0.545582,      0.977497,      1.019861,     -0.781130,
		     -2.385158,     -0.772730,      0.013709,      0.594218,     -0.989504,
		     -0.913026,     -0.054867,     -2.022985,     -2.546884,     -0.234504,
		      1.853129,      0.713004,      1.049044,      1.261374,      1.728427,
		      1.129621,     -0.876447,      0.361778,      1.036769,     -0.862347,
		     -0.130497,     -2.066182,     -0.896728,     -0.844376,     -0.160717,
		     -1.690094,     -0.954862,     -0.821947,      0.148900,      1.117269,
		     -1.306033,     -0.801969,     -0.480075,     -1.490339,     -3.742445,
		     -2.193452,     -1.280246,     -0.038672,     -0.225412,      2.211848,
		      1.237299,     -2.380050,     -0.524308,      2.872309,     -3.346512,
		     -2.295589,      2.884554,      0.581621,     -2.472997,      0.136926,
		     -0.059472,      1.297162,      1.350460,     -1.193048,     -1.417553,
		      0.654720,     -0.605335,     -2.766388,     -0.949727,     -0.218450,
		     -0.253428,      0.659616,      0.510005,      0.811154,     -2.782775,
		     -0.806971,      1.271004,      1.180195,      2.777192,      0.493302,
		      0.503079,     -0.727514,     -0.513587,     -1.022081,      0.589676,
		     -2.477200,     -1.029010,     -0.313022,     -0.110216,     -0.175540,
		      0.165705,     -0.571695,      0.873367,      2.051646,      2.920387,
		     -0.217075,      2.889975,      1.008420,      0.532932,     -0.925679,
		     -0.293316,      0.635524,      0.983632,      1.852764,      1.302155,
		      1.897300,      2.449983,      1.517373,     -0.728021,     -0.412487,
		     -1.950726,     -0.558897,     -1.945858,      1.374776,      1.535574,
		     -2.386316,     -0.068132,      3.210967,     -1.056733,     -3.114863,
		      2.349310,      1.483443,      0.548516,     -0.874035,     -2.137266,
		     -0.385946,      1.563614,      0.545158,      0.454334,      2.524595,
		      0.552771,      0.060922,      1.407537,     -0.045604,      0.365204,
		      0.952404,      0.916033,      0.481625,     -0.752063,     -0.503597,
		     -0.605903,     -0.648678,     -1.788232,     -0.894540,     -0.657138,
		      0.309608,      0.756963,      1.381863,      0.590321,      1.100169,
		      0.561711,     -0.206014,     -0.312630,     -0.022845,     -0.111408,
		     -0.981686,     -3.429268,     -1.961656,     -1.637346,     -1.940433,
		     -0.882239,      1.304037,      1.612375,      0.995610,     -1.049214,
		     -3.174705,     -0.417470,      0.251047,     -1.088612,     -2.116532,
		     -0.942368,      0.181494,      0.361514,     -0.271033,     -0.784609,
		      0.433898,      0.887974,     -1.338073,     -0.006645,      3.220130,
		     -2.957136,     -2.149781,      2.254677,     -0.532622,     -1.604185,
		     -0.248456,     -0.373062,      1.681876,      1.325355,     -0.793334,
		     -0.566799,      1.210319,     -0.952407,     -1.868680,     -0.187160,
		      0.054758,     -0.646057,     -0.820634,     -1.566175,      1.862255,
		      0.708925,     -0.600608,     -0.251537,     -1.234501,      2.212442,
		      2.324427,      1.536667,      0.222559,      0.166316,     -1.694978,
		      0.195023,     -0.777981,     -0.638647,     -0.476762,      0.538291,
		     -0.437901,      0.257321,     -0.769134,      0.263708,      1.175293,
		      0.726674,     -0.622034,      0.830876,      0.425213,      1.468255,
		      1.615848,     -0.395539,      0.067793,      0.523300,     -0.472585,
		     -0.324165,      0.681526,      1.792005,      1.394007,      0.158627,
		      0.946651,      0.195260,     -0.099094,     -2.087390,     -1.275010,
		     -1.757367,      1.347175,      0.118101,     -4.574665,      0.857182,
		      3.446228,     -1.104785,     -2.618158,      1.884110,      0.323231,
		      0.575971,      0.421699,     -1.807094,     -0.084931,      0.792203,
		     -1.429211,      0.196039,      2.901479,      0.095930,     -1.541926,
		      0.132548,     -0.132852,      0.706748,      0.840897,     -0.712144,
		     -0.679883,     -0.363211,      0.566905,     -0.869153,     -0.717657,
		     -1.595476,     -0.577673,     -1.356685,     -1.047584,     -0.041977,
		      1.715181,      0.590599,      1.314509,      1.111696,     -0.291763,
		     -0.297839,     -0.144984,      0.428661,      0.883977,     -0.777836,
		     -0.539615,     -0.643410,     -1.050624,     -1.058484,      0.182094,
		      0.525391,      1.795998,      0.891385,     -0.175499,      0.237233,
		      0.044681,     -0.155173,      0.009021,      0.695699,      0.153116,
		      1.669991,      1.073368,      1.580744,     -1.016610,     -1.226459,
		      1.470405,     -1.876036,     -1.272412,      1.542875,      3.674637,
		     -3.016229,     -1.180138,      2.837579,     -0.613073,     -0.320979,
		     -1.483329,     -0.504955,      0.142069,      0.664792,     -1.002783,
		     -0.273295,      0.608587,     -1.842606,     -0.662676,      0.662424,
		      1.004229,      0.353010,     -1.129865,     -2.958327,      0.379754,
		      2.367536,      1.478432,      0.961559,     -2.050121,     -0.847635,
		      1.410069,      1.658559,      1.350384,      0.736208,     -2.095375,
		     -0.650735,     -1.589108,     -1.369556,     -1.513743,      0.958034,
		      1.374882,     -0.783831,     -1.082473,      0.321299,      2.007096,
		      1.116985,     -1.039382,     -0.892961,     -1.461443,     -2.790937,
		      0.592782,      0.886183,      0.217805,      0.267107,      0.596625,
		     -0.666377,     -0.487202,      0.550709,     -0.760376,     -1.523775,
		      0.175541,      0.923713,      1.002254,      2.311207,      0.032807,
		     -0.047376,     -2.383738,      0.266213,      3.061808,      0.368328,
		     -4.253445,      0.548289,      2.652912,     -0.469111,     -0.851386,
		      0.670172,     -0.080872,      0.375963,      0.026656,     -1.028058,
		     -0.440168,      0.256962,     -0.863221,      0.327028,      1.939004,
		      0.824559,     -1.277288,     -0.016680,      1.049954,      1.676779,
		      0.139760,     -1.501076,     -1.634635,     -1.811301,      1.368658,
		      0.380689,      0.037055,     -1.153053,     -0.344325,     -0.727467,
		      0.547658,      0.460266,      1.603011,      0.048540,      1.590158,
		      0.922399,     -0.142412,     -0.203413,      0.664011,     -0.182507,
		     -1.678439,     -0.087145,      0.548422,      1.182476,      2.208922,
		      1.071497,     -1.132085,     -1.227747,     -1.027692,      2.204693,
		      2.219476,      0.184785,      0.369271,      0.630890,      0.468538,
		      1.077508,     -0.480637,      1.037074,      0.082347,      2.244225,
		     -0.031450,      0.606584,     -2.403271,      1.561091,      1.219540,
		     -3.024289,     -1.672407,      2.351627,      2.054924,     -1.949178,
		     -1.364929,      0.958471,     -1.434725,      1.021742,      0.496383,
		     -0.944679,     -0.524374,      1.071095,      0.476264,      0.798843,
		      1.326781,     -1.675822,     -1.473050,     -0.696759,      0.165138,
		     -0.310746,     -1.045998,     -2.106308,     -0.908814,      1.629109,
		      1.378439,      1.715418,      0.526390,     -1.966388,      0.179603,
		      0.372174,      0.326143,      1.145107,     -0.955759,     -0.691380,
		      0.241334,     -2.093092,     -2.362536,     -0.018689,      1.670364,
		      0.493199,     -1.001549,     -1.065784,      2.282830,      1.591639,
		      0.138594,     -0.902039,     -0.936340,     -1.441554,     -0.496358,
		      0.089231,     -0.864225,     -1.074963,      0.959997,     -0.084961,
		     -0.269381,     -0.294008,      0.111746,     -0.801088,     -2.220292,
		     -0.705353,     -1.620307,      1.264960,      0.653603,     -0.456579,
		      0.785983,     -0.837418,     -3.152007,      0.489824,      1.403123,
		     -0.068948,     -3.083237,     -0.259072,      1.095708,      0.779001,
		      0.663048,     -0.538313,      1.010701,      0.700243,      0.525842,
		      0.018797,     -0.405766,     -0.481139,     -1.575904,      0.512781,
		      0.397131,      0.250775,     -1.134343,     -0.404098,      2.445341,
		      2.976313,      1.091342,     -1.342175,     -0.626820,     -2.723197,
		     -0.675441,      0.625147,      0.594352,     -0.072008,     -0.161730,
		     -1.528081,      0.560827,      0.003729,      1.498623,      0.768766,
		      2.045851,      0.973928,     -1.084650,      0.062785,      1.862976,
		      1.305408,     -1.678655,     -1.745701,     -1.546578,      0.577662,
		      1.272481,      1.886854,     -0.347712,     -0.810920,     -0.335045,
		      0.675110,      0.792867,      1.438873,      0.378143,      0.102093,
		      0.395762,      0.012656,      0.211278,      1.143747,      0.430663,
		     -0.671481,      0.968749,      0.401346,      0.193658,     -2.845934,
		     -2.358495,      2.456685,      0.940127,     -2.218600,     -0.252732,
		      1.100108,      1.120715,      0.251554,     -0.144160,      0.518300,
		     -0.782399,      0.288804,     -0.905149,     -0.997850,     -1.023632,
		     -0.260833,     -0.315170,     -0.616153,      1.653892,      1.821410,
		      1.171936,      1.228218,      0.368434,     -2.041373,     -2.714151,
		     -1.034917,      0.136523,      1.413398,      0.762133,      0.055514,
		      1.427614,     -1.714870,     -0.758084,      0.549751,      1.908871,
		      0.786842,     -0.428294,     -1.186751,      0.596605,     -2.062943,
		     -0.539072,     -0.496394,      1.162790,     -0.983479,     -1.247320,
		     -0.652781,      1.556944,      2.932012,     -0.070439,     -1.380138,
		     -0.518106,     -0.572665,     -0.508205,      1.335969,     -0.604931,
		     -1.009726,      0.056649,      0.758376,     -0.358297,     -0.458999,
		     -0.097606,      0.681329,     -0.223174,     -0.613208,     -1.852107,
		     -1.633760,     -0.585684,     -0.615715,     -0.918440,     -1.504581,
		      2.145103,     -1.153152,     -1.412002,      2.118235,      2.288582,
		     -0.295161,     -0.866978,     -0.638941,     -0.559253,      0.459799,
		      0.847703,     -1.658921,     -0.894921,      0.556259,      0.471680,
		      1.011363,      0.939294,      1.370880,     -1.288724,     -0.996571,
		     -0.899258,     -0.736379,     -1.271191,     -1.090311,      1.245163,
		      1.483047,     -0.292954,     -1.928863,      0.262772,     -0.437870,
		     -0.880264,      0.703516,      0.818794,     -0.228951,      0.101588,
		     -0.544360,      0.420608,      0.154112,     -0.688109,      0.220578,
		      0.498270,      0.839149,     -1.429843,     -0.298532,      2.127919,
		      2.134881,     -0.760411,     -1.011629,     -0.797119,     -1.379543,
		      0.904832,     -0.160620,      0.398049,      0.206470,      0.591536,
		      0.888267,      0.863211,     -0.609760,     -0.114653,      0.781408,
		     -0.329382,     -0.908402,     -0.964932,      1.034452,      2.170424,
		     -0.287636,      0.198127,     -0.895802,     -0.514124,     -0.235715,
		     -0.068562,     -0.778753,     -1.801529,      1.937253,     -0.561212,
		     -1.661922,      0.087892,      0.824421,      0.236335,      0.476902,
		     -0.917496,     -0.497944,      2.676048,      1.186605,     -1.043113,
		     -0.391988,      0.186077,      0.333208,     -0.656423,     -0.894096,
		     -0.715188,     -0.266769,     -0.258260,      2.780925,      2.668847,
		      0.280630,     -2.533115,     -1.578458,      1.058146,      1.473304,
		      0.781340,     -0.535680,      1.605175,     -0.805174,     -1.867420,
		     -1.777792,      0.756542,      0.975071,      1.079328,     -0.093304,
		      1.754895,     -1.862693,     -0.050566,      1.212106,      1.022382,
		     -0.197523,     -1.652169,     -1.416517,     -0.580348,      1.827513,
		     -0.144089,     -1.284920,     -0.381089,      0.139313,     -0.851526,
		     -1.552485,     -1.420018,     -0.249600,      0.585792,      2.285051,
		     -0.858229,     -1.055256,     -0.230492,      1.473897,     -0.685725,
		     -1.343014,     -0.591411,     -0.250643,      0.857416,     -0.403859,
		     -0.370915,      0.640777,     -0.899418,     -1.605992,      1.727812,
		     -2.127170,     -2.338049,      1.252912,      0.335043,     -1.138383,
		     -0.001251,     -0.141544,     -0.552723,      1.061178,      0.343717,
		     -2.582436,     -0.047620,      0.309675,     -1.288346,     -0.717453,
		      0.587634,      2.215858,      0.374866,      0.102503,      0.568286,
		     -1.112415,     -2.217174,     -0.944199,      1.151729,      1.192351,
		      0.767624,     -2.239886,     -1.254418,     -0.400424,     -1.422409,
		      0.893615,      1.157114,      0.694039,     -0.872653,     -0.684146,
		     -0.536071,      0.981917,     -0.674079,      0.880015,     -1.200582,
		     -0.097587,     -2.221712,     -0.133129,      1.136913,      1.536545,
		      0.679847,     -0.180090,      0.100922,      1.061946,      1.187838,
		     -0.578062,     -0.651552,      0.566181,      1.282989,     -0.417209,
		      0.881412,     -0.668198,     -0.304858,     -0.605973,      1.251716,
		     -0.177499,     -0.599929,      0.924751,      1.146849,      0.262672,
		     -0.845359,      0.040091,     -1.698111,      1.218071,     -0.189079,
		     -2.531580,      0.783622,      0.313952,     -0.797783,      1.873644,
		      0.381090,     -1.807275,      0.068714,      0.225940,      0.134676,
		      0.536865,     -1.409055,     -1.323199,      1.006344,      1.058129,
		     -0.289973,      0.000060,      0.103594,     -1.045931,     -1.280807,
		      0.799811,      0.636905,      0.010927,     -1.670870,      0.596321,
		      1.521586,      1.009362,     -1.189026,     -2.204794,     -0.042550,
		     -0.223260,      0.686001,     -0.747668,      0.162674,      1.011610,
		      0.211271,     -1.269619,     -0.544219,     -0.968144,     -0.677838,
		      0.067002,      0.709424,     -0.413291,     -0.563196,      1.027678,
		      0.296353,      0.341440,      0.037083,      0.034483,     -1.180037,
		     -0.465900,      0.038295,     -1.713839,     -0.477202,      1.757925,
		      0.100175,     -2.455805,     -1.595509,     -1.950530,     -0.155419,
		      0.875779,      0.637528,     -0.554370,     -1.377986,     -0.209303,
		     -1.555260,     -0.740265,     -1.446140,      0.124848,      0.673011,
		      1.497715,     -0.865531,      0.869113,      1.008793,      0.483994,
		      0.284661,     -2.317498,      1.863753,      0.328904,     -1.126193,
		      1.637489,      0.997611,      0.043694,      0.538336,      0.380644,
		      0.890879,      1.868114,     -1.278183,     -1.719336,      0.485795,
		      0.690878,      0.108818,     -0.202863,      0.055492,      0.110904,
		     -1.357359,     -0.833736,      2.639986,      0.907821,     -1.237226,
		     -1.253406,      0.055522,      0.590521,      2.965805,      1.327202,
		      0.312270,      0.673071,     -1.103083,      0.712427,      0.507992,
		      1.389516,      0.999662,      0.784382,     -0.842176,      0.288612,
		     -0.725595,      1.619319,     -0.329788,     -1.059015,     -1.859248,
		     -1.044420,      0.776983,     -0.013412,     -0.164623,      0.246984,
		      0.198634,      1.202077,      2.295920,      1.316358,     -0.827085,
		      0.726385,      0.612635,      1.183584,      1.023941,     -1.865700,
		     -0.001364,      0.542687,      0.286066,      0.515704,      0.311629,
		     -0.047940,      2.299711,      0.067753,     -0.466213,     -0.514650,
		     -0.660943,     -1.692835,      0.546906,      0.827686,     -1.058256,
		     -2.911783,      0.966611,      0.096298,     -0.635975,      1.218224,
		     -0.472050,     -1.015149,     -0.083739,      0.210046,      0.804577,
		      0.376031,     -0.995729,     -0.512740,     -0.090465,     -0.493398,
		      0.335248,      1.513039,      1.739957,     -1.712853,     -2.298287,
		      0.500039,      0.968786,      0.382977,     -0.946299,     -0.138823,
		     -0.659606,      0.555207,      0.773954,     -2.108337,     -1.095600,
		     -2.251659,      0.521184,     -0.479718,     -2.508995,     -1.105063,
		      0.445617,      1.085396,      0.285125,      0.378845,     -0.828388,
		      0.999529,     -0.319521,      0.402758,      0.079858,      2.097502,
		     -1.453519,      0.063660,      0.017624,      2.131893,     -0.487722,
		     -1.209392,     -1.413171,     -1.742506,     -1.455033,      0.488739,
		      1.604445,      0.425156,     -1.388252,     -1.367413,      0.787055,
		     -0.357396,     -0.610065,     -0.386108,     -0.246074,     -0.079176,
		     -0.793822,     -1.620830,      0.791008,      0.774050,      0.142088,
		      1.364512,     -0.401272,      1.187173,      0.652253,      0.056490,
		     -1.400773,      3.281217,      0.642073,     -2.390140,      0.383417,
		      0.303416,     -1.999307,      0.258306,      0.059366,     -0.671138,
		     -0.292557,     -0.703145,      0.685290,      0.684666,     -0.929363,
		     -0.353891,     -0.087801,     -2.340486,     -0.979504,      1.481684,
		      3.242503,      1.323400,     -1.221078,     -2.653816,      0.409201,
		      1.228777,      0.637560,      0.127329,     -1.167027,     -1.744869,
		      0.510991,      2.529796,      0.560846,      0.455573,      1.131887,
		      1.900708,     -0.716927,     -1.523867,     -1.146033,      0.891057,
		      0.286848,     -0.319656,     -1.028902,      0.507636,      1.600066,
		     -0.329112,     -0.302803,     -1.135758,      0.939250,     -0.632709,
		      0.115286,      1.148943,      0.633010,      0.393466,      1.875872,
		      1.821205,     -0.022887,     -0.203482,     -0.325799,      1.796883,
		      0.440890,     -0.553182,      0.738443,      1.206193,      0.298087,
		     -1.139239,      0.014529,     -0.318888,      0.441132,     -0.216645,
		     -0.579569,     -0.870365,     -0.279206,     -1.030678,      1.184056,
		      0.146675,     -0.056770,      2.810191,     -1.552501,     -1.840486,
		      1.071968,      0.485794,     -0.723699,      1.936205,     -0.218445,
		      0.078957,     -0.096128,     -0.675722,      1.571626,      0.693201,
		     -1.176085,      1.078274,      0.613106,     -1.264912,     -0.080779,
		      1.379322,      1.990373,     -1.376926,     -2.708640,     -0.075070,
		      1.626966,      0.974235,      0.561143,      1.347604,     -0.852337,
		     -0.452448,      2.906452,     -0.403743,     -1.926091,     -3.021542,
		     -0.366007,      1.289648,     -2.429359,     -1.333751,     -0.244804,
		      1.018073,      1.127787,      1.028937,     -1.509693,     -0.096398,
		     -0.775576,     -0.777061,     -0.859452,      2.431716,      1.140037,
		     -0.944934,     -1.417481,      1.342849,      0.457454,      0.437419,
		     -1.135998,     -0.529036,     -2.644617,      0.054515,      1.064865,
		      1.433984,      0.081679,     -0.061219,      0.714341,     -0.604723,
		     -0.411053,      0.050751,      0.149938,      0.991612,      1.062917,
		     -1.751265,     -0.676915,      1.766745,      1.972108,      0.347983,
		     -1.131098,     -0.964023,      0.093638,      0.034252,     -0.516530,
		     -3.590568,     -0.927289,      2.764237,      0.188220,     -0.379108,
		      0.358955,     -0.367367,     -0.799867,      0.552936,      0.588946,
		      1.576923,      0.634324,     -0.964956,     -0.126210,     -0.851257,
		     -1.113377,      2.194893,      1.637269,     -2.515087,     -4.463211,
		     -1.261985,      1.791369,      2.238376,      0.704021,     -1.760889,
		     -2.422200,     -1.406055,      0.441849,      1.426477,      0.994733,
		      0.200619,      0.308179,      2.230981,     -0.054470,     -1.892344,
		      1.059521,      3.614977,      0.727442,     -0.596424,     -1.153182,
		     -1.430401,      0.164198,      0.589130,      1.245633,     -0.498478,
		      0.954662,     -0.905197,      0.857780,     -0.397815,      1.891367,
		     -0.189590,     -0.283939,      0.750304,      1.027002,      0.775024,
		      0.709809,      2.007512,     -0.026033,      0.111226,      0.201064,
		     -0.103102,     -0.135420,      0.418299,      0.235207,      0.803256,
		      1.051213,     -1.329482,      0.374282,      1.702796,      0.521322,
		     -1.169023,     -2.520942,     -0.460638,      2.051353,     -0.469596,
		      0.436852,      1.170638,      0.212901,     -0.751652,      1.073520,
		      2.944212,     -1.654620,     -0.687160,      0.742309,     -1.125035,
		     -0.902661,      1.277933,      0.475807,      0.385701,     -0.339539,
		     -1.969275,      0.762048,     -0.040752,     -1.666453,      0.387027,
		      0.211487,     -1.771588,      0.415973,      1.926435,      2.464238,
		     -0.054429,     -2.285045,     -1.703333,      1.125782,      1.958871,
		      1.604531,      1.073257,     -1.015653,     -1.980436,      0.824811,
		      0.105720,     -1.825897,     -2.014624,      0.213527,      0.271607,
		     -2.279543,     -0.696390,     -0.288115,      0.405398,      0.629070,
		      2.403643,     -0.032670,     -1.160635,     -0.655103,     -0.496664,
		     -1.298824,     -0.736768,      0.986382,     -0.501159,     -1.546149,
		     -1.064933,     -0.854123,      0.314208,     -0.159617,      0.861916,
		     -1.011342,     -1.918381,      0.008878,      0.089183,     -0.685841,
		     -0.972508,      0.609654,     -0.357430,     -0.115812,     -0.810534,
		      0.593109,      0.428683,      0.933323,     -1.480233,     -1.018149,
		      1.007462,      0.393964,     -0.735979,     -1.291086,     -0.535415,
		      0.458362,      0.526718,      0.932624,      0.110621,      1.835689,
		     -3.545619,      0.027990,      1.189477,     -0.244296,      0.398117,
		      1.889865,     -1.504980,     -0.942651,     -1.040341,     -0.686868,
		      2.447219,      0.738818,     -1.145546,      0.603258,     -0.137109,
		     -2.241869,      0.625949,      2.967531,      1.298027,     -1.924260,
		     -1.804297,      0.658800,      1.013824,      0.856103,      0.053764,
		     -2.294895,     -2.402911,     -0.720348,      0.484432,     -0.851265,
		      0.638025,      2.658084,      3.818603,      1.497002,     -2.191863,
		     -0.170822,      1.525710,     -0.048933,     -0.388730,      0.725831,
		     -1.526824,     -1.569875,     -0.476672,      1.628128,      1.036597,
		      2.986898,      0.771139,      0.466091,     -0.193240,      1.986409,
		      1.157859,      0.397800,      0.306713,      1.191214,     -2.279042,
		     -0.045759,      1.594113,      1.214155,      0.870458,      0.846566,
		      0.366792,      0.539033,     -0.290508,      1.008041,      1.629459,
		      1.078005,     -0.694442,     -0.265104,      1.334247,      0.652199,
		      0.187468,      0.011809,     -1.218354,     -0.142752,     -2.038974,
		      1.077549,      1.324993,     -1.582411,     -1.975440,     -0.616405,
		     -1.232781,      1.035295,      3.039885,     -0.922999,      1.223388,
		      0.281347,     -2.012039,     -0.912598,      0.810538,      0.879754,
		      1.897485,      0.175755,     -1.747250,      0.812081,     -0.121389,
		     -0.586191,      0.923217,     -0.091260,     -1.630493,     -1.697615,
		      0.026224,      1.954092,      1.392934,     -1.283576,     -2.396364,
		     -1.347872,      1.795557,      3.219184,      0.716930,      1.847078,
		      0.731482,     -0.232601,     -1.008720,     -3.725333,     -2.327667,
		      1.376012,      0.488199,     -1.700244,      0.007426,      1.015553,
		     -0.315894,     -0.520470,      1.869633,      2.067755,     -1.297915,
		     -2.241592,     -2.234258,     -0.759608,     -2.321439,      0.025163,
		     -0.786127,     -1.388766,     -2.387004,     -1.125038,     -0.008328,
		     -0.097836,      2.185821,      0.588550,     -0.975065,     -0.662232,
		      0.590132,     -0.948793,     -3.419396,      0.101581,      0.182687,
		      0.772864,     -0.851298,     -0.803737,     -0.234208,      2.065005,
		     -0.316287,     -0.558883,     -0.175648,     -1.382804,      0.186584,
		      1.306835,     -1.722532,     -0.587085,      0.195040,      0.444215,
		      1.792123,     -1.235322,      2.242273,      3.148898,     -2.091982,
		     -0.330279,      0.713120,     -2.403756,      0.512358,      2.630870,
		     -0.079853,     -0.008541,     -0.237199,     -1.353222,      1.027084,
		      0.000091,     -0.642140,      0.677594,      1.486764,     -1.265294,
		     -1.720708,     -0.232606,      1.595323,     -0.329090,     -1.229393,
		      0.237347,      1.253808,      0.791702,      0.812014,     -1.432203,
		     -1.652303,     -0.450609,     -1.068392,     -1.951388,     -1.105178,
		      2.020596,      3.443663,      2.197689,     -0.206910,      0.630779,
		      0.852240,     -0.183953,     -2.396874,      1.512773,      0.129073,
		     -1.288905,     -1.432347,      0.160592,     -0.129799,      1.243807,
		      1.862107,      1.058251,      0.295878,      1.542697,      2.128977,
		      1.369969,      0.086286,      1.766456,     -1.406764,     -4.179302,
		     -1.053074,     -0.736474,      0.989714,      0.989215,      1.032776,
		     -0.190116,      0.400354,      0.269199,      0.984729,      2.318040,
		      1.514249,     -1.849067,      0.306891,     -0.239317,      0.598873,
		      0.532345,     -0.601512,      0.264028,     -0.662413,     -1.712218,
		     -0.106666,      0.301842,     -1.739680,      0.539503,     -0.401542,
		      1.027657,     -3.692542,      0.737318,      1.140219,     -0.575502,
		      1.257823,      1.481165,     -2.048559,     -0.386419,     -0.228065,
		     -0.001611,      1.222560,     -0.449581,     -0.621064,      0.036973,
		     -1.460452,     -1.318062,      1.056097,      1.749996,     -0.029149,
		     -0.842176,     -0.189818,      0.501712,      1.405222,      0.194691,
		     -0.305711,     -1.808387,     -0.029820,      2.002430,     -1.301444,
		      1.011502,      2.794822,      1.860308,     -0.148770,     -2.270830,
		     -3.018741,     -0.094037,      0.302781,     -0.977232,     -0.188016,
		      1.805274,      0.298748,     -0.614637,      0.106419,      2.644217,
		      0.803970,     -0.059981,     -1.192822,     -1.094591,     -2.060316,
		     -0.929751,     -1.550397,     -1.826385,     -2.436772,     -0.905409,
		     -0.789903,      0.287466,      0.967074,      1.976711,      0.142175,
		     -1.936326,     -0.516317,     -0.215875,     -0.711380,     -0.336935,
		     -1.717676,      0.404825,      0.602566,     -1.779570,     -0.545961,
		      0.831190,      0.880568,     -0.453179,     -0.896575,     -0.359491,
		      0.066134,      1.094623,      1.496317,     -0.162778,     -1.509197,
		     -0.999461,      0.432925,      0.067395,     -1.451783,     -0.946503,
		      1.905757,      3.409164,     -0.407401,      0.734638,     -0.261796,
		     -1.476820,     -0.797045,      1.050575,     -0.240865,      1.183152,
		      1.031698,     -0.320452,     -0.271104,     -1.789589,     -0.196985,
		      1.197537,      1.457319,      0.453408,     -0.917831,     -0.247496,
		      0.941590,      0.481056,     -0.565009,      0.194912,      0.088496,
		     -0.557355,      0.413083,     -0.948518,      0.054930,     -0.060106,
		     -1.625104,     -1.989418,     -0.812224,      1.704172,      2.508057,
		      0.742482,      0.369036,     -0.079865,     -0.706244,      0.073507,
		     -1.962388,     -0.645234,      0.335666,     -0.262794,     -1.046664,
		     -0.206416,     -0.285311,     -0.190252,      1.157019,      1.154892,
		      0.119818,      0.516773,      1.386859,      1.658778,     -0.541834,
		      2.492807,      1.124590,     -0.610122,     -1.936462,     -1.056837,
		     -0.912816,      0.158201,     -0.051392,     -0.948923,      0.640128,
		     -1.035482,     -1.583588,      0.692610,      2.256878,      1.217577,
		      1.672301,     -0.658683,     -1.037321,      0.308934,     -0.550783,
		     -0.152715,     -1.256115,      0.202959,      1.445412,      0.180817,
		     -2.138251,      0.465921,      2.597783,      0.064113,      3.731665,
		      0.316861,     -2.296342,      0.153056,      0.456451,     -2.470087,
		      0.825763,      1.202990,      0.141063,     -0.176260,      0.546067,
		     -0.922730,     -0.452691,      0.353809,      1.550473,      0.911033,
		     -1.332171,     -2.403426,     -0.972025,      1.281765,      0.372361,
		     -1.158234,     -1.207963,     -0.898852,     -0.344818,     -0.302708,
		      1.544018,      0.457635,      0.687603,      1.415003,     -1.271347,
		      0.374893,      2.166000,      1.786535,     -0.596137,     -0.693751,
		     -1.321571,     -0.934015,     -0.506894,      0.976506,     -0.278223,
		      1.411491,     -0.139970,      0.524507,     -0.635121,      0.667114,
		     -0.739732,      0.739369,      0.829367,      1.335277,     -0.856091,
		      0.503993,      0.041689,     -0.243167,     -1.853041,     -0.501147,
		     -0.840212,     -1.283314,      0.293386,      0.874681,      1.385159,
		      0.596866,     -0.091400,      0.270354,      0.235483,     -0.305476,
		      0.675252,     -0.429632,     -1.205629,     -2.258141,     -2.086795,
		     -1.228494,      1.436123,      0.904033,      0.097309,      0.319726,
		      1.573351,      1.908298,      0.857030,      0.577722,     -0.387808,
		     -0.490755,     -2.269785,      0.282776,      0.244635,     -0.759315,
		      0.968157,     -2.001641,      1.622496,      0.705900,      0.355936,
		     -0.145234,      0.773725,     -0.916467,     -0.881639,      0.087241,
		     -0.109719,     -0.296934,     -0.111170,      1.456208,     -1.124641,
		     -2.336145,      1.447088,      1.309259,     -0.680468,     -0.765454,
		     -0.712667,     -0.293393,      0.877479,      0.292077,      0.598116,
		      1.667413,      0.576590,     -1.116474,     -1.012995,     -1.924152,
		      0.991480,     -0.151228,     -1.495320,     -1.906800,     -0.466053,
		      1.764750,      1.967292,      0.082261,      1.525816,     -0.175815,
		     -1.774703,     -0.493507,      0.219855,     -0.950821,     -0.028738,
		     -0.424189,      0.935076,      1.144173,      0.059174,     -1.738998,
		     -0.992886,     -0.102609,      0.902150,      0.369174,     -0.156489,
		      0.779559,     -0.985592,     -1.210806,      1.515814,      0.482208,
		     -0.070256,     -0.380946,     -0.670077,      0.532340,     -0.124322,
		     -0.209942,      0.027276,     -0.586126,      1.311437,      0.723221,
		      0.433845,     -0.060297,      1.012864,      0.567615,      0.488783,
		     -0.286898,     -0.272084,     -1.921327,      0.285774,     -0.008729,
		      2.589382,      2.727456,      0.575432,      1.133218,      0.640670,
		      0.786766,      0.351826,      2.236440,      2.197168,      0.978837,
		     -0.709962,      0.948374,     -0.018143,     -1.248970,     -1.887057,
		      0.502917,      0.052236,     -0.414947,      0.293254,     -1.012824,
		     -0.968011,     -1.259014,      1.888408,      1.636018,     -0.896142,
		     -0.755949,     -0.518444,      1.277847,      2.373854,      0.799049,
		     -0.454371,     -1.233376,     -2.251508,     -2.812268,      0.262703,
		      0.562244,      0.913844,      0.330492,     -1.044754,      0.370226,
		      2.470599,      1.844454,     -1.854583,     -1.797723,      0.403963,
		     -0.349995,     -1.781601,      1.776396,      0.589189,      1.139844,
		     -0.014818,      0.526827,      0.322764,      0.531200,     -2.063240,
		     -0.888078,      0.880786,      1.979363,      0.703619,     -0.091526,
		     -0.872947,      1.286603,      0.367964,      1.531569,     -0.043860,
		      0.739460,     -0.085322,      0.043246,      0.226006,      1.504361,
		     -0.169611,      1.163804,     -1.096860,     -0.325175,     -0.533473,
		     -0.132088,      0.200461,     -0.912534,     -1.481963,     -3.378225,
		     -0.278128,     -0.268974,     -1.095621,      1.530098,      1.321804,
		      1.348319,      0.214120,      0.443837,     -1.670501,      0.078038,
		     -0.707909,     -0.027247,     -2.414250,      0.630907,      1.598891,
		      1.827076,     -1.572309,      0.475926,      0.796821,     -0.527175,
		     -1.138842,      1.315634,      1.472024,      0.621437,     -2.317817,
		      0.884278,      0.762276,     -0.587427,      0.460523,      0.740297,
		     -1.118667,     -1.956398,      1.421160,      1.662302,      0.442914,
		     -0.883601,     -1.959810,     -1.723611,      0.035242,      0.403889,
		      0.503615,      2.311378,      1.988286,      0.664059,     -0.916873,
		     -2.771787,      0.429214,      0.337582,     -0.801956,     -1.978602,
		     -0.615520,      1.832321,      1.527785,     -1.451386,      0.797770,
		      0.389744,     -0.910630,     -1.601653,     -0.131026,     -1.712885,
		     -0.060306,     -1.491279,      1.606632,      1.301201,      2.693770,
		     -1.712946,     -1.847131,     -1.739778,      0.152782,      0.444359,
		     -0.268219,      0.343973,      0.709097,     -0.993978,     -1.100668,
		      0.186853,      0.970717,      1.167069,     -0.570431,      0.048527,
		     -0.812731,     -0.516625,     -0.635003,     -0.453367,      0.941086,
		      1.248648,      1.330168,      1.742105,      0.125468,     -0.563720,
		     -0.161766,      0.254685,     -0.488298,     -1.427623,     -1.284613,
		     -0.286511,     -0.153591,     -0.479338,      0.099240,     -1.387064,
		      0.983396,     -1.070144,     -0.802426,      1.018336,      0.582134,
		      0.263611,      0.952760,     -0.429484,      0.670329,     -0.034682,
		      0.596987,     -0.376511,     -1.483748,      0.523596,      1.741839,
		     -2.449437,     -2.145143,      1.409672,     -0.697192,     -0.527309,
		      1.156854,      0.062203,     -1.407055,     -2.094837,     -1.118367,
		     -0.615034,      2.411524,      1.519288,      0.551287,      0.646322,
		     -1.408098,     -2.458489,     -0.647694,      0.562730,      1.975156,
		     -0.282331,     -0.189820,     -0.576060,      1.522451,      3.094627,
		     -1.108147,     -2.021427,      0.941936,      1.108611,     -1.252205,
		      0.592161,      1.477172,      0.874561,      1.112304,      0.587910,
		      1.293522,      0.051050,     -0.552591,     -1.989678,     -1.366746,
		      0.676026,      2.380317,      1.231665,     -1.671160,     -0.666892,
		     -0.970994,     -0.475172,      0.009262,      1.440700,      1.816031,
		     -1.195748,     -0.810458,      1.071174,      1.169229,      0.158745,
		      0.022836,      0.458355,      0.300202,     -1.812835,     -0.606813,
		     -0.787613,     -0.822809,     -0.335620,      0.317002,     -0.823310,
		     -1.885965,      0.236163,      0.296707,      1.201112,      0.445624,
		     -0.920829,      0.279669,     -0.572167,      0.513186,      1.390064,
		      1.414571,     -0.971063,     -1.158233,     -0.274085,      3.519406,
		     -0.826188,      0.188491,      1.323898,     -0.022294,     -1.154778,
		     -0.688206,     -0.576938,      2.151292,      0.347022,     -2.127042,
		     -0.441610,      0.534885,      0.847002,     -0.644252,      0.974828,
		      1.468908,     -1.188321,     -0.136470,      1.503144,      2.106228,
		      1.645537,     -1.641828,     -1.549500,     -0.304124,      0.196974,
		     -0.587536,      0.766686,      1.067539,      0.706903,     -0.202310,
		     -1.466187,     -0.047262,     -0.353492,     -0.080077,     -1.826156,
		     -0.953755,      0.936047,      1.553501,     -1.996209,     -0.463150,
		     -0.963586,      0.528265,     -0.856638,      0.002784,     -1.444572,
		      0.136610,     -1.719694,      0.441971,      0.472710,      2.956175,
		      0.825814,     -1.037721,     -2.601122,      0.864873,      0.992707,
		      0.747254,     -0.623699,      1.742185,      0.616004,     -1.426470,
		     -1.653509,      0.459159,     -0.984633,     -0.046252,     -0.750928,
		      0.368986,     -2.283129,     -0.554534,      0.180121,     -0.344159,
		     -0.069953,     -0.827446,      0.286326,      0.540879,      0.346029,
		     -0.373215,      0.058390,      1.284639,     -1.585586,     -1.765619,
		      0.522279,      0.804456,     -2.056415,     -0.073422,     -1.042102,
		     -0.690605,     -0.866801,     -0.844817,     -0.599351,      0.294349,
		      0.932114,     -0.266051,      0.789470,     -1.374829,     -0.317445,
		      0.767918,      0.696717,     -0.924314,     -0.910576,     -1.946688,
		      2.019504,      1.472755,     -3.062907,     -0.725877,      0.635108,
		     -0.076404,      0.003589,      0.154636,      0.941695,      0.748771,
		     -1.414272,     -1.336213,     -1.284781,      0.695958,      0.099626,
		     -0.413432,      1.526284,      0.177843,     -1.099973,     -1.760532,
		     -0.419793,      2.472380,      0.590580,      1.361888,     -1.191198,
		     -0.716993,      1.594479,     -0.099801,     -1.931516,      0.079431,
		      1.226861,      0.832116,      0.274001,      1.135663,     -0.530916,
		      0.674968,      0.019840,      1.926065,     -0.761697,      0.744175,
		     -1.442158,     -1.719175,     -1.316830,      1.907275,      1.276661,
		     -0.277880,     -0.723249,      0.361431,     -2.040898,     -2.441289,
		      0.237631,      3.075930,      1.184009,     -2.238168,      0.027877,
		     -0.698658,      0.849696,      0.816393,      1.088825,      1.702982,
		     -0.801149,     -0.866371,      0.006607,     -0.291809,     -0.777477,
		      0.120537,      1.543390,      0.449716,     -0.436567,     -0.492363,
		     -0.768770,     -0.831004,      0.229853,      1.682389,      0.769410,
		      1.184658,     -0.039864,      0.045025,      1.436328,     -0.221615,
		     -0.753757,     -1.154138,      1.327368,     -0.740788,     -0.342400,
		      0.668944,      1.674612,     -1.734273,     -0.087130,      0.095565,
		     -0.459468,      1.122884,      0.055331,     -3.545941,      0.457690,
		      2.962796,     -1.733301,     -2.632894,      0.150418,      2.348846,
		     -1.430485,     -2.598240,     -0.126021,      1.590197,      2.276103,
		     -0.797627,     -0.153848,      0.567308,      0.411506,      0.341849,
		      1.645459,      1.128624,      0.734949,     -0.951607,      0.029707,
		     -0.625983,     -1.519955,      0.715304,      0.109644,      0.547964,
		      0.631242,      0.987101,     -0.579591,     -0.217164,     -1.710840,
		     -1.300494,     -0.316713,      0.685033,     -1.072248,     -0.304757,
		     -0.077075,     -0.555375,     -0.922397,      0.603769,      2.548165,
		      1.125756,     -2.415186,     -1.409396,      0.114667,      0.584358,
		     -0.851773,      1.430369,      2.289833,     -0.346097,     -1.747638,
		      0.063599,      0.353488,     -1.556205,     -0.268374,     -0.714939,
		     -1.209919,      0.271249,      2.339014,      0.019777,      0.261450,
		     -1.595062,     -1.883103,     -1.413066,     -2.317749,     -0.722449,
		     -0.566417,      1.171715,      0.547346,      0.277155,      0.763685,
		      0.417385,     -0.129764,      0.272956,      0.477587,     -0.756622,
		     -0.077336,      0.754309,      2.257311,      1.270620,     -0.069050,
		      1.736550,     -0.456081,      1.325826,     -0.417571,     -1.620542,
		     -0.102289,      1.674834,      0.569851,      0.353072,     -1.925409,
		     -0.740917,      0.872619,     -0.675315,     -1.240089,     -0.149713,
		      1.914455,      1.142118,     -1.762273,     -1.537034,      0.565419,
		      1.760113,     -0.791152,     -1.565811,      0.821417,      0.554031,
		     -1.495432,     -1.751902,      0.862697,      0.400588,     -1.255085,
		     -1.769623,     -1.786243,     -0.185734,     -0.277345,      2.380457,
		      0.270144,     -0.503353,     -0.076748,     -0.155283,     -0.822624,
		      0.234055,     -0.173078,      1.002327,     -0.639019,      1.748556,
		     -0.362077,      0.592734,     -1.399975,      1.911731,     -0.613723,
		      1.186646,      0.624832,      0.280162,     -1.660223,      1.437794,
		      1.193909,      0.631140,      0.010074,      0.880719,     -1.074399,
		     -2.665462,     -1.492087,      0.954205,      0.771635,      0.492737,
		     -0.064423,      0.175496,      0.376031,      1.212145,      0.158870,
		     -0.277561,     -0.758234,      0.193545,      1.545870,      0.998739,
		      0.111862,      1.464796,      1.565042,      0.908010,     -0.470302,
		      0.386781,      0.536485,     -2.085166,      0.769600,      0.709245,
		      0.133463,      0.880987,     -0.239062,     -0.374824,     -1.025983,
		      0.206730,     -0.051997,     -0.823334,      1.221732,      2.329695,
		     -1.171634,     -0.725433,     -0.342417,      1.240025,      0.636125,
		     -1.748194,     -1.016341,      0.500348,     -0.693605,      1.491064,
		     -2.242515,     -3.821036,      3.978517,      2.125059,     -1.950436,
		     -1.614994,      1.031413,      3.025551,      0.505624,     -2.472670,
		     -1.076001,      0.219518,      0.381741,     -1.593592,      0.572452,
		      0.858693,     -1.339732,     -1.292201,      0.659895,      1.859049,
		      1.563623,     -0.003755,      1.140995,     -0.166647,     -1.768885,
		     -0.449250,     -0.227973,      0.310118,     -0.502059,     -0.614905,
		     -0.114998,      1.100871,      0.718176,     -0.718771,     -0.823564,
		      0.300947,      0.960495,     -0.405931,      1.486740,     -0.844302,
		     -2.191796,     -2.720660,     -0.430874,      0.084431,     -0.698188,
		     -1.739520,      0.220089,     -0.337586,     -0.718978,      0.429343,
		      1.464019,      1.406947,     -0.374019,      0.410146,     -0.617611,
		      0.126958,      0.249114,     -0.068905,     -1.049896,     -1.007010,
		      2.174826,      0.557560,      0.402027,      0.839219,     -0.259699,
		     -1.289811,     -1.341125,     -0.896335,     -2.213803,     -1.799937,
		      0.121219,      1.421588,      2.380655,      0.393713,     -1.019765,
		      0.198047,      2.955586,      1.098836,     -0.455546,      1.075013,
		      0.239924,      2.129377,      1.779692,     -0.158755,      0.201709,
		     -0.703969,      0.393682,      0.905839,     -0.220167,     -1.847384,
		     -0.054671,      0.626927,     -1.249758,      0.016445,     -0.532369,
		     -0.687216,      0.734198,     -1.037384,     -3.530669,      0.579760,
		      3.338618,      0.738733,     -1.912976,     -1.250679,      0.661255,
		      2.053123,      0.051668,     -1.945793,      1.911783,      1.062334,
		     -1.162851,     -1.077696,      2.180817,      2.334387,     -0.130560,
		     -0.285246,      0.381901,     -0.980849,     -2.167578,      0.846670,
		      0.853719,      0.945273,     -0.590587,      0.329096,     -0.231739,
		      1.193208,     -0.499063,      0.360731,     -1.191657,      1.018567,
		      0.763550,      0.401547,     -1.789793,     -0.294823,     -0.231709,
		      0.838620,      2.640372,      2.132161,      0.510503,      1.135979,
		      0.202748,     -0.058734,      0.074847,      1.345999,      0.627243,
		     -0.445153,     -1.901647,     -0.874265,     -1.280188,     -0.447066,
		      1.729481,     -0.033128,      0.378041,      2.109490,      1.783913,
		      0.009588,     -0.166545,     -0.572527,     -0.043455,      0.350946,
		      0.011361,      1.400090,      0.481109,      3.036624,      1.596567,
		      1.140180,     -0.051435,     -0.258781,     -2.208833,      0.143522,
		     -1.718376,     -2.132840,     -0.779583,      0.061171,     -0.475176,
		      0.259403,      0.729527,      1.239099,      1.276365,      0.354674,
		      0.696293,     -1.780829,      0.352317,     -1.131490,      0.453892,
		      1.890537,      0.936307,     -0.990830,      0.574842,      0.086694,
		      0.903132,     -0.826859,     -2.944008,     -0.568592,      2.696684,
		      0.634365,     -1.834421,     -1.365375,      0.644964,      1.499473,
		      0.860433,     -1.409920,      0.027862,      1.218087,     -1.246473,
		     -1.622746,      1.360329,      2.224822,     -0.774808,     -3.181342,
		     -2.108705,     -0.081885,     -0.024870,     -0.142595,      1.463671,
		      0.222092,     -0.863024,     -0.701609,      0.745111,      0.848741,
		      0.558533,     -1.737000,     -1.300971,     -0.557913,      1.106468,
		     -0.016307,     -0.422982,     -1.588178,      0.422522,     -0.408263,
		      1.811883,      0.892642,     -1.005792,     -2.446031,     -1.684636,
		     -2.347444,     -0.307120,     -1.027356,      1.077972,     -0.809536,
		     -0.477236,     -1.152129,      1.075230,      1.667031,      1.498416,
		      0.995241,     -0.261224,     -0.816406,     -0.349389,     -0.322442,
		     -0.247098,     -1.204991,      1.578720,      0.145475,     -1.312475,
		     -0.075115,      0.259777,      0.567706,      0.669253,     -1.775284,
		     -1.038332,     -0.776406,     -1.046581,      0.912485,      2.193141,
		      0.973586,      0.608002,     -0.084037,      0.204761,      0.957430,
		     -0.739327,     -0.704010,     -1.012239,     -0.765425,     -2.026971,
		      0.974460,     -1.735876,      0.544286,     -0.554875,      0.800501,
		     -0.271930,      0.248269,     -0.554299,     -1.214285,      0.943294,
		      0.110822,     -1.986885,     -0.399050,     -2.516470,      0.423645,
		      4.787495,     -2.387897,     -3.528378,      1.637972,      1.912744,
		     -0.703270,     -2.521378,      0.251012,      0.941209,     -0.062519,
		      0.513329,     -1.157412,      1.674067,      0.004612,     -1.841083,
		     -1.448764,      1.149633,      2.411038,      0.561077,     -0.365548,
		      1.175985,      0.471152,     -1.086655,      0.443338,      0.459804,
		      0.534213,     -1.178122,     -0.910687,      0.145954,      1.554517,
		      0.508611,      1.015331,     -1.352656,      0.190141,      1.346945,
		      1.601715,     -0.600045,     -0.722099,     -0.877700,     -1.179109,
		      0.751755,      2.080097,      1.759206,      0.926836,      0.952013,
		     -0.837403,      0.852670,      0.882231,      0.773147,      1.242771,
		     -0.274417,     -0.348677,     -0.413673,     -1.160161,     -0.770722,
		     -0.314569,      0.078719,      0.500731,      0.631763,     -0.606856,
		      1.787275,      1.026515,      0.777596,     -0.443056,     -1.708742,
		      0.082778,      1.161829,      2.558849,      1.780713,      1.683893,
		     -0.017342,     -0.365381,     -1.564757,      0.015047,     -0.161207,
		     -2.065404,     -2.083351,     -0.875970,      0.471345,      1.682161,
		     -1.061160,     -1.385766,      1.428590,      1.011157,     -0.863056,
		      2.472105,      0.200166,     -2.087110,     -0.388470,     -2.729679,
		     -0.821778,      0.968950,      0.047845,     -0.819105,      0.154002,
		     -0.011554,     -0.597322,      0.077529,     -0.611273,     -0.321396,
		      2.556431,      0.580266,     -1.611522,     -0.267638,      1.303463,
		      1.261042,     -0.347193,     -1.024717,      0.140829,      1.885951,
		     -1.173082,     -1.819542,      1.390202,      2.578583,      1.224338,
		     -0.632454,     -1.039185,     -0.091326,     -0.414715,     -0.917357,
		      1.148826,      0.695929,     -0.064731,     -1.082813,      1.045241,
		      0.771323,      1.460061,     -1.434758,     -0.289600,     -1.120087,
		      0.469933,      0.082291,      0.569674,     -1.441894,     -0.667414,
		     -0.471720,      1.000280,      0.972987,      0.016115,     -0.719666,
		      0.425839,     -1.484838,     -1.265754,     -1.033147,     -0.409329,
		      0.430084,     -0.485012,     -1.653636,     -2.659651,     -1.538833,
		      0.910134,      2.157222,      1.548906,      0.616904,     -0.614323,
		     -1.963326,      0.063216,      1.144096,     -0.429828,     -0.556039,
		     -0.939141,      0.394337,      0.737951,      0.007335,     -0.229953,
		     -1.827295,     -0.159622,      0.566022,     -1.584194,     -1.711763,
		     -0.403666,     -0.019937,     -0.570164,     -0.674510,      0.006034,
		      0.291330,     -1.110377,      0.006163,     -0.178968,     -1.934169,
		     -1.638228,     -1.247021,      0.415459,      0.277496,     -0.726599,
		     -3.429088,     -0.435196,      0.486680,      0.738003,      0.854374,
		      0.042570,     -1.148041,      1.266483,      0.676052,      1.025646,
		     -3.450194,     -3.986406,      1.314010,      2.673863,     -1.203439,
		     -0.620976,      1.460073,      0.310541,     -1.238064,     -1.942599,
		      0.716373,      1.224158,     -0.275349,     -0.733815,      0.459180,
		      2.183621,     -0.517609,     -2.096591,     -1.999325,     -0.055091,
		      1.750452,      1.031189,      0.395264,      0.496297,     -0.633314,
		     -1.740091,     -0.515871,      0.742731,      0.707001,     -0.161231,
		     -0.577111,     -0.706884,      0.099806,      0.071449,      0.619974,
		      0.370237,      0.065549,      0.743810,      0.471359,      1.135206,
		      1.012245,      1.071313,     -0.358852,     -0.717188,      0.029056,
		      0.916465,      0.638136,      2.246070,     -0.223533,      0.512674,
		     -1.097546,      0.545006,      2.356599,      1.662779,      0.109323,
		      0.249843,     -0.630269,      0.261787,     -0.210211,      0.442668,
		     -1.015185,     -1.003493,     -2.464702,     -0.199116,      1.128371,
		     -0.037986,      0.151667,     -1.062826,      0.245195,      0.412528,
		      0.463871,      1.070434,      1.493640,      0.758453,     -0.300968,
		     -0.903208,     -1.131065,      0.472728,      0.232644,     -0.429981,
		      0.713619,     -0.465172,      1.795713,      2.124804,     -0.565726,
		      0.268256,     -0.088207,     -0.929974,      0.669446,     -1.251252,
		      0.184537,      1.826248,     -0.861487,      1.387888,     -0.663295,
		     -1.197769,      0.010410,      0.087962,     -0.925778,     -0.872109,
		     -1.925883,      0.642610,      1.638375,     -2.793560,     -1.133806,
		      1.118355,     -1.493713,     -1.843541,      0.819174,      2.424151,
		      0.310519,      0.033265,     -0.521273,     -0.114504,      1.005707,
		     -0.480600,     -1.323221,      0.320990,      1.708653,     -0.669336,
		     -0.457538,     -0.854304,      0.150399,     -0.648143,     -0.776387,
		      0.850117,      1.139503,      0.048717,     -0.826472,     -0.238733,
		      0.183070,      2.001978,      0.464024,      0.741485,     -0.381861,
		     -0.133547,     -1.446498,      1.260073,     -0.754571,     -0.253851,
		     -2.103737,     -1.752967,     -0.275254,      0.213758,     -1.031184,
		      0.255151,     -0.050547,     -1.709203,     -1.489196,     -0.348789,
		      0.636339,      0.933670,     -0.048133,     -1.420265,     -1.588083,
		     -1.429915,     -1.338760,     -0.482910,      1.933103,      1.041359,
		     -0.616903,     -1.115159,      0.157712,      0.670397,     -0.478025,
		     -0.863776,     -0.207679,      1.553857,      0.439325,     -0.173697,
		     -0.828849,     -0.262052,      0.267701,      0.301470,     -0.224932,
		     -0.981854,     -2.063164,     -1.589829,     -0.899151,      0.922451,
		      0.263991,     -0.627248,     -0.758694,      0.196775,      0.623539,
		      1.417924,     -0.082340,      0.134717,      0.540581,      1.027952,
		      1.711111,      0.534075,     -1.773786,     -1.826314,     -1.319526,
		      0.644737,      0.359027,      1.467995,     -1.930916,     -2.249020,
		      0.365485,      1.057552,     -0.293557,      0.090538,      0.369354,
		      1.185824,     -0.145814,      0.549444,      2.681864,     -1.311528,
		     -2.143804,     -1.519611,      0.187559,      1.188812,     -0.170630,
		     -1.313319,      0.319326,      2.941814,     -0.201090,     -0.883589,
		     -0.483419,     -0.611655,      0.351269,      0.472389,      0.510984,
		      1.732657,      1.037545,     -0.917759,     -0.357423,      0.975257,
		     -0.426722,      1.050991,     -0.443133,     -0.764830,     -0.488612,
		     -0.172657,     -0.297694,      0.292343,      0.026290,      0.426158,
		     -0.290588,      0.534724,      1.281415,      1.972302,      1.052935,
		      1.503155,      0.130475,      0.736894,     -0.226874,      0.881679,
		      0.919016,     -0.120087,     -1.250245,     -1.829170,     -0.296500,
		      2.071976,      0.561658,      0.491552,      0.211117,     -1.202716,
		     -1.272439,      0.088309,      0.786712,      0.355186,     -0.677653,
		     -1.304555,      1.492920,      0.147868,      0.993647,     -0.669791,
		     -0.719486,      0.526798,      1.040194,     -0.555326,     -1.121059,
		      0.693433,      0.601107,      0.554894,      0.918442,     -0.182891,
		      0.026813,      0.030651,      1.520047,      0.696492,     -0.685512,
		     -1.764951,      1.518685,      1.615324,      1.754703,      1.946936,
		      0.377060,      2.643642,     -1.417963,     -3.115934,     -0.850867,
		      0.620806,     -1.653651,      0.620946,      1.175416,     -0.680966,
		     -0.130625,      1.265302,      0.536499,     -3.945167,     -1.087505,
		      4.237032,      1.894346,     -1.620903,      0.214216,      0.090869,
		     -2.695693,     -1.986600,      1.130607,      1.686828,      0.291165,
		     -0.988131,     -0.840394,      1.190497,      0.357139,     -0.941283,
		     -1.100673,      0.787774,      2.738811,     -0.327258,      0.211889,
		      0.112538,     -0.158816,     -1.110241,     -1.963913,     -0.491954,
		      0.567442,      1.497264,     -0.247895,     -0.241276,     -2.135509,
		      0.559499,      0.729400,      1.155032,      0.283705,      1.153801,
		     -1.484454,      0.200866,      0.036676,      1.584720,     -0.395161,
		     -2.441443,     -1.410940,     -0.815208,     -1.993237,     -1.575089,
		      1.766829,     -0.807316,     -1.434036,     -1.492008,     -0.409672,
		      1.513914,      2.110443,      0.541021,      0.681103,     -1.408037,
		     -1.140686,     -1.959126,      1.351324,      0.509812,     -0.340897,
		     -2.211846,      0.950103,      0.970805,     -0.199669,      0.025123,
		     -1.335702,      0.953869,      1.687207,     -0.599310,     -0.264654,
		     -0.295934,      0.496140,     -0.014037,     -1.104708,     -0.579549,
		     -1.407522,      0.245921,      0.068745,      0.871414,     -1.403547,
		      0.253361,      0.081259,      0.784989,     -2.602788,     -1.766504,
		     -0.308325,      1.973046,      0.302759,      1.593089,     -0.494274,
		     -2.091453,     -0.075007,      2.108016,     -0.484037,      0.061392,
		     -0.900155,     -0.515628,     -0.183872,      3.018578,      0.006059,
		     -1.890835,     -3.115318,     -0.277432,      0.160499,     -2.074821,
		      0.073001,      0.743775,     -1.582120,      0.476763,      2.651341,
		      1.214274,     -0.895450,     -1.635513,      0.006011,      1.737281,
		      0.589630,     -1.186627,     -1.346755,      1.244134,     -0.164703,
		     -1.865092,     -0.398244,     -0.708928,     -0.081063,     -0.114229,
		     -0.892409,      0.350880,      1.674386,     -0.068646,     -0.003857,
		     -0.202007,     -0.546794,      1.467887,     -0.032191,     -0.787827,
		     -1.180381,      0.660956,     -0.530323,      0.618699,     -0.227210,
		      0.989009,     -1.049467,     -0.946048,      1.161613,      2.370851,
		      1.028258,      2.375651,      1.612221,      0.150861,     -0.012959,
		      1.368422,      1.396269,      0.351841,     -0.077191,     -2.044723,
		     -2.337840,      0.047798,     -0.403855,      0.921833,      1.625684,
		      0.030862,     -0.181161,     -0.651014,     -0.344978,     -0.222678,
		     -0.888407,     -1.011687,     -0.070888,      0.262166,      0.369455,
		     -0.604809,     -0.530774,      2.092364,      0.611459,      0.988572,
		     -0.108290,      0.730085,     -0.376295,      0.178961,     -0.781113,
		      0.301979,     -1.838069,     -0.919989,      0.257350,      0.249998,
		      0.335520,      0.274331,      0.770780,     -0.490652,     -0.415634,
		      1.986875,      0.979290,      1.989769,      0.002337,      1.183230,
		     -0.223729,     -2.094489,     -1.250736,      1.714576,      0.395325,
		     -1.351143,      1.135375,     -2.265158,     -1.674982,      1.088838,
		     -0.661160,     -1.985233,      0.454078,      2.526733,      0.261572,
		     -1.006744,      0.778199,      1.439793,     -2.428457,     -1.463437,
		      0.200108,      1.400945,      0.798225,     -0.159294,     -1.594934,
		      1.186074,      0.873482,     -0.531044,     -0.372551,     -1.570585,
		      1.502959,     -0.162661,      0.051741,      1.736769,      1.949283,
		      0.489129,     -1.837279,     -1.556915,     -1.097458,     -0.217815,
		      1.649487,      0.005418,     -2.011296,     -0.629458,      0.024908,
		     -0.476326,     -0.825129,      0.680853,     -0.300836,     -0.570364,
		     -0.363617,      0.970522,      0.285525,     -1.909990,     -0.155202,
		     -0.685963,     -0.686364,     -1.494317,      0.970644,      0.767574,
		     -1.100027,     -1.161950,      0.052865,      1.244140,      1.928354,
		     -0.018130,      0.758993,      0.133523,     -0.587219,     -0.818652,
		     -0.312130,      0.078885,      0.057702,      0.817950,      0.500243,
		      0.662932,     -0.252199,      0.835132,     -0.166921,     -0.125536,
		     -0.860332,     -1.318251,      0.160289,      0.020207,     -0.119704,
		      0.375473,      1.427869,      0.943660,     -1.192275,      0.707357,
		      1.020532,      0.983587,     -0.097115,     -0.567496,      0.954882,
		      0.065314,     -0.835310,      0.427822,     -1.343004,     -0.274903,
		     -1.426910,      0.028727,     -1.663490,      0.357239,     -1.237676,
		     -1.406438,     -0.424044,      0.322748,     -1.008750,      1.428474,
		      0.087703,     -0.567939,     -0.901460,      1.772098,      0.762421,
		     -1.879812,      0.982919,      3.080194,     -1.633702,     -2.180755,
		      2.201586,      1.223796,     -2.660808,     -0.572224,      1.942231,
		      1.548119,     -0.714455,     -1.443267,     -0.861646,      0.276093,
		      0.347716,      0.633921,     -0.462657,      0.252079,      1.282976,
		     -0.312483,      0.784567,     -0.928382,     -1.105980,     -0.561507,
		     -0.805100,     -0.593644,      0.717015,     -0.028158,      0.019277,
		     -0.469768,     -1.239236,      2.154484,      2.131077,      0.482224,
		     -1.571931,      0.991660,     -0.496669,      0.656209,     -0.039827,
		      1.760542,      0.085091,     -0.712725,      0.540816,      1.348971,
		     -0.246231,      0.057259,      1.544469,     -0.453315,     -1.147988,
		      0.028779,      0.789400,     -0.856336,     -0.067877,      0.404331,
		     -0.385587,     -1.681638,     -1.363660,     -1.643602,      2.166248,
		      1.559258,      0.809343,     -0.749604,      0.667571,     -1.281725,
		      0.384963,      0.761548,      0.468403,      0.674914,      0.098216,
		      0.443883,      1.285231,     -0.930143,     -1.284591,      0.596955,
		      0.050930,     -0.570108,     -0.994117,      0.950944,     -0.589242,
		     -0.307511,     -0.207620,     -0.150104,     -1.509105,      0.041461,
		     -1.092972,     -0.850825,      1.212572,      1.241113,     -0.320935,
		     -1.362570,     -1.447760,     -0.801016,     -0.228983,      0.466668,
		     -0.863963,      1.303198,     -0.444139,     -0.341512,     -1.106021,
		      0.516383,     -0.092688,     -0.229043,      2.065670,     -0.707411,
		     -0.595258,     -1.574123,     -0.963823,     -0.346199,      0.543247,
		      1.914355,     -0.089085,     -1.961807,      0.215908,      1.943784,
		     -1.513871,     -1.482137,      0.675861,      1.043232,      0.647757,
		      0.494669,     -1.326082,     -0.118801,      0.157142,      0.509867,
		      1.713476,     -1.179344,     -0.380852,     -0.838760,     -1.301632,
		     -0.468064,      0.545453,      1.577298,      0.025395,      1.336381,
		     -0.009590,     -1.071263,      0.561675,     -0.421782,     -1.450788,
		     -1.029224,     -0.131446,     -0.637755,     -1.398157,     -0.133780,
		     -0.393521,     -2.280114,     -1.110208,      0.872857,      1.609411,
		     -1.306945,      0.867697,     -0.181311,     -1.206214,     -1.633791,
		      1.554545,      1.335781,      0.988812,      0.888894,      0.470105,
		     -0.692392,      0.944526,     -0.081949,      1.705260,     -0.160322,
		     -2.381224,     -1.127258,     -1.134083,      0.612332,      0.207811,
		      0.824089,     -0.712442,     -0.000571,     -1.301083,     -1.031948,
		      0.530799,     -0.680348,     -0.326567,     -0.646924,     -0.160364,
		     -1.126644,      0.219829,      1.605851,      1.121332,      0.279312,
		      1.528291,      0.417655,      0.687267,      0.168963,      0.077745,
		      1.187557,      0.325456,     -0.386507,     -0.015362,      0.591600,
		      1.802527,     -0.259300,      1.468360,      1.257713,      1.755112,
		     -2.169591,     -0.096232,     -1.095251,     -2.465634,      0.710369,
		      0.246228,      0.342391,     -1.042448,      1.025676,      1.034158,
		     -0.738454,     -0.520796,     -0.095926,     -2.640175,     -0.286580,
		      2.090041,      1.051211,     -1.940607,     -0.695347,      1.942242,
		      0.672514,     -3.140900,     -0.239763,      1.555319,      1.154126,
		     -0.567758,     -0.254843,     -0.056177,      0.902856,     -0.782959,
		      0.227801,     -0.064152,     -2.189734,     -0.694666,     -0.068166,
		      1.658906,      0.410415,     -0.536158,     -0.810533,     -0.138044,
		     -0.203133,     -0.084322,     -0.137998,     -0.427875,     -0.267623,
		     -1.248147,      0.487969,      0.800910,      0.433237,     -0.852262,
		      1.316614,      0.090989,      0.857150,      0.355735,      1.636327,
		      0.498014,     -1.385395,     -0.329511,      0.250259,     -0.358321,
		     -0.263639,      2.463142,      0.029641,     -1.679686,     -0.703893,
		      0.383514,     -1.119656,     -1.364206,      0.274576,      0.605378,
		      0.025707,      0.158355,      0.243442,      1.123976,      0.806173,
		     -0.473377,      0.753769,      0.766702,      0.324522,     -1.367893,
		      0.086458,      1.951716,      0.480475,     -0.579254,     -0.793754,
		      1.276300,      0.538198,      0.160642,     -0.615926,     -0.510272,
		      0.216582,     -1.196340,      0.057360,     -1.369668,     -0.785409,
		     -0.910719,      0.633978,     -0.729024,     -0.708427,     -0.795604,
		     -2.020477,     -0.772392,     -0.341369,     -0.966093,     -0.597759,
		     -1.891518,     -2.597019,     -1.894495,     -2.592480,     -0.531176,
		     -1.154157,      1.090098,     -0.096860,     -1.260304,      1.125921,
		      0.610141,     -1.559620,      0.086994,     -1.285227,      0.178757,
		     -0.461054,      0.712165,      0.922395,     -0.009935,     -1.600754,
		      0.427674,      1.478228,     -1.157787,     -1.969719,      1.529359,
		      1.965559,     -1.982238,     -1.612663,     -0.735138,      0.981013,
		      0.133036,      0.195631,      0.120741,     -0.290519,     -0.670550,
		      0.893029,      2.046228,      0.534390,     -0.808860,      0.704775,
		      1.155514,     -0.304262,     -1.435461,     -0.159634,     -1.718174,
		      0.599911,      1.474254,      0.746415,      0.916342,      0.062391,
		      0.207871,      0.603352,      1.192889,     -0.902919,     -0.852008,
		     -0.317839,      0.557381,     -1.995372,     -2.661853,     -1.043638,
		      0.823545,     -1.350619,     -0.338846,      1.034074,      0.359697,
		     -2.183752,     -0.504061,     -0.194515,     -0.567455,      1.192152,
		      1.900370,      0.632644,     -0.270731,     -1.071683,      0.139390,
		      0.940011,     -0.188066,     -0.858647,     -0.618030,      0.778123,
		     -1.723727,      0.988709,      1.073192,      0.833909,     -1.838679,
		     -2.446313,      0.826412,      1.574990,      0.738483,     -0.451067,
		     -2.590106,     -0.169103,      0.528024,      0.197350,     -1.021205,
		     -1.160898,      1.898499,      0.739464,      0.128234,      0.435743,
		     -0.690483,     -0.504634,      1.738203,      1.157406,     -0.641575,
		     -0.014314,      2.519011,      0.174168,      1.180542,      0.906467,
		      0.722533,      2.185792,      0.490662,     -0.730407,      1.145715,
		     -0.683832,     -1.400196,      0.912448,     -0.951082,     -0.029501,
		     -0.734316,      1.421871,      0.700390,      0.232900,      1.853037,
		     -1.561755,     -2.453807,      1.086102,      1.260339,     -0.302032,
		     -0.851859,      0.701802,      1.370074,     -0.039558,     -2.135876,
		      0.649722,      1.151917,      0.721065,      0.172805,     -0.707928,
		     -0.561224,      0.296915,     -0.178523,      0.728808,      1.530257,
		     -0.792990,     -1.145786,     -0.750352,      0.361372,      0.170730,
		     -0.613698,     -1.081640,      0.099359,      1.510406,      0.356132,
		     -0.672324,     -0.369955,     -1.390116,     -1.134315,      0.517909,
		     -0.253453,     -1.150847,     -1.427598,      1.067008,     -0.148628,
		     -0.938176,     -0.099370,      1.278573,      2.471304,      0.147121,
		      1.075114,      0.065493,     -0.235238,     -2.302210,      0.785830,
		      2.423008,      0.878603,     -0.304961,      0.970903,     -2.056058,
		     -1.970626,     -0.603508,      1.206047,      0.254150,     -0.315333,
		     -0.114629,      0.399554,      1.509405,     -0.142652,     -0.430013,
		     -0.848386,      0.297121,     -0.883475,      0.304148,      0.856181,
		      0.301944,     -2.018833,     -1.638008,      0.306649,      0.500740,
		      0.823333,      0.325508,      0.930905,      1.675306,      0.288641,
		     -0.348384,      0.313708,     -2.263579,     -0.067052,     -1.647701,
		     -0.214661,      1.246278,     -0.506328,     -0.474364,     -1.299699,
		     -2.589194,     -2.356266,     -0.714569,     -1.886867,     -1.439409,
		      1.161009,      3.551842,      0.072365,     -0.736715,     -2.144592,
		     -0.405674,      0.667429,     -0.567980,     -1.469520,      1.933829,
		      1.820312,     -0.533593,      0.537318,      0.488024,     -0.487604,
		     -4.238056,      0.687515,      1.756920,      0.202702,     -0.113062,
		      1.593938,      0.630038,     -2.510422,     -1.616503,      1.892768,
		      1.970690,      0.207060,     -0.917225,     -1.671242,     -0.330641,
		     -0.334854,     -0.008792,      0.605531,     -0.897328,     -1.849815,
		     -0.904853,      0.675327,      0.413528,     -1.401767,     -0.875474,
		      1.431894,      1.556240,     -0.015783,      0.373683,     -1.471390,
		     -1.560874,      0.734364,      1.166142,      1.175552,     -0.496344,
		     -1.817024,     -0.650092,      1.676537,     -0.124644,     -0.469071,
		      0.409563,      2.106255,      1.473761,     -0.899936,     -2.123101,
		     -0.221841,     -1.179317,     -1.110487,      1.351552,      1.729498,
		      0.536228,     -0.263476,      0.470983,      0.115497,     -0.244294,
		      1.870221,      1.595284,      0.962750,     -0.737579,      0.532843,
		      0.421075,      0.646470,     -0.777585,     -1.123632,      1.543927,
		      1.016071,     -0.123878,     -0.127690,     -0.755995,     -1.264730,
		      0.554282,     -0.600234,      2.620421,      0.596661,      0.514123,
		     -1.311851,     -1.088462,     -1.117936,     -1.079668,     -0.387065,
		     -0.776694,     -0.608111,      1.340535,      1.419015,     -0.540510,
		     -0.428192,     -1.167257,     -0.363006,      1.609646,      1.241203,
		      2.357756,      0.611913,      0.131225,      1.306191,     -0.684579,
		     -0.193702,     -0.133183,     -1.489460,     -0.624751,      0.036880,
	};
	static const std::vector<double> h = {
		      0.000000,     -0.176772,      0.000000,      0.084008,      0.889939,
		      0.000000,      0.176474,     -0.873088,     -0.221023,      0.000000,
		     -1.432377,     -0.869624,      0.406644,      0.750396,      0.000000,
		      1.761490,      0.998714,     -1.236534,     -0.915848,     -1.019891,
		      0.000000,     -0.827691,     -0.063928,      0.104902,      0.703612,
		      1.304685,      0.204100,      0.000000,     -0.594437,      0.492858,
		      0.775275,     -0.254421,      0.285096,     -0.231246,     -0.735806,
		      0.000000,     -0.311790,     -1.964098,     -0.956664,     -0.755940,
		     -0.331052,      1.963661,      1.076495,     -0.380347,      0.000000,
		      0.847213,      1.749613,      0.790421,      0.189780,     -0.055195,
		     -0.343160,     -0.703304,      0.170957,     -0.815060,      0.000000,
		     -0.211845,     -0.734542,     -1.199647,      0.076826,     -0.447561,
		     -1.632536,      0.441557,      0.677000,      0.519366,      0.449706,
		      0.000000,     -0.465921,     -0.342714,      0.570489,      0.441626,
		      0.946414,      1.409180,      0.580421,     -0.495098,      0.141271,
		     -1.753819,     -1.438113,      0.000000,      0.287353,     -0.758273,
		     -0.487969,     -0.948385,     -0.829052,     -0.376816,     -0.923989,
		     -0.595935,     -0.136737,      2.022994,      1.355922,      1.074905,
		      0.000000,     -0.037684,      1.841646,      0.468468,     -0.436285,
		     -0.610393,      0.066800,      0.759272,      0.061745,      1.238583,
		      0.305497,     -1.628962,     -1.794343,     -1.366745,      0.000000,
		      0.436295,     -0.411221,     -1.308476,      1.308172,      1.150864,
		     -0.249788,     -0.556671,      0.164166,     -0.043913,      0.425966,
		      2.589158,      0.516863,      0.122063,     -0.028479,      0.000000,
		     -1.924845,     -1.522375,      1.368378,      0.383107,     -0.341486,
		     -0.356044,      0.962734,      0.695467,     -0.440947,      0.410950,
		      0.132766,      0.106483,     -0.004686,     -0.839596,      0.281408,
		      0.000000,      1.176380,      0.886922,     -0.409245,     -2.099021,
		      0.420277,      1.786598,     -0.766433,     -1.645021,     -1.202896,
		     -2.063627,     -2.040453,      0.701380,      0.523563,     -0.345085,
		     -0.414078,     -2.934250,      0.000000,      1.008195,     -0.189024,
		     -2.064898,      0.592074,     -0.467936,     -2.177127,     -1.489489,
		      1.553676,      1.466413,      2.301104,      1.449556,     -0.837488,
		     -0.700473,      0.493430,     -0.386083,      1.249703,      0.777930,
		      0.000000,     -2.648544,      0.577695,      1.412523,      1.294722,
		      0.134309,      0.632400,      2.744502,      0.766932,     -0.349387,
		     -1.346175,     -0.798916,     -0.784583,      0.996233,      0.912931,
		      0.768492,      0.327688,     -0.763490,     -1.516752,      0.000000,
		     -0.022181,     -0.872120,      0.569810,     -1.637010,      0.346720,
		      0.279521,     -2.626510,     -2.623484,     -1.756056,      0.108769,
		      1.756870,      1.906155,     -1.230917,      0.057826,     -0.146045,
		     -3.157346,     -0.094203,      0.960208,      0.854126,      0.000000,
		      1.182828,     -1.666911,     -1.102873,     -0.653754,     -0.338985,
		      0.706918,      0.858616,      2.661286,      1.696657,      1.277320,
		     -2.011992,     -0.025493,      0.166204,      0.000413,      0.925016,
		      1.935047,      1.511303,      0.468277,     -0.038073,      0.074613,
		      0.000000,      0.707366,      1.779903,     -0.063878,      0.887211,
		     -0.621810,     -1.103166,      0.961526,     -0.563407,     -0.709492,
		     -1.745758,     -0.585235,     -0.176395,      1.337653,      0.454511,
		      1.784041,      1.468660,     -1.912111,     -2.158080,     -1.054587,
		      1.047750,      2.513966,      0.000000,     -1.303636,     -0.946297,
		     -0.580398,      0.243072,      1.076868,     -0.609188,     -0.704117,
		      0.149918,      1.359686,      2.442357,      2.577965,     -0.690631,
		     -1.717098,     -0.364180,     -2.408494,     -1.048695,      0.586088,
		      0.961719,      1.941578,     -0.591364,     -1.635095,      0.095539,
		      0.000000,     -0.673203,     -1.045683,      1.415376,      0.841406,
		      0.753582,      1.591232,     -0.183883,     -0.133131,     -1.615464,
		     -0.558013,     -1.642709,      1.477718,      0.499206,      0.087506,
		      0.584353,      0.824362,      1.916491,      0.425517,     -1.583678,
		     -1.324548,     -0.249393,      2.487686,      1.212168,      0.000000,
		      2.045893,      0.622888,     -2.548701,     -2.631976,     -1.505836,
		     -0.709868,     -0.297136,      0.349611,      1.364194,     -0.987422,
		      0.279611,      0.497118,     -0.569971,     -0.722809,      0.608394,
		     -1.630197,     -2.314534,     -1.386614,      0.304994,      1.818092,
		      0.235558,     -3.199293,     -1.198440,      0.071050,      0.000000,
		     -0.583898,     -0.369661,      0.112568,      0.069278,      0.378018,
		     -0.025170,      1.648405,      0.853307,     -0.546120,     -0.121557,
		      0.785831,     -1.008640,     -0.144931,      0.445127,     -0.473112,
		      2.265576,      1.898491,      1.305242,      0.365457,     -2.961415,
		     -1.247972,      1.322115,      1.102106,      0.637022,     -1.842062,
		      0.000000,     -0.748555,      0.712941,      1.548203,      0.482548,
		      0.096199,     -0.443196,     -1.609594,     -2.408419,     -0.877623,
		      1.839282,     -0.502912,     -0.011485,      0.235130,      0.002650,
		     -1.570576,     -0.652449,     -0.107268,     -0.890556,     -1.175549,
		     -1.002894,     -0.125806,      0.210746,     -0.624205,      0.058294,
		      0.963710,     -1.114648,      0.000000,      1.219393,     -1.682827,
		     -0.687274,      0.341181,     -0.661645,     -0.710540,      0.230098,
		      2.149115,      1.252419,     -1.431551,     -1.552033,      0.340129,
		      0.768659,      0.349877,      0.769829,      0.212515,      0.666695,
		      0.454597,      2.804097,      2.388772,     -2.486532,     -2.109596,
		     -0.835468,      1.505472,      0.716203,     -1.141735,     -1.948435,
		      0.000000,     -0.211111,      0.369580,     -0.678796,     -0.473444,
		     -0.528763,      0.507372,      0.157384,     -0.408782,     -1.127612,
		      0.325912,      2.164256,     -0.124032,     -0.835385,     -0.430136,
		      0.070376,     -0.399404,     -2.266800,     -0.316765,     -1.626711,
		     -1.524883,      1.392747,      1.472107,      1.198281,     -0.306321,
		     -0.938463,      3.524075,      1.241459,     -0.652516,      0.000000,
		     -0.201097,      3.047158,      0.276267,     -0.421271,      1.022241,
		     -0.382272,     -0.473498,     -0.206740,      1.609834,      1.600029,
		     -0.596667,     -1.202216,      0.706310,      1.337769,     -0.312032,
		     -0.197544,      0.876243,      0.897946,     -1.005050,      0.591277,
		      2.087476,      0.450797,     -1.359317,      0.140622,     -0.775037,
		     -0.771165,     -0.197885,     -1.639669,     -0.002338,      0.000000,
		      0.072306,     -1.915638,      0.539382,      0.106382,     -0.456980,
		     -0.638781,     -0.598353,     -0.544364,     -1.536190,     -1.555178,
		     -0.235527,      1.593934,     -0.275867,     -1.948424,     -1.132387,
		      1.281530,      1.028565,     -1.481649,     -0.138678,     -0.105988,
		     -0.608526,      1.598565,      3.256650,      1.759896,      1.820225,
		     -0.613464,     -0.354282,      1.898002,      0.909344,      1.145401,
		      0.000000,     -2.041075,     -2.216061,      1.113545,     -0.448324,
		     -1.015438,      0.531311,     -0.399993,     -0.447753,     -0.062299,
		      0.413756,      0.468388,      0.582775,     -0.076393,      0.248936,
		      1.547722,     -0.280974,     -0.741259,      1.104027,      0.826400,
		     -0.917700,     -1.237869,     -2.260012,     -1.606108,     -1.576311,
		     -0.098719,      1.565184,      0.810341,     -1.566052,     -2.468288,
		     -1.254910,      0.919733,      0.000000,      1.568647,      2.885588,
		     -2.474766,      0.001559,      1.156860,      0.663277,      0.815196,
		      0.636668,     -0.013666,     -0.989076,     -0.575920,     -0.481557,
		      0.715219,      0.262742,     -1.047865,     -1.345309,     -0.926098,
		     -0.029278,     -1.066822,      0.727742,      0.394713,      1.917126,
		      1.075837,      1.613689,     -0.662730,      0.979643,      0.100094,
		      1.000914,      1.427421,     -0.482116,      0.836013,      1.386714,
		      0.000000,      0.881999,     -0.783360,      0.742826,      1.204725,
		     -1.079124,     -2.277565,     -0.353550,     -0.533583,     -0.115970,
		      0.735122,      0.041481,     -1.435947,     -1.405041,      0.521300,
		      0.750497,      0.743676,      0.057359,     -0.067667,      0.156428,
		      1.358060,      0.661914,      1.286332,     -0.065049,     -1.379379,
		     -1.945320,     -1.444980,      2.329671,      1.307134,      0.155512,
		     -0.600173,     -1.492554,     -1.731911,     -0.242889,      0.000000,
		     -2.852549,     -0.241943,      2.360678,     -2.701079,     -0.722518,
		      1.578567,     -0.905520,     -0.643522,      0.076127,      0.588585,
		     -0.612883,      0.800158,      1.645575,      0.610784,     -0.247821,
		     -1.215539,      0.954134,      0.634628,     -0.342916,     -2.255917,
		     -0.810791,     -1.439031,      0.090611,      0.969697,      0.378587,
		     -0.913195,     -1.851945,     -0.464208,      0.574881,      0.469902,
		      0.757694,      0.709586,      0.295695,      0.969453,      0.000000,
		      0.803961,     -1.304131,     -1.878791,      2.744163,      2.616776,
		     -0.529049,     -1.068989,      1.345592,     -0.810026,     -1.600962,
		      0.715462,      1.134605,     -0.822434,     -2.035450,      0.166640,
		      1.211427,     -0.771825,     -0.689729,      0.270779,     -0.168973,
		      0.572037,      1.189176,      0.994514,     -0.520350,      0.312428,
		      1.909999,     -0.020396,      0.823372,     -0.133008,      0.719461,
		      0.736786,     -0.325451,      0.129100,     -2.314665,     -0.830480,
		      0.000000,      1.297225,     -0.851248,     -0.057849,      0.779444,
		     -2.421197,      0.124574,      2.512393,     -0.103091,     -0.253559,
		      1.067631,      0.679220,     -2.819689,     -1.710616,      1.063257,
		      1.347483,      0.304082,     -0.565022,      1.037289,      0.440274,
		      1.305325,     -0.153113,      0.099827,     -0.946504,     -0.014603,
		      0.129532,     -0.401006,      0.386222,     -1.735299,      0.443054,
		     -1.500988,      0.314078,      0.373781,      0.157747,      1.228572,
		     -1.436028,     -1.024610,      0.000000,     -0.995150,      2.310125,
		     -1.133295,     -2.329612,      1.863589,      0.843021,     -2.044139,
		     -1.992574,      1.359749,     -0.223283,     -1.355930,      0.144719,
		      2.190125,     -0.238246,     -1.802409,     -0.061363,      1.776961,
		     -0.827405,      0.363276,      0.791438,      0.046314,     -0.335303,
		     -0.041717,      0.463332,     -1.097955,     -1.799334,      2.115509,
		      1.470283,      1.050001,      1.167374,      1.081353,     -1.068532,
		      0.453835,     -0.695768,     -0.069369,      1.311474,     -1.364667,
		      0.000000,     -0.810233,      0.229448,      0.271883,      0.389809,
		     -0.470762,     -1.577373,      0.432482,      1.916943,     -1.379942,
		     -1.720061,      1.113223,      2.438606,     -1.234069,     -1.093497,
		     -0.649239,      0.362012,     -0.501445,      0.012840,      1.083196,
		     -1.373748,     -0.536268,      0.657557,      1.140801,     -0.328189,
		     -0.149905,      0.221677,     -1.336793,      0.344674,     -3.364725,
		     -1.594443,     -0.461658,      0.977706,      0.341063,      0.813377,
		      0.537023,     -2.715062,     -0.520745,     -0.333804,      0.000000,
		      0.097332,     -2.095196,      1.443082,      0.086632,     -0.078952,
		      2.646965,      1.336005,     -0.235000,     -0.039073,      2.165652,
		     -0.506323,     -1.372469,     -0.401120,      0.988517,      0.646267,
		     -2.004796,     -0.913064,      0.253471,     -2.537432,      0.586666,
		      1.499288,      0.487814,      0.166495,      1.444850,      1.136672,
		      1.122392,     -0.874833,      1.243842,      1.595158,      0.211425,
		      0.566443,      0.110837,     -0.559477,     -1.845648,     -0.166144,
		      1.540565,     -0.826358,      1.885320,     -0.656410,      0.000000,
		      0.409283,     -0.146781,     -0.623554,      0.248952,     -0.021928,
		     -1.954640,     -0.517692,      0.092215,      1.235214,     -1.149319,
		     -1.750798,     -1.499288,      0.439290,     -1.579859,      0.694359,
		      0.319194,      0.663740,     -0.550959,      1.216057,      2.004541,
		     -0.115644,     -2.020718,     -1.861462,     -1.341533,     -1.139410,
		     -2.081110,      0.133513,     -1.433011,     -0.242924,     -2.902704,
		     -2.245535,     -0.914442,     -0.917433,      0.809291,      0.175727,
		      0.157557,     -0.758097,     -1.983803,     -0.119256,      0.780630,
		      0.000000,      0.253390,      0.415982,     -2.381091,      0.112081,
		      0.167495,     -0.888175,      0.369231,      0.966502,     -1.723571,
		     -0.096118,      2.465634,      1.388606,      0.474779,      1.279517,
		      0.181444,      0.770608,      0.673861,      0.659463,      0.010845,
		     -3.037024,      0.017383,      2.120415,      1.514735,      0.872733,
		      0.480309,      0.226125,      1.674190,      1.291417,      1.275740,
		      1.117434,     -0.042982,     -0.009970,      1.195993,     -0.892980,
		     -1.190786,      0.060593,      0.080098,      1.302267,      1.547676,
		      0.649088,      0.195175,      0.000000,     -1.023991,      0.100702,
		      2.436935,     -0.219230,     -0.930501,      1.425003,     -0.514420,
		      0.206122,      1.400526,      1.375079,     -0.663394,      0.008173,
		     -1.328833,     -1.892381,     -2.575540,     -0.574245,     -1.248873,
		     -0.528156,     -1.700087,      0.604531,      1.661770,      0.033896,
		     -1.031920,     -0.611003,     -0.986428,      0.217472,     -2.238434,
		     -1.206777,     -1.742837,      0.743748,      0.056888,     -0.548577,
		     -1.547053,     -1.191114,     -0.915360,     -1.022692,      0.455870,
		     -0.078994,     -2.238959,      1.796990,      1.904316,      0.026248,
		      0.000000,      0.978318,      1.225345,     -0.151494,     -1.917227,
		      0.509066,      1.334889,     -1.024409,     -1.767815,      1.201839,
		     -1.628703,     -0.981642,     -0.699987,      0.528978,      0.833063,
		      3.246688,      0.486885,      0.478278,      0.536857,      1.296750,
		      1.410882,     -1.302765,     -1.057404,      1.401916,      0.581522,
		      1.772365,      0.907488,     -0.069411,      0.735057,      1.349502,
		     -0.351390,      0.561646,      0.893866,      0.874424,      1.295165,
		      0.880833,      0.954252,      0.300564,      1.496551,      1.568885,
		     -1.451581,      1.064729,      0.818314,      1.168690,      0.000000,
		     -0.673809,     -1.326342,     -1.161256,      1.949255,      0.218076,
		     -2.146921,      0.815082,      0.451344,     -1.836843,      1.026338,
		      1.437869,     -0.612356,      0.387468,     -1.218533,     -1.746769,
		     -0.577026,      0.617299,     -0.862151,     -1.038691,     -2.526948,
		     -1.722401,      0.831152,      0.293055,     -1.004648,     -2.529841,
		     -1.911341,      0.236918,      0.717403,     -0.299905,     -0.226744,
		     -0.916382,      0.679869,      1.085490,      0.118966,     -1.181896,
		     -1.173454,     -1.917203,     -0.580348,      0.119961,      0.923554,
		     -1.323677,     -0.124851,      0.261657,      0.382370,      0.000000,
		     -0.473670,      0.469345,      0.605194,     -0.685890,     -0.891093,
		      0.707376,      0.478209,     -0.420884,     -1.018629,      1.806318,
		      0.039184,      1.654905,      0.435320,      0.941633,     -1.144818,
		      0.576033,     -0.280482,     -0.338881,      1.015514,      1.332207,
		      1.128751,      0.411762,     -0.479032,      0.499018,      0.970311,
		      1.556893,      0.886318,     -0.990023,     -0.874387,      1.703575,
		     -0.266626,     -0.367820,     -1.421967,     -0.666652,      1.201346,
		      2.670584,      2.473172,     -0.325937,     -0.541042,     -0.027939,
		      0.089995,     -0.232550,     -0.726612,      0.388336,      0.748609,
		      0.000000,      0.786569,     -0.886573,      0.696803,     -0.217896,
		      0.459229,     -0.631559,     -0.951657,      1.666195,      2.534051,
		     -2.898943,     -0.593114,      0.593075,     -1.461208,     -0.332900,
		     -0.046159,     -1.450207,     -0.377899,      0.368416,     -0.225259,
		      0.174593,     -0.889052,     -2.364226,     -0.499801,      0.159375,
		      1.336520,     -0.809409,     -1.733032,     -1.118057,      0.621014,
		     -1.356866,     -1.398693,      0.093523,      2.785130,      0.627771,
		     -0.157391,     -1.383164,     -0.888411,      0.253840,     -1.166464,
		     -0.375017,      0.242406,      0.043758,      2.604559,     -1.062517,
		     -1.762035,     -0.774460,      0.000000,      0.200258,      0.836454,
		     -0.668392,      0.025870,      0.047096,      1.394767,      0.526148,
		     -1.755537,     -1.480742,     -0.554053,     -0.056549,     -1.764538,
		      0.648042,      1.116595,      3.031287,      1.138715,      1.029239,
		     -0.155160,     -0.028360,      1.316024,      1.229812,      1.067632,
		     -0.477932,     -0.237961,     -1.148296,     -0.876928,      1.041504,
		      1.697991,      1.029917,      0.591316,      0.970063,     -0.018342,
		     -0.913528,     -0.438657,     -1.461585,      0.425153,     -0.223193,
		      0.509740,      1.404176,     -0.547943,     -0.233338,     -0.088849,
		     -1.199110,      0.360341,      0.840362,      0.852164,     -0.470394,
		      0.000000,      0.154855,     -0.162250,     -1.897843,      0.850253,
		      0.592237,     -1.523063,     -1.783209,     -0.208677,      0.597901,
		      3.453864,      0.547105,      0.995917,      0.826860,     -1.337647,
		     -2.308315,     -1.036565,     -0.598232,     -0.572792,     -1.231810,
		     -1.902226,     -1.017506,      0.956226,     -1.071535,     -1.112765,
		     -0.314333,      0.810589,      0.564600,     -0.745800,     -1.840551,
		      0.310513,     -1.012882,     -1.602153,     -0.636446,      1.441507,
		      1.438836,      0.393325,     -0.575003,     -0.858449,     -0.626627,
		      0.756333,     -0.148489,      0.494403,      0.197520,     -0.389671,
		      1.999620,      0.509570,     -0.303248,     -0.433861,      0.000000,
		      0.138460,      0.009711,      0.634675,     -0.963328,     -1.559829,
		      0.100448,      2.940823,      1.482877,     -1.416385,     -2.082284,
		      0.439735,      1.124358,     -0.943733,      0.858077,      0.500351,
		      1.250492,     -0.337770,      0.160206,     -0.098449,      0.269964,
		      1.256629,      1.749708,      2.717355,      0.533419,      0.451469,
		      0.310107,     -0.246908,     -0.219769,      0.874037,      0.308481,
		      0.557060,      0.144859,      0.223780,     -0.236681,      0.381551,
		     -0.818059,      1.028600,     -1.118043,     -1.412046,     -0.221981,
		     -0.590685,     -0.116983,      2.270013,     -0.275407,     -0.283604,
		     -0.654276,      1.768393,     -0.321709,     -1.458051,      0.000000,
		      0.630507,     -0.384580,      0.228128,     -0.363138,      1.490024,
		      0.533768,     -1.970313,     -1.149266,      0.625807,     -0.367172,
		     -0.085384,     -0.805163,      0.686642,      0.404763,      1.117904,
		     -1.708438,     -0.728980,      1.851154,      2.036617,     -0.373386,
		     -2.018792,     -3.939153,     -1.879636,     -0.164408,     -1.354610,
		     -0.254949,     -0.605867,     -0.953448,     -0.026580,     -0.105957,
		      0.328511,     -0.421181,     -1.655164,      0.091786,     -0.143155,
		      1.146483,     -0.660014,      1.185473,     -0.586481,      0.533019,
		      1.965537,      0.789478,     -0.428162,      1.470265,      0.431719,
		      0.238541,     -0.556370,      1.365786,      0.194398,      0.991137,
		      0.000000,     -0.449466,      0.487959,      0.160127,      0.260998,
		     -1.465432,     -1.526620,      0.216479,      1.422627,     -0.252462,
		     -0.561906,     -0.561589,      0.675046,      1.211480,     -0.817770,
		     -0.711786,      1.335028,      1.294954,     -0.425167,     -1.884195,
		     -0.566226,      1.426182,      1.585198,      1.127060,      2.260921,
		     -0.265938,     -0.997853,      0.519076,      0.711831,      1.532741,
		      0.098917,      0.441173,      1.696924,      1.355597,     -0.408529,
		      0.408696,      0.906268,      0.353261,      1.421616,     -1.015240,
		     -2.250843,     -1.198819,     -2.533366,     -1.362206,      0.194151,
		      0.862776,     -0.490671,     -0.093563,     -1.636111,     -0.344434,
		      0.764527,      0.631568,      0.000000,      1.197240,      0.406822,
		     -2.576489,     -0.169771,      0.876828,      1.976296,      0.665021,
		     -2.135041,     -1.156138,      1.260986,      1.195941,     -0.224346,
		     -1.432638,     -0.538652,     -0.626926,      2.675228,     -1.131397,
		     -2.889255,     -0.182241,      2.681694,      0.490393,      0.095017,
		     -1.477687,     -3.316636,     -0.823468,     -0.427099,      0.835312,
		      0.387208,     -2.204090,     -1.643436,     -0.396022,     -0.610449,
		     -0.444822,     -1.251617,      0.784823,     -1.162266,      0.021389,
		     -2.607236,     -0.138620,     -0.404201,      0.907229,      2.855889,
		      2.043122,      0.066812,      1.619625,      1.340897,      0.611685,
		      0.860730,      0.025805,     -0.882236,     -1.476002,      1.371213,
		      0.000000,     -0.476127,     -0.879351,      1.656543,      0.466550,
		      0.011845,      0.100359,     -0.286748,      1.338124,      1.926570,
		     -1.128632,     -2.091678,     -1.275913,      1.011624,      2.467596,
		      1.930310,     -2.757957,     -0.877634,      2.465496,      2.053450,
		     -1.075239,     -1.748459,     -0.066960,      1.505915,      3.063571,
		      4.410400,      2.482651,     -1.624557,     -1.166873,      0.071142,
		      1.836086,      1.127529,      0.525420,      1.029568,      1.139660,
		     -0.652722,      0.215441,      1.016919,      1.338273,      0.452535,
		      2.298410,      0.375433,     -0.166762,     -1.386362,     -2.066840,
		     -2.448117,     -0.421624,      0.695509,      0.796625,     -0.555393,
		     -1.011601,     -1.468142,      0.229988,      0.677676,      0.000000,
		      0.273506,      1.870366,      0.674810,     -2.301330,     -0.518921,
		     -0.078480,      0.036150,      0.260323,     -1.312996,     -0.430602,
		      0.755550,      2.330507,      0.135770,     -2.835086,     -1.831300,
		     -0.931289,      3.366407,      1.296979,     -2.533384,     -1.909024,
		      1.830620,     -0.082410,     -0.621832,     -1.190249,     -4.146216,
		     -2.534149,     -0.532766,      1.278953,      2.374585,     -0.754429,
		     -1.121327,      0.233872,     -1.532383,      0.154867,     -0.941474,
		     -0.004565,     -1.298557,      0.736948,     -0.633770,     -1.675615,
		     -2.526738,     -1.344021,      0.475248,      1.506129,     -0.009211,
		      0.550678,      1.358455,      0.737740,      0.646976,      2.421620,
		      0.459639,      0.095742,     -0.855142,     -1.028131,      0.000000,
		     -0.189213,     -1.170993,     -1.641980,      0.908331,     -0.721021,
		     -0.796668,     -0.142201,     -0.084984,      0.698371,      1.943923,
		      0.793633,     -2.270017,     -2.345591,      0.891477,      2.169623,
		      3.811175,     -2.784436,     -3.695807,      0.402848,      2.560278,
		      0.113554,      0.375978,     -1.114179,     -0.882379,      1.297482,
		      3.525749,      3.509334,     -0.012872,     -0.830519,      0.051973,
		      0.745143,     -1.456788,      0.778934,      0.873684,      0.544574,
		     -1.319602,     -1.110050,     -0.079571,     -0.350919,     -1.519647,
		      2.373103,      2.293782,      0.227630,     -1.138388,      0.148183,
		     -0.569484,     -1.972253,     -1.031595,      0.833787,      1.062511,
		     -0.764204,     -1.576432,     -1.246516,      1.240156,     -1.272782,
		      0.000000,     -0.312672,     -0.663630,      0.122274,      0.610435,
		      0.959277,      1.022833,      0.237317,     -0.685326,      0.992405,
		     -1.617495,     -1.129932,      1.127638,      3.206330,      0.407511,
		     -3.728439,     -2.849171,     -0.532964,      3.420536,      2.924883,
		     -0.359982,     -2.299251,     -0.155169,      0.388346,      0.212586,
		      1.516924,     -2.585877,     -3.915085,     -1.624216,     -0.376315,
		      0.843651,     -0.428884,      0.614484,      1.897670,     -0.957784,
		     -0.477411,      0.373058,      1.094250,     -0.678248,     -0.224898,
		      0.100038,     -0.574330,     -0.870116,     -2.061879,     -1.114822,
		     -0.770656,     -1.124627,     -0.292037,      0.415208,     -0.709520,
		     -0.281535,      1.575596,      2.069641,      0.947287,      0.483796,
		      1.223490,     -0.571170,      0.000000,      0.982080,      0.120167,
		      1.187593,      0.041101,      0.254223,     -0.829276,     -0.318093,
		     -0.385570,     -1.443509,     -0.908209,      1.851746,      1.270527,
		     -2.615883,     -2.268898,      2.625153,      2.495559,      3.109306,
		     -1.447815,     -3.289414,     -0.358195,      1.805789,      1.398211,
		      2.211500,     -0.645734,     -2.052787,      0.809322,      1.853514,
		      2.647763,     -0.073947,     -0.818043,      1.572899,      1.852255,
		     -2.365138,     -0.051546,      0.534900,      0.321101,     -0.612881,
		     -0.827747,      0.395891,      0.315893,     -2.456548,     -0.985669,
		      0.902913,      1.290714,     -0.803052,      0.272669,      0.052197,
		     -0.016097,     -0.136771,     -0.836710,     -0.775105,      0.466076,
		     -2.000064,     -2.441267,     -0.236755,     -0.292204,      0.686688,
		      0.000000,      0.109617,      1.286117,     -1.483381,     -1.476803,
		     -0.962205,      1.756374,      0.279796,     -0.279172,      0.575811,
		      1.653088,     -1.907862,     -2.108950,      1.046507,      3.444155,
		      0.227970,     -3.727331,     -3.167514,     -0.142012,      2.035346,
		      1.891848,      0.795328,     -1.476304,     -2.089472,     -1.178290,
		     -1.469954,      0.258902,     -0.244952,     -1.892799,     -0.723370,
		     -0.179961,     -0.360524,     -2.593465,      0.163370,      2.194566,
		      0.297891,     -0.072944,     -0.264352,      0.823193,      0.366193,
		      0.134217,      1.151670,      0.510930,      0.898773,     -0.723770,
		      0.820861,      0.334705,     -0.349181,     -1.323536,     -0.830610,
		      1.181833,      0.449609,     -1.012609,      1.035359,      2.201795,
		      1.215224,      1.340903,     -0.926222,     -0.066293,      0.000000,
		     -1.051449,     -0.809681,     -0.605308,      1.242280,      1.500063,
		     -1.351436,     -0.386469,      1.582452,      0.357739,     -2.410544,
		     -0.928757,      2.032325,      0.745007,     -3.609077,     -2.687398,
		      3.556282,      2.425017,      1.038684,     -0.301508,     -2.268617,
		     -1.290891,      0.678987,      0.541956,      4.173947,      2.281882,
		     -0.929585,      1.396185,      1.037738,      1.040891,     -0.141299,
		     -2.001638,      0.567812,      2.782617,     -1.667371,      0.006875,
		     -0.801662,     -1.282958,     -1.393646,      0.273687,      0.042826,
		      1.091642,      0.280657,     -1.396556,     -2.260869,     -1.814049,
		     -0.123882,      1.572662,      1.093159,      0.509860,     -0.320817,
		      1.233946,      0.000885,     -0.157942,     -0.274916,     -1.451865,
		     -0.772773,      0.807280,     -0.400300,      1.707485,      0.000000,
		      0.370913,     -0.500789,      2.687623,     -0.375466,     -1.280942,
		     -1.004617,      1.103641,     -1.730376,     -0.892361,      1.934117,
		      3.278888,     -1.624680,     -2.578540,      1.868715,      2.956813,
		     -0.436523,     -2.697859,     -2.354556,      0.537631,      1.428009,
		      0.990154,      1.526197,      0.936200,     -2.254448,     -0.632845,
		     -0.856498,     -2.207282,     -0.635084,     -0.207992,      1.758189,
		      1.469316,      0.741776,     -3.726433,     -0.503407,      0.367256,
		      2.025270,      2.001317,      0.874737,      0.440871,     -0.046622,
		     -1.636204,     -0.110545,      0.690028,      1.318285,      1.067759,
		      0.673719,     -0.633034,      0.659579,      0.540912,     -0.376177,
		     -1.999005,      0.420845,     -1.398066,     -0.617537,      0.455365,
		      1.071254,      1.231227,     -0.797156,     -0.847019,     -0.241734,
		      0.000000,      0.249013,     -1.010059,     -2.079133,      0.356039,
		      0.704424,      1.824233,     -1.375814,      0.033359,      1.892161,
		     -0.585751,     -2.795876,     -1.189410,      2.008860,      0.948101,
		     -2.856320,     -2.761350,      2.721943,      2.958719,      0.058908,
		     -0.348347,     -0.677694,     -1.384607,     -1.422274,     -1.031205,
		      2.055235,      1.780487,     -0.547645,      0.933494,      0.528082,
		     -0.197605,      0.340440,     -0.217151,      0.620508,      2.099060,
		     -0.318583,     -0.342346,      0.206677,     -0.723891,     -2.198037,
		     -0.988060,     -1.162995,     -0.579666,      1.966263,      0.602035,
		     -1.588621,     -1.038039,     -0.660719,     -0.958132,     -0.232249,
		      0.996987,      1.280742,      0.905968,      1.058200,     -1.500668,
		     -0.145656,     -0.624213,     -0.600561,      0.727822,      1.690072,
		      0.637523,      1.033478,      0.000000,     -1.199654,      0.454940,
		     -0.101386,      1.501973,      0.356331,     -0.452016,      0.788068,
		      0.283826,     -2.059057,     -2.058636,      2.708228,      3.486612,
		     -1.404451,     -2.540805,      1.655212,      2.435564,     -1.256205,
		     -2.649033,     -1.517343,      1.235029,      0.661968,     -0.909447,
		      0.204947,      1.896805,     -1.470608,      0.489596,      2.010043,
		     -2.243292,     -1.780309,     -1.394481,      0.629773,      1.167975,
		      2.717937,     -1.744735,     -1.110550,     -2.600594,     -0.718647,
		      1.295385,      3.067341,      2.034133,      3.259271,     -0.520980,
		     -0.638779,     -1.632743,      0.368032,      1.369678,      0.510211,
		     -0.495479,      0.059589,     -0.460845,      0.652626,     -1.098346,
		     -1.379638,     -1.169850,     -1.355157,     -0.278288,     -0.187115,
		     -0.138122,     -1.144665,     -1.677762,     -0.586493,      0.188634,
		      0.000000,      0.224029,      0.923047,      1.015134,     -1.908400,
		     -0.116086,     -0.787870,      0.661725,     -1.215078,      1.339065,
		      2.738925,     -1.010514,     -3.456998,     -1.636065,      2.327511,
		      1.394887,     -1.409710,     -1.149608,      0.897719,      2.239365,
		     -0.688449,     -0.353828,      1.694964,      0.751721,     -0.804145,
		     -1.142907,      0.510111,      0.507475,      0.943184,      2.239146,
		      0.719798,     -1.963840,     -0.940335,     -1.859170,      0.504696,
		      0.611772,      1.409201,      0.633751,      0.381855,     -1.415242,
		     -2.437288,     -1.019463,     -1.469950,     -1.596119,      0.190601,
		      1.614198,     -2.149913,     -0.650638,      0.459308,     -0.296274,
		     -1.202126,     -1.593354,     -1.354749,      0.965521,      2.393948,
		     -0.762782,     -0.732847,      0.814564,      0.129563,      1.408520,
		     -0.064093,      0.991619,      1.096568,     -0.711288,      0.000000,
		      0.645648,     -1.500145,     -0.532808,     -0.167044,      1.312109,
		      0.273837,     -1.164144,      0.994721,      0.586549,     -2.345680,
		     -2.449699,      2.528649,      3.641403,     -0.748329,     -2.116482,
		      1.221871,      2.617005,      0.132270,     -1.320340,     -1.546463,
		      1.217437,      1.253241,     -0.155850,     -0.257403,      1.324537,
		     -1.805393,     -0.909835,      2.240541,     -1.448017,     -0.672386,
		      0.733052,      1.538266,      0.600074,      1.401075,     -0.015423,
		      1.101809,     -0.614983,     -1.380980,     -0.952688,      2.184524,
		      1.112803,      1.888327,      2.249452,     -0.297605,     -1.844930,
		     -0.913300,     -0.561317,      1.414027,     -0.096797,      0.259508,
		      1.948100,      2.111127,     -0.873941,     -2.649846,      0.479872,
		     -0.860819,     -0.931202,     -1.238115,     -0.519322,     -0.538270,
		     -1.741277,     -1.818963,      0.042773,     -0.336154,      0.000000,
		     -1.229783,     -0.975322,      0.616345,      1.938713,     -0.985239,
		     -1.459202,     -0.451706,     -0.488338,     -1.516541,      0.374767,
		      3.748757,      1.170143,     -2.832572,     -1.655979,      1.051427,
		      0.329985,     -0.963561,     -0.187483,      0.184970,      1.794770,
		     -1.641468,     -3.069055,      0.073708,      0.423021,      0.185595,
		      1.028923,      0.604257,     -2.008022,     -0.329081,      1.719054,
		      1.373443,     -2.149594,     -0.390284,     -1.689883,     -0.099157,
		     -2.371760,     -0.519736,      0.333900,      1.927435,      0.474101,
		     -1.510823,     -1.414847,      0.721766,     -0.892527,     -0.040671,
		      0.381476,     -0.788530,     -1.815854,     -0.482044,      0.453831,
		      0.162465,     -1.626295,     -2.863047,     -1.064072,      0.981137,
		      0.170741,      0.851109,      1.160233,     -1.121014,      0.722141,
		      1.430420,      1.634930,      0.127220,      0.465658,     -0.325925,
		      0.000000,     -0.571386,      1.383298,     -0.340250,     -1.345882,
		     -0.252271,      1.750372,      0.996170,     -1.680574,      1.469809,
		      1.195712,     -1.607203,     -3.463364,      0.383979,      2.285704,
		      0.174591,     -1.809806,     -0.415226,      0.815049,      0.000637,
		     -0.045647,      0.339129,      1.632159,      0.958146,      0.498394,
		      0.903034,      0.782476,     -0.929115,      0.246596,      1.844049,
		     -1.449888,     -1.734295,      0.465333,      1.141693,      0.913113,
		      0.568786,      0.187047,     -0.189864,      1.577265,     -1.184254,
		     -0.882696,      1.212945,      0.065784,     -0.909729,      1.400220,
		      1.513350,     -0.273248,      0.727182,      1.055205,      1.006181,
		      0.673768,     -0.555163,     -0.481154,      2.634497,      1.577256,
		     -0.912162,      0.824236,     -1.573574,      0.182934,      1.749179,
		      0.398584,     -0.906503,     -1.532900,     -0.586491,     -2.064550,
		     -1.866878,     -0.608821,      0.000000,      0.288053,     -0.611255,
		     -0.739473,      1.661545,      2.745810,     -0.781870,     -0.945542,
		      1.099065,      0.395211,     -2.620952,     -2.057936,      2.236203,
		      1.051084,     -1.949491,     -0.610545,      1.144240,      1.118016,
		     -1.044118,     -0.016505,      0.936743,      1.885308,     -0.528217,
		     -2.324885,     -0.941604,     -0.797024,     -1.917000,      0.257251,
		      0.685952,     -2.488927,     -0.402007,      0.598045,      1.489134,
		     -1.470017,     -0.665722,     -0.486754,      2.019377,     -0.049641,
		     -2.180775,     -1.630564,     -0.893144,      0.164257,     -0.283582,
		     -0.233001,      0.390957,      0.342186,     -0.674628,     -1.627440,
		     -1.032740,     -2.328702,     -1.448921,      0.468889,      1.688252,
		     -0.433885,     -0.552787,     -1.525267,     -0.660731,      0.209644,
		     -1.929211,      0.484096,     -0.505509,     -0.571324,      0.349655,
		      1.191729,      2.281679,      2.484487,      0.137988,      0.836339,
		      0.000000,     -0.053451,     -0.578955,      1.300158,     -0.409707,
		     -2.285882,     -1.834670,      1.470417,     -0.288687,     -1.811171,
		      0.858106,      3.151483,      0.974139,     -2.363988,     -0.011035,
		      1.142650,     -0.062320,     -1.182212,      0.077210,      0.364608,
		     -1.486326,     -2.478529,     -0.284097,      1.258509,      0.181416,
		      0.432884,      2.447243,      1.038675,     -0.055016,      1.932828,
		      2.724852,      0.128804,     -1.445653,      0.442362,     -0.547340,
		      0.145963,     -1.001835,      0.803900,     -0.353726,      1.210786,
		      1.253528,      0.497553,      2.565895,      1.547751,     -2.158494,
		      0.504514,      0.129215,      0.966253,      1.162600,      2.456797,
		      2.605489,      1.657174,     -1.886761,     -1.308469,     -0.364677,
		      2.323153,      1.124342,      1.414585,     -0.083218,     -1.213945,
		      1.699881,     -0.039883,      0.714349,     -0.792858,     -1.341174,
		      0.199013,      1.213580,     -2.148578,     -0.757375,      0.000000,
		     -0.255750,     -0.171373,     -0.764876,     -0.330682,      1.489647,
		      0.950094,     -2.972297,     -0.960264,      2.574079,      0.774229,
		     -1.665828,     -2.206090,      0.933231,      1.516738,     -0.935428,
		     -0.877512,      0.554425,      1.316047,     -1.007306,      0.247212,
		      1.649891,      0.996695,     -0.527270,     -1.417623,      1.084450,
		      0.198898,     -1.456457,     -0.470807,     -0.063441,     -3.323956,
		     -0.851993,     -0.002626,      0.605746,     -0.581529,     -1.412097,
		     -1.442345,      0.336865,      2.544502,      0.171401,     -0.777845,
		     -1.966930,     -2.700607,     -1.824723,      0.372556,      0.610636,
		      0.527951,     -0.386489,     -0.992746,     -0.948500,     -1.918674,
		     -2.364817,     -1.426430,      0.515157,      1.079127,      0.163334,
		     -0.337504,     -2.394410,      0.384465,      0.146323,     -1.196208,
		     -1.240556,     -1.030098,      0.610578,      1.822963,      0.989778,
		      0.905964,      0.880524,      2.016113,     -0.115727,      0.000000,
		     -0.999335,      0.221398,      0.830316,      1.151633,     -0.330838,
		     -1.471441,      0.842899,      1.491494,     -0.657458,     -2.094905,
		     -0.416570,      1.707719,      0.345258,     -1.592524,      0.424644,
		      0.100131,     -0.760330,     -1.468000,      0.609683,      1.871754,
		     -0.607063,     -2.163952,      0.337363,      2.387109,     -0.103363,
		     -2.262482,      1.088313,      0.581595,     -1.179788,      2.152115,
		      1.883615,      1.771909,      0.507460,      2.026597,      0.965892,
		      0.675700,     -1.301231,     -0.603572,     -1.053191,     -0.233974,
		      0.543183,      1.515896,      2.318417,      1.686831,     -0.090434,
		      0.655177,     -0.033303,      0.020693,      0.055900,      1.356567,
		      2.433461,      2.747828,      0.201294,     -1.693991,     -1.122727,
		      1.100881,      1.441899,      0.415153,      0.855547,     -0.335658,
		      3.226604,      0.014261,     -1.359738,     -0.035750,     -0.986496,
		     -0.768497,      1.031309,      1.212074,      1.896087,      0.652335,
		      0.000000,      0.624074,      0.081136,     -0.711778,     -0.574176,
		      1.036040,      1.887582,      1.051162,     -2.335427,     -0.261831,
		      2.089250,      1.194026,     -0.496849,     -1.915360,     -1.019519,
		     -0.394588,      0.468852,      0.582281,      0.646436,      0.517939,
		     -1.656586,     -0.213164,      2.701173,      1.037923,     -1.799974,
		     -2.439931,      0.727593,      1.307915,      0.801264,      0.612181,
		     -0.045984,     -2.773286,     -1.594993,     -0.118120,     -0.981993,
		     -0.023097,      0.489246,     -0.429735,     -1.239656,      1.494556,
		      1.001842,      1.517094,     -0.318020,     -3.226473,     -2.396870,
		     -0.302186,     -1.016823,      0.402412,      0.742974,     -0.220406,
		     -0.976331,     -0.344008,     -2.586997,     -1.795694,     -1.162955,
		      1.324237,      0.255333,     -0.606355,     -0.041282,      0.231063,
		     -0.603945,     -2.494033,      0.118682,      0.137392,      0.664043,
		      0.465618,     -1.221418,     -0.694674,     -1.009303,     -2.442732,
		      0.781531,      0.324119,      0.000000,     -0.696549,     -0.843878,
		      0.173578,      1.542459,     -0.540844,     -1.030997,     -1.519891,
		      0.522008,     -0.017105,     -1.361327,      0.222595,      0.492451,
		      0.785534,      0.567591,      0.143172,     -0.082154,      0.731733,
		      0.227459,     -1.164786,      0.087162,      0.477379,     -1.550079,
		     -2.866389,      0.023092,      3.117025,      1.161879,     -2.532535,
		     -0.618424,      0.657487,     -0.872915,      2.542818,      0.964030,
		     -0.032975,     -0.423947,      1.005592,      0.954573,      1.787279,
		      0.324796,      0.576216,     -0.263113,     -1.713831,     -0.345634,
		      1.313711,      2.759119,     -0.206029,     -0.556125,      1.142173,
		      0.402618,     -0.016487,     -0.419180,     -1.110323,      0.158242,
		      2.874640,      2.835350,      0.495687,      0.445776,     -0.802923,
		      0.556629,     -1.553959,     -0.126603,      0.576095,      3.537105,
		      2.244192,     -0.459494,      0.182236,     -1.159907,     -2.156116,
		      0.329719,      1.017418,      1.274062,      1.779365,      0.338451,
		      0.000000,      0.374664,      0.772316,     -0.037757,     -0.729197,
		     -0.470560,      0.136988,      0.377147,     -0.446947,     -0.539030,
		     -0.493782,     -0.326977,      0.018064,      0.723002,      0.360710,
		     -1.100686,     -2.619196,     -0.796871,      0.736249,      0.530070,
		      1.398834,     -0.812939,     -1.434595,      2.362205,      2.805902,
		     -0.030994,     -1.585952,      0.063290,      0.046729,      0.457770,
		      0.372159,     -0.757067,     -1.245322,     -0.680819,      0.801805,
		     -1.665074,     -1.973691,     -1.138417,      0.676666,     -1.571299,
		      0.693098,      1.209257,      2.160945,      1.545349,     -2.181166,
		     -1.001353,      0.683874,     -1.839034,     -1.993403,      0.709885,
		      1.307061,      0.075596,     -0.949420,     -1.565821,     -3.437073,
		     -2.788481,     -0.852335,      0.479913,      0.269469,      0.996603,
		      1.392795,      2.163472,     -4.629340,     -2.915538,     -0.077949,
		      0.832718,      1.256602,     -0.278311,     -2.013785,     -1.451741,
		     -2.896013,     -3.310669,     -0.854203,     -0.467110,      0.000000,
		      0.533635,     -0.945459,     -0.099050,      1.079347,      0.893410,
		     -2.287643,     -0.537372,      0.605720,      1.915573,     -1.146510,
		     -2.080731,      0.154344,     -1.182003,     -1.992725,      0.671578,
		      2.145642,     -0.840817,     -0.640047,      0.502690,     -0.497291,
		      1.226364,      1.945977,     -0.582530,     -3.268406,     -1.439152,
		      0.804235,      1.407958,     -0.346275,     -1.610995,     -0.797254,
		     -1.552564,      2.864608,      1.749512,     -0.748559,      0.141177,
		      0.844874,      0.023407,      0.405914,     -0.030058,     -0.488410,
		      0.725123,     -2.001223,     -3.110468,      0.618671,      2.076715,
		      0.419920,      0.065180,      0.656949,      0.749692,     -1.563614,
		      0.785287,     -0.858473,      0.983946,      0.265314,      1.145463,
		      0.916975,      1.290302,     -0.328514,      0.594388,     -1.465057,
		     -1.510117,      0.379384,      1.817701,      2.374553,     -1.069046,
		     -1.822141,      1.579106,      0.078088,      0.538421,     -1.585096,
		     -0.305139,     -1.085997,      0.007004,      0.032176,      0.000000,
		     -0.914725,      0.678764,      0.171035,     -0.420090,      0.371216,
		      1.342937,      0.475598,      0.137658,     -0.518604,      1.127017,
		      2.034227,     -1.134725,     -1.534102,      1.325666,      1.764285,
		     -0.457027,     -0.724768,     -0.225532,     -1.149991,     -1.232380,
		      0.527603,      0.543456,     -1.502755,      0.074030,      1.367821,
		      0.189711,      0.373869,      0.531537,      0.416619,      2.261577,
		      1.490969,     -1.574568,     -1.651007,     -0.439032,      2.681388,
		      1.169813,      0.574775,     -1.989835,      0.364922,     -1.438904,
		     -1.473220,      0.855926,      2.555389,      1.905381,     -1.697196,
		     -1.664879,      0.855254,      0.454471,     -1.187481,     -1.358387,
		      0.226139,     -0.449016,      0.240173,      0.119454,      0.280615,
		     -2.329898,     -2.737681,     -0.533038,     -1.706133,      1.182067,
		      0.692128,      1.985971,     -0.846955,     -2.222619,     -1.317130,
		      0.939372,      0.788400,     -0.679963,     -0.689443,      2.548568,
		      0.232975,     -0.264943,     -0.421531,     -0.747408,     -0.217302,
		      0.000000,      0.404939,      0.129900,     -1.288400,      0.769660,
		      1.118821,      0.570190,     -0.704837,     -1.295900,     -0.217593,
		      0.178930,      0.503163,      0.261423,      0.627090,     -1.094158,
		     -4.068319,     -1.349468,      2.124166,      0.349093,      0.629174,
		      0.723159,     -1.876052,     -1.271101,      1.779312,      2.371049,
		     -0.435618,     -0.778749,     -1.282382,     -0.755107,      0.435749,
		     -0.902014,     -0.542417,     -1.460713,      1.111807,      1.562680,
		     -1.800026,     -1.114966,     -0.357468,      1.355610,      1.451922,
		      1.435349,     -0.919686,      1.025445,     -0.578615,     -2.788652,
		     -1.360728,      0.886545,     -0.391365,     -0.002565,      0.287398,
		      1.522649,     -1.132170,     -1.115438,     -0.573154,     -0.699876,
		     -0.436068,      0.031418,     -0.470995,     -0.112106,      1.803117,
		      0.001123,      1.085834,     -1.437169,      0.223726,     -1.784039,
		      0.300039,     -0.675119,     -3.031697,     -0.181414,      0.336946,
		     -0.262202,      0.944427,     -0.161468,     -1.117770,     -0.994842,
		     -0.995625,      1.278520,      0.000000,      1.302617,     -0.453068,
		      0.275756,     -0.297512,     -2.125078,     -0.075119,      0.336139,
		     -1.211376,      0.005620,     -0.933875,     -0.894588,      0.415707,
		     -1.544268,     -1.390731,      0.996878,      1.375834,     -0.227915,
		      0.864172,      1.344304,     -0.741054,     -0.083328,      0.619509,
		      0.883120,     -0.634424,     -1.372654,     -0.089447,     -0.180037,
		      0.079006,      0.307046,     -1.392645,      1.091342,      2.125387,
		     -0.769687,     -1.135104,     -0.949143,      0.318314,      0.584554,
		      2.459386,     -0.287898,      0.014947,     -0.252683,     -2.186101,
		     -0.316994,      1.981906,      3.127921,      0.726196,     -1.450112,
		     -1.452129,      0.767974,      0.394728,      0.927074,      0.374088,
		      0.284883,     -1.334942,      0.651204,      2.873771,      1.360000,
		     -0.877351,     -1.888108,     -2.378636,     -0.409617,     -0.688533,
		      0.846814,      1.577669,     -1.323618,     -0.061876,      0.382473,
		      0.078856,      1.512833,      0.487327,      0.982457,      1.818660,
		      1.059084,      0.635271,      0.859972,     -0.076298,      0.804078,
		      0.000000,      0.054928,      0.580926,      0.536298,      0.261770,
		      0.391127,     -1.156281,     -0.088330,      1.970758,      0.639463,
		     -1.016546,     -1.365894,      0.127949,      1.362980,      1.921212,
		      0.520991,     -2.615701,     -1.362163,     -0.630967,     -1.517521,
		      1.517327,      2.669746,     -0.488209,     -2.720592,     -1.004936,
		      1.730950,      1.096461,      1.199928,     -0.025144,     -1.116585,
		      0.913957,     -0.926343,     -0.445527,     -0.172156,      0.905121,
		      1.442247,     -0.106100,     -0.434103,     -3.596395,     -1.501958,
		     -0.479684,      2.023665,     -0.139782,      0.169683,     -0.940564,
		     -2.097172,     -1.680042,      0.575883,     -0.036077,      0.482851,
		     -0.531633,      0.761156,     -0.526882,     -0.048987,     -0.543075,
		      0.337040,     -2.696454,     -0.719484,     -0.128291,     -0.427613,
		      2.321953,     -0.634787,      1.195600,     -0.187769,     -0.850830,
		     -1.441013,     -1.004997,     -0.317851,     -0.780533,     -2.729675,
		     -0.891590,     -1.023230,     -0.268349,     -0.508636,     -0.760547,
		      0.119547,     -0.880069,      0.801268,      1.153354,      0.000000,
		     -0.273034,      0.781106,     -1.092569,      0.206016,     -0.061530,
		     -0.441068,      1.858148,     -0.025819,     -0.953897,      1.038886,
		      1.459235,     -0.814123,     -1.345719,     -1.194767,     -0.858944,
		     -0.095936,      0.400017,     -0.628915,      0.686699,      0.013211,
		     -3.016704,     -0.027613,      1.557130,      1.500578,      0.864190,
		     -0.841409,     -0.867511,      0.395344,      0.650405,      0.499734,
		     -0.555161,     -0.079054,     -0.366987,     -1.464833,     -0.211362,
		      0.921848,      0.169942,      0.773978,      2.205403,      1.971627,
		      0.073693,      1.840889,     -1.311616,     -0.403625,      0.827628,
		      1.644764,      1.849456,      1.096700,     -2.424475,     -0.907251,
		     -0.447721,      0.857502,      2.302672,      0.138487,     -1.948187,
		     -0.111743,      2.681177,      2.033816,      1.309992,     -0.557867,
		      1.803449,     -0.348605,     -1.734356,     -1.654187,      1.528103,
		      0.318231,     -1.526249,      1.118465,      1.032358,      0.863599,
		      2.207874,      2.051718,      0.200444,      1.290486,     -0.467246,
		      0.842072,     -0.428238,     -0.237523,      0.760399,      0.000000,
		      1.389451,     -1.019049,     -1.961621,      0.626921,      0.577302,
		      1.632762,     -2.774089,     -2.313748,      2.250352,      0.418019,
		     -0.906263,     -1.750808,     -0.600710,      0.313009,     -0.323204,
		     -0.185749,      0.176958,      0.716660,     -1.118338,     -2.533466,
		      1.493154,      2.574238,      1.463730,     -1.464424,     -2.287277,
		      0.172767,      0.521462,     -0.011526,     -0.243087,     -1.223033,
		      1.762988,     -1.227446,     -1.739648,      0.098898,     -0.051714,
		      0.110250,     -0.314352,      1.561771,     -1.533823,     -2.296723,
		     -2.215088,     -0.331377,      0.428812,      0.427388,     -0.294422,
		     -1.821979,     -2.427713,      0.366266,      1.299284,      1.515501,
		      0.685934,     -0.057779,     -0.990283,      0.160826,      0.366454,
		      0.526329,     -0.982311,     -2.715228,     -2.525545,     -0.981008,
		     -0.099740,     -1.155625,      1.110158,      0.904636,     -0.616201,
		     -1.634709,     -1.449610,     -0.342663,      1.069244,     -1.999139,
		     -1.934854,     -1.882109,     -1.567118,     -0.196911,     -0.099504,
		      0.309942,     -0.510413,      0.526116,     -0.047599,     -0.840229,
		      0.000000,     -0.593482,      1.553141,      1.388299,     -0.088199,
		     -0.620708,     -0.971579,      0.098397,      1.431859,     -0.153238,
		     -1.439628,      0.365755,      1.071999,     -0.439676,     -1.713039,
		     -1.497372,      0.632931,      1.182784,      0.092568,      1.034405,
		      2.458600,      0.939201,     -2.614233,     -2.227682,      0.638751,
		      0.781219,      0.588138,     -0.448147,     -1.083620,     -0.240732,
		      0.471216,     -0.241160,      0.783689,      1.775365,      1.028279,
		     -1.579386,     -1.521612,      0.885712,     -0.806705,     -0.993920,
		      0.651112,      1.362877,     -0.249227,      2.103095,      0.725098,
		      1.186111,      0.227825,      2.234979,      0.124629,      1.955526,
		     -1.069810,     -0.129598,     -1.740111,      0.223578,      0.339764,
		      1.688514,     -0.842101,      0.680181,      1.547267,      1.460295,
		      1.232768,      0.305393,      0.719630,      1.482379,      1.147148,
		     -1.632537,     -0.275842,      1.708329,      0.306455,     -1.245893,
		      0.487794,      0.484011,      0.274122,      2.933826,      0.735495,
		     -1.364606,     -1.742858,     -0.604747,      0.480319,      0.254891,
		      0.099256,      0.321801,      0.000000,      1.330323,      2.272015,
		     -1.300172,     -1.671560,     -0.611543,      0.205299,      3.577287,
		     -0.653735,     -1.114968,      0.620025,     -0.445979,     -0.666732,
		     -0.075816,      1.109023,      0.696760,     -1.745484,     -1.669187,
		     -0.001117,      0.516745,     -1.280719,     -2.698726,     -0.085212,
		      1.210610,      1.130202,      0.230042,     -1.002958,      0.132164,
		      1.776161,      1.053619,      0.599276,     -1.752208,      0.779459,
		     -0.740745,     -2.445008,     -0.232977,      0.515002,     -0.025525,
		     -0.092220,     -0.250194,      0.488552,      0.078684,     -0.017232,
		     -1.949535,     -1.027192,     -0.666340,      0.442931,     -2.326602,
		     -1.316603,     -2.105739,      0.602459,      0.908108,      2.068706,
		      0.619494,     -0.544725,      0.680763,     -0.451139,      0.630448,
		      0.990153,     -0.319940,     -1.796984,     -2.361335,     -0.337759,
		     -0.800171,     -0.975763,      0.821200,      1.188740,     -0.191230,
		     -1.516618,     -0.726537,      1.690172,      0.007041,     -2.121654,
		     -0.517550,     -0.600463,     -0.293140,      1.363104,      0.411767,
		      0.152184,      0.391930,      0.144988,     -0.809648,     -1.747818,
		      0.000000,      0.408809,     -2.198562,     -0.333824,      0.603927,
		     -0.520557,      1.327617,     -1.405584,      0.003238,      0.578850,
		      0.285677,      0.105283,      0.306177,      0.624157,     -0.399612,
		     -1.913920,     -2.850208,      0.925828,      0.006624,     -1.422532,
		      0.241179,      2.887115,      1.462545,     -0.079094,     -1.244335,
		      0.451485,      0.038375,      0.024510,     -0.431686,     -1.093873,
		     -0.445815,      0.193085,     -2.152972,     -1.244721,      2.240351,
		      2.309255,     -0.007072,     -1.279471,      1.391378,     -0.080386,
		     -0.447281,     -0.706471,     -0.506646,     -1.500994,      1.208564,
		      0.668035,     -0.480389,      0.841104,      1.629587,      0.833375,
		      0.983072,     -1.129924,     -0.316786,     -0.924941,      0.141201,
		      0.238167,     -0.231764,     -0.236098,      0.350661,      0.098442,
		      0.448637,      0.650037,      0.045462,     -0.662916,     -0.334995,
		      1.564787,     -0.723290,     -0.642481,      1.895752,      1.677575,
		     -0.001890,      0.914240,     -0.341526,     -0.276281,     -0.430933,
		      0.972122,     -0.962994,      0.703970,      0.723067,     -0.363482,
		      0.785621,      1.486630,      0.894640,      0.180993,      0.000000,
		      0.135882,      1.815170,      0.547176,      0.396170,      0.104045,
		     -0.914021,     -1.366268,      1.719072,      0.811210,     -0.221449,
		      0.470473,     -2.202433,     -1.712061,     -0.943235,      1.097852,
		      1.590398,      0.226202,     -1.099094,      0.607062,      1.097087,
		     -0.605477,     -1.398135,     -0.125443,      0.559968,     -1.072562,
		      0.422646,     -0.057636,     -0.564035,      0.217128,      0.266300,
		      0.993083,      0.234704,      1.199756,     -0.187468,     -2.248732,
		     -2.021334,      0.163948,      0.575441,      1.306971,     -0.471106,
		     -0.338832,     -0.260493,      2.257305,      0.564180,     -0.215526,
		     -1.377389,      0.886434,     -2.347222,     -0.062381,     -1.647094,
		      0.019184,     -0.244221,      1.673783,     -1.219500,      0.303213,
		      1.566532,     -0.098241,     -1.815402,     -1.088039,      0.190801,
		      0.558660,     -1.252788,      0.763121,      0.112492,     -0.714369,
		      0.078108,     -0.398044,      0.158149,     -0.830174,     -1.021395,
		     -0.141145,     -0.114460,     -0.819986,     -1.442199,     -0.026873,
		      0.955223,      1.705841,      0.428382,     -0.270973,     -0.707628,
		      1.549560,     -0.379392,      0.985803,      0.920324,      0.000000,
		      0.851296,      1.648901,     -1.486235,     -0.045854,     -0.571488,
		      0.719858,      3.411067,     -0.790415,     -0.450630,     -0.003344,
		     -0.542203,     -1.202389,      0.297622,      1.537954,     -0.440550,
		     -1.679440,     -0.692810,      1.482714,      0.535369,     -1.644323,
		     -0.254402,      1.215087,      0.002952,     -0.221323,      0.195491,
		      1.457582,      0.293420,     -0.144411,     -0.509655,      0.206005,
		     -0.201614,      0.425948,     -1.435955,     -1.413508,      1.325611,
		      2.774179,      0.231874,     -1.717997,     -0.660872,     -0.218335,
		      0.836297,      0.823218,     -1.042578,     -1.828892,     -0.231332,
		      1.593828,     -0.807126,      0.355459,     -0.260309,      0.581447,
		      0.035146,     -0.541052,     -2.055629,     -1.473744,      1.011119,
		      0.411144,      0.965831,      0.245761,      1.163873,     -0.207229,
		     -1.471933,      0.092823,      0.183135,      0.023253,     -0.025234,
		      0.190527,     -0.021969,     -1.477764,      0.494572,      1.252937,
		     -0.420646,      1.685402,      0.997153,      0.528780,     -1.327229,
		     -1.214663,     -0.948110,     -0.041832,      0.436661,     -0.144565,
		     -0.251485,     -0.771486,      0.243393,     -0.097820,     -0.456166,
		      0.000000,     -0.545915,      0.206565,      0.069931,     -0.753377,
		      0.396553,      1.297118,     -0.942137,     -0.587127,      1.313540,
		      0.472830,      0.125650,      1.218352,      0.570210,     -1.242140,
		     -3.039812,     -2.442547,      1.500022,     -1.084370,     -1.612095,
		      0.830064,      0.687462,     -0.952538,     -0.086436,      0.200344,
		      0.437627,     -2.120264,     -0.796576,      1.788547,      1.621235,
		      0.439925,     -0.844991,     -0.748696,      0.410765,      2.202864,
		      0.068205,     -0.906194,     -1.568166,     -1.139162,     -1.193064,
		      1.305863,     -0.569589,     -0.575529,     -0.847157,      0.651678,
		      0.053671,     -0.045706,     -0.318051,      0.774263,     -1.304858,
		      0.679617,      0.257811,      0.098677,      1.210896,      2.717623,
		     -0.862442,     -1.163223,      0.875685,      1.099314,     -0.705216,
		     -0.754310,     -1.430786,     -0.655084,     -0.054226,     -0.572400,
		      1.545410,      0.052183,     -1.257335,      0.272830,      0.148823,
		     -0.876479,     -0.443571,     -0.234795,     -0.571411,     -1.042161,
		      0.989475,      1.174758,     -0.375714,     -0.346725,      0.456047,
		     -0.044083,     -0.604422,      1.535778,     -0.071881,      1.038126,
		      0.910828,      0.454549,      0.000000,      0.085532,      0.491592,
		      0.438376,     -1.005865,     -1.792583,     -0.676999,      1.314774,
		      2.411389,     -1.359544,     -0.073449,      1.426664,     -0.702728,
		     -1.279113,     -1.144715,      1.815798,     -0.507411,     -1.461991,
		     -0.787718,      1.377522,      1.567945,     -1.133765,      0.055707,
		      1.379877,     -0.235305,     -0.795054,      0.536075,      1.626930,
		      0.109455,     -1.622040,     -1.355305,      0.345774,      0.893395,
		      1.496070,     -1.482637,     -1.773860,      0.406815,      3.269345,
		      1.237848,      0.228173,     -0.461050,      0.018295,     -0.364196,
		      1.430408,      1.192374,     -0.078948,     -0.850136,      0.761685,
		     -1.480551,      2.508429,      0.292598,      0.700945,     -0.565444,
		      0.203755,     -1.994431,     -1.498136,     -0.349278,     -0.930987,
		     -0.430758,     -1.037203,      0.907646,      2.149928,      0.017430,
		     -0.041406,     -0.679280,     -0.282580,     -0.365906,      1.178057,
		      1.852313,     -0.897001,     -0.430009,      1.267952,     -0.309080,
		      0.155927,      1.102019,      0.186251,     -0.932510,      0.227402,
		      0.728725,      0.167029,      1.074246,     -0.552715,     -2.147735,
		     -1.383279,     -1.378395,     -0.478806,      0.874326,     -0.606151,
		      0.000000,      0.821340,     -0.011208,     -0.384034,     -0.204451,
		      0.774410,      2.351711,     -0.337809,     -2.290912,     -0.246293,
		      1.146707,     -1.279075,     -3.028330,      1.263943,      3.088772,
		     -0.219474,     -1.758044,     -0.201778,      0.939153,     -1.473841,
		     -1.268540,      0.917967,     -0.022477,     -2.495831,     -0.255752,
		      0.910657,      0.309943,     -2.200133,     -1.596534,      0.279026,
		      1.986815,      0.828341,     -0.470793,     -1.487561,     -0.630554,
		      2.033410,     -0.140089,     -1.066090,     -1.241788,      0.120761,
		     -0.524265,      0.185840,     -0.446442,     -0.627784,     -0.162886,
		      0.319467,     -1.343202,     -0.610842,      1.367130,     -0.295717,
		     -1.838482,     -1.427251,      1.012480,      0.089923,      0.836646,
		      2.527978,      0.952490,     -0.256302,      0.138348,      1.535473,
		     -0.122799,      0.718048,      0.162182,      1.154035,      0.055726,
		     -1.327275,      0.106402,     -0.949441,     -1.434014,      0.738308,
		      1.141249,     -0.826163,      0.060875,      0.930393,      0.416146,
		      0.099089,      0.814068,      0.473379,     -0.489746,     -1.157442,
		     -1.200719,     -0.502612,     -0.236972,      0.676530,      0.534445,
		     -0.822389,     -0.616567,      0.563515,     -0.618253,      0.000000,
		     -0.742366,      1.821750,      0.052717,      0.838380,      0.036372,
		     -2.028142,     -0.150730,      2.069125,      1.783464,     -1.533707,
		     -0.664970,      1.298066,      1.202353,     -1.067441,     -3.743037,
		     -0.336874,      1.528357,     -1.440873,     -0.808747,      0.633784,
		      0.634826,     -1.222674,      1.293384,      1.173282,      0.146151,
		     -0.624088,      1.043346,      2.173409,      1.241767,     -1.439360,
		     -2.121885,      0.071565,      0.808165,      3.732354,     -0.102016,
		     -1.911212,     -0.703124,      1.904390,      0.545840,      0.145927,
		     -0.892743,      0.213404,     -0.917418,     -0.078023,      0.142732,
		      1.677089,     -0.243645,      0.102576,     -2.452513,      1.461790,
		      1.763808,     -0.276931,      0.002875,      1.109533,     -0.334572,
		     -0.663146,      0.246430,      0.600688,      0.373092,     -0.155235,
		     -1.805996,      0.662647,      0.856049,      1.062512,      0.844449,
		     -1.185179,     -0.719032,      0.075756,     -0.056832,     -0.183026,
		      0.440637,      0.776559,     -0.197580,      0.609748,     -0.323034,
		     -0.148645,     -1.521956,      1.241029,      1.517051,      0.593351,
		      0.528763,      0.528099,     -0.251207,      0.334524,     -0.121339,
		     -1.125797,      0.736416,      0.809820,      0.053304,      0.000000,
		      0.240664,      0.356011,     -1.038646,     -1.965953,     -1.484027,
		      2.648923,      4.126454,     -0.318803,     -1.793177,      1.481679,
		      1.918957,     -1.534752,     -2.222732,      0.534758,      1.325003,
		     -2.512885,     -0.978125,      0.333212,      0.414597,     -0.799434,
		     -0.804778,      1.882308,      0.479093,     -2.255500,     -0.454548,
		      0.680001,     -0.751141,     -1.699985,     -0.832227,     -0.385878,
		      1.254783,      0.481340,      0.741471,     -1.614806,     -0.711225,
		      0.875953,      1.222778,     -0.955728,     -1.103420,      0.848579,
		      1.810140,      0.768676,      0.575115,      0.119665,      0.350052,
		     -0.516245,     -0.624741,     -1.333931,      3.032155,      0.957616,
		     -1.333647,     -2.930210,     -0.156956,     -0.438847,     -0.812295,
		     -0.072250,      0.389737,      0.669949,     -1.362873,     -0.263000,
		      0.373592,      0.963681,     -0.211012,      0.643233,      1.223037,
		      1.007098,     -0.175506,     -0.020832,     -1.008995,     -0.124596,
		     -0.584882,     -1.569948,      0.260976,     -0.142052,      0.804980,
		      1.749084,      0.564736,      0.398131,     -0.935300,     -0.826245,
		     -0.461085,      0.116659,     -0.319101,      0.930024,      1.935199,
		     -1.039884,     -2.284433,     -0.823907,     -2.516839,      0.254589,
		      0.000000,     -0.317741,     -1.433728,      0.990057,      0.656273,
		      1.556768,      0.839390,     -2.948254,     -0.457774,      0.389019,
		      0.708787,     -1.474401,     -1.110119,      2.101744,      1.931713,
		     -0.715621,     -2.223286,      0.106322,      0.651012,     -0.607035,
		      0.341751,      0.236399,     -0.610217,     -1.626547,      0.807598,
		      0.554684,     -0.867417,     -0.228337,      0.744909,      0.086799,
		      0.917905,      0.248966,     -1.443857,     -0.439988,     -0.466843,
		      2.583794,      1.206973,      0.020634,     -1.044609,      1.045356,
		     -0.676727,     -1.032061,     -1.858909,      0.644916,     -0.699665,
		     -0.415783,     -0.707504,      1.356028,      0.939343,     -0.110981,
		     -2.220930,     -0.734367,      1.220912,      0.711189,      0.178530,
		      1.067524,      0.873279,     -0.322095,     -0.389395,      1.026357,
		      1.512587,      1.571450,     -1.898370,      0.094017,      1.059262,
		      0.407928,      0.113283,     -1.967490,      0.478008,      0.450246,
		     -0.948928,     -0.634634,      0.468595,      1.635865,      1.710676,
		     -0.355097,     -1.330563,      0.136459,     -1.268103,     -0.172003,
		      1.897989,     -0.997474,      0.028208,      1.410126,     -0.086006,
		     -0.128991,      0.312264,      1.084860,      0.305287,      0.664684,
		     -0.662124,     -0.072989,      0.000000,     -0.611779,      2.780819,
		      0.634133,      0.141642,     -2.250845,     -1.778663,      1.243776,
		      2.020414,     -0.573267,     -0.937187,      0.448383,      0.162482,
		     -0.173903,     -0.923432,     -0.000355,      0.918556,     -0.410188,
		     -0.717264,     -0.128009,      0.109091,     -0.658422,     -0.390853,
		      1.738399,      0.750925,     -0.242689,      0.935042,      1.688802,
		     -0.809005,     -1.065199,      0.217024,      0.571935,      1.400561,
		      0.013839,      1.091016,     -1.514414,     -0.748830,     -1.244402,
		      0.123471,      0.521304,     -0.451470,      0.121689,      2.063596,
		      0.481635,      1.428842,      0.487659,      1.257320,     -0.914222,
		     -1.180150,     -2.280783,      2.243718,      1.981190,      0.105981,
		     -2.063017,     -0.878737,      0.385983,     -0.089994,     -1.507773,
		     -0.852869,      0.485926,     -0.514374,     -0.673189,      0.468175,
		     -0.399697,     -0.121586,     -1.647281,     -0.175225,      0.158793,
		     -0.352329,     -0.556143,      0.082449,     -0.109035,     -0.507021,
		     -2.434204,     -1.191594,      0.352759,      1.688684,     -0.614613,
		      0.192001,      0.049238,     -2.398917,     -1.728062,     -1.549353,
		     -0.275430,     -0.240900,     -1.019089,     -0.781171,     -0.029435,
		      0.119588,      1.041710,     -0.291821,      0.216603,      1.403771,
		      0.000000,     -0.089687,      0.681999,     -2.790952,     -1.489063,
		      1.013289,      3.008886,      2.604554,     -1.689761,      0.649009,
		      0.883565,     -0.600614,     -1.552535,     -0.373510,      1.747407,
		     -1.177715,     -3.574665,      0.272412,      0.714912,     -0.085033,
		      0.627517,      0.973070,      0.202381,     -0.942951,     -1.053812,
		      1.000477,     -0.183707,     -0.874582,      0.585166,      1.607728,
		     -1.718277,     -0.892407,      0.221199,      0.353936,      0.359040,
		     -0.577103,     -0.150706,      1.034940,      1.404863,      1.500224,
		      1.338596,      0.001588,     -0.816015,     -1.827830,     -0.238814,
		     -1.609852,     -1.094344,     -1.036745,      1.432132,      1.077929,
		     -0.030699,     -0.495606,     -1.156331,     -0.492264,      0.145289,
		      0.031856,      0.265172,      1.656226,      1.333197,     -0.663493,
		     -1.275943,     -1.043020,      1.477939,     -1.056200,     -0.380633,
		      0.441618,      0.850138,      1.248454,      0.304962,     -0.270288,
		      0.641467,     -0.365168,     -1.224056,      0.295980,      0.303033,
		      0.575879,     -0.487631,      0.170305,      1.115484,      0.605716,
		     -0.751971,     -0.728063,      0.092709,     -0.361405,      0.756384,
		      0.829418,      1.790682,      0.281387,      0.355399,      0.900414,
		      1.271217,     -1.493822,      0.038517,      0.659414,      0.000000,
		     -0.849733,     -2.388323,      1.558053,      1.394580,      2.084797,
		     -0.777912,     -1.548384,      0.944229,     -0.631239,      0.982329,
		      0.076938,     -0.490303,      1.387370,      0.477487,     -2.352980,
		     -1.449782,      1.130574,     -1.058096,     -1.165696,     -0.053877,
		     -1.090287,     -0.459304,      0.585691,      0.986518,     -0.968974,
		     -0.684688,      0.109883,      0.547324,     -0.275611,     -0.656963,
		     -0.484141,     -0.462461,      0.084205,     -0.565705,      1.588619,
		     -0.150932,      1.538916,     -0.539815,     -0.801851,     -1.719455,
		     -0.241570,     -0.145182,      1.484790,     -0.935552,      0.704036,
		      0.961827,      2.944584,     -1.165883,      0.177791,     -1.787884,
		      0.110722,      1.518604,      1.573397,      0.352669,     -0.484110,
		     -0.619517,     -0.015123,      0.345165,      0.198927,     -0.160553,
		      0.215607,     -0.696795,      0.872179,     -0.198931,      0.364665,
		      0.439056,     -1.463929,      0.119653,     -1.218713,     -0.890299,
		      0.435955,     -0.107568,     -0.248226,     -0.122558,     -0.692718,
		     -0.290819,      0.170013,     -1.361885,      0.422361,      0.421121,
		     -1.040041,     -1.327025,     -0.496313,     -0.605930,     -0.860850,
		     -1.744445,     -1.227492,     -2.298290,     -0.243038,     -0.920075,
		      0.397514,      0.484664,      0.682572,      1.808946,      0.000000,
		     -0.799137,      2.362222,      1.838601,     -1.906939,     -4.282371,
		      0.138994,      2.348716,      0.811311,     -1.281674,      0.845011,
		      0.021232,     -1.281443,      0.720641,      1.183951,      1.613175,
		     -0.442977,     -2.501978,     -1.026851,      0.437886,      0.433044,
		      0.794177,      0.828563,     -0.289295,     -0.808794,      0.042114,
		      1.291168,      0.016502,     -1.001657,      0.717386,      1.662974,
		     -0.001715,     -0.060196,     -0.089501,      0.128708,      0.470740,
		      0.372525,     -1.855090,     -0.958359,      1.348498,      1.920695,
		      1.248757,      0.062143,     -1.082942,     -0.163138,      0.891531,
		      0.301417,     -1.832166,     -1.161106,     -0.500244,      0.415938,
		     -1.535803,     -0.366591,     -0.522593,     -0.932269,     -0.893850,
		     -0.807816,     -1.721087,     -0.312361,      1.197187,      0.486254,
		      1.036403,     -0.512626,     -0.878658,     -0.110667,     -1.824336,
		     -0.003532,     -1.127139,     -0.183158,      0.042533,      1.953165,
		      1.050317,      0.752621,     -0.659283,     -0.431289,     -1.186458,
		      0.701548,      1.018109,      0.805413,      0.176879,      0.427562,
		      0.586958,     -1.426381,     -1.318584,     -1.678678,      0.867516,
		      2.118798,      3.178608,      1.832584,      0.570482,     -1.643048,
		     -0.747334,     -0.644691,     -1.785961,      0.128180,     -0.585968,
		      0.000000,      0.146992,      0.896552,     -2.249159,     -0.925496,
		      1.794656,      2.786274,     -0.406402,     -0.678145,      0.907453,
		     -1.027041,     -0.367115,     -0.384748,      0.011732,      1.810395,
		     -0.942787,     -2.338085,      1.415981,      2.459728,     -1.248827,
		     -0.607491,     -0.128955,     -1.212412,     -0.445617,      0.485415,
		      1.470959,     -0.496788,      0.438529,     -0.579348,     -0.859090,
		      0.122385,      1.335567,      0.663042,     -0.300084,     -0.671533,
		     -1.007943,     -1.202117,     -1.091410,      1.825557,      0.238101,
		     -0.198219,     -0.779492,      0.496996,      0.400270,      1.791899,
		     -1.178515,     -0.324048,     -0.138602,      2.788740,      0.189871,
		      0.372342,      0.849290,      0.683221,     -0.296268,      0.880330,
		      0.765065,      1.706387,      0.936938,     -0.123450,      0.137794,
		     -0.634920,     -1.475453,      0.264837,     -0.435262,      0.283991,
		      0.097726,      0.051261,      2.632737,      3.274081,      1.670215,
		     -1.780223,     -1.169252,      0.297800,      1.087806,      1.130770,
		      0.773163,     -0.293662,     -0.451123,     -0.231095,     -1.031236,
		     -1.557666,     -0.441061,      0.973357,      0.583317,      1.744751,
		      1.225808,     -1.640239,     -1.879326,     -0.473371,     -1.167255,
		      0.209580,      0.252768,     -0.151423,     -1.412703,     -0.060502,
		     -0.696390,     -1.851251,      0.000000,     -1.392206,     -1.040980,
		      2.142523,      2.779630,      2.131477,     -3.043889,     -0.557369,
		      2.059121,     -0.858627,      0.685014,      1.193810,     -0.948852,
		     -0.893179,     -0.067965,     -1.915650,     -0.485456,      1.091199,
		     -2.413694,     -1.012521,      0.733636,      0.317182,      1.070092,
		      0.771288,     -0.670530,     -1.832903,     -0.125647,      0.917123,
		      0.875976,      0.525498,     -0.129381,     -1.191597,     -1.128518,
		     -0.493146,     -0.092679,      0.290833,      0.285765,      2.206865,
		     -0.170908,     -0.788534,     -1.161873,      0.827482,      1.511231,
		      0.100672,     -2.583157,     -0.714678,     -0.332774,      0.800613,
		     -1.734258,      0.272916,     -1.078831,     -0.231721,     -2.259103,
		     -0.702381,     -0.015324,     -0.148718,     -1.098686,      0.115446,
		     -2.416029,     -1.399633,     -0.236290,      0.639606,      2.089541,
		      1.625006,      0.178927,      1.405968,      0.493550,     -1.745672,
		     -2.140726,     -2.925755,     -0.349869,      1.475659,      0.201396,
		      0.038377,     -0.278066,     -0.199878,     -0.482965,      0.425445,
		     -0.437308,     -0.177315,      1.611750,      1.525324,     -0.681872,
		     -1.900145,     -0.192831,     -0.151646,      0.043896,      1.263567,
		      1.334349,      0.954842,      0.418474,      0.513642,     -0.784703,
		     -0.907326,     -1.826730,      0.857612,      0.513211,     -0.375307,
		      0.000000,     -0.169212,     -0.630538,      1.020070,     -0.925917,
		     -1.844085,      2.663014,      2.128145,     -1.370063,     -1.136509,
		      2.372114,     -1.863672,     -1.214937,      2.633704,      2.535430,
		      0.718912,     -3.183618,     -1.954231,      1.543721,      2.359185,
		     -1.350468,     -1.231750,      0.143808,     -0.139292,      0.369709,
		      0.269468,      1.616677,     -0.173939,     -0.155399,     -0.362334,
		     -0.641314,      0.018480,      0.559987,      0.676178,      0.825163,
		     -0.132617,      0.204284,     -0.728651,     -0.918864,      1.394108,
		      0.733206,      0.200387,     -0.495825,     -0.170345,      0.166091,
		      1.896548,      1.114642,      0.659413,     -0.685556,     -0.281861,
		     -0.376429,      0.032794,      2.055284,      1.332500,      0.320289,
		     -0.925418,     -0.445131,      0.049162,      1.802828,      1.483877,
		      1.259969,      0.295142,     -1.669166,     -0.790829,      0.121653,
		     -0.833844,     -1.054932,     -1.343564,      1.457706,      2.842293,
		      3.259510,     -0.785938,      0.259141,      1.038458,      0.661519,
		      0.476055,      0.142371,      1.326719,      1.096683,      1.166946,
		     -1.109687,     -0.219199,     -1.428820,     -0.969564,     -0.775439,
		      1.353557,      1.233261,      1.136413,     -1.243643,      0.170268,
		      1.068518,      1.153748,      0.941476,     -0.077616,      1.030398,
		      1.838603,      0.869213,      0.146303,     -0.318159,      0.000000,
		     -0.749988,      0.421549,     -1.650590,     -1.154053,      0.994697,
		      1.748499,     -2.809028,      0.178804,      1.216194,     -1.383294,
		      0.152337,      1.131993,      1.291799,      1.361780,     -0.601232,
		     -1.469389,      1.160832,     -0.228135,     -2.570424,      0.663977,
		      1.594658,      0.234280,     -0.353261,      0.907754,     -0.171568,
		     -1.868434,     -0.392359,     -0.035243,      1.577243,      2.441127,
		      1.726680,      0.092421,     -0.114240,     -0.869953,     -0.302016,
		     -0.549023,     -1.228887,      1.560170,     -0.362017,      0.251032,
		     -2.160411,     -0.514171,      0.453783,      1.169823,     -1.556264,
		     -0.492534,     -1.564503,      0.620785,     -0.274611,      1.545044,
		      0.115140,      0.329961,     -1.870738,     -1.418052,      0.171656,
		      0.607857,     -0.903107,      1.447946,     -1.048244,     -1.048634,
		     -1.369600,     -0.765810,      0.000319,      0.301198,      0.332071,
		      0.963622,      2.314404,      0.477386,     -1.729335,     -1.622252,
		      0.262570,     -0.249392,      0.150356,     -0.117434,      0.332256,
		      0.523811,     -0.380101,     -2.049124,     -1.172236,     -1.366965,
		      0.340312,      1.351595,     -0.042216,      0.144006,     -1.360512,
		     -1.828824,     -0.837772,      2.463746,     -0.090628,     -0.975032,
		     -0.416247,      0.013133,     -0.463923,      0.618639,      0.133885,
		      0.012190,      0.534397,     -1.105794,     -0.606681,      0.000000,
		     -0.582210,      1.476678,      0.018669,      1.021080,      0.520947,
		     -2.663639,      1.979009,      2.010643,     -1.457894,      0.294260,
		      1.307054,     -2.477549,     -1.979815,      0.889601,      0.368768,
		      0.130575,     -1.132207,     -1.518005,      0.553603,      2.289003,
		     -0.202661,     -1.059491,      0.306868,     -0.706691,      0.065418,
		      0.362299,      1.979315,     -0.031916,     -1.741745,     -1.545276,
		     -0.989179,      0.815696,     -0.499454,     -1.200337,      0.108255,
		      0.818704,      0.722224,     -0.077532,     -1.617996,     -0.526606,
		      0.920301,      1.756822,      1.249642,     -0.177534,      0.038632,
		      0.233543,      0.479751,      0.972950,      1.642315,     -1.692745,
		     -0.913029,     -1.279000,      0.375554,      0.698764,      1.475529,
		      1.050967,      0.920532,     -2.063934,     -0.264885,     -0.193103,
		      1.818922,      2.095030,     -0.164491,     -0.261415,      1.212695,
		      0.303317,      0.022990,     -0.878121,     -0.590061,      1.026430,
		      0.285208,     -0.007415,      0.311845,     -0.802118,     -0.009328,
		     -1.054298,     -1.502140,      0.893482,      0.169699,      1.698734,
		      1.209845,     -0.245351,     -0.966037,     -1.974873,     -0.260498,
		     -0.408619,     -0.341388,     -0.608879,     -1.412414,     -1.154375,
		     -0.055035,      0.060240,      0.680469,      1.264801,     -1.549901,
		      0.485145,     -0.458543,     -0.094652,      0.853690,     -0.439264,
		      0.000000,     -1.022155,     -1.473093,      2.395450,      0.772617,
		     -0.026596,      1.242507,      0.660681,     -2.903448,     -0.856595,
		      1.613307,     -1.733649,     -0.752645,      2.679698,      1.474900,
		     -2.276951,     -2.699624,      1.370390,      0.679887,     -1.553993,
		     -1.848033,      1.617418,      1.441448,     -0.144629,      0.013735,
		      1.285992,      0.732575,     -1.818833,     -0.867219,     -0.038841,
		      1.837615,      0.383942,     -0.124370,     -0.007776,      1.684050,
		      0.886127,     -0.037344,     -1.171682,     -0.980362,      0.918353,
		      0.071061,      1.254225,     -0.907223,     -2.250589,     -2.084655,
		      0.762647,     -0.508920,      0.701488,     -1.921657,     -1.720710,
		     -0.992835,      1.329655,      2.355931,      1.002255,     -0.909950,
		     -2.114659,     -0.720225,     -0.080608,     -0.377240,      1.079129,
		      1.674918,     -0.453078,     -1.328580,     -0.572659,     -0.141599,
		     -0.200846,     -0.555018,     -0.951511,      1.815263,      1.947200,
		      0.231086,     -0.142920,      0.794563,     -0.384481,     -0.360367,
		     -1.270695,      1.284945,      1.957594,      0.427647,     -1.506140,
		     -1.122085,     -1.060353,     -1.636314,      1.055170,     -0.146571,
		     -1.468745,     -0.527455,     -0.566816,     -0.167560,      2.031760,
		      1.287169,      0.494851,     -0.780146,     -1.767153,     -1.499312,
		     -1.533792,      0.150711,      0.503393,      1.693914,     -1.341150,
		     -1.682690,     -1.481272,      0.000000,     -1.002786,      0.325295,
		      0.392199,     -1.354238,      0.253856,      0.448779,     -2.467558,
		      1.664891,      0.149865,     -0.035668,     -0.154764,      2.228940,
		      1.646056,      0.571512,     -0.812271,     -0.640772,      0.764362,
		     -0.969204,      0.451657,      0.241332,     -0.418035,     -0.817944,
		     -0.029027,      0.427434,     -1.217941,     -0.992137,     -0.207220,
		      1.961525,      2.377425,     -0.514033,     -1.842018,     -0.541895,
		      1.100069,     -0.056689,     -2.103691,     -1.616988,      1.370948,
		      2.277232,      0.186780,     -0.976769,     -2.622176,     -0.078100,
		      1.057936,      3.132214,      1.796428,      1.131471,     -1.000071,
		      0.803834,      0.290615,      1.720903,     -0.836567,     -0.825396,
		     -1.040479,     -0.054751,     -0.148843,     -0.054928,     -0.245572,
		      1.778462,     -0.545340,      0.072502,     -1.583854,      0.003007,
		     -0.256882,     -0.287242,     -0.250753,      0.898477,      1.013669,
		     -0.696705,     -1.044290,      0.802291,      0.017863,     -0.417580,
		     -0.636584,     -0.465256,      0.603672,      0.423806,     -1.560915,
		     -2.170855,     -0.163259,     -0.458671,      2.039908,     -0.566407,
		     -1.863853,      1.339065,     -0.578429,      0.322682,     -0.714963,
		     -1.419189,     -1.912914,     -1.222944,     -1.683077,      0.313055,
		      1.197661,     -0.494983,     -0.874942,      0.177840,      0.297940,
		      0.001816,     -0.000551,      0.387375,      1.334671,     -0.705655,
		      0.000000,      0.657954,     -0.113645,     -2.790658,      1.816002,
		      0.823113,     -1.415037,      0.929854,      1.670519,     -1.496137,
		     -0.313606,      1.178920,     -2.049048,     -0.134831,      3.806054,
		      1.843891,     -2.089772,     -1.211053,      0.834720,     -0.822259,
		     -1.528314,     -1.102078,      0.610421,     -0.255628,     -0.003593,
		     -0.235896,      0.293693,      0.613910,     -0.760837,     -1.097574,
		      0.463469,      2.818271,      1.251745,     -1.683383,     -1.459131,
		      2.023324,      1.844665,      0.677203,     -1.114839,     -2.417977,
		      0.507690,      1.102101,      1.995956,      0.630133,     -1.156591,
		     -2.765678,     -1.075548,     -0.128846,      0.214328,      0.598659,
		     -1.111110,     -0.685305,     -0.082472,      0.350123,      1.043496,
		      1.843615,     -0.393140,      0.503780,     -0.755779,     -1.129316,
		     -1.178440,      2.461304,      1.490389,      0.040422,     -1.112103,
		      0.004372,     -1.093716,      0.470904,      0.082717,      0.788780,
		      0.922329,     -1.422913,      0.244747,      2.033811,      0.547140,
		     -1.088597,     -2.977432,     -2.423174,      2.973289,      2.401414,
		      0.049904,     -0.336110,      1.574964,      0.726993,     -0.792444,
		     -0.475438,     -1.696407,     -0.470941,     -0.585135,     -0.768711,
		      2.145324,      1.333366,      1.154559,     -0.482323,     -1.141045,
		     -1.553902,     -0.207598,     -0.439303,     -0.894633,     -1.489895,
		     -1.992593,     -0.809316,     -0.099755,      1.710814,      0.000000,
		     -0.845917,     -0.715195,      2.681898,      1.175489,     -0.753045,
		     -0.592939,      0.869688,     -2.410990,      1.136192,      1.422532,
		     -0.421557,     -1.754897,      0.962448,      1.291270,     -1.814170,
		     -1.398680,      0.894871,     -1.151588,     -1.031485,      2.674105,
		      1.352356,     -1.203740,      0.856174,      1.200063,      0.046443,
		     -0.085084,      0.727582,     -1.584928,     -0.972857,      1.571706,
		     -1.212535,     -1.479699,      0.141882,      1.513832,      0.408412,
		     -1.466688,     -3.149788,     -0.391830,      1.408240,      1.367635,
		      0.971661,     -2.739881,     -2.261534,     -1.469713,      2.177094,
		      1.931434,      1.986238,     -0.341359,      1.111591,      0.045929,
		      0.444880,      0.067362,     -0.245398,     -0.836557,     -0.113982,
		     -0.122921,     -0.370997,     -1.346239,      0.166196,      0.881450,
		     -0.344396,     -1.136724,     -0.311688,      0.456853,      0.007197,
		     -0.174187,     -1.183259,      1.058770,      0.227827,      0.001000,
		      0.442266,     -0.723483,     -2.127433,     -1.487337,      0.012231,
		      2.697237,      0.144495,     -0.363119,     -3.522003,     -0.793693,
		      0.899565,      0.661975,      0.180121,     -2.219124,      1.506732,
		      0.314951,     -1.028939,      0.272555,      0.270762,     -1.596824,
		     -1.680351,     -0.343518,      0.514606,      0.288127,      1.244423,
		      2.364589,      2.438172,      1.897997,      1.462096,      0.301334,
		     -1.624835,      0.314962,     -1.098381,      0.152372,      0.000000,
		      0.307326,      1.829440,     -0.206880,     -2.751630,      1.289649,
		      0.569153,     -2.052576,      0.537014,      0.303284,      1.061156,
		     -1.413631,      1.318995,      0.234607,     -1.163203,     -0.350891,
		      1.590040,      1.215863,     -1.123784,      0.921894,     -0.648772,
		     -0.069397,      0.698422,     -0.192028,     -1.853688,      0.422580,
		      0.648749,     -1.212199,     -0.095581,      0.762597,     -0.393467,
		     -1.220743,      1.448159,      0.969186,     -1.199141,     -2.039810,
		      0.691013,      2.858239,      1.524247,     -0.932071,     -2.669725,
		     -1.848716,      0.632054,      1.587781,      2.194569,      1.149576,
		     -1.293862,     -0.212423,     -0.361415,     -1.268474,     -0.085031,
		     -0.358055,      0.541815,      0.895996,     -1.179960,     -0.925307,
		      1.120068,     -0.343779,      1.045956,      1.002528,      1.440756,
		     -1.327125,      1.425398,      1.015770,      0.888745,     -0.209730,
		      0.206306,     -1.012607,     -0.529017,     -1.124620,      0.178244,
		      0.911706,      2.234588,      0.686719,      1.289231,      0.703463,
		     -0.221756,     -1.096534,     -2.131470,      0.973252,      1.409369,
		      0.192645,      0.055913,      1.818961,      1.752281,     -0.269353,
		      0.614259,     -1.213070,     -1.734247,      1.049700,      1.089910,
		      0.319277,      1.174248,      0.907694,     -2.021162,     -0.510078,
		     -1.424244,     -1.452380,     -1.849655,      1.105358,      1.418502,
		      1.789880,      1.160706,      1.417304,     -0.189582,      0.637964,
		      0.000000,      1.005639,     -1.220097,     -2.522564,      2.098121,
		      0.360461,     -0.333116,      0.227235,      1.624557,     -3.090720,
		     -0.790977,      2.166865,      0.850555,     -0.269523,      2.483605,
		      1.924018,     -1.933926,     -0.418237,      0.377147,     -2.951034,
		     -1.515957,      0.813896,      0.846231,     -1.566176,      0.344241,
		      0.354602,     -1.718941,     -0.873303,      3.671300,      0.745209,
		     -2.049218,      0.333709,     -0.898741,     -0.934467,      0.590674,
		      1.927584,      0.091440,     -0.086077,     -1.796705,     -0.888857,
		      0.907509,      0.940087,      2.010714,      0.478264,     -1.596000,
		     -3.280559,     -0.976497,     -0.668107,      0.925930,     -0.434996,
		      2.132865,      1.968113,     -1.048961,     -0.389989,      0.722285,
		      1.529129,      0.272025,      1.188434,      1.326158,     -1.831369,
		     -2.839964,      0.154085,     -1.376725,     -0.473411,     -1.000470,
		      0.372022,      0.495681,      0.200754,     -0.566078,      0.490488,
		      1.248983,     -0.279043,     -0.152813,     -0.476854,     -1.208536,
		     -0.775864,     -1.821526,      0.583737,      3.184919,      0.876662,
		     -0.789923,     -0.870147,     -0.909114,      0.081268,      1.259454,
		     -0.819109,     -0.598005,      1.929370,      1.005751,     -0.071128,
		     -0.187820,     -0.183589,      0.091416,     -0.564955,      0.090448,
		      0.950369,     -0.383536,      0.934107,      1.049977,      1.267852,
		      0.136740,     -0.193020,     -0.703199,     -0.073509,      0.069071,
		     -1.465385,      1.296847,      0.000000,     -2.043060,     -0.753407,
		      2.457900,      1.612937,     -1.198590,     -0.977632,      0.129142,
		     -1.043492,      1.642592,      1.159032,      0.152272,     -3.185623,
		      1.805642,      2.764193,     -2.051522,     -2.678942,      1.308131,
		      0.721868,     -1.720057,      2.461785,      0.471879,     -0.771864,
		      0.081465,      0.651562,     -1.080334,      0.634626,      2.013260,
		     -2.192103,     -3.035188,      0.665730,      1.008399,     -0.004884,
		      1.296830,      0.879594,     -1.319559,     -2.805879,     -1.145423,
		      2.071699,      0.287646,     -0.266073,     -0.504932,     -2.141102,
		     -0.180502,     -0.110580,      1.419036,      0.974346,     -0.231614,
		     -0.185187,      0.586219,     -1.623709,     -1.852848,     -0.849075,
		      0.807858,      1.948224,      0.458718,     -1.043953,     -0.211664,
		     -2.172758,     -0.336117,      1.481820,      1.772627,      0.003924,
		      0.835508,      1.768111,      0.815119,      0.636699,     -0.180639,
		     -0.363776,     -0.547247,     -0.987285,     -1.933053,     -0.269854,
		      0.360478,      1.479317,     -0.138383,      1.659671,      0.357284,
		      0.175165,     -2.448755,      0.464578,      0.114593,      0.546811,
		     -1.226318,     -1.053860,      2.029162,      0.829518,     -0.247013,
		     -1.628338,     -2.964196,     -0.778156,     -0.342400,     -0.450505,
		      1.832130,      1.739782,      0.455871,     -1.150807,     -1.530942,
		     -1.106924,      2.094894,      1.068890,      0.144233,      0.318878,
		      0.396534,      0.488721,      0.859340,      1.006663,      1.475461,
		      0.000000,      1.407080,      2.530874,     -0.742478,     -2.725270,
		      0.047025,     -0.188129,     -0.628800,      0.893376,      2.376338,
		     -0.494157,     -2.363415,      1.313072,      1.100196,     -1.677522,
		     -0.475196,      1.727520,     -0.704791,     -1.457797,      2.401485,
		     -0.591036,     -2.154202,     -1.113635,      1.928962,      0.105186,
		      0.388148,      1.106593,     -1.807865,     -2.847211,      2.247749,
		      2.787998,     -1.036048,     -0.342025,     -1.405427,     -1.913763,
		     -0.387221,      2.836022,      1.246240,      0.210632,     -0.886669,
		      0.224008,      1.348357,     -0.727224,     -0.730542,      0.578377,
		      1.130813,     -0.806963,     -0.780555,     -1.217395,     -0.229574,
		     -1.312265,      0.954527,      1.914787,     -0.153972,     -2.225256,
		     -0.276466,      0.554899,     -1.216068,      0.346091,      1.853093,
		      2.518420,     -1.122185,      0.324042,     -0.797750,     -1.025814,
		     -1.300593,     -0.103678,     -0.024819,      1.035606,     -1.344287,
		     -1.288994,     -0.010370,      0.664261,      0.500592,      0.839107,
		     -0.586995,      1.394499,     -0.798755,      0.158452,      1.679429,
		      0.422750,      0.057196,      0.495955,      1.145055,      0.319853,
		     -0.555905,     -0.493263,     -0.515944,      1.428112,      1.520608,
		      0.296359,      2.240728,     -0.777829,     -0.643887,     -1.309531,
		      1.247703,     -1.366935,      1.593843,     -0.740631,     -0.223274,
		     -1.852568,      0.528540,     -0.000741,     -0.857023,     -0.298910,
		      1.051590,     -1.612896,      0.203449,      0.234776,      0.000000,
		      1.800294,     -2.395732,     -2.879814,      0.416692,      2.017166,
		      1.060690,     -1.359067,     -0.079948,     -2.523946,      0.965526,
		      1.244159,      0.116419,     -1.636288,      2.327338,      2.041959,
		     -1.327670,     -1.407063,      0.364434,     -1.079780,     -1.942574,
		      1.296773,      2.187941,      0.176991,     -1.895994,     -0.457500,
		     -0.424176,     -0.205293,      3.054945,      0.314428,     -2.275233,
		     -1.318100,     -0.421574,     -0.587706,      0.861260,      1.860475,
		      0.506330,     -1.172223,     -2.654873,     -0.306589,     -0.370210,
		     -1.010877,     -0.901423,     -0.723726,      0.342430,     -1.278144,
		      0.416987,      0.677587,      0.588584,      0.399805,      2.517210,
		      1.505150,     -1.210046,     -1.825623,     -0.936254,      1.042195,
		      0.944214,      1.035620,      1.519314,     -1.348226,     -2.659701,
		     -0.457951,      0.149226,      0.231706,      0.352703,      0.877201,
		      0.043874,     -0.304660,     -0.861868,      0.568934,      1.695944,
		      0.823168,     -0.574023,     -0.467010,      0.602521,      1.256554,
		     -0.423720,     -0.320961,      0.292348,      0.146200,      1.462157,
		     -0.408256,     -1.150499,      0.241944,     -2.219235,     -1.165192,
		      1.243544,      0.098301,     -0.495246,     -1.270798,     -0.353194,
		      0.707837,      0.168266,     -0.278204,      1.901189,      0.653649,
		      0.290105,      0.248055,     -0.915319,     -1.400997,      0.906439,
		      1.869301,      1.288432,      1.606141,      0.185564,     -0.259251,
		     -1.566354,     -3.253575,     -1.566939,      0.392429,      0.000000,
		     -1.449594,     -1.056680,      1.972857,      3.157787,     -0.022670,
		     -2.192976,     -1.399181,      0.094091,      0.203035,      1.143839,
		      0.881091,     -0.669092,      2.220512,      2.117465,     -1.740778,
		     -0.970102,      1.673708,     -0.054154,     -2.678777,      3.390516,
		      1.593242,     -1.996148,     -2.628727,      1.021490,      0.879578,
		     -0.070809,      0.192703,      0.721097,     -2.212037,     -1.442460,
		      1.416143,      0.689985,      1.307718,      1.342056,     -0.229028,
		     -2.758036,      0.819441,      1.363097,      0.885074,      0.262421,
		      1.212574,      3.218170,      1.800611,      0.130973,     -0.582779,
		      0.048108,      0.680720,     -0.635055,     -1.474374,     -0.201169,
		     -1.111127,     -0.331505,      2.044952,      1.771477,      0.609714,
		     -0.795687,     -0.907498,     -2.211621,     -2.185147,     -0.799897,
		      1.075360,     -1.308633,      0.039221,     -0.250175,     -0.975706,
		      0.272603,     -0.210391,      0.479328,      0.474780,     -0.160546,
		     -0.624277,     -0.445401,     -0.226037,      0.438707,     -2.022245,
		      0.092961,      1.594026,      1.787409,     -0.289470,     -0.957330,
		     -1.498125,     -0.727597,      0.464094,      1.312925,      0.824684,
		      0.834683,      1.767725,     -0.014587,      0.186147,      0.567790,
		     -0.740910,     -1.405765,     -1.045072,     -0.326690,      0.053765,
		     -0.762366,      0.260780,      1.339917,     -0.271218,     -0.784301,
		     -0.764645,     -0.717046,     -1.426446,      0.805221,      0.768326,
		      2.036998,      0.689558,     -1.000148,     -2.325312,     -0.323547,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : mars ---  Model : cain2003 */
coeffStruct& _model_coeff_cain2003() {
	static const std::string name = "cain2003";
	static const std::string body = "mars";
	static const int len = 4185;
	static const int nmax = 90;
	static const int ndef = 90;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,  14,  14,  14,  14,  14,  14,  14,  14,
		  14,  14,  14,  14,  14,  14,  14,  15,  15,  15,  15,  15,  15,  15,  15,  15,
		  15,  15,  15,  15,  15,  15,  15,  16,  16,  16,  16,  16,  16,  16,  16,  16,
		  16,  16,  16,  16,  16,  16,  16,  16,  17,  17,  17,  17,  17,  17,  17,  17,
		  17,  17,  17,  17,  17,  17,  17,  17,  17,  17,  18,  18,  18,  18,  18,  18,
		  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  19,  19,  19,
		  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,
		  19,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,
		  20,  20,  20,  20,  20,  20,  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,
		  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,  22,  22,  22,  22,
		  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,
		  22,  22,  22,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,
		  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  24,  24,  24,  24,  24,
		  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,
		  24,  24,  24,  24,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,
		  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  26,  26,
		  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,
		  26,  26,  26,  26,  26,  26,  26,  26,  26,  27,  27,  27,  27,  27,  27,  27,
		  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,
		  27,  27,  27,  27,  27,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,
		  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,
		  28,  28,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,
		  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,
		  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,
		  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  31,
		  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,
		  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  32,
		  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,
		  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,
		  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,
		  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,
		  33,  33,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,
		  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,
		  34,  34,  34,  34,  34,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,
		  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,
		  35,  35,  35,  35,  35,  35,  35,  35,  35,  36,  36,  36,  36,  36,  36,  36,
		  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,
		  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  37,  37,
		  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,
		  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,
		  37,  37,  37,  37,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,
		  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,
		  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  39,  39,  39,  39,  39,
		  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,
		  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,
		  39,  39,  39,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,
		  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,
		  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  41,  41,  41,  41,
		  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,
		  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,
		  41,  41,  41,  41,  41,  41,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,
		  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,
		  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,
		  42,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,
		  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,
		  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  44,  44,  44,
		  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,
		  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,
		  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  45,  45,  45,  45,  45,  45,
		  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,
		  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,
		  45,  45,  45,  45,  45,  45,  45,  45,  46,  46,  46,  46,  46,  46,  46,  46,
		  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,
		  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,
		  46,  46,  46,  46,  46,  46,  46,  47,  47,  47,  47,  47,  47,  47,  47,  47,
		  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,
		  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,
		  47,  47,  47,  47,  47,  47,  47,  48,  48,  48,  48,  48,  48,  48,  48,  48,
		  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,
		  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,
		  48,  48,  48,  48,  48,  48,  48,  48,  49,  49,  49,  49,  49,  49,  49,  49,
		  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,
		  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,
		  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  50,  50,  50,  50,  50,  50,
		  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,
		  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,
		  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  51,  51,  51,
		  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,
		  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,
		  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,
		  51,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,
		  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,
		  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,
		  52,  52,  52,  52,  52,  52,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,
		  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,
		  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,
		  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  54,  54,  54,  54,
		  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,
		  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,
		  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,
		  54,  54,  54,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,
		  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,
		  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,
		  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  56,  56,  56,  56,  56,
		  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,
		  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,
		  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,
		  56,  56,  56,  56,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,
		  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,
		  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,
		  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  58,  58,
		  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,
		  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,
		  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,
		  58,  58,  58,  58,  58,  58,  58,  58,  58,  59,  59,  59,  59,  59,  59,  59,
		  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,
		  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,
		  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,
		  59,  59,  59,  59,  59,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,
		  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,
		  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,
		  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,
		  60,  60,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,
		  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,
		  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,
		  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,
		  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,
		  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,
		  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,
		  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  63,
		  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,
		  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,
		  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,
		  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  64,
		  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,
		  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,
		  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,
		  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,
		  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,
		  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,
		  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,
		  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,
		  65,  65,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,
		  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,
		  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,
		  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,
		  66,  66,  66,  66,  66,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,
		  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,
		  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,
		  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,
		  67,  67,  67,  67,  67,  67,  67,  67,  67,  68,  68,  68,  68,  68,  68,  68,
		  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,
		  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,
		  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,
		  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  69,  69,
		  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,
		  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,
		  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,
		  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,
		  69,  69,  69,  69,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,
		  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,
		  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,
		  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,
		  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  71,  71,  71,  71,  71,
		  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,
		  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,
		  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,
		  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,
		  71,  71,  71,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,
		  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,
		  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,
		  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,
		  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,
		  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,
		  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,
		  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,
		  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
		  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
		  41,  42,  43,  44,  45,  46,  47,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
		  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
		  41,  42,  43,  44,  45,  46,  47,  48,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,
		  51,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,
		  47,  48,  49,  50,  51,  52,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,
		  52,  53,  54,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,
		  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,
		  53,  54,  55,  56,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,
		  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,
		  50,  51,  52,  53,  54,  55,  56,  57,  58,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,
		  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
		  55,  56,  57,  58,  59,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,
		  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,
		  59,  60,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,
		  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
		  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,
		  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,
		  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
		  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,
		  64,  65,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,
		  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,
		  62,  63,  64,  65,  66,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,
		  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,
		  59,  60,  61,  62,  63,  64,  65,  66,  67,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,
		  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
		  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,
		  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,
		  66,  67,  68,  69,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,
		  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,
		  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,
		  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,
		  69,  70,  71,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,
		  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,
		  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,
		  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,
		  68,  69,  70,  71,  72,  73,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,
		  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,
		  74,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,
		  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,
		  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,
		  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,
		  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,
		  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,
		  70,  71,  72,  73,  74,  75,  76,  77,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,
		  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,
		  72,  73,  74,  75,  76,  77,  78,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
		  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
		  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,
		  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,
		  73,  74,  75,  76,  77,  78,  79,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
		  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
		  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,
		  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,
		  73,  74,  75,  76,  77,  78,  79,  80,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,
		  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,
		  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,
		  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,
		  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,
		  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,
		  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,
		  83,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,
		  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,
		  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,
		  79,  80,  81,  82,  83,  84,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,
		  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,
		  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,
		  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,
		  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,
		  84,  85,  86,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,
		  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,
		  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,
		  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,
		  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,
		  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,
		  85,  86,  87,  88,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,
		  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,
		  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,
		  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,
		  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,
		  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,
		  82,  83,  84,  85,  86,  87,  88,  89,  90,
	};
	static const std::vector<double> g = {
		     -1.896817,     -0.327507,      0.172418,      1.124383,      1.299382,
		      0.881620,     -0.651172,     -1.101305,     -1.036233,     -0.887181,
		     -0.686442,      0.046595,      2.118790,      0.483972,      0.976372,
		      0.266108,      0.361659,     -0.260219,      0.425077,      0.510229,
		     -1.027526,     -0.126124,     -0.200706,     -1.703791,      0.415862,
		      0.345697,     -0.921222,      1.412424,      1.218528,     -0.701681,
		      0.530167,      0.118454,      0.507893,      1.077065,      1.451025,
		     -0.717853,     -3.315539,      0.869583,      1.343017,     -0.697412,
		     -0.917147,     -1.048414,     -2.082533,     -0.728913,     -1.121290,
		      3.047690,      1.565300,     -1.511755,     -0.335576,      0.727818,
		      0.829735,      0.476849,      0.612228,      1.497829,      2.577113,
		     -0.418558,     -4.084970,     -0.675890,      1.865932,      0.816864,
		     -0.401880,     -0.265916,     -0.952864,     -1.871609,     -0.750407,
		     -2.141370,     -2.267442,      2.976905,      2.374975,     -1.047068,
		     -1.973966,     -0.409781,      0.189322,      0.673579,      0.689116,
		      1.114560,      0.921798,      0.674131,      3.453223,     -0.010213,
		     -2.209688,     -1.796245,      0.777987,      1.413212,     -0.147780,
		     -0.302615,      0.701001,     -1.712255,     -2.473379,     -2.744451,
		      0.104067,     -2.862442,     -2.125062,      2.249854,      3.680372,
		      1.190219,     -0.438319,     -0.698326,     -0.026312,      0.213157,
		      1.075350,      2.517445,      2.091796,     -0.107417,     -2.741804,
		      0.519850,      2.772564,     -0.856554,     -2.204955,     -2.176615,
		     -0.070351,      1.026435,      0.417553,      0.525854,      1.173983,
		     -1.335917,     -1.748273,     -2.510686,     -1.773443,      3.896753,
		      0.556453,     -3.099206,     -1.661229,      0.551198,      2.094711,
		      0.184997,     -0.239013,     -1.072142,     -1.095119,     -1.322759,
		     -0.619738,      0.779156,      2.582956,      1.652193,      0.619759,
		     -2.503459,     -3.395779,      2.129764,      3.987065,     -0.081473,
		     -1.601071,     -0.889840,     -0.751338,      0.572628,      0.437977,
		      1.080898,      1.277753,      0.449293,      0.212152,     -1.910569,
		     -1.151281,     -0.068609,     -1.508112,      5.629580,      0.491568,
		     -4.157924,     -0.967560,      1.557886,      2.510860,     -0.248154,
		     -0.431949,     -0.379980,     -0.500582,     -0.862001,     -0.427039,
		     -0.809604,      1.949926,      1.155404,      1.324008,      0.856634,
		      3.319837,     -5.634848,     -4.312572,      1.724551,      2.837812,
		     -0.092847,     -1.615119,      1.164927,     -0.506577,      0.431284,
		      0.808507,      1.549042,      2.044830,      0.820027,     -0.110476,
		     -0.602020,     -0.325221,      0.275050,     -0.993015,     -2.754877,
		      0.573944,      6.307228,      0.981033,     -3.054716,     -2.178223,
		      0.307523,      1.517630,      0.328533,     -0.513611,     -1.297709,
		     -2.979929,     -2.846325,     -0.516619,     -0.209451,      0.847055,
		     -0.088577,      0.044641,      1.008422,     -1.482425,      3.686098,
		      3.452355,     -5.498982,     -3.629000,      2.196075,      1.925685,
		     -0.560882,     -1.633181,      1.460856,      0.417852,      1.040326,
		      2.842339,      2.526339,      1.592117,      0.530687,      0.123439,
		      0.574671,      0.262014,      0.520127,      0.619240,     -0.962478,
		     -3.197181,     -3.734281,      2.114726,      4.549083,      0.593956,
		     -0.315454,     -0.011117,      0.803913,     -1.191095,     -0.208011,
		      1.001895,     -0.090022,     -1.577975,     -2.968666,     -1.223788,
		     -0.760671,      0.478180,      1.006418,     -0.492989,     -1.005950,
		     -1.171476,     -1.149010,      0.699604,      3.644200,      1.200093,
		     -3.709921,     -2.624370,      0.275439,      0.595780,     -0.527485,
		     -0.579922,      1.059750,     -1.602298,     -1.133034,      1.119738,
		      1.905919,      1.239538,     -0.547719,     -2.098013,     -0.512531,
		      0.416589,      1.000073,      1.181410,      0.397347,      1.454291,
		      1.588075,     -4.834492,     -2.851099,      3.006024,      2.050127,
		     -1.041030,     -0.523712,      0.454971,      1.623053,     -1.393681,
		     -1.468850,      1.117740,      0.118958,     -1.244030,     -1.925250,
		      1.265961,      1.813930,      1.132844,      1.247584,     -1.113227,
		     -0.004120,     -1.650612,     -1.308898,      0.721779,     -1.533615,
		      3.050392,      1.828940,     -1.129806,     -1.121035,     -0.398609,
		     -0.636272,     -1.064124,     -0.872712,      0.809728,      2.279572,
		     -0.989532,     -1.619673,      0.890901,      2.214488,     -0.154050,
		     -1.491059,     -2.199164,     -1.864557,     -0.995918,     -0.123288,
		      0.673146,      1.480855,      1.530401,      1.889277,      1.178017,
		     -0.107198,     -1.167758,     -0.798886,      2.356380,      1.353011,
		     -0.245594,      0.196317,     -0.303457,      0.000242,     -0.311183,
		      0.258890,      0.019761,     -1.004490,     -2.068633,     -2.476657,
		      0.971813,      2.806375,      0.844684,      0.905874,     -1.538453,
		     -1.158433,     -0.135012,     -1.999252,     -0.151031,     -0.185293,
		     -0.637829,     -0.279667,      2.019035,      0.712205,     -2.691217,
		     -0.239999,      0.398608,      0.020984,     -0.800935,     -2.415125,
		     -1.488031,      2.117518,      0.834455,      0.061955,      1.085788,
		      3.537067,     -0.035360,     -1.002031,      0.275407,     -0.779926,
		     -0.129796,      0.047215,      0.935908,      1.631506,      1.042434,
		      2.607187,      0.289856,      1.227609,      0.211585,     -2.227893,
		      0.173609,      0.376290,      0.688419,      0.566457,      0.644920,
		      1.774429,      0.668883,     -0.951950,     -1.982120,      1.377174,
		      1.408437,     -0.259102,     -1.855066,     -2.250283,     -1.992350,
		     -0.440192,     -0.570272,      0.687759,     -0.221586,     -0.744367,
		     -0.189153,     -2.574680,     -2.068498,     -0.461713,      0.144152,
		     -1.018004,      0.418794,      1.547338,      1.331359,      0.588815,
		     -1.673890,      0.208095,      0.767010,     -0.736506,      1.248180,
		      1.786722,     -0.873815,     -1.709055,     -1.237735,     -1.074503,
		      1.716376,      3.266436,      1.308311,      0.017049,      1.904963,
		      1.170628,      1.422324,      1.748649,      0.373551,      1.696765,
		      1.613281,      0.795837,     -0.744317,     -0.599313,     -0.290349,
		     -1.084972,     -0.177478,     -1.729652,      1.357502,      1.327681,
		     -2.007380,     -1.765057,     -1.130911,     -0.110507,      0.982268,
		      0.493265,     -1.829983,      0.417183,      0.792468,     -1.073113,
		     -2.440910,     -0.350057,     -0.040562,     -1.515453,     -2.249920,
		     -1.614247,      0.067599,      0.076551,      1.953134,     -1.582996,
		     -1.491604,      0.629678,     -0.414550,      0.121771,     -0.346541,
		      0.293242,     -0.076902,      0.104768,     -1.011840,     -0.127213,
		     -0.175705,      0.839098,      1.927713,     -1.344666,     -2.061988,
		      0.248940,      1.777461,     -0.171971,      0.146432,      0.385253,
		      1.665473,      1.414294,      0.187669,      0.569387,      3.721506,
		      0.729586,      1.131577,      0.476617,     -0.967380,      1.260371,
		      0.440571,      0.667454,     -0.432850,     -0.896628,      1.463071,
		      2.292160,      0.001884,     -0.992667,      0.522002,     -0.989246,
		      1.549234,      3.554966,     -0.365698,     -1.616132,     -0.083519,
		     -0.319989,      0.497791,      1.151862,     -0.276026,     -0.534868,
		      0.426147,     -2.696743,     -2.705393,     -0.878117,      0.018648,
		     -1.789745,     -0.680213,     -1.823933,     -1.077280,     -1.391536,
		      0.220393,      1.499304,      1.159612,      0.467649,      1.189532,
		     -0.592030,     -0.913809,     -1.176977,      0.363650,      0.640266,
		      0.538613,      1.842359,     -1.473481,     -2.324711,     -0.349448,
		     -0.143879,      2.271939,      1.329383,     -1.429226,     -1.832925,
		     -0.958116,     -1.168446,     -1.890947,      1.651565,      4.283289,
		      1.607513,      0.305930,      0.411013,      3.219991,      1.319865,
		      0.488494,      2.148439,     -0.387245,     -1.109917,     -1.085623,
		      0.536805,      0.443969,     -1.196623,      1.854797,      0.353443,
		     -0.072321,     -0.268609,     -0.599414,     -0.867078,      0.765513,
		     -0.814107,      0.308732,      2.508788,     -0.173569,     -1.283665,
		     -0.407741,     -0.988009,     -0.068090,      1.833609,      2.384892,
		      1.971363,      2.198034,     -2.454113,     -3.566429,     -0.749323,
		      1.532881,     -1.964879,      0.161293,     -2.044444,     -1.066551,
		     -0.603780,     -0.904404,      1.125143,     -0.482111,      1.277539,
		      2.947654,     -0.723198,     -0.170118,     -0.717778,     -0.732771,
		      0.189808,      2.326742,     -0.776714,     -2.726885,      2.284785,
		     -0.329355,     -1.494586,      0.968714,     -0.827033,     -0.634612,
		      2.596649,      1.597700,     -1.007206,     -1.455542,     -1.918643,
		     -5.165470,     -0.719388,      3.789602,      2.400857,     -0.237006,
		     -0.658779,      0.326008,      1.368967,     -1.556413,      0.144264,
		      0.830414,     -0.235340,     -0.244211,     -0.489979,     -1.046119,
		      0.742825,      0.246079,     -0.307370,     -2.091371,      3.010400,
		      0.626219,     -3.336906,      0.416300,      0.998150,      0.738465,
		      1.057049,      0.659656,      0.848773,     -0.387982,      0.538272,
		      1.156477,     -1.017630,     -2.007936,      0.462652,      1.933514,
		      3.637892,      3.739518,     -1.302243,     -2.926793,     -1.717430,
		      1.425961,      0.275574,      1.445846,      1.144000,     -0.142023,
		      2.130185,     -0.653640,     -0.486347,     -0.136582,      0.314259,
		      1.186298,      2.070738,      2.246298,      0.892811,     -2.306329,
		     -2.888070,     -2.755467,      1.502553,      3.671192,      1.189046,
		     -2.928097,      0.685716,     -1.122851,     -1.947373,      2.312743,
		      0.116371,     -5.365267,     -1.612231,      2.411220,      1.178392,
		     -0.355400,      0.769319,     -3.897429,     -1.956160,      2.146638,
		      2.788912,      0.744748,     -0.323394,     -1.812085,     -0.045488,
		     -1.042194,     -1.285391,      0.316288,     -0.096728,      1.016177,
		      1.645135,     -0.761190,     -1.211558,     -1.189678,      1.143859,
		     -1.713375,     -3.260170,      0.579223,      4.309883,     -0.678206,
		     -5.202570,     -1.472464,      0.810241,     -1.383456,      2.350742,
		      2.032794,     -0.625246,     -1.748378,      2.340257,      4.380720,
		      1.054186,     -2.710872,     -1.324724,     -0.739449,      2.995672,
		      4.115367,      0.886754,     -1.964443,     -2.896143,     -1.255585,
		     -0.544885,      0.362659,      1.604339,     -0.020973,      1.528955,
		     -1.167727,     -1.288704,     -0.842032,      0.687583,      1.171451,
		     -0.539647,     -1.182269,      2.579627,      2.559358,     -0.697777,
		      2.097451,     -3.671202,     -0.690086,      1.766170,      1.209990,
		      1.888868,      0.167236,     -0.401709,     -1.487845,     -1.169103,
		      4.042519,      2.989621,     -5.328869,     -4.340299,      0.560667,
		      0.079021,     -0.606769,     -1.056121,     -4.202276,     -3.004697,
		      0.867519,      3.075691,      2.213672,      1.185306,     -0.315910,
		     -0.641923,      0.102401,     -1.283484,     -0.649241,     -0.150149,
		      0.302088,      2.134238,     -0.119332,      0.683569,      1.244508,
		     -1.508089,     -0.773869,     -0.526174,      1.335755,     -2.839387,
		      2.960434,      3.164991,      0.208870,     -0.668557,     -0.793091,
		      0.415271,     -1.387486,      1.313098,      1.495484,     -2.099560,
		     -3.558095,      1.887678,      4.852383,      3.533206,      0.088098,
		      0.920515,     -0.350719,      2.718190,      3.105238,      0.800636,
		     -0.718647,     -2.344332,     -1.600632,     -0.765816,     -1.736523,
		      1.043840,      0.424114,      1.920567,     -1.129589,     -3.117378,
		     -3.155937,     -0.062109,     -0.501550,     -0.466384,     -1.118538,
		     -0.314624,     -0.053799,     -0.423378,      0.575831,      1.082665,
		     -1.109593,     -3.191567,      0.203486,      0.252732,     -1.609571,
		      0.543393,      1.594544,     -1.511786,     -1.510920,     -1.271296,
		      3.343354,      3.211082,     -4.051540,     -3.954515,     -0.590323,
		     -0.785931,      0.183159,     -0.019580,     -2.432529,     -2.171015,
		     -0.384406,      1.014240,      0.752290,      0.589180,      0.942787,
		     -0.055275,     -0.413903,     -2.250858,     -1.080362,      2.044212,
		      1.519837,      0.941874,     -1.753086,      1.687592,      1.929584,
		      2.415151,      2.065028,      0.244901,      2.824453,      3.560470,
		     -1.430880,     -0.087222,      2.088248,     -0.948796,     -1.143491,
		      1.725314,     -1.158872,      0.121745,      1.222182,      2.453249,
		      2.232419,     -2.753633,     -3.878884,      2.044904,      3.262212,
		      2.107321,      0.362029,      0.337824,     -0.644525,      2.417881,
		      2.153439,      0.253387,      0.296717,     -2.016637,     -0.949829,
		     -0.479534,     -1.297386,      0.163102,      1.227763,      1.456029,
		      0.989403,     -0.560265,     -2.258989,      0.155684,     -1.829009,
		     -0.463344,     -0.603552,     -1.911307,     -0.830013,     -1.551474,
		     -0.422576,      1.160066,      2.804536,      0.790163,     -2.214149,
		      1.485140,      3.433179,     -0.575139,     -1.253448,     -0.180214,
		      0.589427,     -2.049308,     -0.993347,      0.145110,      2.214260,
		      1.162797,     -1.794470,     -1.827423,     -1.440723,     -1.570214,
		      0.113445,     -1.532803,     -2.565426,     -0.932633,      1.154933,
		      2.741308,      0.421245,     -0.530603,      0.135157,     -0.040573,
		      1.121834,     -1.585852,     -2.223409,      1.359730,      1.763348,
		      0.733349,     -0.201736,     -1.619317,     -0.015903,      1.583619,
		      4.159755,      2.789803,      1.613148,      1.021657,     -0.653596,
		     -2.647834,     -2.140501,      2.543487,      1.129648,     -3.816871,
		     -0.607650,      2.295802,     -2.229951,     -2.320814,      0.569047,
		      1.640904,      2.363899,     -0.759102,     -3.074692,      1.446182,
		      1.272518,      1.990925,      1.631660,      0.566842,     -1.178097,
		      2.133421,      1.930873,     -1.906570,     -1.123516,     -1.122022,
		      0.309306,      1.318586,     -0.473624,     -0.981250,     -0.561395,
		      0.134398,      1.787807,      1.827885,     -0.841967,     -0.103917,
		      0.833031,      1.873297,      0.963781,     -2.248968,     -1.887297,
		     -2.510703,      0.720224,     -0.077793,     -0.170602,     -0.314466,
		      4.197871,     -1.738449,     -5.334570,      3.022298,      2.920617,
		     -1.880823,      1.183487,      1.589502,      0.857488,     -1.985207,
		     -1.339490,      0.808302,      2.823459,     -1.705522,     -0.690961,
		      0.107951,     -1.181069,     -1.858399,      0.563356,     -1.049019,
		     -2.937902,     -1.188366,      0.323311,      3.466168,      0.951950,
		     -2.075183,     -1.118426,      0.437288,      0.984524,     -0.333772,
		     -2.585438,      0.134070,      1.104058,      0.196049,      0.264009,
		     -2.892090,     -2.086633,     -1.209002,      1.580985,      2.056612,
		      1.177942,      0.374700,     -0.446470,     -2.126848,      1.820635,
		     -3.053739,     -0.801692,      4.624813,      0.076643,     -4.068937,
		      1.315998,      3.235801,     -0.499285,     -1.982321,     -0.108852,
		     -0.115978,      1.370124,      0.287452,     -0.548734,      1.311338,
		     -1.116870,      0.015638,      3.286784,      1.549071,     -0.895121,
		      1.162720,      1.790448,     -1.525567,     -2.338599,     -1.529635,
		      1.128852,      2.075807,      1.859932,      0.378693,      0.215643,
		     -0.704372,     -1.084005,      0.228118,     -0.256966,     -1.248980,
		      1.574465,      2.329270,      2.795156,     -1.520001,     -1.413232,
		     -0.132594,     -0.846173,      0.598591,      0.174907,     -2.289542,
		     -1.633793,     -1.128703,      4.316121,     -0.923438,     -4.504062,
		      4.090482,      1.846852,     -5.067710,     -0.707002,      1.162272,
		      1.417688,     -0.685297,     -1.639952,     -0.755595,      2.775544,
		     -2.441324,     -0.313292,      1.654933,     -1.436226,     -1.826437,
		      1.211713,      0.999958,     -0.481457,     -0.555200,     -0.335547,
		      2.255515,      2.147180,     -1.502091,     -2.467697,     -1.406568,
		     -0.564176,      0.808424,      0.319613,      0.598711,      0.747469,
		      0.941966,     -0.007001,     -1.457872,     -3.156496,     -1.840891,
		     -1.762193,      1.508823,      0.267702,      1.168929,     -0.595880,
		      0.189833,      0.434656,      1.050151,      2.214042,     -5.171300,
		     -1.026256,      4.988631,     -0.761227,     -4.264016,      1.101746,
		      2.341272,      0.583762,     -1.768594,      0.128339,     -0.085824,
		      0.393913,      0.373152,      1.432764,      2.445923,     -0.901248,
		     -1.907981,      1.336159,      1.075724,     -0.502797,     -0.030708,
		      0.828885,     -1.071608,     -3.009272,     -1.988424,     -0.251426,
		      1.392914,      2.218488,      0.857678,      0.612469,      0.764476,
		      0.089409,      0.916530,     -0.384751,     -1.802930,     -1.269306,
		      0.890007,      2.949885,      0.585449,     -0.136719,     -0.596926,
		      0.046143,      0.592483,     -1.711453,     -0.409290,     -1.145208,
		     -0.047177,     -1.334930,      0.438571,      2.388623,     -2.051092,
		     -3.573062,      4.431238,      4.358517,     -3.075855,     -0.834106,
		      0.809997,      1.713122,      1.058381,     -0.514321,     -1.815207,
		      0.234498,     -2.871969,     -0.630769,      2.496929,      0.072498,
		     -2.915183,     -1.185986,      0.272611,      0.469057,      2.049565,
		      1.719089,      1.892113,      1.729697,      0.337334,     -0.587080,
		      0.027378,     -2.092993,     -2.466450,     -1.145670,     -1.759088,
		     -0.134828,      1.609032,      2.061549,     -0.599585,     -1.877970,
		      0.571347,      0.221115,      0.561219,     -0.553102,     -0.634308,
		      1.554995,      0.990007,      1.035671,      0.151967,     -1.009824,
		      2.390761,      2.631641,     -4.642401,      0.019152,      4.310099,
		     -1.264359,     -4.545002,      1.146014,      1.334625,      0.082282,
		     -1.968003,     -0.029522,     -0.028174,      0.217995,     -0.447907,
		      2.585106,      1.505176,     -1.288811,     -0.998516,      1.539932,
		      1.895361,      1.129275,     -0.890257,     -0.770324,     -1.109913,
		     -1.965354,     -1.694297,      0.449513,     -1.085898,      1.533961,
		      1.426718,      0.211817,      2.015871,     -0.028333,      0.774593,
		      1.660913,      0.275377,     -0.519325,     -0.575096,     -1.829886,
		     -1.007835,      0.397188,      1.508023,     -1.554446,      0.740577,
		      0.979554,     -0.599050,     -0.298189,      0.082164,      1.545406,
		     -1.670317,     -2.531075,      3.850698,      2.043261,     -2.596099,
		     -2.785397,      1.534036,      2.413332,     -2.305802,     -1.606829,
		      0.828036,     -0.157058,     -0.783329,      0.987251,      1.187701,
		     -0.623927,     -1.762911,      0.456239,      2.323478,      1.256912,
		     -0.860856,     -1.656997,     -1.144833,     -1.586015,     -0.080807,
		      0.651388,      0.255452,      0.530318,      0.135195,     -0.038227,
		      1.008013,      0.317197,      0.008016,      0.515607,     -1.477514,
		     -1.999209,     -2.968166,     -0.482732,      0.789192,      0.707273,
		      0.032065,      0.132564,      1.649917,      0.639126,     -1.921654,
		     -2.207546,     -1.807345,      2.685903,      0.917181,     -1.334554,
		     -2.509516,     -0.608577,      2.512206,     -0.211496,     -4.886073,
		      1.328145,      4.535335,      0.215027,     -3.049211,      1.632703,
		      1.869039,     -0.632191,     -1.558550,      0.703921,      0.581601,
		     -1.691167,     -1.325888,      3.864583,      2.030758,     -1.866768,
		     -1.651025,     -1.039831,      0.014437,      2.834855,      2.517583,
		      0.841433,     -0.124511,     -0.073452,     -0.471639,      0.645605,
		      0.547905,      0.333830,      1.668394,     -0.580793,      0.260409,
		      0.620561,      0.204913,      2.233179,      0.858115,     -0.419518,
		     -0.776959,     -0.188723,     -0.343805,     -2.308993,     -0.074791,
		      0.768178,      1.206115,      0.961471,     -2.285667,      1.112157,
		      2.042154,      1.661797,      2.578404,      3.087644,     -3.107618,
		     -3.047875,      5.911929,      1.058098,     -2.582541,      0.506678,
		      2.144561,      1.261429,     -1.448794,     -0.901534,      2.606749,
		      0.612279,     -1.830119,     -0.712764,      1.391209,     -2.993952,
		     -2.989130,      0.625826,      2.326012,      2.817704,      2.186137,
		     -0.377080,     -0.309884,     -0.875440,      0.636154,      0.303723,
		     -0.127293,     -1.354585,     -1.341140,     -2.292902,     -1.324441,
		      0.521742,     -0.963484,      0.071779,     -0.290808,     -0.171292,
		     -0.968917,     -2.457228,      0.314039,      0.600142,     -0.193558,
		      1.231187,      0.779538,      0.155424,     -1.220227,     -0.741933,
		      0.360906,     -2.178832,     -0.828846,     -1.981996,     -0.481411,
		      0.408918,     -3.798835,      1.432543,      4.106032,     -3.474001,
		     -4.730904,      1.088731,      1.980761,     -0.673853,     -2.808966,
		      1.105947,      1.515788,     -2.560065,     -2.625160,      0.629205,
		      2.281370,      0.010265,     -0.556234,      4.982731,      1.608733,
		     -1.725892,     -1.014273,     -2.203540,     -3.129956,     -0.925313,
		      1.505963,      0.173096,     -0.660642,      1.783644,      1.717829,
		      0.648156,      1.101556,      0.769323,      2.093789,     -0.554550,
		     -0.080835,      0.347583,      0.461918,      0.212719,      1.850551,
		      1.041300,     -0.480570,     -1.289715,     -1.141161,     -1.376338,
		      0.657647,     -1.474130,     -0.482403,      2.173325,     -0.749233,
		     -2.697470,      0.772082,      0.165402,      1.413655,     -3.508218,
		      4.015101,      0.876776,     -4.682697,     -1.707998,      6.512475,
		     -0.229960,     -3.771762,      0.492983,      2.104730,      0.862826,
		      0.052177,      0.258120,      1.704846,      0.750835,     -1.038748,
		     -0.193496,      2.103678,     -3.846457,     -4.165763,     -0.492451,
		      0.109741,      0.817921,      3.072089,      1.339764,     -0.393175,
		     -0.169399,      1.513880,      0.479652,     -0.899244,     -0.618125,
		     -1.139175,     -0.985906,     -3.210474,      0.431696,     -0.482326,
		     -1.131793,     -0.366762,     -0.559300,      1.016362,     -0.881476,
		     -0.095063,      0.115824,     -1.522915,     -0.452041,     -1.933472,
		      0.132028,     -0.323785,     -1.084043,      0.201630,      0.895706,
		      0.227965,     -2.025568,     -1.978109,     -1.616660,      1.963898,
		     -0.822060,     -2.105187,      2.875593,      5.495478,     -3.902974,
		     -3.117334,      2.042804,      1.608127,     -0.729830,     -1.432975,
		     -0.208495,      1.940503,     -0.880158,     -2.212662,     -0.490810,
		      0.277744,     -0.602801,      0.452412,      6.287104,      2.469557,
		     -0.412209,      1.389423,     -0.345160,     -1.630176,     -1.489557,
		      1.722381,     -0.646833,     -3.051651,      0.030905,      2.503572,
		     -0.062207,     -0.243566,      0.652087,      1.635907,      0.726790,
		     -0.105967,     -0.845816,      1.339916,     -1.126765,     -1.301890,
		     -0.733542,      0.475996,      1.327536,      1.660038,      0.497085,
		      0.783626,      0.763870,     -1.230053,      0.068945,      1.663241,
		     -2.484545,      1.078927,     -0.951610,     -0.372145,     -0.454852,
		     -0.632675,     -2.288670,      3.687368,     -0.198668,     -5.927242,
		     -0.207725,      6.516288,      0.453222,     -2.239153,      0.148686,
		     -0.129285,     -1.071949,      0.005394,      0.151511,      2.109923,
		      0.471794,     -0.017314,      0.043581,      2.331690,     -2.795502,
		     -3.046525,     -1.073707,     -1.384434,     -1.887936,      0.622302,
		      1.055839,     -1.984345,     -1.359566,      1.798204,      1.427880,
		     -1.090083,      0.489701,      0.124026,      1.094489,     -3.249987,
		     -0.209316,      0.269852,      0.361928,     -0.743297,     -0.081552,
		      1.029942,      1.589278,     -0.032598,     -1.816899,     -2.342725,
		     -0.146166,     -1.466471,     -1.834468,     -2.055664,     -0.371491,
		      0.138252,      0.637746,     -0.952874,      2.097055,     -0.164459,
		      0.705029,     -0.696943,      1.256386,      3.088439,     -3.577118,
		     -1.692399,      3.092616,      3.951465,     -4.486999,     -2.182096,
		      1.080497,      0.627490,      0.046487,      0.231281,     -1.166825,
		     -0.412289,     -0.089938,     -0.768434,      0.104395,     -1.151841,
		     -2.018065,      0.352620,      4.274510,      1.467311,     -0.793779,
		      2.783782,      1.226023,     -0.148919,     -0.093067,      2.805867,
		      2.430425,     -3.048417,     -2.012024,      0.229451,     -0.147508,
		     -1.865904,      0.281854,      0.443592,     -0.344731,      0.258943,
		      0.073881,      1.555099,      0.217670,      0.435509,     -1.406570,
		     -0.864052,      0.592358,      1.913040,      3.152552,      1.470699,
		      1.589881,     -0.249835,     -0.102573,      0.700700,      0.658278,
		     -0.469155,      0.582003,      1.683666,      0.563902,      1.552997,
		      0.087787,     -0.405748,     -0.095622,      3.050966,      0.007249,
		     -4.941522,      0.215413,      2.816454,     -0.361627,      0.142080,
		      1.083171,      0.240404,     -0.023663,      1.088174,     -0.288006,
		      1.452618,      0.795009,      0.657787,     -0.095924,     -0.553436,
		     -2.131621,     -1.417592,      1.368476,     -0.573941,     -2.681474,
		     -0.963554,      0.874443,     -2.026583,     -1.489189,      1.536323,
		      2.106384,     -0.952131,     -0.213374,      0.400293,      2.671351,
		     -0.562243,     -0.847372,      0.364828,      0.272194,      0.187339,
		     -1.146747,     -0.641003,      1.040274,      0.043190,      0.071030,
		     -1.484233,     -0.258077,     -0.220247,     -0.204617,      0.454781,
		     -1.555472,     -0.351910,     -1.325329,     -1.721331,      0.313129,
		     -1.281677,      0.496445,      0.773745,     -0.050335,     -2.264358,
		      0.171252,      3.323539,     -3.196922,     -1.400396,      3.249574,
		      2.776122,     -2.668014,      0.018086,      0.157184,      0.367017,
		     -0.290015,      0.353407,     -1.327301,     -0.524851,     -1.476516,
		     -1.343423,      1.003429,      1.242123,     -0.422946,      0.003078,
		      1.319395,      0.548197,     -1.648911,      2.424936,      3.262322,
		      0.764330,     -0.050013,      0.211280,      1.799768,     -0.027658,
		      0.078415,      0.177660,     -0.814186,     -3.533176,     -0.094547,
		      0.396315,     -0.225777,     -0.931346,      0.220257,     -0.264439,
		      1.420913,      1.729181,      1.033558,     -1.034693,      0.575503,
		      0.256487,      1.778919,      0.522861,      1.382858,      0.936995,
		      1.183626,      0.653208,      1.578049,      0.589571,      2.817945,
		     -0.625029,      0.065326,      0.983047,      2.137950,      1.298595,
		      0.609924,     -2.581142,      0.137708,      2.309954,      0.118255,
		     -3.124229,      1.647073,      0.897687,     -1.185493,     -0.632732,
		     -0.459028,      0.777238,      0.832047,      0.980342,     -0.733143,
		      1.406373,     -0.225193,      0.743064,      0.953800,     -1.441899,
		     -1.791001,     -0.568790,      0.453501,     -0.389026,     -2.048911,
		     -1.753680,     -0.326670,     -1.519033,     -2.509372,     -0.444825,
		      1.428286,      1.097968,      1.276181,      1.544568,      1.681492,
		      1.401752,     -0.544687,      0.086425,      0.509760,     -0.041659,
		     -0.327029,     -2.854862,     -1.495991,     -0.677688,     -0.945840,
		     -2.012842,     -1.420231,     -0.492184,      0.294760,      1.203778,
		     -1.604747,     -1.700601,     -1.802716,     -1.667852,     -3.705841,
		     -2.474740,     -1.363578,      0.755670,     -0.680222,      2.655643,
		      0.829520,     -1.860107,     -0.281780,      3.071438,     -3.430340,
		     -1.750974,      2.985738,      0.620687,     -2.333195,     -0.038923,
		      0.120274,      1.863314,      0.259675,     -0.979142,     -0.541897,
		      1.697100,     -1.379099,     -2.473864,     -0.631575,     -0.828298,
		      0.614629,      0.857664,      0.159086,      0.187906,     -2.712889,
		     -0.651317,      1.562800,      2.390679,      4.177986,      1.386878,
		      0.418460,     -1.102910,     -0.985853,     -0.440035,      0.924448,
		     -3.102008,     -1.687380,     -0.433703,     -0.289478,     -0.504236,
		     -0.475750,     -0.633202,      1.666526,      2.907832,      2.993153,
		     -0.287615,      3.010239,      1.309633,      0.171143,     -1.437915,
		      0.249559,      0.783478,      1.406657,      2.366246,      1.783425,
		      2.369642,      2.149455,      1.546645,     -0.812080,     -1.320821,
		     -2.145164,     -0.545789,     -2.204432,      2.357396,      2.386679,
		     -3.058818,      1.038767,      2.817918,     -1.581859,     -2.055333,
		      3.477288,      1.575142,      0.765885,     -0.616978,     -2.192254,
		      0.914807,      1.472126,      0.028920,      0.663711,      2.909209,
		      0.448742,     -0.181198,      1.818982,     -0.770387,      0.198698,
		      1.347094,      1.392369,      0.513723,     -0.087486,     -1.053159,
		     -2.448987,     -1.430413,     -1.655305,     -0.566790,     -0.455869,
		      0.164335,      0.359510,      2.094125,      1.069226,      1.690338,
		      0.435669,      0.984692,      0.230151,      0.855654,     -0.542471,
		     -1.490710,     -3.081318,     -1.176411,     -1.644847,     -1.732003,
		     -1.346375,      0.315928,      1.699153,      1.259845,     -2.115880,
		     -4.417871,     -1.229085,     -0.138797,     -1.343532,     -2.654096,
		     -1.873141,     -0.360605,      1.403207,     -1.296205,      0.704210,
		      1.804840,      0.481106,     -1.955711,      0.541648,      3.321952,
		     -3.872247,     -1.694437,      2.495473,     -1.292119,     -1.211234,
		     -0.448224,     -0.806163,      2.037522,      1.141067,     -1.274724,
		     -1.158502,      1.712859,     -1.827111,     -1.118660,      0.992534,
		     -1.615775,     -0.978392,     -0.027949,     -1.797639,      1.501993,
		      0.477545,     -0.970303,     -1.384734,     -1.259955,      2.542037,
		      2.488612,      1.684982,      0.255702,     -1.316418,     -1.479059,
		      0.433830,     -0.845200,     -0.952537,     -0.638968,     -0.215959,
		     -1.766671,     -0.764756,     -1.343418,     -0.091246,      2.107888,
		      0.873421,     -1.092874,      0.483933,      0.734548,      2.186937,
		      1.350096,     -0.777676,      0.220680,      0.262900,      0.388794,
		     -0.061405,      1.949941,      2.989785,      1.370963,      0.588357,
		      1.183133,      0.574872,      0.848068,     -2.822877,     -1.092402,
		     -2.470746,      1.431868,      0.434395,     -5.030089,      2.220976,
		      4.015881,     -2.551897,     -2.985457,      1.803786,      0.097005,
		      0.932771,     -0.181972,     -1.956596,      0.855388,      1.681350,
		     -2.199539,      0.039252,      3.355517,     -0.718009,     -1.827042,
		      0.622320,     -0.263751,      0.039497,      1.315264,     -0.686798,
		     -1.379706,      0.745885,      1.727486,     -1.633782,     -1.670281,
		     -2.227169,     -1.585473,     -0.605615,      0.241769,     -0.713982,
		      1.372298,      0.162679,      1.786787,      1.083714,      0.537949,
		     -0.442863,      0.282865,      0.923423,     -0.292534,     -1.911607,
		     -0.782534,     -0.375160,     -0.781965,     -0.447521,     -0.299040,
		      0.898033,      2.119619,      0.182532,     -1.666292,     -0.060796,
		     -0.706540,     -0.737133,     -0.435314,      0.314990,     -0.289497,
		      1.060416,     -0.360249,      0.958515,     -0.127684,     -0.767239,
		      2.901972,     -0.840772,     -1.332031,      2.645796,      3.518660,
		     -4.075722,     -0.537519,      4.524831,     -1.081620,     -0.050278,
		     -0.649845,     -1.343881,      0.764083,      0.911638,     -1.418519,
		     -0.127885,      1.252627,     -2.743527,     -1.133457,      2.897284,
		      1.192695,     -0.274680,     -0.489101,     -3.835099,      0.399352,
		      3.549934,      2.158930,     -0.164230,     -2.146294,     -0.168198,
		      1.084890,      1.624234,      2.751029,      0.439070,     -1.550271,
		     -0.923009,     -2.098192,     -1.732447,     -0.211774,      2.127583,
		      1.585133,     -1.405243,     -1.860908,     -0.219821,      2.404505,
		      2.050532,     -0.830234,     -1.477206,     -1.972697,     -2.499574,
		      0.686800,      0.526980,     -0.856695,      0.808084,      0.795230,
		     -0.908007,     -0.612691,      1.006142,     -1.850313,     -2.365489,
		      0.674826,      1.481068,      0.563535,      3.238139,     -0.927729,
		      0.246155,     -0.545051,      0.774293,      2.837843,     -0.009148,
		     -6.032522,      1.249112,      4.347929,     -0.872359,     -1.174510,
		      1.136094,     -0.119182,      0.344994,      0.492178,     -1.666851,
		     -0.640973,      1.212047,     -1.644734,      0.317949,      3.324061,
		      0.201154,     -2.094991,      0.576614,      0.986560,      1.297083,
		      0.352290,     -1.480268,     -3.197743,     -1.397113,      2.742639,
		     -0.079558,      0.242331,      0.084090,     -1.633034,     -0.909309,
		      1.057714,     -0.069582,      1.784653,     -0.537815,      1.175992,
		     -0.172120,      0.099920,      0.036964,      0.648039,      1.104265,
		     -1.589259,     -1.350902,      0.160536,      1.790097,      2.399981,
		      2.075210,     -0.260517,     -1.357195,     -0.168390,      1.466109,
		      0.983418,      1.465794,      1.554406,      1.561890,      1.316845,
		      2.067709,     -0.433308,      0.833495,      0.193074,      1.141685,
		     -1.411233,     -0.434046,     -0.634594,      0.965304,      0.269959,
		     -3.206397,     -1.742873,      3.399729,      3.182681,     -3.595778,
		     -2.866545,      1.333623,     -1.834940,      0.621525,      0.498195,
		     -0.872174,     -0.436443,      1.227577,      0.274739,      0.502202,
		      2.107678,     -2.204560,     -3.534911,     -0.933111,      0.485450,
		     -0.623762,     -0.030254,     -2.386457,     -2.618123,      2.298331,
		      2.420865,      0.687388,     -0.118678,     -1.802215,     -0.870444,
		     -1.002169,      0.603682,     -0.320472,     -0.495027,      0.603457,
		     -0.410250,     -3.242723,     -2.391208,      0.838765,      1.425187,
		      0.525414,     -0.704745,     -1.466642,      2.171294,      0.729556,
		     -0.331627,     -1.445619,     -1.786143,     -1.340599,     -0.289341,
		      0.097142,     -0.444653,     -0.768169,      0.606062,     -0.581760,
		     -1.151161,     -1.146577,      0.179327,     -1.349779,     -1.694363,
		     -0.962239,     -0.391153,      1.263587,      0.642091,     -1.254959,
		      1.310817,     -1.254940,     -1.833543,      1.196181,      2.369314,
		      0.780098,     -4.126645,     -1.289278,      1.844329,      0.774342,
		      0.876474,      0.533784,      0.535525,      0.403712,      1.007760,
		     -0.310653,     -0.804222,     -0.035456,     -1.749926,     -0.118245,
		      1.724436,      0.342248,     -1.317512,     -0.137091,      2.625815,
		      2.989663,      1.432530,     -0.380891,     -1.595137,     -3.148590,
		      0.286648,      0.105372,     -0.249235,      1.705566,      0.640039,
		     -0.726084,      1.860515,     -1.193855,      0.292251,      1.635994,
		      3.195169,      0.096862,     -1.351538,      0.364673,      1.120290,
		      1.783211,     -1.067509,     -1.505143,     -1.692159,      0.444170,
		      2.009082,      2.711946,      0.768571,     -1.525080,     -0.334041,
		      0.364485,     -1.594415,      0.479254,      0.768120,      0.355129,
		      1.160786,     -0.387643,      0.623341,      2.746073,      0.272594,
		     -0.687155,      0.771349,      1.353096,     -0.999497,     -2.187590,
		     -0.616441,      3.677754,      1.484079,     -1.940724,     -2.110138,
		      0.876813,      2.738637,      1.424712,     -0.002962,      0.718817,
		     -1.218599,     -0.953636,     -0.408687,     -0.099096,     -1.092506,
		     -0.236931,     -1.018541,     -0.696828,      2.392456,      2.758271,
		      0.950443,      1.697504,      0.290318,     -2.750496,     -3.081589,
		      0.201758,     -0.265829,      0.463471,      1.087411,      0.124945,
		      1.256050,     -0.886183,     -1.660205,     -0.550534,      1.855711,
		     -0.893579,     -1.684460,     -0.104122,      1.774692,     -3.227487,
		     -1.269094,      0.086004,      0.915595,     -0.259443,     -0.910295,
		     -0.991054,      1.153809,      2.522253,     -0.104299,     -1.745507,
		     -1.313151,     -0.987459,     -0.272335,      0.514549,     -0.331002,
		      0.113898,     -0.036889,      0.662242,     -0.544235,      0.261064,
		      0.464531,      0.875799,      0.715832,      0.544967,     -2.210915,
		     -0.251814,      0.283995,      0.467754,      1.110119,      0.148165,
		      2.514879,     -2.032379,     -1.715208,      1.502937,      1.181239,
		      0.657629,     -0.483833,     -1.575218,     -0.679342,      0.140708,
		      0.183666,     -1.455085,      0.358149,      0.201974,     -0.366133,
		      0.815868,      0.391516,      1.431768,     -0.712861,     -1.866378,
		     -1.404120,     -0.445307,     -1.014434,     -1.146406,      1.224224,
		      1.202060,     -0.649524,     -2.257626,      0.494811,     -0.510244,
		     -0.552769,      1.260049,     -0.351613,      0.105809,     -0.029226,
		      0.112653,      2.242960,      1.218828,     -1.321676,     -0.446462,
		      2.206371,      1.023437,     -1.188255,      0.227361,      2.063509,
		      1.806321,     -0.707742,     -0.513850,     -0.525719,     -0.518866,
		      0.769731,      0.398753,      1.611058,      0.758351,      0.839052,
		      0.606487,      1.581262,     -1.436520,     -0.034374,      0.381281,
		     -0.360712,     -0.814558,     -1.327442,      0.908232,      1.993155,
		     -1.544550,      0.526338,     -1.438585,      0.982846,     -0.622517,
		      1.101679,     -1.205232,     -2.249231,      1.279950,     -0.134465,
		     -0.016965,      0.312660,     -0.619590,     -1.298361,     -0.048945,
		     -0.021202,      0.529756,      2.513938,      0.679333,     -2.257229,
		     -0.621570,      1.117933,      0.841871,     -0.548882,     -0.812815,
		     -1.056924,     -0.276822,      0.137391,      2.561031,      2.739474,
		      0.282709,     -3.641073,     -1.812611,      2.111822,      1.194578,
		      0.392675,     -0.980743,      0.803113,     -0.214523,     -1.410253,
		     -2.096272,      0.739366,     -0.144492,     -0.649595,     -0.678591,
		      2.646514,     -1.753284,     -0.982652,      0.292694,      0.375017,
		     -1.029871,     -1.258207,     -0.438360,     -1.825285,      1.131959,
		     -0.286297,     -1.760360,     -1.045141,     -0.820976,     -0.628105,
		     -1.148960,     -1.137895,     -0.188330,      0.268035,      2.176508,
		     -1.254018,     -1.411300,      1.481923,      2.537249,     -0.208139,
		      1.116840,     -1.440020,      0.461758,      1.306618,     -1.847360,
		      0.137601,     -0.226725,     -1.098311,     -0.675222,      3.337636,
		     -1.279478,     -1.594792,      0.894487,      0.450636,     -1.776329,
		      0.619851,      0.514491,     -0.852275,      1.568643,      0.174009,
		     -2.897679,      1.539904,      1.906494,     -1.415623,     -1.043784,
		      0.554097,      1.466809,      0.785840,      0.831390,      0.721075,
		     -2.317198,     -1.116829,     -0.410156,      1.521772,      1.839098,
		      1.329773,     -3.200456,     -1.210372,      0.252250,     -1.787306,
		      1.069483,      1.224057,      0.292932,     -1.477999,     -0.633634,
		     -0.514195,      2.504889,      0.243486,      0.048850,     -1.830322,
		      1.053257,     -1.772111,      0.154033,      0.922425,      0.813731,
		      0.536918,      0.455592,     -0.643974,      1.137770,      1.602987,
		     -0.160028,     -0.444813,      2.019809,      1.760695,     -1.266011,
		      0.218336,      0.257811,      0.497013,     -0.032242,      2.012354,
		     -0.107720,     -0.046638,      1.159056,     -0.208783,     -0.996039,
		     -0.323428,      0.975845,      0.137780,      0.354419,     -0.367572,
		     -1.850345,      2.105348,      1.453217,     -0.399181,      0.743828,
		     -1.071799,     -1.302782,      1.691613,      1.484787,      0.280600,
		     -0.618698,     -2.975901,     -2.442844,      1.817702,      2.218256,
		     -1.177684,     -1.869180,      0.032995,     -1.023220,     -1.457144,
		      1.924901,      0.725811,     -0.185880,     -1.098402,      0.733336,
		      0.637295,      1.228268,     -0.598919,     -3.819690,      0.023351,
		      0.048998,     -0.292738,     -0.507557,      0.191443,      1.049602,
		      0.539656,     -0.669023,     -1.203075,     -1.156160,     -0.160502,
		      0.286575,      0.855252,      0.285720,     -0.851713,      0.273980,
		      0.767787,     -0.092048,     -1.634783,      2.105772,     -0.589531,
		     -0.993022,     -0.424781,     -1.734657,     -0.687033,      2.209644,
		     -1.377451,     -3.230976,     -0.222625,     -1.274878,      0.576615,
		      0.714751,      1.251327,     -0.797333,     -1.553091,     -0.160876,
		     -0.377436,      1.100920,     -0.288407,     -1.260787,     -0.566214,
		      0.517897,     -2.494565,      1.193309,      1.264506,      1.642322,
		      0.629419,     -3.853021,      1.757202,      0.827460,     -1.022558,
		      1.726595,      1.744901,     -1.268394,     -0.208015,      1.251544,
		      0.562705,      1.277573,      0.223183,     -2.541424,     -0.855633,
		      1.513039,     -0.307752,     -0.535969,      1.202947,     -1.091285,
		     -2.134786,      0.182664,      3.114365,     -0.390553,     -1.606049,
		     -1.041093,     -0.137753,      0.503250,      3.173961,      0.649797,
		     -0.962940,      1.577633,     -1.672106,      0.537224,      1.502346,
		      1.915566,      0.018634,     -0.174190,     -2.365125,     -1.300515,
		      0.486668,      3.007256,     -1.525952,     -0.340325,     -2.095750,
		     -0.770542,      1.305704,     -0.989502,     -1.790121,      0.490721,
		      1.268272,     -0.182746,      0.524420,     -0.090034,     -1.187718,
		      1.976517,      1.217183,      0.940773,      0.984912,     -1.793463,
		     -0.059906,      0.756249,      0.511814,      2.160646,      0.840257,
		     -0.552894,      2.602379,      0.406877,      0.528542,     -0.738372,
		      0.202535,      0.221153,     -0.178065,      1.615100,     -0.680646,
		     -3.569349,      0.774131,      1.222816,      0.146647,      1.562343,
		     -0.282810,     -2.667187,     -1.119527,     -0.258899,      1.755693,
		      2.622376,     -0.467488,     -1.172349,     -0.317123,     -0.254538,
		      0.847909,      1.485939,      1.656491,     -1.941243,     -4.096080,
		      1.639180,      2.373965,     -0.082398,     -0.375301,      0.522486,
		     -0.747221,      0.351899,      1.609491,     -2.724756,      0.087613,
		     -0.866847,     -0.940443,     -0.575636,     -2.258497,     -1.103184,
		     -0.341921,      2.043605,     -0.075763,      0.395271,     -1.179994,
		      0.873418,     -0.608485,      1.359291,      0.169044,      0.548080,
		     -1.424768,      1.023575,     -1.364938,      3.020664,      0.509793,
		     -1.685337,     -1.633050,     -0.920075,     -1.555562,      0.919663,
		      1.897802,     -1.601972,     -1.800198,     -1.453712,      0.877976,
		     -0.121562,     -0.298102,      0.971292,     -0.088250,     -1.744021,
		     -0.274428,     -0.450783,      0.989719,     -0.893820,      0.640050,
		      1.851299,     -1.173624,      0.791666,      2.283549,     -0.847193,
		     -1.262759,      3.104972,      2.456249,     -2.453324,     -0.685488,
		      0.534618,     -1.647844,      0.306953,      1.193249,     -0.161718,
		     -1.628748,     -1.688105,      0.350390,      0.250063,      0.049120,
		      0.744267,     -0.532070,     -1.963136,     -1.626250,      1.354962,
		      4.945609,      2.562040,     -1.828192,     -3.104787,      0.080929,
		      1.270437,      0.478119,      0.848763,     -1.278508,     -1.943056,
		      0.414512,      3.187168,      0.541786,      1.811829,      1.351621,
		      0.832467,     -0.609566,     -0.983911,     -0.840987,      1.541319,
		      1.451016,     -1.431113,     -1.719068,     -0.115421,      0.281839,
		      0.556843,      0.848528,     -2.419840,      2.082090,     -0.374972,
		     -0.371877,     -0.366428,      1.355411,      0.753570,      0.633992,
		      0.963739,     -1.978315,     -0.537284,      0.440960,      1.754425,
		     -0.382458,      0.401130,     -0.338740,      1.208397,     -0.052138,
		      0.336144,      0.007639,     -0.200429,     -0.336249,     -0.519922,
		     -1.716065,     -1.719462,     -0.309402,     -0.903188,      1.596534,
		     -0.411565,      1.323580,      2.945986,      0.871708,     -2.532273,
		      0.329020,     -0.186343,     -0.973967,      2.248345,      1.058916,
		     -0.064506,      0.070788,     -1.568883,     -0.168309,      1.035748,
		     -0.100538,      1.670979,      0.871837,     -2.534237,     -1.484217,
		      1.445421,      2.952059,     -1.161401,     -3.489017,     -0.635878,
		      1.864979,      0.965645,      0.876269,      1.411256,     -1.598980,
		     -0.905709,      2.984490,     -1.497684,     -2.172750,     -2.281340,
		     -1.433582,      0.249338,     -2.145166,     -0.612239,     -1.068481,
		      0.911382,      0.431975,      0.678113,     -0.935593,      0.669274,
		     -0.366708,      0.183337,      0.551013,      2.412428,     -0.083978,
		     -1.177047,     -1.574356,      1.663973,      0.931516,     -0.102714,
		     -2.231570,     -0.018744,     -2.296594,      0.218638,      2.390557,
		      0.680196,     -1.807332,     -1.459148,     -0.122865,     -1.019838,
		      1.709032,      2.057954,      0.114015,      1.065874,      0.985151,
		     -1.615506,      0.679604,      2.534132,      1.588962,      0.688908,
		     -0.178479,     -2.791768,      0.394664,      1.347592,      0.680121,
		     -4.193869,     -0.844367,      1.293618,      0.429056,      0.160103,
		      0.380473,     -0.105881,     -0.544171,     -0.350806,     -0.067175,
		      2.881236,      2.065214,     -1.267856,      0.480702,     -0.796664,
		     -3.025019,      2.328316,      2.157390,     -2.238518,     -4.710880,
		     -2.729289,      2.086520,      2.450985,      1.079571,     -0.299991,
		     -3.017012,     -2.396139,      1.416764,      1.726010,      0.820305,
		      1.059141,      0.028828,      1.255790,      0.068374,     -1.482470,
		      1.976759,      3.566887,      0.582079,     -1.935737,     -1.611456,
		     -2.156699,      0.742625,      0.520211,      0.854221,     -0.718152,
		     -0.893734,     -3.041379,      1.041088,     -0.692469,      3.233615,
		     -0.955887,      0.522054,      1.004514,      1.532699,      1.855240,
		      0.157217,      1.734336,     -0.616665,     -1.106136,      0.268661,
		      1.941069,      0.487792,      0.453771,     -0.006428,     -1.453941,
		      0.348470,     -1.501368,      0.857471,      2.868929,      1.717629,
		     -0.540270,     -4.004037,     -1.023591,      0.681501,      0.960855,
		      0.136782,      0.643382,     -0.498267,     -0.659594,     -0.280082,
		      1.248466,     -0.228332,     -0.465978,      0.097124,     -0.581025,
		     -1.015916,      1.242629,      0.323635,     -0.978320,     -0.045642,
		     -1.607819,      0.188034,     -0.238684,     -0.555792,      0.031372,
		      0.955584,     -0.473679,     -0.719803,      1.166150,      2.224015,
		      0.589429,     -1.102020,     -2.149769,     -0.072812,      2.065462,
		      2.416142,      2.587725,     -1.439327,     -2.032299,      1.689519,
		     -0.221682,     -1.367975,     -0.346132,      0.263908,     -0.342083,
		     -2.873431,     -0.640110,     -0.183130,      2.075400,      1.994671,
		      1.672003,     -0.462024,     -2.373310,     -0.395064,     -0.330049,
		      0.175453,      0.866144,      2.294330,     -2.872919,     -3.149788,
		     -1.509076,     -1.112001,      0.792551,     -1.586610,     -0.298080,
		     -1.286964,     -1.962488,      0.589959,      0.745868,     -2.194257,
		     -2.959121,      0.001001,     -1.912247,      0.217641,      1.147520,
		      1.267658,     -0.741271,     -1.015922,     -1.500101,     -3.645390,
		      0.585586,      1.494129,     -1.595987,     -2.377229,     -0.062946,
		      2.125046,     -0.898151,      0.476662,      1.170063,      2.313762,
		     -2.957898,      0.131315,      1.004607,     -0.861957,      0.883940,
		      1.226048,     -1.925146,     -0.771578,      0.368497,     -1.099772,
		      0.750797,      0.703832,     -0.982268,      0.438868,      1.437807,
		     -3.123072,     -0.824790,      2.885901,      1.402285,     -1.052160,
		     -1.364033,      0.897732,      0.792201,     -0.417963,      0.555435,
		     -1.677498,     -4.295737,     -0.395956,      1.028006,     -2.269011,
		      1.082376,      3.728602,      3.009645,      0.897187,     -1.990624,
		      0.813751,      1.378466,     -0.072386,     -0.787880,      0.417848,
		     -1.740236,     -0.378420,     -0.066107,      1.464703,      1.166622,
		      2.481566,     -0.625744,     -0.460355,     -0.158968,      4.614589,
		      1.536726,     -0.440621,      0.544881,      2.341740,     -1.554102,
		      1.125209,      1.999304,      2.030943,      0.860162,     -0.528809,
		      1.228466,      1.606493,      1.496979,      1.742597,      0.544761,
		      0.752776,     -1.253487,      1.799778,      2.769729,      1.092494,
		     -0.130885,     -0.488566,     -1.574797,     -0.398284,     -2.466729,
		      0.654237,      0.999373,     -1.255018,     -0.887642,     -0.173757,
		     -2.651642,      1.343649,      0.886062,     -0.239097,      1.132155,
		     -0.907219,     -0.865182,     -0.214517,      0.346793,      2.375494,
		      1.429971,     -0.070095,     -1.006296,      1.784695,     -0.878906,
		     -0.889603,     -0.414577,     -0.783065,     -1.481169,     -1.293897,
		     -0.806359,      1.217057,      0.563213,     -0.645975,     -1.032949,
		     -1.627622,      1.052316,      2.618137,      0.756696,      2.312792,
		      1.165028,      0.590152,     -2.194143,     -4.155924,     -1.932388,
		      2.065545,      0.144991,     -2.251803,     -0.647707,      0.371660,
		     -0.827602,      0.383056,      1.763479,      2.964745,     -1.906824,
		     -3.111755,     -2.719725,     -1.001066,     -2.185975,      3.269099,
		     -1.043612,     -2.263587,     -2.363080,     -3.263411,      0.155999,
		      1.589298,      2.133068,      0.400325,     -2.654379,     -1.334686,
		      1.623731,     -0.031727,     -4.430468,     -1.646404,      1.519662,
		      0.619712,      0.076389,      0.334650,     -0.365185,      1.634987,
		     -0.947743,      0.232394,     -0.069653,     -1.059162,      1.120504,
		      0.917898,     -2.899729,     -1.865044,      1.988143,      1.009341,
		     -0.489020,      1.269144,      0.052478,      3.412933,     -1.114878,
		     -1.236370,     -0.383506,     -1.048520,      1.458325,      2.749205,
		     -0.029333,     -1.710695,     -0.704087,     -0.293157,      1.406956,
		     -0.245772,      0.341039,      0.846864,      2.369577,     -0.859276,
		     -2.248125,      0.120093,      0.696459,      0.754466,      0.086608,
		     -0.186041,      1.366335,      0.347083,      0.235099,     -0.168096,
		     -2.252042,     -0.187214,     -0.021621,     -2.332224,     -1.447763,
		      3.100572,      3.628523,      1.419317,     -0.803828,      1.354522,
		      1.078800,     -0.901261,     -1.939706,      2.058584,      0.087854,
		     -0.889834,     -1.594083,     -0.010368,      1.524095,      1.854730,
		      2.321581,     -0.634989,     -0.864382,      2.741296,      2.691870,
		      1.466747,     -0.223482,      0.509572,     -3.866114,     -4.747236,
		     -0.128942,     -0.175718,      0.734841,     -0.639307,     -0.481316,
		      0.685025,     -0.398132,     -0.705996,      1.201511,      2.895867,
		      1.886091,     -1.661889,      1.476284,      0.266019,     -1.028916,
		      0.006851,     -1.878567,     -1.508887,      0.159964,     -0.832449,
		      1.506301,      0.419317,     -0.615988,     -5.824423,     -0.587723,
		      0.335945,     -3.090981,      0.637680,      0.705912,     -0.770669,
		      3.039147,     -0.312136,     -1.843387,      1.237850,     -1.375571,
		     -1.213806,      0.102355,     -1.345285,     -0.320402,      1.585365,
		     -0.629159,     -1.925618,      0.043363,      2.091505,     -0.546211,
		      0.083481,      0.185191,      0.040832,      0.474556,     -0.281265,
		     -0.273884,     -1.382313,      0.108209,      2.440743,     -2.063256,
		      0.774704,      2.926443,      2.835685,      0.445025,     -2.915530,
		     -4.014466,      1.280458,      0.998010,     -1.081235,      0.600179,
		      2.537549,     -1.326435,     -1.152597,     -0.944231,      3.005369,
		      2.016933,      0.025927,     -2.350027,     -1.839462,     -2.751325,
		      0.411039,     -1.784348,     -2.669085,     -0.394713,     -0.459369,
		     -1.676515,      1.237136,      3.306889,      2.357557,      0.528899,
		     -2.320636,      0.426481,      2.139107,     -0.346240,     -2.359762,
		     -0.294557,      1.110897,     -0.635428,      0.423994,     -0.455895,
		      0.357387,      1.176005,      0.473160,     -0.509994,      1.274676,
		      1.375148,      0.306766,      3.354532,      0.851707,     -0.935090,
		     -0.738333,      3.247842,      0.504142,     -2.501121,     -2.960958,
		      0.722835,      1.857444,      0.803481,     -0.527365,     -1.222927,
		     -0.726541,      0.051616,     -0.445909,     -0.027511,      2.026415,
		      1.159582,      0.670815,      0.593485,     -1.690140,     -1.048570,
		     -0.377359,      1.733259,     -0.029982,     -1.624677,     -0.361936,
		      0.344775,     -0.049148,      1.105117,      1.215491,     -0.776899,
		     -0.190696,     -0.283784,     -1.769663,      0.423791,     -0.346370,
		     -1.286226,     -2.006911,     -0.751022,      0.806607,      2.959465,
		      1.626105,     -0.307577,      0.004779,      0.327755,     -0.535554,
		     -3.258862,     -0.080387,      0.990350,      0.434118,      0.221600,
		     -1.317571,     -0.800482,     -0.024416,      2.257186,      0.975438,
		      0.900322,      2.022481,      0.598334,      0.757444,     -0.209503,
		      2.448468,     -0.763522,     -2.751514,     -2.122774,     -0.816224,
		     -0.462770,     -1.088974,     -0.210102,     -0.867639,     -0.655111,
		     -2.569866,     -1.032147,      2.330398,      2.162970,      2.466641,
		      1.492513,     -1.507033,     -1.106615,     -1.512540,     -0.787977,
		      0.191793,     -3.236239,     -2.656410,     -0.742868,      1.972120,
		     -2.846698,      1.464809,      1.219596,     -0.588341,      1.060158,
		      0.451547,     -3.263683,     -1.117561,      0.411589,     -1.111882,
		      1.357744,      0.488901,     -1.458510,      1.271991,      2.026283,
		     -0.811691,     -0.265137,      0.279165,      1.192863,      0.431216,
		     -0.150780,     -2.522250,     -2.264707,      1.742183,     -0.099606,
		      0.462451,     -0.238484,     -0.764819,     -1.229059,     -1.025506,
		      1.872825,      1.021028,      0.128806,      2.798750,     -0.993528,
		      0.764176,      2.987424,      1.609706,     -0.438580,      0.752639,
		     -1.527858,     -2.011327,     -1.458400,      0.097511,     -0.053636,
		      4.598814,      1.382964,      0.829476,     -2.946964,     -1.248207,
		     -0.276207,      2.276572,      1.999834,      1.201990,     -1.357696,
		     -0.200076,     -1.011896,     -1.794539,     -1.255673,     -0.282908,
		     -0.986449,      0.282297,      1.217688,      1.422711,      2.703478,
		     -0.456728,      0.627043,     -0.242101,      1.054363,     -0.360845,
		     -0.181936,      0.753903,     -1.672404,     -2.129212,     -1.717809,
		     -2.412790,      2.129417,     -0.131826,      1.586041,      1.305633,
		      1.422350,      0.249222,      1.929646,      2.633607,     -0.384656,
		     -2.950802,     -2.019809,      0.556922,     -0.387156,     -0.522413,
		     -0.713512,     -1.812097,      1.521239,      0.867841,      0.999913,
		     -0.712871,     -0.684465,     -0.048000,      1.447935,      0.655672,
		     -3.280350,     -2.432578,      0.589781,      0.845314,     -0.839371,
		     -0.499747,      0.070397,     -0.159770,      1.885567,     -1.035050,
		     -1.655589,     -0.145437,      1.534799,     -0.537080,     -0.338005,
		      2.026756,      0.205733,     -2.020313,      0.143777,     -1.453961,
		      1.657793,      0.371050,     -2.025605,     -2.287088,      0.022467,
		      0.527084,      1.324111,      1.045622,      1.760169,      0.535834,
		     -0.393357,     -0.820885,     -1.463547,     -1.757355,      0.612858,
		      0.140337,      2.030979,      0.530299,     -1.010497,     -2.593532,
		      0.494093,      0.024250,      1.475062,      2.903175,      0.374002,
		     -0.317654,     -0.205519,     -1.801748,      0.483916,      0.165161,
		     -0.669798,     -0.933397,     -0.654680,      0.283504,     -0.656915,
		     -0.162782,      0.115485,     -1.635133,      1.971315,      1.541390,
		     -0.030838,     -0.615584,      1.572508,      1.763036,     -0.575191,
		     -0.589474,      2.008380,     -1.425168,     -1.389522,     -2.648049,
		     -0.886997,      2.961540,      3.215195,      0.657139,      3.429989,
		      4.664264,      1.226013,     -0.193335,      3.197399,      1.438534,
		     -0.157369,     -0.976087,      0.098017,     -0.807369,     -0.886284,
		      1.215087,     -0.302368,     -2.244861,      0.769725,      0.932207,
		     -1.054027,      1.196728,      1.763684,     -0.255495,     -0.919759,
		     -1.117312,     -1.884578,      1.688767,      2.272148,      1.496288,
		     -0.478843,     -0.888957,     -1.960221,     -2.727026,      0.148688,
		      1.190503,      0.069313,     -1.298078,     -2.133104,     -0.641840,
		      3.626372,      2.050917,     -2.641157,     -0.899065,      2.678372,
		      0.904847,     -0.703260,      1.077036,     -1.381167,      0.853995,
		      1.451631,      2.448224,      1.024861,      0.125068,     -2.681639,
		     -0.854295,      1.457085,      3.632809,      1.841111,     -0.151720,
		     -1.317751,      0.491564,     -0.256791,     -0.237938,     -1.152317,
		      2.575578,      1.404211,     -0.656001,      0.636492,      0.805988,
		     -0.088422,      0.600232,     -0.732239,     -0.851867,     -2.126506,
		     -1.848086,     -1.348684,      0.055269,     -0.017246,     -1.357351,
		     -2.531266,     -1.511169,     -1.483974,     -1.333490,      0.793472,
		      1.701531,     -0.508178,      1.221556,     -3.544210,     -2.499520,
		     -1.378297,     -0.310646,     -3.274310,      0.441986,      1.508198,
		      0.124525,     -1.339261,     -0.861161,     -0.174254,     -0.596388,
		      1.663342,      0.473364,      1.009676,     -0.422237,     -1.590494,
		      2.133399,      1.911749,      0.079685,      1.420205,      0.870371,
		     -2.182073,     -1.320102,      0.862041,     -0.116894,      2.733016,
		      0.530159,     -1.140131,     -2.353506,     -0.097799,      1.529930,
		     -0.294079,      2.623570,      2.245283,     -1.913538,     -0.090223,
		     -1.603025,      0.750017,      1.421159,     -0.736175,     -2.128345,
		      0.510179,      1.985503,      0.651430,     -2.510776,      0.365913,
		     -0.510741,     -0.488178,     -0.548711,     -0.030832,     -3.456001,
		     -0.405278,     -1.555474,      1.774780,      2.340392,      2.447772,
		     -2.943685,     -2.939783,     -1.752799,     -0.536808,      1.250331,
		      0.741656,      1.528815,      2.326756,     -1.278567,     -2.809587,
		      2.100517,      2.345980,      3.227639,     -0.710206,      0.017963,
		      0.752498,     -0.124862,      0.139336,     -0.337745,      2.456815,
		      1.711359,      2.044065,      0.454371,      0.108668,     -0.370630,
		      2.749194,      2.943147,      1.835869,     -1.354864,     -2.243300,
		     -2.541554,     -0.692625,     -0.411749,      2.584973,      0.334142,
		      3.709170,     -0.292179,     -1.324758,     -0.594686,     -1.619896,
		     -0.286532,      0.935315,     -1.242413,      0.183849,     -0.456975,
		      0.484334,      0.424806,     -1.987916,      0.529891,      1.888870,
		     -1.726830,     -1.310107,     -0.419088,     -1.293389,      0.697601,
		      1.827154,     -0.332948,      0.278736,     -1.650863,     -2.105620,
		     -1.487476,      0.993868,      2.252746,      0.942870,     -0.124344,
		     -1.064685,     -3.236899,     -1.369415,      1.322640,      3.147397,
		      0.897680,     -0.250043,     -1.223585,      0.914458,      2.711041,
		     -0.700158,     -2.588973,      0.006117,      1.802272,      0.498053,
		      1.820800,      1.667135,      0.736853,      0.922027,      1.409543,
		      2.216364,      0.723088,     -0.222825,     -1.524031,     -1.280552,
		      0.749470,      3.013981,      2.379170,     -0.658713,     -0.130209,
		      0.287552,     -1.254232,     -3.323652,      1.555153,      2.665872,
		     -1.631919,     -1.684032,      0.807940,      1.801685,      0.224861,
		     -0.433325,      0.811867,      0.818796,     -2.963227,     -3.751369,
		     -3.905189,     -0.100595,      0.317394,      2.190455,     -2.144396,
		     -2.350371,     -2.015088,     -2.100358,      1.388993,      1.833015,
		      0.854395,     -2.031890,     -0.365477,      2.209740,      0.019061,
		     -1.911576,     -1.645252,      0.231066,      0.399562,     -0.249033,
		     -0.140846,      0.006842,      2.008110,      0.636745,     -1.631811,
		     -1.573101,     -0.876351,      2.792351,      2.600009,     -3.979184,
		     -3.405611,      2.628929,      1.782933,      1.084705,     -0.579874,
		     -1.157701,     -0.274162,      1.265856,     -0.653111,      1.577166,
		      3.060435,      1.802071,     -1.181067,     -1.312031,      0.526538,
		      0.298362,     -0.206700,      2.513625,      0.121365,     -1.062390,
		     -1.614132,     -0.780160,     -0.922423,     -0.392191,     -1.092833,
		      0.299991,      1.922018,      2.788428,     -0.612423,      0.021069,
		     -1.373717,     -0.584413,     -1.027402,      1.153473,     -0.849507,
		     -1.209298,     -2.216318,     -0.237274,     -0.368199,      4.222502,
		      1.371789,     -0.908539,     -2.220951,     -0.811032,     -1.037794,
		      0.610103,     -0.035907,      3.579728,      0.829022,     -1.101108,
		     -1.604309,      0.657361,     -0.973367,     -0.382482,     -2.242666,
		      0.709288,     -2.239737,     -1.724022,     -1.454210,      0.763389,
		      1.439467,     -0.955579,     -1.045788,     -0.797173,      1.524254,
		      0.041565,      0.192707,      3.623161,     -1.074928,     -3.954935,
		     -0.619449,      0.809475,     -2.022060,     -2.080611,     -0.177676,
		     -2.896262,     -2.889765,     -1.526700,      0.330796,     -3.715897,
	};
	static const std::vector<double> h = {
		      0.000000,     -0.305520,      0.000000,      0.110437,      0.801207,
		      0.000000,      0.234959,     -0.728060,     -0.335525,      0.000000,
		     -1.423016,     -1.209285,      0.197862,      0.565278,      0.000000,
		      1.884548,      0.938037,     -1.120329,     -1.147798,     -1.138132,
		      0.000000,     -0.846591,     -0.085946,      0.125648,      0.694106,
		      1.141214,      0.258965,      0.000000,     -0.592913,      0.500371,
		      0.862463,     -0.307171,      0.131982,     -0.531486,     -0.814321,
		      0.000000,     -0.388388,     -2.001754,     -0.985415,     -0.788484,
		     -0.213468,      1.949134,      1.268737,     -0.342146,      0.000000,
		      0.953344,      1.845356,      0.893622,      0.187806,     -0.115988,
		     -0.301455,     -0.790592,      0.055659,     -0.992662,      0.000000,
		     -0.187313,     -0.774538,     -1.234950,     -0.026695,     -0.542357,
		     -1.585257,      0.384289,      0.784358,      0.614235,      0.835592,
		      0.000000,     -0.430500,     -0.274749,      0.477536,      0.349057,
		      0.967017,      1.466748,      0.416803,     -0.508151,     -0.053702,
		     -1.781938,     -1.539884,      0.000000,      0.286413,     -0.724002,
		     -0.387448,     -0.835624,     -0.880065,     -0.323018,     -0.836597,
		     -0.541674,     -0.031405,      1.928935,      1.148179,      1.388581,
		      0.000000,     -0.053766,      1.903842,      0.534345,     -0.492663,
		     -0.657703,      0.102624,      0.858852,      0.116114,      0.965188,
		      0.217234,     -1.829912,     -2.151829,     -0.983737,      0.000000,
		      0.460676,     -0.575295,     -1.240947,      1.333377,      1.132848,
		     -0.305256,     -0.614269,      0.133400,      0.042580,      0.207928,
		      2.622271,      0.674392,      0.274435,      0.026913,      0.000000,
		     -1.973789,     -1.431987,      1.457129,      0.460905,     -0.371756,
		     -0.210369,      0.991415,      0.708865,     -0.181466,      0.444773,
		      0.208823,      0.280584,     -0.364242,     -0.697438,      0.229532,
		      0.000000,      1.292699,      0.924328,     -0.492892,     -1.966455,
		      0.518397,      1.928849,     -0.854856,     -1.769973,     -1.397506,
		     -2.083590,     -2.283924,      0.529300,      0.414575,     -0.135592,
		     -0.064760,     -2.212263,      0.000000,      0.868202,     -0.263370,
		     -1.960188,      0.627191,     -0.651025,     -2.230714,     -1.271366,
		      1.582996,      1.673926,      2.350639,      1.488655,     -0.737630,
		     -0.326492,      0.168789,     -0.443767,      1.045150,      0.598509,
		      0.000000,     -2.673381,      0.395177,      1.431935,      1.272565,
		      0.167719,      0.821091,      2.844893,      0.820222,     -0.363342,
		     -1.507216,     -0.990482,     -1.138877,      0.803337,      0.970637,
		      0.700169,      0.409347,     -1.055414,     -1.579494,      0.000000,
		      0.097682,     -0.823227,      0.411642,     -1.500006,      0.396989,
		      0.185379,     -2.903358,     -2.641133,     -2.004346,      0.198585,
		      1.815438,      1.858199,     -1.303566,     -0.126970,     -0.378375,
		     -3.272560,     -0.233400,      0.738118,      1.340367,      0.000000,
		      1.048536,     -1.474210,     -1.113086,     -0.668341,     -0.378193,
		      0.807559,      1.082390,      2.566734,      1.793123,      1.194126,
		     -2.066082,     -0.177986,      0.081385,     -0.108459,      0.988572,
		      1.693698,      1.713027,      0.459499,     -0.469351,      0.135788,
		      0.000000,      0.355269,      1.710206,      0.018333,      0.909062,
		     -0.628950,     -0.943079,      1.161655,     -0.619402,     -0.785977,
		     -2.008337,     -0.413452,      0.008196,      1.111008,      0.318452,
		      1.671551,      1.346828,     -2.257846,     -2.011531,     -1.047473,
		      1.405677,      2.735014,      0.000000,     -1.303704,     -1.040888,
		     -0.570413,      0.315711,      0.780021,     -0.689831,     -0.768777,
		      0.200860,      1.165080,      2.447521,      2.625138,     -0.749908,
		     -1.933490,     -0.417814,     -2.553331,     -1.044548,      0.608588,
		      1.010642,      2.062343,     -0.809418,     -1.329148,      0.183328,
		      0.000000,     -0.517722,     -0.996002,      1.167662,      0.762749,
		      0.746198,      1.685161,     -0.096431,     -0.040994,     -1.437937,
		     -0.440580,     -1.952808,      1.562368,      0.743513,      0.067877,
		      0.539949,      0.712699,      1.757771,      0.209547,     -1.612389,
		     -1.312081,     -0.083582,      2.528417,      1.042296,      0.000000,
		      1.792963,      0.700007,     -2.535440,     -2.579479,     -1.171238,
		     -0.715706,     -0.186208,      0.383630,      1.297990,     -1.376895,
		      0.386056,      0.548049,     -0.781332,     -0.695319,      0.883403,
		     -1.495905,     -2.350541,     -1.363733,      0.249221,      1.901359,
		      0.309731,     -3.428361,     -1.412462,     -0.223365,      0.000000,
		     -0.438222,     -0.214562,      0.066525,      0.208974,      0.322387,
		     -0.024732,      1.706888,      0.932337,     -0.714301,     -0.204413,
		      0.889261,     -1.170691,      0.055364,      0.555940,     -0.491639,
		      2.254056,      1.512193,      1.061822,      0.356376,     -2.730107,
		     -1.066029,      1.712443,      1.638847,      0.099266,     -1.590571,
		      0.000000,     -0.739158,      0.480614,      1.332275,      0.360613,
		     -0.117942,     -0.284193,     -1.770986,     -2.529328,     -0.341071,
		      2.159375,     -0.662897,      0.207356,      0.051866,     -0.237779,
		     -1.709681,     -0.583899,     -0.076022,     -1.094510,     -1.008094,
		     -1.076582,      0.013941,      0.132815,     -0.556114,      0.330045,
		      0.733502,     -1.055427,      0.000000,      1.110988,     -1.843754,
		     -0.769845,      0.272531,     -0.334417,     -0.711609,      0.309342,
		      2.150769,      1.216464,     -1.682033,     -1.251766,      0.145162,
		      0.624127,      0.478938,      0.892979,     -0.067967,      1.055838,
		      0.569927,      2.589202,      2.193750,     -2.166722,     -2.379864,
		     -0.750653,      1.950028,      0.931712,     -0.884626,     -1.803192,
		      0.000000,     -0.206036,      0.495960,     -0.734725,     -0.446804,
		     -0.610624,      0.570435,      0.363191,     -0.463276,     -1.602217,
		      0.218885,      2.240401,     -0.149522,     -0.510316,     -0.418420,
		      0.214291,     -0.320883,     -2.325652,     -0.310572,     -1.835666,
		     -1.801474,      0.905830,      1.476271,      0.816240,     -0.812635,
		     -0.772183,      3.836309,      1.859822,     -0.292081,      0.000000,
		      0.050993,      3.134460,      0.286118,     -0.563551,      0.877278,
		     -0.215564,     -0.385124,     -0.058320,      1.962528,      1.606937,
		     -0.851479,     -1.026260,      0.335087,      1.279225,     -0.182662,
		     -0.000792,      0.779324,      1.046424,     -0.746738,      0.782700,
		      2.307246,      0.083126,     -1.176117,     -0.016917,     -0.735889,
		     -1.178051,     -0.207204,     -1.438487,      0.251913,      0.000000,
		     -0.014946,     -1.997803,      0.382711,      0.288141,     -0.169273,
		     -0.899732,     -0.503842,     -0.286706,     -1.383746,     -1.775187,
		     -0.364069,      1.529168,     -0.177431,     -2.117862,     -1.529369,
		      1.004500,      1.165512,     -1.732554,     -0.359816,     -0.619116,
		     -0.729107,      1.551010,      3.291886,      2.013405,      2.000658,
		     -0.696665,     -0.604601,      1.885779,      1.179330,      0.197201,
		      0.000000,     -1.599091,     -2.416673,      0.782431,     -0.404222,
		     -0.982419,      0.240865,     -0.269859,     -0.443857,      0.291509,
		      0.690388,      0.556730,      0.644521,      0.414167,      0.187988,
		      1.613004,     -0.098360,     -0.276847,      1.179038,      1.200010,
		     -0.606387,     -1.110665,     -1.900086,     -1.724172,     -1.426529,
		     -0.286964,      1.717255,      1.052028,     -1.751148,     -2.806449,
		     -1.491055,      0.435024,      0.000000,      1.539627,      3.132439,
		     -2.235654,     -0.181654,      0.998248,      0.782390,      1.030474,
		      0.531454,      0.030792,     -1.178845,     -0.494357,     -0.197625,
		      0.598494,      0.243703,     -1.130810,     -1.755000,     -1.336820,
		     -0.203039,     -1.287561,      0.401502,      0.292253,      1.880825,
		      1.106702,      1.301655,     -0.265306,      1.106408,     -0.105037,
		      1.210043,      1.845800,     -0.269146,      1.043937,      1.003796,
		      0.000000,      0.991785,     -0.285931,      0.676276,      1.044646,
		     -1.104777,     -2.410126,     -0.222960,     -0.612526,     -0.500430,
		      0.585697,      0.322061,     -1.588539,     -1.614198,      0.553915,
		      1.024466,      0.755068,      0.535898,      0.156134,      0.258850,
		      1.519981,      0.960228,      0.976445,      0.127597,     -1.219195,
		     -1.770768,     -1.150666,      2.407532,      1.147328,      0.224070,
		     -1.132428,     -1.517048,     -1.703386,      0.041371,      0.000000,
		     -2.635431,     -0.661629,      2.184787,     -2.634130,     -0.740615,
		      1.720409,     -0.718111,     -0.622099,      0.327225,      0.729539,
		     -0.848847,      0.578855,      1.685379,      0.704805,     -0.007358,
		     -0.721885,      0.865078,      0.096193,     -0.453853,     -2.072135,
		     -1.274861,     -1.211021,      0.401913,      1.297638,      0.304844,
		     -0.805065,     -2.032314,     -0.400743,      0.564115,      0.732660,
		      0.598864,      0.445694,     -0.009121,      1.977816,      0.000000,
		      1.218532,     -1.279721,     -1.743946,      2.650615,      2.682792,
		     -0.796703,     -1.416125,      1.379306,     -0.334110,     -1.869703,
		      0.823073,      1.610359,     -0.849313,     -2.298186,      0.026034,
		      0.858199,     -0.971569,     -0.395597,      0.704774,     -0.067715,
		      0.685606,      1.137350,      0.247253,     -0.758585,      0.628918,
		      1.691824,      0.020059,      1.328673,     -0.113326,      0.968588,
		      0.608451,     -0.488901,     -0.495858,     -2.808704,     -0.605133,
		      0.000000,      1.577705,     -0.688694,     -0.306044,      0.628111,
		     -2.450566,     -0.085545,      2.770083,     -0.013014,     -0.654315,
		      0.801525,      1.102986,     -2.455909,     -1.560520,      1.132236,
		      1.342347,      0.065959,     -0.370851,      1.015759,      0.028829,
		      0.971783,     -0.194783,     -0.142184,     -0.634077,      0.474840,
		      0.588195,     -0.129034,      1.065823,     -1.863871,      0.230105,
		     -1.944903,      0.593626,      0.516874,      0.456792,      0.888094,
		     -1.197949,     -0.701789,      0.000000,     -1.026596,      2.147816,
		     -1.061482,     -2.419958,      2.156890,      1.250828,     -2.171555,
		     -2.239174,      1.278642,      0.019023,     -1.584225,     -0.133773,
		      2.401272,      0.051814,     -1.777736,     -0.115278,      1.571047,
		     -1.235792,      0.533449,      1.186367,      0.021283,     -0.180905,
		     -0.038084,      0.164630,     -1.609761,     -1.677009,      1.736935,
		      1.935181,      1.380779,      1.239277,      0.904195,     -0.697699,
		      0.547986,     -0.349988,     -0.426316,      1.214343,     -0.924518,
		      0.000000,     -0.775957,      0.241883,      0.539846,      0.522512,
		     -0.758837,     -1.831241,      0.357410,      2.040899,     -1.132477,
		     -1.794714,      1.152393,      2.480772,     -1.280519,     -1.075023,
		     -0.444252,      0.745086,     -0.316946,      0.337685,      1.486743,
		     -1.634409,     -0.820795,      1.028631,      0.749235,     -0.680035,
		     -0.141396,      0.401338,     -0.822406,      0.840872,     -3.613209,
		     -1.449434,     -0.536114,      0.876347,      0.182615,      0.648212,
		      0.581228,     -2.727005,     -0.041322,     -0.215625,      0.000000,
		      0.500467,     -1.894337,      1.616571,      0.142645,     -0.206943,
		      2.624530,      1.596136,     -0.596911,     -0.121381,      2.383986,
		     -0.282970,     -1.591592,     -0.595559,      1.021757,      1.184719,
		     -1.818887,     -1.051671,      0.109770,     -2.864588,      0.562112,
		      1.807904,      0.559176,     -0.113881,      1.863023,      1.800500,
		      0.507389,     -0.745892,      0.706456,      2.580088,      1.188814,
		      1.140758,     -0.472294,     -0.378268,     -2.379845,     -0.940938,
		      1.266009,     -0.680255,      1.947247,     -0.657851,      0.000000,
		      0.762627,      0.202291,     -1.027607,      0.160579,      0.347850,
		     -2.071828,     -0.974676,      0.089393,      1.295674,     -1.222213,
		     -1.963933,     -1.449229,      0.888423,     -1.259693,      0.428646,
		      0.023993,      0.373110,     -0.192321,      1.644819,      2.263820,
		     -0.562536,     -2.075740,     -1.415789,     -1.107860,     -1.517839,
		     -2.277287,     -0.003279,     -1.487573,     -0.356327,     -3.399347,
		     -2.587103,     -1.169940,     -0.522418,      1.233761,     -0.503262,
		     -0.075605,     -0.607325,     -2.167419,     -0.221516,     -0.168113,
		      0.000000,      0.256207,      0.333955,     -2.615275,      0.240359,
		      0.312687,     -0.965111,      0.522224,      1.147878,     -2.038670,
		     -0.336842,      2.799668,      1.883040,      0.106543,      0.892343,
		     -0.243490,      0.735693,      0.758555,      1.017196,      0.054112,
		     -3.396892,      0.102316,      2.416375,      1.539400,      0.888603,
		      0.562062,      1.005553,      1.270192,      1.308987,      0.705677,
		      1.148932,      0.521569,      0.181835,      1.247165,     -0.875847,
		     -0.742988,     -0.068099,     -0.258061,      1.207654,      0.976661,
		      0.355100,     -0.140716,      0.000000,     -1.036575,      0.220395,
		      2.945710,     -0.135189,     -1.047716,      1.718347,     -0.141506,
		     -0.132643,      1.764195,      1.768916,     -0.851983,     -0.024296,
		     -1.420628,     -1.483131,     -1.916519,     -0.086590,     -1.152279,
		     -0.937346,     -1.928029,      1.203750,      1.678268,     -0.614305,
		     -1.431937,     -0.766426,     -0.455410,      0.063007,     -2.280576,
		     -1.097283,     -1.811241,      0.610363,     -0.340680,     -0.841505,
		     -1.533536,     -1.256021,     -0.366276,     -1.339226,      0.194034,
		     -0.537875,     -3.487313,      1.900740,      1.673946,      0.393661,
		      0.000000,      1.298431,      1.303933,     -0.027859,     -1.884330,
		      0.447559,      1.031351,     -1.012621,     -1.401112,      1.237183,
		     -1.709064,     -0.757940,     -0.698678,      1.016640,      1.042300,
		      3.194792,      0.355616,      0.156126,      0.481571,      1.548701,
		      1.193838,     -1.790835,     -0.715992,      1.632442,      1.103882,
		      1.893151,      0.747410,      0.562583,      0.680277,      1.480731,
		     -0.358489,      0.393892,      0.248917,      0.525583,      1.340077,
		      0.730204,      0.219914,     -0.707233,      1.944138,      1.951575,
		     -1.129678,      0.362407,      0.671543,      0.707741,      0.000000,
		     -0.514625,     -1.534280,     -1.523623,      2.068127,      0.025158,
		     -2.276344,      0.423304,      0.388198,     -2.243276,      1.076400,
		      1.771931,     -0.908663,      0.457467,     -1.275507,     -2.248638,
		     -0.465748,      0.472321,     -0.476270,     -1.184439,     -2.735302,
		     -1.326169,      1.409811,      0.041996,     -1.532491,     -2.839741,
		     -1.616514,      0.531129,      0.678564,     -0.573014,     -0.209839,
		     -0.297652,      0.519773,      1.178301,      0.151049,     -1.485842,
		     -0.481390,     -1.177920,     -0.364440,     -0.490798,      0.761327,
		     -1.894044,     -0.199032,      0.492001,      1.220349,      0.000000,
		     -0.602227,      0.512640,      0.720152,     -0.632150,     -0.410389,
		      1.308865,      0.320671,     -0.749444,     -0.789834,      1.834616,
		     -0.089309,      1.930026,      0.188767,      1.049191,     -0.987891,
		      0.930411,     -0.230088,     -0.482757,      0.634509,      1.615592,
		      1.121153,     -0.068548,     -0.218074,      0.697321,      1.175945,
		      1.381852,      0.746424,     -1.114056,     -0.925365,      1.987968,
		     -0.360776,     -0.469995,     -1.767068,     -0.856865,      1.332161,
		      2.215441,      1.581616,     -1.114020,     -0.514009,     -0.378750,
		     -0.197640,      0.132800,     -0.704111,      0.625746,      1.639655,
		      0.000000,      0.652256,     -0.859052,      1.409440,     -0.131321,
		      0.529909,     -0.652219,     -0.424376,      1.716090,      2.816609,
		     -2.879112,     -0.847968,      0.588590,     -1.418996,     -0.227358,
		      0.348909,     -1.326848,     -0.326272,      0.837998,     -0.147237,
		      0.058624,     -1.156559,     -2.266701,     -0.196830,      0.335705,
		      1.055335,     -1.058586,     -1.786491,     -0.721561,      0.422426,
		     -1.058414,     -1.228157,     -0.009709,      2.762227,      0.891762,
		     -0.124059,     -2.115449,     -1.669856,      0.928551,     -0.496301,
		      0.001878,     -0.009467,     -0.318268,      2.370944,     -0.447013,
		     -2.915591,     -1.365067,      0.000000,      0.160008,      1.096834,
		     -0.718517,     -0.290171,     -0.335073,      1.096422,      0.687277,
		     -1.720726,     -1.709924,     -0.553941,      0.203653,     -1.827745,
		      0.887721,      1.136316,      2.744521,      1.060691,      1.207203,
		     -0.035478,     -0.142279,      1.023201,      1.165948,      1.419214,
		     -0.743217,      0.027077,     -0.562322,     -0.740871,      1.235319,
		      1.353711,      0.382040,      0.358218,      0.605944,      0.105895,
		     -0.762482,     -1.422626,     -1.775351,      1.164647,      1.120748,
		      0.921163,      0.498468,     -0.356976,     -0.859018,     -0.232882,
		     -1.556828,      0.483187,      1.458289,      1.121138,     -1.482181,
		      0.000000,     -0.112873,     -0.551836,     -1.899712,      1.353968,
		      0.815873,     -1.567115,     -2.393708,     -0.064896,      0.296753,
		      3.944055,      1.007095,      0.944983,      0.478381,     -1.314226,
		     -2.832703,     -1.333785,     -0.622819,     -0.991184,     -1.194356,
		     -1.358219,     -0.820862,      0.335134,     -0.741167,     -1.018201,
		     -0.387792,      0.084658,     -0.020300,     -1.281856,     -1.553048,
		      0.586871,     -0.741206,     -1.057027,     -0.285254,      2.225614,
		      1.947801,     -0.317855,     -1.462885,     -1.464163,      0.308304,
		      0.132384,      0.238748,      0.869994,      0.429068,     -1.388663,
		      1.406771,      0.094388,     -0.270721,     -0.970413,      0.000000,
		     -0.133562,     -0.121600,      1.458866,     -0.890121,     -1.578888,
		      0.431580,      3.473147,      1.487768,     -1.247239,     -2.099593,
		     -0.161174,      0.932792,     -0.875010,      0.741417,      0.571701,
		      1.719793,     -0.283077,      0.247392,      0.163486,      0.507187,
		      1.185939,      1.410653,      2.679866,     -0.035305,      0.639021,
		      1.088000,     -0.016210,     -0.167011,      1.242939,      0.450873,
		      0.456967,     -0.102880,     -0.349019,     -0.186581,     -0.306105,
		     -1.293340,      0.832882,     -1.246422,     -1.029157,     -0.289042,
		     -0.856325,     -0.420062,      1.169062,      0.301297,     -0.125747,
		     -1.282616,      2.125699,      0.155733,     -1.441118,      0.000000,
		      0.757045,     -0.134572,      0.003964,     -0.607508,      2.013886,
		      0.558709,     -1.788773,     -1.554075,      1.244136,     -0.769306,
		     -0.514263,     -0.782228,      1.070015,      0.459499,      1.555394,
		     -1.685892,     -1.023819,      1.987552,      1.811806,     -1.165087,
		     -2.208689,     -3.322981,     -2.131640,      0.033767,     -1.124553,
		     -0.035077,     -0.636066,     -0.692702,     -0.991476,     -0.979383,
		      0.188251,     -0.545687,     -1.415902,     -0.011921,     -0.004662,
		      2.129176,     -0.126210,      0.900941,     -0.949053,      1.241874,
		      2.031996,      0.771022,     -0.929160,      1.384493,      0.462412,
		     -0.026547,     -0.511734,      0.214025,     -0.139547,      0.667432,
		      0.000000,     -0.663587,      0.724567,      0.015572,      0.706855,
		     -1.399188,     -1.961000,     -0.409025,      1.649502,     -0.177345,
		     -0.385177,     -0.166566,      0.595224,      0.624658,     -0.754901,
		     -1.029491,      0.822531,      1.659369,     -0.147956,     -2.100477,
		     -0.399668,      2.053603,      2.049531,      1.281214,      2.863960,
		     -0.753790,     -1.767660,      0.796076,      0.730843,      1.306602,
		      0.605755,      0.263636,      1.750850,      1.742162,     -0.577243,
		      0.745317,      0.989042,      0.315316,      1.102565,     -0.697776,
		     -3.157871,     -1.565914,     -3.210994,     -0.964912,     -0.652932,
		      1.431817,     -0.786022,     -0.162394,     -2.056806,     -1.437645,
		      1.128194,      1.337938,      0.000000,      0.884080,     -0.363672,
		     -2.310884,     -0.160133,      0.788369,      2.538689,      0.434003,
		     -2.455315,     -1.773678,      1.650017,      1.670357,     -0.352557,
		     -1.996163,     -0.709400,     -1.056664,      3.168720,     -0.393190,
		     -2.946797,     -0.274537,      2.883486,      0.208080,     -0.674827,
		     -1.363367,     -3.879194,     -1.043108,     -0.432909,      0.378555,
		      0.313327,     -1.661248,     -0.771133,     -0.527408,     -0.714946,
		     -0.289821,     -1.103704,      0.281298,     -1.230564,     -0.745974,
		     -2.850309,     -0.655615,     -0.366573,      1.687009,      4.048688,
		      1.790555,      1.258283,      1.137185,      1.653875,      0.162123,
		      1.253905,      0.994046,     -1.854097,     -0.622905,      1.505449,
		      0.000000,     -0.735379,     -1.163365,      2.397324,      0.399909,
		     -0.094606,     -0.077034,      0.111044,      1.690749,      2.231144,
		     -0.903046,     -2.485118,     -1.347723,      1.951514,      2.522615,
		      2.122321,     -2.846066,     -1.235316,      2.399145,      2.214005,
		     -1.160921,     -1.616057,      0.227373,      1.714117,      3.218480,
		      5.051070,      2.794177,     -2.006606,     -1.068105,      0.150518,
		      1.511390,      1.264454,      0.537953,      0.008902,      1.229546,
		     -0.477395,      0.171621,      0.890516,      1.717531,      0.462268,
		      2.577975,      0.078567,     -0.324491,     -0.579378,     -1.769709,
		     -2.687838,     -0.489391,      1.243300,     -0.675165,     -0.494294,
		     -0.345318,     -1.558534,     -1.601228,      0.425067,      0.000000,
		     -0.032882,      2.481629,      0.783288,     -2.530972,     -0.686733,
		     -0.436412,      0.546728,      0.934042,     -1.258229,     -0.805347,
		      0.588555,      2.360340,      0.013975,     -2.946055,     -1.701007,
		     -1.431673,      3.150123,      1.869046,     -1.946296,     -1.735312,
		      1.842217,     -0.220251,     -0.952962,     -0.883380,     -4.575066,
		     -3.215920,     -0.513962,      1.202304,      2.034483,     -1.081653,
		     -0.602065,      1.049543,     -0.782194,     -0.347587,     -0.438217,
		     -0.012259,     -0.328181,      1.088188,     -1.584249,     -1.887427,
		     -2.707356,     -1.170007,      0.508128,      1.218452,     -0.621284,
		      0.843802,      1.958828,      0.899177,      0.925952,      2.873941,
		      0.022809,      1.864538,      0.842843,     -0.655710,      0.000000,
		      0.028325,     -1.116112,     -2.321482,      0.958613,     -0.469009,
		     -0.594418,     -0.527109,     -0.565207,      0.262548,      2.390771,
		      1.236777,     -2.607908,     -3.152238,      1.736348,      2.658645,
		      4.159056,     -2.401110,     -4.125128,     -0.209134,      2.711470,
		     -0.140703,      0.072409,     -0.379550,     -0.604525,      1.837953,
		      3.938030,      3.923814,      0.448279,     -0.738193,     -0.070756,
		      0.291753,     -1.887998,      1.078565,      0.400136,     -0.034816,
		     -0.929068,     -1.290974,     -0.750353,     -0.352141,     -1.589690,
		      2.852939,      2.399266,      0.016657,     -1.170410,      0.554569,
		      0.456596,     -2.369138,     -0.435697,      0.941429,     -0.126445,
		      0.220557,     -1.560913,     -0.730641,      1.144843,     -1.188426,
		      0.000000,     -0.196011,     -1.646377,      0.220999,      0.931948,
		      0.930585,      1.032776,     -0.242200,     -0.525980,      1.347808,
		     -0.994419,     -1.309638,      1.340355,      3.896591,      0.242384,
		     -4.278451,     -2.832211,     -0.379784,      3.889820,      2.542213,
		      0.093480,     -1.955467,      0.000098,      0.617736,     -0.009904,
		      1.519620,     -2.762483,     -4.629493,     -1.918182,     -0.903304,
		      1.458138,     -0.625784,      0.127200,      1.911136,     -0.604180,
		     -0.383713,      0.098347,      0.686925,     -0.617134,      0.906135,
		      0.473607,     -1.053163,     -0.622744,     -1.395210,     -0.675391,
		     -1.146903,     -1.233444,      0.006349,      0.451430,     -0.498810,
		     -0.513935,      1.455548,      2.414629,      1.224587,      0.889780,
		      1.370775,     -0.574587,      0.000000,      0.815865,      0.571815,
		      1.845665,     -0.174273,     -0.036358,     -1.044420,     -0.155573,
		      0.545123,     -1.312016,     -1.485539,      1.523186,      1.426222,
		     -2.639806,     -2.905495,      3.261366,      2.504302,      3.114918,
		     -0.656097,     -3.171209,     -0.798278,      1.946408,      1.359648,
		      0.992143,     -1.081136,     -2.623780,      1.273648,      2.301053,
		      2.647309,      0.068308,     -0.418156,      2.397881,      2.618937,
		     -2.074636,     -0.265238,      0.514291,     -0.669429,     -1.063328,
		     -0.661980,     -0.109412,      0.255652,     -2.194846,     -0.703159,
		      1.314914,      1.204033,     -0.975497,     -0.670861,      0.330373,
		     -0.066554,     -0.089490,     -1.779858,     -0.815887,      0.910829,
		     -2.406198,     -2.541225,      0.194639,     -1.372931,      0.103566,
		      0.000000,      0.267007,      2.115685,     -2.097875,     -1.984341,
		     -0.908166,      1.937574,      0.746090,     -0.433502,      0.098726,
		      2.037144,     -1.643743,     -2.528977,      0.877794,      4.498166,
		      0.721709,     -4.323008,     -2.962004,     -0.546007,      2.182347,
		      2.094732,      0.634907,     -1.073134,     -2.102916,     -0.889703,
		     -1.063679,      0.242782,     -0.141090,     -1.562400,     -0.346092,
		     -0.400988,     -0.742834,     -2.796671,      0.173191,      1.762332,
		      0.569706,      0.510160,      0.041876,      1.337681,     -0.697551,
		      0.150934,      2.446782,      0.265835,      1.077359,      0.347401,
		      0.912664,      0.592168,     -0.275239,     -1.884975,     -0.867118,
		      1.547051,      1.432866,     -0.792694,      1.755721,      2.068293,
		      1.927745,      0.267883,     -2.029821,     -0.945874,      0.000000,
		     -0.736472,     -1.740490,     -0.505753,      1.781720,      1.229520,
		     -1.121238,     -0.208568,      0.924972,     -0.078003,     -1.699243,
		     -0.328721,      2.485464,      1.273699,     -3.847824,     -3.700019,
		      4.423900,      2.894296,      0.715238,     -1.090381,     -2.044060,
		     -1.271580,      0.604861,      1.899128,      3.828907,      1.441326,
		     -1.713297,      0.580283,      1.056080,      0.772824,     -0.631683,
		     -2.843112,     -0.012833,      3.084923,     -1.207339,      0.774793,
		      0.076557,     -1.326061,     -2.031012,      0.047467,     -0.209120,
		     -0.172519,     -0.135022,     -0.993706,     -1.445295,     -2.206893,
		      0.231864,      1.748381,      1.478980,      0.270585,     -0.993687,
		      0.156316,     -0.184224,     -1.166308,      0.217465,     -1.588688,
		     -0.725927,      0.846465,     -0.133620,      1.320015,      0.000000,
		     -0.224861,      0.069546,      3.926861,     -0.549727,     -1.805594,
		     -0.928049,      0.533298,     -1.235985,     -0.533720,      2.118446,
		      3.637365,     -1.615411,     -3.097540,      1.543390,      3.456072,
		      0.184146,     -3.481233,     -1.633845,      0.353925,      1.299966,
		      1.292438,      1.380577,      1.129631,     -3.015182,     -0.380093,
		     -0.326850,     -2.010161,     -0.434531,     -0.157118,      2.007524,
		      2.440311,      1.016131,     -3.660214,     -1.498165,     -0.335874,
		      1.415844,      1.905989,      0.118378,      0.511186,     -0.403893,
		     -1.339569,      0.988025,      0.648341,      1.639994,      2.042193,
		      2.173783,     -0.491188,      0.745927,      0.311162,     -1.270636,
		     -1.243248,      0.685387,     -0.445871,     -0.263354,      2.220649,
		      1.350840,      1.785189,     -1.206521,     -0.600797,     -0.993728,
		      0.000000,      0.964185,     -0.098497,     -3.122920,     -0.624708,
		      0.975806,      1.836269,     -1.698856,      0.887010,      2.185265,
		     -1.160483,     -3.826805,     -1.421460,      2.374123,      1.799445,
		     -1.653812,     -3.576856,      2.545079,      3.774116,      0.612521,
		     -0.516144,     -0.930650,     -1.373905,     -2.390122,      0.173878,
		      2.701666,      1.190619,     -0.449438,      1.089905,      0.028977,
		     -0.386279,      0.199491,     -0.481591,      0.534426,      1.425580,
		     -0.101661,     -0.090111,      1.429600,      0.165250,     -1.644349,
		     -0.354025,     -0.349524,     -1.902565,      1.016021,      0.421991,
		     -1.014884,     -1.881356,     -0.715540,     -0.954217,      0.286642,
		      1.000056,      1.969870,      0.427524,      1.110950,     -1.336067,
		      0.177191,      0.199259,     -0.798649,      0.294154,      2.097504,
		      2.433056,      2.961206,      0.000000,     -0.280163,     -0.017646,
		     -0.609939,      2.128731,      0.409567,     -0.240221,      1.318310,
		      0.155770,     -2.905928,     -1.854238,      3.455765,      4.203134,
		     -0.615356,     -2.697836,      1.317153,      2.074833,     -1.418534,
		     -3.136480,     -1.483233,      1.737622,      0.186720,     -1.141939,
		      0.709360,      1.902932,     -2.122596,     -0.011963,      1.785966,
		     -2.886094,     -1.317784,     -0.180026,      0.840781,      1.145101,
		      2.787013,     -1.451434,     -0.895345,     -2.764899,     -1.850374,
		      0.494072,      2.810328,      1.975453,      2.808176,     -0.625988,
		      0.374031,     -0.941964,      0.546521,      2.878318,      0.680808,
		     -0.231451,     -1.082950,      0.223178,     -0.011632,     -1.554917,
		     -1.374393,     -1.205710,     -0.968903,      0.310344,     -0.025042,
		      0.405067,     -1.619286,     -2.143078,      0.510104,      0.245733,
		      0.000000,      0.216918,      0.935800,      1.734616,     -1.323316,
		     -1.150634,     -0.836218,      0.618117,     -0.915831,      1.422418,
		      4.128456,     -0.467884,     -4.715967,     -1.638461,      1.941409,
		      0.474313,     -0.327749,     -0.715976,      0.736720,      1.797981,
		     -0.847809,     -0.231960,      1.841551,      1.267906,     -2.717159,
		     -1.108247,      1.210867,      0.070611,      1.110500,      2.978308,
		     -0.035489,     -2.128950,     -1.572197,     -2.414306,      0.772789,
		      0.383949,      1.431156,      0.138926,      0.677755,     -1.032431,
		     -2.820395,     -1.123599,     -0.399524,     -1.159390,     -0.657722,
		      0.899989,     -1.840876,     -0.249439,      1.646153,      0.426499,
		     -2.965146,     -2.434078,     -1.257780,      0.913560,      2.506570,
		      1.255405,     -1.013062,      1.426521,      0.535343,      4.110399,
		      0.378068,      1.678082,      1.254897,      2.055297,      0.000000,
		      0.329122,     -1.060808,      0.006744,     -1.227217,      0.951076,
		      0.820574,     -1.733166,      1.568004,      1.192492,     -3.009000,
		     -3.220743,      2.980768,      3.878720,      0.079246,     -1.337716,
		      1.775052,      2.385094,      0.289499,     -1.458693,     -1.790701,
		      1.859788,      1.807389,     -1.003698,     -0.079279,      1.810704,
		     -2.338702,     -0.699285,      2.567098,     -2.648796,     -1.516034,
		      1.095716,      2.076379,      1.773007,      1.425912,      0.024693,
		      1.155454,      0.127869,     -1.351587,     -1.100743,      2.986093,
		      2.081642,      1.654546,      2.013570,     -0.009090,     -1.113947,
		     -0.835073,     -0.156382,      1.150086,     -0.684733,     -0.094385,
		      2.435857,      3.005342,     -1.159519,     -3.081723,     -0.646254,
		     -0.355615,     -0.533073,     -0.175599,     -1.862298,     -0.039887,
		     -2.727061,     -1.286161,     -1.811736,      0.001637,      0.000000,
		      0.199996,     -0.488952,     -0.298870,      1.099674,     -0.642335,
		     -0.437913,      0.196130,     -0.189544,     -2.082243,     -0.139030,
		      4.723714,      2.553064,     -3.219758,     -1.499162,      1.345757,
		     -0.887847,     -1.396163,      0.408997,      0.692542,      2.430025,
		     -2.026276,     -3.572968,      0.342842,      1.951565,     -0.517526,
		      0.090308,      0.935978,     -1.891639,     -0.025466,      2.749732,
		      1.056233,     -2.109176,     -0.101953,     -1.842766,      0.146268,
		     -2.484661,     -0.019890,     -0.261205,      2.015904,      0.140702,
		     -2.117353,     -1.341769,      1.185684,     -0.517974,     -0.696225,
		      0.972785,     -0.749565,     -1.632901,      0.944454,      2.944638,
		     -0.013255,     -2.786121,     -2.399183,      0.249939,      1.039096,
		      0.347324,      1.815245,      0.435570,     -0.713074,      2.001984,
		      1.471238,      1.101898,      0.950810,     -1.269103,      0.001214,
		      0.000000,      0.274000,      1.485021,     -1.085010,     -1.069885,
		     -0.605333,      1.876755,      1.856227,     -2.088198,      0.713272,
		      2.502003,     -1.324211,     -4.472895,      0.948984,      2.322959,
		     -0.625241,     -1.055451,      0.137795,      0.674272,     -0.053071,
		     -0.128858,     -0.178763,      1.584916,      2.259096,     -0.381387,
		     -0.045784,      1.307122,     -1.023154,     -0.496523,      2.782725,
		     -2.001304,     -1.923291,     -0.160647,     -0.079020,      0.972788,
		      1.122082,      1.391903,     -0.912913,      1.492846,     -1.626922,
		     -1.709550,      1.127100,      0.815381,     -0.250949,      1.644734,
		      1.825560,     -0.571034,      0.527725,      1.802900,      1.548356,
		     -0.101763,     -0.253711,     -0.124019,      1.768847,      0.831154,
		     -2.320916,      0.442104,     -1.721139,     -0.579744,      1.943923,
		      0.552935,     -1.274352,     -0.962832,     -0.828252,     -1.948574,
		     -2.714051,     -1.260668,      0.000000,     -0.280928,     -0.697323,
		      0.416166,      1.927900,      2.351095,     -1.620276,     -2.143951,
		      1.793401,      1.198299,     -2.133097,     -2.396430,      1.510671,
		      1.528543,     -1.495004,     -0.341309,      2.432082,      1.093726,
		     -1.728959,     -0.535338,      0.108668,      2.058382,      0.710891,
		     -2.499088,     -2.579702,     -0.862669,     -1.634892,     -0.346017,
		      0.791900,     -2.962146,     -0.982015,      1.406122,      1.840599,
		     -1.920726,     -0.694795,      0.226679,      1.630134,     -1.013407,
		     -1.486703,     -1.126211,      0.029244,      0.699579,     -1.763833,
		     -1.268751,     -0.052054,      0.042081,     -1.798385,     -2.435248,
		     -0.826306,     -2.502728,     -1.687425,      1.692528,      2.451502,
		     -0.604770,     -0.530125,     -0.445103,      1.012486,      1.025545,
		     -1.582165,     -0.836458,     -1.055828,     -0.052945,      1.222774,
		      0.257366,      2.837131,      1.897495,     -0.638032,      0.886687,
		      0.000000,     -0.056485,     -0.329707,      1.516741,     -1.143533,
		     -2.678946,     -0.981542,      1.835478,      0.497507,     -1.776441,
		     -0.183943,      2.934438,      1.742458,     -2.433349,      0.839518,
		      1.609910,     -1.049461,     -1.701718,      0.841218,      1.173645,
		     -0.978616,     -2.995797,     -0.269532,      0.717593,      0.755572,
		      0.786496,      1.257304,      0.987270,      1.177599,      1.647915,
		      3.661801,      0.401928,     -1.891734,      0.355929,     -0.828677,
		     -0.643634,     -1.999009,      2.481041,     -0.888606,      0.996078,
		      1.322601,     -0.140552,      2.255435,      2.187334,     -2.294097,
		      1.897870,      2.657894,      0.676248,      0.570764,      2.964960,
		      3.658631,      1.482240,     -1.874603,     -0.523884,      0.972959,
		      1.737660,      0.763023,      0.104202,     -0.177454,      0.091543,
		      1.224828,      0.667449,      0.405319,     -0.962053,     -2.096831,
		      0.740488,      0.860304,     -2.049469,     -0.843060,      0.000000,
		      1.030117,      0.311962,     -1.728100,     -1.366700,      1.423396,
		      2.543460,     -1.661583,     -1.644014,      1.789751,      0.712472,
		     -1.668994,     -1.107894,      1.602783,      1.103127,     -1.534263,
		     -1.168992,      1.016603,      1.953937,     -0.666178,      0.296670,
		      1.566962,     -0.097005,     -0.536654,     -0.261580,      1.521520,
		      0.348315,     -1.687887,     -2.177673,     -0.998603,     -3.214730,
		     -1.082585,     -0.140795,      0.629628,     -0.059781,     -1.902539,
		     -0.727611,      0.345308,      1.898517,      0.332897,     -0.708290,
		     -2.953216,     -0.957536,     -1.469211,      0.229200,      0.318304,
		     -0.151904,     -1.426474,     -1.168896,     -1.259959,     -2.365582,
		     -1.352084,     -0.367442,      0.925602,     -0.633328,      0.064868,
		     -0.129793,     -1.729574,      0.025306,      0.601597,     -0.729002,
		     -2.760080,     -1.280975,      0.366831,      2.602415,      1.971696,
		      1.937458,      2.831925,      3.163886,     -0.749598,      0.000000,
		     -0.576305,      0.704007,      0.875824,      1.378753,     -0.731149,
		     -2.441301,      1.114314,      2.474431,     -0.999871,     -1.108454,
		      0.409929,      1.010950,     -0.186756,     -1.199237,      0.817867,
		      1.593135,      0.123513,     -2.947989,     -0.198733,      2.098445,
		      0.271113,     -1.671966,      1.385788,      2.481839,     -1.110328,
		     -1.671647,      1.089915,     -0.591729,     -0.621677,      2.105684,
		      1.226039,      2.352685,      0.806818,      2.183146,      1.242229,
		      1.181570,     -2.528225,     -0.042167,     -0.661680,     -1.318893,
		      1.213689,      1.151694,      1.261559,      0.984795,     -0.204632,
		      0.223180,      0.761420,      0.450414,     -0.444582,      1.548982,
		      2.213736,      3.183902,     -0.530019,     -1.796895,     -1.921059,
		      1.876397,      1.747402,     -0.856129,      0.914122,     -0.232743,
		      4.074197,      1.557417,     -0.686637,     -0.939213,     -0.444104,
		     -0.426321,      1.482856,      1.317026,      2.488063,      1.685873,
		      0.000000,      0.037110,      0.148324,     -0.334156,     -0.100250,
		      0.763276,      1.426368,      0.471117,     -1.970802,      0.104188,
		      3.316751,      1.288603,     -2.394007,     -1.965496,      0.442170,
		     -0.586698,      0.733899,      0.448370,      0.073154,      0.477887,
		     -0.921430,     -0.810517,      4.275408,      1.584113,     -3.804622,
		     -3.703473,      0.009208,      1.311506,      2.015165,      1.340956,
		     -0.786092,     -3.271306,     -1.717005,     -0.693806,     -1.239259,
		      0.587609,     -0.085483,      0.127620,     -1.109590,      0.829760,
		      1.182361,      3.779227,     -1.915839,     -3.307179,     -1.361962,
		      1.300174,     -0.191821,      0.793075,      0.875229,     -0.213788,
		     -0.243346,      0.021386,     -2.563615,     -1.031086,     -0.815123,
		      2.371807,      0.483638,     -1.627839,      0.042011,     -1.185512,
		     -0.338686,     -2.992665,      0.603926,     -1.424368,      1.042642,
		      0.682402,     -0.520038,      0.096004,     -0.554988,     -3.597646,
		      0.578994,     -0.744703,      0.000000,     -0.113349,     -1.378554,
		      0.222317,      0.324285,     -0.885267,      0.255857,     -0.957134,
		     -0.177299,      0.543501,     -1.943269,     -1.267628,      2.047470,
		      1.847422,      0.069194,      0.687995,     -0.028665,      0.262441,
		      1.775751,      0.097755,     -0.689806,     -0.685010,     -2.178515,
		     -4.366735,      0.081595,      5.089826,      0.917646,     -3.447360,
		     -0.667850,     -0.766895,     -1.439119,      3.848648,      1.154526,
		     -0.815081,      0.074674,      0.817896,      0.099952,      1.605211,
		      0.113132,      0.275896,      0.681638,     -2.545982,      0.090562,
		      2.558247,      2.871401,     -1.703437,     -1.306603,      1.237676,
		      0.687117,     -0.219078,     -1.249228,     -0.397465,      0.201577,
		      2.921481,      2.984582,      0.175111,     -0.097648,     -0.390960,
		      0.493786,     -1.066667,     -1.158128,      0.159426,      2.100228,
		      2.362941,      1.905535,      1.157530,      0.013620,     -1.566973,
		      0.646793,      1.984618,      2.146688,      3.058675,      0.812564,
		      0.000000,      1.107663,      1.538745,     -0.135936,     -2.521191,
		     -0.090573,      0.484005,      0.671147,      0.142391,     -0.846225,
		     -1.159535,      0.267994,      1.699478,      0.924656,      0.069719,
		     -0.304125,     -2.600359,      0.023801,      1.364150,      0.778635,
		      0.642956,     -0.182190,     -2.151710,      2.652513,      4.443510,
		     -0.598222,     -2.369782,      0.542640,     -1.025232,     -0.661993,
		      1.562833,     -0.521643,     -1.558933,     -0.513464,      1.024108,
		     -2.362782,     -1.465040,     -1.118969,      0.660068,     -1.605861,
		     -0.010493,      0.268031,      3.321910,      1.761430,     -3.582869,
		     -0.793423,      0.594659,     -1.967351,     -2.390093,      1.152600,
		      1.776921,     -1.181898,      0.662958,     -2.232599,     -3.485208,
		     -2.833779,      0.364064,      1.956605,      1.252997,      0.519541,
		      1.168526,      1.437531,     -2.884276,     -1.663674,      0.410836,
		      0.995067,      1.044105,      0.606423,     -0.911631,     -1.301589,
		     -3.982422,     -2.052150,     -0.974469,     -1.731879,      0.000000,
		      0.277946,     -0.246766,     -0.460690,      2.329602,      1.038476,
		     -3.891159,     -0.251248,      2.453705,      0.738753,     -0.449841,
		     -1.837037,     -2.054963,     -1.062873,     -0.496396,      0.114760,
		      3.202454,      0.697005,     -2.206896,     -0.863434,      0.580749,
		      1.992165,      2.943358,      0.748802,     -4.886281,     -3.471656,
		      1.153234,      2.064802,     -0.674659,     -0.792455,     -0.434222,
		     -3.665072,      1.732290,      2.345337,     -0.925141,      0.799489,
		      1.911014,      0.556169,     -0.744465,     -0.085311,     -0.715610,
		      1.802706,     -2.440963,     -3.007036,      0.738249,      3.245793,
		      1.309344,     -0.981697,      0.163190,      1.470364,     -1.665342,
		      1.656806,     -1.249730,     -0.050606,      0.402903,      2.293154,
		      0.255496,     -0.366668,     -1.392078,      1.612544,      0.718709,
		     -0.984409,      0.332732,     -0.968268,      0.847749,     -3.690770,
		     -0.870198,      1.409490,     -0.072791,      1.112964,     -0.440740,
		     -0.173838,     -2.432298,     -0.023045,     -0.153675,      0.000000,
		     -1.680489,     -0.633929,      0.660862,      0.045574,     -0.775840,
		      3.053098,      0.756539,     -1.103092,     -0.486759,      2.703072,
		      1.923939,     -1.682033,     -1.877060,      1.740826,      2.513505,
		      0.608184,     -1.696493,     -0.631351,     -0.886377,     -0.616414,
		     -0.194759,      0.690522,     -2.520076,     -0.861914,      2.599859,
		      0.691756,     -1.284938,      0.801999,      0.397863,      0.624035,
		      1.659929,     -0.643835,     -1.549405,     -0.155299,      2.395142,
		      0.196179,      0.114993,     -1.586239,      0.313112,     -2.099135,
		     -1.194376,      0.039208,      1.097361,      2.376907,     -1.497010,
		     -1.769479,      0.787996,      0.807139,     -2.467196,      0.465125,
		      2.330353,     -1.785752,      1.049382,      1.586463,     -0.698274,
		     -4.318126,     -2.956786,     -0.728154,     -0.883254,      0.423656,
		      0.803380,      1.017643,      0.019148,      0.571623,     -0.106479,
		      0.531151,      0.650943,     -1.671288,      0.013029,      3.981959,
		      1.137660,     -0.696519,     -0.487682,      0.023102,      1.577073,
		      0.000000,      0.318245,      0.488085,     -0.134641,     -0.946845,
		      1.051068,      1.569378,     -1.282926,     -1.655650,      0.966903,
		     -0.561166,      0.477091,      1.477214,      1.414010,     -0.760458,
		     -3.028234,     -1.599657,      1.718108,      1.480010,      2.041316,
		      0.327377,     -2.509193,     -1.955377,      1.182353,      4.112135,
		      0.423072,     -0.900615,     -1.169482,     -2.196579,     -0.825824,
		     -0.291955,      0.505041,     -0.780249,      0.259008,      0.960725,
		     -3.425276,     -1.429855,      0.244736,      2.521144,      0.733735,
		      1.416097,     -1.081940,      1.169274,      0.805729,     -2.217791,
		     -1.224565,     -0.831155,      0.474294,      0.062215,     -0.364237,
		      1.734312,     -2.172845,     -0.177099,      0.603940,     -1.881993,
		     -1.157938,      2.114312,      0.719735,     -0.517749,      0.534052,
		     -0.715432,      1.198526,     -0.274516,      1.117714,     -1.735590,
		     -1.408141,     -0.148247,     -2.398928,     -0.227786,     -0.072856,
		     -1.763790,     -0.483608,     -1.208978,      0.027814,     -3.646328,
		     -2.685943,      0.168840,      0.000000,      2.005183,      0.747940,
		      0.324083,     -0.576985,     -1.479240,     -0.909054,      1.927314,
		      0.137123,     -1.963420,     -1.325646,     -0.686216,      1.010581,
		     -0.223423,     -1.476496,      0.622313,      2.049987,      1.089167,
		      1.192696,      1.134937,     -1.195049,     -0.115764,      1.334166,
		      2.408817,     -1.536833,     -3.366964,     -0.651525,      0.481357,
		      0.031790,      1.008094,     -0.522290,     -0.541595,      1.533122,
		     -0.594093,     -0.591592,      0.952937,      1.289643,      0.120122,
		      1.490885,     -0.076781,      0.220993,     -0.553599,     -3.176462,
		      0.596953,      0.598893,      1.297191,      1.039589,     -1.276589,
		     -1.618408,      1.076347,     -1.292545,      0.283765,      1.680460,
		     -1.341600,     -2.584236,      2.412598,      3.181867,     -0.729292,
		     -1.266962,     -2.232323,     -3.942285,     -0.381093,     -1.245570,
		     -0.347605,      1.377114,      1.385241,      0.527165,     -1.655110,
		      0.069860,     -0.156325,     -0.196114,      1.038399,      3.790900,
		      1.370943,      1.772411,      1.431934,     -0.454179,      1.393323,
		      0.000000,     -0.525992,     -0.759129,     -0.399968,      1.261934,
		      1.153547,     -1.135202,     -1.688697,      1.919607,      0.947358,
		      0.861421,     -1.142473,     -0.224184,      0.323636,      1.707764,
		      1.033581,     -1.567558,     -1.609076,     -0.925922,     -2.219159,
		      1.706536,      2.778569,      0.287367,     -3.552154,     -2.577636,
		      2.005584,      1.344765,      1.171022,      1.354484,     -2.273119,
		     -1.329926,     -1.684695,     -0.879497,      0.695500,      2.257760,
		      0.835600,     -2.331215,     -1.719626,     -3.776547,     -0.441352,
		     -0.261278,      2.492743,     -0.349439,     -0.547606,     -0.316699,
		     -0.865698,      0.684464,      0.095277,     -0.210611,     -0.040461,
		     -0.073483,      2.372526,     -1.089075,      0.213830,      1.198199,
		      1.199699,     -4.717228,     -1.209037,      0.660706,     -1.348239,
		      1.520087,     -2.752636,     -0.329000,     -0.940112,      0.329088,
		     -0.737326,     -1.023756,     -0.923376,     -0.921835,     -0.692755,
		     -0.558755,     -0.101980,     -0.378143,     -1.817476,     -1.427359,
		      0.114112,     -1.368514,      1.555897,      1.793935,      0.000000,
		     -1.500449,      0.596775,     -0.126254,     -0.046939,     -0.509970,
		     -0.508776,      1.279802,      0.845142,      0.407501,      0.027148,
		      1.682861,     -0.819006,     -1.570470,     -0.829748,      0.731263,
		      1.224228,     -0.432357,     -0.247554,      0.765265,      0.001873,
		     -3.616049,     -0.725074,      0.767224,      2.827071,      1.437485,
		     -0.490820,     -0.210418,     -0.442018,     -0.852371,      1.666347,
		      0.486372,      0.416209,     -0.511675,     -1.669066,     -2.192737,
		      1.575973,      1.496363,      0.785798,      1.135346,      1.562572,
		     -0.547207,      2.713295,     -0.965949,     -0.310418,     -0.269482,
		     -0.059062,      2.170416,      1.835454,     -3.581469,     -0.251883,
		     -0.279996,      0.947356,      1.131645,     -0.744904,     -3.469819,
		      1.982431,      4.695758,      1.046799,      1.886671,      1.564740,
		      1.366761,     -0.456721,      0.492071,     -2.201353,     -0.193367,
		      0.991996,      0.099114,      1.630409,     -0.022625,      2.079903,
		      1.710062,      1.297269,     -0.176814,      1.313324,      0.244638,
		      0.724772,     -1.419080,     -3.307845,      0.870111,      0.000000,
		      1.306578,      0.173866,     -0.695001,     -0.457596,      0.829640,
		      2.458198,     -1.585779,     -3.046867,      0.863758,      0.362539,
		     -0.602335,     -2.112070,      1.002981,      1.352162,      0.302096,
		      0.281228,      1.459810,      0.613607,     -1.646621,     -2.023591,
		      2.351271,      2.741667,      2.267655,     -1.283235,     -3.158623,
		      0.263470,      1.063480,     -0.553284,      0.926946,      0.016937,
		      0.027396,     -1.163119,     -2.070014,     -0.535653,      1.162037,
		      0.801773,     -1.323702,      0.694539,     -1.416589,     -2.640213,
		     -2.106467,     -0.662204,      0.812144,     -0.151616,     -0.129843,
		     -2.429839,     -2.612969,     -0.062773,      2.136205,      1.371528,
		     -0.354079,     -0.033365,     -0.589865,      1.189672,      2.242298,
		      0.614313,     -2.471816,     -3.568495,     -4.011193,     -2.909126,
		     -0.502364,     -1.783618,     -1.410153,     -0.147685,      1.030900,
		     -1.823909,     -1.356569,     -2.524917,     -0.541595,     -2.413659,
		     -2.323622,     -0.088871,      0.381849,     -0.397040,      1.268004,
		     -0.767524,      0.354408,      2.930814,     -0.208510,     -0.414708,
		      0.000000,     -0.028511,      0.142385,      1.510385,      0.598634,
		     -0.776734,     -1.772012,     -0.296992,      1.204529,     -0.583432,
		      0.204823,     -0.453178,      1.342900,      0.565728,     -2.413225,
		     -3.098002,      0.429234,      1.367939,     -0.141588,      1.835802,
		      2.859332,     -0.510665,     -3.103836,     -1.356898,      0.285829,
		      1.101494,      0.323972,     -0.913674,     -0.197257,     -0.017185,
		     -1.967416,     -1.832808,      0.777325,      4.686862,      2.425395,
		     -1.340538,     -1.877020,      0.932987,     -0.372431,     -0.266288,
		      0.260503,      2.099749,     -0.420290,      2.209423,      0.853375,
		      2.034064,      1.192583,      1.636089,      0.091081,      2.304268,
		     -1.645252,      0.065635,     -1.500736,      0.785391,      0.858112,
		      0.148500,     -1.452957,     -0.105755,      3.431619,      2.961755,
		      0.387621,      0.129148,      0.165381,      1.222959,      1.397236,
		     -1.887819,     -0.191495,      4.355143,      1.774043,     -0.921327,
		      0.615069,      0.531376,      0.105629,      2.099298,      0.169478,
		     -1.688632,     -1.851282,     -0.297667,     -0.930849,      1.245289,
		      1.428181,     -1.021905,      0.000000,      0.339902,      0.675180,
		     -1.315216,     -2.173538,     -0.694475,      0.678048,      3.316984,
		     -0.536218,     -0.575015,      0.907336,      0.096141,      1.030204,
		     -1.653682,     -0.793609,      1.051994,      0.827355,     -3.313254,
		      0.798010,      0.987198,     -3.052434,     -4.108858,      1.151385,
		      0.609528,      0.383280,      0.778925,     -0.922357,      0.179718,
		      1.752341,      0.183262,      0.376440,     -1.059976,      1.583355,
		     -1.309165,     -3.196317,     -1.929931,     -0.254071,      0.065992,
		     -1.427373,     -1.370129,      0.885744,      0.636049,     -0.076021,
		     -2.430552,     -1.583553,     -2.623357,      0.685174,     -1.503484,
		     -1.686204,     -3.425976,      0.730692,      0.067883,      0.501842,
		      0.291598,     -1.101271,      1.257020,      0.253355,      1.425458,
		      1.025281,      0.128147,     -2.559144,     -2.983105,      0.567306,
		     -0.330564,     -0.809865,      0.465533,      1.461702,     -2.386981,
		     -2.929834,     -0.266167,      0.448162,     -0.011901,     -1.955821,
		     -1.008363,      0.230943,      0.741389,     -0.718135,     -1.708315,
		     -1.866130,     -0.100385,      0.288735,     -2.504199,     -0.216611,
		      0.000000,      0.026827,     -1.403603,      0.786026,      0.326407,
		      1.074440,      1.824506,     -2.778150,     -0.426665,      0.766514,
		     -0.547357,      0.608054,      0.455039,     -0.293743,      1.200341,
		      0.788801,     -1.936017,      0.678723,      0.705427,     -1.639640,
		      0.034093,      4.545088,      2.308582,     -1.428955,     -0.843718,
		      0.539010,      0.336786,      0.835734,     -0.767531,     -1.430689,
		      1.478282,      1.160509,     -1.870426,     -2.921403,      2.613683,
		      2.445711,     -0.734528,     -0.415332,      2.887015,      0.809925,
		     -1.006960,     -2.275369,     -1.427401,     -0.579884,      2.386371,
		      1.481653,     -0.939748,     -0.405092,      2.697113,      2.335200,
		      1.766799,     -1.350864,      1.077890,      0.357801,      0.611218,
		      1.077271,      0.177084,     -0.166879,     -0.156658,     -1.931732,
		      1.115646,      0.955895,     -1.431231,     -0.374658,      0.062345,
		      1.788174,     -1.091504,     -0.327502,      2.584761,      1.536011,
		     -0.907250,     -0.061289,      0.668378,      0.018835,     -1.748116,
		     -0.228460,     -0.450138,      0.081810,     -0.935977,      1.621747,
		      3.513920,      0.944631,     -1.892218,      0.726322,      0.000000,
		     -0.148715,      0.506888,      1.949142,      2.659834,      0.237010,
		     -3.434300,     -2.142400,      2.713898,      0.318357,     -1.280891,
		     -0.023180,     -1.785108,     -0.585101,     -0.067913,      0.014453,
		      1.299514,      1.685356,     -0.993646,      0.447060,      1.456370,
		     -1.141483,     -1.908886,      1.554073,      1.052157,     -1.452811,
		      0.336559,      0.781064,     -1.253583,     -0.416823,     -0.182708,
		      0.207556,     -0.537121,      3.031296,      1.086352,     -2.110842,
		     -1.124683,      0.376375,      0.096302,      0.833136,     -1.457326,
		     -0.380894,      0.334560,      3.285300,     -0.291875,     -0.025355,
		     -1.746534,      0.794113,     -2.615248,     -0.735033,     -3.298790,
		      1.394637,      0.371848,     -1.011784,     -1.575251,      0.626800,
		      1.005389,     -1.287363,     -2.753469,     -0.094619,      0.141757,
		      0.785990,     -1.890628,      1.092518,     -0.213199,     -1.347044,
		      2.130436,      1.475701,     -0.041799,      0.254842,      0.613428,
		     -1.823273,      0.154653,     -1.146116,     -2.086082,      0.435899,
		      3.699348,      2.572157,      1.094766,     -1.579646,     -1.083382,
		     -0.369451,      0.200116,     -1.051283,      1.574738,      0.000000,
		     -0.049008,      1.016678,     -1.683039,     -0.115773,     -1.609858,
		      0.918694,      4.295512,     -0.926183,     -0.632965,      0.362237,
		      0.134807,     -0.996184,     -0.288191,      0.701491,     -0.746641,
		      0.810614,     -1.367795,      1.032105,      0.252559,     -1.315549,
		     -0.986772,      2.193727,      0.858093,     -0.671782,     -0.242873,
		      0.651807,      0.348672,      0.228305,     -0.208615,     -0.779199,
		     -1.713271,     -0.068336,     -0.024696,     -1.383694,      2.136486,
		      2.837979,     -2.691517,     -4.234816,      0.534115,      1.794065,
		      1.792264,      0.188667,     -2.569499,     -3.257813,     -1.421486,
		      3.857651,      0.624647,      0.336028,     -1.348578,      0.648664,
		     -0.556951,     -1.173679,     -0.702806,      0.185603,      0.995299,
		      0.050130,      1.229334,      0.852579,      2.111129,     -1.548784,
		     -1.597240,      0.818492,     -1.946714,      0.141125,      0.542609,
		     -0.179810,     -2.006656,      0.543972,      0.542125,     -0.959305,
		     -0.256839,     -0.024904,      2.493129,      1.434464,     -1.792007,
		     -4.067927,     -2.059570,     -0.151756,      0.700076,     -1.308909,
		      0.529019,      1.705079,     -1.537974,     -0.137356,     -0.762063,
		      0.000000,     -0.221409,     -0.523343,     -1.399693,     -1.573672,
		      1.528475,      2.198316,     -1.909301,     -2.182687,      1.817680,
		      1.867811,     -0.937138,      2.211140,      2.049467,     -1.584171,
		     -1.911412,     -0.451838,     -0.201312,     -0.855931,     -1.094436,
		      0.655872,      1.313069,     -0.013744,     -2.451200,      0.618603,
		      1.192935,     -0.593008,     -0.224835,      2.334772,      2.104901,
		     -0.025272,     -0.910764,      1.338425,      0.075705,      1.664370,
		     -0.454727,     -3.269480,     -1.338468,      0.882863,     -0.387938,
		      0.248113,     -1.488320,     -0.828449,      0.162981,      1.335491,
		      0.470789,     -0.791015,     -1.826384,      1.497359,     -0.768447,
		      0.849709,      0.122719,     -0.033366,      1.808635,      1.788770,
		     -1.881263,     -1.556818,      1.294706,      1.511673,     -2.230543,
		     -1.906245,     -1.648518,     -1.315935,      0.937466,      0.442307,
		      1.344504,     -1.217318,      0.192689,      0.353093,     -0.794132,
		      0.598530,      0.914316,      1.960661,     -1.084231,     -3.959794,
		     -0.200211,      1.568249,      0.251541,      0.366698,     -0.259466,
		     -0.234661,     -1.247830,      2.252475,      1.111478,      1.446587,
		     -0.737947,      2.913036,      0.000000,     -0.397700,     -0.517894,
		      3.153558,      0.290376,     -1.581195,     -2.764717,      1.119532,
		      2.743649,     -1.645033,     -2.168680,      1.197953,      2.272843,
		     -1.940938,     -2.521137,      1.911368,     -0.240038,     -1.757927,
		     -0.220867,      2.058199,      1.964626,     -1.247736,     -0.213117,
		      1.965067,      0.317984,      0.570413,      1.106208,      0.706281,
		     -1.742612,     -1.625340,     -1.447148,      1.146993,      2.160131,
		      0.661671,     -2.464697,     -2.025523,      1.828479,      6.698374,
		      1.490512,     -2.986773,     -2.008268,      0.674109,      0.914788,
		      2.156529,      1.187281,     -0.050212,     -3.411219,     -0.356062,
		     -1.426520,      3.284315,      0.224019,      0.580077,     -1.079849,
		     -1.059168,     -2.364955,     -1.381318,      0.980620,      0.044187,
		     -1.877419,     -2.540623,      1.204935,      2.757381,     -0.876987,
		     -0.244495,     -2.114999,      0.779026,      1.812595,      2.453917,
		     -0.538625,      1.097820,      0.784943,     -0.238314,     -1.196742,
		      0.197950,      2.034583,      3.418148,      0.718767,      0.898629,
		     -0.014758,      0.047941,      0.355442,      0.686951,     -0.269258,
		     -0.009392,     -3.597909,     -0.188402,      1.277837,     -0.485484,
		      0.000000,      0.107543,      0.095496,      0.946328,      0.403391,
		     -0.475060,      2.449687,      0.688608,     -2.248271,     -1.482612,
		      0.857858,      0.994652,     -3.614291,     -1.540366,      3.342203,
		      0.072300,     -0.662565,      0.594316,      1.109517,     -1.445551,
		     -2.397097,     -0.711979,      0.862434,     -0.110943,     -0.724530,
		     -1.228153,     -2.220668,     -2.103809,     -1.312763,     -0.216950,
		      2.904658,      0.374028,     -2.420064,     -0.738004,      0.852811,
		      2.446242,     -0.037987,     -2.307266,     -2.802627,      0.226478,
		      1.057209,      1.518363,     -0.369130,     -1.434198,      0.348108,
		     -0.210196,      0.680976,      1.013153,      1.067148,     -0.830462,
		     -1.351015,     -1.210971,      2.868662,      0.666322,      0.572511,
		      2.059285,      0.896670,     -0.469641,      0.548080,      3.632236,
		     -0.271688,     -1.130943,      0.384257,      1.649115,      0.804766,
		     -2.978317,     -0.689034,     -1.095693,     -0.897255,     -0.005341,
		     -0.718218,     -2.901776,      1.498830,      2.448256,      1.452929,
		     -1.689814,     -3.506286,     -2.222757,      0.012805,      1.367481,
		     -0.402498,     -0.706135,     -1.001467,      1.711576,      1.305732,
		     -0.792370,      1.294953,     -3.439589,     -1.329712,      0.000000,
		      0.541719,     -0.330009,     -2.132772,      1.605435,      2.139441,
		     -1.759834,     -2.990469,      1.426352,      3.307008,     -0.065052,
		     -1.569266,      1.000130,      2.780700,      0.301512,     -2.937646,
		      1.291960,      1.036264,     -1.850284,     -1.462545,     -0.520511,
		      1.328344,      0.606234,     -0.590178,      1.359957,      0.218867,
		      0.402608,      2.131499,      3.467029,      1.099673,     -0.899431,
		     -3.455123,     -0.544513,      1.336453,      3.952639,     -0.198302,
		     -2.873005,     -2.026632,      2.361389,      2.070444,     -0.458256,
		     -2.737532,     -1.304360,     -0.522448,     -1.808788,     -1.239595,
		      1.469279,     -0.135625,      0.327967,     -2.945822,     -0.645798,
		      0.192744,      0.085440,     -0.106777,      0.829634,     -0.215060,
		     -2.607077,      0.406943,      1.450989,      0.313344,     -0.889829,
		     -1.910664,     -0.481273,     -0.567879,      2.399174,      0.859229,
		     -1.553482,      0.084319,      0.027380,      1.212676,      0.193431,
		      2.379280,     -0.515841,     -2.592451,     -0.151694,     -0.211411,
		      1.801811,      2.107230,      1.715389,     -0.071983,      0.874083,
		     -2.565839,     -0.495489,      0.782101,      2.032060,      0.028306,
		     -0.715287,      2.435015,      3.748912,      1.906505,      0.000000,
		     -0.728132,     -0.290043,      1.176731,      0.032787,     -2.139228,
		     -1.686295,      2.921823,      2.026551,     -1.486874,     -1.139898,
		      1.779953,      2.680439,     -1.912721,     -2.659295,      2.403398,
		      0.075369,     -2.357170,      0.485848,      2.320839,      0.388900,
		     -0.643696,      0.686687,      0.351683,     -0.726410,      0.702014,
		      2.072476,     -0.801887,     -3.649461,     -1.489063,     -1.250705,
		      1.628265,      3.183726,      0.442922,     -1.558284,     -0.329324,
		      1.398063,      1.796189,     -0.538361,     -1.364297,     -0.171614,
		      1.183291,      2.444026,      2.089768,      0.813917,      2.256043,
		      0.090435,     -1.409936,     -0.747168,      1.963667,      1.060318,
		      0.072731,     -2.911363,     -0.521956,      0.734030,     -0.167988,
		     -0.001380,      0.201572,     -0.820337,     -3.251337,      0.286295,
		      2.390951,     -0.710844,      1.084774,     -0.641727,      0.777634,
		      2.307580,     -0.686623,     -1.085689,      0.729773,      2.908853,
		      0.001123,     -1.570262,      0.972879,      1.312983,      1.255504,
		      2.260833,     -0.419419,     -2.504618,     -2.178831,     -0.731013,
		      2.124891,      3.669966,     -0.905726,     -1.009653,      0.665342,
		     -0.333772,     -2.935195,     -2.578233,     -0.877376,     -0.272402,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : mars ---  Model : mh2014 */
coeffStruct& _model_coeff_mh2014() {
	static const std::string name = "mh2014";
	static const std::string body = "mars";
	static const int len = 6215;
	static const int nmax = 110;
	static const int ndef = 110;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,  14,  14,  14,  14,  14,  14,  14,  14,
		  14,  14,  14,  14,  14,  14,  14,  15,  15,  15,  15,  15,  15,  15,  15,  15,
		  15,  15,  15,  15,  15,  15,  15,  16,  16,  16,  16,  16,  16,  16,  16,  16,
		  16,  16,  16,  16,  16,  16,  16,  16,  17,  17,  17,  17,  17,  17,  17,  17,
		  17,  17,  17,  17,  17,  17,  17,  17,  17,  17,  18,  18,  18,  18,  18,  18,
		  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  19,  19,  19,
		  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,
		  19,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,
		  20,  20,  20,  20,  20,  20,  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,
		  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,  22,  22,  22,  22,
		  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,
		  22,  22,  22,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,
		  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  24,  24,  24,  24,  24,
		  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,
		  24,  24,  24,  24,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,
		  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  26,  26,
		  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,
		  26,  26,  26,  26,  26,  26,  26,  26,  26,  27,  27,  27,  27,  27,  27,  27,
		  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,
		  27,  27,  27,  27,  27,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,
		  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,
		  28,  28,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,
		  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,
		  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,
		  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  31,
		  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,
		  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  32,
		  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,
		  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,
		  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,
		  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,
		  33,  33,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,
		  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,
		  34,  34,  34,  34,  34,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,
		  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,
		  35,  35,  35,  35,  35,  35,  35,  35,  35,  36,  36,  36,  36,  36,  36,  36,
		  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,
		  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  37,  37,
		  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,
		  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,
		  37,  37,  37,  37,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,
		  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,
		  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  39,  39,  39,  39,  39,
		  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,
		  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,
		  39,  39,  39,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,
		  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,
		  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  41,  41,  41,  41,
		  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,
		  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,
		  41,  41,  41,  41,  41,  41,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,
		  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,
		  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,
		  42,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,
		  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,
		  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  44,  44,  44,
		  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,
		  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,
		  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  45,  45,  45,  45,  45,  45,
		  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,
		  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,
		  45,  45,  45,  45,  45,  45,  45,  45,  46,  46,  46,  46,  46,  46,  46,  46,
		  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,
		  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,
		  46,  46,  46,  46,  46,  46,  46,  47,  47,  47,  47,  47,  47,  47,  47,  47,
		  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,
		  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,
		  47,  47,  47,  47,  47,  47,  47,  48,  48,  48,  48,  48,  48,  48,  48,  48,
		  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,
		  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,
		  48,  48,  48,  48,  48,  48,  48,  48,  49,  49,  49,  49,  49,  49,  49,  49,
		  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,
		  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,
		  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  50,  50,  50,  50,  50,  50,
		  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,
		  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,
		  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  51,  51,  51,
		  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,
		  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,
		  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,
		  51,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,
		  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,
		  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,
		  52,  52,  52,  52,  52,  52,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,
		  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,
		  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,
		  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  54,  54,  54,  54,
		  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,
		  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,
		  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,
		  54,  54,  54,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,
		  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,
		  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,
		  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  56,  56,  56,  56,  56,
		  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,
		  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,
		  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,
		  56,  56,  56,  56,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,
		  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,
		  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,
		  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  58,  58,
		  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,
		  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,
		  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,
		  58,  58,  58,  58,  58,  58,  58,  58,  58,  59,  59,  59,  59,  59,  59,  59,
		  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,
		  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,
		  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,
		  59,  59,  59,  59,  59,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,
		  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,
		  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,
		  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,
		  60,  60,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,
		  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,
		  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,
		  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,
		  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,
		  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,
		  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,
		  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  63,
		  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,
		  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,
		  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,
		  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  64,
		  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,
		  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,
		  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,
		  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,
		  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,
		  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,
		  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,
		  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,
		  65,  65,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,
		  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,
		  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,
		  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,
		  66,  66,  66,  66,  66,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,
		  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,
		  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,
		  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,
		  67,  67,  67,  67,  67,  67,  67,  67,  67,  68,  68,  68,  68,  68,  68,  68,
		  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,
		  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,
		  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,
		  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  69,  69,
		  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,
		  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,
		  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,
		  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,
		  69,  69,  69,  69,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,
		  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,
		  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,
		  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,
		  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  71,  71,  71,  71,  71,
		  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,
		  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,
		  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,
		  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,
		  71,  71,  71,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,
		  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,
		  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,
		  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,
		  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,
		  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,
		  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,
		  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,
		  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  91,  91,  91,  91,  91,  91,  91,
		  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,
		  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,
		  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,
		  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,
		  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,
		  91,  91,  91,  91,  91,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,
		  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,
		  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,
		  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,
		  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,
		  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,
		  92,  92,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,
		  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,
		  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,
		  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,
		  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,
		  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,
		  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,
		  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,
		  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,
		  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,
		  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,
		  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  95,
		  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,
		  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,
		  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,
		  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,
		  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,
		  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  96,
		  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,
		  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,
		  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,
		  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,
		  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,
		  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,
		  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,
		  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,
		  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,
		  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,
		  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,
		  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,
		  97,  97,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,
		  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,
		  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,
		  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,
		  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,
		  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,
		  98,  98,  98,  98,  98,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,
		  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,
		  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,
		  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,
		  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,
		  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,
		  99,  99,  99,  99,  99,  99,  99,  99,  99, 100, 100, 100, 100, 100, 100, 100,
		 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
		 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
		 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
		 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
		 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
		 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 101, 101,
		 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
		 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
		 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
		 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
		 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
		 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
		 101, 101, 101, 101, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102,
		 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102,
		 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102,
		 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102,
		 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102,
		 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102,
		 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 103, 103, 103, 103, 103,
		 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103,
		 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103,
		 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103,
		 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103,
		 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103,
		 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103,
		 103, 103, 103, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104,
		 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104,
		 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104,
		 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104,
		 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104,
		 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104,
		 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
		 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
		 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
		 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
		 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
		 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
		 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
		  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
		  41,  42,  43,  44,  45,  46,  47,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
		  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
		  41,  42,  43,  44,  45,  46,  47,  48,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,
		  51,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,
		  47,  48,  49,  50,  51,  52,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,
		  52,  53,  54,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,
		  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,
		  53,  54,  55,  56,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,
		  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,
		  50,  51,  52,  53,  54,  55,  56,  57,  58,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,
		  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
		  55,  56,  57,  58,  59,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,
		  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,
		  59,  60,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,
		  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
		  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,
		  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,
		  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
		  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,
		  64,  65,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,
		  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,
		  62,  63,  64,  65,  66,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,
		  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,
		  59,  60,  61,  62,  63,  64,  65,  66,  67,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,
		  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
		  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,
		  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,
		  66,  67,  68,  69,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,
		  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,
		  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,
		  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,
		  69,  70,  71,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,
		  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,
		  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,
		  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,
		  68,  69,  70,  71,  72,  73,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,
		  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,
		  74,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,
		  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,
		  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,
		  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,
		  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,
		  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,
		  70,  71,  72,  73,  74,  75,  76,  77,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,
		  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,
		  72,  73,  74,  75,  76,  77,  78,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
		  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
		  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,
		  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,
		  73,  74,  75,  76,  77,  78,  79,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
		  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
		  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,
		  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,
		  73,  74,  75,  76,  77,  78,  79,  80,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,
		  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,
		  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,
		  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,
		  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,
		  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,
		  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,
		  83,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,
		  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,
		  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,
		  79,  80,  81,  82,  83,  84,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,
		  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,
		  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,
		  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,
		  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,
		  84,  85,  86,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,
		  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,
		  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,
		  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,
		  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,
		  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,
		  85,  86,  87,  88,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,
		  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,
		  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,
		  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,
		  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,
		  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,
		  82,  83,  84,  85,  86,  87,  88,  89,  90,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,
		  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
		  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,
		  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,
		  87,  88,  89,  90,  91,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,
		  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,
		  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,
		  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,
		  91,  92,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,
		  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,
		  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,
		  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
		  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,
		  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,
		  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,
		  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,
		  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,
		  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,
		  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,
		  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,
		  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
		  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,
		  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,
		  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,
		  96,  97,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,
		  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,
		  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,
		  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,
		  94,  95,  96,  97,  98,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,
		  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,
		  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,
		  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,
		  91,  92,  93,  94,  95,  96,  97,  98,  99,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,
		  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
		  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,
		  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,
		  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,
		  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,
		  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,
		  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,
		  98,  99, 100, 101,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,
		  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,
		  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,
		  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,
		  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,
		  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,
		  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,
		  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100,
		 101, 102, 103,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,
		  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,
		  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,
		  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,
		  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103, 104,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,
		  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,
		  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,
		  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99,
		 100, 101, 102, 103, 104, 105,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,
		  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,
		  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,
		  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103, 104, 105,
		 106,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,
		  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,
		  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,
		  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,
		  95,  96,  97,  98,  99, 100, 101, 102, 103, 104, 105, 106, 107,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,
		  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,
		  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,
		  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,
		  99, 100, 101, 102, 103, 104, 105, 106, 107, 108,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,
		  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,
		  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,
		  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100, 101,
		 102, 103, 104, 105, 106, 107, 108, 109,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,
		  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,
		  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,
		  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103,
		 104, 105, 106, 107, 108, 109, 110,
	};
	static const std::vector<double> g = {
		     -1.776757,     -0.372729,     -0.109372,      1.045706,      0.961623,
		      0.505398,     -0.506474,     -0.761948,     -1.011396,     -0.891157,
		     -0.875039,      0.033732,      2.135351,      0.577347,      0.745156,
		      0.222002,      0.367999,     -0.254566,      0.415450,      0.525286,
		     -1.153118,     -0.205269,     -0.088238,     -1.694085,      0.443571,
		      0.369833,     -0.942819,      1.097088,      1.253476,     -0.710449,
		      0.503450,      0.184151,      0.601845,      1.048651,      1.425221,
		     -0.820103,     -3.270516,      0.780647,      1.422189,     -0.758096,
		     -0.844431,     -0.950710,     -2.067597,     -0.889619,     -0.918001,
		      3.055187,      1.531397,     -1.575264,     -0.342873,      0.780240,
		      0.744391,      0.480969,      0.552338,      1.513513,      2.570002,
		     -0.487650,     -4.080110,     -0.657841,      1.727017,      0.720290,
		     -0.426108,     -0.123814,     -0.852418,     -1.739478,     -0.688816,
		     -2.159984,     -2.161317,      3.008571,      2.393620,     -1.034536,
		     -1.958354,     -0.379883,      0.179721,      0.447782,      0.477694,
		      0.874656,      0.969499,      0.834827,      3.271649,     -0.060402,
		     -2.216829,     -1.711868,      0.712166,      1.225809,     -0.132177,
		     -0.148564,      0.822249,     -1.401422,     -2.288167,     -3.052688,
		      0.169193,     -2.922715,     -2.195108,      2.193046,      3.663116,
		      1.224155,     -0.351563,     -0.564209,      0.021325,      0.159606,
		      0.978255,      2.218990,      2.110834,     -0.170151,     -2.432871,
		      0.638932,      2.808105,     -0.951251,     -2.193822,     -1.990967,
		     -0.060200,      0.859365,      0.422607,      0.374181,      1.100358,
		     -1.171311,     -1.769825,     -2.442552,     -1.570683,      3.913741,
		      0.442163,     -2.986005,     -1.717596,      0.502075,      2.022486,
		      0.123246,     -0.185718,     -0.975776,     -1.119009,     -1.169257,
		     -0.423626,      0.697858,      2.572779,      1.759686,      0.638786,
		     -2.527126,     -3.276194,      2.035081,      3.928496,     -0.035289,
		     -1.615180,     -0.808469,     -0.606467,      0.540181,      0.441941,
		      0.934303,      1.074750,      0.381348,      0.151125,     -2.098304,
		     -1.061612,      0.016292,     -1.512735,      5.590817,      0.409375,
		     -4.046533,     -0.994726,      1.578503,      2.458212,     -0.341743,
		     -0.352399,     -0.407703,     -0.362698,     -0.801907,     -0.276059,
		     -0.568812,      2.193631,      1.227232,      1.407611,      1.271123,
		      3.292306,     -5.733164,     -4.149346,      1.642469,      2.799066,
		     -0.072276,     -1.687202,      1.153369,     -0.196860,      0.511320,
		      0.835117,      1.506949,      1.831880,      0.788099,     -0.244879,
		     -0.564203,     -0.328880,      0.177862,     -1.009448,     -2.600962,
		      0.401027,      6.091193,      1.013626,     -3.002624,     -2.130258,
		      0.450374,      1.543706,      0.204936,     -0.640738,     -1.392755,
		     -2.870748,     -2.710877,     -0.543715,     -0.016202,      0.974211,
		      0.079346,      0.062585,      1.208894,     -1.552754,      3.757623,
		      3.567009,     -5.427548,     -3.452955,      2.071486,      1.899816,
		     -0.576102,     -1.660319,      1.331278,      0.501616,      1.065884,
		      2.797866,      2.370275,      1.494432,      0.479963,      0.042628,
		      0.498782,      0.499717,      0.639404,      0.896086,     -0.567271,
		     -3.163349,     -3.612447,      2.105640,      4.446688,      0.572847,
		     -0.265071,      0.079000,      0.844419,     -0.959905,     -0.147964,
		      1.072108,     -0.129515,     -1.478698,     -2.729739,     -1.121293,
		     -0.637687,      0.443179,      0.696018,     -0.548355,     -0.948442,
		     -1.155993,     -1.133094,      0.661544,      3.573132,      1.115903,
		     -3.650189,     -2.327278,      0.269279,      0.613280,     -0.467362,
		     -0.671666,      1.002924,     -1.542886,     -1.238911,      1.044524,
		      1.881518,      1.075513,     -0.712190,     -2.107138,     -0.490225,
		      0.570588,      0.801511,      1.105277,      0.261540,      1.333021,
		      1.803335,     -4.865604,     -2.689387,      2.942225,      1.921369,
		     -1.097086,     -0.574000,      0.375985,      1.547388,     -1.287395,
		     -1.555463,      1.227640,      0.185364,     -1.245703,     -1.708497,
		      1.473404,      1.786928,      1.129787,      1.017641,     -0.903730,
		      0.044840,     -1.525167,     -1.143878,      0.790143,     -1.545352,
		      2.956578,      1.816216,     -1.241066,     -1.256216,     -0.297163,
		     -0.493529,     -0.879904,     -0.883391,      0.848290,      2.478130,
		     -0.964898,     -1.706372,      0.927181,      2.193638,     -0.266464,
		     -1.398189,     -2.182150,     -1.748985,     -1.140110,     -0.245684,
		      0.619775,      1.539591,      1.419937,      1.756203,      1.196755,
		      0.131063,     -1.412742,     -0.739660,      2.743306,      1.383463,
		     -0.096283,      0.182467,     -0.456188,      0.167011,     -0.330682,
		     -0.047711,     -0.030437,     -0.853673,     -2.166217,     -2.397297,
		      1.240335,      2.670529,      0.944269,      1.117584,     -1.503209,
		     -1.040677,     -0.236710,     -2.121758,     -0.243980,     -0.010816,
		     -0.516383,     -0.348601,      2.166876,      0.758700,     -2.654684,
		     -0.496502,      0.180579,     -0.139454,     -0.639362,     -2.277514,
		     -1.511415,      2.349457,      0.913049,     -0.018492,      1.119520,
		      3.433155,      0.000496,     -0.831716,      0.205272,     -1.010326,
		     -0.362825,     -0.099996,      0.874790,      1.584285,      1.527469,
		      2.332847,     -0.022355,      0.912361,      0.030488,     -1.940466,
		     -0.035932,      0.143281,      0.639763,      0.519024,      0.858194,
		      1.765175,      0.474713,     -0.680451,     -1.720100,      1.147731,
		      1.291597,     -0.236196,     -1.971199,     -2.299922,     -1.902649,
		     -0.650914,     -0.566632,      0.921155,     -0.157068,     -0.569137,
		     -0.285297,     -2.560690,     -2.157141,     -0.426791,     -0.006347,
		     -0.936260,      0.602479,      1.147607,      1.365332,      0.963611,
		     -1.423793,      0.280278,      0.482984,     -0.839170,      1.404813,
		      1.681303,     -1.155095,     -1.684753,     -1.071251,     -0.841043,
		      1.634996,      3.080410,      1.090815,      0.209396,      1.936230,
		      1.010933,      1.226215,      1.551836,      0.456931,      1.460738,
		      1.650114,      0.901026,     -0.744735,     -0.654915,     -0.023326,
		     -1.097939,     -0.292789,     -1.414159,      1.274379,      1.150928,
		     -1.846477,     -1.744586,     -0.900746,     -0.077198,      0.715206,
		      0.578053,     -1.499961,      0.589890,      0.662271,     -1.164241,
		     -2.314417,     -0.117460,      0.008832,     -1.678402,     -2.315577,
		     -1.733927,      0.163773,      0.044812,      1.818937,     -1.466806,
		     -1.476800,      0.724515,     -0.763646,     -0.079387,     -0.496625,
		      0.047140,      0.091196,     -0.132534,     -1.147660,     -0.225397,
		     -0.050004,      1.079249,      1.728625,     -1.452123,     -1.784621,
		      0.400976,      1.548134,     -0.412493,     -0.057651,      0.344316,
		      1.675663,      1.060256,     -0.036813,      0.666318,      3.715470,
		      0.761641,      1.042805,      0.341059,     -1.075836,      1.141840,
		      0.214408,      0.321316,     -0.439907,     -0.682459,      1.421734,
		      2.003604,      0.332569,     -0.894636,      0.507239,     -0.843213,
		      1.513329,      3.246315,     -0.499403,     -1.526725,      0.071441,
		     -0.272834,      0.314830,      0.961036,     -0.328632,     -0.221593,
		      0.447798,     -2.715557,     -2.717360,     -0.719637,     -0.043513,
		     -1.795736,     -0.649379,     -1.725648,     -0.738306,     -1.298650,
		      0.085382,      1.440008,      1.056776,      0.548826,      1.201149,
		     -0.365203,     -0.808344,     -0.964678,      0.385328,      0.601846,
		      0.446987,      1.621144,     -1.223652,     -2.266853,     -0.519218,
		      0.100709,      2.193074,      1.209547,     -1.244995,     -1.555020,
		     -0.782544,     -1.198957,     -1.864609,      1.707663,      4.554426,
		      1.534745,      0.117870,      0.374078,      2.887941,      1.020401,
		      0.154873,      1.828920,     -0.477978,     -1.089515,     -1.106234,
		      0.145271,      0.445306,     -1.265601,      1.504818,      0.728926,
		     -0.111715,     -0.705398,     -0.659150,     -0.658992,      0.965500,
		     -0.851884,      0.456289,      2.619551,     -0.211076,     -1.329055,
		     -0.407957,     -0.738613,     -0.089100,      1.657246,      2.135889,
		      2.006465,      1.930755,     -2.795807,     -3.590770,     -0.545812,
		      1.503824,     -2.000372,      0.296677,     -1.997775,     -0.857998,
		     -0.441340,     -0.736426,      1.309741,     -0.247780,      1.378487,
		      2.371724,     -0.253062,     -0.089949,     -0.826525,     -0.769033,
		      0.200576,      2.431469,     -0.625116,     -2.731710,      2.041392,
		     -0.069600,     -1.260243,      0.595160,     -0.695570,     -0.586910,
		      2.249801,      1.472956,     -0.943854,     -1.030273,     -1.908235,
		     -5.120792,     -0.911958,      4.090932,      2.637974,     -0.219397,
		     -0.652437,      0.206164,      1.067586,     -1.428987,      0.217745,
		      0.687115,     -0.154244,      0.017280,     -0.797022,     -0.919699,
		      0.596467,      0.118885,     -0.568093,     -2.413615,      3.040354,
		      0.809362,     -3.267893,      0.085529,      1.015133,      1.169465,
		      0.800455,      0.332471,      0.920803,     -0.123302,      0.528178,
		      0.869910,     -0.850565,     -1.762823,      0.392842,      1.792769,
		      3.640777,      3.665765,     -1.373523,     -3.000141,     -1.926493,
		      1.418050,      0.456823,      1.729799,      0.849746,     -0.632537,
		      1.810648,     -0.538055,     -0.651530,     -0.106156,      0.138136,
		      1.067148,      1.710717,      1.980049,      0.861104,     -1.925337,
		     -2.598584,     -3.042261,      1.289066,      3.833144,      1.293377,
		     -3.088614,      0.233016,     -1.025827,     -1.470269,      2.034138,
		      0.051548,     -4.890620,     -1.603919,      2.115915,      0.688254,
		     -0.140220,      0.910294,     -3.904737,     -2.392676,      2.094424,
		      2.807274,      0.834802,     -0.410808,     -2.140335,     -0.153922,
		     -1.075298,     -0.950500,      0.087897,     -0.024503,      1.117556,
		      1.716729,     -0.371778,     -1.303613,     -1.192156,      1.156781,
		     -1.566831,     -3.309451,      0.168039,      4.131500,     -0.120746,
		     -5.000030,     -1.469841,      0.613126,     -0.768206,      2.476795,
		      1.624026,     -0.718366,     -1.390007,      2.606871,      4.136516,
		      0.738719,     -2.547425,     -1.179229,     -0.719771,      2.702256,
		      4.140332,      0.998491,     -1.792175,     -2.941518,     -1.248033,
		     -0.215358,      0.449582,      1.807448,     -0.363632,      1.204309,
		     -1.190901,     -1.322350,     -0.640313,      0.409778,      0.881065,
		     -0.789250,     -1.146772,      2.326816,      2.511115,     -0.878792,
		      2.182938,     -3.031469,     -0.820741,      1.330591,      1.274849,
		      2.153808,      0.303352,     -0.660208,     -1.729066,     -0.619552,
		      3.839930,      2.385111,     -5.292475,     -3.880810,      1.001819,
		      0.032242,     -0.746053,     -0.865367,     -3.637839,     -2.941761,
		      0.536800,      2.976830,      2.292168,      1.222086,     -0.442801,
		     -0.716687,      0.043535,     -0.987089,     -0.591701,     -0.338168,
		      0.216517,      1.969046,     -0.267924,      0.984980,      1.266761,
		     -1.226638,     -0.671386,     -0.117686,      1.373730,     -2.063458,
		      2.441579,      2.566920,      0.679112,     -0.771041,     -0.935097,
		     -0.152025,     -1.468174,      1.860647,      1.307945,     -2.684254,
		     -3.639225,      1.958094,      5.019697,      3.089663,     -0.137048,
		      0.847403,     -0.316680,      2.269571,      2.770961,      0.676102,
		     -0.537286,     -2.334252,     -1.986102,     -0.510323,     -1.381001,
		      1.219179,      0.065724,      1.781205,     -1.004774,     -2.730962,
		     -2.767015,      0.532503,     -0.329209,     -0.777512,     -0.945308,
		     -0.357172,     -0.582681,     -0.468087,      0.420854,      0.808735,
		     -1.337753,     -2.507553,      0.516308,      0.139891,     -1.816068,
		      0.791515,      1.814679,     -1.327937,     -1.795441,     -0.835488,
		      3.803151,      3.065864,     -4.300516,     -4.113835,     -0.408889,
		     -0.375254,      0.080713,     -0.066981,     -2.137316,     -2.094240,
		     -0.526965,      0.869891,      0.939055,      0.534072,      0.685365,
		     -0.336634,     -0.379589,     -1.838516,     -0.924650,      1.703198,
		      1.582188,      0.675403,     -1.731767,      1.218555,      1.703118,
		      2.092941,      1.613630,      0.318721,      2.032358,      3.363631,
		     -1.659327,      0.213959,      2.061558,     -1.427444,     -0.878828,
		      1.998779,     -0.735358,     -0.145677,      0.825517,      2.733999,
		      2.441774,     -2.897305,     -4.072672,      2.004725,      3.523727,
		      2.062134,      0.048549,      0.108257,     -0.488466,      2.398470,
		      2.094868,      0.099410,      0.239900,     -1.566085,     -0.973819,
		     -0.511983,     -1.350680,      0.397733,      1.021313,      1.149696,
		      0.636613,     -0.585721,     -1.785067,      0.317162,     -1.427916,
		     -0.401315,     -0.527286,     -1.477955,     -0.620308,     -1.545084,
		     -0.284928,      0.698165,      3.200990,      0.786895,     -2.623024,
		      1.394023,      3.393651,     -0.511832,     -1.749890,     -0.307269,
		      0.792922,     -2.100433,     -1.406799,      0.045367,      2.407194,
		      1.369427,     -2.011032,     -2.003799,     -1.293003,     -1.361757,
		      0.014236,     -1.706814,     -2.276055,     -0.570947,      1.186746,
		      2.261210,      0.326625,     -0.455352,      0.358422,     -0.140937,
		      0.951408,     -1.627265,     -2.004336,      1.421611,      1.665263,
		      0.768581,     -0.117049,     -1.503164,     -0.003919,      1.458909,
		      3.322328,      2.734948,      1.315549,      1.211237,     -0.304714,
		     -2.278935,     -2.489685,      2.360762,      1.459918,     -3.834168,
		     -0.824794,      2.551356,     -1.624205,     -2.335888,      0.330426,
		      1.696711,      2.601234,     -0.592250,     -3.513069,      1.226879,
		      1.710487,      2.098415,      1.450924,      0.391138,     -1.273815,
		      2.150868,      1.862693,     -2.055084,     -1.529213,     -1.106729,
		      0.243274,      1.186960,     -0.643795,     -0.882993,     -0.270348,
		      0.283793,      1.338683,      1.399553,     -0.796712,     -0.136288,
		      0.993185,      1.479434,      0.705851,     -2.115795,     -1.872389,
		     -2.089612,      0.325567,      0.269444,      0.053532,     -0.602471,
		      3.812622,     -1.317038,     -5.058008,      2.990047,      3.036387,
		     -1.423139,      0.783868,      1.455536,      0.924789,     -1.997757,
		     -1.584511,      0.352714,      2.648194,     -0.825440,     -0.552102,
		     -0.182056,     -0.903012,     -1.394876,      0.789493,     -1.061402,
		     -2.958207,     -0.972794,      0.961327,      3.640010,      0.782226,
		     -2.065470,     -0.811305,      0.546256,      1.092223,     -0.510603,
		     -2.715420,      0.134260,      1.016222,      0.255075,      0.229470,
		     -2.495228,     -1.935589,     -1.004428,      1.805915,      1.788123,
		      1.125290,      0.380053,     -0.635401,     -1.580299,      1.771954,
		     -2.166934,     -0.845977,      4.202907,      0.233082,     -4.042215,
		      0.269123,      2.743534,     -0.184454,     -1.830360,      0.274505,
		      0.029109,      1.244863,      0.903831,     -0.677562,      0.641173,
		     -0.895749,      0.141815,      2.704684,      1.288826,     -1.066362,
		      1.223504,      2.020127,     -1.291722,     -2.578430,     -1.599760,
		      0.858182,      1.804703,      1.700389,      0.294560,      0.460510,
		      0.031884,     -0.607647,      0.252691,     -0.367808,     -1.248194,
		      1.546052,      2.366493,      2.796254,     -1.479165,     -1.263736,
		     -0.275557,     -0.901028,      0.387616,     -0.147200,     -1.985239,
		     -1.024484,     -1.412198,      3.733757,     -1.006348,     -4.494435,
		      3.694738,      1.845962,     -3.852481,     -0.687208,      1.492238,
		      1.485468,     -0.610311,     -1.461778,     -1.144132,      1.683746,
		     -2.151370,      0.200248,      1.123553,     -1.289629,     -1.563163,
		      1.337808,      1.032992,     -0.790458,     -0.903469,     -0.205341,
		      2.467600,      2.059012,     -1.652089,     -2.256843,     -1.512009,
		     -0.521275,      0.599907,     -0.028804,      0.242225,      0.587282,
		      0.728983,      0.289276,     -1.354467,     -3.047312,     -1.657541,
		     -1.195377,      1.205896,      0.537304,      1.098204,     -0.776719,
		      0.333338,      0.157444,      0.645324,      1.503533,     -4.455598,
		     -0.816594,      4.921726,     -0.281946,     -3.064395,      0.930590,
		      1.817589,      0.155583,     -2.203995,      0.190171,      0.115805,
		      0.099417,      0.833165,      2.019145,      2.050965,     -0.933426,
		     -1.171425,      1.455894,      0.858352,     -0.804933,     -0.336252,
		      1.079481,     -0.763623,     -2.948814,     -2.092000,     -0.081328,
		      1.563053,      2.231521,      0.539348,      0.512456,      0.651995,
		      0.286165,      0.815919,     -0.470708,     -2.008742,     -1.544648,
		      1.024720,      2.819480,      0.375439,     -0.239371,     -0.502737,
		     -0.105081,      0.740072,     -1.367497,     -0.583926,     -0.794336,
		      0.124875,     -0.675477,      0.430277,      2.625367,     -2.143230,
		     -3.538088,      3.649019,      2.927107,     -2.636830,     -0.983062,
		      1.429622,      1.687866,      0.627716,      0.071874,     -1.031316,
		      0.079511,     -3.030812,     -0.458593,      1.824423,     -0.460430,
		     -2.698727,     -0.865675,      0.543603,      0.424554,      1.732077,
		      1.267990,      1.908804,      2.222561,      0.002444,     -0.680741,
		     -0.034752,     -1.664387,     -2.087667,     -0.990035,     -1.511552,
		      0.407096,      1.689596,      1.908914,     -0.225092,     -2.039751,
		      0.189711,      0.418850,      0.543921,     -0.306742,     -0.336081,
		      1.024122,      0.497338,      0.863508,     -0.078098,     -0.389473,
		      2.267840,      2.256323,     -4.896918,     -0.242498,      4.117607,
		     -2.081278,     -3.447428,      1.492842,      1.477032,      0.336427,
		     -2.263217,     -0.503349,      0.092087,     -0.056805,     -0.568342,
		      2.826027,      1.423499,     -1.132205,     -0.770135,      1.921284,
		      1.708009,      0.916689,     -0.938603,     -0.785258,     -0.732984,
		     -1.814284,     -1.964468,      0.141448,     -0.668041,      1.765380,
		      1.158060,      0.302320,      2.081571,      0.029288,      0.492780,
		      1.324684,      0.351791,     -0.870832,     -0.851677,     -1.306254,
		     -0.515274,      0.147690,      1.108337,     -1.247694,      0.399620,
		      0.744265,      0.040680,     -0.303335,      0.002513,      1.282622,
		     -1.889921,     -2.529771,      3.353999,      2.942516,     -2.313468,
		     -2.064104,      2.399807,      1.436036,     -2.335451,     -1.904249,
		      0.962652,      0.477899,     -0.727382,      0.563074,      1.149708,
		     -0.566514,     -1.823104,      0.469574,      2.345667,      0.681932,
		     -1.018650,     -1.411381,     -0.622322,     -1.617674,     -0.067856,
		      0.452460,      0.123912,      0.760499,      0.007051,     -0.500455,
		      0.908278,      0.145235,     -0.491647,      0.412900,     -1.360323,
		     -1.981559,     -2.383980,     -0.241884,      1.004204,      0.630180,
		      0.105363,      0.236709,      1.538949,      0.873118,     -1.616692,
		     -1.528742,     -1.540303,      2.393181,      1.079164,     -1.140781,
		     -2.535572,     -0.640308,      2.362847,      0.304566,     -5.141250,
		      0.956046,      5.073068,     -1.203395,     -3.121050,      1.676516,
		      2.145998,     -0.149248,     -1.410067,      0.374723,      0.265017,
		     -1.256313,     -1.337650,      3.494754,      1.689497,     -1.827823,
		     -2.180845,     -0.766804,      0.323231,      2.395054,      2.267068,
		      0.709728,      0.210777,      0.211355,     -0.349991,      0.301366,
		      0.607851,      0.765000,      1.649422,     -0.301526,      0.396872,
		      0.427534,      0.301726,      1.975506,      0.641859,     -0.573339,
		     -0.335076,     -0.521939,     -0.473075,     -1.928040,     -0.401249,
		      0.215817,      1.219193,      0.663887,     -2.206083,      0.393944,
		      1.671759,      1.237625,      2.458601,      2.802802,     -2.852116,
		     -2.983214,      4.811677,      1.648479,     -3.413599,      0.212552,
		      2.824600,      0.871497,     -1.602350,     -1.211310,      1.712611,
		      0.503939,     -1.144465,     -0.287112,      1.277631,     -2.671299,
		     -2.809418,      0.069801,      2.629518,      2.493659,      1.282159,
		     -0.521195,     -0.033141,     -0.903359,      0.082710,      0.437790,
		     -0.198816,     -0.958034,     -1.144972,     -2.394458,     -1.539700,
		      0.669498,     -0.758604,      0.111702,      0.343714,     -0.302776,
		     -1.050122,     -1.781625,      0.094871,      0.566592,      0.206660,
		      0.811942,      0.480451,      0.258859,     -1.097970,     -0.718562,
		      0.406062,     -2.029472,      0.254933,     -1.890310,     -0.589597,
		      0.183874,     -3.049486,      1.808802,      3.940216,     -2.065906,
		     -4.921238,      0.556605,      3.122647,     -1.003832,     -2.744236,
		      0.842607,      1.246094,     -1.680301,     -1.718437,      0.844546,
		      1.509708,     -0.188223,     -0.367517,      3.984027,      1.939102,
		     -0.778749,     -1.077850,     -2.157307,     -2.355944,     -0.929133,
		      1.435334,      0.201206,     -0.634704,      1.286093,      1.890625,
		      0.512816,      1.039145,      1.143598,      2.182568,     -0.169168,
		     -0.016057,     -0.398517,      0.208530,      0.324777,      1.379679,
		      0.248280,     -0.365782,     -0.792559,     -0.691284,     -1.245230,
		      0.471467,     -0.385311,     -0.080970,      1.845269,      0.042402,
		     -1.865408,      0.468175,      0.416784,      0.837489,     -2.318798,
		      3.565142,      0.499062,     -4.342084,     -1.840458,      5.605393,
		      1.035367,     -3.946017,      0.309100,      2.727369,      0.981881,
		      0.107144,      0.299644,      1.588134,      0.146562,     -1.256705,
		     -0.099881,      1.582866,     -3.042363,     -3.440237,     -1.537556,
		      0.337353,      1.341633,      2.549451,      1.199324,     -0.213950,
		      0.129103,      0.828297,      0.471216,     -1.209373,     -0.549021,
		     -0.547052,     -0.947663,     -3.451466,     -0.237391,     -0.370548,
		     -1.082925,     -0.091334,     -0.245621,      0.756538,     -0.567727,
		      0.046131,     -0.432438,     -1.494418,     -0.416878,     -1.529163,
		      0.062758,     -0.715845,     -0.790429,      0.064033,      0.030735,
		      0.341123,     -1.672535,     -1.330513,     -0.983934,      1.702500,
		     -1.823707,     -2.538521,      2.449551,      4.503022,     -2.679768,
		     -3.462880,      1.107132,      1.508710,     -1.106512,     -1.317495,
		      0.107224,      1.048529,     -1.056971,     -1.639256,     -0.087525,
		      0.172834,     -0.724200,      1.090805,      4.484118,      1.783612,
		      0.012209,      1.065584,     -0.257573,     -1.368225,     -1.459055,
		      1.384871,      0.011929,     -2.371560,     -0.059724,      2.412624,
		     -0.497948,     -0.782173,      0.469279,      2.027428,      0.981861,
		     -0.082660,     -0.556722,      0.841753,     -0.761858,     -0.264373,
		     -0.236453,      0.318608,      0.608319,      0.688536,      0.604397,
		      0.407241,      0.405995,     -0.350877,      0.573676,      1.231224,
		     -1.572625,      1.080023,     -0.518742,     -0.052762,     -0.266583,
		     -1.300163,     -1.145484,      3.744801,      0.061138,     -4.900939,
		     -0.450905,      5.238557,      0.672529,     -2.499870,      0.100668,
		      0.316933,     -0.787660,     -0.148502,      0.197370,      2.058730,
		      0.997478,      0.039225,      0.032425,      0.804308,     -2.340496,
		     -1.307632,     -0.942540,     -1.486873,     -1.311908,      0.162588,
		      1.030840,     -1.408947,     -0.709837,      1.322195,      1.644891,
		     -1.048999,      0.013221,      0.771843,      1.710543,     -2.584912,
		     -0.645389,      0.161494,      0.256298,     -0.302416,      0.107694,
		      0.263637,      0.770860,     -0.089186,     -1.349846,     -1.550004,
		      0.277170,     -1.211502,     -1.189354,     -1.403994,     -0.539795,
		      0.132048,      0.440418,     -0.797088,      1.696062,     -0.380019,
		      0.153692,     -0.481019,      1.483055,      2.683240,     -2.842845,
		     -1.758911,      2.913237,      3.098349,     -3.420021,     -1.635176,
		      1.643575,      0.839360,     -0.353415,     -0.294940,     -0.630279,
		     -0.062713,     -0.922404,     -0.927318,     -0.094820,     -0.808627,
		     -1.518750,      1.037220,      3.281885,      0.895187,     -0.640569,
		      2.095866,      1.277682,      0.165042,     -0.406534,      1.564300,
		      1.658702,     -2.750938,     -1.578568,      1.026041,      0.223696,
		     -2.352763,     -0.496676,      0.310176,      0.076670,      0.000539,
		     -0.390042,      1.217364,      0.132374,      0.596650,     -0.680597,
		     -0.677560,      0.562130,      1.312312,      2.039337,      0.938818,
		      1.217465,     -0.571932,      0.329928,      0.657796,      0.331497,
		     -0.260763,      0.488121,      1.348167,      0.515364,      0.563452,
		      0.293306,     -1.599775,     -0.386152,      2.668073,      0.035868,
		     -4.279135,     -0.020845,      2.463409,     -0.594006,     -0.493412,
		      0.790108,      0.726303,      0.728027,      0.999960,     -0.704094,
		      0.494411,      0.861655,      0.577698,      0.243383,     -1.214071,
		     -3.007180,     -0.715106,      1.259973,     -0.110468,     -1.520457,
		     -1.217938,      0.344354,     -1.469570,     -1.060356,      1.016017,
		      1.886114,     -0.875661,     -0.571360,      0.438485,      2.578959,
		     -0.135312,     -0.676440,      0.571301,      0.275964,     -0.194358,
		     -0.349249,     -0.303997,      0.504357,      0.010054,     -0.219432,
		     -1.652335,     -0.250155,     -0.207995,     -0.622546,      0.211241,
		     -1.101448,     -0.449606,     -0.582724,     -1.117826,     -0.105606,
		     -0.911219,      0.602281,      0.801027,      0.321625,     -2.166006,
		      0.272952,      2.708122,     -2.386119,     -1.673305,      3.165407,
		      2.119677,     -2.863954,     -0.506802,      0.308661,      0.310074,
		      0.112129,     -0.621602,     -1.700277,      0.257143,     -0.656223,
		     -1.361435,      0.432020,      0.348437,     -0.127868,      1.160024,
		      1.598225,      0.413306,     -1.574897,      1.301155,      2.692813,
		      1.126280,      0.416566,      0.666278,      1.979981,     -0.723149,
		     -0.723037,      0.541023,      0.186495,     -2.933878,     -0.834900,
		     -0.406619,      0.028894,     -0.487125,      0.129331,     -0.615839,
		      0.970287,      1.304084,      1.260731,     -0.906719,      0.720309,
		      0.928106,      1.444654,     -0.126778,      1.128761,      1.524673,
		      1.125970,      0.120573,      0.882659,      0.296092,      1.809669,
		     -0.422068,      0.749256,      0.896198,      1.267445,      1.272744,
		      1.264978,     -1.761697,     -0.320636,      2.184236,      0.162636,
		     -2.343880,      1.884341,      1.717113,     -0.792112,     -0.767815,
		     -0.972401,      0.037068,      1.541269,      1.382192,      0.066125,
		      0.823860,      0.470191,      0.515085,      0.976190,     -0.848943,
		     -1.972231,     -0.945909,      0.147997,     -0.265543,     -0.904351,
		     -1.397001,     -0.450540,     -1.541321,     -2.164451,     -0.027794,
		      1.586318,      0.970749,      0.458795,      1.610678,      1.834106,
		      1.104280,     -0.625951,      0.072089,      0.807471,     -0.453871,
		     -0.542842,     -1.900778,     -1.764101,     -0.625039,     -0.191152,
		     -1.481514,     -1.102716,     -0.284463,      0.453843,      0.975518,
		     -1.587680,     -1.143639,     -0.918928,     -1.757176,     -2.507472,
		     -2.006139,     -1.728566,     -0.502211,     -0.287266,      1.849218,
		      1.046048,     -2.004681,     -0.584983,      2.765647,     -2.215107,
		     -2.385391,      2.613841,      0.839602,     -2.426121,      0.166945,
		      0.420967,      1.187259,      1.383297,     -0.441856,     -1.770472,
		      0.302894,     -0.991227,     -2.682491,     -0.642185,     -0.445716,
		     -0.278171,      1.005227,      0.500866,      0.727344,     -1.616167,
		     -0.626061,      1.342969,      1.580339,      2.456437,      1.080981,
		      1.162066,     -0.704517,     -1.146076,     -0.591182,      0.413354,
		     -2.169306,     -1.039621,     -0.297960,     -0.408513,     -0.175968,
		     -0.120771,     -0.317867,      1.064604,      1.971444,      2.667024,
		     -0.368642,      1.721370,      1.032360,      0.110681,     -1.256409,
		     -0.630618,      1.203971,      1.302610,      1.721054,      1.460080,
		      2.124864,      2.072220,      0.913736,     -0.053245,     -0.568741,
		     -1.559224,     -0.105653,     -1.924484,      1.241740,      1.759915,
		     -2.138751,     -0.051802,      2.658779,     -1.229946,     -2.755868,
		      2.138236,      1.188132,      0.319257,     -0.246283,     -1.906007,
		     -0.300274,      1.240574,      0.316841,      0.878006,      2.363207,
		      0.923108,     -0.108536,      0.895318,     -0.056441,      0.481877,
		      0.383323,      0.822235,     -0.150843,     -0.751099,     -0.940527,
		     -1.094611,     -1.034712,     -1.817080,     -0.617080,     -0.095658,
		      0.444750,      0.249324,      1.437231,      0.837575,      1.337542,
		      0.349254,      0.381059,     -0.240931,      0.486555,     -0.625856,
		     -0.676656,     -2.791027,     -1.588233,     -1.266183,     -1.493787,
		     -1.002896,      0.699456,      1.166536,      0.875291,     -1.509928,
		     -2.991735,     -0.925766,      0.086438,     -1.235422,     -1.604991,
		     -1.231007,     -0.074595,      0.254552,     -0.831144,      0.045759,
		      1.363441,      0.553576,     -1.238584,      0.124960,      2.842780,
		     -2.017691,     -1.688842,      2.300638,     -0.118016,     -1.861545,
		     -0.639524,     -0.347293,      0.775602,      1.113937,     -0.024704,
		     -0.605224,      0.898933,     -0.676588,     -2.119708,     -0.053918,
		     -0.225888,     -0.722932,     -0.726304,     -1.175212,      0.830378,
		      0.681777,     -0.225113,     -0.062147,     -0.646623,      1.654891,
		      1.779359,      1.371413,      0.430371,     -0.595764,     -0.856167,
		     -0.113913,     -1.092264,     -0.887143,     -0.285419,      0.139173,
		     -0.322526,     -0.403476,     -0.882726,      0.271109,      1.218656,
		      0.890787,     -0.595963,      0.649947,      0.647965,      1.323512,
		      0.807341,     -0.451206,      0.268195,      0.379213,      0.378549,
		     -0.156839,      1.167705,      1.847767,      1.391961,      0.040983,
		      0.981560,      0.203759,     -0.341756,     -1.462658,     -0.843792,
		     -1.327188,      1.467843,      0.479356,     -3.497206,      0.454886,
		      3.301975,     -0.883986,     -2.159162,      1.732491,      0.530127,
		      0.579787,      0.315702,     -1.177483,     -0.135321,      0.616204,
		     -1.625770,     -0.022540,      2.423849,      0.384354,     -0.927288,
		      0.199805,      0.023198,      0.917969,      0.381440,     -0.410479,
		     -0.489955,     -0.314539,      0.342358,     -0.960575,     -1.109300,
		     -2.014437,     -0.561585,     -0.452332,     -0.036681,      0.058737,
		      1.272182,      0.748465,      1.005724,      0.925291,      0.276009,
		     -0.101186,     -0.058218,     -0.084482,      0.143782,     -1.528534,
		     -0.261837,     -0.702919,     -0.789074,     -0.856583,      0.134744,
		      1.051317,      1.302052,      0.036840,     -0.987531,      0.328926,
		     -0.183561,     -0.284206,     -0.214132,      0.052223,      0.082846,
		      0.677368,     -0.247630,      0.849551,     -0.802592,      0.171591,
		      2.255661,     -1.755447,     -1.204938,      1.822295,      2.194642,
		     -2.415593,     -1.029065,      2.325758,     -0.335431,     -0.349657,
		     -0.737540,     -0.649632,      0.201565,      0.483038,     -0.299658,
		      0.146146,      0.414527,     -1.096768,     -1.015997,      0.585840,
		      0.874845,     -0.108197,     -1.427318,     -2.063192,      0.177138,
		      1.573883,      1.497082,      0.891891,     -1.120001,      0.002667,
		      1.390481,      1.323723,      1.200552,      0.085902,     -1.160180,
		     -0.720234,     -1.553447,     -1.802467,     -0.832549,      0.512853,
		      1.160603,     -0.995051,     -0.573777,      0.326660,      1.634399,
		      0.870562,     -0.927607,     -1.206599,     -1.251686,     -1.102604,
		      0.358150,     -0.386329,     -0.490202,      0.591475,      0.271305,
		     -0.281408,      0.190911,      0.041143,     -1.092598,     -0.876761,
		      0.650340,      1.424486,      0.558053,      1.056979,     -0.270475,
		      0.514991,     -2.115191,      0.718436,      2.530310,      0.108300,
		     -3.353160,      0.371117,      2.430787,     -0.635887,     -1.137475,
		      0.617588,      0.101094,      0.097712,      0.322300,     -0.644481,
		     -0.057571,      0.789085,     -0.700997,      0.042427,      1.391386,
		      0.333558,     -0.871743,      0.401210,      1.302206,      1.810999,
		     -0.066825,     -1.598953,     -1.566762,     -1.475323,      1.088192,
		      0.400921,      0.001120,     -1.585813,     -0.978311,     -0.153867,
		      0.461519,      0.341617,      1.161652,      0.568985,      0.968249,
		      0.482800,      0.164944,      0.286968,      0.854851,      0.007487,
		     -1.141739,     -1.024732,      0.462503,      1.268869,      2.225886,
		      1.136704,     -0.697047,     -0.775708,     -0.099209,      1.260512,
		      0.867833,      0.490106,      0.507172,      0.861889,      1.008983,
		      0.669801,     -0.505016,      0.648629,      0.008802,      0.560581,
		     -0.180884,     -0.282761,     -1.359655,      1.784466,      1.512691,
		     -2.710438,     -1.141281,      2.474025,      1.542819,     -1.824882,
		     -0.802150,      0.785520,     -1.007814,      0.707476,      0.111675,
		     -0.788009,     -0.260994,      0.254828,     -0.065962,      0.687152,
		      0.666188,     -1.175260,     -1.037270,     -0.246410,      0.342828,
		     -0.458214,     -1.785181,     -1.627946,      0.006248,      1.265543,
		      0.942412,      1.017310,     -0.122364,     -1.616469,      0.426090,
		      0.626399,      0.783563,      0.204799,     -0.644645,      0.079089,
		     -0.143167,     -1.902638,     -1.487554,      0.110314,      0.847495,
		     -0.548058,     -0.949465,     -0.369407,      1.800880,      0.823650,
		     -0.513457,     -1.474598,     -1.335695,     -0.559401,      0.229796,
		     -0.184418,     -0.988548,     -1.039929,      0.286661,     -0.230243,
		     -0.857394,     -0.380273,     -0.200049,     -1.147594,     -1.390940,
		     -0.438959,     -0.112931,      1.389458,      0.577752,     -0.456704,
		      1.575087,     -0.475859,     -2.741780,      0.921282,      1.633325,
		     -0.974660,     -2.306623,      0.187805,      1.334303,      0.581314,
		      0.503546,     -0.252649,      0.379554,      1.119284,      0.641192,
		     -0.436513,     -0.576752,     -0.260714,     -0.732567,      0.590839,
		      0.212972,     -0.422130,     -1.315889,     -0.176720,      2.011408,
		      2.806943,      1.168884,     -1.046054,     -0.918462,     -1.665513,
		     -0.261396,      0.716588,      1.039282,      0.031949,     -0.601682,
		     -0.572425,      0.112111,     -0.481339,      0.375511,      1.099795,
		      1.635607,      0.737217,     -0.640976,      0.569762,      1.991194,
		      1.134739,     -1.100010,     -1.591159,     -0.557992,      0.554986,
		      1.400124,      1.661631,      0.254658,     -0.829139,     -0.093635,
		      0.915838,      0.462430,      0.598029,      0.244957,      0.529941,
		      0.860284,      0.271395,      0.604274,      1.034574,      0.025717,
		     -0.559309,      1.016279,      0.430964,      0.357710,     -1.446157,
		     -1.697643,      2.647887,      0.971073,     -2.071814,     -0.510293,
		      1.689026,      0.935869,     -0.552574,     -0.371663,      0.096045,
		     -0.683502,      0.222914,     -0.438186,     -1.098122,      0.177546,
		      0.513090,     -0.551174,      0.009762,      0.858678,      0.893979,
		      1.061962,      1.243252,      0.452507,     -1.519416,     -2.873753,
		     -1.572701,      0.498426,      1.417328,      0.714830,      0.312785,
		      0.673615,     -1.982241,     -1.172769,     -0.044370,      1.072730,
		      0.349040,     -0.411746,     -0.125428,      0.525464,     -1.980652,
		     -0.834272,     -0.078350,      1.257341,     -0.958885,     -1.455318,
		     -0.417894,      1.301825,      1.520484,     -0.118135,     -1.640917,
		     -0.890580,     -0.794815,     -0.069966,      0.737357,     -0.569428,
		     -1.184462,     -0.454736,      0.493089,     -0.507091,     -0.570624,
		     -0.019022,      0.558160,     -0.069948,     -0.489704,     -1.146210,
		     -0.850865,     -0.007963,     -0.311507,     -0.304833,      0.409663,
		      2.339781,     -0.846635,     -1.452437,      1.776287,      1.839600,
		     -0.849926,     -1.525488,     -0.460003,     -0.215068,      0.674554,
		      0.864839,     -1.775484,     -0.679318,      1.338787,      0.790132,
		      0.324625,      0.375046,      0.504228,     -0.726245,     -0.155795,
		     -0.396421,     -0.601456,     -1.236392,     -0.536927,      1.145655,
		      1.367763,      0.153826,     -1.400995,     -0.608130,     -0.418223,
		     -0.358854,      0.833144,      0.846424,      0.827725,      0.037209,
		      0.190325,      0.524169,      0.115018,     -0.822361,      0.503106,
		      0.528554,      0.431642,     -0.871080,     -0.297862,      1.945400,
		      1.849712,     -0.419524,     -1.230507,     -0.395902,     -0.072253,
		      0.545895,      0.506035,      0.643813,      0.355451,      0.837893,
		      0.891883,      0.186706,     -0.598807,      0.011555,      0.410823,
		      0.785690,     -0.468762,     -1.008331,      0.898136,      1.597886,
		     -0.274326,     -0.117210,      0.059299,      0.684718,      0.478960,
		      0.661073,     -0.936480,     -1.341406,      1.743824,     -0.281347,
		     -1.892493,      0.085697,      1.375824,      0.500671,      0.121900,
		     -0.516830,     -0.413180,      1.649443,      1.701266,     -0.852585,
		     -1.844989,     -0.190223,      0.606602,     -0.848007,     -0.463914,
		     -0.580241,     -0.358533,      0.187228,      2.072891,      1.800087,
		      0.039642,     -1.875249,     -1.641048,      0.403094,      1.313197,
		      0.217437,     -0.406910,      0.878244,     -0.836961,     -1.562153,
		     -1.425319,     -0.381749,      0.216215,      0.204532,      0.460597,
		      1.420676,     -1.045348,     -0.457645,      0.154652,      1.125826,
		     -0.210074,     -1.391788,     -0.978502,      0.176437,      0.739358,
		     -0.113526,     -1.243481,     -0.668841,     -0.376924,     -0.323640,
		     -0.713328,     -0.928095,     -0.196302,      0.267862,      1.592553,
		     -0.482682,     -1.134613,      0.531900,      1.358130,     -0.042423,
		     -0.523330,     -1.107833,     -0.174834,      0.288868,     -0.884419,
		     -0.018827,      0.107906,     -0.715369,     -0.424790,      1.651530,
		     -1.270083,     -1.467393,      0.997062,      1.214340,     -0.782828,
		     -0.782610,     -0.061999,     -0.592410,      0.826822,      0.408167,
		     -1.972076,     -0.298449,      1.197784,     -0.256742,     -0.439989,
		      1.147039,      1.110721,     -0.079531,      0.414889,      0.285904,
		     -1.285270,     -1.763271,     -0.381990,      1.020103,      1.143852,
		      0.950834,     -0.762514,     -0.726398,      0.044304,     -0.647935,
		      0.874453,      0.520298,      0.673936,     -0.253032,     -0.077965,
		     -0.280829,      0.681737,     -0.579257,      0.865970,     -0.530519,
		     -0.039442,     -1.381503,     -0.525831,      0.561564,      1.247860,
		      0.513673,     -0.213946,     -0.178099,      0.781235,      0.596755,
		      0.293340,      0.365484,      0.938788,      1.157098,      0.048637,
		      0.035745,     -0.726790,     -0.138308,      0.211277,      1.272242,
		      0.017825,     -0.322686,      0.743308,      1.170355,      0.076075,
		     -0.439915,     -0.104380,     -0.816757,      0.027243,     -0.255023,
		     -0.777626,      0.996127,      0.626331,     -0.445966,      1.234606,
		      0.196152,     -1.585669,     -0.233764,      0.868004,      0.215251,
		      0.210157,     -1.491673,     -1.547863,      1.161727,      1.672026,
		      0.194308,     -0.419783,      0.021041,     -0.517958,     -0.992218,
		      0.812765,      0.417845,     -0.217188,     -0.684348,      0.969705,
		      0.985216,      0.409409,     -0.750995,     -1.745927,     -0.615829,
		      0.346591,      0.021793,     -0.533886,     -0.128722,      0.391593,
		     -0.113632,     -0.520808,     -0.993963,     -0.434474,     -0.435922,
		      0.283864,      0.504163,     -0.015713,     -0.322976,     -0.129228,
		      0.283145,     -0.095625,     -0.694688,     -0.251025,      0.125571,
		     -0.328756,     -0.272182,     -1.281693,     -0.603450,      0.141195,
		     -0.432210,     -1.226687,     -0.987921,     -0.934767,      0.628725,
		      1.339077,      0.717400,     -0.819494,     -0.473992,      0.178098,
		     -0.798305,     -0.354593,     -0.429359,     -0.371888,      0.350718,
		      0.685043,     -0.558795,      0.838714,      0.723709,      1.333871,
		      0.066804,     -1.847701,      1.039649,      0.042659,     -0.900954,
		      1.100349,      1.097005,     -0.251241,     -0.360146,      0.317514,
		      0.735443,      1.106941,     -0.334354,     -1.628109,     -0.323743,
		      0.333510,     -0.505735,     -0.458081,      1.260013,     -0.188463,
		     -1.071660,      0.076103,      1.386188,     -0.178781,     -1.313308,
		     -0.813153,      0.294671,      0.455426,      1.253795,      0.533157,
		     -0.037478,      0.241364,     -0.312859,      1.158990,      0.708250,
		      0.594283,      0.597210,      0.240825,     -0.620433,      0.042382,
		     -0.335062,      1.126658,     -0.309441,     -0.331928,     -0.805023,
		     -0.326597,      0.028539,      0.087629,     -0.092759,      0.111047,
		      0.242046,      1.161715,      0.813431,      0.463211,      0.085064,
		      0.992690,      0.952434,      0.934824,      0.798753,     -1.298239,
		     -0.355584,     -0.011371,      0.745045,      0.931124,      0.120519,
		     -0.068981,      1.267846,      0.392179,     -0.263134,     -0.719524,
		     -0.714757,     -0.885464,     -0.905630,      0.127953,     -0.704720,
		     -2.295960,      0.714405,      0.564119,     -0.356650,      0.791603,
		     -0.056162,     -1.066406,     -0.416502,      0.515502,      0.489885,
		      0.564880,     -0.687651,     -0.514464,      0.421627,     -0.086951,
		      0.105612,      1.281128,      0.963698,     -1.714844,     -1.766131,
		      0.695272,      0.940061,     -0.035579,     -0.247338,      0.574694,
		     -0.128213,      0.389728,      0.747898,     -1.124600,     -0.878990,
		     -0.748612,     -0.295502,     -0.501824,     -1.882283,     -1.091829,
		     -0.135621,      0.799079,     -0.481881,      0.098975,     -0.699035,
		      0.057112,     -0.501176,      0.655843,      0.153745,      0.835718,
		     -0.416729,      0.208338,     -0.330933,      0.506208,     -0.421889,
		     -0.934005,     -1.006716,     -1.193062,     -0.765375,      0.033706,
		      0.231660,     -0.514727,     -1.003986,     -0.970782,      0.583365,
		      0.163850,      0.439183,      0.112441,      0.013210,     -0.084527,
		     -0.921723,     -0.968495,     -0.182135,      0.260080,      0.725277,
		      0.938650,      0.097033,      0.558049,      0.260915,      0.035801,
		     -0.566316,      2.882526,      1.122724,     -1.648465,      0.150547,
		      0.013478,     -1.092191,      0.591628,      0.596119,     -0.194260,
		     -0.654671,     -1.075175,      0.341224,      0.890611,     -0.153656,
		     -0.127429,      0.453742,     -0.906155,     -1.296160,      0.574263,
		      3.132897,      0.691637,     -1.517532,     -0.943378,      0.279569,
		      0.170505,      0.039804,      0.056519,     -0.675898,     -0.618067,
		      1.090127,      1.825325,      0.773245,      0.314920,      0.627250,
		      1.353083,      0.305490,     -1.130967,     -0.265774,      0.222618,
		      0.052084,     -0.062494,      0.159182,      0.159205,     -0.030452,
		     -0.747549,     -0.400169,     -0.351699,      0.421350,      0.102802,
		      0.159291,      0.730339,      0.169586,      0.829669,      1.078001,
		      0.813264,      0.099418,     -0.256884,     -0.182153,      0.873638,
		      0.224856,      0.090826,      0.419237,      0.534713,     -0.001735,
		     -0.161376,      0.345315,      0.134395,      0.202657,     -0.185646,
		     -0.585966,     -0.658878,     -0.402713,     -0.382910,      0.709945,
		      0.097511,      0.469373,      2.093219,     -0.894993,     -1.916202,
		      0.851105,      0.119409,     -0.247136,      1.395044,      0.223847,
		     -0.521657,     -0.664042,     -0.396520,      0.658579,      0.410415,
		     -0.371409,      0.475184,      0.471404,     -1.162295,     -0.495969,
		      1.582130,      1.738031,     -1.803536,     -1.972947,      0.141573,
		      0.908221,      0.529591,      0.654402,      0.982871,     -0.904574,
		     -0.348543,      1.313910,     -0.767565,     -2.057290,     -1.825354,
		     -0.301437,      0.415558,     -1.341890,     -0.885797,     -0.050196,
		      1.129436,      0.684269,      0.695200,     -0.204586,     -0.259802,
		     -0.879285,      0.236819,      0.200258,      1.017300,      0.155876,
		     -0.305683,     -0.701625,      0.389985,     -0.127694,     -0.322217,
		     -1.237909,     -0.941260,     -1.548417,     -0.046970,      0.999846,
		      0.961268,     -0.403377,     -0.847893,     -0.149886,     -1.009620,
		      0.448250,      0.900904,      0.514679,      0.583521,      0.008057,
		     -0.670172,      0.134227,      0.627086,      0.909070,      0.665767,
		     -0.593308,     -0.868319,      0.238025,      0.334727,      0.008074,
		     -2.553935,     -1.127784,      1.651093,      0.276201,     -0.527546,
		      0.357300,     -0.408278,     -0.953796,      0.175774,      0.784778,
		      1.345599,      0.563164,     -0.926463,      0.027586,     -0.038658,
		     -0.631520,      0.863722,      1.286811,     -1.220448,     -2.858792,
		     -1.095222,      2.003894,      1.564415,     -0.218481,     -0.793372,
		     -1.546244,     -1.573471,      0.590657,      1.371118,     -0.022700,
		     -0.339359,      1.204857,      2.022209,      0.667028,     -0.963814,
		      0.550161,      1.761399,      0.655270,     -1.250780,     -0.786561,
		     -1.175494,     -0.543319,     -0.267615,      1.119013,      0.255901,
		      0.239704,     -0.808252,      0.244980,      0.014976,      1.229338,
		     -0.008377,      0.246015,      1.031987,      0.708025,      0.596865,
		      0.098397,      0.751910,     -0.107060,     -0.225737,     -0.025112,
		      0.363376,      0.040557,      0.231358,     -0.127680,      0.353209,
		      0.081496,     -0.791867,      0.093907,      0.584751,      0.649690,
		     -0.248419,     -1.469843,     -0.964495,      0.130416,      0.478632,
		      0.309800,      0.452790,      0.400349,      0.089989,      0.559809,
		      1.978756,     -0.695230,     -0.543294,      0.471455,     -0.829391,
		     -0.488836,      1.199741,      0.605777,      0.075805,     -0.961095,
		     -1.534758,      0.361120,      0.027706,     -0.294042,      0.438560,
		      0.597355,     -0.805951,     -0.620118,      1.564859,      2.034900,
		     -0.637831,     -1.496641,     -0.554909,      0.307748,      1.404387,
		      1.965027,      1.251528,     -0.910892,     -0.815794,      1.187114,
		     -0.194396,     -1.920867,     -1.849227,     -0.165377,     -0.069295,
		     -1.536843,     -0.925913,      0.133958,      0.867695,      0.909455,
		      1.517437,      0.685969,     -0.875270,     -1.027912,     -0.299620,
		     -0.067958,      0.028860,      0.489700,     -0.443729,     -1.182861,
		     -1.051912,     -0.875209,     -0.014976,     -0.007482,      0.381795,
		     -0.416072,     -0.954796,     -0.014149,      0.298975,     -0.310568,
		     -0.548444,      0.031422,     -0.062753,     -0.126549,     -0.139022,
		      0.780873,      0.759681,      0.222592,     -0.767827,      0.039350,
		      0.708415,      0.615257,     -0.004149,     -0.540758,     -0.154136,
		      0.398438,      0.069687,      0.243429,      0.201573,      1.274009,
		     -1.883546,     -0.132113,      0.743289,     -0.308132,      0.642601,
		      1.083708,     -0.730880,     -1.088135,     -0.906323,     -0.263794,
		      1.411604,      0.632255,     -0.643305,      0.021385,     -0.017990,
		     -1.300793,      0.128728,      1.849461,      0.798835,     -1.135560,
		     -1.081004,      0.576761,      0.897974,      0.270477,     -0.054080,
		     -1.743196,     -2.852350,     -0.557592,      0.575135,     -0.832390,
		     -0.323949,      1.945525,      2.796280,      1.366669,     -0.773011,
		      0.452800,      1.103632,      0.480384,     -0.899132,     -0.038030,
		     -0.774461,     -1.230033,     -0.756413,      1.026513,      0.634165,
		      0.809827,     -0.019413,      0.160838,      0.071904,      1.673318,
		      1.052305,      0.502294,      0.484368,      1.015026,     -0.558446,
		      0.216219,      0.676095,      0.361972,     -0.029118,     -0.206360,
		     -0.156542,     -0.059983,      0.378570,      0.347865,      0.513666,
		      0.284124,     -0.813501,      0.049693,      0.709633,     -0.084160,
		     -0.332452,     -0.173614,     -0.350751,     -0.212671,     -0.877790,
		     -0.083211,      0.446275,     -0.437710,     -0.520753,     -0.573507,
		     -1.584289,      0.397949,      0.930100,     -0.621652,      0.352007,
		      0.315307,     -1.271936,     -0.488016,      0.760276,      1.017391,
		      0.927104,     -0.622165,     -0.994909,      0.819907,      0.000332,
		     -0.486806,      0.176128,      0.499963,     -0.523860,     -1.454691,
		     -0.092552,      1.197689,      0.085940,     -1.040159,     -0.924729,
		     -0.857140,      0.970438,      2.631714,      1.463358,      0.605890,
		      0.475247,      0.845041,     -0.580540,     -2.424527,     -1.976744,
		      0.260134,      0.106191,     -1.094388,     -0.610843,      0.314222,
		     -0.000052,      0.093638,      1.342650,      1.820121,     -0.582412,
		     -1.365127,     -0.869921,     -0.325967,     -1.125068,      0.146201,
		     -0.541221,     -0.932334,     -1.615614,     -1.292162,     -0.479572,
		     -0.479299,      0.920936,      0.525252,     -0.047891,     -0.336220,
		      0.026241,     -0.429057,     -1.351754,     -0.425530,      0.623016,
		      0.351004,     -0.087539,      0.186918,     -0.003444,      0.696745,
		     -0.247606,     -0.008937,     -0.280133,     -0.332904,      0.385809,
		      0.447558,     -0.526960,     -0.069183,      0.986693,      0.445626,
		      0.489114,     -0.054847,      0.765077,      1.405206,     -1.087949,
		     -0.292001,     -0.083702,     -1.013350,      0.743510,      1.552133,
		     -0.165287,     -0.546198,     -0.780924,     -0.205679,      0.988331,
		      0.005044,      0.005168,      0.814267,      0.912261,     -0.903323,
		     -1.470921,      0.317348,      0.717996,     -0.093479,     -0.292144,
		      0.628554,      0.912069,      0.603621,      0.239900,     -0.714214,
		     -1.600813,     -0.583371,     -0.513162,     -1.385458,     -0.871936,
		      1.511568,      2.364625,      1.342046,     -0.200418,      0.502067,
		      0.421501,     -0.100708,     -1.018112,      0.380363,      0.026309,
		     -1.076612,     -0.994749,      0.490666,      0.431552,      0.229290,
		      0.918408,      0.470455,      0.236133,      1.231209,      1.731272,
		      1.137017,      0.522646,      0.332198,     -1.239688,     -1.789454,
		     -0.399791,      0.181247,      0.502652,      0.428066,      0.317560,
		     -0.266155,     -0.324873,     -0.189053,      0.248803,      0.803452,
		      0.743860,     -0.344627,      0.326595,     -0.320943,     -0.138589,
		      0.062375,     -0.109750,      0.085725,     -0.138746,     -0.350957,
		     -0.415089,      0.014994,     -0.744586,     -0.981798,     -0.162317,
		      0.356164,     -2.017651,      0.395368,      0.799205,     -0.204588,
		      1.054964,      0.228673,     -1.472383,     -0.452855,      0.239398,
		      0.339295,      0.339060,     -1.040640,     -0.519868,      0.422715,
		     -0.637323,     -0.973560,      0.164173,      1.487405,      0.708246,
		     -0.291978,     -0.324913,      0.304086,     -0.078496,     -0.682824,
		     -0.286802,     -0.714313,     -0.162114,      1.256162,      0.055359,
		      0.904365,      1.793382,      1.576752,     -0.515643,     -1.982448,
		     -1.926679,     -0.142411,      0.076022,      0.043067,      0.240884,
		      1.055751,      0.042782,     -0.846475,      0.103746,      1.742367,
		      0.394373,     -0.414731,     -0.404332,     -0.099654,     -1.313208,
		     -0.816594,     -1.003126,     -1.211386,     -1.332813,     -0.381994,
		     -0.187507,      0.337097,      0.299802,      0.748483,      0.036748,
		     -0.663634,     -0.313338,     -0.143121,     -0.539756,     -0.384424,
		     -0.158274,      0.383335,     -0.070053,     -0.352543,     -0.407347,
		      0.218595,      0.363872,     -0.043869,     -0.202286,     -0.202244,
		     -0.158067,      0.442356,      0.495237,      0.402782,     -0.323623,
		     -0.104347,      0.735783,     -0.062885,     -0.401923,     -1.416929,
		      1.111160,      1.175674,     -0.138468,     -0.052042,     -0.184926,
		     -0.790181,      0.427774,      0.808041,     -0.138929,     -0.054951,
		     -0.002920,      0.637899,     -0.020862,     -1.007000,      0.076475,
		      0.832006,      1.228908,     -0.017698,     -1.349165,     -0.326420,
		      0.203680,     -0.221509,      0.205367,      0.985257,      0.407389,
		     -0.243994,     -0.471850,     -0.651051,     -0.092580,     -0.294154,
		     -1.042133,     -1.593712,     -0.518743,      1.230213,      1.765438,
		      0.754609,      0.014157,      0.323386,     -0.309905,     -0.498961,
		     -0.949956,     -0.049659,      0.600891,     -0.442634,     -0.623140,
		      0.113060,      0.105541,     -0.390840,      0.512804,      0.892411,
		      0.551419,      0.716202,      0.986214,      0.874772,      0.228332,
		      0.828767,     -0.325238,     -1.059632,     -1.206718,     -0.333956,
		      0.018445,      0.324999,      0.243012,     -0.153120,     -0.283006,
		     -0.900264,     -0.615447,      0.734000,      1.267832,      0.571671,
		      0.697314,     -0.595855,     -0.784944,      0.180353,      0.084714,
		     -0.307717,     -0.570959,     -0.670594,      0.262596,      0.629324,
		     -0.459046,      0.319566,      0.841539,      0.285537,      1.326823,
		      0.176439,     -2.142381,     -0.318366,      0.165017,     -0.409570,
		      0.794613,      0.518531,     -0.495716,     -0.124872,      0.271756,
		      0.030413,      0.099452,      0.223809,      1.017859,      0.350695,
		     -1.114162,     -1.635276,     -0.837471,      1.149413,      0.646117,
		      0.219160,     -0.133611,     -0.266213,     -0.997554,     -0.808549,
		      0.515263,      0.935169,      0.607449,      0.466601,     -0.734714,
		      0.594227,      1.963708,      1.668243,     -0.497720,     -1.157057,
		     -0.872779,     -0.527994,     -0.775880,      0.288785,      0.372731,
		      1.376110,      0.593697,     -0.143559,     -0.653912,      0.402790,
		     -0.120933,      0.214980,      0.572238,      0.985117,     -0.289823,
		     -0.532664,     -0.731014,     -1.193652,     -1.396069,     -0.674645,
		     -0.213841,      0.247774,      0.576383,      0.795266,      0.696506,
		      0.083171,     -0.206233,     -0.137172,      0.283885,     -0.069199,
		      0.322094,      0.437740,     -0.467261,     -1.347892,     -1.074905,
		     -0.903895,      0.544277,      0.435354,      0.203154,     -0.102301,
		      0.300341,      0.567834,      0.426955,     -0.038474,     -0.524627,
		     -0.175195,     -0.701879,     -0.144953,     -0.147857,     -0.090069,
		     -0.722791,     -1.496630,      1.313806,      0.407700,     -0.045288,
		      0.129411,     -0.056793,     -0.405635,     -0.154720,      0.109463,
		     -0.292175,     -0.197199,     -0.158233,      0.538123,     -0.669613,
		     -1.083249,      0.760644,      1.052483,      0.888138,      0.022657,
		     -0.790703,     -0.603525,      0.023419,     -0.247796,      0.623156,
		      1.400323,      0.770602,     -0.594273,     -1.059653,     -0.874240,
		      0.954593,      0.180685,     -1.405739,     -1.889310,     -0.388853,
		      0.881147,      1.166208,      0.390034,      0.589387,      0.149058,
		     -0.568554,     -1.167274,     -0.834793,     -0.507179,      0.306796,
		      0.114366,      0.350066,      0.140609,      0.032607,     -1.213378,
		     -0.589568,      0.344105,      0.946747,      1.116129,      0.774345,
		      0.468621,      0.023207,     -0.348821,      0.261090,     -0.268962,
		     -0.209853,     -0.363131,     -0.151544,      0.035489,     -0.196692,
		     -0.153652,     -0.258036,     -0.519965,      0.148455,      0.492394,
		      0.484121,      0.452552,      0.576550,      0.030441,     -0.036378,
		      0.436131,      0.142973,     -0.806948,     -0.742486,     -0.375106,
		      0.144478,      0.610514,      0.757165,      0.246215,      0.446666,
		      1.058862,      0.437881,      0.225364,      1.782466,      0.124891,
		     -0.584673,      0.125613,      0.100121,     -0.581459,     -0.360763,
		      0.351843,      0.311299,     -0.253004,     -0.187440,     -0.419844,
		     -0.797757,      0.449373,      1.542835,      0.277356,     -0.843031,
		     -1.173408,     -1.083456,      0.650190,      0.871124,      0.820894,
		      0.571494,     -0.407793,     -1.511797,     -1.657917,     -0.299639,
		      0.783437,      1.115786,     -0.264418,     -0.858059,      0.356539,
		      1.976641,      1.454088,     -0.912047,     -1.090119,      0.291930,
		      0.234657,     -0.516600,      0.758854,      0.749921,      0.991082,
		      0.843312,      0.540533,      0.089337,     -0.085796,     -0.997714,
		     -0.410531,      0.390715,      1.436407,      0.712010,      0.105765,
		     -0.654381,     -0.657999,     -0.735470,     -0.300783,     -0.323343,
		      0.631280,      0.524067,      0.370771,      0.440747,      0.851818,
		      0.157163,      0.072784,     -0.063725,     -0.299195,     -0.038765,
		      0.191710,      0.295893,     -0.250236,     -0.644546,     -1.497896,
		     -0.380767,     -0.357574,     -0.440975,      0.216054,      0.686514,
		      0.795150,      0.537336,      0.030300,     -0.667868,     -0.365928,
		     -0.327325,     -0.152216,     -0.420431,     -0.154196,      0.039682,
		      0.296769,     -1.015816,     -0.730922,      0.593994,     -0.399411,
		     -0.105879,      0.591435,      0.676453,      0.113112,     -0.777007,
		      0.238316,      0.970889,      0.683984,      0.221080,      0.032280,
		     -1.050179,     -1.059627,      0.706704,      1.036600,      1.291837,
		      0.070649,     -0.784816,     -1.275510,     -0.612856,     -0.190346,
		      0.632015,      1.535940,      1.255075,      0.061324,     -1.041428,
		     -1.191764,      0.751425,      0.460031,     -1.036920,     -1.532415,
		      0.134143,      0.925508,      0.457440,     -0.566097,      0.323669,
		      0.129548,     -0.228916,     -1.151904,     -0.701105,     -1.063235,
		     -0.293357,     -0.281624,      1.232499,      0.869766,      0.933471,
		     -0.757976,     -1.472284,     -1.162600,     -0.004062,      0.958194,
		      0.724865,      0.428811,      0.216275,     -0.456527,     -0.529459,
		      0.012265,      0.394116,      0.318220,     -0.368048,     -0.177305,
		     -0.415486,     -0.330346,     -0.603097,     -0.243404,      0.187934,
		      0.196097,      0.372500,      0.862773,      0.334533,      0.166808,
		      0.429510,      0.671005,      0.102954,     -0.921849,     -0.800854,
		     -0.198594,     -0.032439,      0.082310,      0.560696,     -0.044079,
		      0.187097,      0.057886,     -0.343257,     -0.121317,     -0.641179,
		      0.118481,      0.406679,     -0.485636,      0.397827,      0.237106,
		      0.040753,     -0.445866,     -0.904935,      0.759294,      0.867340,
		     -1.075595,     -0.914739,      0.151021,     -0.325610,      0.272092,
		      0.761949,      0.015810,     -0.272852,     -1.132819,     -1.198250,
		     -0.086413,      1.085446,      0.877252,      0.725091,     -0.005059,
		     -0.974803,     -1.402442,     -0.730456,      0.508182,      1.690969,
		     -0.071579,     -0.649543,     -0.079746,      1.332347,      1.190442,
		     -0.832768,     -1.025772,      0.413614,      0.457301,     -0.396603,
		      0.242918,      1.035235,      0.579414,      0.678999,      0.701379,
		      0.696815,     -0.205047,     -0.831460,     -1.001805,     -0.281219,
		      0.943211,      1.453600,      1.114022,     -0.273877,     -0.715318,
		     -0.439940,     -0.263248,      0.129510,      0.897062,      1.023702,
		     -0.576663,     -0.423125,      0.473591,      0.693227,      0.273521,
		      0.208635,      0.289140,     -0.065007,     -0.551321,     -0.107448,
		     -0.278375,     -0.161881,     -0.433685,     -0.089071,     -0.237640,
		     -0.547729,     -0.413273,      0.323100,      0.795081,      0.277711,
		      0.153442,     -0.194203,     -0.257796,      0.233155,     -0.099175,
		     -0.139627,      0.058906,     -0.329834,     -0.433036,      0.285843,
		      0.069222,      0.337151,      0.520099,     -0.108503,     -0.494383,
		     -0.470116,      0.450416,      1.161366,     -0.220973,     -1.985952,
		     -0.073686,      1.406582,      0.274548,     -0.177482,     -0.096854,
		      0.079448,     -0.529904,     -0.352323,      0.455432,      1.700686,
		      1.151190,     -0.210539,     -0.780614,     -0.606412,     -0.077291,
		     -0.068846,      0.545772,      0.963170,      0.497211,     -0.825636,
		     -0.947076,      0.193310,      0.133187,     -0.600295,     -1.081640,
		      0.368999,      1.050828,      0.418758,     -0.619835,     -0.008364,
		     -0.168537,      0.153903,     -0.405098,     -0.136720,     -0.531520,
		     -0.673220,     -0.954159,      0.406458,      0.617960,      1.309960,
		      0.270379,     -0.778100,     -1.865623,     -0.746464,      0.275725,
		      0.599847,      0.071758,      0.634903,     -0.108023,     -0.341565,
		     -0.429084,      0.334194,     -0.265333,     -0.231403,     -0.449214,
		     -0.207415,     -0.651758,     -0.436071,      0.173902,      0.042513,
		     -0.108806,     -0.303863,      0.154792,      0.071185,      0.398664,
		      0.352236,      0.379483,      0.583018,     -0.692140,     -0.683340,
		      0.134024,      0.279632,     -0.258858,     -0.035909,     -0.105922,
		     -0.500426,     -0.281283,     -0.156556,     -0.040112,     -0.085882,
		      0.214551,      0.338683,     -0.379171,     -0.622827,     -0.188934,
		      0.796050,      0.348273,     -0.197861,     -0.747378,     -0.576443,
		      1.787585,      1.415232,     -1.788527,     -1.344797,      0.836238,
		      0.741824,     -0.256189,     -0.185605,      0.457370,      0.729994,
		     -0.946698,     -1.633652,     -0.907202,      0.392844,      0.265815,
		      0.222410,      0.533537,      0.136632,     -0.918637,     -1.259830,
		     -0.218184,      1.338569,      0.384403,      0.295258,     -0.137993,
		      0.306585,      0.307220,     -0.631548,     -0.652558,      0.616427,
		      0.527482,     -0.088871,     -0.287635,      0.693760,      0.113686,
		      0.227925,      0.297517,      1.081728,     -0.008866,     -0.100551,
		     -0.814831,     -0.689733,     -0.137255,      1.430351,      1.520998,
		      0.652454,     -0.410422,     -0.274156,     -0.989151,     -0.963667,
		      0.363784,      1.203595,      0.356275,     -0.638166,      0.113297,
		     -0.014807,      0.225982,      0.364642,      0.702968,      0.129388,
		     -0.363693,     -0.271100,      0.021292,      0.171373,      0.200597,
		      0.228749,      0.407554,     -0.047666,     -0.767631,     -0.141605,
		      0.179257,      0.062856,      0.071442,      0.156308,      0.194108,
		      0.505983,      0.241631,     -0.008747,      0.082712,     -0.278651,
		     -0.750620,     -0.312569,     -0.241069,     -1.021153,     -0.396015,
		      0.942628,      0.424251,     -0.893928,     -0.412617,     -0.183996,
		      0.568048,      1.264696,     -0.440779,     -2.003503,      0.928078,
		      2.093575,     -0.464036,     -1.466921,     -0.288220,      1.256277,
		      0.080314,     -1.414182,     -0.442385,      1.263238,      1.173826,
		     -0.189810,      0.015136,      0.230184,      0.215198,     -0.430055,
		      0.208058,      1.078830,      0.939553,     -0.352772,     -0.422717,
		     -0.297364,     -0.415930,     -0.147720,     -0.336266,      0.289768,
		      0.577850,      0.007138,     -0.451762,     -0.085132,     -0.236263,
		     -0.371917,     -0.181659,      0.044248,      0.115067,     -0.167379,
		     -0.486606,     -0.449137,     -0.162080,      0.534495,      0.650721,
		     -0.282256,     -1.578307,     -1.308535,     -0.026935,      0.278065,
		      0.062172,      0.687633,      0.524687,     -0.205980,     -0.374912,
		      0.033037,      0.246597,     -0.216563,     -0.315285,     -0.598935,
		     -0.847912,     -0.230951,      0.553091,      0.117311,     -0.103648,
		     -0.526156,     -0.176321,     -0.188703,     -0.388137,     -0.047804,
		      0.302524,      0.433298,     -0.027452,     -0.172751,     -0.168745,
		      0.070864,     -0.168226,     -0.118920,      0.077062,     -0.135551,
		     -0.064889,      0.551407,      0.484117,      0.239592,      0.497358,
		     -0.249581,      0.898953,      0.849232,     -0.874544,     -0.755337,
		      0.440283,      0.867942,      0.215337,     -0.541996,     -1.261319,
		     -0.498292,      1.842351,      0.888902,     -2.224323,     -1.127439,
		      1.422487,      1.062207,     -1.303371,     -1.219186,      0.846385,
		      1.501856,     -0.242338,     -0.906812,     -0.018275,      0.131407,
		     -0.813672,     -0.716738,      0.677285,      0.857990,     -0.438826,
		     -1.435717,     -0.753961,      0.186491,      0.223474,      1.015855,
		      0.589273,      0.046588,     -0.381390,     -0.578235,     -0.092404,
		      0.680199,      0.641605,      0.237853,     -0.365594,      0.545459,
		      0.120090,     -0.270550,     -0.466505,      0.549503,      0.041668,
		      0.565365,      0.077289,     -0.241092,     -0.564369,      0.884150,
		      1.176773,      1.031432,      0.270119,      0.486158,     -0.514812,
		     -1.191646,     -1.083500,      0.230263,     -0.017815,      0.005148,
		     -0.019280,      0.316251,      0.080362,      0.245618,      0.208147,
		      0.142224,      0.102872,      0.073479,      0.325702,      0.075303,
		      0.150225,      0.501693,      0.803870,      0.350173,     -0.092152,
		     -0.138240,     -0.082776,     -0.267952,     -0.131354,     -0.004615,
		      0.128164,      0.383495,      0.162639,     -0.215564,     -0.040903,
		      0.175939,     -0.032273,     -0.269055,      0.259219,      0.756885,
		     -0.290853,     -0.897393,      0.314931,      1.089984,      0.149023,
		     -0.987656,     -0.253342,      0.235320,      0.655691,      0.708971,
		     -1.092896,     -1.893861,      1.539131,      2.207513,     -0.805203,
		     -1.786206,      0.352609,      1.934616,      0.358989,     -1.554959,
		     -0.642133,      0.620115,      0.251759,     -0.656160,      0.402573,
		      0.894991,     -0.033907,     -1.208340,     -0.424892,      0.916734,
		      0.813645,     -0.019071,     -0.059537,     -0.498104,     -0.978533,
		     -0.182756,      0.363543,      0.620406,      0.199984,     -0.696397,
		     -0.638783,     -0.102244,      0.261744,     -0.245373,     -0.173772,
		     -0.117757,      0.509670,      0.268882,      0.394260,     -0.490134,
		     -0.553450,     -0.364295,      0.266085,     -0.289719,     -1.089492,
		     -1.473027,     -0.332678,     -0.260459,     -0.294474,      0.441714,
		      0.707666,      0.591073,     -0.168562,      0.081523,     -0.247801,
		      0.122331,      0.019836,      0.077127,     -0.530879,     -0.314557,
		      0.090081,      0.152849,      0.007357,      0.087774,      0.191786,
		     -0.268519,     -0.708703,     -0.475185,     -0.387762,     -0.242561,
		      0.219881,      0.312340,      0.491695,      0.042854,     -0.323565,
		     -0.162896,      0.281050,      0.084103,     -0.185067,      0.103161,
		      0.078692,      0.356538,      0.483462,      0.412958,     -0.721667,
		     -0.031557,      0.765755,      0.217002,     -1.073079,     -0.620481,
		      0.611167,      0.534597,      0.060899,     -0.293400,     -0.756707,
		      0.260004,      1.687462,     -0.112752,     -2.115408,     -0.148669,
		      1.980644,      0.716432,     -1.708348,     -1.352642,      0.653358,
		      0.878327,     -0.314368,     -0.467799,      0.919440,      0.273271,
		     -1.208135,     -1.074971,      0.806566,      1.549347,      0.217090,
		     -0.583170,     -0.239374,     -0.123063,     -0.313489,      0.707820,
		      0.621953,     -0.029824,     -0.753773,     -0.571427,      0.151697,
		      0.748495,      0.484895,      0.094523,     -0.301630,      0.188335,
		      0.414285,     -0.068401,     -0.723051,     -0.231070,     -0.071926,
		      0.323452,      0.634802,      0.464349,      0.099411,      0.683915,
		      0.754122,      0.441010,      0.282725,      0.805122,      0.380014,
		     -0.180529,     -0.879221,     -0.544034,     -0.207053,     -0.116406,
		      0.295298,     -0.091998,     -0.083684,      0.226323,      0.242225,
		     -0.019649,      0.181076,      0.030183,      0.138343,     -0.289581,
		      0.198071,      0.457312,      0.568669,      0.862384,      0.677094,
		      0.001709,     -0.363340,     -0.458767,     -0.305220,     -0.038504,
		     -0.170656,     -0.114089,     -0.109177,     -0.214364,     -0.068747,
		      0.326695,      0.088031,      0.003274,      0.489943,      0.439686,
		      0.189403,      0.083141,     -0.511131,     -0.541269,      0.688182,
		      0.818976,     -0.122739,     -0.790796,     -0.237095,      0.252170,
		      0.758044,      0.445637,     -1.118494,     -1.224892,      1.457015,
		      1.095318,     -1.436704,     -1.365612,      0.868254,      1.730491,
		      0.248938,     -0.932660,     -0.143151,      0.504039,     -0.544856,
		     -0.728453,      0.913850,      1.566377,      0.170845,     -1.579841,
		     -1.045192,      0.385446,      0.445540,      0.145185,      0.499326,
		     -0.229138,     -0.967079,     -0.529118,      0.474826,      0.694460,
		      0.439150,     -0.571363,     -0.707408,     -0.266060,      0.329438,
		     -0.103828,     -0.053910,     -0.511174,      0.164980,      0.339091,
		      0.558810,     -0.219049,     -0.567464,     -0.700193,     -0.296882,
		     -0.571204,     -0.345546,     -0.828531,     -0.104967,     -0.507433,
		     -0.639136,     -0.294542,      0.622287,      0.775125,      0.569317,
		     -0.181518,     -0.385752,     -0.206271,      0.282635,      0.219454,
		      0.140436,      0.242363,      0.541318,      0.117838,     -0.224231,
		      0.051832,      0.220598,      0.062106,     -0.252849,     -0.403047,
		     -0.464821,     -0.557485,     -0.209525,      0.314580,      0.424987,
		      0.283269,     -0.066100,      0.049928,      0.404578,      0.172271,
		     -0.144570,     -0.329196,     -0.371760,     -0.352843,     -0.224715,
		      0.220688,      0.040543,      0.018039,     -0.408475,      0.407111,
		      0.654911,     -0.028908,     -0.651927,     -0.211687,      0.653332,
		      0.179269,     -0.178686,     -0.542383,     -0.767282,      0.495967,
		      1.549235,     -0.715053,     -1.641410,      0.474713,      1.504852,
		      0.050439,     -1.509606,     -0.613342,      0.700899,      0.279877,
		     -0.489672,     -0.149483,      0.970856,      0.080881,     -1.426669,
		     -1.160212,      0.591514,      1.309100,      0.153012,     -0.434946,
		     -0.186516,     -0.221995,     -0.276181,      0.564184,      0.829658,
		      0.082530,     -0.624739,     -0.676300,      0.035210,      0.512071,
		      0.412413,      0.119635,     -0.183169,     -0.097904,      0.389785,
		      0.256326,     -0.232659,     -0.180192,      0.175606,      0.170448,
		      0.375985,      0.484762,      0.518052,      0.457448,      0.781058,
		      0.284584,      0.322568,      0.411535,      0.375021,      0.198563,
		     -0.297965,     -0.079576,     -0.227362,      0.049463,     -0.245934,
		     -0.436188,     -0.568743,      0.119806,      0.093722,     -0.217134,
		      0.236172,      0.490874,      0.007769,     -0.678450,     -0.423039,
		     -0.052801,      0.341275,      0.749672,      0.662771,      0.478953,
		      0.029892,     -0.430548,     -0.442408,     -0.101930,     -0.197689,
		     -0.330922,      0.099329,     -0.139792,     -0.046182,      0.157723,
		     -0.001726,     -0.033759,      0.104241,      0.232067,     -0.181365,
		      0.126193,      0.330679,     -0.202657,     -0.474137,     -0.605432,
		      0.429650,      0.408487,     -0.108061,     -0.071797,     -0.163582,
		     -0.117803,      0.691688,      0.544587,     -0.912067,     -0.295832,
		      1.387425,      0.309652,     -1.374735,     -0.632264,      1.040407,
		      0.998900,     -0.071941,     -0.208483,      0.440024,      0.409227,
		     -0.934105,     -0.896192,      0.748024,      1.440310,      0.518334,
		     -0.988692,     -0.758082,      0.095290,      0.118415,      0.045066,
		      0.669581,      0.173873,     -0.577625,     -0.589421,      0.332656,
		      0.436699,      0.328389,     -0.285164,     -0.431000,     -0.372060,
		      0.136187,      0.062869,      0.160077,     -0.527247,     -0.307923,
		      0.009436,      0.135122,     -0.161678,     -0.313704,     -0.378240,
		     -0.450770,     -0.966423,     -0.806016,     -0.573025,     -0.084057,
		     -0.001774,     -0.174449,     -0.361663,     -0.294808,      0.003909,
		      0.401216,      0.420802,     -0.033635,      0.104907,      0.111724,
		     -0.098393,      0.072374,      0.476998,      0.344969,      0.094259,
		     -0.244759,      0.217031,      0.289569,      0.155335,     -0.081405,
		     -0.205785,     -0.122894,     -0.451160,     -0.503409,     -0.103811,
		      0.047768,      0.004914,     -0.135643,     -0.016585,      0.234524,
		      0.275468,      0.006145,     -0.015123,     -0.158428,     -0.458723,
		     -0.154134,     -0.078911,      0.016107,     -0.247099,     -0.044357,
		      0.068592,     -0.326428,      0.100536,      0.522159,     -0.107209,
		     -0.318229,     -0.228756,      0.095375,      0.214241,      0.710745,
		     -0.288339,     -1.223731,      0.021934,      1.088256,     -0.509928,
		     -0.601629,      0.934280,      0.714268,     -0.631594,     -1.258869,
		     -0.193648,      0.310111,     -0.255654,     -0.467987,      0.518109,
		      1.149940,     -0.079589,     -1.062814,     -1.000141,      0.383721,
		      1.046526,      0.426426,      0.084810,      0.064701,     -0.465023,
		     -0.575836,      0.013162,      0.524299,      0.096800,     -0.116197,
		     -0.359648,     -0.058775,      0.106947,      0.036188,      0.032391,
		      0.133428,     -0.268908,      0.195263,      0.264727,     -0.069141,
		      0.062911,      0.557496,      0.352598,      0.297101,      0.528487,
		      0.998178,      0.619038,      0.748726,      0.248563,      0.252505,
		      0.165905,      0.130737,      0.246607,      0.195118,     -0.181375,
		     -0.206648,     -0.100308,      0.273115,     -0.057676,     -0.257310,
		     -0.274557,     -0.150362,     -0.502516,     -0.150984,      0.192274,
		      0.082161,     -0.148078,     -0.319893,     -0.055584,      0.127507,
		      0.146370,      0.305818,      0.416955,      0.144218,     -0.068003,
		     -0.348987,      0.033278,      0.143549,     -0.240200,     -0.183115,
		     -0.020349,      0.060910,      0.144041,      0.199471,     -0.113097,
		      0.011643,     -0.111669,     -0.337213,     -0.373805,     -0.007713,
		     -0.376009,      0.308112,      0.344023,      0.006883,     -0.323483,
		      0.027770,     -0.078441,      0.092565,      0.395936,     -0.753417,
		     -0.498709,      0.915544,      0.633956,     -0.917043,     -0.376683,
		      0.469278,     -0.391186,     -0.852235,      0.214359,      1.107055,
		      0.337275,     -0.300226,      0.046617,      0.645590,      0.146607,
		     -0.837037,     -0.523096,      0.591654,      0.908508,      0.206718,
		     -0.626952,     -0.558642,     -0.195750,     -0.265699,     -0.167339,
		      0.549774,      0.435600,     -0.057891,     -0.301049,      0.064566,
		      0.052823,      0.159366,      0.151786,      0.074600,     -0.131730,
		     -0.184941,     -0.087854,      0.343512,     -0.234095,     -0.296358,
		     -0.315585,     -0.587205,     -0.657475,     -0.641978,     -0.415640,
		     -0.321105,     -0.547378,     -0.826047,     -0.672375,     -0.351726,
		     -0.174571,      0.204299,      0.200128,      0.017134,     -0.371537,
		     -0.026523,     -0.128935,     -0.164613,     -0.054706,      0.403258,
		      0.012493,     -0.122411,      0.096253,      0.357613,      0.235109,
		     -0.042778,     -0.076922,      0.062844,      0.014235,     -0.187115,
		     -0.053783,      0.081365,     -0.205374,     -0.220233,     -0.146848,
		     -0.161381,     -0.447136,     -0.272295,     -0.154394,      0.180558,
		      0.140966,     -0.165195,      0.020452,      0.161499,      0.132155,
		      0.034055,      0.203712,      0.218092,     -0.010505,     -0.245663,
		     -0.197697,      0.614509,      0.278807,     -0.638388,     -0.275651,
		      0.431916,      0.219862,      0.459126,     -0.385426,     -1.115115,
		      0.260155,      1.390420,     -0.082453,     -1.126947,     -0.025613,
		      0.736203,     -0.235174,      0.072348,      1.067814,      0.144755,
		     -0.887677,     -0.648800,      0.251014,      0.295873,     -0.564582,
		     -0.716333,      0.290200,      0.875436,     -0.216912,     -0.688561,
		     -0.637295,      0.047192,      0.365578,      0.315355,      0.399042,
		      0.563510,     -0.076677,     -0.507197,     -0.247018,      0.151014,
		     -0.123856,      0.187963,      0.014729,     -0.013888,     -0.050195,
		     -0.109303,     -0.038385,      0.324949,     -0.128022,      0.121723,
		      0.234458,      0.065365,      0.404308,      0.857638,      0.621020,
		      0.303490,      0.202220,      0.681172,      0.419521,      0.530143,
		      0.332425,      0.121055,     -0.021071,     -0.234256,     -0.017853,
		      0.365336,      0.211528,     -0.110399,     -0.028351,     -0.122061,
		     -0.100232,     -0.056937,      0.196938,     -0.100434,     -0.490176,
		     -0.707361,      0.005218,      0.329841,      0.293813,     -0.052872,
		      0.057652,      0.038981,     -0.132228,     -0.097843,      0.030863,
		     -0.071433,     -0.024685,      0.089083,      0.246888,      0.189860,
		     -0.224399,     -0.235028,      0.190693,     -0.077532,     -0.156718,
		     -0.086034,      0.171184,      0.183639,      0.060206,      0.022099,
		      0.014826,      0.112632,      0.599521,     -0.640458,     -0.944363,
		      0.448955,      0.373437,     -0.181763,     -0.171697,     -0.444044,
		     -0.094403,      1.074966,      0.566528,     -1.568126,     -0.673708,
		      1.376730,      0.954085,     -0.520670,     -0.105431,      0.071170,
		     -0.949637,     -0.615472,      0.465522,      0.839839,      0.022422,
		     -0.529809,      0.109095,      0.746354,      0.184755,     -0.574323,
		     -0.120892,      0.512093,      0.664921,      0.127758,     -0.165177,
		     -0.108811,     -0.200346,     -0.504912,     -0.438241,      0.101113,
		      0.310036,      0.165623,      0.110627,     -0.052307,     -0.291499,
		     -0.230147,      0.090311,      0.232936,      0.051399,     -0.293732,
		     -0.273135,      0.005541,     -0.125742,      0.006921,     -0.049199,
		     -0.597090,     -0.670144,     -0.769568,     -0.681499,     -0.406711,
		     -0.049033,     -0.302556,     -0.439925,     -0.395676,     -0.355456,
		     -0.013562,      0.301045,      0.196389,     -0.014170,     -0.050745,
		      0.033237,     -0.158504,      0.032383,      0.075767,     -0.097724,
		     -0.232791,      0.159315,      0.371897,      0.304028,      0.043596,
		     -0.114168,      0.008128,      0.011535,     -0.053707,      0.095466,
		      0.177744,      0.148151,     -0.129901,     -0.323043,     -0.184417,
		     -0.216856,     -0.107735,      0.135945,      0.044963,     -0.071995,
		     -0.169766,      0.008187,     -0.004278,     -0.076233,      0.135722,
		      0.302075,      0.179083,      0.010815,     -0.086943,      0.020985,
		     -0.978712,      0.209784,      1.307871,      0.098496,     -0.633922,
		     -0.242369,      0.030691,      0.332716,      1.015018,     -0.518252,
		     -1.466039,      0.797472,      1.341645,     -0.821628,     -1.455567,
		      0.047386,      0.430194,     -0.255439,      0.321560,      0.988434,
		      0.113753,     -0.891375,     -0.514645,      0.345723,      0.431505,
		     -0.205123,     -0.453821,     -0.006345,      0.304542,     -0.352688,
		     -0.373331,     -0.170427,     -0.007300,     -0.158288,     -0.102401,
		      0.277035,      0.614281,      0.249229,     -0.198336,     -0.211273,
		     -0.118851,     -0.187348,      0.301232,      0.176479,     -0.111515,
		     -0.212788,     -0.050243,     -0.026539,      0.348040,      0.113541,
		      0.188048,      0.034899,     -0.187086,      0.214372,      0.886907,
		      0.688491,      0.618958,      0.371555,      0.329653,      0.133719,
		      0.348322,      0.482321,      0.270034,     -0.101493,     -0.505485,
		     -0.477634,      0.089151,      0.144252,      0.041769,     -0.038337,
		      0.040843,     -0.192951,      0.086813,      0.156118,      0.045639,
		     -0.275532,     -0.462399,     -0.338656,     -0.039626,      0.113427,
		     -0.059464,      0.140252,      0.142965,      0.027682,     -0.143762,
		     -0.185674,     -0.330453,     -0.459649,     -0.228044,      0.087824,
		      0.332655,      0.133982,     -0.105859,      0.136440,     -0.029237,
		     -0.163723,     -0.129974,     -0.058942,      0.034809,      0.146063,
		      0.051075,      0.156976,      0.456963,      0.398721,      1.135336,
		      0.583625,     -1.274255,     -0.617641,      0.806051,      0.446454,
		     -0.026412,     -0.052077,     -1.154866,     -0.184719,      1.639805,
		      0.311758,     -1.816348,     -0.366506,      1.565984,      0.453295,
		     -0.637459,      0.108256,      0.289653,     -0.819286,     -0.506811,
		      0.583147,      0.775798,      0.160935,     -0.547650,     -0.224255,
		      0.310934,      0.074336,     -0.272248,      0.228315,      0.268720,
		      0.259660,     -0.059980,      0.131698,      0.375196,      0.144148,
		     -0.355657,     -0.390953,     -0.057826,      0.042803,      0.038099,
		      0.332673,     -0.053561,     -0.224716,     -0.128414,      0.154786,
		      0.192159,      0.132979,     -0.215577,     -0.115182,     -0.198335,
		     -0.273765,     -0.025595,     -0.034259,     -0.539512,     -0.375428,
		     -0.515833,     -0.599524,     -0.569300,      0.009736,     -0.016576,
		     -0.284237,     -0.028005,      0.019105,      0.162375,      0.098142,
		     -0.046486,     -0.099154,      0.042511,      0.008961,     -0.024232,
		     -0.120572,     -0.032204,     -0.197706,     -0.029104,      0.078481,
		      0.422259,      0.367525,      0.361909,      0.014415,     -0.220599,
		     -0.163290,      0.037390,      0.085876,      0.080353,      0.157447,
		      0.149145,      0.126838,      0.042155,     -0.127114,     -0.160807,
		     -0.160182,     -0.099483,     -0.070872,      0.040199,      0.145303,
		      0.055263,     -0.099470,     -0.229162,      0.009789,      0.152981,
		      0.040548,     -0.248511,     -0.186010,      0.007415,     -0.847805,
		     -1.413671,      0.747405,      0.952355,     -0.425845,     -0.407422,
		      0.011815,     -0.216171,      0.479412,      0.833836,     -1.023285,
		     -1.204249,      1.497607,      1.377800,     -1.135221,     -0.955641,
		      0.596632,      0.434049,     -0.433723,      0.098866,      0.655139,
		      0.051121,     -0.744644,     -0.538125,      0.117859,      0.280858,
		      0.115591,      0.139899,      0.170443,     -0.152822,     -0.531513,
		     -0.352671,      0.165487,     -0.007166,     -0.451332,     -0.446285,
		      0.120946,      0.455516,      0.208369,     -0.065549,     -0.042881,
		     -0.307314,     -0.166820,      0.279035,      0.255150,     -0.241259,
		     -0.387327,     -0.072709,     -0.070059,      0.026853,      0.173690,
		      0.454054,      0.332314,     -0.059990,      0.119012,      0.452208,
		      0.381921,      0.427482,      0.572390,      0.320370,     -0.229366,
		     -0.139233,      0.054211,      0.125333,     -0.001376,     -0.073769,
		     -0.174621,     -0.062733,      0.019970,      0.008422,      0.299239,
		      0.200312,      0.038465,     -0.097696,     -0.055367,     -0.142681,
		      0.089580,      0.038836,     -0.137314,     -0.220068,     -0.082827,
		     -0.163217,     -0.142592,      0.022855,      0.107945,     -0.064114,
		     -0.196481,     -0.317380,     -0.418338,     -0.148072,     -0.163152,
		      0.021999,      0.129725,     -0.021757,      0.115267,      0.016265,
		      0.001998,     -0.040233,      0.024064,      0.033130,     -0.041804,
		     -0.011802,      0.195466,      0.133836,      0.032785,      0.045031,
		      0.174365,      1.817395,     -0.008688,     -1.164130,     -0.218221,
		      0.462563,     -0.005629,      0.154707,      0.139294,     -1.036161,
		      0.104625,      1.493660,     -0.283317,     -1.628376,      0.209581,
		      1.288695,     -0.050610,     -0.810730,      0.042632,      0.346055,
		     -0.567163,     -0.468302,      0.447309,      0.572116,      0.310759,
		     -0.167950,     -0.223358,     -0.099242,     -0.183110,     -0.047377,
		      0.618703,      0.484366,      0.027441,     -0.139050,      0.169493,
		      0.484712,      0.146999,     -0.221009,     -0.209815,      0.119165,
		      0.135572,      0.022743,      0.249284,     -0.052114,     -0.229046,
		     -0.000031,      0.249956,      0.095693,      0.040096,     -0.157323,
		     -0.102123,     -0.317657,     -0.347926,     -0.041143,      0.103084,
		     -0.378318,     -0.128997,     -0.161255,     -0.314732,     -0.325292,
		      0.211183,      0.309327,      0.006225,     -0.028351,      0.104281,
		      0.148312,      0.081113,     -0.077663,     -0.097264,     -0.150126,
		     -0.204938,     -0.196614,      0.134599,      0.133710,      0.059408,
		     -0.038414,     -0.113767,     -0.008700,      0.195196,      0.296955,
		      0.219973,     -0.044295,     -0.053115,      0.069499,      0.127768,
		     -0.027525,     -0.076881,      0.074752,      0.127895,      0.101548,
		      0.191461,      0.205365,     -0.069020,     -0.208218,     -0.325994,
		     -0.077792,      0.008761,      0.049972,      0.057181,     -0.014987,
		     -0.025721,     -0.106605,     -0.055564,      0.022259,      0.048150,
		     -0.113391,     -0.157014,      0.477003,     -1.429670,     -0.581836,
		      0.954921,      0.477341,     -0.492112,     -0.034131,      0.094581,
		     -0.204092,      0.620414,      0.546451,     -1.063527,     -0.796103,
		      1.128959,      0.570488,     -1.139462,     -0.608029,      0.693052,
		      0.446888,     -0.249216,      0.108631,      0.489804,      0.064710,
		     -0.404598,     -0.376651,      0.005651,     -0.002363,      0.043159,
		      0.353056,      0.357043,     -0.291751,     -0.608401,     -0.324658,
		      0.245587,      0.097828,     -0.438509,     -0.495195,      0.030748,
		      0.298444,      0.001338,     -0.073550,      0.032600,     -0.238972,
		     -0.115333,      0.175934,      0.193271,     -0.111074,     -0.299848,
		      0.066348,      0.134322,     -0.075196,      0.017422,      0.209874,
		      0.254535,     -0.041353,      0.071813,      0.255359,      0.128159,
		      0.024191,      0.296494,      0.240300,     -0.118606,     -0.175951,
		     -0.090164,     -0.092456,     -0.219852,     -0.113821,      0.011208,
		      0.057149,      0.110577,      0.153869,      0.233501,      0.198275,
		     -0.006012,      0.083110,     -0.022915,     -0.102107,     -0.214261,
		     -0.011376,      0.025635,     -0.146227,     -0.113842,     -0.157399,
		     -0.195294,     -0.108719,     -0.003972,      0.053475,     -0.127239,
		     -0.155316,     -0.245739,     -0.199481,     -0.300079,     -0.124665,
		      0.132065,      0.138541,      0.144652,      0.087390,     -0.030579,
		     -0.077696,      0.140152,      0.162915,      0.211490,     -0.050903,
		     -0.168470,     -0.070683,      0.030611,     -0.079458,     -0.142775,
		     -0.808932,      0.602594,      0.854758,     -0.355001,     -0.440254,
		      0.232168,      0.141499,     -0.193863,      0.063241,     -0.100398,
		     -0.728194,      0.298652,      0.972151,     -0.417623,     -0.827976,
		      0.534616,      0.819362,     -0.185681,     -0.534789,      0.036954,
		      0.224959,     -0.358160,     -0.350493,      0.184648,      0.220510,
		      0.124077,      0.096755,     -0.006191,     -0.245015,     -0.401095,
		     -0.122289,      0.420286,      0.404744,     -0.072571,     -0.155306,
		      0.262610,      0.544619,      0.153467,     -0.223550,     -0.144324,
		      0.074856,      0.118891,      0.064988,      0.162743,     -0.098917,
		     -0.158542,      0.068953,      0.236869,      0.053790,      0.047356,
		      0.043948,      0.103163,     -0.129838,     -0.304776,     -0.129405,
		     -0.017972,     -0.302123,     -0.133788,     -0.022691,     -0.180770,
		     -0.330159,     -0.002934,      0.167403,      0.077266,      0.134090,
		      0.256980,      0.232681,      0.127088,     -0.061702,     -0.057155,
		     -0.068079,     -0.200298,     -0.171974,     -0.044235,      0.062956,
		      0.046234,      0.173372,      0.147491,      0.000300,     -0.151466,
		     -0.008593,      0.203523,      0.194012,      0.047301,      0.054864,
		      0.078726,     -0.088461,     -0.102926,      0.055873,      0.137492,
		      0.015447,      0.133997,      0.237285,      0.033776,     -0.122943,
		     -0.185927,     -0.102984,      0.002365,      0.005927,     -0.127955,
		     -0.088345,     -0.026551,      0.097043,      0.069851,      0.101186,
		      0.055957,     -0.069189,     -0.120878,     -0.108570,      0.677383,
		      0.056205,     -0.766462,     -0.102664,      0.395377,     -0.010565,
		     -0.289636,      0.082710,      0.034809,     -0.132249,      0.414842,
		      0.290414,     -0.501966,     -0.171117,      0.636571,      0.126903,
		     -0.678729,     -0.311562,      0.339064,      0.139034,     -0.145630,
		      0.121990,      0.299147,      0.006100,     -0.120064,     -0.120062,
		     -0.024917,     -0.077960,     -0.049656,      0.270754,      0.373939,
		      0.023902,     -0.263016,     -0.200233,      0.118370,      0.072616,
		     -0.250044,     -0.326671,     -0.014546,      0.175823,      0.001742,
		     -0.067930,      0.035066,     -0.156947,     -0.016401,      0.061348,
		      0.045061,     -0.101020,     -0.212521,     -0.062099,      0.083442,
		     -0.056212,      0.019088,      0.135773,      0.222847,      0.071945,
		      0.115499,      0.129506,      0.158465,     -0.036380,      0.101451,
		      0.119185,     -0.047144,     -0.066892,     -0.048014,     -0.108151,
		     -0.201662,     -0.117916,      0.047781,      0.062781,      0.038924,
		      0.027452,      0.122286,      0.098515,      0.021098,      0.040734,
		     -0.030959,     -0.094802,     -0.104539,      0.044022,      0.095150,
		     -0.137389,     -0.185359,     -0.056437,     -0.049332,     -0.069254,
		     -0.029296,      0.055640,     -0.007005,     -0.015459,     -0.065751,
		      0.001428,     -0.164743,     -0.188923,     -0.004669,      0.171364,
		      0.075107,      0.079094,     -0.073024,     -0.037115,     -0.021187,
		      0.035722,      0.075572,      0.095425,     -0.049124,     -0.108616,
		     -0.013329,     -0.007790,     -0.071923,     -0.002699,     -0.277854,
		     -0.182163,      0.362218,      0.113312,     -0.229626,     -0.006393,
		      0.208425,     -0.011645,     -0.044270,      0.073621,     -0.122998,
		     -0.336247,      0.134997,      0.261340,     -0.281553,     -0.238135,
		      0.337641,      0.270358,     -0.155518,     -0.160167,      0.022291,
		      0.020352,     -0.158325,     -0.042028,      0.088850,      0.078689,
		     -0.021617,      0.070144,      0.095035,     -0.121353,     -0.235651,
		     -0.136885,      0.105507,      0.193317,     -0.052435,     -0.150487,
		      0.095823,      0.263934,      0.069731,     -0.055083,     -0.030396,
		      0.014101,      0.049317,      0.079496,      0.078579,     -0.028195,
		     -0.112242,     -0.002385,      0.145822,      0.004636,     -0.053220,
		      0.012879,      0.010853,     -0.023565,     -0.142918,     -0.079638,
		     -0.006124,     -0.105589,     -0.092010,      0.047995,     -0.041687,
		     -0.114315,     -0.073054,     -0.004160,     -0.004397,      0.046790,
		      0.142658,      0.095479,      0.052432,     -0.015567,      0.004019,
		      0.043918,     -0.055693,     -0.134015,     -0.037861,     -0.008742,
		      0.049276,      0.051226,      0.099926,     -0.020717,     -0.102922,
		     -0.009425,      0.019212,      0.019198,      0.017263,      0.021272,
		      0.067202,     -0.023792,     -0.031818,     -0.021596,      0.039664,
		     -0.060908,      0.018867,      0.164958,      0.059043,     -0.048823,
		     -0.034534,      0.011608,      0.049808,      0.024886,     -0.068479,
		     -0.020989,     -0.051119,     -0.009253,      0.003127,      0.048339,
		      0.043927,     -0.009414,     -0.020766,     -0.103430,     -0.083191,
	};
	static const std::vector<double> h = {
		      0.000000,     -0.344572,      0.000000,      0.047479,      0.817254,
		      0.000000,      0.261673,     -0.929050,     -0.295324,      0.000000,
		     -1.517073,     -0.958639,      0.319322,      0.635529,      0.000000,
		      1.854522,      0.886135,     -1.221881,     -1.000134,     -1.077116,
		      0.000000,     -0.828422,     -0.059367,      0.146140,      0.683628,
		      1.274396,      0.204392,      0.000000,     -0.582967,      0.371783,
		      0.870442,     -0.262640,      0.233301,     -0.324014,     -0.674421,
		      0.000000,     -0.328798,     -1.999496,     -0.997002,     -0.770279,
		     -0.318703,      1.936969,      1.168318,     -0.325846,      0.000000,
		      0.879139,      1.724225,      0.903120,      0.229862,     -0.058309,
		     -0.350676,     -0.749730,      0.129262,     -0.946671,      0.000000,
		     -0.231398,     -0.706443,     -1.163172,      0.039656,     -0.472604,
		     -1.651430,      0.382914,      0.725761,      0.585681,      0.556445,
		      0.000000,     -0.497305,     -0.414477,      0.518771,      0.481228,
		      1.029271,      1.415845,      0.524954,     -0.439537,      0.010668,
		     -1.840548,     -1.493135,      0.000000,      0.326996,     -0.744890,
		     -0.448110,     -0.890300,     -0.867407,     -0.394345,     -0.893456,
		     -0.550043,     -0.195120,      2.003648,      1.360015,      1.271485,
		      0.000000,     -0.068632,      1.803652,      0.467504,     -0.538709,
		     -0.662100,      0.069389,      0.800340,      0.138845,      1.113996,
		      0.129526,     -1.730837,     -1.964482,     -1.266941,      0.000000,
		      0.436027,     -0.461375,     -1.247531,      1.290116,      1.181982,
		     -0.284816,     -0.555027,      0.103924,     -0.071679,      0.259202,
		      2.632791,      0.656424,      0.278216,      0.052610,      0.000000,
		     -1.946260,     -1.490487,      1.442421,      0.444508,     -0.480347,
		     -0.326527,      0.968417,      0.786516,     -0.297663,      0.385857,
		      0.167547,      0.083462,     -0.257112,     -0.916402,      0.255556,
		      0.000000,      1.192102,      0.898079,     -0.454956,     -2.007443,
		      0.417896,      1.831293,     -0.693809,     -1.664532,     -1.201229,
		     -2.102195,     -2.121079,      0.622664,      0.491208,     -0.179990,
		     -0.289232,     -2.693918,      0.000000,      0.886759,     -0.284277,
		     -1.916635,      0.662485,     -0.566463,     -2.231873,     -1.371124,
		      1.529617,      1.599620,      2.262213,      1.427857,     -0.883938,
		     -0.588552,      0.264880,     -0.446336,      1.058884,      0.661733,
		      0.000000,     -2.572504,      0.504407,      1.362096,      1.226985,
		      0.063945,      0.691776,      2.777688,      0.749740,     -0.371653,
		     -1.392741,     -0.933007,     -1.035932,      0.849484,      0.918505,
		      0.777683,      0.305676,     -0.923172,     -1.509228,      0.000000,
		      0.015634,     -0.883924,      0.495079,     -1.517941,      0.352049,
		      0.316622,     -2.751026,     -2.732075,     -1.781807,      0.190031,
		      1.896707,      2.013137,     -1.254099,     -0.019429,     -0.369299,
		     -3.216885,     -0.231111,      0.827051,      1.128126,      0.000000,
		      1.079267,     -1.572491,     -1.104857,     -0.645933,     -0.366367,
		      0.755192,      0.983111,      2.575220,      1.693609,      1.175055,
		     -2.053720,     -0.261552,      0.005333,     -0.093079,      0.900950,
		      1.796624,      1.443751,      0.359151,     -0.138156,      0.104262,
		      0.000000,      0.512696,      1.802740,     -0.064755,      0.914033,
		     -0.688430,     -0.991321,      0.996052,     -0.587464,     -0.763525,
		     -1.872964,     -0.522305,     -0.034267,      1.400418,      0.491433,
		      1.767343,      1.297628,     -2.072060,     -2.026988,     -0.987400,
		      1.217961,      2.659715,      0.000000,     -1.307663,     -1.012127,
		     -0.516958,      0.274399,      0.882677,     -0.574859,     -0.667264,
		      0.180815,      1.265199,      2.406067,      2.602702,     -0.747319,
		     -1.855923,     -0.494274,     -2.368198,     -1.061555,      0.530692,
		      0.877299,      1.958078,     -0.787613,     -1.474379,      0.093022,
		      0.000000,     -0.616747,     -0.974549,      1.234281,      0.775148,
		      0.783501,      1.541771,     -0.181255,     -0.196209,     -1.534865,
		     -0.534622,     -1.847556,      1.461003,      0.641187,      0.169234,
		      0.473076,      0.822905,      1.708068,      0.187144,     -1.539020,
		     -1.339241,     -0.141283,      2.613853,      1.084170,      0.000000,
		      1.864212,      0.638038,     -2.488720,     -2.497883,     -1.359578,
		     -0.712726,     -0.229286,      0.384208,      1.284533,     -1.098042,
		      0.289297,      0.541890,     -0.633831,     -0.752967,      0.727548,
		     -1.455635,     -2.244404,     -1.278965,      0.415028,      1.907830,
		      0.222722,     -3.269551,     -1.401380,     -0.103834,      0.000000,
		     -0.603205,     -0.261694,      0.062571,      0.075498,      0.395352,
		     -0.013891,      1.658832,      0.949637,     -0.540189,     -0.148803,
		      0.827008,     -1.153208,     -0.161260,      0.651507,     -0.328467,
		      2.219863,      1.624132,      1.183466,      0.264776,     -2.781742,
		     -1.112132,      1.476504,      1.382519,      0.437780,     -1.731541,
		      0.000000,     -0.813695,      0.625248,      1.413697,      0.444289,
		      0.024760,     -0.358369,     -1.610776,     -2.395241,     -0.631656,
		      1.874376,     -0.593774,     -0.017273,      0.203808,     -0.122897,
		     -1.682638,     -0.643422,     -0.035869,     -1.082411,     -1.058139,
		     -0.951020,      0.044446,      0.148715,     -0.580979,      0.077775,
		      0.745594,     -0.989786,      0.000000,      1.276993,     -1.731311,
		     -0.807219,      0.259066,     -0.555861,     -0.667403,      0.252325,
		      2.033770,      1.219033,     -1.480577,     -1.296097,      0.329269,
		      0.732135,      0.361757,      0.929759,      0.220349,      0.975632,
		      0.602995,      2.662533,      2.156860,     -2.223546,     -2.205264,
		     -0.738164,      1.887894,      0.915919,     -0.834504,     -1.738528,
		      0.000000,     -0.232956,      0.258918,     -0.702659,     -0.443252,
		     -0.472377,      0.521623,      0.361649,     -0.381967,     -1.337415,
		      0.171564,      2.176146,     -0.199093,     -0.804947,     -0.395194,
		      0.048502,     -0.533200,     -2.273106,     -0.255585,     -1.773517,
		     -1.696654,      1.118927,      1.268863,      1.058978,     -0.639502,
		     -0.883881,      3.477265,      1.597111,     -0.427915,      0.000000,
		     -0.046099,      3.052294,      0.255039,     -0.439008,      0.949357,
		     -0.234697,     -0.241711,     -0.110353,      1.749667,      1.579495,
		     -0.751502,     -1.115001,      0.588821,      1.306092,     -0.203127,
		      0.005640,      0.887772,      0.807490,     -0.802478,      0.740740,
		      2.174431,      0.183626,     -1.262252,     -0.150507,     -0.645232,
		     -0.708189,     -0.270753,     -1.434670,      0.034151,      0.000000,
		      0.153430,     -1.984815,      0.419405,      0.196529,     -0.484087,
		     -0.618248,     -0.531703,     -0.506095,     -1.337349,     -1.501901,
		     -0.359893,      1.558289,     -0.080829,     -2.095296,     -1.195953,
		      1.055629,      0.997333,     -1.631829,     -0.176617,     -0.553904,
		     -0.775174,      1.570688,      3.146021,      2.064340,      1.983597,
		     -0.763969,     -0.488155,      1.779777,      1.003536,      0.661757,
		      0.000000,     -1.613265,     -2.142176,      0.882809,     -0.503017,
		     -0.891625,      0.404552,     -0.280627,     -0.344247,      0.188831,
		      0.447597,      0.513024,      0.464481,     -0.006596,      0.207957,
		      1.552065,     -0.170869,     -0.581734,      1.232729,      0.922408,
		     -0.656003,     -1.102063,     -1.910212,     -1.633565,     -1.335262,
		     -0.218943,      1.547082,      0.682984,     -1.652095,     -2.678399,
		     -1.496637,      0.718240,      0.000000,      1.540966,      3.008003,
		     -2.497077,     -0.098968,      1.170367,      0.516821,      1.061602,
		      0.587415,     -0.025127,     -1.031863,     -0.496962,     -0.338612,
		      0.776061,      0.391083,     -1.045922,     -1.505259,     -1.172673,
		     -0.169253,     -1.155339,      0.585350,      0.246851,      1.576399,
		      1.166202,      1.227721,     -0.310046,      1.059815,     -0.049172,
		      1.057805,      1.638698,     -0.275776,      0.645639,      1.017911,
		      0.000000,      0.894289,     -0.797722,      0.851105,      1.151629,
		     -1.162622,     -2.199590,     -0.303783,     -0.470042,     -0.202903,
		      0.701908,      0.163529,     -1.371459,     -1.421956,      0.324041,
		      0.762705,      0.586376,      0.511571,      0.015170,      0.417281,
		      1.352638,      0.788511,      1.244905,      0.097964,     -1.243958,
		     -1.642548,     -1.342448,      2.324765,      1.284526,     -0.109216,
		     -1.008209,     -1.433057,     -1.784766,     -0.122760,      0.000000,
		     -2.684957,     -0.306328,      2.399588,     -2.619069,     -0.681019,
		      1.700624,     -0.762499,     -0.499396,      0.248025,      0.499926,
		     -0.809061,      0.609191,      1.492425,      0.645712,     -0.052045,
		     -0.729801,      0.915828,      0.361929,     -0.506465,     -2.031116,
		     -0.974217,     -1.345737,      0.139677,      1.189785,      0.445348,
		     -0.735092,     -1.818097,     -0.449752,      0.597800,      0.622192,
		      0.430865,      0.384080,      0.255528,      1.463871,      0.000000,
		      1.150953,     -1.093480,     -2.008604,      2.699374,      2.642687,
		     -0.723791,     -1.091190,      1.104129,     -0.499599,     -1.501856,
		      0.731852,      1.266228,     -0.737667,     -2.069005,      0.011402,
		      0.951216,     -0.995969,     -0.448381,      0.530101,      0.076499,
		      0.402613,      1.013806,      0.572051,     -0.629492,      0.458056,
		      1.697433,      0.276815,      1.055687,      0.105752,      0.837673,
		      0.626460,     -0.644870,     -0.288720,     -2.493187,     -0.491770,
		      0.000000,      1.423258,     -0.773881,     -0.209992,      0.621328,
		     -2.367444,     -0.096770,      2.460958,      0.017047,     -0.472724,
		      1.070712,      1.034608,     -2.546543,     -1.467884,      1.124499,
		      1.235497,      0.027081,     -0.298468,      1.113637,      0.296304,
		      1.049898,     -0.212627,      0.128702,     -0.811946,      0.191348,
		      0.407075,     -0.166980,      0.591279,     -1.588881,      0.204281,
		     -1.708081,      0.366851,      0.571973,      0.419732,      0.979229,
		     -1.081259,     -0.554864,      0.000000,     -0.911399,      2.086317,
		     -0.920132,     -2.441311,      1.978683,      1.008324,     -2.079903,
		     -1.776973,      1.171680,     -0.299348,     -1.435644,     -0.017811,
		      2.238545,      0.080036,     -1.785211,     -0.141865,      1.383379,
		     -1.234030,      0.398848,      0.967775,      0.296658,     -0.213010,
		     -0.126440,      0.249207,     -1.391213,     -1.566405,      2.001128,
		      1.804875,      1.440859,      1.141527,      0.835675,     -0.879131,
		      0.369848,     -0.716895,     -0.342903,      0.938139,     -0.847650,
		      0.000000,     -0.673384,      0.240717,      0.397943,      0.570322,
		     -0.697311,     -1.640383,      0.573825,      1.758861,     -1.073453,
		     -1.754320,      0.927522,      2.577714,     -1.124646,     -1.031016,
		     -0.356745,      0.694074,     -0.359519,      0.382016,      1.310727,
		     -1.429580,     -1.027706,      0.474810,      0.763499,     -0.460750,
		     -0.148735,      0.193266,     -1.119595,      0.614988,     -3.169371,
		     -1.398845,     -0.668765,      0.956907,      0.157182,      0.490275,
		      0.665702,     -2.481784,     -0.078423,     -0.414823,      0.000000,
		      0.419890,     -1.848011,      1.344103,      0.226515,     -0.210814,
		      2.391811,      1.292084,     -0.497747,     -0.045722,      2.321148,
		     -0.358702,     -1.442017,     -0.651146,      0.813753,      0.802735,
		     -1.886969,     -0.681621,      0.290214,     -2.487449,      0.485303,
		      1.549279,      0.761961,      0.344516,      1.671049,      1.491876,
		      0.679654,     -0.821598,      0.901302,      2.026858,      0.927669,
		      0.934151,     -0.097290,     -0.650313,     -1.898805,     -0.469271,
		      1.429804,     -0.630779,      1.987192,     -0.543984,      0.000000,
		      0.654015,      0.078386,     -0.722815,      0.101028,      0.169507,
		     -1.802824,     -0.786062,      0.265603,      1.164495,     -1.117054,
		     -1.824001,     -1.468671,      0.639513,     -1.127109,      0.460880,
		      0.372877,      0.431274,     -0.382719,      1.480747,      2.188697,
		     -0.221734,     -1.989628,     -1.811017,     -1.242280,     -1.360383,
		     -1.954913,      0.046412,     -1.464272,     -0.148112,     -2.948562,
		     -2.184352,     -1.241353,     -0.278857,      0.767054,     -0.301947,
		     -0.339166,     -0.940379,     -1.981490,     -0.568217,      0.062079,
		      0.000000,      0.243729,      0.394026,     -2.207567,      0.088652,
		      0.410761,     -0.720379,      0.453360,      0.786673,     -1.762909,
		     -0.265918,      2.802337,      1.792262,      0.268301,      1.143896,
		      0.086610,      0.630470,      0.308818,      0.593108,     -0.214260,
		     -3.022672,     -0.025696,      2.018727,      1.548459,      0.952454,
		      0.802010,      0.842946,      1.329400,      0.935367,      0.593025,
		      1.030696,      0.341974,      0.183554,      1.099824,     -0.325744,
		     -1.008161,      0.068013,      0.176017,      1.224406,      1.257482,
		      0.787386,     -0.284349,      0.000000,     -0.860708,      0.184999,
		      2.330291,     -0.075105,     -0.912781,      1.146687,     -0.273188,
		      0.075891,      1.540721,      1.639038,     -0.749933,     -0.231802,
		     -1.306537,     -1.650978,     -2.135095,     -0.480585,     -1.029145,
		     -0.516134,     -1.637453,      0.919746,      1.575553,     -0.319786,
		     -1.180745,     -0.759498,     -0.681983,      0.124757,     -2.292801,
		     -1.099301,     -1.540745,      0.618109,     -0.167009,     -0.720174,
		     -1.470655,     -1.364971,     -0.357641,     -1.292212,      0.120622,
		     -0.546794,     -2.955224,      1.438902,      1.477482,      0.405653,
		      0.000000,      1.050083,      1.066616,     -0.161746,     -1.705821,
		      0.374517,      1.086021,     -0.969693,     -1.191599,      1.009911,
		     -1.580270,     -1.226846,     -0.715756,      0.833506,      0.850985,
		      2.902719,      0.610667,      0.301309,      0.520065,      1.605995,
		      1.039704,     -1.507782,     -0.824302,      1.169111,      0.858756,
		      1.656159,      0.716011,      0.363098,      0.972947,      1.467364,
		     -0.323898,      0.150367,      0.311615,      0.870446,      1.097459,
		      0.400993,      0.331450,     -0.542422,      1.587788,      1.858256,
		     -1.015800,      0.346123,      0.810840,      0.628576,      0.000000,
		     -0.717396,     -1.191791,     -0.834851,      1.910771,      0.268683,
		     -1.920124,      0.641900,      0.427946,     -1.766578,      0.893743,
		      1.733778,     -0.473894,      0.173402,     -0.860336,     -1.827735,
		     -0.646939,      0.524963,     -0.463275,     -1.265152,     -2.461896,
		     -1.220500,      0.961702,      0.050246,     -1.239653,     -2.509274,
		     -1.598288,      0.429577,      0.475516,     -0.493249,     -0.290661,
		     -0.306203,      0.801321,      0.987770,      0.000189,     -1.324434,
		     -0.838818,     -1.445914,     -0.214095,     -0.547197,      0.379347,
		     -1.444668,      0.273968,      0.417224,      1.013166,      0.000000,
		     -0.344856,      0.818525,      0.468865,     -0.927664,     -0.488925,
		      0.941476,      0.564361,     -0.760684,     -0.727883,      1.627734,
		      0.030940,      1.634633,      0.349124,      1.028922,     -1.014675,
		      0.570085,      0.003946,     -0.603660,      0.554000,      1.657331,
		      1.114737,      0.302946,      0.028696,      0.690772,      1.018266,
		      1.359844,      0.803422,     -1.081358,     -0.972867,      1.738260,
		      0.025956,     -0.407766,     -1.667534,     -0.368416,      1.539124,
		      2.158168,      1.682666,     -1.130785,     -0.328343,     -0.142551,
		     -0.231232,     -0.237239,     -0.770267,      0.551881,      1.245352,
		      0.000000,      0.692753,     -1.155998,      0.722660,     -0.104782,
		      0.122456,     -0.555459,     -0.942100,      1.519770,      2.397253,
		     -2.388038,     -0.791987,      0.262371,     -1.230057,     -0.631578,
		      0.159790,     -0.887792,     -0.605349,      0.230418,      0.157753,
		     -0.214857,     -1.255921,     -1.969611,     -0.441318,      0.023616,
		      0.924676,     -1.001266,     -1.701882,     -0.758213,      0.509044,
		     -0.921805,     -1.260606,      0.162669,      2.814749,      0.684317,
		     -0.287356,     -1.645925,     -1.007821,      0.805572,     -0.599673,
		     -0.033460,      0.195026,     -0.159675,      2.172464,     -0.775848,
		     -2.151419,     -1.503284,      0.000000,     -0.245688,      0.801718,
		      0.023796,      0.188302,     -0.084657,      1.120041,      0.531222,
		     -1.380873,     -1.547261,     -0.231500,      0.299575,     -1.742117,
		      0.532566,      1.189238,      2.422281,      0.920694,      1.047742,
		      0.246674,     -0.054743,      0.991725,      1.254630,      1.301041,
		     -0.639026,      0.095182,     -0.444094,     -0.758770,      0.755135,
		      1.287853,      0.656968,      0.438049,      0.648382,     -0.082174,
		     -0.939559,     -1.033836,     -1.197770,      0.720482,      0.358329,
		      0.249143,      0.576231,     -0.701225,     -0.749721,     -0.323063,
		     -1.109076,      0.567994,      1.037942,      1.254767,     -0.799810,
		      0.000000,      0.009615,     -0.336001,     -1.573724,      1.062202,
		      1.153483,     -1.470521,     -1.490043,     -0.028144,      0.684602,
		      3.150002,      0.523628,      1.125666,      0.784962,     -1.159465,
		     -2.737856,     -1.298579,     -0.237738,     -0.514997,     -1.064844,
		     -1.087380,     -0.732489,      0.322265,     -0.859624,     -0.906443,
		     -0.635507,      0.430119,      0.220972,     -1.034693,     -1.815797,
		      0.318062,     -0.913526,     -1.218005,     -0.268225,      2.106552,
		      1.268514,     -0.061709,     -1.036562,     -0.667351,      0.256742,
		      0.380103,      0.043871,      0.203734,      0.456933,     -0.879369,
		      1.353775,     -0.034665,     -0.365399,     -1.160449,      0.000000,
		      0.001467,     -0.106175,      0.628171,     -1.171165,     -1.551270,
		      0.543092,      2.636866,      1.273350,     -1.217525,     -2.212378,
		      0.082912,      1.100027,     -0.793332,      0.658666,      0.894655,
		      1.310775,     -0.444754,     -0.185605,      0.122029,      0.070083,
		      1.271797,      1.565865,      2.342558,      0.197946,      0.465080,
		      0.778638,      0.029013,     -0.076930,      1.125088,      0.273905,
		      0.437722,      0.291764,      0.061216,     -0.182705,      0.029424,
		     -0.466552,      0.903101,     -1.244007,     -1.407635,     -0.320775,
		     -1.039677,     -0.123417,      1.488974,      0.405496,     -0.102308,
		     -0.913522,      1.117882,      0.234053,     -1.319547,      0.000000,
		      0.718204,     -0.207442,      0.009392,     -0.141118,      1.497064,
		      0.504743,     -2.239671,     -1.153221,      0.620954,     -0.073370,
		      0.233957,     -1.137007,      0.410993,      0.588933,      1.188557,
		     -1.269802,     -0.659429,      1.517398,      1.206100,     -0.789427,
		     -1.854101,     -3.214419,     -1.771052,     -0.073837,     -1.008428,
		     -0.449993,     -0.656568,     -0.723081,     -0.440229,     -0.519166,
		      0.030805,     -0.337000,     -1.822558,     -0.258082,      0.497422,
		      1.432386,     -0.249742,      0.698041,     -0.502183,      0.952897,
		      1.923177,      0.610379,     -0.778768,      0.910184,      0.176468,
		      0.245346,     -0.145012,      0.387834,     -0.160924,      1.056140,
		      0.000000,     -0.676065,      0.319687,      0.710997,      0.943160,
		     -1.371230,     -1.824700,      0.671565,      1.292415,     -0.031699,
		     -0.119630,     -0.685776,      0.628523,      1.002967,     -0.802768,
		     -0.941620,      1.142576,      1.219598,     -0.215841,     -1.434309,
		     -0.085646,      1.264995,      1.914447,      1.350028,      2.232605,
		     -0.567362,     -1.365610,      0.521002,      0.424710,      0.939124,
		      0.237587,      0.329909,      1.335041,      1.544908,     -0.222584,
		      0.469784,      0.880526,      0.301057,      1.593560,     -0.634079,
		     -2.259281,     -1.790910,     -2.176791,     -0.732559,     -0.284959,
		      1.157630,     -0.513899,     -0.705956,     -1.420921,     -1.121516,
		      0.095019,      0.990832,      0.000000,      0.373902,      0.011041,
		     -1.975659,     -0.786879,      1.078085,      2.205372,      0.499480,
		     -2.118040,     -0.882159,      0.999588,      1.052504,      0.329337,
		     -1.479513,     -0.812293,     -0.475561,      1.875460,     -0.809423,
		     -2.003431,      0.349301,      2.037386,      0.407827,     -0.596790,
		     -1.831184,     -3.085200,     -1.063461,     -0.366541,      0.655035,
		      0.615912,     -1.352324,     -0.971022,     -0.033596,     -0.593721,
		     -0.405499,     -1.022322,      0.191887,     -0.945154,     -0.267165,
		     -2.360150,     -0.130554,     -0.578189,      1.188178,      3.290398,
		      1.651343,      0.502421,      1.488162,      1.281214,      0.329049,
		      0.631801,      0.628490,     -1.189743,     -0.522212,      1.565588,
		      0.000000,     -0.260247,     -0.648709,      1.082507,      0.387195,
		      0.273699,     -0.108598,     -0.359397,      1.717934,      1.311688,
		     -1.184706,     -1.716849,     -1.729170,      1.113073,      2.699182,
		      1.542629,     -2.534805,     -0.414192,      1.736244,      1.246939,
		     -0.975858,     -1.248047,     -0.230216,      1.511472,      3.088592,
		      4.154632,      2.214850,     -1.626221,     -1.097933,      0.180940,
		      1.250870,      0.833740,      0.653900,      0.252380,      1.128817,
		     -0.250911,      0.239900,      0.930351,      1.025798,      0.057786,
		      1.842736,      0.477276,     -0.030685,     -1.152023,     -1.514268,
		     -2.152784,     -0.550563,      0.901216,     -0.258926,     -0.571596,
		     -0.539123,     -1.288586,     -0.663028,      0.945301,      0.000000,
		      0.622365,      1.621012,      0.523205,     -1.302508,     -0.771970,
		     -0.164601,      0.454858,      0.078015,     -1.220276,     -0.121194,
		      0.640247,      2.017263,      0.259239,     -2.596551,     -1.970711,
		     -0.284788,      2.929564,      0.970652,     -2.056509,     -1.141459,
		      1.130702,      0.211261,     -0.614684,     -1.392027,     -3.841083,
		     -2.822097,     -1.062517,      0.856348,      1.926224,     -1.057175,
		     -1.064804,      0.585029,     -0.607529,     -0.355420,     -0.489364,
		     -0.133372,     -1.014196,      0.733312,     -1.090951,     -1.448413,
		     -2.004849,     -1.315209,      0.737033,      1.304905,     -0.037058,
		      0.796374,      1.750064,      0.870891,      0.668645,      2.170374,
		      0.697131,      1.358555,      0.336667,     -0.342985,      0.000000,
		     -0.661774,     -1.107618,     -0.745611,      0.689165,     -0.661451,
		     -0.687627,     -0.357230,     -0.010624,      1.231528,      2.060797,
		      0.795731,     -1.961005,     -2.737624,      1.150356,      2.929069,
		      2.993542,     -2.722989,     -3.041231,      0.199429,      2.279874,
		      0.546139,      0.315488,     -0.685056,     -0.423729,      1.711013,
		      3.301980,      3.527308,      0.730795,     -0.748690,      0.102831,
		      0.868966,     -1.312455,      0.889842,      0.430425,      0.350762,
		     -0.882114,     -0.792034,     -0.106970,     -0.378445,     -1.346812,
		      2.050049,      2.233522,      0.277098,     -1.153234,     -0.095193,
		     -0.594867,     -1.712312,     -0.417599,      0.392788,     -0.058784,
		     -0.469779,     -2.126275,     -0.583407,      0.584623,     -1.352997,
		      0.000000,     -0.554425,     -0.453070,     -0.091938,     -0.008895,
		      1.125600,      0.917148,     -0.062962,     -0.234711,      0.707307,
		     -1.258896,     -1.173248,      0.951412,      3.414252,      0.467943,
		     -3.453353,     -3.217488,      0.177687,      3.620425,      2.533534,
		     -0.569095,     -1.769842,     -0.791436,      0.278040,      0.350944,
		      0.967733,     -2.263662,     -3.272697,     -1.975185,     -0.530238,
		      1.124569,     -0.345877,      0.069624,      1.382940,     -0.402056,
		     -0.543043,      0.135533,      0.734032,     -0.356528,      0.149069,
		      0.416116,     -0.834329,     -0.426805,     -1.499152,     -0.565160,
		     -0.949852,     -0.384108,      0.601508,      0.647818,     -0.389345,
		      0.015955,      1.319315,      2.379575,      0.842760,      1.302341,
		      1.171383,     -0.816920,      0.000000,      1.308854,      0.133692,
		      0.506153,      0.401412,     -0.011985,     -0.737032,     -0.159354,
		     -0.332017,     -1.670969,     -0.810445,      1.607158,      1.295502,
		     -2.230311,     -2.684510,      2.341668,      3.032131,      2.670238,
		     -1.375128,     -3.109665,     -0.915909,      1.285872,      1.714313,
		      1.438565,     -0.706548,     -1.810639,      0.937005,      1.389991,
		      2.203820,      0.474248,     -0.660447,      1.153137,      1.842334,
		     -1.449908,     -0.092826,      0.626177,     -0.198747,     -0.927507,
		     -0.825065,      0.407032,      0.244773,     -1.778265,     -0.678654,
		      1.183191,      0.700895,     -1.075982,     -0.065240,      0.174607,
		     -0.379847,      0.088514,     -0.448002,     -1.120553,      0.362903,
		     -1.624514,     -2.272105,     -0.944795,     -0.932556,     -0.258281,
		      0.000000,     -0.113919,      1.352239,     -0.812657,     -1.387709,
		     -1.088944,      1.409976,      0.607698,     -0.178312,      0.812057,
		      2.081799,     -1.336945,     -2.437165,      0.906278,      3.771266,
		      0.008185,     -3.547638,     -3.142727,     -0.173651,      1.988326,
		      2.319794,      0.795683,     -1.052133,     -1.983741,     -1.200489,
		     -0.849575,     -0.102205,     -0.340597,     -0.950080,     -0.773895,
		     -0.416292,     -0.205455,     -2.252617,     -0.319768,      1.350769,
		      0.947444,      0.152891,     -0.463653,      0.786593,     -0.045804,
		     -0.035910,      1.358081,     -0.099995,      1.124288,     -0.072508,
		      0.592407,      0.296818,     -0.158284,     -1.442307,     -0.893641,
		      0.710883,      0.405204,     -0.855505,      1.482589,      1.807467,
		      1.523769,      0.203062,     -1.438341,     -0.663376,      0.000000,
		     -0.937011,     -0.836799,     -0.219947,      0.959196,      1.141664,
		     -1.192687,     -0.391957,      1.662351,      0.409151,     -2.058320,
		     -0.896175,      1.981625,      0.923189,     -3.192881,     -2.453268,
		      3.237124,      2.965824,      0.814363,     -0.727269,     -1.942097,
		     -1.498639,     -0.100355,      1.589884,      3.545389,      1.657028,
		     -0.823991,      1.301998,      0.693081,      0.666956,     -0.065420,
		     -1.424959,      0.408286,      2.257503,     -1.061633,     -0.148502,
		     -0.235596,     -0.657798,     -1.063384,     -0.382799,      0.050664,
		      0.889757,      0.381362,     -1.319061,     -1.104933,     -1.449893,
		      0.091968,      1.054159,      0.960473,      0.155539,     -0.585878,
		      0.469399,      0.245296,     -0.493314,     -0.594135,     -1.521350,
		     -0.192078,      0.431475,     -0.237479,      0.879490,      0.000000,
		      0.318252,     -0.266637,      2.184145,     -0.413237,     -1.265382,
		     -0.370454,      0.493949,     -1.678112,     -1.213387,      1.666103,
		      3.230053,     -0.907680,     -2.581668,      1.495817,      3.253734,
		     -0.852045,     -3.041565,     -1.805905,      0.542304,      1.128202,
		      1.112469,      1.443790,      0.506197,     -2.050794,     -0.692623,
		     -0.578706,     -2.150617,     -1.118619,      0.227510,      1.401014,
		      1.007587,      0.651898,     -2.781781,     -1.309879,     -0.312648,
		      1.615056,      1.982876,      0.400040,      0.477510,      0.287376,
		     -1.154086,      0.006236,      0.462117,      1.324468,      1.464660,
		      1.231607,     -0.084117,      0.761211,      0.237347,     -1.055110,
		     -1.130838,     -0.036184,     -1.222279,     -0.071076,      1.363256,
		      0.869307,      1.028385,     -0.889277,     -0.887377,     -0.830752,
		      0.000000,      0.835839,     -0.368781,     -2.303397,      0.164276,
		      1.060071,      1.170536,     -1.078722,      0.550104,      1.869588,
		     -0.245844,     -2.940415,     -1.415558,      2.374563,      0.867270,
		     -2.968251,     -2.305778,      2.527685,      3.051761,     -0.026131,
		     -0.622574,     -0.285301,     -0.826192,     -2.236051,     -0.272096,
		      2.046816,      1.105590,      0.022574,      1.476220,      0.190980,
		     -0.643396,     -0.057591,     -0.520201,      1.018191,      1.636937,
		      0.155599,     -0.333925,      0.114203,     -0.671365,     -1.524215,
		     -1.184245,     -0.930372,     -0.904681,      1.464726,     -0.116220,
		     -1.473317,     -1.500156,     -0.443366,     -0.272055,     -0.214058,
		      0.578213,      1.328766,      0.994342,      0.323081,     -0.889439,
		      0.055070,     -0.623802,     -0.502030,      0.654094,      1.549376,
		      2.624067,      1.800557,      0.000000,     -1.065785,      0.242525,
		      0.201448,      1.236787,     -0.218733,     -0.677539,      1.265124,
		      0.787680,     -1.971122,     -1.449077,      2.330779,      3.209380,
		     -0.828132,     -2.240303,      1.495068,      2.965840,     -1.249438,
		     -3.142143,     -1.547137,      1.048487,      0.403681,     -0.459245,
		      1.094964,      1.555077,     -1.337622,      0.131801,      1.671267,
		     -1.567171,     -1.300282,     -0.272543,      1.214123,      0.993261,
		      1.952587,     -1.053220,     -0.894726,     -2.160706,     -0.733835,
		      1.295667,      2.551945,      2.014304,      2.922435,     -0.164640,
		     -0.524200,     -0.802189,      0.298028,      1.610723,      0.896594,
		     -0.706816,     -0.731022,     -0.191141,      0.203370,     -0.699882,
		     -0.624887,     -1.070016,     -1.336439,     -0.083042,      0.317812,
		     -0.039250,     -1.286258,     -1.377153,     -0.241239,      0.225346,
		      0.000000,      0.188565,      1.078947,      1.175228,     -1.853044,
		     -0.728958,      0.138698,     -0.179230,     -1.414759,      1.241382,
		      2.729404,     -0.442938,     -3.167866,     -1.681815,      2.311486,
		      1.258996,     -1.903894,     -0.822016,      1.320021,      2.330808,
		     -0.613045,     -0.815134,      1.053322,      1.034778,     -1.373836,
		     -1.058062,      0.668844,     -0.207644,      0.472496,      2.165795,
		      0.705187,     -1.851385,     -1.236088,     -1.992396,      0.319423,
		      0.454846,      1.143820,      0.481061,      0.471909,     -1.081795,
		     -1.835922,     -1.191211,     -0.907404,     -0.600619,      0.644492,
		      0.768564,     -1.024672,     -0.843267,      0.563347,      0.188604,
		     -1.699542,     -1.341663,     -0.988788,      0.713255,      2.243694,
		      0.385352,     -0.035441,      0.691295,      0.285275,      1.832777,
		      0.177952,      1.993353,      1.171278,      0.602944,      0.000000,
		      0.972905,     -0.890079,     -0.979317,     -0.296336,      1.300705,
		      0.627259,     -0.970792,      0.904031,      0.325432,     -2.414809,
		     -1.914609,      2.156314,      3.462903,     -0.688560,     -2.101781,
		      0.692269,      2.003310,      0.347507,     -1.453203,     -1.066563,
		      1.171558,      0.768989,     -0.971568,      0.230908,      1.408876,
		     -1.380131,     -0.559500,      1.292450,     -1.542348,     -0.914085,
		      0.993097,      1.624639,      1.236674,      1.019963,      0.107464,
		      0.532709,     -0.592916,     -1.561455,     -0.552779,      1.758423,
		      1.489088,      1.592240,      1.561045,     -0.432163,     -1.183411,
		     -0.365502,     -0.148868,      0.823659,      0.015758,      0.582954,
		      1.469553,      1.464807,     -0.861599,     -2.174032,     -0.233411,
		     -0.875313,     -0.668958,     -0.948942,     -0.952069,     -0.630656,
		     -2.127843,     -1.105082,     -0.466315,     -0.305964,      0.000000,
		     -0.549936,     -0.873371,      0.093891,      1.696640,     -0.934730,
		     -1.164562,      0.737498,      0.279776,     -1.552918,      0.859101,
		      3.104906,      0.722899,     -2.388761,     -1.682783,      0.889880,
		      0.720944,     -1.418935,     -0.648368,      0.199147,      1.600799,
		     -1.232259,     -2.309250,      0.173893,      0.938467,     -0.252072,
		      0.232923,      0.439920,     -1.433492,     -0.121517,      1.383090,
		      0.892781,     -1.660504,     -0.579440,     -1.299596,      0.227107,
		     -1.324645,     -0.225440,     -0.071059,      1.461677,      0.150226,
		     -0.945593,     -1.466331,      0.491821,     -0.950556,     -0.008795,
		     -0.233961,     -0.946378,     -1.029307,      0.155425,      0.929188,
		     -0.077095,     -1.447346,     -1.486323,     -0.247345,      0.461037,
		      0.488791,      1.055327,      0.819614,      0.013418,      1.007469,
		      1.063653,      0.906685,      0.088636,     -0.085457,      0.325865,
		      0.000000,     -0.344245,      1.627032,      0.260556,     -1.548626,
		     -0.749207,      1.448146,      0.735333,     -1.158342,      1.272257,
		      1.053087,     -1.779644,     -2.649568,      0.689690,      2.737549,
		      0.377355,     -1.265308,      0.121431,      0.517202,     -0.046615,
		     -0.671699,      0.077043,      1.586406,      1.192044,     -0.141942,
		      0.258653,      0.860504,     -0.302025,      0.925651,      1.693366,
		     -1.229155,     -1.536830,      0.239278,      0.561591,      1.066831,
		      0.753912,      0.373312,     -0.044760,      1.155515,     -1.406719,
		     -0.762474,      0.666283,      0.678234,     -0.034078,      1.333315,
		      1.041733,     -0.242301,      0.631148,      1.039656,      0.873631,
		      0.156155,     -0.032533,      0.030965,      1.420926,      0.482107,
		     -0.690661,      0.591063,     -0.989427,     -0.253185,      0.793209,
		     -0.032598,     -0.719103,     -1.156490,     -0.610412,     -1.610147,
		     -2.127745,     -0.883540,      0.000000,      0.567341,     -0.141204,
		     -0.498975,      1.009038,      2.150039,     -0.767557,     -1.782831,
		      0.858769,      0.404938,     -2.160078,     -1.031224,      2.113207,
		      1.001762,     -1.752596,     -0.776519,      0.760581,      1.080307,
		     -0.616570,     -0.081412,      0.690163,      1.499737,     -0.547990,
		     -2.114061,     -0.768114,     -0.364316,     -1.211437,     -0.153583,
		     -0.180852,     -2.191998,     -0.363114,      1.034362,      0.973780,
		     -0.990872,     -0.896578,     -0.655803,      0.817518,      0.329425,
		     -1.162521,     -0.908863,     -0.560997,     -0.267678,     -0.788609,
		     -0.418366,      0.422585,     -0.023629,     -0.812287,     -1.911637,
		     -1.120280,     -1.655624,     -0.701102,      0.960546,      1.462957,
		     -0.619375,     -0.516205,     -0.366939,      0.046024,      0.142729,
		     -1.132278,     -0.145171,     -0.776345,     -0.108263,      1.143431,
		      0.748785,      1.780054,      1.026101,     -0.165969,      0.731772,
		      0.000000,     -0.078194,     -0.751461,      0.613147,     -0.445412,
		     -2.072815,     -0.293730,      2.138700,      0.015672,     -1.854519,
		      1.266237,      2.552101,     -0.042009,     -2.019444,      0.057963,
		      1.203708,     -0.092485,     -1.430579,      0.318328,      0.397427,
		     -1.089026,     -1.953980,      0.065701,      1.510281,      0.770607,
		      0.209292,      0.996494,      0.551317,     -0.028228,      1.719521,
		      2.578287,      0.064322,     -0.869350,      0.740119,     -0.247948,
		      0.160360,     -0.128286,      0.505976,     -0.615896,      1.420700,
		      0.618971,      0.540445,      1.481519,      1.000438,     -1.669738,
		      1.004970,      0.665712,      0.701647,      1.105541,      1.725501,
		      2.095200,      1.494869,     -0.647270,     -0.858898,     -0.052001,
		      1.230380,      0.868549,      0.689015,      0.152002,     -0.196494,
		      0.789837,      0.608829,      0.286138,     -0.675497,     -1.004152,
		      0.243532,      0.193088,     -0.936369,     -0.610582,      0.000000,
		      0.005685,      0.066802,     -0.751755,     -0.662033,      1.181750,
		      1.300858,     -1.721986,     -0.511499,      2.047911,      0.900580,
		     -1.669261,     -1.741499,      1.227960,      1.300049,     -1.082259,
		     -0.303267,      0.856104,      0.426867,     -0.772481,      0.004631,
		      1.071059,      0.749676,     -0.549222,     -1.424333,      0.302518,
		      0.159451,     -1.002421,     -0.199956,     -0.299602,     -2.686413,
		     -1.015646,      0.099123,     -0.111222,     -0.370522,     -1.111904,
		     -1.184766,     -0.401565,      1.591427,      0.064045,     -0.124047,
		     -1.752548,     -1.572819,     -1.554849,     -0.072694,      0.551585,
		      0.146906,     -0.389447,     -1.462190,     -0.932516,     -1.474997,
		     -1.674755,     -1.065537,      0.569513,      0.199200,     -0.058847,
		     -0.352749,     -0.989200,      0.358999,     -0.331603,     -0.870608,
		     -1.562862,     -1.242867,      0.125800,      1.395071,      1.124852,
		      0.915118,      1.377325,      1.261234,      0.208998,      0.000000,
		     -0.215156,      0.804381,      0.907986,      0.557172,     -0.863855,
		     -1.979661,      0.650863,      1.445651,     -0.705646,     -2.048848,
		      0.035549,      1.823710,      0.374733,     -0.899404,      0.638360,
		      0.534595,     -0.333116,     -1.268181,      0.788028,      1.523480,
		     -0.819975,     -2.107128,      0.087143,      2.013351,      0.221564,
		     -0.790941,      0.619132,      0.161516,     -0.398398,      1.620827,
		      1.921073,      0.940870,      0.188690,      1.810864,      1.378721,
		      0.317688,     -0.331870,     -0.419361,     -1.211259,     -0.353937,
		      0.477415,      1.456435,      2.087041,      1.134929,     -0.601191,
		      0.624324,      0.121461,      0.496927,      0.642761,      1.312323,
		      1.710809,      2.360561,      0.428134,     -1.230420,     -1.499331,
		      0.913837,      0.929036,     -0.109534,      0.760025,      0.557413,
		      2.415353,      0.768438,     -0.434393,     -0.820276,     -0.938837,
		      0.203790,      0.730737,      0.974103,      1.622727,      1.011179,
		      0.000000,      0.420136,     -0.001969,     -0.581879,     -0.467626,
		      0.797925,      1.860066,      0.376809,     -1.888346,     -0.208989,
		      2.031955,      1.057761,     -0.664401,     -1.306579,     -0.236001,
		     -0.414324,     -0.434710,      0.430739,      1.118591,     -0.033010,
		     -1.242925,      0.421002,      2.495908,      1.107514,     -1.451749,
		     -1.937801,     -0.008154,      0.699857,     -0.208476,     -0.000459,
		     -0.321015,     -2.137613,     -1.226711,     -0.096871,     -0.969406,
		     -0.227799,     -0.016098,     -0.073820,     -0.778844,      1.224769,
		      0.651678,      1.622918,     -0.811705,     -2.376461,     -1.744502,
		     -0.004527,     -0.625301,      0.213135,      0.139311,     -0.427169,
		     -0.703110,     -0.233736,     -1.772075,     -1.657709,     -0.494316,
		      1.265800,      0.605715,     -0.632152,     -0.921065,     -0.703062,
		     -0.399230,     -1.721906,     -0.444336,     -0.162515,      0.803537,
		      0.269004,     -0.232856,      0.011613,     -0.808195,     -1.092185,
		     -0.238955,     -0.430712,      0.000000,     -0.448591,     -0.944405,
		      0.296478,      1.019071,     -0.945989,     -0.763591,     -0.318549,
		      1.026594,      0.116620,     -0.811633,     -0.398570,      0.133390,
		      0.771275,      0.360881,      0.256772,      0.783648,      0.455960,
		     -0.678215,     -1.079777,      0.077669,      0.412904,     -1.249818,
		     -2.490704,     -0.222515,      2.088057,      0.500421,     -1.423498,
		     -0.256361,     -0.028092,     -0.222798,      2.061890,      1.473229,
		     -0.201285,     -0.664201,      0.340472,      0.937218,      0.807484,
		      0.168421,      0.114798,     -0.214127,     -1.593891,     -0.312962,
		      1.129012,      2.470552,      0.730076,     -0.347520,      0.597118,
		      0.235694,     -0.249314,     -0.563988,     -0.257446,      0.152859,
		      2.545534,      1.758213,      0.206644,     -0.387754,     -0.176122,
		      0.359811,     -0.693285,     -0.192891,      0.585272,      1.794816,
		      1.350736,      0.284848,     -0.282093,     -0.467879,     -0.901518,
		      0.681747,      0.796456,      1.005873,      1.229066,      0.521499,
		      0.000000,      0.259323,      1.045121,     -0.077323,     -1.526984,
		     -0.103492,      0.876582,      0.334119,     -0.765991,     -0.429062,
		      0.384858,     -0.107375,     -0.014779,      0.843902,      0.500342,
		     -0.360763,     -1.565760,     -0.663384,      0.751113,      1.144750,
		      0.516192,     -1.215542,     -0.670120,      1.874353,      2.203974,
		     -0.072781,     -0.830434,      0.408190,      0.838900,      0.427075,
		      0.125139,     -0.875934,     -1.332635,     -0.265043,      1.049600,
		     -0.668331,     -1.143417,     -1.151452,     -0.179545,     -0.783185,
		      0.457250,      0.968594,      2.234411,      1.064124,     -1.960188,
		     -1.294584,     -0.181700,     -1.232415,     -0.885642,      0.674506,
		      1.027953,      0.209881,      0.013157,     -1.359765,     -2.519098,
		     -1.921394,     -0.013324,      0.827250,      0.419493,      0.409967,
		      0.595940,      0.417449,     -2.367199,     -1.422244,     -0.047576,
		      1.114377,      1.316089,      0.346747,     -0.669716,     -0.801062,
		     -1.587833,     -1.408276,     -0.999582,     -0.491986,      0.000000,
		      0.211948,     -0.421616,     -0.126865,      1.393961,      1.072193,
		     -2.243005,     -0.958245,      1.222144,      1.086715,     -1.127780,
		     -1.002881,      0.518675,     -0.647960,     -1.103597,      0.099468,
		      1.301508,      0.453603,      0.025654,      0.072682,     -0.001401,
		      0.998158,      1.085654,     -0.324897,     -2.585233,     -1.249470,
		      0.410004,      0.258023,     -0.961600,     -0.963427,     -0.851185,
		     -0.902635,      1.595490,      1.299549,     -0.431962,     -0.006803,
		      0.429639,      1.027073,      0.618297,      0.005518,     -0.502152,
		      0.459434,     -1.730249,     -2.142261,      0.331362,      1.498409,
		      0.939426,      0.236925,      0.172278,      0.422018,     -0.754164,
		      0.090103,     -0.739752,     -0.145045,      0.685334,      1.624018,
		      0.997420,      0.121306,     -0.368990,      0.737540,     -0.108858,
		     -0.977475,      0.088129,      0.812559,      0.737573,     -0.788275,
		     -1.052813,      0.201036,      0.465545,      0.795436,     -0.724131,
		     -0.420605,     -0.241251,     -0.550634,     -0.164707,      0.000000,
		     -0.537357,     -0.026932,      0.327200,     -0.630620,     -0.935244,
		      1.572471,      1.424642,     -0.363049,     -1.231327,      1.049454,
		      1.716180,     -1.171149,     -0.931223,      1.261420,      1.464789,
		     -0.031961,     -1.150498,     -0.815464,     -0.822105,     -0.504345,
		      0.295598,      0.163851,     -0.933664,     -0.256254,      0.879138,
		      0.248485,     -0.233820,      0.256645,      0.867059,      1.523211,
		      1.116288,     -1.048416,     -1.656835,     -0.378962,      1.431321,
		      0.732367,      0.176312,     -0.955005,     -0.377408,     -0.651449,
		     -0.586719,      0.178380,      1.875155,      1.700205,     -0.988437,
		     -1.041522,      0.058517,     -0.558211,     -0.956340,      0.025542,
		      0.769994,     -0.483909,      0.644090,      0.204717,     -0.165447,
		     -2.107446,     -1.607559,     -0.296822,     -0.847600,      0.616785,
		      1.561480,      1.335317,     -0.675676,     -0.761324,     -0.484044,
		      0.495796,      0.288410,     -0.463004,     -0.057656,      1.580345,
		      0.521203,     -0.408810,     -0.299938,     -0.120007,      0.521943,
		      0.000000,      0.052140,      0.063719,     -0.571646,     -0.228239,
		      0.901548,      0.456567,     -1.381557,     -0.788024,      0.974507,
		      0.193148,     -0.677974,      0.698762,      1.388938,     -0.773167,
		     -2.263771,     -0.584646,      0.941303,      0.407770,      0.456564,
		      0.390942,     -0.707954,     -0.606395,      0.683343,      2.140077,
		      0.291293,     -0.358247,     -0.626885,     -0.534142,     -0.548819,
		     -0.779206,     -0.659392,     -0.502437,      0.999012,      0.973716,
		     -1.483873,     -1.102356,     -0.902941,      0.467252,      0.883407,
		      0.691015,     -0.475001,      0.853254,     -0.843573,     -2.213702,
		     -0.784360,      0.521372,      0.345960,      0.399451,      0.173460,
		      0.345851,     -0.706909,     -0.521386,      0.035138,     -0.747555,
		     -0.907541,      0.140387,      0.644000,      0.008570,      0.109357,
		      0.076770,      0.320487,     -1.071795,     -0.218460,     -0.587171,
		      0.248457,     -0.152474,     -1.324409,     -0.572036,      0.004390,
		     -0.291383,      0.015472,     -0.120385,     -0.205212,     -1.000879,
		     -1.087004,      0.363700,      0.000000,      0.721971,      0.498102,
		      0.508023,     -0.000430,     -1.108851,     -0.566626,      0.952708,
		      0.534785,     -0.663721,     -0.779530,      0.339355,      0.556919,
		     -1.556074,     -0.850383,      1.198380,      1.098861,      0.638689,
		      0.669247,      0.664693,     -0.602265,     -0.499498,      0.301636,
		      1.268619,     -0.489559,     -1.805795,     -0.707006,      0.363208,
		      0.174069,      0.161363,     -0.138555,      0.897307,      1.205684,
		     -0.288007,     -0.898440,     -0.068897,      0.651511,      1.002203,
		      1.114570,      0.069600,     -0.462692,     -0.066675,     -0.899399,
		      0.028212,      1.373589,      1.738962,      0.156926,     -1.180808,
		     -0.688945,      0.016936,     -0.124704,      0.864936,      0.443812,
		     -0.394208,     -0.712965,      1.191322,      2.013740,      0.351304,
		     -0.674746,     -0.770392,     -1.390892,     -0.712588,     -0.147912,
		      0.875983,      1.267924,      0.022616,     -0.431496,      0.047991,
		      0.351232,      0.290369,     -0.041468,      0.655624,      0.878983,
		      0.993096,      0.965087,      0.344553,      0.046947,      0.380830,
		      0.000000,     -0.228809,     -0.719164,      0.110433,      0.574909,
		      0.495432,     -0.069517,     -0.222637,      0.278272,      0.040391,
		      0.066757,     -0.868047,     -0.696740,      1.679246,      1.944387,
		     -0.206012,     -1.484990,     -0.856264,     -0.901549,     -0.920073,
		      0.922405,      1.546086,     -0.132730,     -2.171262,     -1.686767,
		      1.482430,      1.814315,      0.807488,     -0.235313,     -0.790433,
		     -0.418598,     -1.081965,     -0.896855,     -0.115733,      0.816794,
		      0.763969,     -0.635923,     -0.732785,     -2.026432,     -0.771464,
		      0.110657,      0.939696,     -0.228774,      0.603074,     -0.713649,
		     -1.974155,     -1.178989,      0.213277,      0.265360,      0.276437,
		     -0.000086,      0.269616,     -0.934966,     -0.138997,      0.051607,
		      0.107693,     -1.904094,     -1.204895,     -0.555178,     -0.250864,
		      0.568126,     -0.237983,      0.382287,     -0.606765,     -1.110722,
		     -0.912981,     -0.069060,      0.032986,     -0.347830,     -1.033585,
		     -0.363759,     -0.275208,     -0.428696,     -0.815898,     -0.364006,
		     -0.337511,     -0.302628,      0.566183,      1.124975,      0.000000,
		     -0.927813,      0.606316,      0.261032,     -0.448364,     -0.597189,
		     -0.305754,      0.681185,      0.191490,      0.043595,      0.426657,
		      0.896841,      0.805559,     -0.430656,     -1.379865,     -0.124261,
		      0.677332,     -0.318095,      0.181275,      0.765307,      0.144958,
		     -1.586889,     -0.467615,      0.783002,      2.127730,      0.839078,
		     -1.253785,     -1.442354,      0.129574,      0.551815,      0.396382,
		      0.314975,      0.809581,      0.047474,     -0.579641,     -0.726669,
		      0.516470,      0.174802,      0.851353,      1.050740,      0.939045,
		     -0.295199,      0.776045,     -0.669376,      0.005303,      1.169201,
		      2.044915,      1.444580,      0.052988,     -1.276826,     -0.341424,
		      0.168294,      0.762880,      1.248209,     -0.579180,     -1.232592,
		      0.247406,      1.891045,      1.481570,      0.736815,      0.002330,
		      0.579977,     -0.678601,     -0.843361,     -0.437271,      0.788083,
		      0.327556,     -0.569318,      0.038590,      0.581760,      0.700224,
		      0.919971,      0.783829,      0.514371,      0.472598,      0.366828,
		     -0.004535,     -0.509577,     -0.923225,     -0.188244,      0.000000,
		      1.094163,     -0.162581,     -1.189580,     -0.083959,      0.808192,
		      0.798403,     -1.041628,     -0.763995,      0.753983,      0.188189,
		     -0.498195,     -1.336569,     -0.749244,      0.988003,      0.903303,
		     -0.061855,      0.551380,      0.351880,     -1.351591,     -1.472464,
		      1.179624,      1.712528,      1.057940,     -1.352109,     -2.193286,
		     -0.252298,      1.327595,      0.612446,      0.098683,     -0.292998,
		      0.215869,     -1.002743,     -1.176201,     -0.429595,      0.418070,
		      0.063697,     -0.534344,     -0.097661,     -1.250008,     -1.456343,
		     -0.461813,     -0.019035,     -0.105091,      0.474985,     -0.403198,
		     -2.067403,     -2.041762,     -0.191152,      0.548292,      0.613593,
		     -0.071950,     -0.066606,     -0.385177,      0.172364,      1.053363,
		      0.561395,     -0.900708,     -1.598266,     -1.662324,     -1.193720,
		     -0.613906,     -0.584674,      0.706917,      0.218732,     -0.447153,
		     -1.288243,     -0.885462,     -0.275317,      0.214723,     -0.832851,
		     -0.709235,     -0.742435,     -0.839127,     -0.575371,      0.080119,
		     -0.011247,     -0.345566,      1.048730,      0.344548,      0.081338,
		      0.000000,     -0.462579,     -0.249920,      1.273265,      0.752567,
		     -0.148913,     -0.613270,      0.008672,      0.500462,     -0.707548,
		     -0.638577,      0.128708,      0.756308,      0.723062,     -0.987941,
		     -1.208761,      0.673150,      0.683773,     -0.537465,      0.856303,
		      1.705389,      0.010965,     -1.776643,     -1.341965,      0.298882,
		      1.528026,      1.038458,     -0.664896,     -1.181519,     -0.790127,
		     -0.287437,     -0.406589,      0.332930,      2.025682,      1.170671,
		     -0.441533,     -0.670227,      0.665275,     -0.246840,      0.474121,
		      0.653936,      0.772272,     -0.287826,      0.951436,     -0.037402,
		      0.315054,      0.623488,      1.918322,      1.470621,      0.933373,
		     -0.847451,     -0.322286,     -0.463003,      0.585613,      0.666309,
		      0.637703,     -0.247974,      0.469338,      0.967240,      0.880513,
		      0.392746,      0.200720,      0.653343,      0.069505,     -0.009922,
		     -0.507347,      0.416798,      1.413649,      0.402672,     -0.737920,
		     -0.047713,      0.205218,      0.941630,      1.258691,      0.157820,
		     -0.479275,     -0.247948,     -0.192936,     -0.595612,     -0.054354,
		     -0.096786,     -0.294071,      0.000000,     -0.127305,      0.971490,
		     -0.406379,     -1.195444,     -0.438415,      0.183642,      0.967960,
		     -0.292539,      0.085599,      0.620461,      0.154803,      0.135010,
		     -0.342327,      0.267492,      0.728147,     -0.333107,     -1.303951,
		      0.770677,      0.556447,     -1.428830,     -1.848158,      0.374289,
		      0.932285,      0.546067,     -0.207524,     -0.759417,     -0.207942,
		      1.029016,      0.744844,      0.456354,      0.042047,      0.518981,
		     -0.953025,     -1.803245,     -0.862399,      0.599720,      0.117045,
		     -0.248348,     -0.708415,     -0.394953,     -0.600184,      0.173623,
		     -0.438391,     -0.283354,     -0.170895,      0.295666,     -1.762470,
		     -1.717691,     -1.324489,      0.277212,      0.605576,      0.416054,
		     -0.125651,     -0.449729,      0.173922,      0.155691,      0.372904,
		      0.232388,     -0.186562,     -1.124632,     -1.215641,     -1.012840,
		     -0.483719,     -0.131886,      0.631455,      0.222855,     -1.184803,
		     -1.029257,     -0.343314,      0.379727,     -0.052931,     -1.002716,
		     -0.939963,     -0.197301,      0.076134,      0.264267,     -0.057294,
		     -0.471753,      0.115659,      0.341904,     -0.489537,     -0.784352,
		      0.000000,      0.190364,     -1.003217,     -0.210461,      0.256925,
		      0.169033,      0.466613,     -0.419908,      0.217302,      0.066100,
		     -0.368239,     -0.088065,      0.143840,      0.391455,      0.518397,
		     -0.595597,     -0.897341,      0.798222,     -0.020599,     -1.110857,
		      0.501917,      2.237111,      0.672120,     -0.634858,     -0.792104,
		     -0.060641,      0.295820,      0.466873,     -0.189627,     -0.187475,
		      0.018103,     -0.141101,     -1.017979,     -0.767134,      1.477083,
		      1.764470,     -0.208248,     -0.991434,      0.460960,      0.329479,
		      0.160517,      0.070238,     -0.169863,     -0.625766,      0.483136,
		      0.238019,     -0.129916,      0.538711,      1.160267,      1.230776,
		      0.900723,     -0.651818,     -0.219965,     -0.343719,      0.191324,
		      0.284809,      0.060821,      0.061003,      0.410100,      0.243718,
		      0.138700,      0.067898,      0.001905,      0.259784,      0.355464,
		      0.476308,     -0.524924,     -0.014773,      0.985731,      0.549707,
		     -0.324442,      0.251044,      0.108679,      0.406001,      0.560146,
		     -0.186253,     -0.488481,     -0.059275,      0.117187,     -0.015660,
		      0.819455,      0.244889,     -0.122262,      0.163137,      0.000000,
		      0.042229,      0.082521,      0.838595,      0.910656,      0.155775,
		     -1.085963,     -0.829989,      0.385409,      0.103603,     -0.047790,
		     -0.237444,     -0.437139,     -0.385212,     -0.694395,      0.746802,
		      1.242904,     -0.136137,     -0.988291,      0.719635,      0.598029,
		     -0.951999,     -1.142094,      0.106811,      0.627715,     -0.239698,
		     -0.050635,      0.118390,     -0.238927,     -0.176449,     -0.206716,
		      0.177106,      0.541690,      1.417010,     -0.221515,     -1.852181,
		     -1.619157,      0.389270,      0.833851,      0.705976,     -0.650961,
		     -0.336606,     -0.042833,      0.739537,     -0.135467,     -0.047287,
		     -0.445103,      0.372619,     -1.283727,     -0.789028,     -1.189193,
		     -0.035834,      0.351514,      0.767381,     -0.336862,      0.297964,
		      0.507714,     -0.167335,     -0.956396,     -0.422283,     -0.044502,
		      0.040477,     -0.732062,     -0.463432,     -0.398823,     -0.520661,
		      0.274171,      0.180942,     -0.092140,     -0.235781,     -0.293149,
		     -0.485820,     -0.051205,     -0.569646,     -1.019604,     -0.049034,
		      0.976171,      0.841479,      0.195821,     -0.342703,     -0.414461,
		      0.509834,     -0.044266,     -0.787484,     -0.032522,      0.000000,
		     -0.035038,      0.558053,     -0.459102,     -0.298991,     -0.759200,
		      0.516061,      1.603107,     -0.271975,     -0.404346,      0.087429,
		      0.532424,      0.003018,     -0.241779,      0.587309,     -0.092126,
		     -0.623204,     -0.286782,      1.234165,      0.159570,     -1.048609,
		     -0.189669,      1.299738,      0.430730,     -0.337877,      0.280950,
		      0.610478,     -0.201524,     -0.477717,     -0.540819,      0.118887,
		      0.293144,     -0.014554,     -0.845153,     -0.860653,      1.090719,
		      2.608416,      0.347190,     -1.321634,     -0.748289,      0.287345,
		      0.407951,      0.245401,     -0.597301,     -0.649306,     -0.315781,
		      0.731799,     -0.171759,      0.945278,      0.472091,      0.526520,
		      0.258395,     -0.535299,     -0.982666,     -0.578808,      0.175934,
		      0.022096,      0.156343,      0.186156,      0.561986,     -0.148003,
		     -0.343144,     -0.283643,     -0.110032,      0.035056,      0.470204,
		      0.339350,      0.007052,     -0.353553,      0.000876,      0.466258,
		     -0.161409,      0.141427,      0.419261,      0.314461,     -0.136081,
		     -0.796071,     -0.775373,     -0.052285,      0.195363,     -0.065459,
		      0.241050,      0.521671,      0.321665,     -0.237718,      0.067670,
		      0.000000,     -0.224233,     -0.168299,     -0.656014,     -0.382348,
		      1.402775,      0.745988,     -1.581571,     -0.641044,      0.957342,
		      0.474894,     -0.382622,      0.086381,      0.783233,     -0.184299,
		     -0.993634,     -0.020767,      0.824642,     -0.568250,     -0.843666,
		      0.687051,      0.462744,     -0.779189,     -0.834394,     -0.109595,
		     -0.063475,     -1.106051,     -0.159969,      1.274146,      1.359990,
		      0.133366,     -0.548232,     -0.297035,      0.095916,      1.356725,
		      0.212297,     -1.810281,     -2.043423,     -0.483843,      0.481829,
		      0.926495,     -0.277971,     -0.373938,      0.245229,      0.465694,
		     -0.181243,      0.070359,     -0.175296,     -0.156920,     -1.003494,
		      0.042180,     -0.001032,      0.280316,      0.457711,      0.963045,
		     -0.318104,     -0.176633,      0.412326,      0.245241,     -0.774989,
		     -0.925149,     -0.377058,      0.126693,      0.222392,      0.068169,
		      0.375714,     -0.173604,     -0.620754,     -0.521984,      0.076261,
		     -0.194574,      0.133970,      0.182080,      0.109075,     -0.636388,
		     -0.189115,      0.593523,      0.281138,      0.022891,      0.093442,
		     -0.197941,     -0.277115,      0.634944,      0.104265,      0.339692,
		      0.261595,      0.550575,      0.000000,      0.102388,     -0.289611,
		      0.926381,      0.481114,     -1.097924,     -1.609218,      0.873097,
		      1.642741,     -0.843523,     -1.242748,      0.253155,      0.862844,
		     -0.602661,     -0.771924,      1.175183,      0.171155,     -1.130593,
		     -0.225667,      1.109715,      0.345425,     -0.543261,      0.156733,
		      1.169686,      0.428520,     -0.371400,      0.954286,      1.253378,
		     -0.468302,     -1.292027,     -0.840838,      0.253223,      0.857047,
		      0.308398,     -1.005539,     -0.911482,      0.438982,      2.533359,
		      1.355674,     -0.596810,     -1.228559,     -0.037235,      0.048891,
		      0.238531,     -0.095466,     -0.144121,     -0.726073,     -0.030055,
		     -0.677275,      1.326815,      0.598838,      0.220290,      0.058265,
		      0.042549,     -0.853442,     -0.499423,     -0.346879,     -0.150868,
		     -0.369494,      0.093902,      0.479161,      0.502979,     -0.328335,
		     -0.235976,     -0.398574,     -0.130472,      0.238186,      0.777284,
		      0.945729,     -0.153370,     -0.386756,     -0.285486,     -0.443110,
		     -0.088805,      0.446785,      0.277842,     -0.197807,     -0.083434,
		     -0.104472,     -0.232210,     -0.021760,     -0.063386,     -0.305232,
		     -0.215177,     -0.544552,     -0.152112,      0.461170,      0.283566,
		      0.000000,      0.150962,      0.190053,      0.055017,     -0.718421,
		     -0.167099,      1.945834,      0.361121,     -2.110450,     -0.307980,
		      1.600064,      0.138718,     -1.643478,     -0.292973,      1.432478,
		     -0.130620,     -0.526383,      0.588465,      0.759206,     -0.791954,
		     -0.817866,      0.652956,      0.279443,     -0.939530,     -0.556947,
		      0.284808,     -0.562973,     -1.908285,     -0.937983,      0.909581,
		      1.657239,      0.277478,     -0.707926,     -0.567264,     -0.213971,
		      0.972095,      0.516431,     -1.059733,     -1.440965,     -0.320636,
		      0.490893,      0.605984,      0.104876,     -0.119971,      0.397252,
		      0.355400,     -0.131280,      0.206809,      0.934554,     -0.370878,
		     -1.277891,     -0.754289,      0.332374,      0.190617,      0.444369,
		      0.586634,      0.246710,     -0.111737,     -0.140662,      0.234409,
		     -0.266350,     -0.053044,     -0.114759,      0.468061,      0.155269,
		     -0.259154,      0.183595,      0.014491,     -0.476259,      0.164872,
		      0.374133,     -0.328244,      0.375720,      0.304534,      0.330253,
		     -0.088248,      0.021502,      0.263203,      0.371392,      0.043398,
		     -0.293595,     -0.232080,     -0.301044,      0.634819,      0.373712,
		     -0.279451,     -0.010593,      0.416509,     -0.051195,      0.000000,
		     -0.294481,     -0.085686,     -0.553531,      1.108223,      0.965702,
		     -1.659533,     -1.344208,      1.351463,      1.525204,     -0.924578,
		     -0.956794,      1.151321,      1.468824,     -0.912438,     -1.079300,
		      1.183915,      0.196190,     -1.245128,     -0.104018,      0.591638,
		     -0.384920,     -0.669837,      0.359928,      0.738494,      0.182105,
		     -0.096936,      1.307364,      1.691679,     -0.155995,     -1.425532,
		     -1.130393,      0.315391,      0.949328,      1.092315,     -0.537716,
		     -0.808781,     -0.314485,      1.272677,      0.959031,     -0.076502,
		     -1.003377,     -0.337424,     -0.445953,     -0.457798,     -0.004824,
		      0.630031,     -0.398656,     -0.426905,     -1.024731,      0.720731,
		      1.065132,      0.136805,      0.098882,      0.171773,     -0.507186,
		     -0.156026,     -0.018865,      0.191166,     -0.078615,     -0.014157,
		     -0.120810,      0.189040,      0.267796,      0.288481,      0.022389,
		     -0.707617,     -0.583840,     -0.082753,      0.466851,      0.327608,
		      0.451668,     -0.151095,     -0.538188,     -0.419655,     -0.115489,
		     -0.139623,     -0.297631,      0.280434,      0.047375,      0.237826,
		      0.049577,     -0.175922,     -0.557834,      0.092713,      0.022219,
		     -0.293155,      0.130972,      0.518399,      0.309005,      0.000000,
		     -0.005205,      0.322063,      0.468388,     -0.887334,     -1.086464,
		      0.486644,      1.780459,     -0.091771,     -1.742045,     -0.111840,
		      1.609960,      0.259842,     -1.719636,     -0.137362,      1.277077,
		     -0.942219,     -0.923928,      0.905652,      0.750559,     -0.370683,
		     -0.030192,      0.922128,      0.142615,     -0.767759,     -0.277700,
		      0.613996,     -0.487843,     -1.708452,     -0.692113,      0.612594,
		      1.445549,      0.247673,     -0.534522,     -0.621045,     -0.079652,
		      0.361597,      0.557936,     -0.540420,     -0.933976,      0.062449,
		      0.885133,      0.569107,      0.505055,      0.408653,      0.314539,
		     -0.239215,     -0.358062,     -0.034539,      1.496824,      0.454389,
		     -0.680184,     -0.957485,     -0.212615,      0.061349,      0.229242,
		     -0.053299,      0.289794,     -0.181967,     -0.424412,     -0.189400,
		      0.264888,      0.133975,      0.212744,      0.162434,      0.313119,
		      0.345933,      0.569807,      0.198271,     -0.110058,      0.344071,
		      0.186978,     -0.234037,      0.413792,      0.403702,      0.457716,
		      0.559066,      0.244300,     -0.056262,     -0.153455,     -0.260316,
		     -0.082626,      0.262809,      0.003277,      0.035379,      0.311438,
		     -0.082139,     -0.459588,     -0.380049,     -0.412223,     -0.462570,
		      0.000000,      0.538746,     -0.514951,     -0.575153,      0.150408,
		      1.359451,      0.750192,     -1.537043,     -0.536590,      0.911957,
		      0.511947,     -1.365560,     -1.186779,      0.893546,      0.980526,
		     -0.861431,     -0.372381,      1.314354,      0.202432,     -0.728114,
		      0.187262,      0.452675,     -0.760166,     -0.592290,      0.426939,
		      0.157090,     -0.597555,     -0.154773,      1.047413,      1.024068,
		     -0.023978,     -0.814929,     -0.928027,     -0.122039,      0.256105,
		      0.731320,      0.057868,      0.008541,     -0.017309,      0.510858,
		      0.137719,     -0.385342,     -0.695045,     -0.209792,     -0.605545,
		     -0.599372,      0.069104,      0.977097,      0.104503,     -0.799209,
		     -1.089555,     -0.288005,      0.533399,      0.294968,      0.361867,
		      0.109881,     -0.185353,     -0.025474,      0.282057,      0.688887,
		      0.339071,      0.140404,     -0.798837,     -0.177622,      0.031240,
		      0.266028,     -0.135873,     -0.742808,     -0.425216,     -0.115463,
		     -0.353032,     -0.358638,      0.075150,      0.147256,      0.212352,
		     -0.397419,     -0.707371,     -0.276579,     -0.468290,     -0.068238,
		      0.454501,      0.160107,     -0.185152,      0.108254,      0.082726,
		      0.073244,      0.232213,      0.382149,      0.133617,      0.334183,
		      0.152244,     -0.168051,      0.000000,     -0.530678,      0.019816,
		      0.931016,      0.885622,     -1.256714,     -1.183196,      0.620662,
		      1.028090,     -0.185663,     -0.696172,      0.314750,      1.185665,
		      0.080480,     -1.136962,      0.679072,      1.465980,     -0.855838,
		     -1.089229,      0.318345,     -0.089178,     -0.575403,      0.480600,
		      0.904260,      0.126492,     -0.389693,      0.326945,      0.399174,
		     -0.595814,     -1.220815,     -0.182971,      0.354199,      1.060987,
		      0.268435,     -0.137473,     -0.396851,      0.037320,     -0.347187,
		      0.076978,     -0.081483,     -0.304754,      0.354264,      0.933377,
		      0.236105,      0.686820,      0.887350,      0.587040,     -0.631391,
		     -0.484510,     -0.292074,      1.254413,      1.084777,      0.242630,
		     -0.337211,     -0.359688,     -0.089789,      0.272470,     -0.423113,
		     -0.127601,     -0.201279,     -0.341693,     -0.275287,      0.262902,
		      0.197085,      0.101667,      0.018697,      0.170677,      0.426000,
		      0.388661,      0.297350,      0.165192,      0.564586,      0.258753,
		     -0.345749,     -0.107219,      0.727718,      0.786849,      0.410564,
		      0.290429,     -0.374025,     -0.536365,     -0.439597,     -0.049987,
		      0.119612,     -0.133998,     -0.382844,      0.089268,     -0.040307,
		     -0.033380,     -0.100557,     -0.475730,     -0.589195,      0.034395,
		      0.000000,      0.092170,      0.917649,     -0.815750,     -1.493136,
		      0.226535,      1.116837,      0.428560,     -1.098855,      0.178072,
		      0.800612,      0.388225,     -0.691029,     -0.492016,      0.835956,
		     -0.059863,     -1.490019,     -0.242908,      1.272795,      0.183449,
		     -0.076583,      0.384345,     -0.113612,     -1.013978,     -0.359198,
		      0.670287,     -0.000968,     -0.194446,      0.481929,      1.054884,
		      0.139762,     -0.295992,     -0.337116,     -0.064440,      0.315049,
		      0.051465,     -0.112360,     -0.016273,      0.225708,      0.351536,
		      0.242948,     -0.257411,     -0.705484,     -0.440576,     -0.412762,
		     -0.932724,     -0.954911,     -0.075675,      0.945698,      0.438606,
		     -0.722348,     -0.837383,     -0.761983,     -0.606747,     -0.278316,
		      0.314220,      0.137528,      0.015051,     -0.046069,      0.149892,
		      0.307061,      0.376071,      0.343142,     -0.353593,     -0.496644,
		     -0.383595,     -0.053266,     -0.293975,     -0.522577,     -0.108550,
		      0.135525,     -0.232174,     -0.599925,     -0.210426,      0.016860,
		     -0.004158,     -0.226388,     -0.069153,     -0.131930,     -0.163883,
		     -0.197694,     -0.033422,      0.028533,      0.055191,      0.125455,
		      0.404440,      0.390882,      0.387581,      0.361745,      0.091234,
		      0.423815,      0.116964,     -0.550555,     -0.072758,      0.000000,
		      0.235898,     -1.119674,     -0.163950,      1.281568,      1.132703,
		     -0.678098,     -0.976759,      0.453733,      0.075441,     -0.508704,
		     -0.390967,      0.308262,      0.510166,     -0.618943,     -0.981567,
		      0.782332,      1.218119,     -0.683298,     -0.625499,      0.234106,
		     -0.217952,     -0.395003,      0.872113,      0.719633,     -0.367430,
		     -0.302333,      0.401398,     -0.079982,     -0.710859,     -0.536711,
		      0.135183,     -0.099532,     -0.037937,     -0.268139,     -0.128143,
		      0.171076,      0.669924,     -0.260835,     -0.344464,     -0.077356,
		      0.098604,      0.645972,      0.989422,      0.060303,      0.591362,
		      0.988476,      0.931808,     -0.526554,     -0.464373,     -0.192696,
		      0.841606,      1.128254,      1.075722,      0.578932,     -0.063720,
		     -0.247834,      0.203630,      0.049508,     -0.047506,     -0.204079,
		     -0.182765,     -0.578458,      0.109440,      0.197470,      0.373462,
		      0.022656,     -0.226945,      0.195735,      0.186096,      0.223439,
		     -0.031892,      0.300754,      0.467612,      0.047680,     -0.298191,
		      0.326572,     -0.031028,     -0.173067,      0.342508,      0.197630,
		     -0.104559,     -0.295371,     -0.260411,      0.090320,     -0.342657,
		     -0.730291,     -0.632134,     -0.363790,      0.027164,     -0.097843,
		     -0.040128,     -0.003094,     -0.114954,      0.200807,      0.000000,
		     -0.413747,      0.472549,      1.274427,     -0.336065,     -1.691247,
		     -0.158570,      0.974082,      0.445786,     -0.459562,      0.025871,
		      0.215416,     -0.201234,     -0.485987,      0.339738,      1.420686,
		      0.075603,     -1.516265,     -0.475727,      0.768312,      0.166981,
		      0.226010,      0.734675,     -0.288874,     -1.007652,     -0.336263,
		      0.385443,     -0.595789,     -0.319808,      0.550719,      0.797996,
		     -0.246962,     -0.256550,      0.043244,      0.246774,      0.510193,
		      0.328425,     -0.468323,     -0.227539,      0.097719,      0.342638,
		      0.100283,     -0.457290,     -0.915252,     -0.070784,     -0.166302,
		     -0.657300,     -1.047006,     -0.175548,      0.400912,      0.419404,
		     -0.814931,     -0.630003,     -0.590053,     -1.058121,     -0.936663,
		     -0.242168,     -0.247298,     -0.189843,     -0.072785,      0.210017,
		      0.351626,      0.044509,     -0.040823,     -0.262557,     -0.397311,
		     -0.188622,     -0.054442,     -0.426208,     -0.786395,     -0.352809,
		      0.371655,     -0.011984,     -0.616912,     -0.278429,      0.130290,
		      0.138077,      0.213588,     -0.021976,     -0.256120,     -0.110760,
		     -0.326877,     -0.157031,      0.044394,     -0.142281,      0.328744,
		      0.756016,      0.714211,      0.516951,      0.122903,     -0.091258,
		     -0.116815,     -0.262271,     -0.177995,     -0.059332,      0.284698,
		      0.000000,      0.331153,      0.412460,     -1.288387,     -0.787307,
		      0.911311,      1.006440,     -0.242143,     -0.801432,      0.355400,
		      0.226077,     -0.164666,      0.071628,      0.579618,      0.338003,
		     -0.963403,     -0.719918,      1.067182,      1.355915,     -0.658276,
		     -0.713765,     -0.184455,     -0.785630,     -0.547687,      0.759000,
		      0.979605,     -0.059577,      0.142242,      0.385493,     -0.201996,
		     -0.437059,      0.265193,      0.574828,     -0.145598,     -0.458962,
		     -0.502305,     -0.639534,     -0.015908,      0.623107,      0.035657,
		     -0.276831,     -0.062963,      0.183134,      0.686958,      0.677632,
		      0.034988,      0.368764,      0.708862,      0.935142,     -0.025342,
		     -0.153436,      0.091021,      0.614204,      0.699472,      1.014909,
		      1.057075,      0.725829,      0.082448,      0.050756,     -0.075477,
		     -0.267507,     -0.272457,      0.127055,     -0.030778,     -0.014273,
		      0.113660,      0.351699,      0.335459,      0.604253,      0.599611,
		      0.004224,     -0.251524,     -0.255767,      0.232366,      0.447215,
		      0.302226,     -0.088041,      0.073982,     -0.063808,     -0.284934,
		     -0.354925,     -0.068771,     -0.007793,      0.031654,     -0.032832,
		      0.167931,     -0.378630,     -0.749659,     -0.682010,     -0.281278,
		     -0.184958,     -0.015502,      0.189997,     -0.075704,     -0.220967,
		     -0.151109,      0.117945,      0.000000,     -0.199677,     -0.770793,
		      0.480482,      1.332247,      0.386817,     -1.429067,     -0.802439,
		      0.613391,      0.346484,     -0.044476,      0.167604,      0.079269,
		     -0.653341,     -0.971982,      0.155531,      1.203300,     -0.101053,
		     -1.567265,     -0.139410,      0.652822,      0.153756,      0.320735,
		      0.911219,     -0.147910,     -1.029582,     -0.277262,      0.421938,
		     -0.288484,     -0.085913,      0.301480,      0.081931,     -0.596414,
		     -0.104506,      0.373102,      0.193161,      0.269215,      0.443895,
		     -0.225993,     -0.106832,     -0.186311,      0.094040,      0.012338,
		     -0.450604,     -0.871876,      0.070483,     -0.268905,     -0.336202,
		     -0.884799,     -0.238694,     -0.024280,      0.120468,     -0.847266,
		     -0.696150,     -0.416324,     -0.619559,     -1.014523,     -0.434609,
		     -0.347753,     -0.096500,     -0.031842,      0.430366,      0.379178,
		      0.103363,     -0.096171,     -0.092207,     -0.310515,     -0.568057,
		     -0.525544,     -0.429003,     -0.206394,      0.072133,      0.276543,
		     -0.152215,     -0.418378,     -0.473601,     -0.108792,      0.018437,
		      0.177489,      0.075055,      0.082121,      0.236107,     -0.294299,
		     -0.443377,     -0.190981,     -0.213392,      0.037068,      0.579062,
		      0.793253,      0.594381,      0.342376,     -0.070844,     -0.425636,
		     -0.073753,     -0.085020,      0.143730,      0.227517,      0.266263,
		      0.000000,     -0.015072,      0.413912,      0.590625,     -0.703945,
		     -0.977698,      0.781580,      1.233484,      0.011497,     -0.863214,
		     -0.242140,     -0.150928,     -0.132154,      0.410932,      0.961719,
		      0.311722,     -1.236822,     -0.677328,      1.261228,      1.247689,
		     -0.484702,     -0.369210,      0.235465,     -0.692809,     -0.645275,
		      0.361221,      0.791473,     -0.204643,     -0.109715,     -0.142330,
		     -0.339660,     -0.312613,      0.416747,      0.480881,     -0.144537,
		     -0.320269,     -0.030287,     -0.439709,     -0.395467,      0.100612,
		      0.418321,      0.156879,      0.314613,      0.469781,      0.654480,
		      0.338132,      0.096925,      0.188223,      0.473372,      0.432746,
		      0.284591,      0.116838,      0.509117,      0.651941,      0.582248,
		      0.335480,      0.605208,      0.686116,      0.476458,      0.091069,
		     -0.050238,     -0.353745,     -0.517685,      0.186766,      0.267127,
		      0.073252,     -0.167995,     -0.159034,      0.056473,      0.451821,
		      0.702303,      0.316978,      0.160268,      0.202495,      0.070818,
		      0.024244,      0.138546,      0.056854,      0.078088,     -0.026778,
		     -0.356928,     -0.438939,     -0.298629,     -0.241211,     -0.002556,
		      0.251517,      0.039641,     -0.167501,     -0.678046,     -0.437869,
		     -0.016905,      0.121566,      0.125949,     -0.127800,      0.017356,
		     -0.083239,     -0.041842,     -0.049335,      0.047018,      0.000000,
		      0.208994,     -0.113543,     -0.991035,     -0.246215,      0.812734,
		      0.653512,     -0.915181,     -0.780513,      0.563802,      0.390123,
		      0.233930,      0.302289,      0.069972,     -0.538755,     -0.538048,
		      0.700534,      1.042914,     -0.575832,     -1.552309,      0.151663,
		      0.507024,     -0.308362,      0.083162,      0.840902,      0.133735,
		     -0.987770,     -0.386079,      0.476504,      0.511837,      0.276938,
		      0.224758,     -0.216578,     -0.595387,     -0.044950,      0.562145,
		     -0.041213,     -0.128364,      0.538873,      0.276338,      0.046121,
		     -0.412662,     -0.578909,     -0.526127,     -0.684291,     -0.660055,
		      0.189688,     -0.225795,     -0.222273,     -0.403185,     -0.060014,
		     -0.075804,      0.004915,     -0.553472,     -0.812004,     -0.501532,
		     -0.179134,     -0.589258,     -0.347912,     -0.308118,     -0.092586,
		     -0.233159,      0.162912,      0.194661,      0.114783,      0.220803,
		      0.397995,      0.255158,      0.028121,     -0.519795,     -0.532406,
		     -0.205777,     -0.022066,      0.205240,      0.110293,     -0.106293,
		     -0.396091,     -0.437161,     -0.180788,     -0.004653,      0.122082,
		      0.141809,      0.098325,      0.019212,     -0.230032,     -0.261450,
		     -0.378839,     -0.098605,      0.351485,      0.309525,      0.090828,
		      0.238277,      0.120927,     -0.063871,     -0.159913,     -0.188814,
		      0.038351,      0.188143,     -0.004439,     -0.169252,      0.000000,
		     -0.156400,     -0.028800,      0.263374,      0.675690,     -0.239733,
		     -1.261538,      0.207332,      1.145183,      0.417578,     -0.371189,
		     -0.396218,     -0.429612,     -0.356215,      0.237767,      0.845173,
		     -0.156268,     -1.211991,     -0.110981,      1.268568,      0.584792,
		     -0.539823,     -0.108323,      0.554278,     -0.346288,     -0.461050,
		      0.373053,      0.832932,     -0.189130,     -0.551994,     -0.350988,
		     -0.020403,      0.189270,      0.402346,      0.165330,     -0.466704,
		     -0.104867,      0.528153,     -0.076476,     -0.688484,     -0.653990,
		      0.110997,      0.421880,      0.748276,      0.924575,      0.771446,
		     -0.037310,      0.154587,      0.179262,      0.227620,     -0.127670,
		      0.035822,     -0.067215,      0.392491,      0.704499,      0.959729,
		      0.230141,      0.186707,      0.271789,      0.609138,      0.437065,
		      0.392601,     -0.103858,     -0.485037,     -0.402447,     -0.104705,
		      0.067283,      0.066792,     -0.137644,     -0.171495,      0.063048,
		      0.323421,      0.269677,      0.062860,      0.113219,     -0.103249,
		     -0.097451,     -0.134536,      0.034418,      0.185460,      0.109346,
		     -0.222999,     -0.268867,     -0.523841,     -0.429475,     -0.138429,
		      0.231639,     -0.010206,     -0.171830,     -0.229207,     -0.150494,
		     -0.131710,     -0.046830,      0.127518,      0.120206,      0.040145,
		     -0.069603,     -0.043750,     -0.031769,      0.164213,     -0.048795,
		      0.000000,      0.020133,      0.148297,      0.474295,     -0.375235,
		     -0.406042,      0.592945,      0.607678,     -0.736399,     -1.033826,
		      0.146149,      0.495677,      0.293535,      0.271323,     -0.187702,
		     -0.833084,     -0.223523,      1.056463,      0.637599,     -0.951022,
		     -1.153691,      0.450269,      0.636170,     -0.588561,     -0.147217,
		      0.629540,      0.221850,     -0.800894,     -0.417437,      0.358893,
		      0.672195,      0.124948,     -0.185784,     -0.183575,     -0.220767,
		      0.264582,      0.434444,     -0.410848,     -0.682409,      0.386996,
		      0.810884,      0.664265,      0.111331,     -0.534548,     -0.950749,
		     -1.075092,     -0.535493,      0.230683,     -0.123624,     -0.331758,
		     -0.061533,      0.059332,     -0.011057,     -0.229489,     -0.274356,
		     -0.783807,     -0.506217,      0.024572,     -0.112297,     -0.248349,
		     -0.252157,     -0.174135,     -0.153146,      0.269751,      0.191762,
		      0.022251,     -0.084614,      0.033016,      0.274195,      0.618180,
		      0.358288,     -0.150458,     -0.303990,     -0.215577,     -0.155786,
		     -0.001161,      0.064149,      0.047975,     -0.082838,     -0.333329,
		     -0.322791,     -0.097296,     -0.110894,      0.099835,      0.263689,
		      0.034130,     -0.119792,     -0.050623,     -0.113701,      0.151799,
		      0.208550,      0.085836,      0.056295,      0.017824,      0.049703,
		     -0.013928,     -0.057973,     -0.310469,     -0.233808,     -0.199519,
		     -0.224491,     -0.397021,      0.000000,     -0.080202,     -0.018856,
		     -0.251434,     -0.185645,      0.546471,      0.287201,     -0.794601,
		     -0.138661,      0.742765,      0.407228,     -0.341614,     -0.169479,
		     -0.052184,      0.090914,      0.405703,      0.512372,     -0.604877,
		     -0.952974,      0.524208,      1.231539,      0.058483,     -0.724679,
		     -0.074606,      0.297418,     -0.184328,     -0.315413,      0.387597,
		      0.722436,     -0.135403,     -0.827043,     -0.514530,      0.162072,
		      0.270253,      0.181959,     -0.193759,     -0.614863,      0.081354,
		      0.988527,      0.482975,     -0.556258,     -1.062752,     -0.500072,
		     -0.046598,      0.765013,      1.306189,      0.928900,     -0.179219,
		      0.025720,      0.286092,      0.253251,     -0.119405,      0.073990,
		      0.065891,      0.017159,      0.224995,      0.652581,      0.017658,
		     -0.154085,     -0.292647,      0.169525,      0.185085,      0.332504,
		      0.020554,     -0.026448,     -0.022997,     -0.090550,     -0.112103,
		     -0.147401,     -0.262142,     -0.094450,      0.037302,      0.116434,
		      0.046945,     -0.124953,     -0.043675,     -0.148715,     -0.243672,
		     -0.182503,      0.103021,      0.216365,      0.057589,     -0.267783,
		     -0.331658,     -0.400448,     -0.314110,     -0.093987,      0.004628,
		     -0.064452,     -0.200313,     -0.188576,     -0.110087,     -0.125695,
		      0.035527,     -0.084118,     -0.075198,      0.012265,      0.212860,
		      0.115262,      0.088995,      0.090324,      0.012579,     -0.229007,
		      0.000000,      0.182727,     -0.132827,     -0.355758,      0.420225,
		     -0.139895,     -0.617240,      0.315497,      0.690140,     -0.177073,
		     -0.936031,      0.048695,      0.476989,      0.139186,      0.040189,
		     -0.120249,     -0.616896,      0.109933,      1.020925,      0.039494,
		     -1.011320,     -0.663681,      0.305332,      0.601134,     -0.192911,
		     -0.196599,      0.137435,     -0.165074,     -0.573091,     -0.021616,
		      0.739014,      0.880987,      0.055284,     -0.469948,     -0.178622,
		      0.064700,      0.460777,      0.380641,     -0.634867,     -1.044046,
		     -0.051318,      0.754968,      0.731270,      0.662745,     -0.008018,
		     -0.900658,     -1.222226,     -0.526700,      0.059372,     -0.000527,
		     -0.402834,     -0.008876,      0.183743,      0.028910,     -0.175218,
		      0.063922,     -0.525513,     -0.283736,      0.137750,      0.330174,
		      0.116064,      0.095817,     -0.040686,     -0.198233,     -0.049873,
		      0.012899,      0.033491,      0.161132,      0.118953,     -0.045730,
		      0.198574,      0.347416,      0.387875,      0.262649,      0.048313,
		     -0.445070,     -0.468955,     -0.248775,      0.214810,      0.254838,
		     -0.029787,      0.000278,      0.121494,      0.045417,     -0.059141,
		      0.008819,      0.043107,     -0.069021,      0.072291,      0.255356,
		      0.454811,      0.232716,      0.080999,     -0.050636,      0.005994,
		      0.021847,      0.018291,     -0.049185,     -0.100911,     -0.082867,
		     -0.142370,      0.007190,     -0.066382,     -0.054904,      0.000000,
		     -0.175557,     -0.191631,      0.484782,     -0.041660,     -0.244303,
		      0.342037,      0.287567,     -0.498575,     -0.166324,      0.830541,
		      0.295423,     -0.745711,     -0.417340,      0.109714,      0.188879,
		      0.378197,      0.076939,     -0.811450,     -0.585005,      0.583583,
		      0.807959,      0.080135,     -0.283529,      0.022193,     -0.053695,
		      0.026221,      0.298462,      0.394945,      0.267828,     -0.434148,
		     -0.976907,     -0.424309,      0.407307,      0.344463,      0.180657,
		     -0.296819,     -0.673025,      0.049559,      0.972160,      0.695337,
		     -0.191761,     -0.729266,     -0.827689,     -0.659007,      0.313355,
		      1.164926,      1.075052,      0.008649,     -0.074927,      0.248912,
		     -0.059748,     -0.318477,     -0.098505,      0.222775,      0.034498,
		      0.120404,      0.350781,     -0.040772,     -0.252411,     -0.382284,
		     -0.214241,      0.000332,      0.066924,     -0.094382,     -0.007941,
		      0.049101,     -0.126103,     -0.152074,      0.032209,      0.018177,
		     -0.096713,     -0.272683,     -0.353665,     -0.047602,      0.167122,
		      0.298609,     -0.057444,     -0.379976,     -0.460784,     -0.070176,
		      0.011564,      0.058540,     -0.034224,     -0.087910,     -0.055393,
		     -0.170557,     -0.206276,     -0.167582,     -0.202980,     -0.266510,
		     -0.121217,      0.073733,     -0.100965,     -0.060627,     -0.063093,
		      0.006068,      0.011921,      0.089185,      0.108212,      0.029919,
		     -0.005818,      0.168725,      0.015668,     -0.077765,      0.000000,
		     -0.002164,      0.562028,     -0.070152,     -0.348449,      0.431612,
		      0.045638,     -0.594256,      0.059086,      0.462559,     -0.234552,
		     -0.786847,      0.403212,      0.652566,     -0.250231,     -0.358937,
		     -0.065718,     -0.158978,      0.356679,      0.801524,     -0.123253,
		     -0.469930,     -0.197220,     -0.211976,     -0.000243,      0.203397,
		      0.142184,     -0.229185,     -0.662092,     -0.648238,      0.069498,
		      0.777117,      0.779197,     -0.074186,     -0.493925,     -0.264465,
		      0.096784,      0.571481,      0.315162,     -0.602599,     -0.971835,
		     -0.347574,      0.363210,      0.462834,      0.770986,      0.430743,
		     -0.474283,     -0.970713,     -0.375398,     -0.003471,      0.066282,
		     -0.132710,      0.113849,      0.319566,      0.004846,     -0.330028,
		     -0.012224,     -0.363285,     -0.229192,      0.087544,      0.339365,
		      0.329585,      0.393857,      0.300984,      0.143013,     -0.090192,
		     -0.143333,     -0.145082,      0.027619,      0.023278,     -0.085993,
		     -0.020778,      0.241945,      0.224227,      0.208804,      0.173438,
		     -0.023648,     -0.129082,     -0.173815,     -0.012488,      0.097480,
		      0.021430,      0.006803,      0.107745,      0.181240,     -0.030461,
		     -0.034074,     -0.044447,     -0.063475,      0.040796,      0.102202,
		      0.222705,      0.140765,      0.021959,      0.027836,      0.092954,
		      0.035571,     -0.186846,     -0.094534,     -0.055317,      0.216632,
		      0.272358,      0.153434,      0.199291,      0.133015,      0.202489,
		      0.000000,      0.261812,     -0.414249,     -0.466820,      0.330588,
		     -0.172734,     -0.226962,      0.345016,      0.283460,     -0.508553,
		     -0.268581,      0.958276,      0.344939,     -0.741394,     -0.065952,
		      0.525792,      0.187613,      0.223753,     -0.137456,     -0.610729,
		     -0.182986,      0.184388,      0.096978,      0.133498,      0.218079,
		      0.015953,     -0.471648,     -0.150228,      0.725515,      0.750682,
		      0.235109,     -0.335299,     -0.797938,     -0.390837,      0.327936,
		      0.309637,      0.147452,     -0.173128,     -0.430399,      0.099574,
		      0.783641,      0.628250,      0.063360,     -0.135710,     -0.542523,
		     -0.871458,     -0.230455,      0.532268,      0.647616,      0.152129,
		     -0.045128,      0.304903,     -0.004517,     -0.448078,     -0.252392,
		      0.183726,      0.117588,      0.246173,      0.229898,     -0.100951,
		     -0.314841,     -0.440657,     -0.408644,     -0.149130,     -0.002392,
		     -0.061791,      0.002124,      0.207463,      0.028248,     -0.143237,
		      0.011623,      0.097390,      0.075861,     -0.245689,     -0.334771,
		     -0.323208,     -0.093286,      0.179459,      0.398669,      0.067560,
		     -0.237976,     -0.168396,     -0.099803,      0.041070,      0.051274,
		      0.042028,      0.121156,      0.020965,      0.120869,      0.046043,
		      0.045510,     -0.123571,     -0.169234,     -0.132268,     -0.119684,
		     -0.132559,      0.058056,      0.259777,      0.067630,     -0.104387,
		     -0.185357,     -0.001030,      0.078359,      0.092074,      0.056749,
		     -0.018892,      0.141824,      0.000000,     -0.366947,     -0.064597,
		      0.594234,     -0.110676,     -0.207764,      0.312680,      0.073668,
		     -0.357537,      0.203716,      0.421915,     -0.472686,     -0.778434,
		      0.523205,      0.572970,     -0.422193,     -0.436641,     -0.088523,
		      0.046822,      0.182937,      0.230621,     -0.216309,      0.050680,
		      0.229581,     -0.345880,     -0.232553,      0.431717,      0.492603,
		     -0.219136,     -0.638399,     -0.536318,     -0.063586,      0.513292,
		      0.601694,     -0.019405,     -0.334134,     -0.308451,     -0.045974,
		      0.352857,      0.184054,     -0.525552,     -0.645374,     -0.341441,
		      0.024620,      0.103774,      0.526030,      0.500369,     -0.093356,
		     -0.602355,     -0.331892,     -0.148338,     -0.137990,     -0.127234,
		      0.192544,      0.486860,      0.236678,     -0.121713,      0.012681,
		     -0.027022,      0.084017,      0.217797,      0.273770,      0.232866,
		      0.193713,      0.325760,      0.292520,      0.035040,     -0.241115,
		     -0.318305,     -0.161498,     -0.008004,      0.045934,      0.111094,
		      0.140254,      0.166190,      0.103392,      0.225513,      0.071061,
		     -0.127225,     -0.237808,     -0.036388,      0.003912,     -0.013333,
		     -0.030129,      0.051800,      0.193005,      0.005393,     -0.110329,
		     -0.182181,     -0.217688,     -0.178099,     -0.052550,      0.192160,
		      0.106951,      0.040243,     -0.029273,     -0.011442,     -0.130842,
		     -0.110667,     -0.000777,      0.040022,      0.084823,      0.148469,
		      0.162069,      0.138346,      0.168861,      0.055265,      0.069206,
		      0.000000,      0.215713,      0.371029,     -0.298900,     -0.112943,
		      0.241434,     -0.227300,     -0.185313,      0.271423,      0.196377,
		     -0.339719,     -0.165725,      0.645594,     -0.040027,     -0.682671,
		      0.053581,      0.424574,      0.034681,      0.071995,      0.026264,
		     -0.095786,      0.182430,     -0.071828,     -0.306170,      0.111457,
		      0.335052,     -0.009841,     -0.483724,     -0.287863,      0.398325,
		      0.541104,      0.213634,     -0.165967,     -0.445707,     -0.232344,
		      0.129971,      0.240662,      0.122248,     -0.126209,     -0.183400,
		      0.236449,      0.508330,      0.340470,      0.050670,      0.186322,
		     -0.047504,     -0.426984,     -0.265771,      0.144103,      0.288413,
		      0.130314,      0.000046,      0.190023,      0.046746,     -0.357356,
		     -0.383910,     -0.102763,     -0.128587,      0.000310,      0.087010,
		      0.020959,     -0.067280,     -0.187597,     -0.336415,     -0.277900,
		     -0.168929,     -0.058720,      0.112872,      0.309062,      0.127764,
		     -0.163597,     -0.188368,      0.076809,      0.091358,     -0.004896,
		     -0.156714,     -0.207419,     -0.213648,      0.009338,      0.159364,
		      0.007073,     -0.164904,     -0.025928,      0.063166,      0.041804,
		      0.029576,      0.036157,      0.015410,      0.062233,      0.200716,
		      0.205711,      0.187679,     -0.079879,     -0.103866,     -0.118181,
		     -0.072952,     -0.068921,      0.107008,      0.230869,      0.100375,
		      0.037627,     -0.117443,     -0.110660,     -0.035839,      0.051554,
		     -0.037816,     -0.003485,      0.068830,      0.019564,      0.000000,
		     -0.000094,     -0.328141,     -0.029328,      0.270222,     -0.123133,
		     -0.039977,      0.141258,     -0.042532,     -0.241014,      0.151056,
		      0.324933,     -0.266474,     -0.306852,      0.392191,      0.284712,
		     -0.278263,     -0.179828,      0.013795,      0.051519,      0.022401,
		     -0.051074,     -0.093058,      0.159828,      0.170575,     -0.222780,
		     -0.240533,      0.176403,      0.376494,     -0.030912,     -0.277849,
		     -0.267738,     -0.075943,      0.161437,      0.281126,      0.091335,
		     -0.070490,     -0.146411,     -0.094506,      0.157644,      0.051431,
		     -0.290334,     -0.310842,     -0.194923,     -0.110846,     -0.136662,
		      0.131052,      0.260310,      0.166591,     -0.062374,     -0.038401,
		      0.012953,     -0.100632,     -0.161970,     -0.008608,      0.219898,
		      0.241630,      0.042997,     -0.005035,     -0.049528,      0.014145,
		      0.083500,      0.134820,      0.176498,      0.102401,      0.131601,
		      0.119693,      0.033109,     -0.066732,     -0.097141,     -0.032345,
		     -0.028590,     -0.015377,     -0.009572,      0.128759,      0.096216,
		      0.102525,      0.105690,      0.051285,     -0.058559,      0.019749,
		      0.019638,     -0.048139,     -0.061402,     -0.015841,     -0.032131,
		      0.061483,      0.062523,      0.014607,     -0.030599,     -0.077367,
		     -0.178385,     -0.112317,      0.008921,      0.040314,      0.001074,
		     -0.023060,     -0.034548,     -0.058840,     -0.065868,     -0.111970,
		      0.049348,      0.037412,      0.114185,      0.026859,      0.073130,
		      0.031943,     -0.128186,     -0.063316,      0.015139,      0.000000,
		     -0.089643,      0.134494,      0.083777,     -0.191391,      0.057424,
		      0.110777,     -0.112959,     -0.082140,      0.110506,      0.043182,
		     -0.151805,      0.026901,      0.283394,     -0.098678,     -0.274746,
		      0.141935,      0.191292,     -0.101524,     -0.107676,     -0.008413,
		      0.037686,      0.124636,     -0.062666,     -0.170864,      0.074622,
		      0.167862,      0.032669,     -0.156691,     -0.085014,      0.088614,
		      0.203748,      0.098805,     -0.012286,     -0.147439,     -0.128667,
		     -0.046861,      0.147598,      0.088807,     -0.088237,     -0.124505,
		      0.125688,      0.260511,      0.163723,     -0.017500,      0.087090,
		      0.014546,     -0.135270,     -0.165486,     -0.076186,     -0.019305,
		      0.042758,      0.057203,      0.144320,      0.112976,     -0.071137,
		     -0.141640,     -0.003762,     -0.024653,      0.004137,     -0.008373,
		     -0.010397,     -0.004828,     -0.063425,     -0.077886,     -0.141471,
		     -0.066294,     -0.063126,     -0.023186,      0.108083,      0.101988,
		      0.016845,     -0.060372,     -0.042655,      0.008744,     -0.094449,
		     -0.004284,     -0.023726,      0.010857,     -0.017267,     -0.030299,
		     -0.080820,     -0.065077,     -0.044583,     -0.017021,      0.003928,
		      0.022160,     -0.016482,     -0.001676,      0.003760,      0.137749,
		      0.145208,      0.070523,     -0.037595,     -0.045048,     -0.056158,
		     -0.004689,      0.008863,      0.013166,      0.055510,      0.074071,
		     -0.052473,     -0.035535,     -0.024297,      0.016705,     -0.055054,
		     -0.024942,      0.032999,      0.027964,     -0.017394,     -0.045103,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : mars ---  Model : langlais2019 */
coeffStruct& _model_coeff_langlais2019() {
	static const std::string name = "langlais2019";
	static const std::string body = "mars";
	static const int len = 9179;
	static const int nmax = 134;
	static const int ndef = 134;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,  14,  14,  14,  14,  14,  14,  14,  14,
		  14,  14,  14,  14,  14,  14,  14,  15,  15,  15,  15,  15,  15,  15,  15,  15,
		  15,  15,  15,  15,  15,  15,  15,  16,  16,  16,  16,  16,  16,  16,  16,  16,
		  16,  16,  16,  16,  16,  16,  16,  16,  17,  17,  17,  17,  17,  17,  17,  17,
		  17,  17,  17,  17,  17,  17,  17,  17,  17,  17,  18,  18,  18,  18,  18,  18,
		  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  19,  19,  19,
		  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,
		  19,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,
		  20,  20,  20,  20,  20,  20,  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,
		  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,  22,  22,  22,  22,
		  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,
		  22,  22,  22,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,
		  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  24,  24,  24,  24,  24,
		  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,
		  24,  24,  24,  24,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,
		  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  26,  26,
		  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,
		  26,  26,  26,  26,  26,  26,  26,  26,  26,  27,  27,  27,  27,  27,  27,  27,
		  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,
		  27,  27,  27,  27,  27,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,
		  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,
		  28,  28,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,
		  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,
		  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,
		  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  31,
		  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,
		  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  31,  32,
		  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,
		  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,  32,
		  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,
		  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,  33,
		  33,  33,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,
		  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,  34,
		  34,  34,  34,  34,  34,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,
		  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,  35,
		  35,  35,  35,  35,  35,  35,  35,  35,  35,  36,  36,  36,  36,  36,  36,  36,
		  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,
		  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  36,  37,  37,
		  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,
		  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,  37,
		  37,  37,  37,  37,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,
		  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,
		  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  38,  39,  39,  39,  39,  39,
		  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,
		  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,  39,
		  39,  39,  39,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,
		  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,
		  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  40,  41,  41,  41,  41,
		  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,
		  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,  41,
		  41,  41,  41,  41,  41,  41,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,
		  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,
		  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,  42,
		  42,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,
		  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,
		  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  43,  44,  44,  44,
		  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,
		  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,
		  44,  44,  44,  44,  44,  44,  44,  44,  44,  44,  45,  45,  45,  45,  45,  45,
		  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,
		  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,  45,
		  45,  45,  45,  45,  45,  45,  45,  45,  46,  46,  46,  46,  46,  46,  46,  46,
		  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,
		  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,  46,
		  46,  46,  46,  46,  46,  46,  46,  47,  47,  47,  47,  47,  47,  47,  47,  47,
		  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,
		  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,  47,
		  47,  47,  47,  47,  47,  47,  47,  48,  48,  48,  48,  48,  48,  48,  48,  48,
		  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,
		  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,  48,
		  48,  48,  48,  48,  48,  48,  48,  48,  49,  49,  49,  49,  49,  49,  49,  49,
		  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,
		  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,
		  49,  49,  49,  49,  49,  49,  49,  49,  49,  49,  50,  50,  50,  50,  50,  50,
		  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,
		  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,
		  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  50,  51,  51,  51,
		  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,
		  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,
		  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,  51,
		  51,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,
		  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,
		  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,  52,
		  52,  52,  52,  52,  52,  52,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,
		  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,
		  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,
		  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  53,  54,  54,  54,  54,
		  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,
		  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,
		  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,  54,
		  54,  54,  54,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,
		  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,
		  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,
		  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  55,  56,  56,  56,  56,  56,
		  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,
		  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,
		  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,  56,
		  56,  56,  56,  56,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,
		  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,
		  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,
		  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  57,  58,  58,
		  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,
		  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,
		  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,  58,
		  58,  58,  58,  58,  58,  58,  58,  58,  58,  59,  59,  59,  59,  59,  59,  59,
		  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,
		  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,
		  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,  59,
		  59,  59,  59,  59,  59,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,
		  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,
		  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,
		  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,  60,
		  60,  60,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,
		  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,
		  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,
		  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,  61,
		  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,
		  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,
		  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,
		  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  62,  63,
		  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,
		  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,
		  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,
		  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  63,  64,
		  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,
		  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,
		  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,
		  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,  64,
		  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,
		  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,
		  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,
		  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,  65,
		  65,  65,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,
		  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,
		  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,
		  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,  66,
		  66,  66,  66,  66,  66,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,
		  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,
		  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,
		  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,  67,
		  67,  67,  67,  67,  67,  67,  67,  67,  67,  68,  68,  68,  68,  68,  68,  68,
		  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,
		  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,
		  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,
		  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  68,  69,  69,
		  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,
		  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,
		  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,
		  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,  69,
		  69,  69,  69,  69,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,
		  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,
		  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,
		  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,
		  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  70,  71,  71,  71,  71,  71,
		  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,
		  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,
		  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,
		  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,  71,
		  71,  71,  71,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,
		  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,
		  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,
		  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,
		  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  72,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,  73,
		  73,  73,  73,  73,  73,  73,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,  74,
		  74,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,
		  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,
		  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,
		  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,
		  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  75,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,
		  76,  76,  76,  76,  76,  76,  76,  76,  76,  76,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,  77,
		  77,  77,  77,  77,  77,  77,  77,  77,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,  78,
		  78,  78,  78,  78,  78,  78,  78,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,  79,
		  79,  79,  79,  79,  79,  79,  79,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,  80,
		  80,  80,  80,  80,  80,  80,  80,  80,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,
		  81,  81,  81,  81,  81,  81,  81,  81,  81,  81,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,
		  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  82,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,  83,
		  83,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,  84,
		  84,  84,  84,  84,  84,  84,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,
		  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  85,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,  86,
		  86,  86,  86,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,
		  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  87,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,  88,
		  88,  88,  88,  88,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,
		  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  89,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,  90,
		  90,  90,  90,  90,  90,  90,  90,  90,  90,  91,  91,  91,  91,  91,  91,  91,
		  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,
		  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,
		  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,
		  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,
		  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,  91,
		  91,  91,  91,  91,  91,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,
		  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,
		  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,
		  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,
		  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,
		  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,  92,
		  92,  92,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,
		  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,
		  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,
		  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,
		  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,
		  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,  93,
		  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,
		  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,
		  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,
		  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,
		  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,
		  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  94,  95,
		  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,
		  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,
		  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,
		  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,
		  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,
		  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  95,  96,
		  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,
		  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,
		  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,
		  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,
		  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,
		  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,  96,
		  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,
		  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,
		  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,
		  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,
		  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,
		  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,  97,
		  97,  97,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,
		  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,
		  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,
		  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,
		  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,
		  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,  98,
		  98,  98,  98,  98,  98,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,
		  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,
		  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,
		  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,
		  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,
		  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,  99,
		  99,  99,  99,  99,  99,  99,  99,  99,  99, 100, 100, 100, 100, 100, 100, 100,
		 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
		 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
		 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
		 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
		 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100,
		 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 101, 101,
		 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
		 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
		 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
		 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
		 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
		 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101, 101,
		 101, 101, 101, 101, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102,
		 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102,
		 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102,
		 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102,
		 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102,
		 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102,
		 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 102, 103, 103, 103, 103, 103,
		 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103,
		 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103,
		 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103,
		 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103,
		 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103,
		 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103, 103,
		 103, 103, 103, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104,
		 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104,
		 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104,
		 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104,
		 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104,
		 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104,
		 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105,
		 105, 105, 105, 105, 105, 105, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106,
		 106, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
		 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
		 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
		 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
		 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
		 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107,
		 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108,
		 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109,
		 109, 109, 109, 109, 109, 109, 109, 109, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110,
		 110, 110, 110, 110, 110, 110, 110, 111, 111, 111, 111, 111, 111, 111, 111, 111,
		 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111,
		 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111,
		 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111,
		 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111,
		 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111,
		 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111,
		 111, 111, 111, 111, 111, 111, 111, 112, 112, 112, 112, 112, 112, 112, 112, 112,
		 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112,
		 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112,
		 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112,
		 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112,
		 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112,
		 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 112,
		 112, 112, 112, 112, 112, 112, 112, 112, 113, 113, 113, 113, 113, 113, 113, 113,
		 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113,
		 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113,
		 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113,
		 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113,
		 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113,
		 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 113,
		 113, 113, 113, 113, 113, 113, 113, 113, 113, 113, 114, 114, 114, 114, 114, 114,
		 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114,
		 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114,
		 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114,
		 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114,
		 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114,
		 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114,
		 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 114, 115, 115, 115,
		 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115,
		 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115,
		 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115,
		 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115,
		 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115,
		 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115,
		 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115, 115,
		 115, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116,
		 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116,
		 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116,
		 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116,
		 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116,
		 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116,
		 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116, 116,
		 116, 116, 116, 116, 116, 116, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117,
		 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117,
		 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117,
		 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117,
		 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117,
		 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117,
		 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117,
		 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 117, 118, 118, 118, 118,
		 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118,
		 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118,
		 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118,
		 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118,
		 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118,
		 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118,
		 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118, 118,
		 118, 118, 118, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119,
		 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119,
		 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119,
		 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119,
		 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119,
		 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119,
		 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119,
		 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 119, 120, 120, 120, 120, 120,
		 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120,
		 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120,
		 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120,
		 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120,
		 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120,
		 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120,
		 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120, 120,
		 120, 120, 120, 120, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121,
		 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121,
		 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121,
		 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121,
		 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121,
		 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121,
		 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121,
		 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 121, 122, 122,
		 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122,
		 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122,
		 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122,
		 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122,
		 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122,
		 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122,
		 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122, 122,
		 122, 122, 122, 122, 122, 122, 122, 122, 122, 123, 123, 123, 123, 123, 123, 123,
		 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123,
		 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123,
		 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123,
		 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123,
		 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123,
		 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123,
		 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123, 123,
		 123, 123, 123, 123, 123, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124,
		 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124,
		 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124,
		 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124,
		 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124,
		 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124,
		 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124,
		 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124, 124,
		 124, 124, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125,
		 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125,
		 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125,
		 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125,
		 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125,
		 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125,
		 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125,
		 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125, 125,
		 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126,
		 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126,
		 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126,
		 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126,
		 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126,
		 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126,
		 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126,
		 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 126, 127,
		 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127,
		 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127,
		 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127,
		 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127,
		 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127,
		 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127,
		 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127,
		 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 127, 128,
		 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128,
		 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128,
		 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128,
		 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128,
		 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128,
		 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128,
		 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128,
		 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128,
		 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129,
		 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129,
		 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129,
		 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129,
		 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129,
		 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129,
		 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129,
		 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129, 129,
		 129, 129, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130,
		 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130,
		 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130,
		 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130,
		 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130,
		 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130,
		 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130,
		 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130, 130,
		 130, 130, 130, 130, 130, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131,
		 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131,
		 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131,
		 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131,
		 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131,
		 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131,
		 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131,
		 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131, 131,
		 131, 131, 131, 131, 131, 131, 131, 131, 131, 132, 132, 132, 132, 132, 132, 132,
		 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132,
		 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132,
		 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132,
		 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132,
		 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132,
		 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132,
		 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132,
		 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 132, 133, 133,
		 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133,
		 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133,
		 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133,
		 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133,
		 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133,
		 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133,
		 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133,
		 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133, 133,
		 133, 133, 133, 133, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134,
		 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134,
		 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134,
		 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134,
		 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134,
		 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134,
		 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134,
		 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134,
		 134, 134, 134, 134, 134, 134, 134, 134, 134, 134, 134,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
		  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
		  41,  42,  43,  44,  45,  46,  47,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
		  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
		  41,  42,  43,  44,  45,  46,  47,  48,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,
		  51,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,
		  47,  48,  49,  50,  51,  52,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,
		  52,  53,  54,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,
		  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,
		  53,  54,  55,  56,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,
		  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,
		  50,  51,  52,  53,  54,  55,  56,  57,  58,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,
		  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
		  55,  56,  57,  58,  59,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,
		  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,
		  59,  60,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,
		  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
		  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,
		  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,
		  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
		  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,
		  64,  65,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,
		  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,
		  62,  63,  64,  65,  66,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,
		  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,
		  59,  60,  61,  62,  63,  64,  65,  66,  67,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,
		  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
		  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,
		  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,
		  66,  67,  68,  69,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,
		  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,
		  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,
		  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,
		  69,  70,  71,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,
		  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,
		  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,
		  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,
		  68,  69,  70,  71,  72,  73,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,
		  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,
		  74,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,
		  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,
		  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,
		  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,
		  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,
		  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,
		  70,  71,  72,  73,  74,  75,  76,  77,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,
		  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,
		  72,  73,  74,  75,  76,  77,  78,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
		  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
		  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,
		  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,
		  73,  74,  75,  76,  77,  78,  79,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
		  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
		  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,
		  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,
		  73,  74,  75,  76,  77,  78,  79,  80,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,
		  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,
		  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,
		  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,
		  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,
		  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,
		  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,
		  83,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,
		  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,
		  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,
		  79,  80,  81,  82,  83,  84,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,
		  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,
		  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,
		  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,
		  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,
		  84,  85,  86,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,
		  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,
		  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,
		  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,
		  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,
		  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,
		  85,  86,  87,  88,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,
		  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,
		  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,
		  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,
		  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,
		  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,
		  82,  83,  84,  85,  86,  87,  88,  89,  90,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,
		  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
		  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,
		  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,
		  87,  88,  89,  90,  91,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,
		  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,
		  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,
		  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,
		  91,  92,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,
		  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,
		  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,
		  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
		  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,
		  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,
		  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,
		  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,
		  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,
		  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,
		  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,
		  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,
		  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
		  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,
		  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,
		  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,
		  96,  97,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,
		  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,
		  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,
		  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,
		  94,  95,  96,  97,  98,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,
		  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,
		  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,
		  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,
		  91,  92,  93,  94,  95,  96,  97,  98,  99,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,
		  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
		  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,
		  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,
		  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,
		  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,
		  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,
		  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,
		  98,  99, 100, 101,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,
		  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,
		  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,
		  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,
		  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,
		  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,
		  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,
		  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100,
		 101, 102, 103,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,
		  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,
		  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,
		  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,
		  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103, 104,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,
		  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,
		  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,
		  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99,
		 100, 101, 102, 103, 104, 105,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,
		  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,
		  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,
		  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103, 104, 105,
		 106,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,
		  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,
		  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,
		  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,
		  95,  96,  97,  98,  99, 100, 101, 102, 103, 104, 105, 106, 107,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,
		  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,
		  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,
		  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,
		  99, 100, 101, 102, 103, 104, 105, 106, 107, 108,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,
		  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,
		  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,
		  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100, 101,
		 102, 103, 104, 105, 106, 107, 108, 109,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,
		  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,
		  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,
		  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103,
		 104, 105, 106, 107, 108, 109, 110,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
		  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
		  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,
		  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,
		  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,
		  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103, 104,
		 105, 106, 107, 108, 109, 110, 111,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,
		  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,
		  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,
		  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,
		  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,
		  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103, 104,
		 105, 106, 107, 108, 109, 110, 111, 112,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,
		  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,
		  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,
		  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,
		  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,
		  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103,
		 104, 105, 106, 107, 108, 109, 110, 111, 112, 113,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,
		  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,
		  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,
		  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,
		  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,
		  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100, 101,
		 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,
		  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,
		  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,
		  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,
		  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,
		  99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114,
		 115,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
		  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,
		  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,
		  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,
		  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,
		  95,  96,  97,  98,  99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110,
		 111, 112, 113, 114, 115, 116,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,
		  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,
		  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,
		  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,
		  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,
		  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103, 104, 105,
		 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,
		  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,
		  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,
		  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,
		  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99,
		 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115,
		 116, 117, 118,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,
		  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,
		  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,
		  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,
		  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,
		  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103, 104, 105, 106, 107, 108,
		 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,
		  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,
		  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,
		  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,
		  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100,
		 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116,
		 117, 118, 119, 120,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,
		  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,
		  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,
		  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,
		  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103, 104, 105, 106, 107,
		 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,
		  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,
		  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,
		  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,
		  98,  99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113,
		 114, 115, 116, 117, 118, 119, 120, 121, 122,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,
		  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
		  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,
		  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,
		  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102,
		 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118,
		 119, 120, 121, 122, 123,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,
		  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,
		  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,
		  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,
		  91,  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103, 104, 105, 106,
		 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122,
		 123, 124,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,
		  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,
		  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,
		  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,
		  94,  95,  96,  97,  98,  99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109,
		 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
		  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,
		  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,
		  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,
		  96,  97,  98,  99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111,
		 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,
		  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,
		  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,
		  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,
		  97,  98,  99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112,
		 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127,   0,
		   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,
		  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,
		  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,
		  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,
		  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,
		  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,
		  97,  98,  99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112,
		 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
		  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
		  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,
		  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,
		  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,
		  96,  97,  98,  99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111,
		 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127,
		 128, 129,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,
		  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,
		  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,
		  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,
		  94,  95,  96,  97,  98,  99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109,
		 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125,
		 126, 127, 128, 129, 130,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,
		  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,
		  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,
		  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,
		  91,  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103, 104, 105, 106,
		 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122,
		 123, 124, 125, 126, 127, 128, 129, 130, 131,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,
		  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,
		  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,
		  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,
		  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102,
		 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118,
		 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 131, 132,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,  32,  33,
		  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,  48,  49,
		  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,  64,  65,
		  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,
		  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,  96,  97,
		  98,  99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 113,
		 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129,
		 130, 131, 132, 133,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,
		  28,  29,  30,  31,  32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,
		  44,  45,  46,  47,  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,
		  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,
		  76,  77,  78,  79,  80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,
		  92,  93,  94,  95,  96,  97,  98,  99, 100, 101, 102, 103, 104, 105, 106, 107,
		 108, 109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123,
		 124, 125, 126, 127, 128, 129, 130, 131, 132, 133, 134,
	};
	static const std::vector<double> g = {
		     -1.515500,     -0.605640,     -0.153740,      0.709210,      1.199160,
		      0.839580,     -0.829350,     -0.995860,     -0.938990,     -1.027990,
		     -0.924970,      0.084240,      2.221360,      0.475290,      0.642180,
		      0.320970,      0.447520,     -0.363570,      0.413670,      0.555340,
		     -1.062270,     -0.220400,     -0.092080,     -1.764090,      0.457980,
		      0.357560,     -0.961390,      0.965380,      1.234460,     -0.749650,
		      0.461430,      0.158130,      0.554210,      1.073480,      1.417760,
		     -0.764420,     -3.258320,      0.781120,      1.362150,     -0.804950,
		     -0.980590,     -1.047490,     -2.064980,     -0.865780,     -0.944140,
		      3.156450,      1.542920,     -1.664510,     -0.307950,      0.685550,
		      0.577880,      0.504220,      0.557300,      1.595230,      2.593500,
		     -0.498870,     -4.008540,     -0.658870,      1.660640,      0.705430,
		     -0.545570,     -0.079710,     -0.843070,     -1.725130,     -0.737390,
		     -2.080060,     -2.143150,      2.971900,      2.353700,     -1.089720,
		     -2.023420,     -0.436270,      0.182720,      0.506670,      0.382700,
		      0.852800,      0.907390,      0.931890,      3.181060,     -0.117220,
		     -2.135300,     -1.712760,      0.666390,      1.145900,     -0.042270,
		     -0.025090,      0.902920,     -1.393460,     -2.318810,     -3.137540,
		      0.240930,     -2.853580,     -2.160690,      2.169290,      3.672680,
		      1.241340,     -0.355450,     -0.574650,      0.101190,      0.108290,
		      0.975160,      2.258090,      2.126740,     -0.135230,     -2.355920,
		      0.635030,      2.828270,     -0.979030,     -2.212680,     -1.944360,
		     -0.156790,      0.861940,      0.439520,      0.324520,      1.069260,
		     -1.275080,     -1.802530,     -2.435560,     -1.411170,      4.057250,
		      0.512040,     -2.947200,     -1.709470,      0.506070,      2.101930,
		      0.157090,     -0.248330,     -0.979530,     -1.215710,     -1.156940,
		     -0.327460,      0.788370,      2.586330,      1.833780,      0.658480,
		     -2.449420,     -3.279230,      2.057040,      3.891500,     -0.001780,
		     -1.543160,     -0.848310,     -0.691520,      0.503870,      0.487710,
		      0.956930,      1.065820,      0.406070,      0.302730,     -2.323410,
		     -0.985860,     -0.029050,     -1.368710,      5.704600,      0.419080,
		     -4.169250,     -1.024380,      1.670990,      2.561210,     -0.373400,
		     -0.304840,     -0.355880,     -0.336020,     -0.929030,     -0.299560,
		     -0.613720,      2.198430,      1.292360,      1.353690,      1.185150,
		      3.563740,     -5.722050,     -4.132090,      1.729750,      2.826390,
		     -0.071920,     -1.760580,      1.176280,     -0.235580,      0.537060,
		      0.910500,      1.566880,      1.863110,      0.797110,     -0.322670,
		     -0.626230,     -0.254770,      0.139430,     -0.981340,     -2.434790,
		      0.340940,      6.175640,      0.987800,     -2.975110,     -2.116230,
		      0.541400,      1.626120,      0.259740,     -0.694260,     -1.431410,
		     -2.906800,     -2.798700,     -0.605210,     -0.039900,      1.039650,
		      0.183740,      0.123250,      1.307280,     -1.796520,      3.859120,
		      3.541260,     -5.451780,     -3.549190,      2.079640,      1.943090,
		     -0.432150,     -1.742780,      1.397270,      0.552320,      1.104770,
		      2.857470,      2.317210,      1.548120,      0.423150,     -0.013960,
		      0.430120,      0.537490,      0.723540,      0.926760,     -0.532740,
		     -2.910270,     -3.433770,      2.199160,      4.428640,      0.519320,
		     -0.158990,      0.142840,      0.797140,     -0.874970,     -0.141480,
		      1.138450,     -0.232160,     -1.386790,     -2.847570,     -1.170740,
		     -0.713980,      0.374810,      0.793610,     -0.585150,     -0.955510,
		     -1.079090,     -1.312930,      0.768460,      3.673510,      1.238840,
		     -3.578140,     -2.240200,      0.208840,      0.628940,     -0.491890,
		     -0.769500,      1.025940,     -1.747580,     -1.443260,      1.156620,
		      1.732690,      0.887630,     -0.672390,     -2.071350,     -0.568190,
		      0.492720,      0.723890,      1.063110,      0.254230,      1.313390,
		      1.827370,     -4.833250,     -2.834670,      2.952430,      1.936580,
		     -1.100800,     -0.569040,      0.368260,      1.650850,     -1.317330,
		     -1.393170,      1.228200,      0.093770,     -1.286320,     -1.803740,
		      1.518520,      1.654240,      0.969640,      1.067480,     -0.881740,
		      0.217150,     -1.417020,     -1.074140,      0.957380,     -1.524940,
		      3.065410,      1.804580,     -1.419660,     -1.094600,     -0.049090,
		     -0.363670,     -0.984390,     -0.842060,      0.936180,      2.757960,
		     -1.163080,     -1.603050,      1.115220,      2.187140,     -0.234130,
		     -1.255780,     -2.081890,     -1.690040,     -1.160960,     -0.391590,
		      0.465220,      1.703460,      1.645670,      1.859260,      1.259290,
		      0.301230,     -1.341280,     -0.677980,      2.806810,      1.398710,
		     -0.149890,      0.145070,     -0.494080,      0.179540,     -0.414120,
		     -0.126920,      0.068370,     -0.830660,     -2.237560,     -2.366460,
		      1.356270,      2.560430,      0.869700,      1.228770,     -1.302920,
		     -0.916070,     -0.263060,     -2.283720,     -0.234420,     -0.212080,
		     -0.657020,     -0.322410,      2.231850,      1.012590,     -2.796590,
		     -0.574940,      0.180220,     -0.165280,     -0.550290,     -2.381830,
		     -1.673650,      2.414350,      1.079850,      0.046370,      1.083440,
		      3.620480,      0.043860,     -0.825670,      0.268270,     -1.130200,
		     -0.456160,     -0.000340,      0.768860,      1.656090,      1.508500,
		      2.511490,      0.041620,      0.737860,     -0.088550,     -2.157110,
		      0.246470,      0.226250,      0.693030,      0.493920,      0.878450,
		      1.759560,      0.430720,     -0.676730,     -1.885450,      1.283210,
		      1.263930,     -0.283540,     -1.983090,     -2.354880,     -1.950590,
		     -0.580860,     -0.709040,      0.745320,     -0.050530,     -0.376710,
		     -0.022380,     -2.840550,     -2.189940,     -0.216860,      0.276560,
		     -0.982370,      0.602650,      1.138210,      1.295690,      1.231380,
		     -1.324030,      0.453510,      0.292740,     -1.046090,      1.611170,
		      1.733080,     -1.579930,     -1.694310,     -0.990980,     -0.898320,
		      1.634110,      3.168180,      1.046490,      0.194700,      2.291160,
		      1.182020,      1.108060,      1.786640,      0.743490,      1.581620,
		      1.822480,      0.947970,     -1.356550,     -0.536330,      0.062270,
		     -1.129840,     -0.603160,     -1.570820,      1.236640,      1.037120,
		     -2.061140,     -1.910640,     -1.021270,     -0.190670,      0.528610,
		      0.554820,     -1.550650,      0.725030,      0.670100,     -1.231080,
		     -2.190420,     -0.051560,     -0.144560,     -1.668550,     -2.206660,
		     -1.801730,      0.219490,      0.205150,      1.791930,     -1.496740,
		     -1.373010,      1.014380,     -0.558050,     -0.453420,     -0.629340,
		      0.280500,      0.061590,     -0.136570,     -1.098200,     -0.018920,
		     -0.150600,      1.162290,      1.925670,     -1.680100,     -1.984470,
		      0.517040,      1.637030,     -0.523200,      0.085420,      0.409910,
		      1.634040,      1.119050,     -0.011350,      0.345980,      3.549070,
		      0.956760,      0.736270,     -0.020000,     -1.345690,      0.865040,
		      0.294900,      0.439780,     -0.483700,     -0.750940,      1.739510,
		      2.015470,      0.392130,     -0.912180,      0.435480,     -0.832680,
		      1.777880,      3.375420,     -0.494690,     -1.546480,      0.162630,
		     -0.325130,      0.192600,      1.042830,     -0.303400,     -0.459090,
		      0.473040,     -2.699320,     -2.571890,     -0.696800,     -0.359510,
		     -1.923840,     -0.319750,     -1.813230,     -0.451550,     -1.116710,
		     -0.002880,      1.293690,      1.081650,      0.539630,      1.620410,
		     -0.440520,     -1.253690,     -0.921110,      0.187200,      0.555440,
		      0.254950,      1.598900,     -1.278950,     -2.060590,     -0.453390,
		     -0.039000,      2.052030,      1.270510,     -1.429850,     -1.591130,
		     -0.848720,     -1.035870,     -1.816740,      1.803610,      4.514470,
		      1.607640,      0.490010,      0.270360,      3.064790,      0.875290,
		      0.231270,      1.692460,     -0.480070,     -1.210090,     -0.975290,
		     -0.108980,      0.585800,     -1.256020,      1.707380,      0.867220,
		      0.107450,     -0.750390,     -0.872530,     -0.598220,      0.842290,
		     -1.109720,      0.588170,      2.947690,     -0.249940,     -1.525070,
		     -0.587810,     -0.828660,     -0.132280,      1.764420,      2.155820,
		      2.034590,      1.828300,     -2.892360,     -3.511690,     -0.795870,
		      1.416160,     -2.209090,      0.365760,     -2.169090,     -0.925750,
		     -0.461880,     -0.677060,      1.110560,     -0.645300,      1.258020,
		      2.532640,     -0.173550,     -0.278350,     -0.887090,     -0.857900,
		      0.210660,      2.520180,     -0.707170,     -2.794950,      2.056760,
		     -0.245310,     -1.037590,      0.835460,     -0.736750,     -0.612220,
		      2.207170,      1.405690,     -1.130260,     -1.107880,     -1.925860,
		     -5.311250,     -0.905760,      4.101610,      2.366930,     -0.108540,
		     -0.674860,      0.308100,      1.316490,     -1.389200,      0.248910,
		      1.082980,      0.032960,      0.114250,     -0.829860,     -1.119440,
		      0.981380,      0.063960,     -1.202490,     -2.299980,      3.185450,
		      0.849230,     -3.513310,      0.128540,      1.145940,      1.341430,
		      1.020560,      0.346060,      1.211510,     -0.187300,      0.650500,
		      0.815170,     -1.192150,     -1.982070,      0.240460,      1.672800,
		      3.736280,      3.627310,     -1.429970,     -2.963740,     -2.091450,
		      1.695350,      0.422820,      1.903940,      0.855470,     -0.649400,
		      1.815950,     -0.612780,     -0.636950,      0.119070,     -0.116120,
		      0.911740,      1.775820,      1.816840,      0.535540,     -1.780800,
		     -2.385430,     -3.196630,      1.056230,      3.882250,      1.385050,
		     -3.274900,      0.598490,     -1.376250,     -1.350290,      2.147100,
		      0.080390,     -4.993170,     -1.753720,      2.281590,      0.991620,
		     -0.193400,      1.022770,     -3.773270,     -2.530720,      2.101930,
		      2.925460,      0.707820,     -0.466270,     -2.005540,     -0.113230,
		     -1.220900,     -0.724970,      0.122280,     -0.237360,      1.082040,
		      1.798650,     -0.450250,     -1.314270,     -1.476290,      0.784170,
		     -1.830150,     -3.505060,      0.095720,      4.202280,     -0.227930,
		     -5.316750,     -1.588940,      0.609360,     -1.062340,      2.322670,
		      1.568810,     -0.564100,     -1.373560,      2.612010,      4.398030,
		      0.810680,     -2.852280,     -1.240090,     -1.045280,      2.872190,
		      4.220580,      0.823910,     -1.759200,     -3.124170,     -1.285290,
		     -0.054230,      0.590620,      2.163790,     -0.449400,      1.182260,
		     -1.212170,     -1.607710,     -0.725210,      0.119290,      0.470580,
		     -0.836580,     -1.173120,      2.249530,      2.452240,     -0.414960,
		      2.565130,     -2.916010,     -0.689530,      0.933610,      1.203950,
		      2.330900,      0.211920,     -0.620040,     -1.984490,     -0.569970,
		      3.938900,      2.585570,     -5.287310,     -4.262120,      0.783110,
		     -0.219390,     -1.094750,     -1.204080,     -3.697410,     -2.935540,
		      0.571380,      3.240180,      2.431260,      1.210020,     -0.508120,
		     -0.788940,     -0.178610,     -0.637460,     -0.806400,     -0.292250,
		      0.040240,      2.229210,     -0.549750,      0.554550,      1.005170,
		     -1.681710,     -0.868080,     -0.123030,      1.307690,     -1.891320,
		      2.659680,      2.579810,      0.547550,     -0.525000,     -0.936540,
		      0.050850,     -1.529270,      2.457170,      0.878810,     -2.532420,
		     -3.452130,      1.782350,      5.038810,      3.173830,     -0.263090,
		      0.965300,     -0.144130,      2.243330,      2.817500,      0.710820,
		     -0.540110,     -2.270890,     -2.246250,     -0.779720,     -1.805610,
		      1.540640,      0.141740,      1.681830,     -1.295370,     -3.113910,
		     -2.930180,      0.165030,     -0.377910,     -0.634540,     -0.815350,
		     -0.749220,     -0.189560,     -0.579790,     -0.316980,      0.708260,
		     -1.560480,     -2.996410,      0.717510,     -0.130820,     -2.123950,
		      0.748140,      1.738460,     -1.510300,     -2.196870,     -0.860520,
		      3.666290,      3.161540,     -4.026320,     -4.030280,     -0.323420,
		     -0.335650,      0.202110,     -0.244420,     -2.241970,     -1.970790,
		     -0.735550,      0.831290,      1.128740,      0.628620,      0.981070,
		     -0.346120,     -0.461880,     -1.531370,     -0.827620,      2.100920,
		      1.499830,      0.862020,     -2.171580,      0.989530,      1.609110,
		      2.267470,      1.598430,      0.412190,      1.982210,      3.065870,
		     -1.290570,      0.021380,      2.007780,     -1.477010,     -0.723510,
		      2.131030,     -1.067600,     -0.372230,      0.625690,      3.171380,
		      2.216380,     -3.010520,     -3.782680,      1.838980,      3.462510,
		      2.480220,     -0.238320,      0.209860,     -0.756790,      2.349190,
		      2.031440,     -0.138830,      0.102960,     -1.457960,     -0.997760,
		     -0.424030,     -1.289250,      0.495590,      0.885950,      1.328400,
		      0.539060,     -0.416130,     -1.983130,     -0.000320,     -1.595310,
		     -0.225430,     -0.547490,     -1.436970,     -0.536400,     -1.043660,
		     -0.188950,      0.299150,      4.061200,      0.830110,     -2.541640,
		      1.075160,      3.422000,     -0.646710,     -1.624980,     -0.623630,
		      0.934960,     -1.965540,     -1.210190,      0.241260,      2.428690,
		      1.416460,     -2.171780,     -1.768600,     -1.379150,     -1.496020,
		      0.060880,     -1.125250,     -2.295690,     -0.332990,      1.599940,
		      2.073420,      0.204450,     -0.798100,     -0.227270,      0.183620,
		      0.837710,     -1.343470,     -2.277090,      1.361480,      1.782780,
		      1.106870,      0.472780,     -1.854930,      0.593090,      1.354340,
		      2.952490,      3.579220,      1.387570,      1.077290,     -0.321660,
		     -2.356810,     -2.118120,      1.943380,      1.176790,     -3.924720,
		     -0.726360,      2.598690,     -1.563210,     -2.399580,     -0.149300,
		      1.850750,      2.530020,     -0.717240,     -3.490960,      1.329380,
		      1.394760,      2.391990,      1.675880,      0.318690,     -1.717240,
		      2.189760,      1.819770,     -2.458310,     -1.658260,     -0.880950,
		      0.074420,      1.255220,     -0.580920,     -0.910640,     -0.447730,
		      0.735310,      1.301740,      1.611310,     -0.889310,     -0.233120,
		      0.676320,      1.657040,      0.763190,     -1.990550,     -2.242210,
		     -1.731560,      0.648230,      0.561440,      0.392460,     -0.691940,
		      3.941660,     -1.250370,     -4.941910,      2.710750,      3.052620,
		     -1.408050,      0.895070,      0.947290,      0.784010,     -2.344760,
		     -1.640200,      0.425830,      2.869120,     -0.580420,     -0.127760,
		     -0.009340,     -0.810930,     -1.783030,      0.578350,     -1.105100,
		     -3.054220,     -1.215080,      0.907570,      3.403070,      0.950970,
		     -1.565940,     -1.072480,      0.318200,      1.210130,     -0.795330,
		     -3.065790,     -0.345350,      0.790340,      0.216750,      0.948600,
		     -2.293920,     -1.774160,     -1.224370,      1.466310,      1.725140,
		      1.030750,      0.242110,     -0.222980,     -1.244840,      2.542640,
		     -1.885430,     -0.634020,      3.439290,      0.119770,     -3.949060,
		      0.283020,      2.589060,     -0.075580,     -1.396630,      0.204030,
		      0.022950,      1.082580,      0.861060,     -0.724670,      1.034530,
		     -1.170120,      0.364330,      3.051800,      1.807170,     -1.321020,
		      1.231510,      2.347040,     -1.639750,     -2.531420,     -1.631260,
		      0.713430,      1.733920,      1.646370,      0.536410,     -0.086480,
		      0.102790,     -0.656450,      0.031090,     -0.425160,     -1.482930,
		      1.105000,      1.651350,      2.887010,     -1.220680,     -1.112990,
		     -0.220250,     -1.691650,      0.311670,      0.180160,     -2.538780,
		     -0.620770,     -1.374710,      3.268650,     -0.669290,     -4.864680,
		      3.268050,      1.902620,     -4.024710,     -0.739680,      1.033560,
		      1.733810,     -1.106060,     -1.719450,     -1.105040,      1.816020,
		     -1.942530,      0.321690,      1.206300,     -1.398930,     -1.586900,
		      1.013910,      0.906020,     -0.998770,     -1.133980,     -0.334850,
		      2.050630,      2.046800,     -1.454380,     -2.194740,     -1.250300,
		      0.170380,      0.745070,      0.007400,      0.039410,      0.546930,
		      0.428840,     -0.166840,     -1.459590,     -3.359690,     -1.867670,
		     -0.989150,      0.684270,      0.538580,      0.442180,     -0.749090,
		      0.574430,      0.624340,      0.619910,      2.124350,     -4.291400,
		     -0.467040,      4.411880,     -0.471200,     -3.127840,      1.049320,
		      1.614440,      0.284880,     -2.468550,      0.131340,      0.204010,
		      0.054500,      0.896760,      1.943870,      2.454580,     -0.912730,
		     -1.585780,      1.260620,      0.514340,     -1.234760,     -0.315140,
		      1.167650,     -0.718620,     -2.920150,     -1.861950,      0.027750,
		      0.968550,      2.179390,      0.739270,      0.348680,      0.686700,
		      0.468830,      0.587800,     -0.311000,     -1.865130,     -1.666750,
		      0.139820,      2.533170,      0.208080,     -0.684280,      0.130020,
		      0.290570,     -0.132820,     -1.444840,     -0.379500,     -0.727570,
		      0.324520,     -0.306680,      0.512380,      2.115700,     -1.804870,
		     -3.606190,      3.074450,      3.457340,     -2.511770,     -0.865160,
		      0.975490,      2.492180,      0.721290,     -0.545210,     -1.012150,
		      0.567980,     -3.189970,     -0.396070,      2.081540,      0.095660,
		     -2.571170,     -0.663710,      0.382570,      0.613140,      1.554170,
		      1.307780,      1.725110,      2.118180,      0.240160,     -0.672110,
		     -0.392340,     -1.241950,     -1.568210,     -1.144730,     -1.965690,
		      0.074060,      2.134490,      1.643750,     -0.299540,     -1.927870,
		      0.273980,      0.820410,     -0.133560,     -0.226540,      0.983520,
		      0.926990,      0.428810,      0.172020,      0.308710,     -0.381140,
		      2.239390,      2.349550,     -5.553480,     -0.376090,      3.811010,
		     -2.341510,     -3.502360,      1.440420,      1.283520,      1.136550,
		     -2.332980,     -0.801790,      0.466350,     -0.170280,     -0.361380,
		      2.712720,      1.608080,     -0.692390,     -0.610190,      2.044120,
		      1.218700,      0.673030,     -1.346190,     -0.765470,     -1.123770,
		     -2.273570,     -1.880520,      0.080560,     -0.931240,      1.618160,
		      0.757460,      0.242860,      1.987590,      0.257030,      0.741630,
		      1.011210,      0.580330,     -0.616390,     -0.897680,     -1.605760,
		     -0.818720,      0.172030,      0.364470,     -1.474300,      0.111220,
		      0.817550,      0.068740,     -0.058540,     -0.029900,      1.262930,
		     -1.658350,     -2.569820,      3.486250,      2.134820,     -2.300000,
		     -1.798730,      2.065210,      1.551150,     -2.769570,     -2.123970,
		      0.498400,      0.666130,     -0.631710,      0.236270,      1.183630,
		     -0.112700,     -1.703430,      0.320100,      2.194150,      0.337660,
		     -0.935850,     -1.205580,     -1.123820,     -1.771020,     -0.342830,
		      0.556660,      0.275190,      0.702410,      0.393210,     -0.362480,
		      0.986200,      0.360590,     -0.348360,      0.703280,     -1.199980,
		     -2.370400,     -2.054580,     -0.305170,      0.810930,      0.391050,
		     -0.476100,     -0.174660,      1.789410,      0.942070,     -1.040840,
		     -0.952580,     -1.667320,      2.062830,      1.056500,     -0.934780,
		     -2.483380,     -0.298440,      2.615540,      0.635970,     -5.453670,
		      1.505350,      4.575740,     -1.087030,     -3.499740,      1.518710,
		      1.618030,     -0.111150,     -1.593020,      0.511060,      0.360620,
		     -1.336960,     -1.205960,      3.501650,      1.887760,     -1.847050,
		     -2.394000,     -0.504900,      0.402800,      2.522300,      1.846840,
		      0.980010,      0.317670,     -0.149540,     -0.416300,      0.330120,
		      0.360830,      0.954710,      1.745480,     -0.622080,      0.369380,
		      0.148250,      0.513730,      1.595690,      1.007310,     -0.142230,
		      0.002850,     -0.328880,     -0.270430,     -1.557230,     -0.581800,
		      0.394950,      1.006970,      1.160660,     -1.636830,      0.414850,
		      1.062400,      1.056310,      2.608830,      3.064320,     -2.844130,
		     -3.070710,      5.070870,      0.557180,     -3.964870,      0.557770,
		      2.848480,      1.333820,     -0.830360,     -1.245160,      1.559230,
		      0.629760,     -1.215240,     -0.318730,      0.915830,     -2.731410,
		     -2.891360,      0.442310,      2.504710,      2.609930,      1.437810,
		     -0.046890,      0.547110,     -0.548350,     -0.202880,      0.072100,
		     -0.459210,     -0.760860,     -0.811180,     -2.907910,     -1.152180,
		      0.902930,     -0.801060,      0.290860,      0.440240,     -0.166180,
		     -0.776660,     -1.756090,     -0.478000,      0.577830,     -0.589580,
		      0.100890,      0.633730,     -0.181280,     -1.207020,     -0.496640,
		     -0.318820,     -2.130200,      0.605150,     -2.050670,     -1.273800,
		     -0.240340,     -3.622640,      2.016100,      3.411130,     -2.435950,
		     -5.201660,      0.872250,      3.097960,     -0.746800,     -2.682840,
		      0.793550,      1.233100,     -1.704330,     -2.311130,      0.559160,
		      1.555060,     -0.264210,     -0.632910,      4.617060,      1.934950,
		     -0.903310,     -1.109720,     -2.547380,     -3.258120,     -0.479050,
		      0.905700,     -0.080410,     -0.253780,      1.046620,      2.323290,
		      0.313350,      1.606660,      1.444440,      2.356820,     -0.375010,
		      0.031560,     -0.301890,      0.273830,     -0.700040,      1.694650,
		      0.665230,     -0.401070,     -0.950000,     -0.257750,     -0.377790,
		      0.945090,     -0.572310,      0.102420,      2.512280,      0.466380,
		     -1.299020,      0.372750,      0.414160,      1.266980,     -2.726050,
		      3.966910,      0.281140,     -4.421650,     -1.784520,      6.051820,
		      0.567770,     -4.228760,     -0.322290,      1.995870,      0.287320,
		      0.663090,      0.463290,      1.535290,      0.271150,     -1.416690,
		      0.134740,      1.764710,     -3.022730,     -3.565060,     -1.029980,
		      0.353520,      0.889330,      2.220660,      0.520850,     -0.294200,
		      0.363520,      1.522580,      0.988250,     -1.188790,     -0.475410,
		     -0.556770,     -1.020190,     -3.705080,     -0.100970,     -0.944020,
		     -0.939050,     -0.731620,     -0.214770,      0.772900,     -0.779460,
		     -0.018940,     -0.185840,     -1.702290,     -0.585880,     -1.168200,
		      0.097260,     -0.951860,     -1.036080,     -0.412430,     -0.807690,
		      0.619430,     -1.866580,     -1.319830,     -1.981690,      2.409530,
		     -1.864180,     -2.442390,      2.776850,      4.212830,     -3.016010,
		     -3.703330,      1.282640,      1.400720,     -1.025130,     -0.665540,
		     -0.601650,      1.063670,     -0.723800,     -1.720260,      0.024330,
		     -0.241240,     -0.502810,      0.730120,      5.237860,      1.960110,
		     -0.060830,      1.869850,      0.190320,     -1.429400,     -1.417250,
		      1.420050,     -0.716750,     -2.581020,     -0.290420,      2.085630,
		     -1.012840,     -0.395800,      0.526350,      1.770060,      0.804070,
		     -0.744870,     -0.751860,      1.238490,     -1.271610,     -0.259480,
		     -0.363680,      0.650620,      1.041260,      0.504910,      0.277450,
		      0.385100,      0.309390,     -0.709230,      0.282260,      1.805100,
		     -1.644760,      0.933630,     -1.029600,      0.101290,     -0.586950,
		     -1.152110,     -1.212970,      3.981160,     -0.217180,     -4.907800,
		     -0.396060,      5.158220,      0.765280,     -1.991230,      0.412990,
		      0.404100,     -1.228560,      0.110690,      0.191870,      2.108820,
		      1.040250,      0.101300,     -0.030430,      0.839260,     -2.356890,
		     -1.722730,     -0.501920,     -1.020860,     -1.618580,      1.204440,
		      0.841440,     -1.595890,     -1.047650,      1.088960,      2.271020,
		     -1.034410,      0.446980,      0.880620,      1.792440,     -2.178430,
		     -0.298210,      0.348760,      0.347610,     -0.881340,     -0.065930,
		     -0.047280,      0.798400,     -0.547520,     -1.209270,     -1.297330,
		      0.453150,     -0.881110,     -1.736310,     -1.137230,      0.375060,
		     -0.261210,     -0.474940,     -0.387550,      2.405280,     -0.757800,
		      0.025070,     -0.349280,      1.274320,      2.425830,     -3.260530,
		     -1.227650,      2.705730,      2.876100,     -3.799840,     -1.502710,
		      1.146890,      0.589850,     -0.919160,      0.046130,     -1.054810,
		     -0.165370,     -0.600610,     -1.429060,      0.003380,     -1.396300,
		     -1.535520,      0.688800,      3.714420,      0.914050,     -0.980240,
		      1.876920,      1.409480,      0.670060,     -0.230860,      1.993270,
		      1.998650,     -3.102010,     -1.801130,      0.883070,     -0.384410,
		     -2.281430,      0.124930,      0.383060,      0.204870,     -0.046960,
		     -0.464080,      1.143900,     -0.021610,      1.052630,     -0.844090,
		     -0.262490,      0.786520,      1.389760,      2.549380,      1.536540,
		      0.929280,     -1.211990,     -0.214780,      1.072410,     -0.545970,
		      0.104950,      0.443060,      2.118850,      0.376870,      1.010180,
		      0.611930,     -1.142210,     -0.302320,      2.538060,     -0.571800,
		     -4.326130,      0.171390,      2.641140,     -0.721480,     -0.878670,
		      0.774320,      0.907190,      0.782030,      0.692300,     -0.895630,
		      1.017960,      1.418330,      1.381310,      0.676120,     -1.261020,
		     -3.129970,     -0.848420,      1.364330,     -0.101330,     -2.129580,
		     -0.925170,      0.335950,     -1.305520,     -0.898680,      1.121950,
		      2.776130,     -1.021680,     -0.587500,      0.152250,      2.227950,
		     -0.528440,     -0.771610,      0.720420,      0.284290,     -0.250500,
		     -0.303770,     -0.492010,      0.867490,     -0.621050,     -0.123450,
		     -1.033520,     -0.493140,      0.080980,      0.008750,      0.988650,
		     -1.018320,     -0.114180,     -0.409790,     -1.187670,     -0.331460,
		     -0.412090,      1.125490,      1.085560,      0.997990,     -1.852970,
		      0.112120,      2.830010,     -2.398060,     -1.663730,      3.157350,
		      2.132430,     -2.736480,     -0.292940,      0.546870,      1.407330,
		     -0.163200,     -0.671170,     -1.687830,      0.437010,     -0.246940,
		     -1.709140,      0.683660,     -0.098300,     -0.290000,      1.031180,
		      2.197420,      0.275610,     -2.019680,      1.981720,      2.471840,
		      1.110730,      0.132600,      0.280550,      2.316920,     -0.120490,
		     -0.158740,      1.038930,      0.307870,     -3.510320,     -0.655620,
		     -0.396730,     -0.006110,     -0.428680,      0.426990,     -0.480520,
		      0.998120,      1.064630,      0.872470,     -1.523710,      0.612410,
		      0.588780,      1.814090,      1.075950,      1.320700,      1.467320,
		      0.688470,     -0.230750,      0.881030,      0.612200,      2.531100,
		     -0.913660,      0.199200,      1.041690,      1.813760,      1.360590,
		      1.131190,     -1.859020,      0.327580,      2.627100,     -0.329480,
		     -3.327440,      2.220140,      1.799050,     -0.689810,     -1.147600,
		     -1.127600,      0.290270,      1.885240,      1.019570,     -0.744740,
		      0.768310,      0.195550,      0.545910,      1.318360,     -0.655150,
		     -2.105920,     -0.547980,     -0.129100,      0.772220,     -0.991740,
		     -0.797370,     -0.025310,     -1.983410,     -1.984140,     -0.281000,
		      1.780110,      1.450210,      0.506880,      1.596690,      2.080590,
		      1.717350,     -0.363150,      0.133970,      0.614730,     -0.812930,
		     -0.060300,     -2.675880,     -0.961010,     -0.575880,     -0.235220,
		     -1.395550,     -0.518480,     -0.729840,      0.756490,      1.212250,
		     -1.221070,     -0.879290,     -1.051010,     -1.391840,     -2.666010,
		     -1.974920,     -0.960210,      0.062980,      0.297950,      2.369150,
		      1.369580,     -2.026010,     -0.722080,      2.969160,     -2.848840,
		     -2.341980,      2.590010,      1.052830,     -2.759690,     -0.047520,
		      0.581420,      1.569080,      0.768020,     -1.122390,     -1.559080,
		      0.404460,     -0.378400,     -2.273100,     -0.343790,     -0.765360,
		     -0.691090,      0.800730,      0.560070,      0.564180,     -2.410300,
		     -0.495430,      1.379290,      1.827630,      2.911880,      0.659610,
		      0.717480,     -0.764280,     -0.742680,     -0.730460,      0.750070,
		     -2.806170,     -1.599880,     -0.583100,     -0.756220,     -0.401750,
		      0.330050,     -0.557750,      0.965620,      2.097320,      2.925450,
		     -0.607290,      2.515820,      1.232840,      0.219320,     -0.780060,
		     -0.367040,      1.220980,      1.029980,      1.455660,      1.194260,
		      1.970420,      1.898580,      1.061720,     -0.597000,     -0.700320,
		     -1.420150,     -1.031100,     -1.418990,      1.145430,      1.892890,
		     -2.359850,      0.094320,      3.250410,     -1.426890,     -2.928950,
		      2.253500,      1.545780,      0.772450,     -0.402010,     -1.839370,
		     -0.187480,      1.666830,      0.874790,      0.859550,      2.521660,
		      0.835560,     -0.163910,      1.299300,      0.127320,     -0.177850,
		      0.495880,      0.809370,      0.545490,     -0.604040,     -0.746020,
		     -0.817520,     -0.898080,     -1.482920,     -0.694060,     -0.332030,
		      0.602610,      0.527170,      0.831040,      0.352330,      1.602420,
		      0.634810,      0.307960,     -0.158690,      0.378370,      0.145740,
		     -0.860950,     -3.667010,     -1.748230,     -1.171470,     -1.734210,
		     -0.979700,      1.180370,      0.903520,      0.948220,     -1.902700,
		     -3.387680,     -0.566710,     -0.668420,     -0.365790,     -1.838710,
		     -0.759760,      0.292320,      0.798090,     -0.332110,     -0.215450,
		      1.648990,      0.863300,     -0.854870,      0.568650,      3.274210,
		     -2.862420,     -2.347370,      2.284800,     -0.190050,     -1.766250,
		     -0.425720,     -0.241500,      1.358640,      1.172990,     -0.464370,
		     -0.900620,      0.606540,     -0.724480,     -2.173270,      0.249650,
		     -0.075920,     -0.882560,     -0.723620,     -1.515630,      1.652090,
		      0.419130,     -0.520230,     -0.385380,     -1.181340,      2.254200,
		      2.264830,      1.202090,      0.473310,     -0.135810,     -0.765640,
		      0.690370,     -1.079990,     -0.941430,     -0.287300,      0.548010,
		     -0.872700,      0.442520,     -1.297230,      0.422980,      0.950890,
		      0.960280,     -0.957950,      0.926420,      0.868520,      1.581720,
		      1.008210,     -0.263560,      0.264610,      0.601510,     -0.099810,
		      0.220040,      0.359630,      1.656050,      0.729130,     -0.782690,
		      1.141250,      0.457490,      0.283780,     -2.512090,     -0.804630,
		     -0.869130,      1.521580,     -0.027800,     -3.911130,      0.473130,
		      3.662260,     -0.952760,     -2.284960,      1.876510,      0.436400,
		      0.677330,      0.348790,     -1.711590,     -0.774510,      0.484810,
		     -1.179600,      0.530780,      2.594770,     -0.099940,     -1.612750,
		      0.449320,      0.662930,      0.292910,      0.739680,     -0.730960,
		     -0.366720,      0.616040,      0.295130,     -0.641160,     -0.784510,
		     -1.328840,     -0.015970,     -1.062850,     -0.503340,      0.102860,
		      1.239850,      0.293850,      1.030680,      0.903550,      0.149340,
		     -0.300320,      0.229720,      0.524700,      0.634680,     -0.749350,
		     -0.498370,     -0.438200,     -0.157680,     -0.941960,      0.052740,
		      0.883020,      1.731990,      0.090160,     -0.911420,     -0.067460,
		     -0.781310,     -0.146740,      0.315710,      0.731890,      0.322470,
		      0.994410,      0.794930,      1.114870,     -0.621500,      0.311900,
		      1.478510,     -2.113790,     -1.152340,      1.838440,      3.071160,
		     -2.913410,     -0.973620,      2.422750,     -0.559500,     -0.082140,
		     -0.630220,     -0.528120,      0.002680,      0.912480,     -0.393030,
		     -0.315410,      0.246130,     -0.852180,     -0.843520,      0.901980,
		      1.108080,     -0.250430,     -1.310220,     -2.390070,      0.629810,
		      1.795220,      1.751110,      1.119380,     -2.158040,     -0.118800,
		      1.657110,      0.842260,      1.201450,      0.428260,     -1.811870,
		     -0.092640,     -1.428720,     -1.892130,     -1.158390,      0.923040,
		      1.305920,     -0.766160,     -0.796540,      0.489480,      2.110140,
		      0.567360,     -1.317390,     -1.284660,     -1.210590,     -2.358370,
		      0.410520,      0.562430,     -0.075770,      0.685130,      1.046370,
		     -0.223330,      0.056670,      0.296260,     -1.123600,     -1.563250,
		      0.636640,      1.794670,      0.818920,      2.080130,     -0.531550,
		      1.421070,     -2.788890,      1.023860,      3.253170,      0.568260,
		     -3.845310,      0.628780,      2.752270,     -0.428110,     -0.842090,
		      0.760130,     -0.210860,      0.420640,      0.871060,     -0.461910,
		     -1.077400,      0.103900,     -0.529750,      0.200450,      1.967580,
		      0.458810,     -1.678930,     -0.396960,      1.368100,      1.619460,
		      0.073610,     -1.852960,     -1.834610,     -1.164890,      1.331930,
		      0.444500,      0.094070,     -1.060980,     -0.373460,     -0.551470,
		      0.280700,     -0.122210,      1.568650,      0.125580,      1.234930,
		      0.611960,     -0.135410,     -0.268380,      1.041680,     -0.166930,
		     -1.288480,      0.013140,      0.490250,      1.429630,      1.731160,
		      0.598150,     -1.066270,     -1.090050,     -0.168590,      2.237520,
		      1.352020,      0.330230,     -0.218430,      0.159080,      0.848080,
		      1.034590,     -1.030320,      0.998640,     -0.179520,      1.133470,
		     -0.547750,     -0.076280,     -0.646270,      2.353100,      1.486000,
		     -2.828840,     -1.398540,      1.718200,      2.188700,     -2.169440,
		     -1.066290,      0.724490,     -1.059290,      0.646370,      0.320420,
		     -0.866770,     -0.633240,      1.069300,      1.015610,      0.391460,
		      0.327770,     -1.432850,     -1.139700,     -0.306330,      0.567080,
		     -0.441370,     -1.286380,     -1.874780,     -0.635570,      1.671560,
		      0.828830,      1.374170,      0.178660,     -1.818360,      0.798250,
		      0.576000,      0.878000,      0.406910,     -1.264100,     -0.234720,
		     -0.264540,     -2.011160,     -2.346090,     -0.504140,      0.695110,
		      0.168780,     -0.560110,     -0.857810,      2.490510,      1.509610,
		     -0.123130,     -0.991610,     -0.835050,     -0.783030,     -0.406120,
		     -0.360820,     -0.558890,     -1.395570,      0.901960,      0.338950,
		     -0.161280,     -0.118930,      0.655900,     -0.829020,     -1.266820,
		      0.088350,     -0.484310,      1.040380,      0.717420,      0.222530,
		      1.991270,     -0.336460,     -3.369690,      0.303600,      1.775070,
		     -0.597190,     -2.843230,     -0.174380,      1.190270,      0.848610,
		      0.987970,     -0.074880,      0.591410,      0.953610,      0.882400,
		      0.406970,     -0.674320,     -0.596800,     -0.961860,      0.240930,
		      0.439350,      0.250360,     -1.026170,     -0.703530,      2.516150,
		      2.680820,      0.998620,     -0.825120,     -0.930050,     -2.088630,
		     -0.234240,      0.936460,      0.484070,     -0.564770,     -0.281510,
		     -0.746130,      0.455580,     -0.574420,      1.234250,      1.185770,
		      2.461460,      1.180100,     -0.349880,      0.274830,      1.067450,
		      1.063750,     -2.315290,     -1.372730,     -0.836050,      0.747480,
		      1.405340,      1.132890,     -0.442580,     -1.396390,      0.003700,
		      1.582250,      0.736480,      1.509340,     -0.096770,     -0.329740,
		      0.618190,      0.133940,      0.293160,      0.636480,      0.347020,
		     -0.785290,      0.903320,      0.318420,     -0.339540,     -1.463350,
		     -2.190970,      3.246670,      1.166850,     -1.377710,     -0.484640,
		      1.369080,      1.182970,     -0.221250,     -0.177030,      0.060740,
		     -0.829310,      0.378650,     -0.485630,     -0.619930,     -1.041360,
		     -0.087600,      0.002050,     -0.270540,      1.270240,      1.089780,
		      0.914570,      1.303340,      0.586090,     -1.948420,     -2.598120,
		     -1.384920,      0.149710,      1.132650,      0.530500,      0.404020,
		      1.243640,     -2.145970,     -1.075480,      0.179510,      1.748830,
		      0.444420,     -0.536750,     -0.513390,      0.338590,     -2.685530,
		     -0.510790,     -0.361050,      0.777690,     -0.582950,     -1.206850,
		     -0.656370,      1.569300,      2.346620,     -0.290790,     -1.027710,
		     -0.055020,     -0.388870,      0.321530,      1.082880,     -0.807130,
		     -1.124640,     -0.760710,      1.005610,      0.016760,     -0.089920,
		      0.086150,      1.505020,      0.224540,      0.003910,     -1.177070,
		     -1.058250,     -0.424810,     -0.576310,     -0.560250,      0.304830,
		      2.517560,     -0.210490,     -1.426750,      1.854160,      1.730200,
		     -0.139070,     -1.171580,     -0.176340,     -0.654190,      0.670440,
		      0.634890,     -1.556380,     -0.638900,      0.709180,      0.534770,
		      0.809990,      0.746500,      1.242130,     -0.806860,     -0.561930,
		     -0.841240,     -0.962970,     -1.019820,     -0.537230,      1.254030,
		      1.122150,     -0.265780,     -1.815560,     -0.461610,     -0.473000,
		     -0.614710,      1.173940,      0.882600,      0.052220,     -0.138940,
		     -0.213560,      0.301730,     -0.193530,     -0.623300,      0.412600,
		      0.496260,      0.720460,     -0.527670,      0.268220,      2.168600,
		      2.006350,     -1.613140,     -1.074240,     -0.425220,     -0.629620,
		      0.596690,     -0.142940,      0.537890,     -0.276250,      0.528250,
		      1.669660,      1.125850,     -0.127360,      0.244650,     -0.019560,
		      0.313960,     -0.961950,     -1.429040,      0.840790,      1.547860,
		     -0.393750,      0.252670,      0.092730,      0.292230,     -0.057270,
		      1.252040,     -0.789570,     -1.932960,      1.568290,     -0.446180,
		     -1.900660,      0.362060,      0.890880,      0.678200,      0.382680,
		     -0.325650,     -0.775770,      1.868870,      1.512950,     -0.427010,
		     -0.492180,     -0.039040,     -0.015730,     -0.572160,     -0.740220,
		     -0.311390,     -0.299990,      0.215460,      2.499820,      2.158800,
		     -0.171350,     -2.027750,     -1.291850,      0.643370,      1.403100,
		      0.280310,     -0.263640,      1.386850,     -0.747720,     -1.272860,
		     -1.543060,      0.146630,      0.401540,      0.674980,      0.612880,
		      2.040330,     -1.620650,     -0.386760,     -0.001420,      0.523800,
		     -0.184930,     -1.179670,     -0.839850,      0.046370,      1.109170,
		     -0.807760,     -1.533470,     -0.355730,     -0.252550,     -0.088130,
		     -0.700780,     -0.717460,     -0.464670,      0.348590,      1.927820,
		     -0.633460,     -1.710110,     -0.120130,      1.567680,     -0.692010,
		     -0.949600,     -1.047670,      0.155990,      1.136280,     -1.004130,
		      0.100740,     -0.586190,     -0.458960,     -0.759790,      2.020630,
		     -1.431650,     -1.432660,      0.560200,      0.633400,     -1.257690,
		     -0.501560,      0.288030,     -0.383970,      0.925760,      0.414650,
		     -1.729460,     -0.082820,      0.598910,     -0.812340,     -0.750270,
		      0.382850,      1.846090,      0.071570,      0.499790,      0.484700,
		     -1.457560,     -1.837170,     -0.150870,      1.143640,      0.890290,
		      0.868420,     -1.010090,     -0.569970,     -0.495550,     -1.335830,
		      0.332210,      0.729070,      0.812690,     -0.331830,     -0.034490,
		     -0.491730,      0.056450,     -1.194250,      0.993990,     -1.035910,
		      0.058530,     -1.204240,     -0.139220,      0.854760,      1.881710,
		      0.436790,     -0.826830,      0.612080,      0.750250,      0.803880,
		     -0.511300,     -0.579030,      0.730740,      0.689520,     -0.409420,
		      0.381390,     -0.149570,      0.262370,     -0.565540,      1.836890,
		      0.036480,     -0.235280,      0.633730,      0.838690,     -0.578510,
		     -1.236300,     -0.049070,     -1.176250,      0.099800,     -0.597540,
		     -0.720680,      0.897080,      0.924790,     -0.794030,      2.023140,
		      0.051700,     -1.518570,     -0.326830,      0.573310,      0.147980,
		      0.535780,     -1.360880,     -1.412570,      0.689310,      1.437370,
		      0.323900,      0.005300,      0.295440,     -0.314780,     -1.305770,
		      0.443950,      0.465550,      0.062660,     -0.956320,      1.278270,
		      0.957960,      0.281830,     -0.671470,     -1.823220,     -0.593730,
		     -0.162470,      0.044070,     -1.049960,      0.148950,      0.690550,
		      0.156150,     -0.944820,     -1.072900,     -0.950970,     -0.734280,
		      0.356650,      0.758930,     -0.029070,     -0.048730,      0.189080,
		     -0.409150,     -0.422730,     -0.851380,      0.212380,     -0.366390,
		      0.286190,     -0.160270,     -1.759720,     -0.634480,      0.261780,
		     -0.524700,     -1.765530,     -1.056730,     -1.181560,      0.591570,
		      0.594170,      0.833410,     -1.373210,     -1.006220,     -0.541160,
		     -1.074000,     -0.447960,     -0.990940,      0.141820,      0.452300,
		      0.740810,     -1.100100,      1.707700,      1.252180,      1.461740,
		     -0.173040,     -1.837110,      1.276560,     -0.093620,     -0.994540,
		      1.476620,      1.011210,     -0.187330,     -0.073460,      0.514870,
		      0.367720,      1.334330,     -0.433190,     -1.566410,      0.021220,
		      0.526660,      0.034630,     -0.268140,      0.143850,      0.163430,
		     -1.357480,     -0.383190,      1.975660,     -0.012790,     -1.000870,
		     -0.024520,      0.444440,      0.093600,      1.549490,      1.097170,
		      0.842150,      0.852170,     -0.966520,      0.993000,      0.405620,
		      0.743660,      0.861330,      0.709230,     -0.421500,      0.001110,
		     -0.929940,      1.376010,     -0.559130,     -0.479370,     -0.761410,
		     -0.154620,      0.349890,     -0.038350,     -0.495330,      0.112000,
		      0.551800,      0.910770,      1.998530,      1.037080,     -0.312170,
		      1.193720,      0.530730,      0.624490,      0.790860,     -1.701110,
		     -0.049980,      0.556800,      0.551270,      1.526850,      0.797750,
		     -0.414590,      1.716590,      0.075680,     -0.507630,     -0.593630,
		     -0.495940,     -1.309950,     -0.327090,      0.752260,     -0.705330,
		     -2.244840,      0.632890,      0.164510,     -0.008320,      0.981320,
		     -0.176420,     -1.055650,      0.061430,      0.395090,      0.839650,
		      0.675460,     -0.529080,     -0.631540,     -0.324760,     -0.521230,
		      0.193490,      1.079030,      1.445650,     -1.262290,     -2.034980,
		      0.626950,      0.790060,      0.252500,     -0.404270,      0.363620,
		     -0.727830,      0.172510,      1.006020,     -0.991540,     -0.739570,
		     -2.104450,     -0.365000,     -0.639310,     -2.241510,     -0.788260,
		     -0.036090,      0.896750,     -0.439900,      0.148530,     -0.696080,
		      0.330690,     -0.480030,      0.350380,      0.111000,      1.260810,
		     -1.006900,      0.435310,     -0.174360,      1.449000,     -0.646690,
		     -1.154220,     -0.771670,     -1.324100,     -0.718000,     -0.133520,
		      0.020490,     -0.393550,     -1.209720,     -1.020250,      1.156800,
		      0.095450,     -0.230280,     -0.017760,     -0.370660,     -0.359330,
		     -1.071650,     -0.916350,      0.468660,      0.350720,      0.646660,
		      1.110090,     -0.116410,      1.193220,      0.706640,     -0.402470,
		     -0.849880,      2.538470,      1.251740,     -1.516740,      0.188110,
		     -0.140810,     -1.661730,      0.147430,      0.257330,     -0.280020,
		     -0.300670,     -0.955830,      0.399430,      1.142850,     -0.277440,
		     -0.173650,      0.061000,     -1.395510,     -0.689890,      0.848170,
		      2.566880,      1.033480,     -1.091980,     -1.693530,     -0.139300,
		      0.318940,      0.486300,      0.898180,     -0.089570,     -1.073900,
		      0.749390,      1.621310,      0.715260,      0.346840,      0.459970,
		      1.712560,      0.356900,     -1.498230,     -0.651570,      0.389250,
		      0.139190,     -0.291930,     -0.413920,      0.463130,      0.932570,
		     -0.453810,     -0.469460,     -0.753860,      0.552270,     -0.380740,
		      0.320440,      0.726640,      0.469000,      0.151620,      0.993860,
		      1.807880,      0.428730,      0.038240,     -0.023610,      1.213400,
		     -0.118350,      0.071680,      0.404480,      1.203150,     -0.250330,
		     -0.454610,      0.626410,     -0.008170,     -0.388910,     -0.297630,
		     -0.325580,     -0.853590,     -0.035500,     -0.861990,      1.330150,
		      0.482330,      0.538320,      2.361920,     -1.561370,     -1.144960,
		      0.253300,      0.166030,      0.019450,      1.708610,     -0.268620,
		     -0.446140,     -0.620020,     -0.633260,      0.696460,      1.068220,
		     -0.393930,      1.147880,      0.107040,     -1.405110,     -0.397730,
		      0.935950,      1.647390,     -1.103990,     -2.518160,     -0.139800,
		      1.024460,      1.026960,      1.314430,      1.568870,     -1.173710,
		     -0.755110,      1.904330,     -0.359520,     -1.652510,     -2.654940,
		     -0.714600,      0.636300,     -1.494980,     -0.968350,     -0.186920,
		      0.718320,      0.925220,      0.886390,     -0.629170,      0.158310,
		     -0.133400,     -0.280060,     -0.338690,      1.105050,      0.127110,
		     -0.467370,     -0.425160,      1.313410,     -0.228250,     -0.415140,
		     -1.022070,     -0.926620,     -1.335840,      0.701290,      0.609600,
		      0.692080,     -0.743660,     -0.768380,      0.514090,     -0.914220,
		      0.022680,      1.009440,      0.266160,      0.406340,      0.460040,
		     -1.292320,     -0.088350,      0.484730,      0.892970,     -0.399150,
		     -1.111380,     -0.987510,      0.580530,      0.844010,      0.010760,
		     -2.501890,     -1.189290,      1.001480,      0.213150,     -0.182600,
		      0.422580,     -0.240180,     -0.937160,      0.348880,      0.717300,
		      1.067410,      0.331700,     -0.664130,     -0.238900,     -0.293800,
		     -0.500320,      1.454800,      1.211800,     -1.670440,     -3.068160,
		     -1.010800,      1.536830,      1.584480,      0.548620,     -0.946240,
		     -2.079630,     -1.963080,      0.715180,      1.781920,      0.377280,
		     -0.061340,      1.115330,      2.103910,      0.417690,     -1.185640,
		      0.225760,      2.368130,      0.700140,     -1.075070,     -0.960690,
		     -1.033960,      0.032610,     -0.239090,      0.496960,     -0.221150,
		      0.856750,     -0.410960,      0.704770,     -0.213050,      0.747220,
		     -1.033780,      0.037320,      1.154610,      1.198960,      0.961550,
		     -0.202950,      1.157940,     -0.246240,      0.122510,      0.190130,
		     -0.210660,     -0.327890,      0.630470,      0.434150,      0.526380,
		     -0.085760,     -1.466880,      0.577140,      1.412850,      0.375440,
		     -0.841480,     -1.841760,     -0.420430,      0.730680,      0.130670,
		      0.278040,      0.013760,      0.176630,      0.237410,      0.437260,
		      1.720120,     -0.879420,     -0.358030,     -0.000360,     -0.844640,
		     -0.239810,      1.138570,      0.520400,      0.502610,     -0.542290,
		     -1.809890,      0.544800,      0.224050,     -0.544640,      0.390270,
		      0.149790,     -1.052650,     -0.145010,      1.357950,      1.645410,
		     -0.223850,     -1.335850,     -1.196670,      0.032440,      1.456600,
		      2.476980,      1.695460,     -0.730820,     -1.299720,      1.065380,
		     -0.081870,     -1.738530,     -1.921650,     -0.382650,      0.063290,
		     -1.941860,     -0.616080,     -0.071850,      0.145390,      0.916960,
		      1.487820,      0.485280,     -0.900900,     -0.243220,     -0.072260,
		     -0.793750,     -0.564110,      0.163840,     -0.121220,     -0.860640,
		     -0.150430,     -0.566310,      0.088090,     -0.235850,      0.133810,
		     -0.763020,     -0.739480,      0.558570,      0.457870,     -1.089250,
		     -0.884560,      0.231130,     -0.856510,     -0.735650,     -0.014770,
		      0.682510,     -0.100540,     -0.324770,     -0.668310,     -0.431960,
		      0.861120,      0.348170,     -0.613170,     -1.071330,     -0.327530,
		      0.518970,      0.442320,      0.438420,      0.641480,      1.366650,
		     -2.533310,      0.220030,      0.202930,     -0.123260,      0.613830,
		      1.027240,     -1.355180,     -1.046220,     -0.834830,     -0.063080,
		      1.096910,      1.129610,     -0.165080,     -0.120720,     -0.218720,
		     -1.223260,      0.456940,      1.896620,      0.559280,     -1.434680,
		     -1.212980,      0.545560,      0.603630,      0.697150,      0.275620,
		     -2.259950,     -2.922960,     -0.134610,      1.173950,     -0.764750,
		     -0.164320,      2.450270,      3.330690,      1.445640,     -0.836240,
		      0.495030,      1.284100,     -0.031020,     -0.910660,     -0.071270,
		     -0.602030,     -0.798240,     -0.206930,      0.569590,     -0.024100,
		      1.568490,      0.580830,      0.862330,      0.700640,      1.579560,
		      0.143120,     -0.001510,      0.117910,      1.187470,     -0.549960,
		      0.115090,      0.485910,      0.335830,     -0.062340,      0.119730,
		      0.257720,     -0.070140,      0.675780,      1.011310,      0.678950,
		      0.468520,     -0.631730,      0.582890,      1.253180,      0.056030,
		     -1.028430,     -0.301520,     -0.558110,     -0.657920,     -1.416820,
		     -0.082730,      0.357520,     -0.902160,     -0.771060,     -0.352670,
		     -1.619680,     -0.100350,      1.351450,     -0.777670,      0.440090,
		      0.261210,     -1.267430,     -0.486330,      0.625210,      0.486100,
		      0.901860,     -0.216050,     -0.677610,      0.271050,      0.146400,
		      0.092090,      0.361380,      0.121520,     -0.818910,     -1.345700,
		     -0.020230,      0.776930,      0.141500,     -0.917570,     -1.404970,
		     -1.230960,      1.285330,      3.562830,      1.231280,      0.466620,
		      0.770790,      0.561380,     -0.747820,     -2.960200,     -2.231500,
		      0.340580,     -0.101130,     -1.354810,     -0.256060,      0.684170,
		     -0.260960,     -0.053640,      1.151830,      1.632840,     -0.730830,
		     -1.369520,     -1.339470,     -0.442880,     -1.853890,     -0.451180,
		     -0.540300,     -0.765560,     -1.253240,     -0.889120,     -0.196370,
		     -0.024970,      1.358450,     -0.217520,     -0.292690,     -0.498940,
		      0.195550,     -0.522360,     -1.861090,     -0.219470,      0.418040,
		     -0.221080,     -0.575250,     -0.004740,     -0.122250,      0.871800,
		      0.110520,      0.129300,     -0.144870,     -0.202840,      0.339030,
		      0.564050,     -0.418810,      0.037580,      1.220530,      0.478580,
		      0.111750,     -0.271610,      0.971280,      1.344550,     -1.520100,
		     -0.178790,      0.015210,     -0.795970,      0.725880,      1.689780,
		     -0.186440,     -0.331750,     -0.838310,     -0.910230,      0.762490,
		      0.570530,      0.372580,      0.400270,      0.738720,     -0.563880,
		     -1.155420,      0.014890,      0.841060,     -0.077870,     -0.438240,
		      0.549450,      0.977500,      0.843580,      0.763090,     -1.247900,
		     -2.047910,     -0.173380,     -0.233810,     -1.176960,     -0.854680,
		      1.900210,      2.981360,      1.593830,     -0.082150,      0.832670,
		      0.873230,     -0.530320,     -1.537900,      0.117320,     -0.093450,
		     -0.853230,     -0.608720,      0.467300,     -0.065400,      0.354910,
		      1.123570,      0.602310,      0.546850,      1.659540,      1.587060,
		      0.947570,     -0.382280,     -0.018070,     -1.101410,     -2.085700,
		     -0.075120,     -0.299650,     -0.042680,     -0.092510,      0.427920,
		     -0.610420,      0.329740,      0.366560,      0.647130,      1.180980,
		      1.075180,     -0.455240,      1.039220,      0.028020,     -0.429810,
		     -0.317820,     -0.097280,     -0.779520,     -0.673330,     -0.439810,
		     -0.245490,      0.554300,     -0.968820,     -0.821080,     -0.510200,
		      0.345920,     -2.559720,      0.335290,      0.627390,     -0.320250,
		      0.949200,      0.332360,     -1.732360,      0.055120,      0.250080,
		      0.021500,      0.235640,     -0.120590,     -0.647060,     -0.246190,
		     -0.642110,     -0.405700,      0.231860,      1.152740,     -0.001990,
		     -0.731140,      0.244580,      0.254450,      0.112800,     -0.629470,
		     -0.662100,     -1.299140,     -0.081770,      2.566370,      0.129070,
		      0.424610,      2.018890,      1.811720,     -0.185100,     -1.879050,
		     -2.020630,     -0.233410,     -0.143260,     -0.537260,     -0.121390,
		      1.446340,      0.369640,     -0.388450,      0.046820,      1.817760,
		      0.546510,     -0.324210,     -0.613800,     -0.720340,     -1.339040,
		     -1.637240,     -1.214630,     -0.774210,     -1.365740,     -0.123240,
		     -0.234980,      0.481330,      0.778100,      0.412940,     -0.248130,
		     -0.686840,      0.056890,      0.164480,     -0.461460,     -0.449880,
		     -0.940690,      0.426020,      0.130870,     -0.856230,     -0.527270,
		      0.179650,      0.554060,      0.600670,     -0.212030,      0.020720,
		      0.230350,      0.037240,      1.261760,      0.577280,     -0.444830,
		     -0.218360,      0.349940,      0.162150,     -1.285330,     -1.185080,
		      0.719670,      1.254130,     -0.486100,      0.198020,     -0.138880,
		     -0.595580,      0.099160,      0.559920,     -0.379630,      0.553660,
		      0.276370,      0.272020,     -0.262920,     -0.813860,      0.694540,
		      0.577280,      1.199390,      0.226560,     -1.252400,     -0.309980,
		     -0.040810,      0.057100,     -0.118910,      0.819150,      0.503380,
		     -0.142200,      0.233830,     -1.059020,     -0.871720,     -0.506170,
		     -0.948600,     -1.450470,     -0.800390,      1.252230,      2.217760,
		      1.048100,      0.405740,      0.325170,     -0.228230,     -0.145640,
		     -0.653960,     -0.393110,      0.096170,     -0.516450,     -0.989470,
		     -0.290990,      0.104600,      0.398760,      0.764040,      0.737920,
		     -0.028450,      0.760400,      1.051740,      1.146820,     -0.089640,
		      0.843100,     -0.577990,     -0.666100,     -1.379900,     -0.278710,
		     -0.414630,      0.004910,      0.299940,     -0.318990,      0.076430,
		     -0.510860,     -0.545880,      0.833920,      0.937590,      0.875590,
		      1.115480,     -0.103450,     -0.532570,     -0.000380,      0.064260,
		      0.033140,     -0.849110,     -0.569420,      0.282260,      0.288570,
		     -0.693130,      0.140840,      1.072270,      0.168500,      1.102580,
		     -0.511430,     -1.887650,     -0.286180,     -0.225650,     -0.498880,
		      0.825360,      0.465140,     -0.534590,      0.214820,      0.290640,
		     -0.733230,      0.365350,      0.681330,      1.064210,     -0.154190,
		     -1.127090,     -1.240910,     -0.538920,      1.040160,      0.196530,
		     -0.419030,     -0.450560,     -0.249870,     -0.999850,     -0.562760,
		      0.574000,      0.286420,      0.386030,      1.352880,      0.236490,
		      0.855460,      1.803590,      1.663510,     -0.500830,     -1.049310,
		     -1.245450,     -0.586170,     -0.570850,      0.573720,     -0.159670,
		      0.714680,      0.161720,      0.464520,     -0.179240,      0.774320,
		      0.024400,      0.050890,      0.123860,      0.382200,      0.027610,
		     -0.209930,     -0.456580,     -0.460870,     -1.457810,     -0.715530,
		     -0.254220,      0.105180,      1.308800,      0.671500,      0.190390,
		      0.527410,      0.339110,     -0.072980,      0.406530,     -0.347070,
		      0.038930,      0.450880,     -0.594950,     -1.063370,     -0.826060,
		     -0.700170,      0.652780,      0.073180,      0.307860,     -0.364890,
		      0.545590,      0.615380,      0.485300,      0.527920,     -0.514610,
		     -0.798550,     -1.087320,      0.112030,     -0.527230,     -0.166610,
		     -0.201650,     -1.576700,      1.179840,      0.043530,      0.235510,
		      0.197290,     -0.045900,     -0.093890,      0.105250,     -0.203780,
		     -0.722010,      0.063190,      0.472920,      0.071310,     -1.120090,
		     -0.880050,      1.182760,      0.993210,      0.352420,     -0.315850,
		     -0.665530,     -0.153740,     -0.180060,     -0.300290,      0.419320,
		      1.038520,      0.860740,     -0.331500,     -0.316970,     -1.150480,
		      0.231040,     -0.206560,     -1.153250,     -1.458730,     -0.288010,
		      1.008370,      1.797210,      0.650210,      0.720190,     -0.352870,
		     -0.917130,     -0.738470,      0.166700,     -0.366870,     -0.105320,
		      0.009740,      0.243140,      0.064430,      0.356040,     -0.589220,
		      0.055000,      0.601700,      0.481540,      0.435310,      0.103960,
		      0.334090,      0.206530,     -0.393100,      0.238560,      0.246470,
		      0.478570,     -0.553310,     -0.059770,      0.077070,     -0.225630,
		      0.278370,     -0.005370,     -0.121380,      0.746490,      0.191690,
		     -0.050690,      0.050750,      0.292520,      0.158420,     -0.067560,
		      0.140820,      0.206520,     -0.416460,     -0.328090,     -0.104070,
		      0.869340,      0.660580,      0.654140,      0.346730,      0.465030,
		      0.959460,      0.633530,      0.758820,      0.828790,      0.021130,
		     -0.405950,      0.202840,      0.062880,     -0.415690,     -0.723130,
		     -0.078780,      0.270690,      0.149680,     -0.301250,     -0.758260,
		     -0.616620,      0.706120,      1.722010,      0.087140,     -0.794730,
		     -0.623230,     -0.970130,      0.689470,      1.144200,      0.799580,
		      0.174110,     -0.732400,     -1.925450,     -1.737000,     -0.137410,
		      0.060910,      0.392570,     -0.191500,     -0.127190,      0.558890,
		      2.019130,      1.451040,     -0.890460,     -1.343900,      0.001060,
		      0.313650,     -0.318490,      1.364490,      1.057770,      0.489140,
		      0.263690,      0.336330,      0.253920,      0.706310,     -0.856930,
		     -0.371490,      0.126030,      0.664740,      0.415080,      0.066200,
		     -0.339580,      0.782550,      0.069470,      0.261680,     -0.643620,
		      0.519330,      0.496460,      0.617060,      0.078250,      0.991460,
		      0.273830,      0.107500,     -0.718220,     -0.526510,     -0.460820,
		      0.076940,      0.013100,     -0.502110,     -0.307670,     -1.015070,
		     -0.427370,     -0.217650,     -0.358610,      0.176390,      0.197820,
		      0.342370,      0.253990,      0.151290,     -1.120240,     -0.237800,
		     -0.181620,     -0.075510,     -0.617220,      0.411500,      0.745110,
		      0.365510,     -1.403920,     -0.157850,      0.388830,     -0.587170,
		      0.060940,      0.516560,      0.642570,      0.506710,     -0.636050,
		     -0.202630,      0.365630,      1.153280,      0.860510,     -0.162750,
		     -1.248980,     -0.606610,      1.386600,      0.897690,      0.651520,
		     -0.274750,     -0.939820,     -1.060220,     -0.607980,      0.014900,
		      0.854900,      1.409020,      0.923890,      0.128610,     -0.379450,
		     -1.168540,      0.354520,     -0.377120,     -1.024700,     -1.072260,
		     -0.086440,      1.130340,      1.386620,     -0.363520,      0.119590,
		     -0.197640,     -0.674340,     -1.019670,     -0.227670,     -0.642540,
		     -0.102770,     -0.679310,      0.887070,      0.380840,      0.842390,
		     -0.189840,     -0.659640,     -0.204170,      0.143320,      0.332100,
		     -0.511290,      0.039150,      0.462080,      0.187070,     -0.794260,
		      0.464680,      0.900440,      0.669770,     -0.178140,     -0.061180,
		      0.126100,      0.224080,     -0.338570,      0.158490,      0.802860,
		      0.186110,      0.180050,      0.686230,      0.002430,     -0.515850,
		      0.379120,      0.657380,     -0.043020,     -0.598970,     -0.539540,
		     -0.258820,      0.386740,      0.121240,      1.132560,     -0.224250,
		      0.558250,     -0.276190,     -0.746890,      0.166520,     -0.091590,
		      0.082570,      0.028010,     -0.721010,      0.317150,      0.282940,
		      0.075440,     -0.150510,     -1.004760,      0.526250,      0.881510,
		     -0.838510,     -0.818130,     -0.613260,     -0.360050,      0.916510,
		      0.659510,     -0.680050,     -0.262720,     -0.777950,     -0.772230,
		     -0.403490,      1.032770,      1.384140,      0.606780,     -0.091230,
		     -1.433430,     -1.645560,     -0.240240,      0.174210,      1.259760,
		     -0.270650,     -0.069200,      0.140230,      1.210230,      1.488800,
		     -0.821600,     -1.297880,     -0.281030,      0.498680,      0.114070,
		      0.421170,      1.430020,      0.764520,      0.518440,      0.123740,
		      0.708580,      0.056850,     -0.240610,     -0.760050,     -0.117570,
		      0.174990,      0.577950,      0.500770,     -0.832400,      0.160340,
		      0.268020,      0.298490,     -0.256910,      0.503510,      0.869060,
		     -0.638090,     -0.481770,      0.658020,      0.921040,     -0.235660,
		     -0.517260,      0.212770,     -0.113830,     -0.940250,     -0.291840,
		     -0.707150,     -0.561130,     -0.358240,      0.232750,     -0.278520,
		     -0.412620,      0.113650,     -0.376850,      0.414130,     -0.109410,
		     -0.280040,     -0.234070,     -0.525440,      0.802230,      0.437590,
		      0.087370,      0.037870,     -0.083610,     -0.289920,      0.739550,
		     -0.928920,      0.100010,      0.833500,     -0.168090,     -0.221750,
		     -0.248040,      0.124080,      1.304730,      0.049530,     -1.489060,
		     -0.427410,      1.213860,      0.805530,      0.331320,     -0.428570,
		     -0.370640,      0.442090,      0.508350,      0.420810,      1.130190,
		      1.268670,     -0.253940,     -0.930170,     -0.702070,      0.172650,
		      0.720390,      0.652320,      0.256190,      0.138180,     -0.147210,
		     -0.476570,     -0.136340,     -0.555280,     -0.656460,     -0.777900,
		      0.174890,      1.021920,      1.118680,     -0.270860,     -0.532340,
		     -0.913230,     -0.124830,     -0.637900,     -0.128810,     -0.310710,
		     -0.244910,     -0.994520,      0.340170,     -0.155470,      0.811550,
		      0.589400,     -0.187400,     -0.871620,      0.302620,      0.094250,
		      0.275840,     -1.043760,      0.395240,      0.150280,      0.159700,
		     -0.247520,      0.777680,     -0.317970,     -0.341350,     -0.004070,
		      0.342100,     -0.937300,     -0.604210,      0.262320,      0.091910,
		      0.445070,     -0.423070,     -0.127580,      0.063330,      0.689570,
		     -0.150360,      0.000430,      0.822930,     -0.237350,     -0.536530,
		      0.545840,      0.205340,     -0.033360,      0.412120,      0.181190,
		     -0.687750,     -0.787370,     -0.057800,     -0.028200,     -0.310630,
		      0.410140,      0.318010,     -0.055100,     -0.987510,     -0.399360,
		      0.774330,      0.296540,     -0.394860,     -0.602000,     -0.422090,
		      1.234200,      1.090080,     -0.990050,     -0.986600,     -0.396280,
		      0.503480,      0.907810,     -0.485680,     -0.189930,      0.495230,
		     -0.379880,     -1.040270,     -1.129220,      0.153400,      0.645490,
		     -0.046220,     -0.070310,     -0.279240,     -0.868400,     -0.851440,
		     -0.565200,      1.108670,      0.072060,      1.074600,      0.085300,
		     -0.061780,      0.376090,     -0.558470,     -0.753760,      0.061600,
		      0.614650,      0.837640,     -0.026080,      0.480070,      0.252630,
		      0.468210,      0.265040,      1.108570,     -0.198830,     -0.257770,
		     -0.593670,     -0.443710,     -0.152160,      0.884100,      0.630900,
		     -0.570750,     -0.434690,      0.492220,     -0.092680,     -1.206880,
		      0.088910,      0.954320,      0.302240,     -0.869290,      0.291980,
		      0.044370,      0.135210,      0.101150,      0.725200,      0.474280,
		     -0.568150,     -0.289230,     -0.081810,     -0.193010,      0.072810,
		     -0.327980,      0.582880,      0.473840,     -0.317040,     -0.219620,
		     -0.058900,      0.043680,     -0.069090,     -0.098540,     -0.067330,
		      0.414240,      0.043100,     -0.113580,      0.228820,      0.026060,
		     -0.385800,     -0.393670,      0.508500,     -1.456060,     -0.744520,
		      0.663440,      0.599810,     -0.918350,     -0.308260,      0.289130,
		      0.300710,      0.798230,     -0.671590,     -1.861180,      0.618030,
		      1.484550,      0.331640,     -0.788710,     -1.101820,      0.963960,
		      0.756110,     -0.810310,     -0.589480,      0.848310,      1.393640,
		     -0.075970,     -0.306540,      0.217660,      0.596670,      0.649000,
		      0.805970,      0.252870,      0.494860,     -0.312710,      0.024500,
		     -0.345110,     -0.938500,     -0.156940,      0.075130,      0.285210,
		      0.399180,      0.423760,      0.056810,     -0.454500,     -0.568770,
		     -0.539170,     -0.297270,     -0.340190,     -0.425290,     -0.032680,
		     -0.049260,     -0.049230,     -0.035020,      0.131730,      0.411200,
		     -0.149830,     -1.126540,     -0.395060,      0.481390,      0.379840,
		     -0.932500,     -0.242840,      0.457160,     -0.064780,     -0.402530,
		      0.318510,      0.198880,     -0.645510,      0.089850,     -0.251850,
		     -0.445610,     -0.403480,      0.563320,      0.121890,      0.176200,
		     -0.384150,     -0.296680,      0.191330,     -0.514750,     -0.454460,
		     -0.175970,      0.501570,     -0.320830,     -0.081110,     -0.275800,
		     -0.134900,      0.129010,      0.433290,      0.733420,     -0.038700,
		     -0.172420,      0.750470,      0.672770,      0.176580,      0.513870,
		     -0.139760,      0.574800,      1.244610,     -0.831090,     -1.139140,
		      0.209430,      0.926540,      0.336770,     -0.631390,     -1.002940,
		      0.087170,      1.090150,      0.803170,     -1.142090,     -0.670070,
		      0.533670,      1.111080,     -0.418830,     -1.397620,     -0.222000,
		      1.071470,      0.077930,     -0.602930,     -0.071020,      0.210000,
		     -0.538330,     -1.065940,     -0.345220,      0.331140,     -0.216800,
		     -0.926960,     -0.691190,     -0.298650,     -0.199980,      1.254240,
		      0.669500,     -0.136440,     -0.047270,     -0.585970,     -0.240680,
		      0.138550,      0.358850,      0.965030,     -0.329790,      0.421210,
		      0.266080,      0.326140,     -0.315890,      0.860000,     -0.231280,
		      0.275910,      0.168900,      0.029990,     -0.102630,      1.067310,
		      0.891700,      0.338250,     -0.230840,      0.724820,     -0.060950,
		     -0.991840,     -0.789570,      0.159860,     -0.248030,      0.136650,
		     -0.375140,      0.065740,      0.157120,      0.206950,      0.366660,
		      0.370080,     -0.057270,      0.063520,      0.403790,      0.111380,
		      0.015750,      0.380950,      0.417110,      0.264520,      0.225720,
		     -0.086550,      0.252010,      0.024710,      0.114430,      0.187820,
		     -0.346210,     -0.046110,     -0.061300,     -0.161920,      0.093760,
		      0.684810,      0.110580,      0.326000,      0.120790,      1.183230,
		     -0.301750,     -1.129750,     -0.306670,      1.067850,      0.490760,
		     -1.196370,     -0.676850,      0.749350,      0.798720,      0.519270,
		     -1.036740,     -1.248210,      1.100750,      1.153620,      0.075930,
		     -1.205920,     -0.101780,      1.435100,      0.595630,     -1.047520,
		     -0.750280,      0.019450,      0.244710,     -0.576460,      0.151250,
		      0.736680,      0.215390,     -0.395380,     -0.118570,      0.270640,
		      0.537150,     -0.129070,      0.264100,     -0.438250,     -0.798640,
		     -0.259640,      0.180150,      0.451840,     -0.004280,     -0.295830,
		     -0.207800,     -0.358220,     -0.040240,      0.045920,     -0.222660,
		     -0.054400,      0.187030,      0.039550,      0.212660,     -0.458790,
		     -0.474000,     -0.447810,     -0.085860,     -0.347400,     -1.168290,
		     -0.864390,      0.057240,      0.267680,     -0.652220,     -0.224070,
		      0.006600,      0.552050,     -0.221150,      0.065250,     -0.327940,
		     -0.184870,     -0.124940,      0.374670,     -0.255970,     -0.033000,
		      0.555630,      0.105430,      0.443620,      0.312630,      0.227210,
		      0.233290,     -0.187910,      0.157900,     -0.311930,     -0.206940,
		     -0.054650,     -0.113210,      0.421750,     -0.135810,     -0.492610,
		      0.097100,      0.714600,      0.263690,     -0.133220,      0.559220,
		      0.414570,      0.387270,      0.508760,      0.002450,     -0.328310,
		     -0.644800,      0.808660,      0.384280,     -0.730230,     -0.861100,
		      0.154920,      0.730840,     -0.258840,     -0.637500,     -0.468970,
		      0.407160,      0.849340,     -0.364040,     -1.324800,      0.052140,
		      1.476210,      0.669680,     -0.781350,     -1.129880,      0.054490,
		      0.797780,      0.050520,     -0.471880,      1.106260,      0.430380,
		     -0.394360,     -1.023480,     -0.203290,      1.090180,      0.463600,
		     -0.221810,      0.063100,     -0.166410,     -0.503460,      0.299040,
		      0.414080,      0.139350,     -0.166680,     -0.013980,      0.008470,
		      0.410360,      0.340750,      0.253080,     -0.512970,      0.069610,
		      0.486600,      0.088780,     -0.489000,      0.030540,      0.350770,
		      0.443290,      0.737280,      0.607450,      0.170150,      0.565950,
		      0.449230,      0.174860,     -0.136130,      0.734790,      0.223930,
		      0.381140,     -0.359500,     -0.258290,     -0.107240,      0.045450,
		      0.484190,     -0.576640,     -0.238350,      0.054690,      0.399960,
		     -0.011910,      0.281020,      0.039180,      0.027940,      0.048590,
		      0.127880,      0.163410,     -0.198780,      1.010080,      0.491490,
		     -0.098280,     -0.345860,      0.170960,     -0.422670,      0.059500,
		     -0.322530,     -0.508020,     -0.297300,     -0.115520,     -0.145050,
		      0.248870,      0.352390,      0.385870,      0.591710,      0.603770,
		      0.232810,     -0.075630,     -0.219870,     -1.046790,      0.270650,
		      1.049940,      0.229520,     -0.722860,     -0.022760,      0.552840,
		      0.441520,     -0.237830,     -0.511510,     -0.346260,      1.122920,
		      0.257410,     -1.049850,     -0.560560,      0.724030,      1.351880,
		      0.249030,     -0.414300,     -0.015630,      0.142530,     -0.798840,
		     -0.742920,      0.206520,      1.194140,      0.446190,     -0.982570,
		     -0.804830,     -0.204120,     -0.295810,     -0.073610,      0.443760,
		     -0.192500,     -0.580540,     -0.308650,      0.043260,      0.373510,
		      0.238830,     -0.324220,     -0.258040,     -0.210970,      0.046750,
		     -0.277760,     -0.052420,     -0.471760,      0.079120,      0.094070,
		      0.389860,     -0.291190,     -0.689140,     -0.523130,     -0.469790,
		     -0.519430,      0.133230,     -0.670890,      0.141440,     -0.459640,
		     -0.698020,     -0.651720,      0.413800,      0.452280,      0.248610,
		     -0.198650,     -0.296950,     -0.168470,      0.642480,      0.297100,
		     -0.157860,     -0.012620,      0.450330,      0.178890,      0.011210,
		     -0.169550,      0.052750,      0.142590,      0.328390,     -0.101400,
		     -0.350410,      0.024340,      0.027950,      0.450160,      0.621230,
		      0.240620,     -0.059800,     -0.182750,      0.305520,     -0.107530,
		     -0.176280,     -0.408420,     -0.566450,     -0.402500,     -0.020800,
		      0.173220,     -0.191810,      0.365220,     -0.856700,      0.554880,
		      0.422970,      0.104590,     -0.560490,     -0.470800,      0.697890,
		      0.145310,     -0.779330,     -0.416080,     -0.166720,      0.491850,
		      0.644240,     -1.005310,     -0.681830,      0.877770,      0.769310,
		     -0.379460,     -1.342150,     -0.356830,      0.196550,     -0.001790,
		     -0.080740,     -0.098520,      0.863550,      0.331930,     -0.694680,
		     -0.740970,      0.114600,      0.834810,      0.383740,     -0.050100,
		      0.119350,      0.106810,     -0.174610,      0.022200,      0.407730,
		     -0.029080,     -0.258390,     -0.181370,     -0.077600,      0.400730,
		      0.189390,      0.311020,     -0.254050,      0.324240,      0.603410,
		      0.296660,     -0.309010,     -0.293070,      0.327550,      0.425790,
		      0.450770,      0.761940,      0.443040,      0.070140,      0.148700,
		      0.148120,      0.558670,      0.681230,      0.433130,      0.229100,
		     -0.193600,     -0.038210,     -0.423500,     -0.075900,     -0.620270,
		     -0.269530,     -0.356770,      0.428720,      0.133630,     -0.077450,
		      0.339420,      0.183810,      0.393630,     -0.237120,     -0.377420,
		     -0.073030,      0.134700,      0.629240,      0.181720,      0.107230,
		     -0.254030,     -0.234770,     -0.063740,      0.082990,      0.057080,
		     -0.479400,      0.161570,     -0.164220,      0.068340,      0.198680,
		     -0.018760,     -0.216620,      0.028540,      0.694120,      0.091460,
		      0.309400,      0.252030,     -0.117850,     -0.541820,     -1.224500,
		      0.270040,      0.484130,     -0.469280,     -0.254010,      0.503570,
		      0.318320,      0.094600,     -0.112970,     -0.434570,      0.503130,
		      1.123750,     -0.618820,     -1.013250,      0.081840,      0.915160,
		      0.846650,      0.138520,      0.034010,      0.391650,      0.531340,
		     -0.629400,     -0.589290,      0.151900,      0.833930,      0.682920,
		     -0.529890,     -0.220710,     -0.087460,     -0.383400,     -0.203870,
		      0.695320,      0.320010,     -0.073170,     -0.366310,      0.269820,
		      0.268820,      0.122110,     -0.434140,     -0.200080,     -0.058850,
		     -0.095950,     -0.347670,     -0.084830,     -0.569570,     -0.385320,
		      0.110840,      0.320680,     -0.206320,     -0.466770,     -0.585890,
		     -0.534340,     -0.673790,     -0.017190,     -0.111850,     -0.217260,
		     -0.406820,     -0.279100,     -0.341970,     -0.255490,      0.242240,
		      0.496070,      0.621210,      0.069130,      0.225050,      0.120450,
		     -0.278130,     -0.380890,      0.397480,     -0.038880,      0.047420,
		     -0.002700,      0.119790,     -0.157730,     -0.230630,     -0.120370,
		     -0.320150,      0.023150,      0.001830,     -0.193520,     -0.187810,
		     -0.074260,     -0.212900,     -0.353080,     -0.202610,     -0.207380,
		     -0.158170,     -0.374570,      0.184550,     -0.275230,     -0.702130,
		     -0.099870,     -0.247620,     -0.209470,     -0.046430,      0.269120,
		     -0.547890,     -0.295300,      0.091740,      0.640630,     -0.062230,
		     -0.343200,      0.001020,      0.497850,     -0.266500,      0.243940,
		     -0.071790,     -0.356440,     -0.182010,     -0.079760,     -0.866610,
		      0.232110,      1.132860,      0.589150,     -0.759480,     -1.418200,
		     -0.032750,     -0.142250,     -0.411780,     -0.103880,      0.487510,
		      0.873270,     -0.040480,     -0.564950,     -0.539530,      0.241100,
		      0.473920,      0.386610,      0.291950,      0.091370,     -0.502820,
		     -0.453740,     -0.235250,      0.381630,      0.203580,      0.100950,
		     -0.048090,      0.073450,     -0.048280,     -0.197900,      0.000080,
		      0.215710,      0.188640,      0.478790,      0.235840,     -0.139650,
		     -0.044910,      0.652970,      0.394000,      0.031000,      0.649030,
		      0.916520,     -0.078140,     -0.049800,     -0.045050,      0.526210,
		      0.460280,      0.276630,      0.538690,      0.197560,     -0.350510,
		     -0.336620,     -0.323290,      0.175230,     -0.234410,     -0.097460,
		     -0.187010,     -0.023430,     -0.340480,     -0.060580,      0.038100,
		      0.095350,      0.449760,     -0.282720,      0.249490,      0.120930,
		      0.088840,      0.181760,      0.503160,      0.004720,     -0.230320,
		     -0.384580,     -0.199960,      0.332280,     -0.076030,     -0.083820,
		      0.272330,      0.011340,      0.221720,      0.627280,     -0.243590,
		     -0.014850,      0.103240,     -0.139500,     -0.362940,      0.009740,
		     -0.484470,      0.480220,      0.046500,     -0.162930,     -0.411640,
		      0.382110,     -0.021150,     -0.597510,      0.382330,     -0.116550,
		     -0.223420,      0.118210,     -0.112310,     -0.220270,      0.465340,
		     -0.020340,     -0.944120,     -0.763340,      0.467140,      0.912560,
		      0.311340,      0.369690,      0.025400,      0.126710,     -0.201400,
		     -0.508080,     -0.015150,      0.320630,      0.405460,     -0.272400,
		     -0.372420,     -0.123380,     -0.109760,     -0.362600,     -0.116710,
		      0.416340,      0.460640,     -0.290940,     -0.163730,      0.169360,
		      0.088050,      0.408790,      0.191340,      0.103430,      0.108210,
		     -0.045440,     -0.523110,      0.282830,     -0.400740,     -0.534700,
		     -0.547310,     -0.473080,     -0.268170,     -0.388050,     -0.439190,
		     -0.554350,     -0.534020,     -0.187350,     -0.258500,     -0.312190,
		     -0.682480,     -0.137060,     -0.126490,     -0.175750,     -0.242090,
		      0.015090,     -0.159320,     -0.074940,      0.294850,      0.715630,
		     -0.055910,     -0.559100,     -0.254870,     -0.002380,      0.010050,
		     -0.202040,      0.137850,      0.135310,     -0.242710,     -0.332000,
		      0.074260,     -0.038360,     -0.489340,     -0.118870,      0.120710,
		     -0.189660,     -0.533750,     -0.485450,     -0.479170,      0.124490,
		     -0.042190,     -0.368120,      0.200280,     -0.089680,      0.207730,
		      0.200860,     -0.055250,     -0.087620,      0.513040,     -0.227820,
		     -0.121540,      0.089260,      0.372190,     -0.623170,     -0.042370,
		      0.738400,     -0.324970,      0.005510,      0.101910,     -0.746280,
		     -0.160740,      0.685910,      0.464730,     -0.150040,     -0.251160,
		     -0.071160,     -0.117580,      0.609740,      0.934360,     -0.146050,
		     -0.639090,     -0.475880,     -0.065400,     -0.143680,     -0.485440,
		     -0.048630,      0.483530,      0.502310,     -0.233070,     -0.471020,
		     -0.157850,      0.185840,      0.019780,      0.218720,      0.491230,
		      0.515850,      0.023770,     -0.186530,     -0.070560,      0.275550,
		     -0.153720,     -0.029120,      0.038930,      0.196610,     -0.081460,
		     -0.084270,     -0.150230,      0.149000,     -0.073730,      0.221900,
		      0.239530,      0.460110,      0.505030,      0.494020,      0.450210,
		      0.093450,      0.287490,      0.865380,      0.159790,     -0.089690,
		      0.088520,      0.215460,      0.120350,     -0.191070,     -0.082120,
		      0.032930,     -0.010610,     -0.489530,     -0.219190,     -0.283290,
		     -0.159970,     -0.261930,      0.433090,      0.067580,     -0.217930,
		     -0.571340,      0.002880,      0.119010,      0.168270,     -0.003380,
		      0.238430,      0.386260,     -0.034810,      0.434860,      0.301940,
		     -0.167030,     -0.558380,     -0.249930,     -0.171610,      0.454580,
		     -0.121260,     -0.369110,      0.399780,     -0.093910,     -0.033010,
		     -0.262220,      0.226960,      0.165560,      0.046690,      0.039780,
		      0.278280,      0.376480,      0.537820,     -0.605650,     -0.522450,
		      0.123290,     -0.271910,     -0.450260,      0.089760,     -0.008520,
		     -0.043110,      0.543410,      0.586600,     -0.687420,     -0.425720,
		      0.381170,      0.522090,      0.150960,      0.037160,     -0.385230,
		     -0.767200,     -0.324260,      0.419240,      0.226790,     -0.067240,
		      0.341230,      0.288340,      0.098220,     -0.282030,     -0.421190,
		      0.131740,      0.785920,      0.459100,     -0.210100,     -0.205390,
		      0.007680,     -0.242990,     -0.387670,     -0.122910,     -0.059050,
		      0.252510,     -0.058650,      0.155260,     -0.069930,     -0.526980,
		     -0.087620,      0.298540,      0.018960,      0.267350,      0.087350,
		     -0.384920,      0.121640,     -0.144550,     -0.260410,     -0.360360,
		     -0.585780,     -0.531270,     -0.333810,     -0.674350,     -0.698180,
		     -0.127410,     -0.006890,     -0.004630,     -0.148060,     -0.417420,
		     -0.175690,      0.427620,      0.149310,      0.280120,      0.144270,
		      0.119980,     -0.361600,      0.188670,      0.274870,     -0.160500,
		     -0.478140,     -0.064320,     -0.004510,     -0.181580,     -0.294320,
		     -0.165750,      0.027590,      0.052380,     -0.109170,      0.096430,
		      0.164380,     -0.140330,     -0.281410,     -0.237080,     -0.005320,
		     -0.072010,     -0.283830,      0.175100,      0.241410,     -0.129220,
		      0.230780,      0.000240,      0.192950,     -0.460170,      0.198280,
		      0.277590,     -0.051650,      0.061350,      0.266310,      0.031750,
		     -0.401220,     -0.542200,      0.924720,     -0.130400,     -0.314370,
		      0.132660,      0.307720,     -0.050830,      0.614820,     -0.089730,
		     -0.883770,      0.050740,      0.348190,     -0.273400,     -0.687160,
		     -0.415140,     -0.033670,      0.085240,      0.739140,      0.430830,
		     -0.055190,     -0.352280,     -0.287560,     -0.039680,     -0.003600,
		     -0.015590,      0.202030,      0.179740,      0.058430,     -0.321410,
		     -0.420960,     -0.019340,      0.223570,     -0.223940,     -0.346460,
		     -0.136520,      0.061690,      0.006630,     -0.089320,      0.169080,
		      0.035240,     -0.065180,      0.283540,     -0.029320,     -0.159610,
		     -0.044840,      0.192470,      0.005580,      0.375370,      0.051250,
		     -0.037030,     -0.030650,      0.315900,      0.541360,      0.760290,
		      0.396690,      0.480950,      0.409490,      0.410690,      0.191640,
		      0.187790,      0.424610,      0.137840,      0.014530,     -0.776810,
		     -0.527370,     -0.099740,     -0.159920,     -0.319540,     -0.527230,
		      0.067910,     -0.237290,     -0.080330,      0.171720,      0.114830,
		     -0.139390,     -0.128570,      0.070130,     -0.025750,      0.232640,
		     -0.209970,      0.099100,      0.281090,      0.145530,      0.322030,
		      0.344400,      0.057910,     -0.330780,     -0.516830,     -0.167020,
		      0.248020,     -0.052900,     -0.456940,     -0.113560,      0.278840,
		     -0.068560,      0.075200,      0.065970,     -0.171090,      0.034530,
		      0.059740,     -0.320930,      0.317600,     -0.017320,      0.873520,
		      0.125550,     -0.769520,     -0.231530,      0.743920,      0.007690,
		     -0.549660,     -0.017270,     -0.619810,     -0.037510,      0.877500,
		      0.046370,     -0.662580,     -0.315980,      0.885090,      0.563280,
		     -0.050080,     -0.240590,     -0.249840,     -0.288520,     -0.004230,
		      0.472870,      0.184220,     -0.043890,      0.155920,     -0.050000,
		      0.146290,     -0.155050,     -0.330190,      0.207270,      0.188030,
		      0.343820,     -0.350880,      0.098000,      0.554360,      0.406840,
		      0.075910,     -0.033140,     -0.439330,     -0.178930,     -0.217880,
		      0.410320,     -0.089100,     -0.493470,     -0.145870,      0.155920,
		      0.024090,     -0.094300,     -0.142560,     -0.101290,     -0.026300,
		     -0.042640,     -0.157850,     -0.291580,     -0.503560,     -0.019900,
		     -0.131740,     -0.266450,     -0.508240,     -0.171390,      0.107920,
		     -0.337730,      0.281680,     -0.044950,      0.299760,      0.268520,
		      0.033530,      0.203550,      0.664790,      0.656910,      0.068510,
		     -0.171260,      0.046430,     -0.251350,      0.124680,      0.091120,
		      0.067420,     -0.011860,      0.058100,     -0.216480,     -0.235390,
		     -0.159670,     -0.101380,      0.162530,      0.196640,     -0.231020,
		     -0.220370,     -0.174030,      0.203780,      0.009110,      0.146200,
		      0.079880,      0.224260,      0.343200,      0.127470,      0.096500,
		     -0.318870,     -0.051220,      0.123200,     -0.095100,      0.061060,
		      0.298820,      0.116920,     -0.118670,     -0.150120,     -0.531350,
		     -1.185830,      0.566490,      0.387710,     -0.734180,     -0.139590,
		      0.410190,     -0.103020,      0.304130,      0.264030,     -0.416900,
		     -0.267360,      0.602910,      0.298970,     -0.715690,     -0.575600,
		      0.177950,      0.552860,      0.143050,      0.166810,      0.174910,
		     -0.049650,     -0.202290,     -0.099990,     -0.241840,     -0.224940,
		      0.011120,      0.426240,      0.353500,     -0.081800,     -0.375650,
		     -0.281020,      0.349040,     -0.077700,     -0.337480,     -0.374540,
		     -0.096750,     -0.053960,      0.012730,     -0.140870,      0.060710,
		     -0.433540,      0.008740,      0.423310,      0.299090,     -0.127220,
		     -0.420930,      0.027450,     -0.017500,      0.045110,      0.136200,
		      0.251950,      0.186600,      0.134820,      0.367170,      0.401390,
		     -0.040220,      0.021230,      0.438970,      0.435470,     -0.057140,
		      0.030890,     -0.028250,      0.055390,     -0.085990,     -0.161550,
		     -0.159820,     -0.127400,     -0.295430,     -0.560940,      0.078710,
		      0.276740,      0.344730,     -0.177680,     -0.130090,     -0.168720,
		      0.212460,      0.159800,      0.197910,      0.209740,      0.053500,
		      0.167300,      0.050840,      0.020370,     -0.132190,      0.124380,
		      0.102780,      0.110840,     -0.382120,      0.285340,     -0.168750,
		      0.006810,     -0.035240,     -0.430840,     -0.203210,      0.255720,
		      0.261960,      0.163940,      0.162700,      0.160590,     -0.039470,
		     -0.056140,      0.231570,      0.014870,     -0.548440,     -0.108670,
		      0.469950,      0.733890,     -0.216460,     -0.845170,      0.187230,
		      0.532180,     -0.000240,     -0.226190,     -0.040040,     -0.372750,
		      0.261120,      0.542530,     -0.379950,     -0.666800,      0.214770,
		      0.598290,      0.232200,     -0.488970,     -0.219820,     -0.079500,
		     -0.240390,     -0.216980,      0.365550,      0.153910,      0.081940,
		      0.167130,     -0.125600,     -0.222650,     -0.277020,      0.130930,
		      0.579760,      0.399790,      0.039490,     -0.291870,     -0.159700,
		      0.487070,      0.344290,      0.241660,      0.135390,      0.097470,
		      0.146540,     -0.057160,      0.192430,      0.078250,     -0.248790,
		     -0.220430,      0.105110,      0.176440,     -0.078430,     -0.111040,
		     -0.125120,     -0.293860,     -0.070960,     -0.092130,     -0.152420,
		     -0.530600,      0.101570,      0.162080,     -0.094810,     -0.133880,
		      0.279350,      0.221040,      0.143810,      0.222790,      0.182450,
		      0.176370,      0.159000,      0.080930,      0.011010,      0.007480,
		      0.025690,     -0.241870,      0.006710,      0.203640,     -0.108380,
		      0.100970,     -0.039880,     -0.077860,      0.093080,      0.103120,
		     -0.047630,     -0.119530,     -0.254570,     -0.035370,      0.121440,
		     -0.204820,     -0.051120,     -0.000030,     -0.051970,     -0.170010,
		      0.235720,      0.072630,     -0.034840,     -0.036190,     -0.152520,
		     -0.003610,     -0.128100,     -0.082520,     -0.085850,      0.104790,
		      0.035690,     -0.135070,     -0.159850,      0.055670,      0.438270,
		     -0.223970,     -0.202330,      0.047870,     -1.450750,      0.270530,
		      0.701430,     -0.123880,     -0.674270,      0.246230,      0.288740,
		     -0.069890,      0.370520,      0.081580,     -0.537810,      0.104600,
		      0.470650,     -0.037690,     -0.796220,     -0.548950,      0.409230,
		      0.611880,     -0.005130,      0.128850,      0.160470,     -0.281390,
		      0.002650,      0.163920,     -0.015210,     -0.313850,      0.112400,
		      0.286520,      0.298900,     -0.305380,     -0.473800,      0.010940,
		      0.502500,     -0.087600,     -0.422110,     -0.521170,     -0.081520,
		      0.223210,      0.001900,     -0.165030,     -0.114890,     -0.458510,
		     -0.081450,      0.070750,      0.366700,      0.144460,     -0.264650,
		     -0.080090,     -0.006250,     -0.092920,     -0.005990,     -0.161690,
		      0.103470,     -0.077250,      0.200830,      0.224740,     -0.156770,
		     -0.225710,      0.116490,      0.288780,     -0.003760,     -0.110330,
		     -0.206660,     -0.212750,     -0.183560,     -0.101700,     -0.071350,
		     -0.006940,      0.143240,     -0.004670,      0.044300,      0.165410,
		      0.001190,      0.099150,     -0.083180,      0.090720,     -0.118030,
		     -0.058370,      0.467630,     -0.046400,     -0.145650,     -0.015620,
		     -0.088620,      0.031700,     -0.069140,     -0.121640,     -0.374690,
		      0.047910,     -0.258130,      0.013880,      0.011400,     -0.031520,
		     -0.114910,     -0.187790,     -0.009210,     -0.052440,      0.195230,
		     -0.014700,      0.247150,      0.196000,      0.379360,      0.239930,
		     -0.035610,      0.162290,      0.198680,     -0.226610,     -0.280840,
		     -0.047840,      0.467570,      0.182150,     -0.736210,      0.004400,
		      0.321180,      0.073100,     -0.189440,     -0.152580,     -0.126970,
		     -0.275150,      0.264340,      0.233830,     -0.421380,     -0.137170,
		      0.717020,      0.584410,     -0.192530,     -0.558750,      0.170330,
		      0.192760,     -0.415400,     -0.175130,      0.032440,     -0.086540,
		     -0.085220,      0.373830,     -0.015040,     -0.313910,     -0.450060,
		      0.155300,      0.551600,      0.256120,     -0.284270,     -0.118470,
		      0.305190,      0.491120,      0.123260,     -0.251900,     -0.198420,
		      0.234930,      0.331150,      0.325820,      0.295580,      0.192560,
		      0.047570,      0.175250,      0.258670,      0.195020,      0.002070,
		      0.044450,      0.134570,     -0.132900,     -0.276860,     -0.258070,
		     -0.042940,     -0.255570,     -0.089660,     -0.260170,     -0.053400,
		     -0.294950,     -0.096580,      0.003590,     -0.046800,      0.358560,
		      0.273490,      0.324170,      0.196670,      0.068150,     -0.024970,
		     -0.194610,     -0.170040,     -0.073310,     -0.098680,      0.396680,
		     -0.068890,      0.214910,      0.126420,      0.076640,     -0.199950,
		     -0.099840,      0.070750,      0.022950,     -0.102040,     -0.130830,
		     -0.123910,     -0.030770,     -0.003360,     -0.062200,     -0.063440,
		     -0.293060,      0.312710,      0.227760,      0.190860,      0.270650,
		      0.058570,     -0.080510,     -0.160840,     -0.095950,     -0.288510,
		      0.024300,      0.056530,      0.018140,      0.022360,      0.103930,
		      0.097800,     -0.029160,     -0.189650,     -0.049040,      0.478650,
		     -0.918980,     -0.346370,      0.462700,      0.088820,     -0.507540,
		     -0.215240,      0.316830,      0.226890,     -0.004100,      0.380830,
		      0.180010,     -0.306780,      0.108220,      0.163280,     -0.399750,
		     -0.722500,     -0.036430,      0.624620,     -0.021970,     -0.159630,
		      0.400910,      0.140550,     -0.199640,      0.179740,      0.155540,
		     -0.180660,     -0.165390,      0.055890,      0.498010,      0.166090,
		     -0.179420,     -0.412440,      0.214350,      0.451580,     -0.013350,
		     -0.335840,     -0.375610,      0.017310,      0.180020,     -0.099950,
		     -0.233140,     -0.286820,     -0.457500,     -0.082630,     -0.082630,
		      0.031550,     -0.033620,     -0.239860,     -0.013950,      0.023480,
		      0.043990,      0.068400,      0.042070,      0.220610,      0.008010,
		     -0.008660,      0.043480,      0.272530,     -0.212280,      0.160570,
		      0.413230,      0.160600,      0.223450,     -0.099720,     -0.364910,
		     -0.297120,      0.098850,     -0.112680,     -0.042210,     -0.009940,
		      0.194590,      0.058180,      0.006450,     -0.156480,      0.072290,
		      0.045190,     -0.043940,     -0.182470,      0.045000,      0.386090,
		     -0.003650,     -0.352260,      0.092430,     -0.004860,     -0.146240,
		     -0.084820,     -0.102570,     -0.125230,      0.160210,     -0.059880,
		     -0.247970,     -0.132400,     -0.211620,     -0.052000,      0.064860,
		      0.076620,      0.180480,     -0.128820,      0.044050,     -0.064900,
		     -0.173130,      0.044070,      0.104490,     -0.002850,     -0.023380,
		      0.070780,      0.294370,     -0.130640,      0.050810,     -0.140940,
		      0.077540,      0.362080,     -0.558510,     -0.278560,      0.485340,
		      0.415630,     -0.217530,     -0.079800,      0.082680,     -0.095640,
		     -0.551580,      0.149350,     -0.017740,     -0.214610,      0.156230,
		      0.820430,      0.278560,     -0.410890,     -0.272280,      0.083100,
		     -0.096830,     -0.186490,      0.197810,     -0.231450,     -0.259070,
		     -0.069190,      0.297450,      0.066810,     -0.458790,     -0.330480,
		      0.072620,      0.414870,      0.031860,     -0.550030,     -0.172780,
		      0.330640,      0.522210,      0.045020,     -0.161370,     -0.143230,
		      0.023200,      0.199330,      0.299080,      0.064670,     -0.040500,
		     -0.057010,      0.119520,      0.203960,      0.022230,     -0.047310,
		      0.070120,      0.053330,      0.010180,     -0.206940,     -0.364270,
		     -0.101680,      0.044590,     -0.152900,     -0.058680,     -0.072000,
		     -0.062200,     -0.339970,     -0.275100,      0.046230,      0.262470,
		      0.343780,     -0.038810,      0.140380,      0.058120,      0.017010,
		     -0.125670,     -0.208220,     -0.179370,     -0.127610,      0.232200,
		      0.144070,      0.051280,      0.119480,     -0.229080,     -0.377590,
		      0.220250,     -0.039700,      0.010610,     -0.149570,     -0.160800,
		      0.070030,     -0.056510,      0.210370,     -0.178640,      0.290370,
		      0.168940,      0.080240,      0.109750,      0.063320,      0.276500,
		      0.245660,      0.079960,     -0.119900,     -0.039880,     -0.092920,
		     -0.165200,     -0.146710,     -0.096700,      0.139640,     -0.111140,
		     -0.067730,     -0.073380,     -0.096450,      0.119070,     -0.188810,
		      0.249190,     -0.559450,     -0.151790,      0.224590,      0.347660,
		     -0.281660,     -0.373490,     -0.073630,      0.078870,      0.047390,
		     -0.235710,      0.590490,      0.272720,     -0.107110,      0.117100,
		      0.044300,     -0.690220,     -0.363340,      0.444950,      0.344850,
		     -0.185460,     -0.056790,      0.099140,     -0.264170,     -0.084050,
		      0.212880,      0.094250,     -0.248780,     -0.350140,      0.170180,
		      0.508380,      0.136310,     -0.132040,     -0.173680,      0.436790,
		      0.252690,     -0.266710,     -0.204950,     -0.062870,      0.191480,
		      0.145550,     -0.141600,     -0.096690,     -0.163060,     -0.171420,
		     -0.007190,      0.019050,      0.037830,     -0.186800,     -0.064020,
		     -0.043590,     -0.128370,     -0.181440,      0.019610,      0.141020,
		      0.392030,     -0.054460,     -0.080390,     -0.024920,      0.027900,
		      0.064470,     -0.026250,      0.022560,      0.093620,     -0.045750,
		     -0.081530,     -0.227990,      0.072280,      0.173000,      0.056570,
		     -0.110740,      0.055430,      0.245030,      0.143390,     -0.046550,
		     -0.528490,      0.068900,      0.136250,      0.151020,      0.144250,
		      0.203390,      0.017020,      0.061520,     -0.114710,     -0.010290,
		      0.152390,     -0.213280,     -0.037170,     -0.091020,      0.138880,
		      0.002990,     -0.003010,     -0.056300,      0.161440,      0.047510,
		     -0.107910,      0.099420,      0.080290,     -0.021410,     -0.066690,
		     -0.110600,      0.051290,      0.031500,     -0.011940,     -0.097160,
		      0.112660,     -0.120070,      0.030430,     -0.173250,     -0.078780,
		      0.288410,      0.162920,      0.110690,     -0.005100,      0.211730,
		     -0.328490,     -0.369160,      0.009090,      0.415340,      0.178930,
		     -0.220490,      0.023350,      0.451980,     -0.148110,     -0.525110,
		     -0.158750,     -0.065770,     -0.042430,      0.419950,      0.479670,
		     -0.118060,     -0.234100,      0.292180,      0.083740,     -0.211340,
		      0.175440,      0.299990,     -0.186320,     -0.405850,      0.082830,
		      0.429370,      0.143200,     -0.380440,     -0.306180,     -0.030420,
		      0.254130,     -0.164140,     -0.282340,     -0.052420,      0.017140,
		     -0.150380,     -0.263860,     -0.254860,     -0.046360,      0.256200,
		      0.138440,      0.117550,     -0.027130,     -0.054760,     -0.010580,
		      0.130340,      0.126590,      0.024330,      0.077890,      0.014390,
		     -0.060410,      0.091820,     -0.060320,     -0.145690,     -0.072550,
		      0.010700,     -0.117900,     -0.127000,      0.094140,      0.254490,
		      0.138910,     -0.216850,     -0.081000,      0.211590,     -0.065180,
		     -0.083940,     -0.056960,     -0.052050,      0.046030,     -0.119550,
		     -0.045920,     -0.068880,      0.088130,     -0.012130,     -0.028950,
		     -0.041200,     -0.005730,     -0.190530,     -0.235200,     -0.089410,
		     -0.052230,     -0.088760,     -0.129240,      0.099010,     -0.074340,
		     -0.131140,      0.011950,     -0.187490,      0.215090,      0.348040,
		      0.062440,      0.092920,      0.133010,      0.100510,     -0.105520,
		     -0.019760,     -0.113280,      0.028140,     -0.029700,     -0.071040,
		     -0.257460,     -0.006920,     -0.121260,     -0.049370,      0.045290,
		      0.006940,     -0.145310,      0.097090,      0.152890,     -0.027030,
		      0.284920,     -0.596750,     -0.339560,      0.170880,      0.286270,
		     -0.122510,     -0.050350,      0.087200,      0.157360,     -0.032580,
		     -0.343340,     -0.115370,      0.489270,      0.235980,     -0.170740,
		      0.110740,     -0.216620,     -0.458710,      0.086280,      0.346090,
		     -0.293060,     -0.177670,      0.130250,     -0.015300,     -0.351290,
		     -0.058060,      0.340720,      0.115240,     -0.401430,     -0.226110,
		      0.233700,      0.308300,     -0.026690,     -0.067480,     -0.021490,
		      0.366080,      0.198950,     -0.033450,      0.182640,      0.154180,
		      0.302550,      0.069590,     -0.215290,     -0.223230,     -0.125830,
		      0.014570,      0.090310,     -0.017140,     -0.053910,     -0.167740,
		     -0.014960,      0.227010,      0.058020,     -0.037230,     -0.261730,
		      0.117630,      0.118350,     -0.036850,      0.000030,      0.033760,
		      0.108420,     -0.055470,     -0.228210,      0.003600,     -0.008430,
		      0.052330,     -0.231660,     -0.124110,      0.052770,      0.108010,
		     -0.013540,     -0.060740,     -0.177710,      0.012130,     -0.067250,
		     -0.170860,     -0.461220,     -0.099430,      0.019620,     -0.173630,
		      0.082510,      0.284140,      0.102120,      0.038690,     -0.224630,
		     -0.065800,      0.072340,     -0.030650,      0.040910,      0.027740,
		      0.135560,     -0.072170,     -0.138840,     -0.122270,      0.028080,
		      0.172150,      0.032990,      0.298170,     -0.048250,     -0.221030,
		     -0.492380,     -0.195050,      0.186360,      0.264250,      0.255740,
		     -0.022150,     -0.004130,      0.067950,      0.149810,      0.016360,
		     -0.335520,      0.200790,      0.071690,      0.097030,     -0.204870,
		      0.041560,      0.379200,     -0.387490,     -0.169610,     -0.003300,
		      0.143340,      0.033650,     -0.081690,     -0.007680,      0.276050,
		      0.339270,     -0.326800,     -0.375950,      0.124830,     -0.111680,
		      0.027990,      0.444630,      0.042240,     -0.479670,      0.089170,
		      0.394200,      0.004090,     -0.070690,      0.266690,      0.186690,
		     -0.242330,     -0.469480,      0.182560,      0.298750,      0.179130,
		     -0.111680,      0.086910,      0.013770,     -0.132850,     -0.279020,
		     -0.112640,      0.205900,     -0.042290,     -0.335990,     -0.335660,
		     -0.185610,      0.111100,      0.313050,      0.237030,     -0.012980,
		     -0.145320,     -0.098190,     -0.117940,      0.068990,      0.091580,
		     -0.181840,     -0.051120,      0.055000,     -0.057040,      0.000250,
		     -0.096300,      0.101030,     -0.096320,     -0.173470,     -0.156360,
		      0.006980,     -0.035220,      0.139970,      0.078370,      0.153250,
		      0.129990,      0.243430,      0.132870,     -0.066500,     -0.185100,
		     -0.103420,      0.092610,      0.096720,      0.077000,     -0.024410,
		      0.202740,     -0.047940,     -0.100400,      0.041920,      0.032790,
		     -0.055160,     -0.199040,      0.085610,      0.190700,      0.046340,
		     -0.287580,      0.060840,      0.039130,      0.039370,     -0.058300,
		     -0.123190,      0.139010,      0.311060,      0.120280,      0.105490,
		      0.182310,     -0.054420,      0.034890,      0.033030,     -0.016940,
		     -0.097320,      0.105330,     -0.054150,      0.229990,      0.106090,
		      0.138100,     -0.095510,     -0.060200,      0.037290,      0.001960,
		      0.007620,      0.029660,      0.266530,      0.073000,      0.343730,
		     -0.416060,     -0.282110,      0.100020,      0.242880,     -0.161690,
		     -0.328340,     -0.004310,      0.143890,      0.209010,     -0.202170,
		     -0.232320,      0.232630,      0.389180,     -0.007700,      0.021350,
		     -0.034820,     -0.265430,     -0.102290,      0.422090,      0.169990,
		     -0.402770,     -0.214250,      0.184430,     -0.050420,     -0.288830,
		      0.011120,      0.433330,     -0.097760,     -0.389180,     -0.238830,
		      0.092180,      0.049470,      0.058810,      0.104470,      0.129640,
		     -0.046780,     -0.190690,     -0.032180,      0.339030,      0.196040,
		      0.104840,     -0.020900,     -0.177580,     -0.300700,     -0.082860,
		      0.246490,      0.267490,      0.320460,      0.052440,     -0.129920,
		      0.043420,      0.044540,      0.171640,      0.105600,     -0.054220,
		      0.055710,     -0.129680,     -0.306050,     -0.060150,      0.131650,
		      0.031070,      0.006550,     -0.147310,     -0.025190,      0.026770,
		     -0.147210,     -0.251440,     -0.011080,      0.188990,      0.111050,
		     -0.027770,     -0.194580,     -0.214450,     -0.168060,      0.057810,
		      0.046610,     -0.101640,     -0.038560,     -0.070100,     -0.226100,
		     -0.239260,     -0.023010,     -0.165380,     -0.104490,     -0.014980,
		      0.069760,     -0.027130,     -0.046290,     -0.042420,     -0.075740,
		      0.140820,     -0.209360,     -0.024250,     -0.090380,     -0.056240,
		      0.207360,      0.111510,      0.243880,     -0.024040,      0.040480,
		     -0.191780,     -0.303400,      0.060420,      0.254880,      0.160020,
		      0.131260,      0.005420,      0.081570,     -0.001950,      0.118190,
		     -0.258460,     -0.348850,     -0.064610,     -0.003440,      0.306670,
		     -0.122550,     -0.138700,      0.426680,     -0.256090,     -0.335890,
		      0.078360,      0.242530,      0.046690,     -0.161150,     -0.219240,
		      0.189200,      0.288490,      0.045600,     -0.390610,     -0.115730,
		      0.171480,      0.051510,      0.222750,      0.216710,     -0.249830,
		     -0.498090,      0.191030,      0.157440,     -0.085430,     -0.037600,
		      0.275870,      0.121670,     -0.197880,     -0.005340,      0.392220,
		      0.173870,     -0.072080,      0.005680,      0.125110,      0.147350,
		     -0.095650,     -0.025480,      0.009830,      0.173880,     -0.067810,
		     -0.166330,     -0.127490,      0.080040,      0.076790,      0.223910,
		      0.080000,     -0.062400,     -0.232870,     -0.246290,     -0.170630,
		      0.087720,      0.124230,     -0.159250,     -0.157090,     -0.063930,
		     -0.042110,     -0.010760,      0.067150,      0.207010,      0.149070,
		     -0.247940,     -0.144450,     -0.115240,      0.095730,      0.017920,
		      0.082100,      0.009620,      0.274220,      0.176810,      0.039850,
		      0.077970,     -0.150710,     -0.054480,      0.179040,      0.030550,
		      0.049650,      0.052560,      0.359500,      0.256430,      0.028640,
		      0.031510,      0.052150,     -0.059060,      0.021180,      0.049840,
		      0.059800,     -0.099900,     -0.054160,      0.014970,      0.089520,
		      0.096720,     -0.085950,      0.046350,     -0.061260,      0.145430,
		      0.177050,      0.069050,      0.213880,     -0.181080,      0.107130,
		     -0.013460,      0.091300,      0.106350,     -0.017540,      0.124180,
		      0.061830,      0.127820,      0.274580,      0.057590,      0.199480,
		      0.029250,      0.036330,     -0.121260,      0.107920,     -0.111520,
		     -0.213310,      0.064630,      0.385340,     -0.425260,     -0.357080,
		      0.121950,      0.419560,     -0.032750,     -0.254820,     -0.097760,
		      0.068380,      0.246140,     -0.136360,     -0.368350,     -0.167140,
		      0.289870,      0.096390,     -0.025050,     -0.110740,     -0.191600,
		     -0.182240,      0.055090,      0.520270,      0.089270,     -0.149800,
		     -0.123620,      0.107730,     -0.016110,     -0.228490,     -0.000700,
		      0.190340,     -0.176610,     -0.131490,      0.017800,     -0.012560,
		     -0.121110,     -0.056310,      0.092050,      0.100710,     -0.096290,
		     -0.340420,     -0.095670,      0.193100,      0.093300,     -0.151550,
		     -0.063160,     -0.133690,     -0.349570,     -0.136540,     -0.018370,
		      0.141660,      0.289950,     -0.044450,      0.003440,      0.068380,
		      0.041360,      0.048330,      0.108460,     -0.069140,      0.051030,
		     -0.186930,     -0.290060,      0.008120,      0.126760,      0.115370,
		     -0.073500,     -0.132140,      0.131270,      0.047850,     -0.107020,
		     -0.233990,      0.016110,     -0.072060,      0.022420,     -0.074250,
		      0.063850,     -0.138940,      0.038790,     -0.089180,     -0.155740,
		     -0.086550,      0.024900,      0.159410,      0.106890,     -0.093900,
		     -0.167290,     -0.291610,     -0.094140,      0.073600,      0.149780,
		      0.015940,     -0.046900,     -0.138210,     -0.013000,      0.015250,
		     -0.088370,      0.131010,      0.122340,     -0.064020,     -0.031420,
		      0.328430,     -0.095570,      0.058420,     -0.088620,     -0.087960,
		      0.041560,      0.140860,      0.074850,      0.153860,     -0.035190,
		      0.138280,      0.141180,     -0.021220,      0.174550,     -0.034080,
		     -0.184440,      0.127200,     -0.038490,      0.047860,     -0.040610,
		     -0.305770,     -0.090500,      0.355580,     -0.131360,     -0.411540,
		     -0.029940,      0.323140,      0.017450,     -0.032290,     -0.095150,
		      0.078250,      0.249290,      0.029900,     -0.281280,     -0.162310,
		      0.026070,      0.088720,      0.197490,      0.259000,     -0.003600,
		     -0.432920,     -0.258160,      0.152540,      0.208650,     -0.109070,
		     -0.103480,      0.129530,      0.001050,     -0.243240,      0.106820,
		      0.238390,      0.105100,     -0.072160,      0.053590,     -0.048530,
		      0.010460,     -0.049620,      0.166340,      0.214700,      0.009330,
		     -0.120750,      0.022190,      0.157240,      0.136090,      0.128620,
		      0.196040,      0.213070,     -0.013850,     -0.256110,     -0.290930,
		     -0.137130,     -0.023920,     -0.056070,     -0.079690,     -0.083730,
		     -0.050210,     -0.158000,     -0.078390,     -0.025560,      0.272090,
		     -0.001080,     -0.126680,     -0.086730,     -0.058070,      0.065270,
		     -0.013810,     -0.073230,     -0.015150,      0.265160,      0.177880,
		      0.205160,      0.095260,      0.153210,     -0.090080,      0.079250,
		     -0.054470,      0.049030,      0.173260,      0.231660,      0.191200,
		      0.069050,     -0.075210,     -0.068250,     -0.034590,      0.130280,
		      0.196520,     -0.210310,     -0.114070,      0.107850,      0.082690,
		      0.031970,     -0.072660,     -0.248030,      0.069440,     -0.050300,
		      0.035500,      0.108270,      0.186220,      0.199510,      0.094090,
		     -0.161960,     -0.068790,     -0.009660,     -0.109200,     -0.010440,
		      0.125310,      0.098380,      0.174640,     -0.044890,      0.049810,
		      0.149980,      0.013670,     -0.027540,     -0.139440,     -0.108620,
		      0.056250,      0.018210,     -0.067210,     -0.162570,      0.443400,
		     -0.238890,     -0.425180,     -0.069630,      0.434290,      0.021120,
		     -0.207070,     -0.071650,      0.069330,      0.114950,      0.062930,
		     -0.100660,     -0.103030,      0.171070,      0.054450,      0.062850,
		     -0.027320,     -0.134710,     -0.204180,     -0.058200,      0.169330,
		      0.220120,     -0.096500,     -0.021120,     -0.082270,      0.029260,
		      0.000300,      0.058800,      0.114420,     -0.092650,     -0.073940,
		     -0.086360,      0.101570,     -0.005330,      0.055590,      0.017940,
		      0.012700,     -0.003640,     -0.089830,     -0.023050,      0.044990,
		     -0.016650,     -0.054560,     -0.200110,     -0.074560,     -0.137890,
		     -0.302920,     -0.149560,      0.186790,      0.191270,      0.229630,
		     -0.001690,      0.085620,      0.197270,      0.031820,      0.019330,
		      0.041240,     -0.085790,     -0.087470,     -0.193010,     -0.235210,
		      0.110280,      0.114650,      0.066310,     -0.218030,     -0.161880,
		     -0.038470,      0.195240,     -0.158170,     -0.088560,     -0.073390,
		     -0.129430,     -0.270760,     -0.090950,     -0.191040,      0.156680,
		      0.252530,     -0.003230,     -0.163670,     -0.056600,     -0.061240,
		      0.144760,      0.008890,     -0.019880,     -0.203290,     -0.240400,
		     -0.066370,      0.040450,      0.057220,      0.083490,      0.074690,
		     -0.063180,      0.061630,     -0.136250,     -0.080560,      0.038730,
		      0.157720,     -0.001670,      0.037410,      0.113690,      0.169960,
		      0.054400,     -0.223230,      0.040050,      0.068390,      0.031010,
		      0.075910,     -0.139560,     -0.021840,      0.080560,      0.019450,
		      0.108160,      0.008690,     -0.000690,     -0.181040,      0.056970,
		      0.065480,      0.184780,      0.086970,     -0.002240,     -0.343670,
		     -0.216920,      0.410610,      0.010910,     -0.412820,     -0.217170,
		      0.200530,      0.124250,     -0.130780,     -0.038690,      0.040850,
		      0.263880,      0.186920,     -0.234770,     -0.185900,     -0.084010,
		      0.003270,      0.091830,      0.209680,      0.146190,     -0.083550,
		     -0.198080,     -0.083130,      0.106580,      0.141190,     -0.010770,
		     -0.087360,     -0.075140,      0.040130,     -0.033910,      0.062990,
		      0.067710,     -0.016400,      0.055500,      0.020920,     -0.099150,
		     -0.164620,     -0.154820,      0.161900,      0.073570,     -0.125320,
		     -0.047900,      0.125970,      0.252000,      0.120080,      0.068290,
		      0.241730,      0.139620,     -0.092610,     -0.207210,     -0.224360,
		     -0.080370,     -0.084250,     -0.124950,     -0.113280,      0.044440,
		      0.114270,      0.069140,      0.009470,      0.108770,      0.225090,
		      0.017590,     -0.107620,      0.110670,      0.100200,      0.090570,
		      0.059280,     -0.138620,     -0.028470,      0.120530,      0.031900,
		     -0.013960,     -0.062510,      0.155180,      0.041510,     -0.068270,
		     -0.344700,     -0.056530,      0.090300,      0.110390,      0.249400,
		      0.101730,      0.125240,      0.114270,     -0.005910,      0.004840,
		     -0.092810,     -0.202390,     -0.364840,      0.036570,      0.046590,
		      0.134860,     -0.102810,     -0.072630,     -0.016220,     -0.075290,
		     -0.001570,      0.064440,      0.045210,      0.151800,      0.092620,
		     -0.053890,     -0.031340,     -0.145220,      0.071400,     -0.139680,
		     -0.046040,      0.103850,      0.236840,      0.022800,     -0.171080,
		     -0.025820,      0.120100,     -0.250200,      0.041530,     -0.030850,
		      0.040860,      0.194200,      0.105580,      0.225790,      0.034730,
		      0.368350,     -0.069230,     -0.393600,     -0.241910,      0.408530,
		      0.156580,     -0.264260,     -0.100930,      0.122660,      0.039750,
		      0.033740,     -0.191170,     -0.239110,      0.031230,      0.246850,
		      0.089870,      0.054590,      0.015930,     -0.199490,     -0.256440,
		      0.018580,      0.166600,      0.026080,     -0.040450,     -0.071300,
		     -0.054060,     -0.040440,      0.134240,      0.051680,      0.027310,
		     -0.051600,     -0.010720,     -0.094690,     -0.084530,     -0.012530,
		      0.129630,      0.102640,      0.118170,     -0.024020,     -0.072830,
		      0.089260,      0.068610,     -0.171070,     -0.206810,     -0.120650,
		      0.103530,     -0.162260,     -0.294480,     -0.096480,      0.143360,
		      0.203430,      0.201000,      0.039890,      0.141590,      0.132940,
		      0.107720,     -0.035670,     -0.041900,     -0.034040,     -0.099480,
		     -0.279580,     -0.130880,     -0.067830,      0.022810,      0.042390,
		     -0.091600,     -0.122590,     -0.051320,     -0.064950,     -0.079460,
		     -0.067170,      0.023220,      0.096010,     -0.092910,     -0.053140,
		     -0.155700,      0.039220,      0.211000,     -0.092420,     -0.227670,
		     -0.129950,      0.023370,      0.078600,     -0.104170,     -0.102850,
		     -0.021260,     -0.020940,     -0.057800,      0.023590,      0.064950,
		     -0.042310,      0.022830,      0.088350,      0.200320,      0.114830,
		      0.051880,     -0.078330,      0.117270,      0.072000,     -0.056540,
		      0.083750,     -0.047910,      0.011800,      0.040980,      0.024050,
		      0.222370,      0.083810,     -0.121050,     -0.037950,      0.039840,
		      0.130000,      0.129530,     -0.089080,     -0.027320,     -0.030520,
		     -0.128050,     -0.140800,      0.137400,      0.096780,     -0.089220,
		      0.129300,     -0.018420,     -0.283920,     -0.404480,      0.391450,
		      0.158420,     -0.293000,     -0.293140,      0.163580,      0.083270,
		     -0.150760,     -0.114580,     -0.081950,      0.201380,      0.189030,
		      0.083580,     -0.140130,     -0.006560,     -0.157550,      0.027300,
		      0.208490,      0.213810,      0.021960,     -0.149080,     -0.127850,
		     -0.079430,      0.100850,      0.064700,      0.079820,     -0.212380,
		     -0.130120,      0.150640,      0.091270,      0.026130,      0.042830,
		      0.168510,      0.113680,     -0.099540,     -0.076140,     -0.144850,
		     -0.053870,      0.109670,      0.004220,     -0.108890,      0.068950,
		      0.121210,      0.110970,     -0.147410,      0.055090,      0.262310,
		      0.144990,     -0.116660,     -0.096460,     -0.172200,     -0.029510,
		     -0.051640,     -0.130670,     -0.124920,     -0.029280,      0.119420,
		     -0.003970,      0.070630,      0.190970,      0.265820,      0.057260,
		     -0.062050,     -0.002200,      0.082130,      0.064540,      0.138760,
		      0.037150,      0.088010,      0.041290,     -0.027360,     -0.170410,
		     -0.095670,     -0.103480,      0.021000,      0.015770,     -0.137830,
		     -0.050680,      0.001880,      0.010260,      0.096440,     -0.049260,
		      0.092420,      0.004290,      0.099390,      0.019190,     -0.152200,
		     -0.065460,     -0.197100,     -0.105850,     -0.008750,     -0.067030,
		     -0.209590,     -0.127750,     -0.094340,     -0.007850,     -0.054810,
		     -0.166550,      0.008020,      0.056480,      0.189680,     -0.000820,
		     -0.036050,     -0.141290,     -0.172380,     -0.003150,     -0.092850,
		     -0.035490,      0.049050,     -0.036450,     -0.074210,      0.002060,
		      0.042050,     -0.051750,     -0.004090,      0.040090,     -0.077420,
		     -0.039510,      0.235930,      0.007090,     -0.053240,      0.173100,
		      0.260550,      0.169110,     -0.233680,     -0.380020,      0.243270,
		      0.258580,     -0.143340,     -0.218310,      0.182100,      0.169710,
		      0.045210,      0.027480,     -0.125170,     -0.164210,     -0.025010,
		      0.017650,      0.081940,      0.078540,      0.028610,     -0.140870,
		     -0.106100,      0.041560,      0.155370,      0.139500,     -0.069090,
		     -0.083920,     -0.087830,      0.121040,      0.048200,     -0.050130,
		     -0.076570,     -0.037070,      0.071440,     -0.122210,     -0.106680,
		      0.068090,      0.090400,      0.069690,      0.031000,     -0.067390,
		     -0.011090,      0.105000,      0.053310,     -0.059140,     -0.087230,
		      0.059360,      0.038720,     -0.258890,     -0.241570,     -0.119100,
		     -0.017220,      0.095990,      0.054280,      0.096070,      0.097370,
		      0.128790,      0.043530,     -0.093510,     -0.087980,     -0.048230,
		     -0.161650,     -0.256650,     -0.049650,      0.051790,     -0.029910,
		      0.072020,      0.013850,     -0.040710,      0.049570,     -0.014110,
		     -0.088040,     -0.078030,     -0.102360,      0.102580,     -0.011940,
		      0.076940,     -0.161190,      0.048710,      0.131660,     -0.016330,
		     -0.134890,     -0.025300,      0.007120,      0.191200,     -0.026960,
		     -0.124930,     -0.145130,      0.011690,     -0.085910,      0.057720,
		     -0.053110,     -0.010230,      0.140540,      0.103320,      0.080340,
		      0.138230,     -0.036160,     -0.064510,      0.079710,     -0.025680,
		      0.094770,      0.054490,      0.124210,      0.130990,      0.182300,
		      0.153800,      0.133100,      0.117580,      0.033520,      0.060960,
		      0.039510,      0.042570,      0.084760,      0.104070,      0.012800,
		     -0.071570,     -0.008370,      0.051360,      0.043290,      0.027340,
		     -0.009720,      0.101680,      0.000180,     -0.017460,     -0.153060,
		     -0.503570,      0.187190,      0.341080,     -0.275090,     -0.305900,
		      0.099910,      0.211840,     -0.076650,     -0.141120,      0.001040,
		     -0.034590,      0.059730,     -0.003810,      0.088050,      0.093560,
		     -0.134010,     -0.214270,     -0.047770,      0.134710,      0.198520,
		      0.078600,     -0.161370,     -0.138920,     -0.064790,      0.145030,
		      0.172450,     -0.009160,     -0.168680,     -0.030550,      0.131940,
		     -0.037920,     -0.128430,     -0.011590,      0.062930,      0.066220,
		     -0.035010,      0.002970,     -0.019000,     -0.011080,      0.056290,
		     -0.043620,     -0.155500,      0.011160,      0.068120,     -0.080100,
		     -0.141490,      0.078920,      0.203360,      0.036070,     -0.145750,
		     -0.058310,     -0.087430,     -0.175870,     -0.117940,     -0.162490,
		     -0.064270,     -0.035850,      0.086870,      0.066680,      0.035440,
		      0.093330,      0.071320,      0.130470,      0.034280,     -0.062100,
		     -0.063320,      0.038260,      0.001090,      0.060710,      0.147760,
		      0.108440,      0.025200,     -0.108510,     -0.043710,     -0.076460,
		      0.091020,      0.033750,     -0.066410,      0.015300,      0.081000,
		     -0.044660,     -0.012760,      0.005700,     -0.057220,      0.103610,
		      0.157080,      0.078010,     -0.057200,      0.028980,     -0.039140,
		     -0.012960,     -0.205310,      0.046890,     -0.071000,      0.018690,
		      0.016970,      0.038570,     -0.124900,     -0.095330,     -0.178950,
		     -0.068500,     -0.040990,      0.005500,      0.051370,     -0.096050,
		      0.081210,      0.007600,      0.006770,      0.035410,     -0.036830,
		     -0.006220,     -0.142340,      0.012410,      0.154070,      0.064930,
		      0.061010,     -0.087840,     -0.004740,     -0.045890,      0.017680,
		      0.073590,     -0.084650,     -0.008950,      0.038470,      0.156000,
		      0.103090,      0.018740,     -0.398240,      0.121900,      0.208570,
		     -0.040920,     -0.264580,     -0.072000,      0.206480,      0.059120,
		      0.004260,     -0.019710,      0.019710,     -0.033130,     -0.096890,
		      0.049970,      0.178690,      0.073430,     -0.064370,     -0.136530,
		     -0.152230,     -0.021010,      0.125420,      0.099790,     -0.090630,
		     -0.175970,     -0.035660,      0.188990,      0.065520,     -0.030810,
		     -0.018850,      0.137920,      0.146120,     -0.025710,     -0.075850,
		      0.029330,      0.055820,      0.046620,      0.042130,     -0.051200,
		      0.020230,      0.061040,      0.016210,      0.006820,      0.095890,
		      0.093250,     -0.088170,     -0.199470,     -0.051840,     -0.028640,
		     -0.010070,      0.104960,      0.040310,      0.131330,      0.099080,
		      0.015990,     -0.013470,     -0.164350,     -0.032940,      0.010180,
		     -0.050770,     -0.111640,     -0.090680,     -0.071880,      0.040170,
		      0.112070,      0.052260,     -0.000730,      0.003600,     -0.066680,
		     -0.003740,     -0.044520,      0.012730,     -0.065330,      0.125840,
		     -0.005470,     -0.010660,     -0.078800,     -0.019220,     -0.033930,
		     -0.019710,      0.067380,     -0.009700,     -0.017300,      0.000400,
		     -0.094210,     -0.071030,     -0.044510,     -0.095110,      0.055270,
		      0.023780,     -0.017940,      0.059070,      0.056360,      0.060430,
		      0.033280,     -0.108140,     -0.045120,      0.006290,     -0.127920,
		      0.014790,      0.031290,      0.119550,      0.018560,      0.089930,
		     -0.051390,      0.004950,      0.151920,     -0.041380,      0.130410,
		      0.037430,     -0.126720,      0.070690,     -0.049300,      0.136340,
		      0.009460,      0.126480,     -0.017590,      0.034740,      0.047150,
		      0.020480,     -0.043580,     -0.093690,      0.140350,     -0.012940,
		      0.013670,     -0.439260,     -0.101260,      0.362080,      0.027930,
		     -0.369120,     -0.015360,      0.239420,      0.039810,     -0.136280,
		     -0.018120,      0.113530,      0.033590,     -0.026390,     -0.023640,
		      0.131330,      0.071770,     -0.131750,     -0.090720,      0.022760,
		      0.044090,      0.120480,      0.158440,      0.003760,     -0.131620,
		     -0.105310,      0.130730,      0.065770,     -0.156090,     -0.027770,
		      0.062730,      0.129560,     -0.123540,     -0.115060,     -0.025830,
		     -0.049120,     -0.094570,     -0.099290,     -0.001060,     -0.057760,
		     -0.122540,      0.045460,     -0.005260,     -0.070400,      0.024980,
		     -0.014970,     -0.050930,     -0.009400,      0.112900,      0.111840,
		     -0.055930,     -0.187600,     -0.081630,     -0.036590,     -0.230830,
		     -0.106940,     -0.071610,     -0.076060,     -0.038080,      0.002550,
		      0.026860,     -0.009150,     -0.010380,      0.019260,      0.088040,
		     -0.027590,     -0.146210,     -0.096380,      0.019800,      0.004030,
		      0.122460,      0.039570,      0.008890,     -0.022210,     -0.102040,
		     -0.117080,      0.015320,      0.117080,      0.177070,      0.095220,
		     -0.021210,      0.068920,     -0.107530,      0.037420,     -0.041680,
		      0.070620,      0.035690,      0.104290,      0.027870,      0.005150,
		      0.139610,      0.093170,     -0.098330,     -0.153050,      0.111210,
		      0.088910,      0.020280,      0.051980,     -0.073690,     -0.016600,
		     -0.077270,     -0.067060,      0.094920,     -0.063810,      0.019130,
		      0.140620,      0.014470,      0.017320,     -0.041780,      0.170510,
		      0.009630,      0.015160,      0.070310,     -0.120550,      0.032680,
		     -0.035580,      0.064530,     -0.035770,     -0.004400,      0.022690,
		     -0.095660,      0.109960,      0.023470,     -0.048440,     -0.019110,
		     -0.030320,      0.067170,     -0.044520,      0.111500,      0.100630,
		     -0.309330,     -0.010460,      0.241070,      0.052870,     -0.157960,
		     -0.062200,      0.070930,      0.117770,     -0.067290,     -0.020760,
		      0.001660,      0.046340,     -0.148980,     -0.081950,      0.181850,
		      0.115140,      0.046170,     -0.020120,     -0.034420,     -0.259580,
		     -0.117400,      0.050520,      0.094170,     -0.054680,     -0.025580,
		      0.079960,      0.054480,     -0.042330,     -0.119860,      0.042950,
		      0.103910,      0.038530,      0.085030,      0.096670,      0.050490,
		      0.027870,      0.062390,      0.135640,     -0.052220,     -0.041740,
		     -0.027630,     -0.032880,      0.051270,      0.131890,      0.017300,
		     -0.095810,     -0.093780,      0.020990,      0.135870,      0.106080,
		      0.089730,      0.166910,      0.141160,      0.149260,      0.100030,
		      0.047570,     -0.141860,      0.016190,      0.113070,      0.046950,
		      0.049390,     -0.108180,      0.012230,      0.048590,      0.147570,
		      0.081820,     -0.023520,     -0.134770,     -0.115770,     -0.048980,
		     -0.038860,      0.028990,      0.067860,     -0.023900,      0.046580,
		      0.025740,     -0.102000,     -0.039230,     -0.025960,      0.088040,
		     -0.043470,     -0.024770,     -0.028370,     -0.058940,      0.121250,
		      0.007110,      0.052560,     -0.112790,      0.016460,     -0.120300,
		      0.016670,     -0.176640,     -0.016190,     -0.049370,      0.121650,
		      0.062480,     -0.031810,      0.095650,      0.065710,     -0.068010,
		      0.051600,      0.076270,     -0.045480,     -0.114540,     -0.155260,
		     -0.004820,     -0.128230,      0.028550,      0.027700,      0.007610,
		     -0.038440,     -0.113930,     -0.066820,      0.095160,      0.052000,
		      0.087270,      0.026650,     -0.018760,     -0.036840,     -0.039650,
		     -0.009760,      0.000720,     -0.120080,      0.103220,     -0.042520,
		      0.105750,     -0.311100,     -0.094680,      0.099800,      0.082650,
		     -0.269720,     -0.095290,      0.169960,      0.119430,     -0.044610,
		     -0.091620,      0.095960,      0.019040,     -0.020350,     -0.028540,
		      0.099240,      0.034740,     -0.075150,     -0.110090,     -0.015700,
		      0.093640,      0.015600,      0.199950,      0.135980,      0.059210,
		     -0.068540,     -0.039980,     -0.011860,     -0.076840,     -0.124060,
		      0.136350,      0.248160,      0.104090,     -0.146090,     -0.088660,
		     -0.035800,     -0.112440,     -0.084160,     -0.060990,      0.008940,
		     -0.185320,     -0.111570,      0.057190,      0.074150,     -0.037130,
		     -0.039350,     -0.097340,     -0.002790,      0.060250,      0.129650,
		      0.048370,     -0.080160,     -0.099560,     -0.037760,     -0.103220,
		     -0.223260,     -0.128490,     -0.100900,     -0.019820,      0.027730,
		     -0.032560,     -0.033390,     -0.040640,     -0.037240,      0.008660,
		     -0.006370,     -0.098300,     -0.157280,     -0.111760,      0.023870,
		      0.005690,      0.035540,      0.029110,      0.027560,     -0.051080,
		     -0.093420,     -0.122150,     -0.086580,     -0.000980,      0.162140,
		      0.109460,      0.047590,      0.015160,      0.025550,      0.039630,
		     -0.030270,     -0.039690,     -0.078490,      0.028480,     -0.045000,
		     -0.009170,      0.161100,      0.086190,      0.006030,      0.025680,
		      0.056180,      0.179540,      0.024420,     -0.183880,     -0.073790,
		     -0.202990,     -0.055130,      0.054970,      0.035470,     -0.085540,
		     -0.082640,     -0.010190,      0.024950,      0.010910,     -0.054780,
		      0.002700,      0.100980,     -0.024070,      0.104580,     -0.031180,
		     -0.001550,     -0.008200,     -0.088950,     -0.055860,     -0.010410,
		      0.014340,      0.002820,      0.033260,      0.070620,      0.066300,
		     -0.030890,     -0.075320,      0.003010,     -0.083520,     -0.069270,
		     -0.055760,      0.133070,     -0.270970,     -0.111080,      0.138410,
		      0.095710,     -0.179920,     -0.078900,      0.014770,      0.120930,
		     -0.028000,      0.035350,      0.024500,      0.070320,     -0.052790,
		     -0.163340,      0.116390,      0.136070,      0.024970,     -0.033010,
		      0.063810,     -0.069580,     -0.123160,     -0.054070,     -0.013960,
		     -0.065970,      0.052180,      0.078720,      0.138780,     -0.087480,
		     -0.220770,     -0.163340,      0.162400,      0.094620,      0.010170,
		      0.097380,      0.100260,      0.057980,     -0.025680,      0.098940,
		      0.113930,     -0.059130,     -0.117040,     -0.032910,      0.024760,
		      0.053180,      0.006110,     -0.073250,     -0.149640,     -0.115680,
		     -0.012940,      0.065860,      0.111150,      0.140330,      0.204000,
		      0.152880,      0.152900,      0.091520,      0.047200,     -0.026140,
		      0.011850,      0.059570,      0.095520,      0.084800,      0.006240,
		     -0.039010,      0.013410,      0.076040,      0.077840,     -0.018490,
		     -0.078880,     -0.093210,     -0.035620,      0.022640,      0.102640,
		      0.061370,      0.012100,     -0.016670,     -0.014440,     -0.074230,
		     -0.044670,     -0.027190,      0.015640,     -0.155080,     -0.004450,
		     -0.133790,      0.112760,      0.061390,     -0.028600,      0.035190,
		     -0.005460,      0.076990,      0.017260,     -0.135720,     -0.036610,
		      0.012050,     -0.072100,      0.008510,      0.039380,      0.098030,
		      0.039250,     -0.055610,     -0.031840,      0.077100,      0.174320,
		     -0.045030,     -0.048900,     -0.101440,     -0.063920,     -0.052780,
		     -0.091750,     -0.007650,      0.049840,      0.073000,      0.036610,
		     -0.054110,      0.001300,      0.105380,     -0.047830,     -0.043850,
		      0.016810,      0.076740,     -0.052740,     -0.136500,      0.028430,
		      0.065110,     -0.077120,     -0.112900,      0.013020,      0.126320,
		     -0.243970,     -0.098680,      0.192610,      0.154060,     -0.229650,
		     -0.090020,      0.170200,      0.060750,      0.002030,     -0.107030,
		      0.046600,      0.087020,     -0.079980,     -0.070450,      0.084000,
		      0.083280,     -0.055540,     -0.123410,     -0.034070,     -0.017050,
		     -0.085200,     -0.050400,      0.159710,      0.104630,      0.060950,
		      0.058760,     -0.028320,     -0.071700,     -0.123360,      0.032320,
		      0.218380,      0.289100,     -0.010520,     -0.152190,     -0.059390,
		     -0.053500,     -0.115820,     -0.044180,     -0.006620,     -0.109420,
		     -0.148310,     -0.066020,      0.024930,      0.029390,     -0.024100,
		      0.005600,      0.008200,      0.111770,      0.120320,      0.105120,
		      0.124010,      0.018110,     -0.147660,     -0.128610,     -0.136940,
		     -0.167340,     -0.160450,     -0.055300,     -0.033080,      0.112830,
		     -0.002830,     -0.042410,     -0.068470,     -0.082340,     -0.093980,
		     -0.028330,     -0.096950,     -0.108130,     -0.132560,     -0.059560,
		     -0.047510,      0.051790,      0.054110,      0.017490,     -0.045000,
		     -0.080370,      0.019820,     -0.039010,     -0.059220,      0.094200,
		      0.106910,      0.036590,     -0.060490,      0.146140,      0.108530,
		     -0.018850,     -0.090430,     -0.099650,      0.011760,     -0.102340,
		     -0.027100,     -0.071430,      0.059660,      0.082280,      0.098310,
		     -0.013490,      0.142250,      0.018560,     -0.034170,     -0.112560,
		     -0.141240,      0.032930,      0.053970,      0.018530,     -0.125020,
		      0.009420,     -0.101990,     -0.094420,      0.096630,      0.033960,
		      0.007380,      0.048150,     -0.093460,      0.008480,      0.065780,
		      0.064920,     -0.108390,     -0.098150,      0.065630,      0.007810,
		      0.079900,     -0.094680,     -0.029360,      0.106330,      0.001250,
		     -0.040790,      0.004230,     -0.081570,     -0.108680,      0.004650,
		     -0.052280,     -0.073510,      0.237290,     -0.171250,     -0.202720,
		      0.123570,      0.083320,     -0.193320,     -0.073260,      0.059450,
		      0.046640,     -0.051200,     -0.090610,      0.115380,      0.084740,
		     -0.071530,     -0.071370,      0.034470,      0.159390,      0.076100,
		      0.075120,      0.108620,      0.040010,     -0.132010,     -0.107930,
		      0.015240,     -0.085670,     -0.080390,      0.047980,      0.146260,
		      0.112530,     -0.109790,     -0.280470,     -0.063580,      0.146170,
		      0.068820,     -0.003780,      0.083870,      0.085700,     -0.015220,
		      0.041430,      0.152740,      0.058890,     -0.033050,     -0.018750,
		      0.013380,     -0.041430,     -0.013230,     -0.051960,     -0.108150,
		     -0.132440,     -0.134080,     -0.058580,      0.057470,      0.154380,
		      0.148880,      0.093400,      0.057730,      0.053320,      0.000550,
		     -0.065660,     -0.023120,      0.076000,      0.100270,      0.089800,
		      0.079980,     -0.021130,      0.078240,     -0.039610,      0.076730,
		      0.032250,     -0.057630,     -0.026910,     -0.168770,     -0.024070,
		      0.055490,      0.097140,      0.113040,     -0.000370,     -0.033880,
		     -0.044300,     -0.051480,     -0.007280,      0.036260,     -0.020370,
		     -0.036710,     -0.036220,     -0.007710,      0.152390,      0.117620,
		      0.019170,     -0.042320,      0.032640,      0.110260,     -0.114760,
		     -0.081420,     -0.032930,      0.008770,     -0.065680,     -0.035080,
		     -0.032460,      0.255640,      0.041700,      0.023180,     -0.103160,
		     -0.062160,     -0.031750,      0.019860,      0.049510,     -0.100760,
		     -0.034970,     -0.113910,     -0.054180,     -0.010880,      0.009560,
		      0.056570,     -0.094280,     -0.011840,      0.067860,     -0.110170,
		     -0.030010,     -0.076890,     -0.096400,      0.083810,      0.036700,
		      0.026160,      0.001070,     -0.007040,      0.037200,      0.006620,
		     -0.068350,     -0.076970,      0.104260,     -0.171820,     -0.211840,
		      0.047230,      0.183940,     -0.174240,     -0.097640,      0.198360,
		      0.117090,     -0.064220,      0.009640,      0.056100,      0.085460,
		     -0.080540,     -0.103960,      0.093300,      0.102850,     -0.012350,
		     -0.165310,     -0.110030,      0.021940,     -0.017600,     -0.113250,
		      0.074940,      0.052840,     -0.055800,      0.059480,      0.110500,
		     -0.081120,     -0.120730,     -0.106910,      0.026120,      0.211870,
		      0.182060,     -0.064270,     -0.115700,      0.009660,     -0.054280,
		      0.006500,      0.024930,     -0.085640,     -0.182300,     -0.079980,
		      0.028140,      0.015880,      0.036430,     -0.001580,      0.075270,
		      0.089940,      0.112460,      0.004420,      0.065040,      0.110860,
		      0.055500,     -0.126890,     -0.152450,     -0.127230,     -0.044780,
		     -0.100850,     -0.035530,      0.028460,      0.011960,     -0.090180,
		     -0.033390,     -0.022020,     -0.061480,     -0.051330,     -0.011530,
		      0.016690,      0.045790,      0.006690,      0.050660,     -0.080100,
		      0.048250,      0.075460,      0.006750,     -0.063720,     -0.152800,
		      0.018550,      0.020250,      0.011140,     -0.012310,      0.031200,
		     -0.039590,     -0.070050,     -0.024120,     -0.034490,      0.009490,
		     -0.153650,     -0.073260,     -0.078620,     -0.030490,      0.049740,
		     -0.070110,      0.049740,      0.103970,      0.081330,     -0.032930,
		      0.079340,     -0.023390,     -0.084320,     -0.137250,     -0.025250,
		     -0.064520,      0.005250,      0.025480,     -0.054760,      0.008850,
		     -0.087090,     -0.014630,     -0.030210,      0.084440,     -0.011020,
		      0.009180,     -0.051360,      0.020360,      0.040660,      0.022690,
		      0.039280,      0.019640,      0.046960,      0.038980,      0.126640,
		     -0.051850,     -0.086610,      0.096770,      0.153330,     -0.070790,
		     -0.091530,     -0.041020,      0.117450,      0.046270,      0.086740,
		      0.045290,     -0.136480,      0.129710,     -0.025620,     -0.146310,
		     -0.007060,      0.189600,     -0.167230,     -0.235930,      0.026510,
		      0.062910,     -0.026050,     -0.092870,     -0.002360,      0.136010,
		     -0.002970,     -0.129090,      0.036320,      0.098490,      0.087270,
		     -0.001280,      0.057960,      0.091460,      0.014100,     -0.050840,
		     -0.031940,     -0.030440,     -0.032420,     -0.002920,      0.073540,
		      0.127780,      0.052640,     -0.111600,     -0.205570,     -0.020370,
		      0.107350,     -0.025240,     -0.035970,      0.033310,      0.007870,
		      0.081800,      0.126180,      0.101530,     -0.063450,     -0.033880,
		     -0.048540,      0.000640,     -0.105840,     -0.056260,     -0.102860,
		      0.041170,     -0.036380,     -0.074340,     -0.048870,      0.041910,
		      0.179300,      0.205270,      0.106940,      0.067140,     -0.019170,
		      0.024680,     -0.106650,      0.019510,      0.018470,      0.073590,
		      0.058440,     -0.014070,     -0.109790,     -0.084250,     -0.090730,
		     -0.033910,     -0.059410,     -0.019560,     -0.001560,     -0.116830,
		     -0.059560,     -0.026950,      0.151490,      0.094350,      0.023800,
		     -0.104300,     -0.123160,     -0.100800,      0.031270,      0.124690,
		      0.105250,      0.090010,     -0.003120,      0.037260,      0.154610,
		      0.166250,     -0.090090,     -0.180480,      0.066440,      0.053280,
		      0.066180,     -0.032970,      0.140230,      0.077320,      0.041910,
		     -0.177870,     -0.017330,      0.036880,      0.017200,      0.049470,
		     -0.035080,      0.114460,     -0.048720,      0.080400,      0.005870,
		      0.042580,     -0.056050,     -0.092420,     -0.088850,      0.000730,
		     -0.030730,     -0.010570,     -0.007700,     -0.033110,     -0.038800,
		     -0.025440,     -0.081640,     -0.138550,     -0.009790,      0.126940,
		      0.034200,     -0.027560,      0.027500,      0.024200,     -0.063100,
		     -0.032720,      0.152330,      0.061700,     -0.029790,      0.049700,
		     -0.113030,     -0.092920,     -0.019660,      0.152830,     -0.118200,
		     -0.268880,      0.148930,      0.275800,      0.010800,     -0.084380,
		      0.080080,      0.129150,      0.024810,     -0.096820,     -0.050290,
		      0.110710,      0.098750,     -0.065610,     -0.125110,     -0.016740,
		     -0.015870,     -0.113560,     -0.017930,      0.157050,      0.011930,
		     -0.024680,      0.045170,      0.004980,     -0.082220,     -0.067900,
		     -0.056950,      0.029590,      0.162050,      0.033520,     -0.085490,
		     -0.064090,     -0.001710,      0.001800,     -0.007550,     -0.062280,
		     -0.091130,     -0.159270,      0.036380,      0.029660,      0.049560,
		      0.052560,      0.108910,      0.127770,      0.128210,      0.014080,
		      0.021860,      0.065810,      0.131480,      0.051500,     -0.078180,
		     -0.228650,     -0.077320,     -0.048130,     -0.022850,      0.032260,
		      0.118930,      0.041770,     -0.025780,     -0.044180,     -0.001250,
		     -0.047490,     -0.049690,      0.081160,      0.108270,      0.121990,
		      0.015370,      0.037050,     -0.057430,      0.062580,      0.004040,
		      0.064210,     -0.081780,     -0.064160,     -0.081110,      0.003080,
		      0.066140,     -0.019370,     -0.035070,     -0.104660,     -0.091550,
		     -0.001390,      0.010850,     -0.047870,     -0.003980,     -0.030120,
		     -0.004190,      0.010810,     -0.038470,     -0.022410,     -0.059680,
		     -0.014020,     -0.069630,     -0.012400,      0.009390,      0.074490,
		     -0.003250,     -0.056750,      0.082560,     -0.095800,      0.024150,
		     -0.127480,     -0.063910,      0.024760,      0.109520,      0.018200,
		      0.004190,      0.120920,     -0.038440,     -0.026670,      0.035570,
		      0.016400,     -0.007680,      0.053550,     -0.021180,     -0.016420,
		      0.066340,      0.155040,      0.103630,      0.096580,     -0.063760,
		      0.078630,      0.066520,     -0.012780,     -0.022720,      0.133310,
		     -0.004810,     -0.056820,      0.093220,     -0.048240,
	};
	static const std::vector<double> h = {
		      0.000000,     -0.267510,      0.000000,      0.068490,      1.134440,
		      0.000000,      0.298190,     -0.921890,     -0.382880,      0.000000,
		     -1.465650,     -0.933020,      0.272650,      0.587410,      0.000000,
		      1.816210,      0.985840,     -1.210710,     -1.143330,     -1.047630,
		      0.000000,     -0.754250,      0.046900,      0.035530,      0.749940,
		      1.309010,      0.228950,      0.000000,     -0.526180,      0.504580,
		      0.682830,     -0.271570,      0.210410,     -0.267510,     -0.645290,
		      0.000000,     -0.332440,     -1.957980,     -1.081090,     -0.775590,
		     -0.388290,      1.962780,      1.118030,     -0.306300,      0.000000,
		      0.894310,      1.765890,      0.859970,      0.243060,     -0.156460,
		     -0.335760,     -0.852870,      0.273150,     -1.018950,      0.000000,
		     -0.310290,     -0.734170,     -1.236900,      0.097550,     -0.507940,
		     -1.632090,      0.374700,      0.709370,      0.589310,      0.525070,
		      0.000000,     -0.534770,     -0.356010,      0.558730,      0.430630,
		      0.854860,      1.474990,      0.567820,     -0.441790,     -0.031580,
		     -1.812790,     -1.377820,      0.000000,      0.235570,     -0.835920,
		     -0.416580,     -0.851700,     -0.919910,     -0.425000,     -0.858380,
		     -0.569840,     -0.272860,      1.994160,      1.387880,      1.283110,
		      0.000000,     -0.155880,      1.777680,      0.508360,     -0.494690,
		     -0.689600,     -0.016880,      0.731370,      0.170680,      1.102110,
		      0.159760,     -1.731040,     -1.990210,     -1.224640,      0.000000,
		      0.408260,     -0.450070,     -1.244740,      1.357950,      1.217770,
		     -0.283280,     -0.570220,      0.130560,     -0.085180,      0.403190,
		      2.656620,      0.644620,      0.381350,      0.089800,      0.000000,
		     -2.049830,     -1.566040,      1.508110,      0.429520,     -0.511000,
		     -0.347070,      0.948470,      0.748420,     -0.404370,      0.370390,
		      0.166130,      0.134190,     -0.081710,     -0.904740,      0.376280,
		      0.000000,      1.229430,      0.903710,     -0.466030,     -1.989530,
		      0.353660,      1.854900,     -0.750200,     -1.742650,     -1.245320,
		     -2.177200,     -2.201030,      0.736520,      0.642680,     -0.292190,
		     -0.392270,     -3.038620,      0.000000,      0.906250,     -0.253900,
		     -2.001130,      0.691080,     -0.646500,     -2.308340,     -1.442690,
		      1.579710,      1.623670,      2.296360,      1.384310,     -0.854010,
		     -0.524810,      0.421980,     -0.511050,      1.070610,      0.660990,
		      0.000000,     -2.696320,      0.563020,      1.317830,      1.277630,
		      0.198990,      0.645170,      2.831760,      0.644690,     -0.279440,
		     -1.514480,     -0.935340,     -0.966310,      0.943750,      0.923490,
		      0.866140,      0.263070,     -1.003540,     -1.590460,      0.000000,
		     -0.031680,     -0.970170,      0.504320,     -1.569900,      0.327150,
		      0.312190,     -2.773960,     -2.716940,     -1.635180,      0.100020,
		      1.803370,      1.940970,     -1.272550,      0.008180,     -0.278030,
		     -3.335810,     -0.213420,      0.860300,      1.104690,      0.000000,
		      1.241310,     -1.661710,     -1.194220,     -0.724860,     -0.409690,
		      0.740740,      0.991350,      2.566940,      1.721260,      1.264620,
		     -2.131600,     -0.143060,      0.128790,     -0.107310,      0.924670,
		      1.822130,      1.621320,      0.347480,      0.044640,      0.028040,
		      0.000000,      0.634180,      1.731800,     -0.087340,      0.931870,
		     -0.581630,     -1.083680,      1.031060,     -0.607760,     -0.606690,
		     -1.775230,     -0.576110,     -0.130560,      1.539060,      0.520430,
		      1.864110,      1.339460,     -2.011640,     -1.823030,     -0.941380,
		      0.991760,      2.492490,      0.000000,     -1.305040,     -0.937390,
		     -0.702370,      0.175400,      1.068400,     -0.545910,     -0.764410,
		     -0.034540,      1.402170,      2.450540,      2.671550,     -0.850180,
		     -1.773820,     -0.364000,     -2.335370,     -1.015890,      0.703010,
		      0.979460,      1.959970,     -0.660980,     -1.666370,     -0.087040,
		      0.000000,     -0.570990,     -1.052520,      1.330270,      0.684920,
		      0.635810,      1.532660,     -0.278880,     -0.197880,     -1.573900,
		     -0.422670,     -1.758450,      1.484900,      0.508010,      0.140030,
		      0.523610,      0.680550,      1.817500,      0.469050,     -1.401190,
		     -1.231500,     -0.159950,      2.520550,      1.011890,      0.000000,
		      1.966490,      0.642060,     -2.619350,     -2.783650,     -1.602470,
		     -0.631460,     -0.275680,      0.289340,      1.375790,     -1.071720,
		      0.342520,      0.577400,     -0.608440,     -0.809040,      0.550280,
		     -1.666740,     -2.115170,     -1.214240,      0.419970,      1.969850,
		      0.105460,     -3.060220,     -1.431470,     -0.021020,      0.000000,
		     -0.593000,     -0.245400,      0.014880,      0.037530,      0.487910,
		     -0.078280,      1.698060,      0.866270,     -0.402050,     -0.106530,
		      0.852860,     -1.159430,     -0.095330,      0.552260,     -0.463090,
		      2.344970,      1.695300,      1.273210,      0.436740,     -2.879710,
		     -1.001450,      1.360910,      1.332980,      0.600460,     -1.728810,
		      0.000000,     -0.767800,      0.695890,      1.463500,      0.443260,
		      0.030520,     -0.473530,     -1.682720,     -2.642020,     -0.819640,
		      1.918780,     -0.537480,      0.031700,      0.282410,     -0.095050,
		     -1.716600,     -0.680680,     -0.108000,     -0.975060,     -1.165240,
		     -0.791170,     -0.061300,     -0.055720,     -0.798650,      0.170030,
		      0.744890,     -1.460230,      0.000000,      1.193550,     -1.691050,
		     -0.796680,      0.188160,     -0.715480,     -0.844820,      0.173560,
		      2.128620,      1.250850,     -1.299870,     -1.555810,      0.452460,
		      0.629440,      0.389920,      0.768840,      0.151140,      0.893480,
		      0.470060,      2.783270,      2.403380,     -2.391410,     -2.395640,
		     -0.920980,      1.751570,      0.732830,     -0.757330,     -2.003460,
		      0.000000,     -0.244080,      0.261190,     -0.832260,     -0.450250,
		     -0.590700,      0.592300,      0.092860,     -0.469950,     -1.178650,
		      0.405590,      2.298070,     -0.042610,     -0.810460,     -0.430210,
		      0.083750,     -0.532420,     -2.386920,     -0.190680,     -1.778890,
		     -1.621480,      1.564280,      1.306710,      1.434350,     -0.292120,
		     -1.012690,      3.645380,      1.647990,     -0.337370,      0.000000,
		     -0.113350,      3.143080,      0.256710,     -0.295860,      1.172840,
		     -0.413880,     -0.569780,     -0.405110,      1.696400,      1.664680,
		     -0.679060,     -1.204010,      0.665390,      1.459720,     -0.241090,
		     -0.137280,      0.864030,      0.869390,     -0.900620,      0.829580,
		      2.341540,      0.274720,     -1.190900,     -0.063130,     -0.765660,
		     -0.747130,     -0.182940,     -1.691330,     -0.188700,      0.000000,
		      0.009200,     -1.944930,      0.410670,     -0.069730,     -0.688250,
		     -0.767020,     -0.577030,     -0.751220,     -1.529920,     -1.587890,
		     -0.312960,      1.800300,     -0.350660,     -1.986940,     -1.242050,
		      1.189500,      1.066950,     -1.605790,     -0.108740,     -0.372190,
		     -0.576050,      1.379540,      3.335540,      1.851290,      2.076660,
		     -0.700590,     -0.521130,      1.741160,      0.998010,      0.773730,
		      0.000000,     -1.984350,     -2.372840,      0.902170,     -0.480420,
		     -1.101000,      0.435280,     -0.394170,     -0.388740,     -0.049670,
		      0.417470,      0.501080,      0.754880,     -0.122570,      0.232650,
		      1.505210,     -0.282640,     -0.562420,      0.997430,      0.737580,
		     -0.778030,     -1.109160,     -2.218440,     -1.396670,     -1.423820,
		     -0.205100,      1.584750,      0.858030,     -1.841570,     -2.669950,
		     -1.358100,      1.255880,      0.000000,      1.737300,      2.985660,
		     -2.512610,      0.131750,      1.142780,      0.344050,      0.986450,
		      0.617240,     -0.106170,     -0.869700,     -0.355130,     -0.433380,
		      0.969410,      0.461690,     -1.023960,     -1.266270,     -1.079150,
		      0.100990,     -1.257900,      0.780660,      0.300740,      1.563760,
		      1.101210,      1.399420,     -0.670570,      0.975480,      0.077960,
		      1.031740,      1.568460,     -0.413180,      0.742390,      1.390110,
		      0.000000,      0.948850,     -0.721810,      0.562840,      1.233040,
		     -1.083890,     -2.353210,     -0.417910,     -0.665290,     -0.185160,
		      0.560570,      0.062760,     -1.469910,     -1.482540,      0.397350,
		      0.615350,      0.515780,      0.496920,      0.195570,      0.193890,
		      1.603350,      0.921470,      1.091710,     -0.031850,     -1.370130,
		     -1.618530,     -1.611600,      2.513400,      1.407170,      0.022000,
		     -0.587660,     -1.702740,     -1.849730,      0.108650,      0.000000,
		     -2.987520,     -0.135550,      2.179090,     -3.046250,     -0.771090,
		      1.760970,     -0.748050,     -0.651020,     -0.068970,      0.605880,
		     -0.645330,      0.928940,      1.508190,      0.774960,     -0.322120,
		     -0.857090,      1.123940,      0.552810,     -0.372110,     -2.302550,
		     -0.791560,     -1.447420,      0.094230,      0.938310,      0.433310,
		     -0.934010,     -1.836370,     -0.261440,      0.811580,      0.287510,
		      0.834870,      0.720520,      0.159390,      1.190010,      0.000000,
		      1.031090,     -1.369120,     -2.037640,      2.581120,      2.583900,
		     -0.560350,     -1.195360,      1.521380,     -0.685930,     -1.663880,
		      0.590050,      1.328230,     -0.622410,     -2.042390,      0.088710,
		      0.963900,     -1.018580,     -0.704260,      0.323930,      0.100880,
		      0.407530,      1.068230,      0.808320,     -0.692260,      0.507300,
		      1.917450,     -0.177500,      0.735840,     -0.361360,      0.548800,
		      0.856720,     -0.517770,     -0.080330,     -2.785020,     -0.708780,
		      0.000000,      1.549270,     -0.793580,     -0.021630,      0.739610,
		     -2.577660,     -0.144310,      2.553300,      0.185430,     -0.321560,
		      0.923160,      0.829070,     -2.713360,     -1.575050,      0.955970,
		      1.210060,      0.107460,     -0.591820,      1.301620,      0.279750,
		      1.255160,     -0.573930,      0.287990,     -0.830660,      0.077280,
		      0.382370,     -0.556140,      1.061360,     -1.406890,      0.534900,
		     -1.722960,      0.289750,      0.720850,      0.620100,      1.311410,
		     -1.335740,     -1.020200,      0.000000,     -1.019070,      2.366560,
		     -1.391420,     -2.369350,      1.623440,      0.831670,     -1.902330,
		     -2.164050,      1.490410,     -0.329180,     -1.445470,      0.043670,
		      2.213370,      0.046000,     -1.679020,      0.002090,      1.827960,
		     -0.894690,      0.334620,      0.890650,      0.259960,     -0.284710,
		     -0.355750,      0.436700,     -0.944150,     -1.843090,      2.250650,
		      1.562130,      1.254040,      1.093980,      1.011820,     -1.164200,
		      0.427950,     -1.014820,     -0.359860,      0.915260,     -1.586180,
		      0.000000,     -0.911090,      0.280030,      0.369030,      0.480660,
		     -0.660960,     -1.671080,      0.333440,      1.741820,     -1.270710,
		     -1.788110,      0.860740,      2.789610,     -1.330420,     -0.895160,
		     -0.661370,      0.541500,     -0.449940,      0.156180,      1.219240,
		     -1.341600,     -0.716140,      0.582930,      1.153630,     -0.488520,
		     -0.257930,      0.478670,     -1.413610,      0.438840,     -3.518090,
		     -1.634220,     -0.489000,      1.112150,      0.439470,      0.820450,
		      0.659280,     -2.586280,     -0.556500,      0.063960,      0.000000,
		      0.669660,     -2.134490,      1.440190,     -0.032180,     -0.014670,
		      2.307290,      1.113500,      0.066210,      0.007170,      2.519710,
		     -0.361870,     -1.423850,     -0.655210,      0.688640,      0.870300,
		     -2.202900,     -1.099320,      0.296290,     -2.643510,      0.576110,
		      1.567600,      0.698800,      0.185030,      1.588110,      0.926350,
		      1.111990,     -0.707730,      1.239290,      1.940710,      0.287900,
		      0.785400,      0.176020,     -0.220360,     -2.025720,     -0.589960,
		      1.461640,     -0.763750,      1.462870,     -0.715430,      0.000000,
		      0.276990,     -0.279420,     -0.755450,      0.267710,      0.039640,
		     -2.064570,     -0.446160,      0.155580,      1.393470,     -1.187470,
		     -1.781460,     -1.708610,      0.421250,     -1.509130,      0.854810,
		      0.281990,      0.753490,     -0.422070,      1.087400,      2.315500,
		     -0.053860,     -2.117920,     -1.820960,     -1.243660,     -1.117910,
		     -2.226790,      0.502510,     -1.550610,      0.188580,     -3.012980,
		     -2.493630,     -1.112200,     -0.508670,      0.977490,      0.019410,
		     -0.135830,     -0.406720,     -1.849960,     -0.731300,      0.354520,
		      0.000000,      0.031000,      0.525700,     -2.470090,      0.380850,
		      0.278370,     -1.002870,      0.349380,      0.493890,     -1.655160,
		     -0.051000,      2.807840,      1.349340,      0.393690,      1.396150,
		     -0.143060,      0.798540,      0.469500,      0.602730,      0.214580,
		     -3.126720,     -0.097540,      2.197710,      1.483950,      0.655340,
		      0.783540,      0.220090,      1.432090,      1.183970,      0.953120,
		      1.093940,      0.085410,     -0.151610,      1.209210,     -0.524920,
		     -1.007530,     -0.276930,     -0.144130,      1.238560,      1.190680,
		      0.552520,      0.706630,      0.000000,     -0.632880,     -0.065410,
		      2.524860,     -0.139320,     -1.092700,      1.002650,     -0.624100,
		      0.003850,      1.624460,      1.732250,     -0.676360,      0.174540,
		     -1.366760,     -1.586360,     -2.764700,     -0.340610,     -1.368520,
		     -0.465210,     -1.432820,      0.510810,      1.550130,      0.039160,
		     -0.821990,     -0.673430,     -1.069360,      0.589460,     -2.676640,
		     -1.106850,     -1.326510,      0.794820,      0.326930,     -0.630470,
		     -1.478080,     -1.519810,     -1.203970,     -0.978810,      0.218040,
		     -0.316280,     -2.337320,      1.629770,      1.684470,     -0.029520,
		      0.000000,      0.924620,      1.184090,     -0.172490,     -2.268060,
		      0.739590,      1.222770,     -0.827160,     -1.765050,      1.318820,
		     -1.563280,     -0.976220,     -0.572120,      0.730510,      0.771670,
		      3.311320,      0.141990,      0.296680,      0.152770,      1.205030,
		      1.431080,     -1.361790,     -0.678890,      1.662540,      0.686070,
		      1.758380,      0.856430,     -0.223320,      0.840790,      1.718300,
		     -0.501410,      0.391300,      0.581790,      0.909320,      1.155150,
		      0.609610,      0.396310,     -0.006510,      1.543140,      1.193420,
		     -1.456720,      0.741250,      0.814390,      0.989450,      0.000000,
		     -0.892800,     -1.244960,     -0.800180,      2.190230,      0.384940,
		     -2.056310,      0.531230,      0.397350,     -1.955870,      0.725390,
		      1.667600,     -0.651860,      0.474030,     -1.301260,     -1.669520,
		     -0.481680,      0.548650,     -0.848330,     -1.111110,     -2.635200,
		     -1.656290,      0.940210,     -0.042510,     -1.117430,     -2.468390,
		     -2.048790,      0.871190,      0.411580,     -0.547940,     -0.197480,
		     -0.459160,      0.550540,      1.148670,      0.225870,     -1.097960,
		     -1.427090,     -1.743650,     -0.329050,     -0.202550,      0.851880,
		     -1.599830,     -0.290880,      0.144450,      0.698800,      0.000000,
		     -0.437020,      0.152610,      0.835410,     -0.310880,     -0.866200,
		      0.720720,      0.495750,     -0.500420,     -0.877900,      1.745970,
		      0.198040,      1.753140,      0.618560,      1.090100,     -1.210200,
		      0.836740,     -0.497420,     -0.395260,      1.135000,      1.438250,
		      0.987030,      0.139620,     -0.403800,      0.798550,      0.938690,
		      1.317700,      0.918070,     -1.301420,     -0.856450,      1.835190,
		     -0.022220,     -0.380020,     -1.365960,     -0.453260,      1.135840,
		      2.640590,      2.087030,     -0.174560,     -0.353590,      0.099580,
		      0.107850,      0.052070,     -0.380230,      0.631700,      0.657400,
		      0.000000,      0.761850,     -1.427310,      0.345000,     -0.075730,
		      0.514680,     -1.172470,     -0.696800,      1.290410,      2.739900,
		     -2.580120,     -0.624220,      0.509590,     -1.320170,     -0.171100,
		     -0.055210,     -1.346010,     -0.454680,      0.344210,     -0.196220,
		      0.010290,     -0.973970,     -2.523310,     -0.251820,     -0.003200,
		      0.916240,     -0.773960,     -2.127490,     -0.854810,      0.405060,
		     -1.510750,     -1.112230,      0.331670,      2.581950,      0.608480,
		     -0.024560,     -1.466150,     -1.234260,      0.623180,     -0.480960,
		     -0.384710,     -0.144240,     -0.030370,      1.126340,     -0.716130,
		     -1.403160,     -1.635830,      0.000000,     -0.011250,      0.756180,
		     -0.453690,     -0.129040,      0.286770,      1.214840,      0.600580,
		     -1.615730,     -1.376050,     -0.245290,      0.036370,     -1.915010,
		      0.761650,      1.049380,      2.761380,      1.237730,      0.894240,
		     -0.541650,     -0.257930,      1.078800,      1.263100,      1.543850,
		     -0.874120,     -0.548290,     -0.697800,     -0.871830,      0.830030,
		      1.308020,      0.828430,      0.184860,      0.752220,      0.058180,
		     -0.808380,     -0.252110,     -1.245990,      0.244420,      0.028560,
		      0.273610,      0.933200,     -0.216690,     -0.352440,      0.025590,
		     -0.508230,      0.665620,      0.839690,      1.418990,     -0.550920,
		      0.000000,      0.323320,      0.000820,     -1.520350,      1.002960,
		      0.802650,     -1.149990,     -2.102290,     -0.259370,      0.087960,
		      3.069540,      0.873880,      1.049820,      0.967360,     -1.253540,
		     -2.024320,     -0.902290,     -0.848900,     -0.701850,     -1.431040,
		     -1.674450,     -1.167940,      1.081350,     -1.018400,     -1.068260,
		     -0.145590,      0.337420,      0.666320,     -0.716060,     -1.922280,
		      0.395480,     -0.787910,     -1.157970,     -0.101570,      1.881240,
		      1.233580,      0.168430,     -0.607720,     -0.841670,     -0.441370,
		      0.897970,      0.078170,      0.879610,     -0.020460,     -0.525990,
		      1.521820,      0.684700,     -0.156100,     -0.278330,      0.000000,
		      0.039360,     -0.319960,      0.867700,     -0.633680,     -1.730810,
		      0.177110,      2.877530,      1.270930,     -1.105850,     -1.909510,
		      0.529670,      1.307660,     -1.132560,      1.057420,      0.889120,
		      1.255130,     -0.156160,      0.296570,     -0.057650,      0.199830,
		      1.140360,      1.267550,      3.016360,      0.424090,     -0.017290,
		      0.764010,     -0.283280,     -0.401870,      0.871680,      0.694840,
		      0.501880,      0.062020,      0.127050,     -0.188200,      0.232160,
		     -0.373120,      0.593070,     -1.395770,     -1.256630,     -0.181790,
		     -0.856520,     -0.082220,      1.940540,      0.467710,     -0.093090,
		     -1.138650,      1.262670,     -0.215570,     -1.441600,      0.000000,
		      0.605020,     -0.645760,     -0.078830,     -0.341090,      1.911600,
		      0.251940,     -1.508820,     -0.949920,      0.893750,     -0.006100,
		     -0.497360,     -0.842760,      0.650760,      0.430880,      0.826180,
		     -1.268240,     -0.592240,      1.453990,      1.958580,     -0.676140,
		     -2.126570,     -3.875590,     -1.977510,     -0.491770,     -1.396910,
		     -0.349020,     -1.092750,     -0.722860,     -0.058570,     -0.560390,
		      0.207450,     -0.636890,     -1.427260,      0.241720,     -0.284580,
		      1.366090,     -0.283240,      1.093020,     -0.158180,      0.177250,
		      1.952320,      0.863620,     -1.262630,      1.594780,      0.891840,
		      0.051110,     -0.134470,      1.337180,     -0.118050,      0.931130,
		      0.000000,     -0.864160,      0.567200,      0.281930,      0.205550,
		     -1.186630,     -1.521250,      0.396470,      1.375020,     -0.284910,
		     -0.550920,     -0.595450,      0.334570,      1.228270,     -0.586190,
		     -0.855400,      1.507980,      1.240960,     -0.534160,     -1.800650,
		     -0.336570,      1.269970,      1.902050,      0.766310,      2.413180,
		      0.153940,     -1.533780,      0.707260,      0.596620,      1.177080,
		     -0.176770,      0.414010,      1.879900,      1.253140,     -0.301060,
		      0.693970,      0.793260,      0.311860,      1.287420,     -1.310470,
		     -2.026250,     -1.154280,     -2.487720,     -1.479640,     -0.384470,
		      1.018270,     -0.055880,     -0.383620,     -1.679700,     -0.902130,
		      0.785090,      0.706140,      0.000000,      1.156000,      0.604080,
		     -2.343120,     -0.066500,      0.756580,      1.872660,      0.187420,
		     -2.173350,     -0.946200,      1.404170,      1.444000,     -0.049440,
		     -1.428730,     -0.091020,     -0.152340,      2.455850,     -0.677340,
		     -2.770980,      0.084120,      2.441330,     -0.034250,     -0.327330,
		     -1.713970,     -3.082770,     -0.886170,     -0.912920,      0.657820,
		      0.460000,     -1.929960,     -1.093290,     -0.093010,     -0.443950,
		     -0.832420,     -1.236630,      0.423540,     -1.210820,     -0.249130,
		     -2.671670,      0.053950,     -0.140380,      0.532990,      2.984230,
		      2.457580,      0.473750,      1.181690,      1.305230,      0.478570,
		      1.010630,     -0.417560,     -1.725350,     -1.782320,      0.934950,
		      0.000000,     -0.693530,     -1.130460,      1.895460,      0.389590,
		      0.119050,      0.116530,     -0.044810,      1.719140,      1.935620,
		     -1.065700,     -1.924740,     -1.170800,      0.629550,      2.450880,
		      1.897300,     -2.821030,     -1.253260,      2.294540,      1.816240,
		     -1.219690,     -1.677490,     -0.202700,      1.665660,      2.597590,
		      3.862950,      2.653340,     -1.946090,     -1.197970,     -0.109600,
		      1.274550,      0.931550,      0.384450,      0.915010,      1.092560,
		     -0.812920,      0.637170,      0.771420,      1.225670,      0.417850,
		      1.760950,      0.973610,      0.186050,     -1.217360,     -2.005160,
		     -2.215540,     -0.160110,      1.232940,      0.400100,      0.059850,
		     -0.343180,     -1.483590,      0.019120,      0.706340,      0.000000,
		      0.176830,      1.555130,      0.441720,     -2.361330,     -0.112260,
		      0.236350,      0.108930,      0.314670,     -1.323490,     -0.250860,
		      0.687350,      1.931670,      0.284780,     -2.840160,     -1.918740,
		     -0.766600,      3.129120,      1.257000,     -2.313990,     -1.649870,
		      1.892170,      0.175520,     -1.190870,     -1.136860,     -4.256690,
		     -2.743730,     -0.688090,      0.894980,      2.374040,     -0.645220,
		     -0.853350,      0.671780,     -0.821120,      0.136950,     -0.835880,
		     -0.221220,     -1.102620,      0.697310,     -1.015100,     -1.320550,
		     -2.376340,     -1.230350,      0.427480,      1.249090,      0.082450,
		      0.471110,      1.291120,      0.631760,      0.326760,      2.409720,
		      0.075350,      0.444210,     -1.056000,      0.047630,      0.000000,
		     -0.264460,     -1.148960,     -1.461140,      0.762170,     -0.725460,
		     -0.905070,     -0.511000,     -0.155400,      1.094940,      2.553340,
		      0.654260,     -2.296950,     -2.265180,      1.061980,      2.482800,
		      3.782200,     -1.850540,     -3.816740,      0.563230,      2.445640,
		      0.119550,      0.644330,     -1.033170,     -0.355860,      1.120700,
		      2.808490,      3.654140,      0.139180,     -0.809500,      0.178180,
		      0.544640,     -1.273370,      0.484920,      0.525650,      0.297320,
		     -1.503160,     -0.913140,     -0.319200,     -0.137840,     -1.536070,
		      2.131990,      2.129090,      0.144040,     -0.757270,      0.111390,
		     -0.170220,     -1.618290,     -0.205830,      0.762460,      0.170340,
		     -0.549750,     -1.833310,     -0.424360,      0.574840,     -1.227920,
		      0.000000,     -0.333690,     -0.423420,      0.300240,      0.686490,
		      0.738100,      0.619480,      0.219130,     -0.633640,      0.666370,
		     -1.743500,     -0.807780,      1.560080,      3.396300,      0.855700,
		     -3.764380,     -2.785930,     -0.454590,      3.238340,      2.504820,
		     -0.317000,     -2.240000,     -0.376380,      0.560250,     -0.645940,
		      1.378340,     -2.195750,     -3.681560,     -1.632070,     -0.732060,
		      0.718970,     -0.546230,      0.194090,      1.848660,     -0.738890,
		     -0.460640,      0.420240,      0.493300,     -0.333350,      0.066710,
		      0.253710,      0.194210,     -1.182140,     -1.268770,     -0.889720,
		     -0.875310,     -1.201470,      0.109250,     -0.010030,     -0.087900,
		     -0.351280,      1.278270,      2.135320,      1.097260,      0.659050,
		      0.857750,     -0.616920,      0.000000,      1.065310,      0.221550,
		      0.947560,     -0.043750,      0.216460,     -0.232080,      0.335610,
		     -0.258150,     -1.813400,     -0.742050,      2.022450,      0.665710,
		     -2.448840,     -2.283750,      2.650950,      2.217180,      2.616090,
		     -1.065910,     -3.442170,     -0.215190,      1.865280,      1.292340,
		      2.201890,     -0.508590,     -1.688900,      1.030400,      0.967040,
		      2.516420,      0.296630,     -0.816080,      1.370760,      1.738450,
		     -1.669100,      0.235330,      0.676490,     -0.037160,     -0.609520,
		     -0.820580,      0.160090,      0.387230,     -2.418100,     -1.294550,
		      0.847240,      1.289220,     -0.852310,      0.116990,     -0.274130,
		     -0.078400,     -0.062680,     -1.208700,     -0.533040,      0.063580,
		     -2.089940,     -1.606900,     -0.663080,     -1.262450,      0.683690,
		      0.000000,     -0.269570,      1.026590,     -1.370410,     -1.570800,
		     -0.774500,      1.762150,      0.153560,     -0.112090,      0.284680,
		      2.151750,     -2.210360,     -2.425100,      1.009400,      3.424120,
		      0.463430,     -3.926500,     -2.615110,     -0.041160,      2.110420,
		      1.610020,      1.031690,     -1.045950,     -2.063100,     -0.747770,
		     -1.629190,     -0.152470,     -0.266890,     -1.851480,     -0.882540,
		     -0.331810,     -0.422540,     -2.585710,     -0.099570,      2.055810,
		      0.639330,      0.296740,     -0.047930,      0.699360,      0.205380,
		      0.186330,      1.108360,      0.479920,      1.007140,     -0.946680,
		      0.900200,      0.491900,      0.556610,     -1.331710,     -0.701160,
		      1.201180,      0.235640,     -1.261480,      1.365110,      2.007140,
		      1.399220,      0.286990,     -1.067850,     -0.171050,      0.000000,
		     -1.150650,     -0.511800,     -0.228600,      1.053660,      1.096070,
		     -1.633990,     -0.414290,      1.774390,      0.207720,     -2.113200,
		     -0.352490,      2.627770,      0.816700,     -3.252910,     -2.227780,
		      3.765760,      2.624080,      0.518560,     -0.475300,     -2.511070,
		     -1.610260,      0.862760,      0.747620,      3.610870,      1.831490,
		     -0.914310,      1.324670,      0.430100,      0.779280,     -0.093700,
		     -1.919700,      0.413570,      2.302080,     -1.710890,     -0.177270,
		     -0.496920,     -1.070320,     -0.856710,      0.240030,      0.115200,
		      1.267240,      0.081600,     -0.630310,     -1.486460,     -1.416590,
		      0.211810,      1.567380,      1.505220,      0.256200,     -0.419280,
		      0.991840,      0.027300,     -0.582430,     -0.547960,     -1.127990,
		     -0.096380,      0.675720,     -0.618050,      0.624490,      0.000000,
		      0.748890,     -0.288710,      2.289590,     -0.575460,     -1.041560,
		     -0.342040,      1.289520,     -1.774430,     -1.168010,      1.532740,
		      3.533500,     -1.723610,     -2.355510,      2.256360,      3.338030,
		     -0.457110,     -3.246130,     -1.972380,      0.326220,      1.555210,
		      1.060160,      1.597930,      1.151460,     -2.467940,      0.008620,
		     -1.032040,     -1.950970,     -0.281940,     -0.257540,      1.345410,
		      0.976320,      0.581790,     -3.084130,     -0.732110,      0.202750,
		      1.720420,      1.820470,      0.734530,      0.842010,      0.081340,
		     -1.919560,     -0.459800,     -0.045790,      1.347660,      0.932750,
		      0.928240,     -0.445100,      0.926030,      0.898470,     -0.438270,
		     -1.378570,      0.689420,     -0.995670,     -0.423610,      1.206770,
		      1.222420,      1.180900,     -1.525300,     -0.878080,     -0.946700,
		      0.000000,      0.592510,     -0.836570,     -2.352440,      0.382990,
		      0.715530,      1.867050,     -1.080560,      0.574320,      1.655850,
		     -0.438130,     -3.078330,     -1.242410,      2.332450,      0.767500,
		     -2.875010,     -2.728050,      2.833970,      3.006920,      0.033140,
		     -0.156780,     -0.354690,     -1.313320,     -1.395160,     -0.606330,
		      2.383950,      1.284130,     -0.502060,      1.281910,     -0.038360,
		     -0.367500,      0.530940,      0.053970,      0.731120,      1.942120,
		      0.235760,     -0.319640,      0.205820,     -0.847430,     -1.379240,
		     -0.623730,     -1.200150,     -0.662040,      0.956940,      0.410530,
		     -0.949560,     -1.101210,     -0.150010,     -0.748810,     -0.190000,
		      0.575450,      1.409590,      0.660090,      0.955280,     -1.141340,
		      0.547700,     -0.137520,     -0.292690,      0.279740,      1.669780,
		      1.839380,      1.805030,      0.000000,     -1.396200,      1.156860,
		      0.108070,      0.991750,     -0.268830,     -0.106150,      1.297200,
		      0.811680,     -2.283390,     -1.456080,      2.107500,      3.301280,
		     -1.080660,     -2.510060,      1.809520,      2.609100,     -0.997820,
		     -2.843660,     -1.719940,      0.734800,      0.371060,     -0.392860,
		      0.777600,      1.584490,     -1.382510,      0.427980,      2.054090,
		     -1.944220,     -1.061900,     -0.863800,      0.676080,      0.712930,
		      1.977920,     -1.610240,     -0.874710,     -2.203200,     -0.408850,
		      0.993490,      2.167190,      2.037510,      3.395450,     -0.267130,
		     -0.279730,     -2.080110,      0.530430,      2.136680,      0.655020,
		     -0.122380,     -0.161310,     -0.347370,      0.495110,     -0.756720,
		     -1.066720,     -0.430000,     -1.070590,     -0.500320,      0.782480,
		     -0.003630,     -1.442500,     -1.128280,     -0.652950,     -0.089180,
		      0.000000,      0.359270,      1.567270,      0.814510,     -1.922590,
		     -0.241190,     -0.249050,      0.575200,     -1.094270,      0.903840,
		      2.712310,     -0.175330,     -3.398080,     -1.001420,      3.143650,
		      1.645900,     -1.305000,     -0.756680,      1.212320,      1.926440,
		     -1.171880,     -0.637620,      2.057780,      0.560540,     -1.070570,
		     -1.132600,      0.485120,      0.413090,      0.791910,      2.396760,
		      0.344700,     -2.419310,     -0.627510,     -1.683440,      0.897910,
		      0.263270,      1.188410,      0.571210,      0.426310,     -1.778370,
		     -2.080840,     -0.636180,     -1.137080,     -1.175690,      0.179900,
		      1.049850,     -1.726980,     -1.068990,      0.214140,     -0.226570,
		     -1.516990,     -1.674530,     -1.333840,      0.206710,      1.320600,
		      0.079940,     -1.146180,      1.087880,      0.097440,      1.507640,
		     -0.592870,      1.577220,      0.882350,      0.415650,      0.000000,
		      1.404620,     -1.288080,     -0.940830,     -0.057660,      1.138500,
		      0.517680,     -0.926080,      1.463480,      0.491290,     -2.260630,
		     -1.976200,      2.313980,      3.446010,     -0.565670,     -1.899850,
		      1.078910,      2.103080,      0.104150,     -1.344070,     -1.405520,
		      0.786620,      0.688410,     -0.626560,      0.104610,      1.635780,
		     -2.127860,     -0.761160,      1.735890,     -1.632890,     -1.034480,
		      0.815250,      1.765060,      1.246940,      1.223530,      0.002560,
		      1.171790,     -0.538000,     -0.938190,     -0.229320,      2.237320,
		      0.596960,      1.861390,      1.594100,      0.234660,     -2.128360,
		     -1.104360,      0.058040,      1.485740,     -0.233650,      0.316170,
		      1.843540,      1.838220,     -0.969640,     -2.575310,     -0.050350,
		     -0.592240,     -0.987320,     -1.025090,      0.376110,     -1.000650,
		     -1.658600,     -1.653580,     -0.488730,      0.195420,      0.000000,
		     -0.815130,     -1.027790,     -0.123860,      1.669700,     -0.969340,
		     -0.775070,      0.014590,      0.057300,     -1.622180,      0.640420,
		      3.468310,      1.175120,     -2.913870,     -1.833590,      1.527040,
		      0.827010,     -1.220340,     -0.017980,      0.355290,      1.799140,
		     -1.617930,     -2.783550,      0.677790,      0.531950,      0.288780,
		      0.272260,      0.402430,     -1.098450,     -0.054490,      2.175800,
		      1.203350,     -1.893570,     -0.557370,     -1.339010,      0.854600,
		     -1.955980,     -0.744480,     -0.167360,      1.895670,      0.352910,
		     -1.509070,     -0.694040,      0.721030,     -0.886030,     -0.182180,
		      0.411280,     -0.724770,     -1.689270,     -0.981000,      0.932870,
		      0.611660,     -1.437550,     -1.545460,     -0.377270,      0.477360,
		      0.162660,      0.511070,      0.527220,     -1.219250,      0.465020,
		      0.726020,      1.668260,      0.284600,      0.550480,      0.106440,
		      0.000000,     -0.972390,      2.487230,      0.139670,     -1.685670,
		     -0.280870,      2.045610,      1.115000,     -1.382280,      1.125990,
		      1.693700,     -1.366290,     -3.115880,      0.776450,      2.651360,
		      0.292800,     -1.219310,     -0.044820,      0.862780,     -0.246970,
		     -0.331420,      0.167850,      1.529210,      0.757020,      0.164590,
		      1.009550,      1.057560,     -0.817620,      0.373420,      1.869600,
		     -1.480530,     -1.772070,      0.003270,      0.495360,      1.214770,
		      0.303070,      0.152790,      0.034830,      1.572770,     -1.486790,
		     -0.901770,      1.097190,      0.195390,     -0.748430,      1.128920,
		      1.492030,     -0.489460,      0.476030,      1.164230,      0.691120,
		      0.499360,     -0.942920,     -0.563170,      2.127470,      0.515340,
		     -1.289410,      0.498250,     -1.603250,      0.340600,      1.465930,
		      0.578460,     -0.619090,     -1.629850,     -0.598240,     -2.576480,
		     -2.202740,     -0.970570,      0.000000,      1.176440,      0.155540,
		     -0.653850,      1.290210,      1.970210,     -0.502770,     -0.776090,
		      1.482680,     -0.137670,     -2.255610,     -1.236300,      2.276530,
		      1.579130,     -1.510680,     -0.428320,      1.301560,      1.029130,
		     -1.048960,      0.138870,      0.974090,      1.827160,     -0.565780,
		     -2.707820,     -0.458800,     -0.379680,     -1.546980,     -0.053390,
		      0.076840,     -2.172480,     -0.735400,      1.325750,      1.959860,
		     -1.072340,     -0.638460,     -0.792010,      2.136800,      0.455520,
		     -1.963610,     -1.413120,     -0.468160,     -0.288670,     -0.450070,
		     -0.500420,      0.402970,      0.347260,     -0.821010,     -1.772910,
		     -0.982070,     -2.085090,     -1.550100,      0.234720,      2.038100,
		     -0.622220,     -0.547880,     -0.640450,     -0.209960,      0.694700,
		     -1.621460,      0.215180,     -0.525680,     -0.073530,      0.449980,
		      1.327660,      2.533290,      2.200690,      0.142760,      0.206700,
		      0.000000,      0.302800,     -0.808010,      0.750090,     -0.543580,
		     -2.231090,     -0.446280,      2.144990,      0.004720,     -1.940390,
		      0.877530,      2.852860,      0.561320,     -1.961250,      0.311580,
		      1.551300,      0.047380,     -1.183680,      0.249980,      0.041660,
		     -1.538550,     -2.188320,     -0.336910,      1.468170,      0.313160,
		      0.319670,      2.117280,      1.253630,     -0.050790,      2.170500,
		      2.623760,     -0.144800,     -1.103910,      0.340010,     -0.520910,
		      0.513790,     -1.272810,      0.278310,     -0.780980,      1.158140,
		      0.811050,      0.269180,      2.040490,      0.955190,     -1.940180,
		      0.482220,      0.722110,      0.898350,      0.732420,      2.188370,
		      2.565010,      1.329200,     -1.432960,     -0.641850,     -0.068760,
		      1.301150,      0.556280,      0.630470,     -0.707280,     -0.881900,
		      1.507710,      0.246470,      0.454790,     -0.995870,     -0.942670,
		      0.876420,      0.417930,     -1.316770,     -1.013370,      0.000000,
		     -0.685470,      0.269690,     -0.970060,     -0.470050,      1.442450,
		      1.052800,     -2.377680,     -0.376980,      2.188950,      1.224990,
		     -1.368140,     -2.121130,      1.133450,      1.822500,     -0.710450,
		      0.245640,      0.992110,      0.972000,     -1.417660,      0.300810,
		      1.809310,      0.710410,     -0.548230,     -1.647760,      1.026230,
		     -0.076680,     -1.377820,      0.172320,     -0.190810,     -2.617440,
		     -1.276190,      0.065440,      0.675420,     -0.184760,     -0.962990,
		     -1.280570,      0.378040,      2.513120,     -0.120220,     -0.645500,
		     -1.914520,     -1.953380,     -1.781170,      0.420800,      0.285830,
		     -0.414820,     -0.478700,     -0.980390,     -0.730270,     -1.812350,
		     -1.922600,     -1.102150,      0.327720,     -0.124060,     -0.734470,
		     -0.282450,     -2.291370,      0.638550,      0.796350,     -0.501910,
		     -0.628730,     -0.729920,      0.840440,      1.792350,      0.967200,
		      1.170600,      1.709840,      2.610730,      0.161060,      0.000000,
		     -0.371550,      0.919900,      1.200550,      0.812250,     -0.828960,
		     -1.357690,      1.426710,      1.778850,     -0.933060,     -1.456530,
		     -0.251220,      1.702120,      0.837030,     -0.827880,      0.835360,
		      0.607910,     -0.781950,     -1.340750,      0.981210,      1.644510,
		     -1.011350,     -2.248680,      0.184820,      2.529260,      0.029760,
		     -1.953960,      0.970070,      0.419790,     -0.991170,      1.828120,
		      2.106150,      1.191330,      0.496550,      1.753120,      0.669770,
		      1.160140,     -0.524160,     -1.014260,     -1.058300,      0.027420,
		      0.592860,      1.437080,      2.058950,      1.284770,     -0.187850,
		      0.624830,     -0.116560,      0.310470,      0.346020,      1.628170,
		      2.211830,      2.644720,     -0.256850,     -1.208960,     -0.709710,
		      1.474660,      1.051360,     -0.001250,      0.222020,      0.047980,
		      2.677470,      0.289670,     -0.706060,     -0.953490,     -1.141010,
		     -0.008940,      0.843460,      1.200060,      1.453800,      1.918460,
		      0.000000,      1.121120,      0.157830,     -1.130860,     -0.625580,
		      1.359480,      2.535310,      0.974950,     -2.016420,     -0.518690,
		      1.720060,      1.154380,     -0.095200,     -1.110920,     -0.646820,
		     -0.165490,     -0.051140,      0.602890,      0.996630,      0.163900,
		     -1.346780,      0.243290,      2.470250,      1.014110,     -1.409910,
		     -2.132640,      0.382400,      0.465800,     -0.237330,      0.454370,
		      0.181920,     -2.192810,     -1.340790,     -0.638650,     -0.990470,
		     -0.195780,      0.155540,     -0.203020,     -0.644430,      1.679520,
		      0.548410,      1.692630,     -0.905170,     -2.581530,     -1.764640,
		     -0.031770,     -0.986790,      0.346540,     -0.005360,     -0.857500,
		     -0.999510,     -0.252180,     -1.411680,     -0.889620,     -0.923360,
		      0.957340,      0.170350,     -0.777090,     -0.281090,      0.398170,
		      0.225830,     -2.126440,      0.420960,      0.302270,      1.406110,
		      0.467130,     -0.104370,     -0.078390,     -1.123850,     -1.820200,
		      0.148930,      0.411290,      0.000000,     -0.731920,     -0.773090,
		     -0.314330,      1.044280,     -0.869920,     -0.651530,     -0.298320,
		      1.232470,      0.051130,     -1.001150,      0.268910,     -0.060800,
		      0.875160,      1.275200,      0.726840,      1.113590,      0.658040,
		     -0.386760,     -1.274510,      0.303450,      0.583070,     -1.637260,
		     -2.837090,     -0.268370,      2.784660,      0.725090,     -1.960660,
		     -0.125120,      0.299390,     -1.121380,      2.278920,      1.323480,
		      0.015920,     -0.008390,      1.089640,      0.649920,      1.816000,
		      0.364920,      0.074350,     -0.061660,     -1.200190,     -0.183640,
		      0.919120,      2.708850,      0.107630,     -0.470360,      0.993600,
		      0.243160,     -0.063230,     -0.782530,     -0.301750,      0.020430,
		      2.672240,      1.840790,      0.069370,      0.561570,     -0.265610,
		      0.111700,     -0.831210,      0.186060,      0.590960,      2.162770,
		      1.222690,     -0.175590,      0.311130,     -0.397160,     -2.017130,
		      0.754210,      0.718220,      0.671940,      1.072060,      0.700440,
		      0.000000,      0.084660,      1.289220,      0.323060,     -1.299350,
		     -0.501590,      0.667120,      0.510680,     -0.095210,     -0.556570,
		      0.320030,     -0.527780,      0.064230,      1.404060,      1.055160,
		     -0.020560,     -1.160040,     -0.666850,      0.551770,      0.596710,
		      0.842450,     -0.944120,     -0.968940,      2.010150,      2.181550,
		     -0.019290,     -0.943080,      0.516960,      0.349250,     -0.013210,
		      0.616550,     -0.974210,     -1.213780,     -0.343170,      0.038640,
		     -1.317320,     -1.487630,     -1.213140,     -0.094390,     -1.439260,
		      0.732600,      0.917800,      1.983680,      1.268830,     -2.007290,
		     -0.772180,      0.220860,     -0.928260,     -1.581920,      0.349360,
		      0.736580,     -0.593270,     -0.875220,     -1.384020,     -1.997180,
		     -1.432000,     -0.766850,      0.204410,      0.184530,      0.632080,
		      1.097990,      0.905590,     -3.354810,     -1.986270,     -0.044250,
		      1.318560,      1.507470,      0.447930,     -0.723410,     -1.334730,
		     -2.041660,     -1.240530,     -1.261640,     -1.193380,      0.000000,
		      0.631050,     -0.534270,      0.354970,      1.475380,      0.948410,
		     -2.572190,     -0.511220,      1.163050,      1.279240,     -1.178500,
		     -1.354370,      0.513850,     -0.804170,     -1.414070,      0.948450,
		      2.100700,     -0.322190,     -0.313340,      0.352400,     -0.315720,
		      1.246840,      1.521600,     -0.380760,     -2.487370,     -1.054790,
		      0.496830,      0.383780,     -1.036150,     -1.357560,     -0.810510,
		     -1.300850,      1.954900,      1.176600,     -0.907320,      0.243710,
		      1.099480,      0.439510,      0.706200,      0.056020,     -0.703760,
		      0.447590,     -1.437510,     -2.189730,      0.226390,      1.697040,
		      0.408910,     -0.286400,      0.313860,      1.055890,     -0.858680,
		      0.320880,     -1.098600,      0.678710,      0.207130,      0.764020,
		      0.861660,      1.265640,     -0.309890,      0.138750,     -0.338600,
		     -1.109920,      0.642480,      1.283820,      1.774750,     -1.560020,
		     -1.077330,      0.451670,     -0.465090,      0.879290,     -0.370420,
		     -0.650040,     -1.138870,     -0.026110,     -0.073790,      0.000000,
		     -1.014520,      0.269470,      0.002450,     -0.550140,     -0.300460,
		      1.952980,      1.267030,      0.153720,     -1.046180,      1.040100,
		      2.121470,     -0.968210,     -0.918390,      1.576710,      2.065970,
		      0.196850,     -0.760240,     -0.103770,     -0.631540,     -0.989090,
		      0.288000,      0.106360,     -0.867220,     -0.254460,      0.849630,
		      0.023560,      0.037910,      0.625310,      0.274120,      1.777790,
		      1.409140,     -1.250060,     -1.401360,     -0.042180,      1.614350,
		      0.494890,      0.582240,     -1.207230,      0.201120,     -1.121590,
		     -1.301090,     -0.253950,      1.430890,      1.738930,     -0.897590,
		     -1.308870,      0.029470,     -0.025940,     -0.733090,     -0.523000,
		      0.870030,      0.091090,     -0.007820,     -0.458690,      0.417060,
		     -2.078320,     -2.392540,     -0.223610,     -1.003030,      0.444920,
		      0.794180,      0.664090,     -1.371850,     -0.715200,     -0.686180,
		      0.241330,      0.624270,     -0.237060,     -0.220810,      2.420280,
		      0.083680,     -0.376500,     -0.084380,     -0.309000,      0.026890,
		      0.000000,      0.145390,      0.251890,     -1.126160,      0.155160,
		      1.282590,      0.667520,     -0.849070,     -0.350590,      0.595280,
		      0.824300,     -0.236350,      0.100130,      1.100960,     -0.264550,
		     -2.826790,     -0.618970,      1.883660,      0.223270,     -0.036550,
		      0.461790,     -0.826070,     -0.567050,      1.028330,      2.131350,
		      0.019180,     -0.337210,     -0.739320,     -0.664310,     -0.356230,
		     -0.766420,     -0.869390,     -1.123190,      1.077670,      0.996960,
		     -1.671460,     -1.104050,     -0.521290,      0.525200,      0.703140,
		      1.132880,     -0.522040,      1.012770,     -0.286480,     -2.359990,
		     -0.878340,      0.947250,      0.304800,     -0.056110,     -0.261900,
		      0.808870,     -0.562880,     -0.946460,     -0.223170,     -0.626630,
		     -0.381260,     -0.042160,     -0.274850,     -0.190810,      1.329480,
		     -0.187040,      0.479830,     -0.811810,     -0.086070,     -0.678360,
		      0.388760,     -0.531330,     -1.633540,      0.022130,      0.188050,
		     -0.461240,      0.773880,      0.072310,      0.181960,     -1.107500,
		     -1.222790,      0.480080,      0.000000,      0.493920,      0.120070,
		      1.167720,      0.045630,     -1.635100,     -0.462740,      0.688420,
		     -0.050160,     -0.328700,     -0.899790,     -0.098260,      0.793270,
		     -1.024150,     -0.902600,      1.617990,      1.942810,      0.172800,
		      0.218800,      1.271390,     -0.223840,     -0.806030,     -0.284540,
		      1.090830,     -0.026750,     -1.444970,     -0.423810,     -0.315950,
		      0.246370,      0.605310,     -0.983190,      0.739040,      1.374450,
		     -0.648530,     -0.964170,     -0.172910,      0.554920,      0.492300,
		      1.881580,      0.112160,     -0.072110,      0.286800,     -1.259190,
		     -0.410080,      0.809580,      1.796670,      0.570870,     -0.762610,
		     -0.883130,      0.408070,      0.124500,      0.455810,      0.414350,
		      0.148590,     -0.995330,      0.762660,      2.366540,      0.046920,
		     -1.381300,     -1.316440,     -1.436530,     -0.508690,      0.163750,
		      0.318490,      1.274470,     -0.332390,      0.016490,     -0.496630,
		     -0.373080,      0.908200,      0.144190,      0.853470,      1.211560,
		      0.567520,      0.438020,      0.511130,     -0.287940,      0.494060,
		      0.000000,      0.088280,     -0.642890,      0.294710,      0.662410,
		      0.269730,     -0.205480,     -0.283500,      0.777890,      0.202360,
		      0.002570,     -0.571680,     -0.019240,      1.586380,      1.879920,
		      0.572000,     -1.922080,     -1.016100,     -0.064490,     -0.670910,
		      0.711890,      1.492670,     -0.364010,     -1.569910,     -1.384510,
		      1.644330,      1.766870,      0.886030,     -0.308970,     -1.311210,
		      0.030960,     -0.806740,     -0.880350,     -0.618310,      0.367390,
		      0.859790,     -0.104670,     -0.121850,     -2.133750,     -1.107490,
		     -0.612680,      0.989840,     -0.163910,      0.436420,     -0.171540,
		     -1.756180,     -1.545640,      0.012990,     -0.121970,      0.707900,
		     -0.030160,      0.549180,     -0.690490,     -0.012350,     -0.685120,
		      0.440870,     -2.253860,     -0.142470,      0.294600,     -0.137400,
		      1.104590,     -0.647420,     -0.013790,     -0.246990,     -0.642260,
		     -0.846730,     -0.710400,     -0.142790,     -0.283720,     -1.119400,
		     -0.307960,     -0.024130,      0.108710,     -0.306320,     -0.417620,
		     -0.264000,     -0.663390,      0.172650,      1.696530,      0.000000,
		     -1.098920,      0.363620,     -0.229990,      0.096430,     -0.141950,
		     -0.141450,      1.133460,      0.338990,     -0.160970,      0.815730,
		      0.776340,      0.012280,     -0.300660,     -0.836160,     -0.036260,
		      0.663220,      0.299330,     -0.199930,      0.084020,      0.133640,
		     -1.259880,     -0.032810,      0.408300,      1.379840,      0.992630,
		     -0.965520,     -1.106400,      0.147840,      0.614660,      0.774690,
		     -0.600290,      0.159430,      0.188000,     -0.818950,     -0.665090,
		      0.414950,     -0.082140,      0.380890,      1.465590,      1.280610,
		      0.212780,      1.374430,     -0.465580,      0.493290,      0.838810,
		      1.388010,      1.402410,      0.513510,     -1.404660,     -0.735070,
		      0.244810,      0.797550,      1.302970,     -0.248750,     -1.394680,
		      0.302170,      2.036400,      1.204360,      0.010600,     -0.544860,
		      1.197530,     -0.597120,     -1.144960,     -0.694720,      0.940030,
		      0.452840,     -0.822980,      0.150140,      0.409440,      0.639080,
		      1.144350,      1.447910,      0.059430,      0.225010,      0.198940,
		      0.431220,     -0.882170,     -1.107240,      0.449310,      0.000000,
		      0.967650,     -0.300910,     -1.455050,      0.652600,      0.812920,
		      0.600390,     -1.337930,     -0.804870,      0.813350,      0.133620,
		     -0.330840,     -1.302040,     -0.469090,      0.697360,      0.609060,
		      0.455140,      0.165140,      0.023770,     -0.789980,     -1.260450,
		      1.013080,      1.509200,      1.437390,     -0.764800,     -2.033000,
		      0.221150,      1.232020,      0.548120,     -0.399540,     -1.245780,
		      0.799910,     -0.744350,     -1.205620,     -0.366750,     -0.289370,
		     -0.439960,     -0.475630,      0.516970,     -1.008020,     -1.631560,
		     -0.965020,     -0.847300,     -0.287130,      0.133630,      0.002670,
		     -1.376910,     -1.656440,     -0.069280,      0.271650,      0.713670,
		     -0.042940,      0.073280,     -0.160020,      1.074190,      1.070960,
		      0.110110,     -1.395830,     -1.356530,     -1.295770,     -0.276280,
		     -0.031590,     -0.658560,      0.606290,      0.598630,     -0.401830,
		     -1.162060,     -1.101900,     -0.650870,      0.144680,     -1.233010,
		     -0.855200,     -0.478410,     -0.233200,     -0.195500,      0.342770,
		      0.368130,      0.036340,      1.026880,      0.581330,      0.112580,
		      0.000000,     -0.907390,     -0.091360,      1.500290,      0.072040,
		     -0.030410,     -0.019000,     -0.209470,      0.239730,     -0.371700,
		     -0.374790,      0.432590,      1.145440,      0.752460,     -0.998170,
		     -1.061950,      0.952370,      0.898840,      0.212790,      0.961270,
		      0.854140,     -0.170030,     -1.714190,     -1.097620,      0.277960,
		      0.980010,      0.648060,     -0.552150,     -0.780120,     -0.210580,
		      0.101310,     -0.157720,     -0.272030,      1.373650,      1.139530,
		     -0.550440,     -0.319140,      0.798850,     -0.267070,     -0.404550,
		      0.550660,      1.106490,      0.158310,      1.425490,      0.552600,
		      0.906040,     -0.067560,      1.327100,      0.694450,      1.603710,
		     -0.035750,      0.117470,     -0.528720,      0.731330,      0.166620,
		      0.700540,     -0.182320,      0.645090,      1.315320,      0.943870,
		      0.431310,     -0.459580,      0.715120,      0.689770,      0.241670,
		     -1.186550,     -0.336850,      1.362430,      0.555030,     -0.780200,
		     -0.044100,      0.114700,      0.198160,      1.559770,      0.296720,
		     -1.007580,     -0.876270,     -0.062290,     -0.516130,      0.083090,
		      0.118810,      0.027750,      0.000000,      0.422490,      0.938490,
		     -1.033220,     -0.939750,      0.104620,     -0.018600,      1.030830,
		     -0.004390,      0.123410,      0.079600,     -0.162390,      0.249800,
		     -0.171470,      0.176350,      0.942060,     -0.336830,     -1.566560,
		      0.409890,      0.450540,     -1.088120,     -1.557100,      0.048890,
		      0.515520,      1.062620,      0.087090,     -0.818520,      0.023080,
		      1.495460,      1.189070,      0.337100,     -1.262740,      0.710560,
		     -0.562950,     -2.034120,     -0.695720,     -0.067010,     -0.250660,
		      0.037420,     -0.617490,      0.045660,     -0.548440,     -0.135080,
		     -1.018080,     -0.912710,     -0.604800,      0.675410,     -1.284080,
		     -1.197920,     -1.087330,     -0.227980,      0.354310,      0.224200,
		     -0.278530,     -0.052540,      1.243950,      0.376930,      0.289080,
		      0.158350,     -0.220980,     -1.418230,     -1.713450,     -0.483470,
		     -0.135720,     -0.538920,      0.596630,      1.015970,     -0.916480,
		     -1.175950,     -0.173720,      0.584190,      0.257270,     -1.280240,
		     -1.026260,     -0.103890,      0.049820,      0.683800,     -0.008070,
		     -0.641420,      0.334100,      0.504270,     -0.974660,     -0.970820,
		      0.000000,     -0.073050,     -1.463150,     -0.146590,      0.752300,
		     -0.681880,      0.761840,      0.056960,      0.530110,     -0.377630,
		     -0.372170,      0.040170,      0.051840,      0.874520,      0.740360,
		     -0.544950,     -1.162030,      0.574670,      0.218480,     -0.750200,
		      0.497260,      1.928060,      0.783870,     -0.351600,     -0.803850,
		      0.309030,      0.680260,      0.246530,     -0.494880,     -0.502600,
		      0.001040,      0.457600,     -0.985110,     -0.977530,      0.950020,
		      1.630430,     -0.390250,     -0.611420,      0.881870,      0.025330,
		     -0.303720,     -0.173680,     -0.318170,     -0.537790,      0.903160,
		      0.469120,      0.018480,      0.207840,      0.732240,      0.507300,
		      0.762420,     -0.411870,      0.247970,     -0.277870,      0.434600,
		     -0.094490,     -0.102580,      0.117170,      0.824190,      0.260460,
		      0.275980,     -0.075490,     -0.304610,     -0.301770,      0.264970,
		      0.991460,     -0.829620,     -0.710430,      0.933920,      0.488520,
		     -0.330390,      0.152400,     -0.196340,      0.216200,      0.023420,
		      0.308360,     -0.406120,      0.556460,      0.247170,      0.179130,
		      1.240430,      1.142950,     -0.171420,      0.325960,      0.000000,
		     -0.233740,      0.798460,      0.700640,      0.442880,      0.525100,
		     -0.751520,     -1.303820,      0.409170,      0.500910,      0.151950,
		     -0.136260,     -0.767910,     -0.442860,     -0.564390,      0.668050,
		      1.508070,      0.141320,     -1.003500,      0.285550,      0.234070,
		     -0.506730,     -0.868840,      0.054300,      0.513320,     -0.255890,
		      0.063200,     -0.096960,     -0.044030,      0.273430,      0.057480,
		      0.462830,     -0.177470,      1.178990,      0.210210,     -1.418840,
		     -1.427740,     -0.355400,      0.478750,      1.190860,     -0.271420,
		     -0.106930,      0.036650,      0.913540,     -0.219790,     -0.238060,
		     -0.845140,      0.670420,     -1.120090,     -0.613900,     -1.025060,
		      0.000920,      0.122770,      0.627690,     -0.961580,      0.505930,
		      1.131930,     -0.075410,     -1.241370,     -0.548310,      0.073280,
		     -0.123570,     -1.311410,     -0.003690,     -0.069530,     -0.674140,
		      0.051710,     -0.094790,     -0.157680,     -0.530800,     -0.684580,
		     -0.627570,      0.186870,     -0.314190,     -1.160130,     -0.535760,
		      0.715290,      0.478820,      0.182330,     -0.546920,     -0.440180,
		      0.134410,     -0.048860,     -0.383310,     -0.108560,      0.000000,
		     -0.110020,      0.468080,     -1.028670,      0.286500,     -0.202530,
		     -0.195180,      1.597460,     -0.026920,     -0.059320,     -0.338950,
		      0.493130,     -0.092000,     -0.296930,      0.735870,      0.337200,
		     -0.082260,     -0.273210,      1.030520,      0.188240,     -0.882660,
		     -0.300270,      1.031530,      0.453320,     -0.110490,     -0.068130,
		      0.843910,      0.491950,     -0.192940,     -0.511180,     -0.235680,
		     -0.065060,      0.216730,     -0.721160,     -0.844010,      0.688830,
		      1.960880,     -0.022280,     -0.954600,     -0.361930,      0.082270,
		      0.187450,      0.500940,     -0.643030,     -0.502300,     -0.210840,
		      0.775370,     -0.435260,      0.545740,     -0.118070,     -0.043750,
		      0.216750,     -0.390760,     -0.690110,     -0.558490,      0.222460,
		     -0.176480,      0.130630,     -0.340470,      0.765980,     -0.237730,
		     -0.971440,      0.035130,     -0.496470,     -0.105750,      0.591600,
		      0.665510,      0.153030,     -0.322340,     -0.087470,      0.237980,
		      0.389240,      0.519590,      0.748130,      0.643180,     -0.462050,
		     -0.671380,     -0.128520,      0.662850,      0.430790,      0.400200,
		      0.140190,      0.592100,      0.173280,     -0.138900,      0.013860,
		      0.000000,     -0.648090,     -0.289190,     -0.167590,     -0.152590,
		      0.536080,      0.945240,     -1.186600,     -0.588030,      0.860890,
		      0.444040,     -0.002970,      0.423940,      0.246620,     -0.625440,
		     -1.041770,     -0.460950,      0.431560,     -0.345650,     -0.640460,
		      0.182390,      0.174310,     -0.416080,     -0.485430,      0.039230,
		      0.025070,     -1.045230,     -0.442340,      0.925670,      1.657750,
		      0.456710,      0.004350,     -0.100550,     -0.202350,      0.809850,
		     -0.053620,     -1.252180,     -1.156970,     -0.891240,     -0.905840,
		      0.920580,     -0.368590,     -0.075160,      0.263090,     -0.135660,
		     -0.537070,     -0.170410,     -0.346000,      0.278470,     -0.159860,
		      0.413430,      0.183970,     -0.130640,      0.295190,      1.455450,
		     -0.733570,     -0.017250,      1.113530,      0.572600,     -0.948250,
		     -0.737670,     -0.445960,     -0.116470,      0.021650,     -0.337110,
		      0.560410,      0.357040,     -0.592310,      0.007460,      0.178820,
		     -0.278700,     -0.159590,     -0.093850,     -0.072560,     -0.759430,
		      0.036560,      0.533880,     -0.466310,     -0.124440,      0.119130,
		     -0.647800,     -0.258500,      0.699370,     -0.059950,      0.792550,
		     -0.402190,      0.609320,      0.000000,     -0.133180,      0.071400,
		      0.263590,     -0.290630,     -0.523240,     -0.850500,      0.785770,
		      1.507650,     -0.484110,     -0.903190,      0.085670,      0.450810,
		     -0.112660,     -0.963920,      0.996260,      0.581020,     -0.722740,
		     -0.239760,      0.949190,      0.781740,     -0.389040,     -0.251950,
		      0.961120,      0.421680,      0.023590,      0.747110,      1.344180,
		      0.148540,     -0.985620,     -0.842350,      0.186920,      0.581690,
		      0.808150,     -0.744910,     -0.569600,      0.328270,      1.903840,
		      0.974640,     -0.205960,     -0.343590,     -0.086430,     -0.466930,
		      0.481220,      0.482840,      0.449570,      0.039510,      0.028300,
		     -1.079990,      0.967910,      0.022600,     -0.079720,      0.362000,
		      0.625020,     -0.852680,     -1.064020,     -0.621310,     -0.304210,
		     -0.338510,     -0.509150,      0.218880,      0.601060,     -0.549140,
		     -0.331640,     -0.445730,     -0.309360,     -0.282560,      0.493880,
		      0.950810,     -0.441610,     -0.472590,      0.039120,      0.292830,
		      0.476890,      0.844390,      0.406510,     -0.151500,     -0.170920,
		      0.551040,      0.011620,      0.435230,      0.366430,     -0.144470,
		     -0.269460,     -1.189300,     -0.340160,      0.217970,     -0.091070,
		      0.000000,      0.226960,      0.003330,      0.132320,     -0.057940,
		      0.120650,      1.122380,      0.135540,     -1.621780,     -0.432490,
		      1.221530,      0.020900,     -1.318320,      0.033490,      1.138790,
		     -0.023310,     -0.122120,      0.091860,      0.168720,     -0.723480,
		     -0.940240,      0.068730,      0.011580,     -1.175130,     -0.482940,
		      0.096600,     -0.630560,     -1.717860,     -1.056850,      0.477190,
		      1.460950,      0.245620,     -0.255840,     -0.511320,     -0.387320,
		      0.793070,     -0.013470,     -0.898890,     -0.583310,     -0.117600,
		     -0.396700,      0.169990,      0.074240,     -0.345570,      0.665750,
		      0.269010,     -0.659780,     -0.140990,      0.836530,     -0.378710,
		     -0.825550,     -0.153700,      0.859720,     -0.049170,      0.139590,
		      1.103300,      0.326670,      0.372710,      0.526340,      0.929750,
		     -0.340800,      0.160750,      0.116490,      1.134150,      0.028170,
		     -1.085100,      0.041820,      0.415360,     -0.309290,      0.376670,
		      0.464750,     -0.334540,      0.193860,      0.355980,     -0.037710,
		      0.240300,      0.561410,      0.459450,      0.088890,     -0.215450,
		     -0.705720,     -0.669200,     -0.202140,      0.461590,      0.670640,
		     -0.253040,     -0.307000,     -0.366650,      0.029720,      0.000000,
		     -0.410900,      0.419810,     -0.246310,      0.746770,      0.400530,
		     -1.258950,     -0.967750,      1.022750,      1.246120,     -0.693350,
		     -0.652480,      0.657010,      1.084860,     -0.448730,     -1.234390,
		      0.703830,      0.505650,     -0.618380,     -0.143070,      0.117130,
		      0.301780,     -0.132290,      0.509500,      0.428980,      0.086520,
		      0.177910,      1.216890,      1.706830,      0.543590,     -1.253590,
		     -1.083540,      0.164670,      0.277260,      1.586830,     -0.115000,
		     -0.748840,      0.052210,      1.107080,      0.534780,      0.280960,
		     -0.503210,      0.114210,     -0.434730,     -0.717190,     -0.454130,
		      0.828480,      0.361100,      0.124460,     -0.923920,      0.564270,
		      0.313170,     -0.611150,      0.451600,      0.911430,     -0.207300,
		     -0.369140,     -0.506430,     -0.315580,     -0.221260,     -0.161050,
		     -0.473460,      0.101580,     -0.101280,      0.200850,      0.668710,
		     -0.568130,     -0.454320,     -0.255010,      0.076590,      0.120080,
		      0.169660,      0.009330,     -0.082160,      0.765890,      0.111650,
		     -0.629950,     -0.503700,      0.412670,     -0.418820,      0.267330,
		      0.358780,     -0.050170,     -0.657450,     -0.014820,     -0.468860,
		     -0.475570,      0.349270,      0.263410,      0.646000,      0.000000,
		     -0.185760,      0.248990,     -0.108460,     -0.486640,     -0.650250,
		      0.560810,      1.313840,      0.159050,     -0.618610,      0.035370,
		      1.211020,      0.240490,     -1.273150,     -0.095160,      0.894620,
		     -0.862980,     -0.384510,      0.704480,      0.270820,     -0.434940,
		     -0.088150,      0.565050,     -0.039260,     -0.804680,     -0.088750,
		      0.381140,     -0.997470,     -1.829400,     -0.302450,      0.489780,
		      1.317440,      0.336180,      0.289210,     -0.875210,     -0.458300,
		      0.281760,     -0.041680,     -0.638640,     -0.831760,      0.304120,
		      0.559150,      0.447040,      0.533520,      0.048880,      0.931820,
		      0.341910,     -0.500610,     -0.340540,      1.170060,      0.097040,
		     -0.568770,     -0.375420,      0.131690,     -0.083980,     -0.445800,
		     -0.064250,      0.502520,      0.546320,     -0.385090,      0.105330,
		      0.396850,      0.173580,      0.712540,      0.945060,      0.415290,
		     -0.072020,     -0.135510,      0.115620,     -0.011630,      0.206860,
		     -0.180880,     -0.638490,     -0.026780,     -0.495480,     -0.041860,
		      1.185410,      0.158140,     -0.211150,     -0.411770,     -0.428380,
		     -0.070310,      0.239290,     -0.091400,      0.437110,      0.774280,
		     -0.191230,     -0.455160,     -0.268330,     -0.652890,      0.097030,
		      0.000000,      0.256380,     -0.489050,     -0.101810,      0.575810,
		      0.874230,      0.125830,     -0.825390,     -0.057140,      0.392640,
		      0.084660,     -1.045040,     -0.625500,      0.762400,      0.388760,
		     -0.187110,      0.087370,      0.412130,      0.151410,      0.170190,
		      0.131450,     -0.232520,     -0.422500,     -0.368950,      0.582750,
		      0.160620,     -0.854430,      0.278880,      1.294740,      0.333130,
		      0.034760,     -0.424120,     -0.829910,     -0.195650,      0.014810,
		      1.243400,      0.548630,      0.225400,      0.208390,      0.362090,
		     -0.373810,     -0.338530,     -0.331720,      0.173290,     -0.325290,
		     -0.563830,     -0.537800,      0.603710,      0.266510,     -0.305260,
		     -0.591990,     -0.255600,     -0.017530,     -0.359200,      0.135820,
		      0.695210,      0.182880,     -0.004510,     -0.278710,      0.390760,
		     -0.130120,     -0.147290,     -1.151570,     -0.130510,     -0.152860,
		      0.190210,     -0.018770,     -0.587800,     -0.286560,      0.095380,
		     -0.257960,     -0.416140,      0.442400,      0.479210,      0.674490,
		      0.629830,     -0.526370,      0.142670,     -0.342360,     -0.256610,
		      0.261350,     -0.408790,     -0.302890,      0.341010,      0.270650,
		     -0.255750,      0.023810,      0.569080,     -0.242820,      0.272280,
		      0.313550,     -0.156930,      0.000000,     -0.413990,      0.668740,
		      0.357560,      0.411800,     -0.572730,     -0.352050,      0.054860,
		      0.345350,     -0.174680,     -0.192160,      0.524620,      0.855090,
		      0.087440,     -0.958980,      0.448530,      1.058160,     -0.538370,
		     -0.397490,      0.080180,     -0.347070,      0.200220,      0.731020,
		      0.360780,     -0.252460,     -0.139490,      0.618090,      0.563180,
		     -0.913130,     -1.391610,      0.121290,      0.726300,      0.800430,
		     -0.093500,      0.452690,     -0.340980,     -0.096490,     -0.316200,
		      0.091170,     -0.224210,     -0.357730,      0.049930,      0.883870,
		      0.060820,      0.769140,      0.359740,      0.580510,     -0.255040,
		     -0.313980,     -0.315610,      1.042930,      0.793540,     -0.018760,
		     -0.152310,     -0.223980,     -0.171800,     -0.233410,     -0.580600,
		      0.104390,      0.271720,      0.109670,      0.344540,      0.623640,
		      0.203420,      0.181070,      0.080740,      0.267400,      0.378030,
		     -0.062460,     -0.274970,      0.147630,     -0.096350,     -0.127620,
		     -0.766330,     -0.506510,      0.288020,      0.990370,      0.062900,
		     -0.021890,     -0.767270,     -1.026900,     -0.800840,     -0.236410,
		     -0.201530,      0.085160,      0.120410,      0.150190,      0.131550,
		     -0.483720,     -0.138360,     -0.342980,     -0.185940,      0.310720,
		      0.000000,     -0.086130,      0.482570,     -1.024690,     -0.498400,
		      0.509460,      0.443670,      0.421200,     -0.358060,      0.705020,
		      0.440270,     -0.307240,     -0.838600,     -0.240550,      0.609030,
		     -0.495620,     -0.932580,      0.405870,      0.495680,      0.206890,
		      0.907280,      0.541170,     -0.625360,     -0.805520,     -0.118250,
		      0.588930,     -0.234010,     -0.519970,      0.820730,      1.667090,
		     -0.316650,     -0.553120,     -0.104860,      0.246810,      0.010430,
		     -0.290960,     -0.283600,      0.220700,      0.586550,      0.762700,
		      0.622710,     -0.199360,     -0.789450,     -0.277150,     -0.272340,
		     -0.635060,     -0.498010,     -0.206990,      0.587110,     -0.005540,
		     -0.602370,     -0.487850,     -0.082120,     -0.649900,     -0.526650,
		     -0.203750,     -0.117770,      0.199610,      0.121380,     -0.360440,
		     -0.553250,     -0.597570,      0.004290,     -0.705610,     -0.489190,
		      0.041840,     -0.020080,      0.003330,     -0.484840,     -0.031580,
		      0.496400,     -0.073570,     -0.604220,      0.072770,     -0.185110,
		      0.091290,      0.125760,      0.233000,      0.097680,      0.188980,
		     -0.083730,     -0.252330,     -0.494380,     -0.234050,     -0.295770,
		     -0.039720,      0.203810,      0.126470,      0.339090,      0.213860,
		      0.314640,      0.034030,     -0.357430,     -0.108890,      0.000000,
		      0.171960,     -0.544210,      0.444850,      0.706680,      0.277810,
		     -0.311680,     -0.301380,      0.274070,     -0.211160,     -0.479420,
		     -0.194090,      0.541420,      0.728720,     -0.338940,     -0.574990,
		      0.281020,      0.432400,     -0.300550,     -0.507240,     -0.808310,
		     -0.455920,      0.315280,      0.951620,      0.380400,     -0.552510,
		     -0.202950,      0.566850,     -0.274870,     -1.091250,     -0.608050,
		      0.382600,      0.228090,     -0.156880,     -0.394350,      0.249740,
		      0.341470,      0.449410,     -0.336910,     -0.631860,     -0.279200,
		      0.143050,      0.406390,      0.910000,      0.076130,      0.761950,
		      0.645040,      0.960290,     -0.366450,     -0.331310,     -0.122760,
		      0.566820,      0.949360,      0.957670,      0.624170,      0.242440,
		      0.000400,      0.089550,      0.256500,      0.334900,      0.037060,
		     -0.061900,     -0.260400,      0.486990,     -0.179540,      0.182690,
		     -0.083270,     -0.617790,      0.499620,      0.283980,      0.131920,
		     -0.275690,     -0.220540,     -0.193530,     -0.222360,     -0.129290,
		      0.446980,      0.384290,     -0.270440,      0.460410,      0.116990,
		     -0.335020,     -0.494570,     -0.073130,      0.077580,     -0.044230,
		     -0.095690,     -0.210810,     -0.457540,      0.481550,     -0.139540,
		     -0.319650,     -0.088820,      0.432260,     -0.193080,      0.000000,
		     -0.386730,      0.668590,      0.424130,     -0.340040,     -0.960700,
		     -0.140710,      0.314090,      0.324200,      0.073340,      0.380270,
		      0.423330,     -0.218470,     -0.340430,      0.118720,      0.873100,
		      0.197650,     -0.919690,     -0.376650,      0.113730,      0.374730,
		      0.978200,      0.555930,     -0.599380,     -0.658120,     -0.132660,
		      0.174060,     -0.578090,     -0.288270,      0.799980,      1.231130,
		     -0.497290,     -0.574340,      0.298250,      0.724900,      0.365760,
		      0.156200,     -0.600070,     -0.025800,      0.461780,      0.512750,
		      0.031030,     -0.390230,     -0.820750,     -0.085510,     -0.214340,
		     -0.428460,     -0.589960,     -0.278170,      0.172500,     -0.003220,
		     -0.894150,     -0.608150,     -0.445170,     -1.023950,     -0.824240,
		     -0.566680,     -0.453980,     -0.337860,     -0.059090,     -0.153540,
		      0.096340,     -0.141190,     -0.551320,     -0.384700,     -0.426150,
		     -0.093710,      0.011000,     -0.516210,     -0.781670,      0.144190,
		      0.736950,      0.544870,     -0.345530,     -0.240900,     -0.247540,
		     -0.145860,      0.072170,      0.287930,     -0.290970,     -0.058650,
		     -0.411090,     -0.572550,     -0.339800,     -0.342040,     -0.218750,
		      0.305060,      0.112850,      0.121700,      0.244680,     -0.392440,
		     -0.010150,     -0.490140,     -0.264670,     -0.178850,      0.088770,
		      0.000000,      0.085000,      0.290890,     -0.843790,      0.184210,
		      0.762820,      0.323860,     -0.136440,     -0.168900,      0.170390,
		     -0.301790,     -0.235600,      0.188200,      0.472930,      0.289990,
		     -0.421830,     -0.126760,      0.514240,      1.067360,     -0.034850,
		     -0.610630,     -0.790790,     -0.850920,      0.038410,      0.476760,
		      0.777550,     -0.027410,      0.395060,      0.296990,     -0.706610,
		     -0.843150,      0.432290,      0.794640,     -0.170860,     -0.768220,
		     -0.692410,     -0.501420,      0.331400,      0.491550,     -0.551520,
		     -0.325200,      0.100350,      0.437510,      0.470920,      0.433520,
		      0.198260,      0.404950,      0.490920,      1.215410,      0.289500,
		     -0.182550,      0.005870,      0.676660,      0.634090,      0.987000,
		      0.618690,      0.650770,      0.045540,      0.023190,      0.125310,
		      0.160990,     -0.107200,      0.211380,      0.411240,      0.335730,
		     -0.042830,     -0.206720,     -0.073080,      0.611720,      0.753650,
		      0.032990,     -0.153060,      0.161180,      0.067070,      0.073100,
		      0.115510,     -0.245190,      0.110080,      0.343320,     -0.006780,
		     -0.148860,      0.087680,      0.159530,      0.236320,     -0.067430,
		      0.476920,     -0.374140,     -0.118630,     -0.063050,     -0.047720,
		      0.241760,      0.002750,      0.087900,      0.011410,      0.078060,
		     -0.646640,     -0.091030,      0.000000,     -0.277380,     -0.248890,
		      0.654800,      0.595120,      0.156830,     -0.620820,     -0.368640,
		      0.293470,     -0.013210,     -0.159530,      0.170520,      0.113140,
		     -0.388440,     -0.419580,     -0.133260,      0.381460,      0.279010,
		     -0.882650,     -0.395290,      0.082180,      0.496920,      0.808370,
		      0.407690,     -0.346300,     -0.769390,     -0.196090,     -0.109030,
		     -0.378710,      0.427080,      0.578270,     -0.129080,     -0.894360,
		     -0.338300,      0.569120,      0.693830,      0.160960,      0.097790,
		     -0.481290,      0.117190,      0.379110,      0.328860,     -0.076260,
		     -0.466460,     -0.778400,     -0.140700,     -0.521880,     -0.295980,
		     -0.738010,     -0.122690,     -0.315270,     -0.069970,     -0.546290,
		     -0.473050,     -0.204710,     -0.633700,     -0.918360,     -0.177590,
		     -0.557410,     -0.304530,     -0.195880,     -0.152410,      0.102040,
		     -0.082210,     -0.153220,     -0.029370,     -0.033770,     -0.424440,
		     -0.420820,     -0.599270,     -0.256870,      0.153890,      0.424180,
		      0.004780,      0.033580,     -0.305420,     -0.072310,      0.068100,
		      0.058390,     -0.316890,     -0.092160,     -0.190460,     -0.503610,
		     -0.899140,     -0.137300,     -0.118060,      0.102040,      0.251020,
		      0.378220,      0.261580,      0.076730,     -0.084130,     -0.604600,
		     -0.125130,     -0.317220,      0.002090,     -0.138410,      0.044630,
		      0.000000,      0.373860,      0.301500,     -0.312520,     -0.382060,
		     -0.172390,      0.803830,      0.456860,     -0.161290,      0.066500,
		      0.142570,     -0.505540,     -0.012020,      0.742390,      0.612440,
		      0.222460,     -0.447230,     -0.375440,      0.913170,      1.085600,
		     -0.205440,     -0.389150,     -0.354320,     -0.389640,     -0.203970,
		      0.016800,      0.603290,      0.118060,      0.126190,     -0.403750,
		     -0.681530,     -0.487850,      0.632500,      0.616880,     -0.065710,
		     -0.718820,     -0.249410,     -0.343260,      0.275400,      0.082980,
		     -0.419170,     -0.302950,      0.536030,      0.676350,      0.500460,
		      0.368640,      0.239000,      0.377940,      0.235740,      0.201110,
		      0.552780,      0.379860,      0.312750,      0.474450,      0.413620,
		      0.478820,      0.474990,      0.406150,      0.554480,      0.265220,
		      0.164170,      0.230210,     -0.314250,      0.543570,      0.292580,
		      0.055380,     -0.252650,     -0.411500,     -0.028450,      0.452990,
		      0.840020,      0.001890,      0.371280,      0.508330,      0.303420,
		      0.074270,     -0.295110,     -0.132580,      0.072210,      0.300450,
		     -0.039960,     -0.242810,     -0.310800,     -0.063410,     -0.034220,
		      0.160100,      0.145350,      0.000120,     -0.473450,     -0.061050,
		      0.089510,      0.133510,      0.310670,     -0.218300,      0.184910,
		      0.060950,      0.088590,      0.235460,      0.228990,      0.000000,
		     -0.183600,     -0.194570,     -0.488510,      0.310440,      0.121110,
		     -0.067190,     -0.505330,     -0.025270,      0.297990,     -0.048520,
		      0.285710,      0.323450,     -0.331410,     -0.345450,      0.094830,
		      0.548190,      0.322010,     -0.457000,     -1.037360,     -0.167280,
		      0.178690,      0.215630,      0.409640,      0.465780,      0.067840,
		     -0.652150,     -0.314400,      0.141160,      0.554010,      1.064500,
		      0.931450,     -0.376570,     -0.736780,     -0.302250,      0.671050,
		      0.438930,      0.030430,     -0.032920,     -0.406970,      0.115700,
		     -0.061230,     -0.620350,     -0.382990,     -0.541950,     -0.524240,
		     -0.098840,     -0.425560,     -0.106650,     -0.267810,      0.012760,
		     -0.398840,     -0.180140,     -0.293830,     -0.276770,     -0.247880,
		     -0.212150,     -0.534930,      0.089670,     -0.268310,      0.082540,
		     -0.321390,     -0.214740,     -0.142100,     -0.014540,      0.308700,
		      0.430840,      0.481550,      0.130110,     -0.318820,     -0.137290,
		      0.057910,     -0.229280,      0.154500,      0.265850,      0.043370,
		     -0.132210,     -0.084950,      0.179330,     -0.154410,     -0.210140,
		     -0.215600,     -0.228730,     -0.100940,     -0.323740,     -0.355770,
		     -0.453810,     -0.041460,      0.290850,      0.001700,     -0.100840,
		     -0.216060,     -0.300770,     -0.061750,      0.040240,      0.218610,
		      0.103670,      0.114240,     -0.223560,      0.176390,      0.000000,
		     -0.044180,      0.866490,      0.206770,     -0.069720,     -0.190960,
		     -0.306590,      0.533020,      0.322140,     -0.044890,      0.149080,
		     -0.243380,     -0.532190,     -0.110510,      0.094310,      0.323760,
		     -0.073310,     -0.626270,     -0.067550,      0.964460,      0.574000,
		     -0.052840,     -0.047480,      0.118870,     -0.449680,     -0.028700,
		      0.290070,      0.678950,      0.100410,     -0.665650,     -1.102170,
		     -0.668700,      0.093570,      0.443740,      0.373300,     -0.390230,
		     -0.411640,      0.090560,      0.052130,      0.116250,     -0.118970,
		     -0.164130,      0.411540,      0.664460,      0.698790,      0.629030,
		      0.121100,      0.477450,      0.425010,      0.397790,     -0.500310,
		      0.009740,      0.204480,      0.225970,      0.125890,      0.383640,
		      0.427430,      0.210550,      0.298490,      0.356830,      0.329730,
		      0.312040,      0.342570,      0.089970,      0.211610,      0.158400,
		      0.157800,      0.160370,      0.012150,     -0.112100,      0.069120,
		     -0.067610,     -0.035780,     -0.144390,     -0.048100,     -0.079660,
		      0.241000,     -0.206580,     -0.191250,     -0.006620,      0.248980,
		     -0.168930,     -0.002710,     -0.445840,      0.003460,      0.290250,
		      0.313080,     -0.262180,     -0.393760,     -0.399140,     -0.000660,
		      0.110010,     -0.028890,      0.225600,      0.003240,     -0.098120,
		     -0.001590,     -0.072940,      0.046200,      0.561750,     -0.164390,
		      0.000000,     -0.087850,      0.003170,      0.245810,      0.124360,
		      0.155080,      0.140270,     -0.184390,     -0.107350,     -0.197650,
		     -0.286090,      0.263510,      0.519280,      0.449350,     -0.403280,
		     -0.750420,      0.196760,      0.835650,      0.356350,     -0.897660,
		     -0.652570,      0.141000,      0.049920,     -0.389890,      0.128740,
		      0.549120,      0.193320,     -0.890330,     -0.403460,      0.325170,
		      0.868200,      0.746690,      0.351000,     -0.069870,     -0.068910,
		      0.005480,      0.440370,     -0.051850,     -0.194070,     -0.113130,
		      0.124160,      0.502540,     -0.047960,     -0.753820,     -0.700000,
		     -0.559830,     -0.530330,     -0.083290,     -0.627760,     -0.565550,
		     -0.248330,      0.328040,      0.163740,     -0.143160,     -0.034730,
		     -0.256730,     -0.338610,     -0.069480,     -0.343750,     -0.099930,
		     -0.087580,      0.106780,     -0.224090,      0.058900,     -0.152310,
		     -0.213960,     -0.198710,     -0.140680,      0.255070,      0.402360,
		      0.496620,      0.055280,      0.325140,      0.059630,      0.366410,
		     -0.003660,     -0.016960,     -0.167610,     -0.053050,      0.045680,
		     -0.014240,     -0.226500,     -0.589340,      0.057350,     -0.058600,
		     -0.294390,     -0.174810,      0.135420,     -0.130090,      0.153270,
		      0.402770,      0.040810,     -0.259710,     -0.527850,      0.162760,
		      0.232910,      0.224510,      0.107080,     -0.168160,     -0.450880,
		     -0.090570,     -0.344360,      0.000000,     -0.232870,      0.014940,
		      0.017570,     -0.128870,     -0.250310,      0.198210,      0.057210,
		      0.177450,      0.096720,      0.439870,     -0.183530,     -0.231310,
		     -0.241600,      0.432750,      0.288680,      0.150220,     -0.538100,
		     -0.309870,      0.647610,      0.359480,     -0.055600,     -0.121690,
		      0.327790,      0.043150,     -0.533180,     -0.478440,      0.782750,
		      0.794640,      0.162960,     -0.795110,     -0.931100,     -0.395990,
		      0.129130,      0.201860,     -0.009420,     -0.405700,     -0.036030,
		      0.387020,      0.330420,      0.124860,     -0.350070,     -0.036750,
		      0.334590,      0.794790,      0.767910,      0.406350,      0.198760,
		      0.520820,      0.534030,      0.377220,     -0.278960,     -0.235600,
		      0.075910,      0.056760,     -0.142070,      0.267280,     -0.176930,
		     -0.053600,     -0.152320,      0.450370,      0.017280,     -0.023150,
		     -0.245010,     -0.021040,      0.317900,      0.083930,      0.027830,
		     -0.052340,     -0.005720,      0.345040,      0.301060,     -0.343350,
		     -0.420190,     -0.534260,     -0.212440,     -0.101220,     -0.094820,
		     -0.020830,     -0.046550,      0.079590,      0.129480,     -0.096180,
		     -0.449320,     -0.082300,     -0.286260,      0.265140,     -0.166650,
		     -0.107000,     -0.219430,     -0.084680,     -0.064100,      0.031180,
		      0.081400,      0.042240,      0.199030,      0.062060,      0.056190,
		     -0.166590,     -0.122100,      0.104380,      0.076720,     -0.213020,
		      0.000000,      0.092010,      0.300070,     -0.546340,      0.322300,
		      0.443560,     -0.125690,     -0.262160,      0.177970,      0.188830,
		     -0.528900,     -0.317250,      0.366820,      0.565320,     -0.020800,
		     -0.108870,     -0.169630,      0.295150,      0.547370,     -0.411580,
		     -0.849340,     -0.105250,      0.275490,     -0.082930,     -0.313010,
		     -0.024380,      0.485400,     -0.381660,     -0.731540,     -0.223990,
		      0.768930,      0.784710,      0.265760,      0.078370,      0.056110,
		      0.204690,      0.425260,      0.257460,     -0.229840,     -0.424460,
		     -0.254500,      0.211470,      0.358770,      0.108780,     -0.365920,
		     -0.782180,     -0.350440,     -0.198320,     -0.303240,     -0.189100,
		     -0.615260,     -0.216440,      0.162580,      0.084950,     -0.171860,
		      0.099300,     -0.301940,      0.111450,      0.272210,     -0.056760,
		      0.071640,      0.132790,      0.481760,      0.138530,     -0.090680,
		     -0.296420,     -0.350180,      0.084610,      0.086370,     -0.182110,
		      0.004980,     -0.148010,      0.181430,      0.151540,      0.231220,
		     -0.074450,     -0.246360,     -0.331460,     -0.041310,     -0.075250,
		     -0.272570,     -0.039920,      0.082740,      0.142940,     -0.135320,
		     -0.021390,     -0.161900,     -0.226190,     -0.108700,      0.148660,
		      0.369330,      0.397220,      0.526910,     -0.277070,     -0.024800,
		      0.004470,      0.228270,     -0.088980,     -0.040780,     -0.131450,
		     -0.258630,      0.002510,      0.052330,      0.097700,      0.000000,
		      0.017250,     -0.049760,      0.219860,      0.111560,     -0.276410,
		     -0.188560,      0.403420,      0.161900,     -0.109450,      0.340530,
		      0.559960,     -0.245040,     -0.598780,     -0.112060,      0.233690,
		      0.088220,     -0.434460,     -0.740710,      0.371640,      0.824100,
		     -0.067190,     -0.320300,      0.135640,      0.464620,      0.078240,
		     -0.402380,      0.205430,      0.678650,      0.384710,     -0.428490,
		     -0.765170,     -0.322470,      0.050030,      0.028500,     -0.144920,
		     -0.394780,     -0.441370,      0.062210,      0.349650,      0.508010,
		      0.078860,     -0.277510,     -0.333080,     -0.002580,      0.711250,
		      0.578030,      0.517210,     -0.000810,      0.497290,      0.316580,
		     -0.000590,     -0.101270,     -0.106600,      0.263640,      0.217400,
		      0.153850,      0.123370,     -0.176580,     -0.199050,     -0.144310,
		      0.019700,     -0.098740,     -0.128430,     -0.269980,     -0.036560,
		      0.139390,     -0.237800,     -0.234970,      0.245540,      0.166500,
		      0.420870,      0.224170,     -0.544640,     -0.192780,     -0.378630,
		      0.199740,     -0.355720,     -0.107150,     -0.362770,      0.168940,
		      0.293830,      0.091770,     -0.096240,     -0.253950,      0.223410,
		     -0.062570,     -0.405920,     -0.152050,     -0.168570,     -0.161360,
		     -0.082690,     -0.093940,      0.009030,     -0.199050,     -0.034320,
		      0.361530,      0.257160,      0.373920,      0.362250,     -0.174520,
		     -0.264960,      0.073580,      0.045380,     -0.185500,      0.000000,
		     -0.368150,      0.521540,      0.107110,     -0.456140,      0.322660,
		      0.469040,     -0.393870,     -0.155670,      0.213410,      0.144380,
		     -0.647740,     -0.153660,      0.480930,      0.106210,     -0.488570,
		      0.131550,      0.487370,      0.854930,      0.123960,     -0.648210,
		      0.014170,      0.364570,     -0.181460,     -0.539060,      0.013290,
		      0.339720,      0.220880,     -0.881240,     -0.736120,      0.130650,
		      0.602380,      0.376910,     -0.145200,     -0.095050,      0.031000,
		      0.204070,      0.296220,      0.004490,     -0.491920,     -0.456300,
		     -0.456280,      0.111500,      0.348670,      0.240720,     -0.248970,
		     -0.543370,     -0.175130,      0.117590,     -0.241420,     -0.172040,
		     -0.093240,     -0.066120,      0.415380,      0.081280,     -0.332960,
		      0.026760,     -0.258390,      0.070820,      0.139930,      0.182170,
		      0.142440,      0.440130,      0.351410,      0.373730,      0.005530,
		     -0.117740,     -0.297710,     -0.072270,     -0.304950,     -0.106640,
		     -0.019080,      0.200020,      0.311510,      0.109180,      0.138550,
		     -0.003300,     -0.104550,     -0.433260,     -0.030920,     -0.041660,
		     -0.194180,      0.090670,      0.183360,      0.331280,     -0.045760,
		     -0.042250,      0.160740,     -0.123830,     -0.035340,      0.044590,
		      0.443020,      0.524260,      0.216020,      0.082870,      0.019770,
		      0.091440,     -0.149020,     -0.312880,     -0.185470,      0.450420,
		      0.416040,      0.284100,      0.219750,      0.221780,      0.381080,
		      0.000000,      0.284070,     -0.055390,     -0.573780,      0.363420,
		     -0.193260,     -0.141920,      0.176060,      0.399680,     -0.178390,
		     -0.395220,      0.601680,      0.498030,     -0.413960,     -0.172340,
		      0.606310,      0.277160,     -0.146870,     -0.828290,     -0.509320,
		      0.515360,      0.395390,     -0.431010,     -0.077190,      0.399240,
		      0.169450,     -0.437490,     -0.405050,      0.773110,      0.825960,
		      0.068020,     -0.519910,     -0.525430,      0.143320,      0.339810,
		      0.186980,     -0.086620,     -0.127820,     -0.138720,      0.080000,
		      0.483180,      0.505650,      0.091960,     -0.145100,     -0.304270,
		     -0.345990,      0.185650,      0.393320,      0.154840,      0.109360,
		      0.293200,      0.651940,     -0.025620,     -0.398880,     -0.236540,
		      0.290190,      0.258810,      0.428940,      0.424960,     -0.238270,
		     -0.205560,     -0.133570,     -0.137990,     -0.106830,     -0.311690,
		     -0.233520,     -0.033000,      0.427280,      0.149120,     -0.096980,
		      0.268260,      0.071880,      0.083440,     -0.140510,     -0.323750,
		     -0.177210,      0.126720,     -0.029480,      0.446320,     -0.106190,
		     -0.282140,     -0.165330,      0.171850,      0.194160,      0.421580,
		      0.097520,      0.218550,      0.041630,     -0.077860,     -0.095420,
		      0.125240,      0.050850,     -0.300390,     -0.293130,     -0.027570,
		      0.050240,     -0.274380,      0.274710,      0.198600,     -0.043400,
		     -0.412170,      0.151150,      0.204720,      0.023330,      0.294990,
		     -0.145300,      0.137950,      0.000000,     -0.631340,      0.083850,
		      0.470810,     -0.082350,     -0.010870,      0.155100,      0.184030,
		     -0.217550,      0.213080,      0.225660,     -0.279460,     -0.558390,
		      0.173130,      0.414490,     -0.302780,     -0.625380,      0.028110,
		      0.910530,      0.587690,     -0.259580,     -0.554900,      0.568230,
		      0.288160,     -0.371330,     -0.267370,      0.242910,      0.475810,
		     -0.098300,     -0.717780,     -0.279550,      0.285650,      0.431160,
		      0.150580,     -0.519330,     -0.330690,      0.005040,     -0.118450,
		      0.143740,     -0.035620,     -0.469650,     -0.539480,     -0.378740,
		      0.050360,      0.198140,      0.121570,      0.006710,     -0.359400,
		     -0.249000,      0.055220,     -0.371150,     -0.179390,     -0.115770,
		      0.266250,      0.563080,      0.142710,     -0.264450,     -0.004930,
		     -0.164630,      0.162830,      0.247560,      0.067960,      0.155810,
		      0.229660,      0.640650,      0.459100,     -0.005910,     -0.192880,
		     -0.347340,     -0.181920,     -0.247230,     -0.258540,      0.122970,
		     -0.014410,      0.374890,     -0.184910,      0.189710,      0.005410,
		     -0.218600,     -0.476020,      0.305720,      0.033640,      0.076110,
		      0.087870,     -0.101890,      0.321700,      0.023390,      0.022090,
		      0.139660,      0.004180,     -0.202540,      0.190610,      0.484250,
		      0.029130,     -0.016880,     -0.159640,      0.207540,     -0.000980,
		      0.084830,     -0.006610,      0.119920,      0.076830,      0.078960,
		     -0.128070,     -0.093940,      0.205030,      0.074850,      0.021260,
		      0.000000,      0.335390,      0.627910,     -0.344680,     -0.066230,
		      0.212370,     -0.293110,     -0.170630,      0.441590,      0.376860,
		     -0.061220,     -0.220210,      0.663640,      0.224900,     -0.706750,
		     -0.087000,      0.576900,      0.065590,     -0.449450,     -0.221930,
		     -0.078420,      0.564650,     -0.396440,     -0.551480,      0.339090,
		      0.415770,     -0.145200,     -0.515230,     -0.299260,      0.628320,
		      0.338270,      0.024290,     -0.336340,     -0.100930,      0.361190,
		      0.208770,      0.099420,      0.019060,      0.078560,      0.003980,
		      0.427170,      0.664110,      0.510960,      0.002190,     -0.007770,
		     -0.012350,     -0.019980,     -0.072530,      0.017350,      0.110430,
		     -0.027050,      0.035550,      0.273250,     -0.028360,     -0.269770,
		     -0.229270,      0.106720,      0.037630,     -0.004270,      0.059900,
		      0.068050,     -0.052720,      0.043460,     -0.273560,     -0.256900,
		     -0.372060,     -0.203550,      0.085880,      0.380160,      0.049240,
		     -0.274030,     -0.053490,      0.207540,      0.097770,     -0.062270,
		     -0.170850,     -0.067750,     -0.009180,      0.332320,      0.400990,
		     -0.114180,     -0.371590,     -0.154510,      0.011570,      0.169250,
		      0.042330,      0.204160,      0.115880,      0.069420,      0.096830,
		     -0.143320,      0.255390,     -0.086330,      0.071490,     -0.405470,
		     -0.240630,     -0.167590,      0.011240,     -0.189150,      0.100050,
		      0.069030,      0.051840,      0.215010,     -0.040240,     -0.178430,
		     -0.016870,     -0.052270,      0.095590,     -0.360690,      0.000000,
		     -0.245380,     -0.510220,      0.021020,      0.042040,     -0.171860,
		      0.230810,      0.117280,      0.001920,     -0.283510,      0.118880,
		      0.288880,     -0.463900,     -0.400650,      0.388560,      0.216400,
		     -0.232640,     -0.331090,      0.355070,      0.409480,      0.094290,
		     -0.272950,      0.165550,      0.627870,     -0.315680,     -0.510000,
		      0.162680,      0.555590,      0.411240,     -0.353600,     -0.367820,
		     -0.250500,      0.166690,      0.190690,     -0.014480,     -0.233680,
		     -0.077980,     -0.149960,     -0.333690,     -0.084500,     -0.129150,
		     -0.602500,     -0.519450,     -0.130100,      0.229440,      0.101780,
		      0.171700,      0.210170,      0.099750,      0.016660,      0.112060,
		      0.121200,     -0.318600,     -0.405210,     -0.068320,      0.240350,
		      0.168300,      0.002570,     -0.030350,     -0.102550,     -0.053630,
		     -0.055780,     -0.120340,     -0.055430,      0.123620,      0.502880,
		      0.185100,     -0.046320,     -0.145210,      0.098670,      0.386000,
		     -0.029390,     -0.134690,     -0.005090,      0.191480,      0.183030,
		      0.082100,     -0.091900,     -0.229590,     -0.458550,     -0.088500,
		      0.073780,      0.066440,     -0.005540,      0.216820,      0.015600,
		      0.175920,     -0.102950,     -0.162670,      0.013230,      0.063540,
		     -0.060300,      0.156300,      0.125660,      0.160050,      0.175760,
		     -0.005490,      0.068780,      0.152560,     -0.045110,     -0.159910,
		      0.045400,      0.003740,      0.351750,     -0.014400,     -0.439380,
		     -0.043970,     -0.121870,     -0.135210,      0.101430,      0.000000,
		     -0.160190,      0.394850,      0.095080,     -0.167020,      0.306130,
		     -0.031690,     -0.149660,     -0.026320,      0.378480,      0.076460,
		     -0.276130,      0.173060,      0.620820,      0.029760,     -0.257270,
		      0.227470,      0.311710,     -0.345700,     -0.385830,      0.186130,
		      0.308990,     -0.050400,     -0.553710,     -0.049740,      0.520730,
		      0.108930,     -0.297530,     -0.333950,      0.145850,      0.418780,
		      0.195880,     -0.200310,     -0.076690,      0.148500,      0.367150,
		      0.026100,      0.211050,      0.150120,      0.162950,      0.069570,
		      0.389360,      0.624110,      0.413210,     -0.044870,     -0.085830,
		     -0.032780,      0.197060,     -0.118690,     -0.066760,     -0.135880,
		      0.080000,      0.262250,      0.277720,      0.015440,     -0.116000,
		     -0.125300,      0.045070,      0.008200,     -0.101010,     -0.125130,
		     -0.000770,      0.241800,      0.011230,     -0.045080,     -0.381090,
		     -0.143130,     -0.045030,     -0.002960,     -0.012880,     -0.318560,
		     -0.289850,     -0.086990,     -0.058440,      0.101250,     -0.260190,
		     -0.069760,      0.038540,      0.310310,      0.043040,      0.112420,
		     -0.181880,      0.026470,     -0.092540,     -0.110900,     -0.043580,
		     -0.066590,      0.112110,      0.000280,     -0.006350,     -0.073960,
		     -0.138760,     -0.077100,     -0.017880,     -0.152260,     -0.301390,
		     -0.031720,     -0.018610,     -0.201180,     -0.118960,      0.004620,
		      0.156470,      0.103840,      0.007790,      0.168420,     -0.091320,
		     -0.182940,      0.034630,      0.079260,     -0.490440,     -0.256580,
		      0.000000,      0.022160,      0.093890,     -0.285340,      0.153120,
		     -0.134280,     -0.030980,      0.122710,      0.198310,      0.013280,
		     -0.072460,      0.352470,      0.176500,     -0.569610,     -0.453440,
		      0.186930,     -0.025200,     -0.246860,      0.137460,      0.432860,
		     -0.128090,     -0.286010,     -0.016840,      0.587170,      0.436740,
		     -0.540070,     -0.567030,      0.161810,      0.136800,      0.018780,
		     -0.253090,     -0.049810,      0.150640,      0.023180,     -0.177190,
		     -0.320630,      0.034960,      0.046610,      0.082150,     -0.314750,
		     -0.157810,     -0.418580,     -0.530060,     -0.545510,     -0.082790,
		      0.151500,      0.120110,     -0.205030,     -0.069290,      0.101940,
		     -0.042630,     -0.112840,      0.053400,      0.033980,     -0.156860,
		     -0.114440,     -0.005010,     -0.060990,     -0.162100,     -0.022460,
		      0.066260,      0.217950,      0.153110,     -0.006140,      0.032450,
		      0.111760,      0.339570,      0.025440,      0.050750,      0.058310,
		      0.314060,      0.248990,      0.059730,     -0.092990,      0.164380,
		      0.150510,      0.175540,     -0.093030,      0.012980,     -0.241100,
		     -0.171660,     -0.161960,     -0.061590,      0.071450,      0.094540,
		      0.135760,      0.148450,      0.059390,      0.112200,      0.045700,
		      0.178800,      0.251730,      0.023750,      0.131280,     -0.120090,
		     -0.022850,     -0.023770,      0.019470,      0.091510,      0.195190,
		     -0.234740,     -0.144240,     -0.089610,      0.098780,     -0.005260,
		      0.085870,      0.048180,     -0.060950,      0.337300,      0.235540,
		      0.029680,      0.170350,      0.000000,      0.108600,      0.121270,
		      0.032010,     -0.106210,      0.118340,      0.105960,     -0.147150,
		     -0.174460,      0.091010,      0.198330,     -0.167630,     -0.283840,
		      0.371410,      0.471060,     -0.087800,     -0.058530,      0.404300,
		      0.114390,     -0.365690,     -0.079190,      0.349580,      0.049070,
		     -0.506730,     -0.457870,      0.284490,      0.625360,     -0.071830,
		     -0.164450,     -0.195690,      0.037220,      0.109680,     -0.129850,
		     -0.095130,      0.149580,      0.427940,      0.210340,     -0.045020,
		     -0.056550,      0.117540,      0.300820,      0.258680,      0.370880,
		      0.592860,      0.280080,     -0.086450,     -0.056620,      0.245430,
		      0.282520,     -0.024400,      0.042980,     -0.097240,     -0.060310,
		      0.050170,      0.110010,      0.163490,      0.084370,      0.029300,
		      0.046230,     -0.121030,     -0.390060,     -0.407170,     -0.183430,
		     -0.095210,     -0.070750,     -0.156980,     -0.476810,     -0.241060,
		     -0.163000,      0.104780,     -0.027580,     -0.044080,     -0.034890,
		      0.056250,     -0.065500,     -0.001700,     -0.060700,     -0.073670,
		     -0.090710,     -0.067120,     -0.023370,      0.012470,      0.080930,
		     -0.047760,      0.037710,     -0.163170,      0.011260,     -0.067100,
		      0.047130,     -0.107190,      0.059740,      0.047660,     -0.217120,
		     -0.146930,     -0.120740,      0.071800,      0.001030,      0.214210,
		     -0.026280,     -0.152220,     -0.016430,      0.016370,     -0.058820,
		     -0.096910,      0.087820,     -0.208000,     -0.018770,      0.064240,
		      0.292010,     -0.137060,     -0.229270,     -0.110460,      0.284600,
		      0.000000,     -0.274570,     -0.004880,     -0.008430,     -0.068850,
		      0.016470,     -0.083220,      0.103690,      0.309470,      0.057310,
		     -0.297830,      0.022170,      0.612630,      0.077010,     -0.467930,
		     -0.179650,      0.191970,     -0.230540,      0.017870,      0.519600,
		      0.118730,     -0.494110,     -0.258400,      0.332820,      0.352670,
		     -0.082370,     -0.416380,     -0.051890,      0.244690,      0.026570,
		     -0.111730,     -0.035750,      0.076620,      0.274130,     -0.145350,
		     -0.435880,     -0.375700,     -0.148250,      0.060830,      0.198360,
		     -0.047470,     -0.283250,     -0.489700,     -0.414660,     -0.277740,
		     -0.116400,      0.143450,     -0.042680,     -0.246520,     -0.054250,
		      0.053440,      0.120070,     -0.056630,     -0.052880,     -0.185700,
		      0.037830,      0.138980,      0.055700,      0.039800,     -0.061890,
		      0.015760,      0.248650,      0.266630,      0.198270,      0.256610,
		      0.183380,      0.387180,      0.171830,      0.013060,     -0.186310,
		      0.125430,      0.157140,      0.228950,      0.116760,     -0.010770,
		      0.000120,      0.092770,     -0.029240,      0.066450,     -0.001710,
		      0.037140,     -0.199120,     -0.298360,     -0.255450,      0.150320,
		      0.138810,     -0.093280,      0.126780,      0.070360,      0.102280,
		      0.087000,     -0.066720,      0.307800,     -0.020470,     -0.030110,
		     -0.164330,     -0.121690,     -0.148570,     -0.026970,      0.011340,
		     -0.073390,     -0.014510,     -0.085040,     -0.127820,     -0.087080,
		     -0.079750,      0.054460,     -0.190570,      0.138080,      0.186780,
		      0.090150,      0.074620,      0.009830,      0.339360,      0.000000,
		      0.068260,      0.362940,      0.061460,     -0.143520,      0.109410,
		      0.319570,      0.109970,     -0.084780,     -0.002370,      0.274430,
		     -0.010250,     -0.493960,     -0.251990,      0.337790,      0.313260,
		     -0.263970,      0.004760,      0.091960,     -0.200090,     -0.135510,
		      0.539020,      0.350400,     -0.228170,     -0.451660,     -0.122180,
		      0.283150,      0.108770,     -0.244950,     -0.117800,      0.159770,
		      0.051740,     -0.139190,     -0.332500,     -0.027870,      0.302970,
		      0.531410,      0.213100,     -0.033110,     -0.226510,     -0.245410,
		      0.166370,      0.333970,      0.350950,      0.499240,      0.231370,
		     -0.114990,     -0.181530,      0.193450,      0.165570,     -0.111280,
		     -0.049680,      0.018150,     -0.042150,      0.052360,      0.017480,
		      0.153050,      0.016240,      0.026270,      0.244440,     -0.128420,
		     -0.279520,     -0.257410,     -0.309590,     -0.230770,     -0.224230,
		     -0.269140,     -0.359040,     -0.153250,     -0.042240,     -0.201880,
		     -0.320150,      0.022750,     -0.102470,      0.129690,     -0.125130,
		     -0.011250,     -0.092220,     -0.013870,     -0.209110,      0.023820,
		     -0.072080,      0.242170,      0.009220,     -0.208150,     -0.041570,
		     -0.058920,      0.107170,      0.143840,      0.012670,     -0.223260,
		      0.075510,     -0.007550,     -0.028050,     -0.119740,     -0.165500,
		     -0.142540,     -0.037840,      0.096510,     -0.029110,     -0.119640,
		     -0.202710,     -0.191080,      0.163530,     -0.034860,     -0.111700,
		     -0.148660,     -0.152350,      0.151410,      0.139750,      0.234030,
		     -0.235770,     -0.025730,      0.098180,      0.114990,      0.000000,
		     -0.161820,     -0.170410,     -0.102360,      0.034340,      0.073870,
		     -0.232260,     -0.207120,      0.038370,      0.422090,     -0.000340,
		     -0.319520,      0.255440,      0.589860,     -0.189800,     -0.442180,
		      0.044050,      0.095190,     -0.123210,      0.201150,      0.319270,
		     -0.323040,     -0.498870,      0.074420,      0.491900,      0.099020,
		     -0.343490,     -0.148620,      0.116520,      0.201210,     -0.169220,
		     -0.208900,      0.070460,      0.243690,      0.373500,     -0.058750,
		     -0.442620,     -0.297640,     -0.089700,      0.175020,      0.327880,
		     -0.051550,     -0.198500,     -0.331960,     -0.344460,     -0.163250,
		      0.200120,      0.186780,     -0.111320,     -0.136080,     -0.018660,
		      0.140870,      0.175680,      0.039090,      0.007990,     -0.156130,
		     -0.067880,      0.020550,      0.069080,     -0.042290,      0.061750,
		      0.086440,      0.162830,      0.182360,      0.219500,      0.189590,
		      0.183040,      0.281440,      0.187020,      0.069180,      0.251740,
		      0.205060,     -0.095960,     -0.056020,     -0.156360,     -0.008050,
		     -0.002610,      0.011130,     -0.104410,     -0.192140,     -0.119150,
		     -0.022800,      0.080930,     -0.118210,     -0.155330,     -0.253840,
		      0.064410,     -0.249390,      0.098820,      0.057210,      0.189010,
		      0.139000,      0.087940,     -0.056600,     -0.103760,     -0.037000,
		     -0.053760,     -0.129550,     -0.184510,     -0.101180,     -0.100190,
		     -0.099690,     -0.061280,     -0.183580,      0.220400,      0.177630,
		      0.057930,     -0.081020,     -0.127240,      0.157650,      0.040460,
		     -0.044450,     -0.277710,     -0.319160,      0.069830,      0.075980,
		      0.000000,      0.017700,      0.195430,      0.113300,     -0.226380,
		     -0.142080,      0.142310,      0.198540,      0.037610,     -0.148070,
		      0.144790,      0.394230,      0.034720,     -0.451820,      0.078820,
		      0.432780,      0.186700,     -0.051680,      0.054890,     -0.136590,
		     -0.303350,      0.203480,      0.576930,      0.058970,     -0.435710,
		     -0.247850,      0.145690,      0.276700,     -0.104790,     -0.340580,
		     -0.065300,      0.234640,      0.131030,     -0.148820,     -0.360190,
		     -0.116240,      0.210310,      0.296020,      0.111280,     -0.077800,
		     -0.195960,     -0.128230,      0.115600,      0.281360,      0.188200,
		      0.153620,      0.166650,      0.036780,     -0.020590,      0.258060,
		      0.134520,     -0.111110,     -0.140080,     -0.043170,     -0.124360,
		     -0.018110,     -0.094720,      0.031080,     -0.011140,     -0.012870,
		     -0.018700,     -0.051970,     -0.098300,     -0.166560,     -0.187600,
		     -0.208080,     -0.090200,     -0.183230,     -0.107040,     -0.035540,
		     -0.074710,     -0.261850,     -0.050000,      0.157260,      0.123030,
		     -0.019630,     -0.174640,     -0.062260,      0.014230,     -0.037240,
		      0.067260,      0.036260,     -0.039540,      0.044710,      0.166940,
		      0.015020,     -0.154800,     -0.058780,     -0.041840,      0.116030,
		      0.116370,     -0.230880,     -0.379210,     -0.178000,      0.086130,
		     -0.014870,      0.006290,     -0.069670,      0.156670,     -0.010650,
		      0.003740,     -0.116320,      0.007650,     -0.043440,     -0.286990,
		     -0.023800,      0.099900,     -0.038070,     -0.060280,     -0.024560,
		     -0.088100,     -0.001790,      0.019670,     -0.092600,      0.005460,
		     -0.053490,      0.024930,      0.000000,      0.035900,     -0.129150,
		     -0.143930,      0.137710,      0.169450,      0.084070,     -0.184520,
		     -0.000720,      0.082450,      0.100560,     -0.180520,     -0.170760,
		      0.533730,      0.226520,     -0.313940,     -0.321410,      0.043440,
		      0.051760,      0.051810,      0.231340,      0.007450,     -0.514710,
		     -0.192160,      0.307660,      0.309700,     -0.120700,     -0.380750,
		      0.127140,      0.274770,      0.244560,     -0.114240,     -0.241660,
		     -0.021830,      0.216560,      0.364310,     -0.001850,     -0.286950,
		     -0.232060,     -0.013270,      0.133560,      0.153910,     -0.025340,
		     -0.203160,     -0.039930,     -0.070320,     -0.050320,      0.062830,
		      0.098510,     -0.160010,     -0.060160,      0.099530,      0.128600,
		      0.164820,      0.064840,      0.170860,      0.150460,      0.118200,
		      0.078890,      0.010430,     -0.008750,      0.018460,      0.115420,
		      0.160530,      0.286100,      0.230630,      0.083910,      0.097900,
		      0.066150,     -0.048410,      0.114860,      0.169090,      0.260460,
		     -0.197950,     -0.196760,     -0.299260,      0.195780,      0.097530,
		      0.167670,     -0.044860,     -0.198270,     -0.240130,      0.117510,
		     -0.010760,     -0.072430,     -0.197990,     -0.061560,     -0.082660,
		     -0.212430,     -0.016750,     -0.083670,      0.003210,      0.186820,
		      0.159380,     -0.218530,     -0.253880,     -0.072810,     -0.086380,
		     -0.097450,     -0.134430,     -0.040310,      0.014880,     -0.245310,
		     -0.165700,     -0.183310,      0.157420,      0.117690,     -0.013190,
		      0.123990,      0.175870,      0.129480,      0.165200,      0.154030,
		     -0.166600,     -0.142650,     -0.142880,     -0.049480,     -0.047880,
		      0.000000,     -0.021970,      0.176580,      0.207050,     -0.121290,
		     -0.175490,     -0.061580,      0.094880,     -0.029700,     -0.107450,
		     -0.224530,      0.121780,      0.283290,     -0.341250,     -0.417330,
		      0.310010,      0.459790,      0.053810,      0.040030,     -0.062170,
		     -0.253530,     -0.097170,      0.332140,      0.304050,     -0.089290,
		     -0.287850,      0.002130,      0.345370,      0.205940,     -0.193920,
		     -0.240280,     -0.036730,      0.163180,      0.041460,     -0.062820,
		     -0.138050,      0.055170,      0.206790,      0.167720,     -0.021440,
		     -0.230820,     -0.207140,     -0.026120,      0.219280,      0.117750,
		     -0.077300,     -0.102950,     -0.055630,     -0.089440,      0.051640,
		      0.096580,     -0.096190,     -0.015460,     -0.120970,     -0.146920,
		     -0.153580,     -0.135490,     -0.217550,     -0.021760,     -0.071830,
		      0.004030,      0.001770,     -0.136720,     -0.120040,     -0.167690,
		     -0.202980,      0.040250,     -0.022590,     -0.057740,      0.158690,
		      0.165060,      0.032190,     -0.189560,     -0.001690,      0.149060,
		      0.264250,     -0.149570,     -0.188950,     -0.011870,     -0.026480,
		      0.059000,      0.038910,      0.078590,      0.106750,      0.002310,
		      0.096920,      0.020360,     -0.205640,      0.242020,     -0.074360,
		      0.022040,      0.076910,      0.021090,     -0.119330,     -0.048920,
		      0.008780,     -0.088190,     -0.065570,     -0.117350,      0.017310,
		      0.045500,      0.124400,      0.264680,      0.175180,      0.088760,
		     -0.191460,     -0.211270,     -0.007180,      0.137000,     -0.084760,
		      0.082790,      0.065360,     -0.085350,      0.029540,      0.032230,
		      0.155970,     -0.125590,     -0.027680,     -0.023010,      0.000000,
		     -0.035260,      0.105150,     -0.239000,     -0.006300,      0.289120,
		      0.134960,     -0.115750,      0.009890,      0.331620,      0.213610,
		     -0.176240,     -0.188520,      0.148180,      0.445720,     -0.034980,
		     -0.343340,     -0.169890,      0.039670,      0.170790,      0.184920,
		      0.311620,     -0.273940,     -0.307820,     -0.021630,      0.191830,
		      0.009090,     -0.353000,     -0.309390,      0.103960,      0.226860,
		      0.141160,      0.006260,     -0.062080,      0.018720,      0.053820,
		     -0.012920,     -0.149140,     -0.192380,     -0.066870,      0.111620,
		      0.091370,      0.051120,     -0.100520,     -0.027630,      0.109710,
		      0.080080,      0.123770,      0.102460,     -0.113530,     -0.277060,
		      0.056850,      0.107220,      0.128890,      0.082400,      0.018640,
		      0.010540,      0.083140,      0.046210,      0.023280,      0.025140,
		      0.062730,      0.173960,      0.135010,      0.062510,      0.118110,
		      0.137170,      0.003330,      0.103910,      0.090980,      0.112320,
		      0.086960,      0.202930,      0.100590,      0.042310,     -0.135180,
		      0.006470,      0.053970,      0.030600,      0.064890,      0.134660,
		     -0.131990,      0.021370,      0.025970,      0.050230,     -0.036440,
		      0.063530,      0.040160,     -0.075670,      0.068880,      0.000250,
		     -0.231820,     -0.365240,     -0.115850,      0.022120,     -0.127030,
		     -0.084760,     -0.098930,     -0.096850,     -0.269830,     -0.132860,
		     -0.057280,     -0.195030,     -0.042620,     -0.017940,      0.024530,
		      0.037620,      0.034770,     -0.101360,      0.002800,      0.104160,
		      0.114000,      0.026510,     -0.063160,      0.071210,     -0.065010,
		     -0.089080,     -0.037600,     -0.209030,     -0.072800,      0.000000,
		      0.039060,     -0.065560,      0.173890,      0.162760,     -0.249170,
		     -0.115900,      0.177530,      0.104630,     -0.142080,     -0.187270,
		      0.003880,      0.239850,      0.050150,     -0.426220,     -0.161670,
		      0.383980,      0.209950,     -0.068570,     -0.072970,     -0.180290,
		     -0.283280,      0.132930,      0.223780,      0.128650,     -0.154950,
		     -0.031450,      0.233710,      0.307640,      0.008510,     -0.154230,
		     -0.130100,      0.053310,      0.091900,      0.110410,     -0.119990,
		      0.055870,      0.228640,      0.237800,      0.073780,     -0.055600,
		     -0.080790,     -0.081080,      0.073920,      0.184650,      0.105260,
		     -0.149290,     -0.178930,     -0.149580,     -0.013880,      0.242390,
		      0.040250,     -0.187670,     -0.158330,     -0.133820,     -0.009370,
		     -0.033510,     -0.159520,     -0.107650,     -0.174860,     -0.214610,
		     -0.068970,     -0.022390,     -0.094920,      0.019950,     -0.097350,
		     -0.061470,      0.044550,     -0.119410,      0.045900,      0.151970,
		      0.017790,      0.040330,     -0.007940,      0.038120,      0.015060,
		     -0.051540,      0.111730,      0.094650,     -0.178000,     -0.028790,
		     -0.057500,     -0.003810,     -0.146290,      0.063420,      0.037660,
		     -0.021350,     -0.157570,     -0.140870,      0.027530,     -0.067600,
		     -0.006000,      0.017190,      0.044690,     -0.065290,      0.108840,
		      0.016310,     -0.008010,      0.012760,      0.039730,      0.038770,
		     -0.127390,     -0.040380,     -0.035140,     -0.011490,      0.160420,
		     -0.009920,     -0.018520,      0.004210,      0.030870,      0.073520,
		     -0.075660,      0.171390,      0.224930,     -0.015030,      0.115730,
		      0.059100,     -0.085880,      0.113300,     -0.080360,      0.094050,
		      0.000000,     -0.189260,      0.168820,     -0.018720,     -0.257940,
		      0.104860,      0.224200,     -0.079260,     -0.192530,      0.222080,
		      0.311250,      0.010940,     -0.245660,      0.029750,      0.381280,
		      0.228060,     -0.232470,     -0.271830,      0.013400,      0.139200,
		      0.155320,      0.160260,      0.080010,     -0.167050,     -0.046430,
		      0.100470,      0.111780,     -0.075720,     -0.260050,     -0.059390,
		      0.125380,      0.162290,     -0.108920,     -0.184120,     -0.138240,
		      0.188700,     -0.023390,     -0.203060,     -0.228010,     -0.127330,
		     -0.041560,      0.104460,     -0.048540,     -0.226710,     -0.197700,
		     -0.018020,      0.158840,      0.145820,      0.185870,      0.159740,
		     -0.168220,     -0.177140,      0.027240,      0.184720,     -0.017710,
		      0.091440,      0.132320,      0.180360,      0.180430,      0.148410,
		      0.171440,      0.017690,     -0.028710,      0.069390,      0.232210,
		      0.068530,      0.020140,     -0.048180,      0.009600,     -0.041490,
		     -0.008740,      0.048060,      0.163410,     -0.030970,     -0.021900,
		     -0.064700,      0.022810,      0.054240,     -0.037780,     -0.063470,
		      0.059430,      0.179490,      0.273310,      0.203570,      0.173080,
		      0.053670,      0.083570,     -0.025650,      0.013740,     -0.064250,
		      0.151250,     -0.018450,     -0.007620,     -0.219580,     -0.133060,
		     -0.087440,     -0.067470,      0.005610,     -0.022840,     -0.177380,
		     -0.197350,     -0.097200,     -0.071750,      0.120970,      0.009120,
		      0.079880,      0.121400,     -0.027610,      0.025260,     -0.087630,
		      0.047580,      0.148150,     -0.068650,     -0.063120,     -0.045940,
		      0.084610,      0.026080,     -0.001340,     -0.005050,      0.009030,
		     -0.088610,     -0.013650,      0.000000,      0.172820,      0.006160,
		     -0.084470,      0.297110,     -0.001910,     -0.226550,      0.017860,
		      0.219290,     -0.083890,     -0.166520,     -0.110970,      0.119030,
		      0.078920,     -0.215920,     -0.086040,      0.134220,      0.254180,
		      0.039520,     -0.202570,     -0.095730,     -0.103400,     -0.210990,
		      0.107840,     -0.021420,     -0.103090,     -0.208350,      0.043980,
		      0.229150,      0.118710,     -0.074820,     -0.131070,      0.086280,
		      0.161160,      0.219040,     -0.065000,     -0.082100,      0.026810,
		      0.213280,      0.086210,     -0.031940,     -0.102190,      0.161570,
		      0.295960,      0.243390,      0.152380,      0.063870,     -0.182380,
		     -0.196500,     -0.153540,     -0.023630,      0.212110,      0.138300,
		     -0.036590,     -0.028200,     -0.207130,     -0.074510,     -0.115320,
		     -0.153270,     -0.040160,     -0.130480,     -0.214760,     -0.031610,
		      0.010200,     -0.166660,     -0.027130,     -0.048590,      0.008880,
		      0.019920,     -0.042650,     -0.007960,      0.049270,     -0.027040,
		      0.052420,      0.155950,      0.122930,      0.004850,     -0.196900,
		     -0.031790,      0.042990,     -0.125770,     -0.227530,     -0.191890,
		     -0.159840,     -0.222400,      0.039640,      0.064200,     -0.026530,
		     -0.011660,     -0.063490,     -0.137020,     -0.066470,      0.004460,
		     -0.109900,     -0.120040,     -0.070120,      0.058350,      0.000920,
		     -0.027170,      0.067720,      0.115500,      0.073320,      0.046060,
		     -0.168960,     -0.269650,     -0.080470,     -0.176330,      0.060770,
		      0.000250,      0.152450,      0.143330,      0.089970,     -0.049690,
		     -0.017670,      0.138380,      0.036120,      0.028850,      0.055500,
		      0.020330,      0.113730,      0.065080,      0.037020,     -0.012030,
		      0.000000,     -0.220740,     -0.014010,      0.143850,     -0.136320,
		     -0.023850,      0.126550,      0.040850,     -0.254800,     -0.065480,
		      0.266500,      0.103260,     -0.113840,     -0.144910,      0.237760,
		      0.213970,     -0.016470,     -0.212920,     -0.036660,      0.128640,
		      0.119410,      0.105270,      0.074860,     -0.023510,      0.044600,
		      0.137580,      0.193160,     -0.015420,     -0.115810,     -0.163760,
		      0.149210,      0.125120,      0.079160,     -0.117280,     -0.123170,
		      0.009860,      0.107770,     -0.012830,     -0.226820,     -0.108940,
		     -0.014730,      0.025050,     -0.072120,     -0.257160,     -0.353090,
		     -0.266890,     -0.040010,      0.185530,      0.111490,      0.145130,
		      0.082550,     -0.063150,     -0.116900,      0.019580,      0.140940,
		      0.150060,      0.016340,      0.070400,      0.235770,      0.149720,
		      0.199710,      0.110720,      0.075420,     -0.017570,     -0.019750,
		      0.104160,      0.196400,      0.049720,      0.022010,      0.031770,
		      0.031570,     -0.001740,     -0.020240,     -0.038910,     -0.096260,
		     -0.149450,     -0.110230,      0.126440,      0.182290,      0.015760,
		     -0.075710,     -0.099680,      0.314210,      0.173670,      0.204990,
		     -0.074310,     -0.043840,      0.169270,     -0.074170,     -0.081540,
		      0.001370,      0.084750,     -0.138030,      0.008180,     -0.101190,
		      0.060980,     -0.122730,     -0.135120,      0.070520,      0.037390,
		     -0.137480,     -0.120020,     -0.018140,      0.109800,      0.189200,
		      0.030740,      0.166360,      0.013860,     -0.017430,     -0.057690,
		     -0.092640,     -0.003080,      0.139630,      0.077510,     -0.180650,
		      0.011180,      0.115700,     -0.086750,     -0.085770,      0.113710,
		      0.026900,      0.175490,     -0.024690,      0.072360,      0.000000,
		      0.137240,      0.073200,     -0.143710,      0.025050,      0.116690,
		     -0.091350,     -0.101040,      0.221360,      0.186640,     -0.218480,
		     -0.104390,      0.076930,      0.122420,     -0.093750,     -0.181040,
		      0.149560,      0.175570,      0.196100,     -0.054220,     -0.261360,
		     -0.064850,     -0.046620,      0.001100,      0.000200,     -0.138950,
		     -0.215940,     -0.082300,      0.127410,      0.191910,     -0.108950,
		     -0.179920,     -0.168840,      0.095540,      0.156190,      0.019080,
		     -0.065320,     -0.025990,      0.153860,      0.274950,     -0.003280,
		     -0.094310,     -0.089350,      0.186470,      0.406360,      0.296500,
		      0.025150,     -0.042730,     -0.099050,     -0.013330,     -0.136750,
		     -0.027030,      0.074450,     -0.036770,     -0.022270,      0.037790,
		     -0.024640,      0.051310,     -0.092800,     -0.160940,     -0.045810,
		     -0.044990,     -0.008490,     -0.055400,      0.069990,     -0.019770,
		     -0.094440,     -0.087730,      0.006450,     -0.024470,     -0.058440,
		      0.047410,      0.112720,     -0.051070,      0.075820,      0.049160,
		      0.043590,     -0.131910,     -0.293490,     -0.106950,      0.015640,
		      0.014070,     -0.064680,     -0.033680,      0.024880,     -0.230320,
		      0.023060,     -0.058270,      0.000750,     -0.092360,      0.056040,
		     -0.025110,     -0.077800,     -0.088590,     -0.184570,     -0.181530,
		      0.009240,      0.101490,     -0.034000,     -0.018560,      0.013420,
		      0.104170,      0.086170,      0.147660,     -0.029430,     -0.002930,
		     -0.072880,      0.095960,      0.020180,     -0.053600,      0.148580,
		      0.039620,     -0.088320,     -0.019690,     -0.105250,      0.056540,
		     -0.039790,     -0.046000,      0.087350,     -0.010060,     -0.078040,
		      0.173400,      0.108110,     -0.043820,     -0.107900,      0.000000,
		     -0.119000,      0.022000,      0.055170,      0.018280,     -0.088660,
		      0.160880,      0.138400,     -0.145150,     -0.141270,      0.137650,
		      0.071670,     -0.094160,     -0.145780,      0.026700,      0.207590,
		     -0.101350,     -0.235350,     -0.176460,      0.082820,      0.190960,
		      0.172750,      0.075820,     -0.068720,      0.011380,      0.079080,
		      0.219640,      0.209030,     -0.074680,     -0.213520,     -0.034980,
		      0.197710,      0.154510,     -0.072790,     -0.133020,     -0.049550,
		      0.148080,      0.056980,     -0.122390,     -0.283020,     -0.035410,
		      0.087760,      0.136810,     -0.127790,     -0.310110,     -0.360130,
		     -0.151930,     -0.036460,     -0.056110,     -0.074790,      0.022100,
		     -0.024900,     -0.107090,      0.001940,     -0.078780,      0.011190,
		      0.036540,     -0.101860,      0.008920,      0.106000,      0.086180,
		      0.094910,      0.080060,      0.113770,      0.068280,     -0.071680,
		     -0.045610,      0.092250,      0.049020,      0.041710,      0.108330,
		      0.043960,     -0.040640,      0.007320,      0.019680,     -0.024110,
		     -0.000550,      0.071790,      0.278090,      0.196580,      0.063430,
		     -0.156580,     -0.114460,      0.007220,      0.095820,      0.147520,
		     -0.162440,     -0.065720,      0.152340,      0.017270,     -0.021070,
		     -0.091800,     -0.097350,     -0.131060,      0.039000,     -0.139660,
		     -0.057890,     -0.061330,      0.022790,      0.078740,      0.007290,
		     -0.084060,     -0.134590,     -0.109450,     -0.054600,      0.214800,
		      0.051490,      0.084960,      0.170640,      0.094640,     -0.001710,
		      0.055250,      0.015180,      0.081690,      0.063090,      0.023910,
		     -0.005190,      0.098740,     -0.154100,     -0.114540,     -0.020870,
		      0.130880,     -0.159520,      0.075160,      0.032140,     -0.132180,
		      0.000000,      0.129070,      0.088560,     -0.051200,     -0.019510,
		      0.069570,     -0.100080,     -0.224790,      0.024860,      0.283360,
		     -0.030780,     -0.167870,      0.008730,      0.063640,     -0.009110,
		     -0.073890,      0.132770,      0.219980,      0.116060,      0.030910,
		     -0.180260,     -0.221130,     -0.013060,      0.066500,     -0.015500,
		     -0.015750,     -0.065260,     -0.182420,      0.030130,      0.163510,
		      0.100400,     -0.176310,     -0.261220,     -0.017380,      0.230050,
		      0.126900,     -0.082790,     -0.105890,      0.045570,      0.242950,
		      0.191430,     -0.123730,     -0.141300,     -0.017480,      0.185180,
		      0.378300,      0.206080,     -0.015140,     -0.008510,      0.032740,
		     -0.027340,     -0.054800,      0.062530,      0.025750,     -0.068880,
		     -0.094400,     -0.017900,      0.033500,      0.130950,     -0.029850,
		     -0.080720,     -0.137950,     -0.038470,     -0.066190,      0.039100,
		      0.136980,      0.125930,     -0.054330,     -0.053820,     -0.036750,
		      0.015190,      0.010870,      0.037500,      0.107720,      0.015540,
		      0.017730,     -0.092700,     -0.001230,     -0.140620,     -0.222340,
		     -0.121890,      0.042220,      0.096050,     -0.057980,      0.048170,
		     -0.014110,     -0.038530,     -0.094650,     -0.175590,     -0.072180,
		      0.024370,      0.250000,      0.196620,      0.104060,     -0.225560,
		     -0.121030,     -0.195890,      0.086730,      0.060050,      0.019700,
		      0.108540,     -0.142370,     -0.026920,     -0.088770,      0.104770,
		     -0.019550,     -0.028260,      0.013700,      0.034660,      0.134040,
		     -0.027620,     -0.068240,     -0.005500,     -0.181820,     -0.012860,
		     -0.033980,      0.026210,      0.012500,      0.063250,      0.007310,
		     -0.019600,      0.050040,     -0.006300,      0.038540,     -0.127010,
		      0.009320,      0.024110,      0.000000,     -0.148380,      0.024540,
		      0.047010,     -0.036820,     -0.034200,      0.017580,      0.205500,
		     -0.003730,     -0.205390,      0.083300,      0.113230,     -0.000250,
		     -0.048270,      0.059130,      0.094840,     -0.001980,     -0.183000,
		     -0.129990,     -0.068050,      0.073730,      0.100350,      0.056780,
		     -0.052940,     -0.039080,     -0.007320,     -0.032330,      0.135160,
		      0.075750,     -0.019410,     -0.072150,      0.074810,      0.250900,
		      0.098870,     -0.185420,     -0.231720,     -0.019670,      0.132960,
		      0.032770,     -0.189330,     -0.215220,      0.098630,      0.137120,
		      0.084000,     -0.137960,     -0.330470,     -0.247190,      0.056140,
		      0.015950,     -0.087270,     -0.142960,      0.047870,     -0.054450,
		     -0.061770,     -0.054110,      0.004160,      0.011880,     -0.032350,
		     -0.144690,     -0.035300,      0.081520,      0.127030,      0.116460,
		      0.017520,      0.022530,     -0.000110,     -0.064710,     -0.047870,
		      0.027620,      0.102970,      0.041690,     -0.002390,     -0.002030,
		     -0.029940,      0.034730,     -0.068710,      0.098510,     -0.009840,
		      0.107810,      0.197260,      0.115620,     -0.002670,      0.025590,
		      0.075580,      0.107350,     -0.057120,     -0.110890,     -0.123320,
		      0.076640,      0.095880,     -0.043510,     -0.183120,     -0.059680,
		     -0.070940,     -0.021660,     -0.031770,     -0.002380,     -0.144680,
		     -0.136690,     -0.061000,      0.043140,      0.203180,      0.015890,
		      0.026530,     -0.095150,      0.021450,      0.080210,      0.029650,
		      0.150120,      0.095010,      0.103680,      0.128090,     -0.008000,
		      0.133590,     -0.095470,      0.094020,      0.019110,      0.026470,
		      0.080810,     -0.091500,      0.004060,      0.037940,     -0.045980,
		     -0.117260,     -0.098340,      0.097210,     -0.207180,     -0.000000,
		      0.000000,      0.118210,      0.161150,      0.021340,      0.057850,
		      0.029590,      0.127440,     -0.153070,     -0.089750,      0.217460,
		      0.036610,     -0.164940,     -0.078210,      0.128010,     -0.066820,
		     -0.122710,     -0.037800,      0.214930,      0.137540,      0.126410,
		      0.002510,     -0.104250,     -0.122700,      0.056600,      0.070620,
		     -0.038190,      0.036660,     -0.026250,     -0.105140,     -0.006130,
		      0.124460,      0.024370,     -0.256480,     -0.244570,      0.133440,
		      0.229670,      0.051460,     -0.113790,     -0.078390,      0.040660,
		      0.174850,     -0.002690,     -0.086580,     -0.062970,      0.109470,
		      0.231930,      0.227980,     -0.000340,     -0.038730,      0.087120,
		      0.089650,     -0.123510,     -0.075750,      0.005670,      0.049240,
		     -0.009160,     -0.030060,     -0.066190,      0.045070,      0.061480,
		     -0.081550,     -0.076340,     -0.111950,     -0.060360,      0.117690,
		     -0.040500,      0.047190,      0.024790,     -0.059180,      0.003250,
		     -0.052510,      0.052900,     -0.008260,      0.036660,     -0.018340,
		      0.087680,     -0.056370,     -0.035550,     -0.025820,     -0.011630,
		     -0.022500,      0.006940,      0.010740,     -0.047930,     -0.001510,
		      0.072570,      0.119670,      0.008360,     -0.084370,     -0.112000,
		     -0.036950,     -0.013420,      0.115110,      0.047680,     -0.017720,
		     -0.003090,     -0.100400,     -0.007830,      0.059860,      0.154670,
		      0.156230,     -0.135070,     -0.074530,     -0.046150,     -0.082580,
		      0.013050,     -0.068360,      0.073230,     -0.157540,      0.044810,
		      0.127390,      0.003460,     -0.001600,     -0.208190,     -0.187270,
		     -0.005910,     -0.091270,      0.140850,     -0.013560,      0.070850,
		     -0.089580,      0.021010,      0.045640,      0.100740,     -0.084150,
		      0.036330,     -0.031940,     -0.031080,     -0.052050,      0.000000,
		     -0.168230,     -0.084760,     -0.049320,      0.015950,      0.025330,
		     -0.141000,      0.098840,      0.099490,     -0.055430,     -0.040810,
		      0.108000,     -0.005170,     -0.141060,      0.101630,      0.173290,
		      0.130560,     -0.191490,     -0.104820,     -0.083430,     -0.018450,
		      0.106220,      0.061920,     -0.105670,     -0.046460,      0.028810,
		      0.007070,     -0.006570,      0.048160,      0.026460,     -0.083460,
		     -0.013580,      0.147380,      0.238010,     -0.018260,     -0.190760,
		     -0.060440,      0.055500,      0.131210,      0.026150,     -0.103550,
		     -0.102800,      0.048260,      0.068930,     -0.071180,     -0.170300,
		     -0.150360,     -0.061810,      0.031130,      0.030370,     -0.072680,
		     -0.000030,      0.070560,      0.065830,     -0.008360,     -0.019390,
		     -0.012480,      0.111650,      0.041410,     -0.095870,      0.069040,
		      0.039230,      0.132550,      0.192050,      0.095420,      0.053300,
		      0.115200,      0.012300,      0.022960,     -0.027260,      0.122470,
		      0.007900,     -0.011460,     -0.110080,     -0.002200,     -0.052320,
		     -0.052510,      0.110720,      0.036060,     -0.009700,      0.007380,
		     -0.058300,     -0.059370,      0.031710,      0.052710,      0.120510,
		     -0.001830,     -0.118270,     -0.056870,      0.094540,      0.188970,
		      0.081030,     -0.158830,     -0.100850,     -0.037520,     -0.032380,
		     -0.010480,     -0.019440,     -0.040560,     -0.143200,     -0.165660,
		      0.041090,      0.119750,      0.005100,      0.035210,     -0.105450,
		     -0.005920,     -0.109870,      0.162410,      0.050790,     -0.064080,
		     -0.012100,     -0.033170,      0.181960,      0.202480,     -0.004890,
		      0.038260,     -0.007090,     -0.069770,      0.023570,     -0.050550,
		     -0.104820,      0.117720,      0.078460,     -0.036920,     -0.065930,
		      0.014580,     -0.005640,     -0.081400,      0.010110,      0.000000,
		      0.098430,      0.062680,      0.056610,      0.003360,     -0.004980,
		      0.007920,      0.022030,     -0.081320,      0.106210,      0.134710,
		     -0.095510,     -0.104700,      0.065900,      0.058190,     -0.121390,
		     -0.145710,      0.130650,      0.171850,     -0.022200,     -0.010280,
		      0.010050,     -0.074960,      0.104140,      0.031950,     -0.013580,
		     -0.050150,      0.041910,      0.063520,     -0.006560,      0.034880,
		      0.027370,     -0.068310,     -0.191360,     -0.086470,      0.159140,
		      0.050210,     -0.126470,     -0.083290,      0.034550,      0.050890,
		     -0.016490,     -0.088770,     -0.035920,      0.042640,      0.177490,
		      0.068910,     -0.046930,     -0.074490,      0.006380,      0.116790,
		      0.077800,     -0.095900,     -0.082260,     -0.015280,      0.002520,
		     -0.019850,     -0.060810,     -0.065820,      0.053570,     -0.012330,
		     -0.022140,     -0.095310,     -0.171610,     -0.099300,     -0.010290,
		     -0.038340,     -0.012400,     -0.115440,     -0.087440,     -0.016730,
		     -0.027480,      0.001680,      0.065550,      0.085010,      0.022760,
		      0.085310,     -0.058860,     -0.033430,     -0.050970,      0.072640,
		      0.086820,      0.110470,      0.046500,     -0.038740,     -0.115000,
		      0.037090,     -0.030640,     -0.047520,     -0.039020,     -0.073650,
		     -0.083420,      0.107680,      0.089540,      0.005460,     -0.059340,
		     -0.058850,      0.048100,     -0.035050,      0.000430,      0.004330,
		      0.024140,     -0.056140,     -0.026750,     -0.024520,     -0.012000,
		     -0.025300,      0.104220,     -0.056150,      0.045350,      0.090250,
		      0.105260,      0.148750,     -0.079720,     -0.128810,      0.026970,
		      0.035850,      0.036230,      0.013080,     -0.006140,      0.035160,
		      0.081280,     -0.148920,     -0.073010,     -0.067950,      0.081610,
		     -0.074640,     -0.001560,     -0.005340,     -0.252280,     -0.073630,
		      0.000000,     -0.047030,      0.039800,     -0.087350,     -0.061550,
		      0.087180,      0.030570,     -0.059380,      0.080140,     -0.039020,
		     -0.089240,      0.137100,      0.097000,     -0.088720,     -0.090110,
		      0.115580,      0.175420,     -0.055520,     -0.125820,     -0.024530,
		     -0.012250,      0.047820,      0.105260,     -0.105480,     -0.005810,
		      0.031570,      0.043530,     -0.032950,     -0.068890,     -0.005210,
		     -0.042360,     -0.026120,      0.064300,      0.115200,      0.166710,
		     -0.116460,     -0.079950,      0.065930,      0.077140,     -0.010760,
		      0.010950,     -0.014860,      0.027900,      0.022200,      0.011260,
		     -0.121460,     -0.123760,      0.105910,      0.081130,     -0.027030,
		     -0.075320,     -0.062790,      0.016800,      0.006570,      0.036480,
		      0.073340,      0.022500,      0.059970,      0.130960,     -0.011230,
		     -0.034170,      0.044900,      0.081200,      0.129310,      0.171820,
		      0.119380,      0.044180,      0.007800,     -0.002170,      0.070610,
		      0.024830,      0.122570,     -0.010240,     -0.120530,     -0.162430,
		      0.066190,     -0.099710,     -0.053810,      0.071640,      0.056970,
		      0.032330,     -0.008620,     -0.080220,     -0.093320,     -0.010210,
		     -0.020080,      0.091200,      0.025950,      0.036600,     -0.024250,
		      0.138830,      0.112450,      0.084820,     -0.028100,     -0.053150,
		      0.079140,      0.036220,     -0.083130,      0.036850,      0.079720,
		     -0.052820,     -0.062210,      0.051470,      0.022030,     -0.036010,
		      0.008040,     -0.042880,     -0.067790,     -0.007730,      0.125190,
		      0.002560,     -0.094990,     -0.161490,     -0.045380,     -0.036210,
		      0.196390,      0.073670,      0.097620,      0.072060,     -0.015200,
		     -0.034600,     -0.092220,     -0.053110,      0.122350,      0.048190,
		     -0.187250,     -0.007130,      0.073200,     -0.042700,     -0.012500,
		      0.185390,      0.085420,      0.000000,      0.028760,      0.046770,
		      0.061190,      0.091430,      0.024380,     -0.010330,      0.007840,
		     -0.055490,      0.092120,      0.121520,     -0.108420,     -0.155980,
		      0.054550,      0.094940,     -0.025030,     -0.089660,     -0.017780,
		      0.105440,      0.058730,     -0.050630,     -0.038680,     -0.059670,
		      0.061810,      0.051720,     -0.075160,     -0.039070,     -0.005010,
		      0.096970,      0.106660,      0.096330,      0.034570,     -0.064260,
		     -0.062130,     -0.163280,      0.074740,      0.134140,     -0.035730,
		     -0.173600,     -0.072520,      0.051300,      0.003240,     -0.115600,
		     -0.117620,     -0.032330,      0.040380,      0.183720,      0.005840,
		     -0.122870,     -0.043260,      0.064370,      0.150880,      0.054020,
		     -0.083240,     -0.081550,     -0.085190,     -0.059190,     -0.000020,
		     -0.148490,     -0.037700,      0.052760,     -0.013050,     -0.032050,
		     -0.094600,     -0.091300,     -0.130760,     -0.003490,     -0.063380,
		      0.019210,     -0.081530,     -0.034610,     -0.000420,      0.042420,
		      0.039640,      0.105280,     -0.007780,      0.005490,      0.023930,
		     -0.011590,      0.054620,     -0.029650,      0.033840,      0.103900,
		      0.159070,      0.028070,      0.021480,     -0.049500,     -0.003470,
		     -0.057210,     -0.097060,     -0.028540,     -0.116490,     -0.023070,
		     -0.075110,     -0.032350,     -0.052840,     -0.142090,     -0.035690,
		      0.042000,     -0.003610,     -0.053410,     -0.062700,      0.029780,
		      0.013990,      0.016930,     -0.041420,     -0.058570,     -0.155710,
		     -0.122370,     -0.114400,      0.067500,      0.100890,      0.201810,
		      0.043850,      0.028320,     -0.095520,     -0.031080,      0.052680,
		     -0.037280,     -0.028590,      0.053140,      0.015970,      0.049560,
		     -0.066130,     -0.078410,     -0.108940,     -0.015240,      0.000710,
		      0.011630,     -0.026410,     -0.152790,     -0.088790,     -0.014010,
		      0.000000,     -0.016750,      0.033650,     -0.027920,     -0.071330,
		     -0.067300,      0.009000,      0.061020,      0.014670,      0.047630,
		     -0.090980,      0.016010,      0.126550,     -0.059730,     -0.075540,
		      0.001010,      0.124530,      0.030060,     -0.037420,      0.029170,
		     -0.014980,      0.002080,      0.081560,     -0.037160,     -0.095220,
		      0.021490,      0.039620,      0.014270,     -0.096240,     -0.118290,
		     -0.103300,     -0.069750,      0.063210,      0.086680,      0.106460,
		      0.012360,     -0.105050,     -0.008760,      0.163910,      0.172980,
		      0.031440,     -0.018160,      0.067660,      0.117540,      0.038340,
		     -0.032790,     -0.118520,     -0.082660,      0.116340,      0.078950,
		     -0.040100,     -0.127060,     -0.099470,      0.092610,      0.029480,
		     -0.038260,      0.016530,     -0.035720,      0.004670,      0.047470,
		     -0.041120,     -0.071580,      0.015560,      0.108460,      0.117340,
		      0.134310,      0.097710,      0.044650,     -0.048010,      0.004980,
		      0.041280,      0.020500,      0.096200,     -0.012690,     -0.042420,
		     -0.074120,     -0.003890,     -0.003130,     -0.091340,     -0.050390,
		      0.024470,      0.001230,      0.047560,     -0.108230,      0.008300,
		     -0.045470,     -0.046400,     -0.062970,      0.015640,      0.057440,
		      0.099310,      0.070050,      0.026790,      0.134010,      0.068990,
		      0.037860,      0.062960,      0.072120,     -0.094980,     -0.011420,
		     -0.030470,     -0.017320,     -0.093130,     -0.031530,      0.033800,
		     -0.006200,     -0.026270,      0.025550,      0.114630,     -0.010630,
		     -0.106370,     -0.045060,     -0.106150,     -0.032160,     -0.050060,
		     -0.065480,      0.025820,      0.066760,      0.008310,      0.056840,
		      0.059720,      0.018670,     -0.018470,     -0.064650,      0.031240,
		      0.063730,      0.007100,      0.041470,      0.037310,     -0.093570,
		     -0.140050,      0.067150,      0.084200,      0.082550,      0.000000,
		     -0.008440,      0.026190,      0.046990,      0.053210,      0.069130,
		     -0.015710,      0.040310,      0.017880,     -0.041750,      0.132610,
		      0.054070,     -0.158570,     -0.019090,      0.049450,      0.005650,
		     -0.081070,      0.027670,      0.054480,     -0.036350,     -0.065650,
		     -0.053780,     -0.064170,      0.004150,      0.162940,      0.027240,
		     -0.078030,     -0.064590,      0.072660,      0.082080,      0.113910,
		      0.053660,     -0.064850,     -0.111650,     -0.040470,     -0.029530,
		      0.067670,      0.052590,     -0.063760,     -0.143850,     -0.142210,
		     -0.019370,     -0.027880,     -0.110720,     -0.044620,     -0.003000,
		      0.005030,      0.107070,     -0.043640,     -0.117750,     -0.037700,
		      0.024520,      0.170100,      0.009130,     -0.011470,      0.010150,
		      0.025990,      0.060520,     -0.002180,     -0.044290,     -0.038910,
		      0.004440,     -0.032250,     -0.017300,     -0.098720,     -0.089200,
		     -0.069030,     -0.077480,      0.025740,      0.001070,     -0.068880,
		      0.018870,     -0.049620,     -0.034000,      0.026480,      0.034840,
		      0.049120,      0.032130,     -0.007190,      0.084710,      0.058510,
		      0.040090,     -0.110970,     -0.001780,      0.007910,      0.006730,
		      0.040140,      0.015840,      0.069520,     -0.030390,     -0.051250,
		     -0.117900,     -0.038190,     -0.122330,      0.067490,      0.034560,
		     -0.020920,     -0.034260,      0.025380,      0.120730,      0.024200,
		      0.035330,     -0.131390,      0.026620,     -0.043490,      0.008570,
		      0.019310,     -0.010570,     -0.154760,     -0.015760,      0.041890,
		      0.034400,      0.024430,      0.003780,      0.011130,      0.005580,
		     -0.041840,     -0.049010,      0.049400,      0.110320,     -0.034730,
		     -0.111450,     -0.113510,      0.021800,     -0.023040,     -0.037870,
		     -0.151720,      0.008690,      0.040010,     -0.001630,      0.011040,
		     -0.033480,     -0.056280,     -0.046410,      0.031210,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : uranus ---  Model : gsfcq3full */
coeffStruct& _model_coeff_gsfcq3full() {
	static const std::string name = "gsfcq3full";
	static const std::string body = "uranus";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		  11893.000000,  11579.000000,  -6030.000000, -12587.000000,    196.000000,
		   2705.000000,   1188.000000,  -4808.000000,  -2412.000000,
	};
	static const std::vector<double> h = {
		      0.000000, -15684.000000,      0.000000,   6116.000000,   4759.000000,
		      0.000000,  -7095.000000,  -1616.000000,  -2608.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : uranus ---  Model : gsfcq3 */
coeffStruct& _model_coeff_gsfcq3() {
	static const std::string name = "gsfcq3";
	static const std::string body = "uranus";
	static const int len = 5;
	static const int nmax = 2;
	static const int ndef = 2;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,
	};
	static const std::vector<double> g = {
		  11893.000000,  11579.000000,  -6030.000000, -12587.000000,    196.000000,
	};
	static const std::vector<double> h = {
		      0.000000, -15684.000000,      0.000000,   6116.000000,   4759.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : uranus ---  Model : ah5 */
coeffStruct& _model_coeff_ah5() {
	static const std::string name = "ah5";
	static const std::string body = "uranus";
	static const int len = 14;
	static const int nmax = 4;
	static const int ndef = 4;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,
	};
	static const std::vector<double> g = {
		  11278.000000,  10928.000000,  -9648.000000, -12284.000000,   1453.000000,
		  -1265.000000,   2778.000000,  -4535.000000,  -6297.000000,   3388.000000,
		    -29.000000,    955.000000,   5588.000000,   8136.000000,
	};
	static const std::vector<double> h = {
		      0.000000, -16049.000000,      0.000000,   6405.000000,   4220.000000,
		      0.000000,  -1548.000000,  -2165.000000,  -3036.000000,      0.000000,
		  -2036.000000,  -3437.000000,  -1154.000000,  -2920.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : uranus ---  Model : umoh */
coeffStruct& _model_coeff_umoh() {
	static const std::string name = "umoh";
	static const std::string body = "uranus";
	static const int len = 152;
	static const int nmax = 16;
	static const int ndef = 16;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,  14,  14,  14,  14,  14,  14,  14,  14,
		  14,  14,  14,  14,  14,  14,  14,  15,  15,  15,  15,  15,  15,  15,  15,  15,
		  15,  15,  15,  15,  15,  15,  15,  16,  16,  16,  16,  16,  16,  16,  16,  16,
		  16,  16,  16,  16,  16,  16,  16,  16,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,
	};
	static const std::vector<double> g = {
		  11854.690000,  11507.230000,  -5877.080000, -13084.900000,   -604.660000,
		   4182.820000,  -1335.730000,  -6776.280000,  -4021.380000,    398.370000,
		   -431.860000,   2168.910000,    465.440000,  -1758.430000,    292.220000,
		    -51.660000,     14.560000,     36.970000,    241.050000,   -263.430000,
		   -100.690000,     61.040000,    -94.530000,     59.670000,    -60.990000,
		     27.690000,     31.420000,      2.760000,    -12.320000,      8.860000,
		    -17.050000,     13.010000,    -11.650000,      0.830000,      8.750000,
		      2.510000,      0.480000,      1.500000,      0.570000,     -0.660000,
		      0.950000,     -0.430000,     -0.520000,      0.100000,     -0.310000,
		      0.190000,     -0.240000,      0.340000,     -0.100000,      0.160000,
		     -0.080000,      0.160000,     -0.110000,     -0.130000,     -0.020000,
		     -0.030000,     -0.020000,     -0.050000,      0.020000,     -0.030000,
		      0.030000,     -0.030000,      0.030000,      0.000000,     -0.010000,
		      0.010000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,
	};
	static const std::vector<double> h = {
		      0.000000, -15811.900000,      0.000000,   5850.660000,   4184.690000,
		      0.000000,  -5816.660000,   -357.270000,  -2265.150000,      0.000000,
		  -1013.870000,    414.660000,   -637.840000,     65.550000,      0.000000,
		    520.220000,   -383.170000,    353.480000,     22.320000,   -331.660000,
		      0.000000,     12.060000,     63.350000,    -44.290000,     37.320000,
		     26.180000,    -58.620000,      0.000000,    -21.490000,      5.460000,
		     -6.880000,      3.040000,     -4.860000,      5.500000,      1.570000,
		      0.000000,      1.820000,     -2.790000,      1.660000,     -2.410000,
		      1.750000,     -1.810000,      0.620000,      1.120000,      0.000000,
		      0.320000,      0.250000,     -0.030000,      0.230000,     -0.230000,
		      0.240000,     -0.180000,     -0.020000,      0.050000,      0.000000,
		     -0.060000,      0.020000,     -0.010000,      0.020000,      0.010000,
		      0.000000,      0.010000,      0.000000,     -0.010000,     -0.010000,
		      0.000000,      0.000000,     -0.010000,      0.000000,     -0.010000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,      0.000000,      0.000000,      0.000000,
		      0.000000,      0.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : jupiter ---  Model : jrm33 */
coeffStruct& _model_coeff_jrm33() {
	static const std::string name = "jrm33";
	static const std::string body = "jupiter";
	static const int len = 495;
	static const int nmax = 30;
	static const int ndef = 13;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,  14,  14,  14,  14,  14,  14,  14,  14,
		  14,  14,  14,  14,  14,  14,  14,  15,  15,  15,  15,  15,  15,  15,  15,  15,
		  15,  15,  15,  15,  15,  15,  15,  16,  16,  16,  16,  16,  16,  16,  16,  16,
		  16,  16,  16,  16,  16,  16,  16,  16,  17,  17,  17,  17,  17,  17,  17,  17,
		  17,  17,  17,  17,  17,  17,  17,  17,  17,  17,  18,  18,  18,  18,  18,  18,
		  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  19,  19,  19,
		  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,
		  19,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,
		  20,  20,  20,  20,  20,  20,  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,
		  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,  21,  22,  22,  22,  22,
		  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,  22,
		  22,  22,  22,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,
		  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  23,  24,  24,  24,  24,  24,
		  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,  24,
		  24,  24,  24,  24,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,
		  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  25,  26,  26,
		  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,  26,
		  26,  26,  26,  26,  26,  26,  26,  26,  26,  27,  27,  27,  27,  27,  27,  27,
		  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,  27,
		  27,  27,  27,  27,  27,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,
		  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,  28,
		  28,  28,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,
		  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,  29,
		  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,
		  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,  30,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,
		  20,  21,  22,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,
		  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,   0,   1,   2,   3,   4,
		   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,
		  21,  22,  23,  24,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,
		  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,   0,   1,
		   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,
		  18,  19,  20,  21,  22,  23,  24,  25,  26,   0,   1,   2,   3,   4,   5,   6,
		   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,
		  23,  24,  25,  26,  27,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
		  11,  12,  13,  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,
		  27,  28,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,
		  14,  15,  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,
		   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
		  16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,
	};
	static const std::vector<double> g = {
		 410993.400000, -71305.900000,  11796.700000, -56972.400000,  48250.200000,
		   2799.300000, -37488.400000,  15396.800000,  -1489.800000, -34402.000000,
		  -8080.800000,  -2440.500000, -10848.300000, -17919.100000, -18265.700000,
		   4221.800000,  16599.500000, -17345.800000,  -2544.500000,  -4987.700000,
		 -20968.000000,   9887.600000,  12192.400000, -12548.700000,   2742.200000,
		   1557.600000,   8018.200000,     59.900000,   5366.100000,  -7099.500000,
		  -1533.400000,  -7055.700000,   3060.600000,  -2488.300000,   3700.800000,
		  10849.500000,   1323.800000,  -6952.200000,    -95.000000,  -4746.600000,
		  -1301.700000,  -4284.600000,  -1436.700000,  -3024.600000,   8914.400000,
		  -3506.700000,    288.100000,    773.600000,   3592.700000,  -3170.800000,
		   1406.300000,  -1526.600000,   1313.300000,  -2314.600000,  -2516.500000,
		   1883.200000,   2836.100000,   4259.400000,   3776.700000,    764.400000,
		   2112.300000,    724.800000,   2496.300000,    840.100000,   1179.400000,
		   1311.600000,   3056.900000,  -2060.600000,   3550.100000,    589.700000,
		    194.800000,  -1073.300000,   -521.700000,    685.300000,    350.200000,
		   -326.600000,   1088.800000,   2300.500000,   1688.400000,   -291.700000,
		    483.200000,   -581.900000,   -818.400000,  -1095.600000,  -1987.800000,
		    484.500000,  -1473.100000,   -666.500000,   -755.400000,   -220.800000,
		    751.500000,   -456.400000,   1160.400000,  -2307.800000,    -77.100000,
		    -16.900000,    594.800000,   -950.700000,   1042.300000,   -641.800000,
		     41.600000,   -229.100000,    -40.300000,   -287.800000,   1164.600000,
		  -2478.000000,    907.700000,  -1280.700000,    148.500000,   1468.800000,
		     -8.100000,    604.500000,    787.200000,    327.700000,    111.600000,
		    245.200000,    127.300000,    425.900000,      6.300000,    469.400000,
		  -1929.200000,    -38.700000,   -815.900000,   -775.700000,     31.000000,
		   -149.900000,    318.200000,   -298.300000,   -149.300000,    241.400000,
		    -69.400000,    367.000000,    -22.200000,     91.800000,    -43.700000,
		   -809.200000,   -888.900000,   -604.000000,   -339.100000,   -350.600000,
		   -503.800000,   -235.500000,      9.000000,   -498.700000,   -283.200000,
		    -50.900000,   -515.200000,   -255.300000,     15.600000,    -24.700000,
		   -176.500000,     49.200000,  -1238.700000,   -190.200000,   -101.500000,
		    -58.000000,     10.000000,   -525.900000,    468.400000,     88.100000,
		   -385.100000,     77.600000,     75.700000,   -148.900000,    201.800000,
		     61.700000,   -243.600000,    168.200000,    -42.200000,      7.300000,
		   -796.000000,    443.600000,    266.900000,    391.200000,    366.200000,
		    175.800000,    512.500000,    151.000000,    214.900000,    243.400000,
		    337.500000,    -10.200000,    -34.900000,    161.200000,    222.300000,
		   -119.600000,     28.000000,    132.600000,   -180.600000,   -493.700000,
		    486.700000,    -62.000000,    539.200000,    187.400000,    136.800000,
		     -9.400000,   -581.200000,    291.800000,   -198.100000,     95.500000,
		   -372.600000,    190.700000,   -119.600000,    -11.900000,     74.400000,
		   -391.400000,    -33.100000,   -118.000000,    -57.200000,   -319.900000,
		    113.600000,   -262.200000,     94.200000,   -306.700000,   -139.200000,
		   -291.500000,   -716.600000,     55.500000,   -219.000000,   -165.100000,
		   -206.600000,   -115.700000,   -117.500000,    -56.900000,     -6.900000,
		     31.400000,   -146.900000,    453.700000,   -256.500000,    424.200000,
		    -71.500000,   -157.000000,   -127.900000,    -73.900000,    -98.000000,
		    222.100000,   -299.300000,   -124.800000,    155.600000,     61.000000,
		   -137.500000,    -52.100000,   -177.100000,    -51.600000,     44.300000,
		     35.700000,   -365.800000,     91.400000,   -200.600000,     28.000000,
		   -113.800000,     16.700000,    -48.600000,   -197.900000,    -16.400000,
		     84.000000,    155.600000,    194.900000,    -91.600000,     55.500000,
		    183.600000,     43.100000,    -16.400000,   -149.600000,     33.100000,
		    -39.400000,    123.400000,    -62.200000,    -74.900000,    -28.000000,
		    142.100000,    147.200000,   -307.500000,    228.200000,   -365.400000,
		    -53.700000,   -120.700000,     23.000000,     53.900000,    -25.800000,
		   -185.500000,     65.200000,    -45.700000,    -22.300000,    -34.500000,
		     10.400000,   -158.900000,    -15.600000,    -59.400000,    122.200000,
		   -119.500000,    -18.300000,    108.200000,    -35.600000,    -94.600000,
		    147.000000,    -88.500000,    199.100000,   -121.600000,    128.500000,
		    -31.900000,     38.500000,    -80.700000,   -118.600000,   -177.400000,
		     44.800000,    -26.800000,   -153.300000,      7.600000,    -49.600000,
		    -48.700000,   -122.100000,     -1.700000,     40.100000,    -57.100000,
		    -82.700000,    170.500000,   -109.100000,     28.000000,    -45.500000,
		    -42.100000,     16.400000,    -42.700000,    174.800000,    203.000000,
		     42.300000,     17.100000,    -79.500000,     -1.500000,     31.400000,
		     18.800000,     21.500000,    -94.600000,     35.000000,    -60.600000,
		    -11.700000,    -84.700000,     60.100000,      5.800000,     -4.900000,
		     25.900000,    117.300000,      9.200000,     99.600000,   -126.200000,
		     34.200000,     25.900000,     23.700000,     89.800000,     63.200000,
		     42.000000,     69.700000,    -12.600000,     -7.000000,     58.400000,
		     71.800000,     30.100000,     24.800000,      0.700000,      8.700000,
		     10.800000,    -21.400000,     -0.100000,     27.600000,     17.100000,
		     23.100000,     59.900000,     89.400000,     40.000000,    -30.500000,
		    -42.500000,     29.000000,    -21.300000,      5.100000,   -205.500000,
		     60.100000,   -142.300000,    -97.300000,     29.400000,    -17.800000,
		     30.400000,     23.100000,     16.100000,     13.100000,      8.600000,
		     28.300000,    -14.000000,     52.400000,     10.300000,     17.400000,
		    -33.200000,     12.000000,     22.100000,     15.700000,     42.500000,
		     18.800000,    -62.500000,    -23.700000,     15.400000,    -38.500000,
		    -10.200000,   -106.400000,    -25.500000,     -2.600000,    -86.800000,
		    -66.800000,    -19.600000,     -3.900000,     27.800000,     -9.300000,
		     -6.200000,    -12.200000,     -6.200000,     16.700000,     -9.100000,
		     17.800000,     34.200000,     -0.400000,    -30.600000,     -3.200000,
		      2.900000,      1.700000,    -19.700000,      2.200000,     16.700000,
		    -20.500000,     -3.500000,      9.800000,      7.600000,     12.800000,
		    -35.400000,     43.700000,     10.500000,    -23.600000,     15.300000,
		    -22.200000,      8.200000,      5.400000,    -10.100000,     -4.400000,
		    -13.100000,     -6.500000,      3.000000,      4.200000,    -19.700000,
		     13.200000,      1.200000,      6.900000,     -7.700000,     -1.800000,
		      6.100000,    -27.100000,    -11.200000,     40.600000,      8.100000,
		    -14.300000,     23.900000,     11.500000,      2.000000,     -9.800000,
		      5.200000,      4.100000,     42.800000,      5.300000,     32.200000,
		     -0.100000,      8.000000,    -12.800000,     -0.800000,     -4.600000,
		     -3.200000,      0.800000,     -5.900000,      4.400000,    -14.300000,
		     -9.400000,      8.000000,     16.100000,     -3.300000,      3.600000,
		      3.700000,      1.000000,    -10.900000,      4.300000,     21.200000,
		     -5.400000,      0.700000,     -5.200000,    -13.000000,      5.000000,
		    -19.200000,    -14.300000,     32.100000,     28.000000,     26.900000,
	};
	static const std::vector<double> h = {
		      0.000000,  20958.400000,      0.000000, -42549.000000,  20221.500000,
		      0.000000, -32890.600000,  42518.400000, -27397.700000,      0.000000,
		  32452.400000,  27438.600000,   -501.400000,  -1325.100000,      0.000000,
		  45363.100000,   -826.200000,   6000.600000,  10568.800000,  10091.500000,
		      0.000000,  14016.900000, -10119.100000,   -294.900000,  13948.300000,
		  -3686.900000,   4783.100000,      0.000000,  -7654.800000, -11398.600000,
		   2171.000000,   5301.600000,  -6618.100000,  -1933.800000,  -5802.900000,
		      0.000000,  -2297.400000, -12833.500000,  10019.600000,  -1725.600000,
		   2387.100000,  -3237.100000,    906.100000,  -3178.300000,      0.000000,
		  -7899.600000,  -1328.300000,   6566.500000,  -1275.300000,   3617.100000,
		   -194.800000,   1960.300000,    956.900000,   1831.700000,      0.000000,
		  -5689.800000,   5570.300000,  -2541.600000,  -1795.200000,   -217.800000,
		   -628.300000,  -1619.600000,    454.600000,   -840.000000,   1581.400000,
		      0.000000,    549.100000,   4527.600000,  -4223.000000,  -2761.900000,
		   -476.000000,  -2714.400000,  -1349.200000,  -1649.000000,  -1471.500000,
		   -583.000000,   -499.800000,      0.000000,   4204.200000,   2228.000000,
		  -1901.100000,  -1271.200000,   1073.400000,  -1111.900000,    872.900000,
		   -837.900000,   -462.800000,    -22.000000,    110.300000,   -712.700000,
		      0.000000,   4125.600000,    -44.000000,   -455.400000,   2160.600000,
		    255.400000,   1105.100000,   1214.200000,    196.100000,   -207.700000,
		   1195.700000,    472.400000,    721.600000,     51.300000,      0.000000,
		   -847.800000,   -124.900000,    108.800000,   1456.200000,  -1284.600000,
		    896.500000,    237.000000,    225.800000,    -80.900000,    696.900000,
		    326.500000,    225.900000,     71.000000,    124.400000,      0.000000,
		  -1146.400000,   -580.400000,    527.300000,    120.000000,   -478.900000,
		    148.400000,     45.700000,   -882.500000,   -206.600000,    -74.900000,
		     51.000000,     36.400000,    -44.400000,   -285.900000,    135.100000,
		      0.000000,   -932.800000,   -608.100000,    510.200000,   -587.400000,
		    -39.400000,    358.100000,    279.800000,    244.500000,    487.300000,
		    366.700000,     93.900000,    282.800000,   -267.700000,     39.800000,
		   -213.100000,     32.400000,      0.000000,   -209.200000,    -15.800000,
		     54.400000,    308.700000,    722.000000,    233.300000,    -82.100000,
		    221.500000,    488.600000,    206.300000,    176.700000,    353.400000,
		    204.800000,    -29.500000,     12.800000,    -89.600000,   -166.600000,
		      0.000000,    670.900000,   -176.100000,   -340.300000,     37.000000,
		    304.300000,   -348.800000,   -291.900000,    165.600000,    360.900000,
		   -119.000000,    100.100000,     26.900000,      1.000000,    -60.200000,
		     66.500000,    277.800000,     29.100000,     15.300000,      0.000000,
		     85.800000,   -412.100000,   -502.000000,   -425.100000,   -232.200000,
		   -477.000000,   -463.500000,     73.600000,   -228.300000,    -96.800000,
		   -256.000000,   -228.000000,    119.300000,    -76.900000,   -319.800000,
		   -284.000000,    197.900000,   -239.600000,    302.600000,      0.000000,
		   -297.600000,    -13.700000,   -118.400000,    105.600000,    101.900000,
		     45.300000,   -213.000000,    372.500000,   -382.800000,    154.200000,
		   -290.300000,     17.100000,      6.700000,     13.200000,    -49.300000,
		    -32.200000,   -152.800000,     62.200000,      8.500000,     35.000000,
		      0.000000,   -155.800000,    312.300000,    291.100000,    207.900000,
		    165.800000,    291.200000,    198.800000,    442.100000,   -104.000000,
		    209.600000,    -45.000000,   -139.600000,    229.400000,    -59.800000,
		    -30.000000,   -182.000000,    162.100000,    131.700000,   -356.900000,
		    379.500000,   -318.600000,      0.000000,    -52.500000,    180.600000,
		    182.500000,    -69.500000,    -24.500000,     93.200000,    188.100000,
		     54.500000,    -68.500000,     93.800000,    -87.300000,   -175.300000,
		    202.400000,    -49.800000,   -129.200000,   -241.100000,    -63.600000,
		     68.400000,    168.100000,   -160.000000,     62.600000,   -117.500000,
		      0.000000,     52.800000,     -2.100000,    -51.900000,    -92.900000,
		     -7.900000,     21.100000,     16.400000,    -51.300000,   -135.800000,
		     56.100000,   -172.600000,     40.800000,     21.300000,     19.600000,
		    -63.100000,   -135.900000,     -3.100000,     55.900000,    -63.500000,
		     -6.300000,    188.100000,   -119.300000,    313.200000,      0.000000,
		     59.400000,    -45.400000,    -76.000000,     13.900000,     26.500000,
		     63.000000,      9.100000,     29.600000,     -7.600000,     38.300000,
		    -81.600000,     63.300000,     40.000000,     50.300000,    -52.500000,
		    -88.200000,     99.900000,     59.300000,    -52.200000,    106.400000,
		    -62.100000,     34.600000,   -171.100000,     43.500000,      0.000000,
		     -6.200000,    -62.400000,    -17.500000,     72.800000,     -6.100000,
		    -12.200000,     28.200000,    -18.000000,    122.400000,    -33.400000,
		     24.000000,     19.700000,     70.600000,     62.200000,    -60.100000,
		    -54.400000,     54.000000,    -48.500000,     45.400000,     24.900000,
		    -77.300000,     36.900000,      4.800000,     85.700000,   -262.000000,
		      0.000000,      5.100000,    -50.800000,      9.500000,     33.900000,
		     -7.200000,    -73.000000,    -12.900000,    -48.100000,     55.400000,
		    -60.800000,     39.000000,     46.900000,     43.800000,     29.200000,
		     -1.900000,      5.800000,     36.000000,   -113.800000,     29.200000,
		    -22.900000,    -10.000000,    -23.400000,    -30.500000,     66.700000,
		    126.900000,    -43.300000,      0.000000,     23.200000,      8.100000,
		     16.800000,    -44.100000,      6.200000,    -17.000000,    -30.200000,
		      0.700000,    -45.500000,    -18.100000,     13.200000,     31.800000,
		     20.700000,    -14.400000,     30.700000,     41.000000,     23.000000,
		    -56.400000,      1.600000,    -14.700000,    -16.300000,    -31.100000,
		    -32.500000,    -49.200000,     79.700000,    -45.300000,     35.800000,
		      0.000000,     -9.500000,     37.400000,     14.000000,    -46.100000,
		     -2.300000,     34.700000,     -6.700000,     21.600000,    -33.100000,
		     16.600000,     -0.500000,    -20.500000,     -3.900000,    -16.500000,
		     14.500000,     20.400000,    -14.200000,     15.700000,    -12.900000,
		    -11.800000,    -42.800000,     -8.600000,     27.400000,    -67.500000,
		     -2.200000,    -19.200000,     28.900000,     79.400000,      0.000000,
		    -23.600000,     13.400000,      0.800000,      9.200000,     -9.700000,
		     17.200000,     11.000000,      4.000000,     14.700000,     13.900000,
		     -2.400000,    -28.100000,    -21.700000,      1.600000,      1.500000,
		     -9.800000,    -21.900000,     26.900000,    -15.600000,    -12.600000,
		    -28.600000,      8.000000,     25.600000,    -11.000000,    -10.300000,
		    -17.900000,      2.000000,      5.800000,     40.600000,      0.000000,
		     -1.000000,    -10.200000,     -8.800000,     29.100000,     -2.400000,
		    -10.400000,      6.700000,     -4.900000,     16.900000,      2.300000,
		     -1.600000,     -3.200000,    -14.000000,      8.200000,      0.500000,
		     -9.800000,     -4.400000,      6.200000,     -5.800000,     -4.700000,
		      0.100000,     10.200000,     -3.400000,      9.900000,     -2.500000,
		    -11.700000,    -10.500000,    -21.500000,     14.600000,     38.300000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : jupiter ---  Model : vit4 */
coeffStruct& _model_coeff_vit4() {
	static const std::string name = "vit4";
	static const std::string body = "jupiter";
	static const int len = 14;
	static const int nmax = 4;
	static const int ndef = 4;
	static const double rscale = 1.0023695021241394442768069;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,
	};
	static const std::vector<double> g = {
		 428077.000000, -75306.000000,  -4283.000000, -59426.000000,  44386.000000,
		   8906.000000, -21447.000000,  21130.000000,  -1190.000000, -22925.000000,
		  18940.000000,  -3851.000000,   9926.000000,   1271.000000,
	};
	static const std::vector<double> h = {
		      0.000000,  24616.000000,      0.000000, -50154.000000,  38452.000000,
		      0.000000, -17187.000000,  40667.000000, -35263.000000,      0.000000,
		  16088.000000,  11807.000000,   6195.000000,  12641.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : jupiter ---  Model : isaac */
coeffStruct& _model_coeff_isaac() {
	static const std::string name = "isaac";
	static const std::string body = "jupiter";
	static const int len = 65;
	static const int nmax = 10;
	static const int ndef = 10;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,
	};
	static const std::vector<double> g = {
		 406650.000000, -71420.000000, -12860.000000, -69810.000000,  38520.000000,
		  -4790.000000, -46420.000000,  28670.000000,  -9340.000000, -22300.000000,
		  18930.000000,   2760.000000, -13170.000000,   1110.000000,  -1650.000000,
		   7550.000000,   6230.000000,  -1500.000000,  -4000.000000,   1420.000000,
		  -6370.000000,   3240.000000,   8020.000000,    270.000000,  -4070.000000,
		   2790.000000,   -680.000000,    640.000000,   8920.000000,   3660.000000,
		  -6980.000000,  -1140.000000,   2390.000000,  -1230.000000,    310.000000,
		  -5110.000000,   7200.000000,   4530.000000,  -3710.000000,   -200.000000,
		   3120.000000,  -2100.000000,    710.000000,   -133.000000,    100.000000,
		    330.000000,   -960.000000,    370.000000,   3390.000000,   -740.000000,
		   -680.000000,    400.000000,    -92.000000,      9.000000,    280.000000,
		    490.000000,   -630.000000,    910.000000,   2850.000000,   -380.000000,
		   -620.000000,    440.000000,   -169.000000,     31.000000,     -0.800000,
	};
	static const std::vector<double> h = {
		      0.000000,  23530.000000,      0.000000, -31700.000000,   7950.000000,
		      0.000000,  -7503.000000,  40310.000000, -36860.000000,      0.000000,
		  20780.000000,  32930.000000, -16950.000000,   5960.000000,      0.000000,
		     30.000000,  -3340.000000,   2540.000000,   3490.000000,   -840.000000,
		      0.000000,  10510.000000,  -9600.000000,   5030.000000,   1270.000000,
		  -1060.000000,    180.000000,      0.000000,   -270.000000,  -2970.000000,
		   7390.000000,   1980.000000,  -2120.000000,    350.000000,     19.000000,
		      0.000000,   7810.000000,  -7380.000000,   8520.000000,   -870.000000,
		  -1680.000000,    720.000000,     -8.000000,    -46.000000,      0.000000,
		   -290.000000,   1890.000000,   1850.000000,    690.000000,    320.000000,
		   -430.000000,    270.000000,   -120.000000,     27.000000,      0.000000,
		    230.000000,   1720.000000,   1250.000000,   -110.000000,   -110.000000,
		    -15.000000,    360.000000,   -250.000000,     80.000000,    -14.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : jupiter ---  Model : vipal */
coeffStruct& _model_coeff_vipal() {
	static const std::string name = "vipal";
	static const std::string body = "jupiter";
	static const int len = 20;
	static const int nmax = 5;
	static const int ndef = 5;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,
	};
	static const std::vector<double> g = {
		 420000.000000, -69750.000000,  64410.000000, -86720.000000,  95980.000000,
		 -10580.000000, -59000.000000,  63220.000000,  46710.000000, -74660.000000,
		  32820.000000, -33800.000000,  18260.000000, -14290.000000,  -6600.000000,
		   7370.000000, -17110.000000, -17930.000000,   -770.000000,  -7400.000000,
	};
	static const std::vector<double> h = {
		      0.000000,  19730.000000,      0.000000, -40410.000000,  60300.000000,
		      0.000000, -23100.000000,  51600.000000, -11310.000000,      0.000000,
		  32830.000000, -21310.000000,  -6060.000000,  -4860.000000,      0.000000,
		  20650.000000, -11670.000000,  -2880.000000,   -500.000000, -22790.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : jupiter ---  Model : vip4 */
coeffStruct& _model_coeff_vip4() {
	static const std::string name = "vip4";
	static const std::string body = "jupiter";
	static const int len = 14;
	static const int nmax = 4;
	static const int ndef = 4;
	static const double rscale = 1.0023695021241394442768069;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,
	};
	static const std::vector<double> g = {
		 420543.000000, -65920.000000,  -5118.000000, -61904.000000,  49690.000000,
		  -1576.000000, -52036.000000,  24386.000000, -17597.000000, -16758.000000,
		  22210.000000,  -6074.000000, -20243.000000,   6643.000000,
	};
	static const std::vector<double> h = {
		      0.000000,  24992.000000,      0.000000, -36052.000000,   5250.000000,
		      0.000000,  -8804.000000,  40829.000000, -31586.000000,      0.000000,
		   7557.000000,  40411.000000, -16597.000000,   3866.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : jupiter ---  Model : jpl15evs */
coeffStruct& _model_coeff_jpl15evs() {
	static const std::string name = "jpl15evs";
	static const std::string body = "jupiter";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		 406800.000000, -66800.000000,  -9300.000000, -67200.000000,  50200.000000,
		   -111.000000, -31600.000000,  22000.000000, -25000.000000,
	};
	static const std::vector<double> h = {
		      0.000000,  24300.000000,      0.000000, -49800.000000,  11900.000000,
		      0.000000, -47600.000000,  38000.000000, -22800.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : jupiter ---  Model : p11a */
coeffStruct& _model_coeff_p11a() {
	static const std::string name = "p11a";
	static const std::string body = "jupiter";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		 414400.000000, -69200.000000,   3600.000000, -58100.000000,  44200.000000,
		  -4700.000000, -50200.000000,  35200.000000, -13600.000000,
	};
	static const std::vector<double> h = {
		      0.000000,  23500.000000,      0.000000, -42700.000000,  13400.000000,
		      0.000000, -34200.000000,  29600.000000,   4100.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : jupiter ---  Model : jrm09 */
coeffStruct& _model_coeff_jrm09() {
	static const std::string name = "jrm09";
	static const std::string body = "jupiter";
	static const int len = 230;
	static const int nmax = 20;
	static const int ndef = 10;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,   4,   4,   4,   4,   4,   5,   5,
		   5,   5,   5,   5,   6,   6,   6,   6,   6,   6,   6,   7,   7,   7,   7,   7,
		   7,   7,   7,   8,   8,   8,   8,   8,   8,   8,   8,   8,   9,   9,   9,   9,
		   9,   9,   9,   9,   9,   9,  10,  10,  10,  10,  10,  10,  10,  10,  10,  10,
		  10,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  11,  12,  12,  12,
		  12,  12,  12,  12,  12,  12,  12,  12,  12,  12,  13,  13,  13,  13,  13,  13,
		  13,  13,  13,  13,  13,  13,  13,  13,  14,  14,  14,  14,  14,  14,  14,  14,
		  14,  14,  14,  14,  14,  14,  14,  15,  15,  15,  15,  15,  15,  15,  15,  15,
		  15,  15,  15,  15,  15,  15,  15,  16,  16,  16,  16,  16,  16,  16,  16,  16,
		  16,  16,  16,  16,  16,  16,  16,  16,  17,  17,  17,  17,  17,  17,  17,  17,
		  17,  17,  17,  17,  17,  17,  17,  17,  17,  17,  18,  18,  18,  18,  18,  18,
		  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  18,  19,  19,  19,
		  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,  19,
		  19,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,  20,
		  20,  20,  20,  20,  20,  20,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,   0,   1,   2,   3,   4,   0,   1,
		   2,   3,   4,   5,   0,   1,   2,   3,   4,   5,   6,   0,   1,   2,   3,   4,
		   5,   6,   7,   0,   1,   2,   3,   4,   5,   6,   7,   8,   0,   1,   2,   3,
		   4,   5,   6,   7,   8,   9,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,
		  10,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,   0,   1,   2,   3,   4,   5,   6,   7,   8,
		   9,  10,  11,  12,  13,  14,  15,  16,   0,   1,   2,   3,   4,   5,   6,   7,
		   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,   0,   1,   2,   3,   4,   5,
		   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,   0,   1,   2,
		   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,  16,  17,  18,
		  19,   0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,
		  15,  16,  17,  18,  19,  20,
	};
	static const std::vector<double> g = {
		 410244.700000, -71498.300000,  11670.400000, -56835.800000,  48689.500000,
		   4018.600000, -37791.100000,  15926.300000,  -2710.500000, -34645.400000,
		  -8247.600000,  -2406.100000, -11083.800000, -17837.200000, -18023.600000,
		   4683.900000,  16160.000000, -16402.000000,  -2600.700000,  -3660.700000,
		 -20819.600000,   9992.900000,  11791.800000, -12574.700000,   2669.700000,
		   1113.200000,   7584.900000,    598.400000,   4665.900000,  -6495.700000,
		  -2516.500000,  -6448.500000,   1855.300000,  -2892.900000,   2968.000000,
		  10059.200000,   1934.400000,  -6702.900000,    153.700000,  -4124.200000,
		   -867.200000,  -3740.600000,   -732.400000,  -2433.200000,   9671.800000,
		  -3046.200000,    260.900000,   2071.300000,   3329.600000,  -2523.100000,
		   1787.100000,  -1148.200000,   1276.500000,  -1976.800000,  -2299.500000,
		   2009.700000,   2127.800000,   3498.300000,   2967.600000,     16.300000,
		   1806.500000,    -46.500000,   2897.800000,    574.500000,   1298.900000,
		   2304.000000,   2861.700000,   -969.500000,   2984.400000,    769.100000,
		   -277.100000,  -1145.800000,   -947.300000,    496.900000,   -130.600000,
		   -531.800000,   1298.600000,   2462.600000,   2517.700000,   -830.700000,
		    193.900000,   -360.200000,   -539.100000,   -825.300000,  -1628.100000,
		   1817.300000,   -874.100000,  -1357.700000,   -216.500000,  -1199.300000,
		    531.200000,    442.000000,    910.800000,  -1571.800000,   -124.700000,
		    612.300000,     48.100000,    261.500000,    415.900000,   -779.200000,
		     83.800000,   -257.100000,    473.500000,   -445.800000,   1390.200000,
		   -838.600000,    212.900000,  -1626.900000,   -242.500000,    605.300000,
		   -346.300000,    104.600000,    394.000000,    233.900000,    277.700000,
		    177.300000,   1259.300000,    525.500000,   1320.400000,    764.300000,
		   -803.200000,   -141.200000,   -750.400000,   -420.600000,    383.800000,
		   -729.800000,    444.900000,   -884.100000,    107.600000,   -319.200000,
		    358.000000,   -637.900000,   -194.000000,   -280.600000,    -41.300000,
		   -359.200000,   -395.100000,   -653.100000,   -242.000000,   -388.000000,
		   -279.000000,   -461.000000,    -53.800000,  -1211.900000,    315.400000,
		   -767.300000,     25.600000,   -790.900000,    249.900000,   -340.300000,
		   -515.600000,   -247.400000,   -566.400000,     88.800000,   -480.600000,
		     95.000000,    -43.400000,   -310.200000,    177.200000,   -155.400000,
		   -820.300000,    272.600000,    -88.500000,   -137.000000,   -101.200000,
		    237.400000,    279.800000,     -3.400000,    551.200000,    -85.900000,
		   -171.300000,    194.500000,    612.200000,    280.700000,    471.100000,
		   -101.900000,    486.500000,    192.200000,     11.700000,    119.600000,
		    404.900000,    -96.000000,     90.800000,    358.900000,    123.800000,
		    164.900000,    -15.600000,     40.600000,    297.500000,   -157.100000,
		    415.500000,    436.600000,    371.000000,    258.500000,   -248.500000,
		    302.200000,   -262.300000,    279.000000,   -109.900000,    -57.700000,
		   -282.300000,   -171.100000,     42.200000,    -88.300000,    200.400000,
		   -171.500000,     21.100000,    -81.900000,    -14.100000,   -205.200000,
		    222.600000,   -297.100000,    241.200000,   -286.400000,    -13.700000,
		   -133.200000,   -234.200000,    118.000000,   -397.000000,   -123.300000,
		   -217.200000,    -95.300000,   -164.000000,     32.200000,    110.100000,
		   -107.200000,    -49.700000,   -107.300000,    -17.100000,   -172.800000,
	};
	static const std::vector<double> h = {
		      0.000000,  21330.500000,      0.000000, -42027.300000,  19353.200000,
		      0.000000, -32957.300000,  42084.500000, -27544.200000,      0.000000,
		  31994.500000,  27811.200000,   -926.100000,    367.100000,      0.000000,
		  45347.900000,   -749.000000,   6268.500000,  10859.600000,   9608.400000,
		      0.000000,  14533.100000, -10592.900000,    568.600000,  12871.700000,
		  -4147.800000,   3604.400000,      0.000000,  -7626.300000, -10948.400000,
		   2633.300000,   5394.200000,  -6050.800000,  -1526.000000,  -5684.200000,
		      0.000000,  -2409.700000, -11614.600000,   9287.000000,   -911.900000,
		   2754.500000,  -2446.100000,   1207.300000,  -2887.300000,      0.000000,
		  -8467.400000,  -1383.800000,   5697.700000,  -2056.300000,   3081.500000,
		   -721.200000,   1352.500000,   -210.100000,   1567.600000,      0.000000,
		  -4692.600000,   4445.800000,  -2378.600000,  -2204.300000,    164.100000,
		  -1361.600000,  -2031.500000,   1411.800000,   -714.300000,   1676.500000,
		      0.000000,    246.000000,   4100.400000,  -3942.600000,  -2078.200000,
		   -324.400000,  -1703.300000,   -715.800000,  -1400.800000,  -2105.700000,
		   -143.600000,   -941.400000,      0.000000,   3918.700000,   2545.400000,
		  -2112.600000,   -294.500000,   1060.400000,   -429.800000,    774.800000,
		   -837.000000,    185.500000,    449.400000,    438.100000,  -1132.500000,
		      0.000000,   3437.400000,    133.500000,   -838.300000,   1179.400000,
		   -120.000000,   1369.200000,   1291.900000,    533.200000,     -1.400000,
		   1339.700000,   1654.200000,     35.200000,   1516.200000,      0.000000,
		    186.200000,    -44.600000,     54.700000,    802.400000,      8.200000,
		    658.800000,   -273.600000,    340.300000,   -230.600000,   1202.900000,
		   -643.500000,     -6.400000,   -282.700000,    143.600000,      0.000000,
		  -1159.500000,   -270.000000,     34.700000,    179.100000,      4.200000,
		    311.300000,      9.300000,    -36.000000,     -8.600000,   1084.700000,
		   -694.700000,   -142.200000,   -517.700000,   -353.300000,   -747.700000,
		      0.000000,   -928.100000,   -397.000000,    222.700000,   -143.400000,
		    231.000000,    161.600000,    312.000000,    177.800000,    778.700000,
		    307.900000,   -306.700000,    301.600000,    474.300000,     43.200000,
		    601.700000,    638.300000,      0.000000,    194.100000,    103.700000,
		    743.500000,    181.200000,    467.700000,     33.500000,    358.700000,
		    353.800000,    799.900000,    179.600000,    267.800000,    556.400000,
		     28.400000,    102.400000,    531.000000,   -108.700000,    371.400000,
		      0.000000,    617.900000,    -37.700000,    176.300000,    -18.700000,
		   -174.300000,   -219.000000,    -85.600000,    158.500000,    342.700000,
		   -153.600000,    -34.100000,    220.000000,   -352.100000,    107.600000,
		     19.100000,   -317.900000,   -116.500000,     22.900000,      0.000000,
		    -98.000000,   -151.900000,   -407.900000,   -229.000000,   -416.400000,
		   -108.700000,   -173.600000,    129.000000,   -295.400000,   -221.700000,
		   -317.600000,    -54.200000,   -329.200000,     75.200000,    -91.900000,
		    -38.100000,   -161.800000,    -84.500000,   -207.600000,      0.000000,
		   -417.400000,    162.900000,   -160.200000,    116.100000,     26.700000,
		    150.200000,    119.200000,    254.800000,   -488.300000,    216.400000,
		    -66.000000,     71.500000,    163.400000,    -68.200000,    -27.600000,
		    129.000000,    -42.600000,     31.400000,     58.600000,     78.500000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : jupiter ---  Model : jpl15ev */
coeffStruct& _model_coeff_jpl15ev() {
	static const std::string name = "jpl15ev";
	static const std::string body = "jupiter";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		 406400.000000, -67500.000000,  -9800.000000, -75700.000000,  55000.000000,
		   -306.000000, -33300.000000,  24900.000000, -35000.000000,
	};
	static const std::vector<double> h = {
		      0.000000,  26900.000000,      0.000000, -55600.000000,  11000.000000,
		      0.000000, -61500.000000,  46500.000000, -26400.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : jupiter ---  Model : o4 */
coeffStruct& _model_coeff_o4() {
	static const std::string name = "o4";
	static const std::string body = "jupiter";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0016813316146389034599906;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		 421800.000000, -66400.000000, -20300.000000, -73500.000000,  51300.000000,
		 -23300.000000,  -7600.000000,  16800.000000, -23100.000000,
	};
	static const std::vector<double> h = {
		      0.000000,  26400.000000,      0.000000, -46900.000000,   8800.000000,
		      0.000000, -58000.000000,  48700.000000, -29400.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : jupiter ---  Model : o6 */
coeffStruct& _model_coeff_o6() {
	static const std::string name = "o6";
	static const std::string body = "jupiter";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0016813316146389034599906;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		 424202.000000, -65929.000000,  -2181.000000, -71106.000000,  48714.000000,
		   7565.000000, -15493.000000,  19775.000000, -17958.000000,
	};
	static const std::vector<double> h = {
		      0.000000,  24116.000000,      0.000000, -40304.000000,   7179.000000,
		      0.000000, -38824.000000,  34243.000000, -22439.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : jupiter ---  Model : sha */
coeffStruct& _model_coeff_sha() {
	static const std::string name = "sha";
	static const std::string body = "jupiter";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0013165634891734168121502;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		 409200.000000, -70500.000000,  -3300.000000, -69900.000000,  53700.000000,
		 -11300.000000, -58500.000000,  28300.000000,   6700.000000,
	};
	static const std::vector<double> h = {
		      0.000000,  23100.000000,      0.000000, -53100.000000,   7400.000000,
		      0.000000, -42300.000000,  12000.000000, -17100.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : jupiter ---  Model : u17ev */
coeffStruct& _model_coeff_u17ev() {
	static const std::string name = "u17ev";
	static const std::string body = "jupiter";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0013165634891734168121502;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		 410879.000000, -67885.000000,   7086.000000, -64371.000000,  46437.000000,
		  -5104.000000, -15682.000000,  25148.000000,  -4253.000000,
	};
	static const std::vector<double> h = {
		      0.000000,  22881.000000,      0.000000, -30924.000000,  13288.000000,
		      0.000000, -15040.000000,  45743.000000, -21705.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : jupiter ---  Model : gsfc15ev */
coeffStruct& _model_coeff_gsfc15ev() {
	static const std::string name = "gsfc15ev";
	static const std::string body = "jupiter";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		 418400.000000, -64700.000000, -21000.000000, -73700.000000,  55400.000000,
		 -22300.000000,   1900.000000,  22000.000000, -31200.000000,
	};
	static const std::vector<double> h = {
		      0.000000,  23500.000000,      0.000000, -49900.000000,   9200.000000,
		      0.000000, -66800.000000,  61900.000000, -30700.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : jupiter ---  Model : gsfc13ev */
coeffStruct& _model_coeff_gsfc13ev() {
	static const std::string name = "gsfc13ev";
	static const std::string body = "jupiter";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		 435200.000000, -65600.000000, -19700.000000, -55700.000000,  26700.000000,
		 -13800.000000, -32400.000000, -17600.000000,  12400.000000,
	};
	static const std::vector<double> h = {
		      0.000000,  34100.000000,      0.000000, -30600.000000,  18700.000000,
		      0.000000,  -5300.000000,   8300.000000, -31100.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : jupiter ---  Model : gsfc15evs */
coeffStruct& _model_coeff_gsfc15evs() {
	static const std::string name = "gsfc15evs";
	static const std::string body = "jupiter";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0000000000000000000000000;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		 420100.000000, -65700.000000, -21100.000000, -68600.000000,  50900.000000,
		 -18600.000000,   -200.000000,  15000.000000, -23500.000000,
	};
	static const std::vector<double> h = {
		      0.000000,  25800.000000,      0.000000, -46100.000000,  11000.000000,
		      0.000000, -57500.000000,  51200.000000,  30700.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



/* Body : jupiter ---  Model : v117ev */
coeffStruct& _model_coeff_v117ev() {
	static const std::string name = "v117ev";
	static const std::string body = "jupiter";
	static const int len = 9;
	static const int nmax = 3;
	static const int ndef = 3;
	static const double rscale = 1.0023695021241394442768069;
	static const std::vector<int> n = {
		   1,   1,   2,   2,   2,   3,   3,   3,   3,
	};
	static const std::vector<int> m = {
		   0,   1,   0,   1,   2,   0,   1,   2,   3,
	};
	static const std::vector<double> g = {
		 420825.000000, -65980.000000,  -3411.000000, -75856.000000,  48321.000000,
		   2153.000000,  -3295.000000,  26315.000000,  -6905.000000,
	};
	static const std::vector<double> h = {
		      0.000000,  26122.000000,      0.000000, -29424.000000,  10704.000000,
		      0.000000,   8883.000000,  69538.000000, -24718.000000,
	};
	static coeffStruct out = {
		name, body, len, nmax, ndef, rscale, n, m, g, h
	};
	return out;
}



std::map<std::string,coeffStructFunc> getCoeffMap() {
	static std::map<std::string,coeffStructFunc> coeffMap = {
		{"spv", _model_coeff_spv},
		{"z3", _model_coeff_z3},
		{"cassini11", _model_coeff_cassini11},
		{"cassini5", _model_coeff_cassini5},
		{"soi", _model_coeff_soi},
		{"p1184", _model_coeff_p1184},
		{"burton2009", _model_coeff_burton2009},
		{"v1", _model_coeff_v1},
		{"cassini3", _model_coeff_cassini3},
		{"v2", _model_coeff_v2},
		{"p11as", _model_coeff_p11as},
		{"igrf1960", _model_coeff_igrf1960},
		{"igrf1935", _model_coeff_igrf1935},
		{"igrf1945", _model_coeff_igrf1945},
		{"igrf1925", _model_coeff_igrf1925},
		{"igrf1910", _model_coeff_igrf1910},
		{"igrf1905", _model_coeff_igrf1905},
		{"igrf1980", _model_coeff_igrf1980},
		{"igrf1990", _model_coeff_igrf1990},
		{"igrf2015", _model_coeff_igrf2015},
		{"igrf1940", _model_coeff_igrf1940},
		{"igrf1930", _model_coeff_igrf1930},
		{"igrf2005", _model_coeff_igrf2005},
		{"igrf2020", _model_coeff_igrf2020},
		{"igrf2010", _model_coeff_igrf2010},
		{"igrf1915", _model_coeff_igrf1915},
		{"igrf2000", _model_coeff_igrf2000},
		{"igrf1950", _model_coeff_igrf1950},
		{"igrf1970", _model_coeff_igrf1970},
		{"igrf1965", _model_coeff_igrf1965},
		{"igrf1920", _model_coeff_igrf1920},
		{"igrf2025", _model_coeff_igrf2025},
		{"igrf1985", _model_coeff_igrf1985},
		{"igrf1975", _model_coeff_igrf1975},
		{"igrf1955", _model_coeff_igrf1955},
		{"igrf1900", _model_coeff_igrf1900},
		{"igrf1995", _model_coeff_igrf1995},
		{"gsfco8full", _model_coeff_gsfco8full},
		{"gsfco8", _model_coeff_gsfco8},
		{"nmoh", _model_coeff_nmoh},
		{"kivelson2002a", _model_coeff_kivelson2002a},
		{"weber2022quad", _model_coeff_weber2022quad},
		{"kivelson2002b", _model_coeff_kivelson2002b},
		{"kivelson2002c", _model_coeff_kivelson2002c},
		{"weber2022dip", _model_coeff_weber2022dip},
		{"uno2009", _model_coeff_uno2009},
		{"thebault2018m2", _model_coeff_thebault2018m2},
		{"uno2009svd", _model_coeff_uno2009svd},
		{"anderson2010qts04", _model_coeff_anderson2010qts04},
		{"anderson2010dsha", _model_coeff_anderson2010dsha},
		{"anderson2012", _model_coeff_anderson2012},
		{"anderson2010dts04", _model_coeff_anderson2010dts04},
		{"anderson2010qsha", _model_coeff_anderson2010qsha},
		{"ness1975", _model_coeff_ness1975},
		{"thebault2018m1", _model_coeff_thebault2018m1},
		{"anderson2010r", _model_coeff_anderson2010r},
		{"anderson2010d", _model_coeff_anderson2010d},
		{"anderson2010q", _model_coeff_anderson2010q},
		{"thebault2018m3", _model_coeff_thebault2018m3},
		{"gao2021", _model_coeff_gao2021},
		{"cain2003", _model_coeff_cain2003},
		{"mh2014", _model_coeff_mh2014},
		{"langlais2019", _model_coeff_langlais2019},
		{"gsfcq3full", _model_coeff_gsfcq3full},
		{"gsfcq3", _model_coeff_gsfcq3},
		{"ah5", _model_coeff_ah5},
		{"umoh", _model_coeff_umoh},
		{"jrm33", _model_coeff_jrm33},
		{"vit4", _model_coeff_vit4},
		{"isaac", _model_coeff_isaac},
		{"vipal", _model_coeff_vipal},
		{"vip4", _model_coeff_vip4},
		{"jpl15evs", _model_coeff_jpl15evs},
		{"p11a", _model_coeff_p11a},
		{"jrm09", _model_coeff_jrm09},
		{"jpl15ev", _model_coeff_jpl15ev},
		{"o4", _model_coeff_o4},
		{"o6", _model_coeff_o6},
		{"sha", _model_coeff_sha},
		{"u17ev", _model_coeff_u17ev},
		{"gsfc15ev", _model_coeff_gsfc15ev},
		{"gsfc13ev", _model_coeff_gsfc13ev},
		{"gsfc15evs", _model_coeff_gsfc15evs},
		{"v117ev", _model_coeff_v117ev},
	};
	return coeffMap;
}


/***********************************************************************
 * NAME : getModelCoeffStruct(Model)
 *
 * DESCRIPTION : Function to return a structure containing model 
        coefficients.
 *		
 * INPUTS : 
 *		std::string Model	Model name (use lower case!).
 *
 * RETURNS :
 *		coeffStructFunc	cstr    Model coefficient function.
 *
 **********************************************************************/
coeffStructFunc getModelCoeffStruct(std::string Model) {
    std::map<std::string,coeffStructFunc> coeffMap = getCoeffMap();
    return coeffMap[Model];
}

/***********************************************************************
 * NAME : getModelCoeffStruct(Model)
 *
 * DESCRIPTION : Function to return a structure containing model 
        coefficients.
 *		
 * INPUTS : 
 *		const char *Model	Model name (use lower case!).
 *
 * RETURNS :
 *		coeffStructFunc	cstr    Model coefficient function.
 *
 **********************************************************************/
coeffStructFunc getModelCoeffStruct(const char *Model) {
    std::map<std::string,coeffStructFunc> coeffMap = getCoeffMap();
    return coeffMap[Model];
}

